Package: easyRasch2
Title: Psychometric Analysis with Rasch Measurement Theory
Version: 1.2.0
Authors@R: c(
    person("Magnus", "Johansson",
           role = c("aut", "cre"),
           email = "pgmj@pm.me",
           comment = c(ORCID = "0000-0003-1669-592X")),
    person("Nicklas", "Korsell",
           role = "ctb",
           comment = "PCM simulation code"),
    person("Mirka", "Henninger",
           role = "ctb",
           comment = c(ORCID = "0000-0003-4676-2361",
           "MH / partial-gamma effect-size and ETS-classification algorithms in dif_tree.R, adapted under MIT licence from the raschtreeMH and effecttree packages")),
    person("Jan", "Radek",
           role = "ctb",
           comment = c(ORCID = "0009-0003-8842-9206",
           "partial-gamma effect-size and ETS-classification algorithms in dif_tree.R, adapted under MIT licence from the effecttree package")))
Description: Streamlines reproducible Rasch measurement theory analyses
    for ordinal item-response data, combining estimation routines from
    'eRm', 'psychotools', 'mirt', 'iarm', and 'lavaan' with consistent
    diagnostic, plotting, and reporting layers. Covers the four basic
    psychometric criteria summarised by Christensen et al. (2021)
    <doi:10.1111/sms.13908> -- unidimensionality, local independence,
    ordered response category thresholds, and invariance across
    subgroups -- together with item fit, targeting, reliability,
    category functioning, and descriptive item-response plots. A
    distinguishing feature is the use of simulation-based critical
    values to replace rule-of-thumb cutoffs for conditional infit mean-square,
    Yen's Q3 local-dependence statistic, the largest residual-PCA eigenvalue,
    ordinal CFA fit indices, and partial-gamma DIF and local-dependence
    coefficients, optionally augmented with multiplicity-corrected bootstrap
    p-values. Outputs are knitr::kable() tables and
    'ggplot2' figures suitable for direct inclusion in 'Quarto' and 'R Markdown'
    reports.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
URL: https://github.com/pgmj/easyRasch2,
        https://pgmj.github.io/easyRasch2/
BugReports: https://github.com/pgmj/easyRasch2/issues
Depends: R (>= 4.1.0)
Imports: graphics, knitr, mirt, psychotools (>= 0.7-3), stats, utils,
        rlang
Suggests: difR, eRm, geomtextpath, ggdist, ggtext, iarm, mirai, ggplot2
        (>= 3.4.0), partykit, psychotree, stablelearner, testthat (>=
        3.0.0), rmarkdown, patchwork, scales, mice, ggrepel, lavaan,
        withr
Config/testthat/edition: 3
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-23 08:13:15 UTC; magnus.johansson.3
Author: Magnus Johansson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1669-592X>),
  Nicklas Korsell [ctb] (PCM simulation code),
  Mirka Henninger [ctb] (ORCID: <https://orcid.org/0000-0003-4676-2361>,
    MH / partial-gamma effect-size and ETS-classification algorithms in
    dif_tree.R, adapted under MIT licence from the raschtreeMH and
    effecttree packages),
  Jan Radek [ctb] (ORCID: <https://orcid.org/0009-0003-8842-9206>,
    partial-gamma effect-size and ETS-classification algorithms in
    dif_tree.R, adapted under MIT licence from the effecttree package)
Maintainer: Magnus Johansson <pgmj@pm.me>
Repository: CRAN
Date/Publication: 2026-08-23 08:30:02 UTC
Built: R 4.5.2; ; 2026-08-23 08:46:25 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(base::print,RMlocdepGamma)
S3method(knitr::knit_print,RMlocdepGamma)
export(RMUreliability)
export(RMdifGamma)
export(RMdifGammaCutoff)
export(RMdifGammaPlot)
export(RMdifLR)
export(RMdifTree)
export(RMdimCFA)
export(RMdimCFACutoff)
export(RMdimCFAPlot)
export(RMdimMartinLof)
export(RMdimMartinLofResiduals)
export(RMdimResidualPCA)
export(RMdimResidualPCACutoff)
export(RMitemCatProb)
export(RMitemHierarchy)
export(RMitemICCPlot)
export(RMitemInfit)
export(RMitemInfitCutoff)
export(RMitemInfitCutoffMI)
export(RMitemInfitCutoffPlot)
export(RMitemInfitMI)
export(RMitemInfitPlot)
export(RMitemParameters)
export(RMitemRestscore)
export(RMitemRestscoreBoot)
export(RMlocdepGamma)
export(RMlocdepGammaCutoff)
export(RMlocdepGammaPlot)
export(RMlocdepQ3)
export(RMlocdepQ3Cutoff)
export(RMlocdepQ3Plot)
export(RMpersonFit)
export(RMpersonParameters)
export(RMplotBar)
export(RMplotStackedbar)
export(RMplotTile)
export(RMreliability)
export(RMscoreSE)
export(RMtargeting)
importFrom(rlang,.data)

# easyRasch2

<!-- badges: start -->
[![CRAN Version](https://www.r-pkg.org/badges/version/easyRasch2)](https://cran.r-project.org/package=easyRasch2)
[![Downloads](https://cranlogs.r-pkg.org/badges/easyRasch2?color=brightgreen)](https://CRAN.R-project.org/package=easyRasch2)
![Downloads Status](https://cranlogs.r-pkg.org/badges/grand-total/easyRasch2)
[![R-CMD-check](https://github.com/pgmj/easyRasch2/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/pgmj/easyRasch2/actions/workflows/R-CMD-check.yaml)
[![Codecov test coverage](https://codecov.io/gh/pgmj/easyRasch2/graph/badge.svg)](https://app.codecov.io/gh/pgmj/easyRasch2)
<a href="https://buymeacoffee.com/pgmj" target="_blank"><img src="https://cdn.buymeacoffee.com/buttons/default-orange.png" alt="Buy Me A Coffee" height="41" width="174"></a>
<!-- badges: end -->

`easyRasch2` is an R package for Rasch measurement theory
analysis workflows. It is the successor to
[`easyRasch`](https://pgmj.github.io/easyRasch/), offering a lightweight and consistent structure with proper namespacing and minimal dependencies.

A central design choice is **simulation-based critical values** for various fit
statistics. Rather than relying on rule-of-thumb cutoffs, most diagnostics
are paired with a parametric-bootstrap function that generates an empirical
null distribution from the fitted Rasch / PCM model and the observed
sample.

The [Get Started](https://pgmj.github.io/easyRasch2/articles/easyRasch2.html) link above contains a short introduction. For broader Rasch-analysis tutorials, see the
[vignette](https://pgmj.github.io/raschrvignette/RaschRvign.html) for the
archived sibling package [`easyRasch`](https://pgmj.github.io/easyRasch/).

## Statement of need

A complete Rasch analysis requires many separate procedures — item fit, local
dependence, dimensionality, differential item functioning, reliability,
targeting, and more. In R these are spread across packages with differing data formats,
argument conventions, and output objects, which raises the barrier to entry and
can make analyses hard to reproduce. A further problem is that fit statistics
(item fit MSQ, Yen's $Q_3$ residuals, the first residual-PCA contrast, CFA fit indices) are
usually judged against fixed rule-of-thumb cutoffs that are known to depend on
sample size, number of items and other factors such as targeting, and the number of response categories.

`easyRasch2` targets applied researchers and students validating rating scales
and tests in health, education, and psychology using modern psychometric methods. It provides a single,
consistently named interface across the whole workflow with publication-ready
output, and — as its distinguishing feature — replaces rule-of-thumb cutoffs
with sample-specific critical values obtained by parametric bootstrap from the
fitted Rasch/PCM model (Johansson, 2025). Several methods, including the
polytomous Martin-Löf test with Monte Carlo *p*-values (Christensen & Kreiner, 2007) and the bootstrap
item-restscore test, are not available in other R packages.

## Installation

Install from CRAN:

```r
install.packages("easyRasch2")
```

Install the development version from GitHub:

```r
# install.packages("remotes") # if needed
remotes::install_github("pgmj/easyRasch2")
```

## Key design principles

- **Estimation**: a single engine across the package — Conditional Maximum
  Likelihood (CML) item parameters via `psychotools` and Warm's Weighted
  Likelihood Estimation (WLE) for person parameters. `eRm` is used for
  Andersen's LR test (`RMdifLR()`); `mirt` (MML) is available as an optional
  engine (`estimator = "MML"` in `RMlocdepQ3()` and `RMitemParameters()`)
  and for the plausible values behind the RMU reliability metric.
- **Inference**: simulation-based cutoffs throughout, with optional
  bootstrap *p*-values (`p_value = TRUE`) using Westfall–Young family-wise
  correction (default) or FDR alternatives (Ferreira, 2024).
- **Output**: `knitr::kable()` for tables (Quarto-friendly), `ggplot2`
  for figures, and `"dataframe"` output options for downstream use.
  Every caption reports the estimation sample size and missing-data policy.
- **Naming**: Functions use the `RM` prefix (e.g., `RMlocdepQ3()`).

## Functions by domain

### Item fit

- `RMitemInfit()` — conditional infit MSQ; optional bootstrap *p*-values
  (`p_value = TRUE`) with family-wise (Westfall–Young) or FDR multiple-comparison
  correction
- `RMitemInfitCutoff()` + `RMitemInfitPlot()` — simulation-based cutoffs and plot
- `RMitemInfitMI()` + `RMitemInfitCutoffMI()` — multiple-imputation variants
- `RMitemRestscore()` — item-restscore with Goodman-Kruskal's $\gamma$ (gamma)
- `RMitemRestscoreBoot()` — non-parametric bootstrap of item-restscore fit
- `RMitemICCPlot()` - conditional item characteristic curves

### Local dependence

- `RMlocdepQ3()` + `RMlocdepQ3Cutoff()` + `RMlocdepQ3Plot()` — Yen's $Q_3$
  residual correlations (CML/WLE by default, `estimator = "MML"` optional);
  table and plot share a `$matrix` ($Q_3$ heatmap) / `$pairs` (per-pair
  observed-vs-simulated) structure; optional per-pair bootstrap *p*-values
- `RMlocdepGamma()` + `RMlocdepGammaCutoff()` + `RMlocdepGammaPlot()` —
  partial-$\gamma$ local dependence; optional per-pair bootstrap *p*-values

### Dimensionality / unidimensionality

- `RMdimResidualPCA()` + `RMdimResidualPCACutoff()` — PCA of standardized residuals,
  with simulation-based first-contrast cutoff (Chou & Wang, 2010) and an
  optional bootstrap *p*-value
- `RMdimMartinLof()` + `RMdimMartinLofResiduals()` — Martin-Löf LR test
  (Christensen & Kreiner, 2007), supports polytomous data with Monte Carlo *p*-values
- `RMdimCFACutoff()` + `RMdimCFA()` + `RMdimCFAPlot()` — posterior-predictive CFA
  fit-index and per-item loading checks under PCM unidimensionality (via `lavaan`
  WLSMV) with simulation-based cutoffs and optional bootstrap *p*-values

### Differential item functioning

- `RMdifLR()` — Andersen's likelihood-ratio test (`eRm::LRtest`)
- `RMdifTree()` — Rasch / partial-credit trees (`psychotree`) with
  Mantel-Haenszel or partial-$\gamma$ effect sizes per split, optional
  iterative purification, and `stablelearner`-based stability assessment
- `RMdifGamma()` + `RMdifGammaCutoff()` + `RMdifGammaPlot()` — partial-$\gamma$
  DIF; optional bootstrap *p*-values calibrated against the simulated Rasch null
- `RMitemICCPlot()` - evaluates DIF across class intervals

### Item category threshold ordering

- `RMitemCatProb()` for classic item probability function trace plots.
- `RMitemHierarchy()` renders a plot sorting items based on difficulty, showing
  item threshold locations and confidence intervals.

### Reliability, targeting, score conversion

- `RMreliability()` + `RMUreliability()` — Cronbach's α, PSI, marginal
  reliability, and Relative Measurement Uncertainty from plausible values
- `RMtargeting()` — Wright-map style person-item targeting plot
- `RMscoreSE()` — raw-score → logit transformation table (WLE / EAP)

### Item & person parameters

- `RMitemParameters()` — item difficulty / threshold locations in long or wide
  format, with optional standard errors and confidence intervals (CML via
  `psychotools`, or MML via `mirt` for sparse data)
- `RMpersonParameters()` — per-respondent person locations (WLE or EAP),
  estimated on each response pattern so partial missingness is handled directly

### Person fit

- `RMpersonFit()` — per-respondent conditional infit / outfit MSQ and the
  standardized log-likelihood $\ell_z$, with resampling-based *p*-values rather
  than unreliable asymptotic nulls (Sinharay, 2016; Müller, 2020)

### Data visualization

- `RMplotTile()` — response-distribution heatmap with optional group faceting
- `RMplotBar()` & `RMplotStackedbar()`


## Example

```r
library(easyRasch2)
data("pcmdat2", package = "eRm")
options(mc.cores = 4)
set.seed(42)

# Conditional item infit with simulation-based cutoffs
simfit <- RMitemInfitCutoff(pcmdat2, iterations = 250)
RMitemInfit(pcmdat2, cutoff = simfit)

# Test of unidimensionality via posterior-predictive ordinal CFA
cfa_sim <- RMdimCFACutoff(pcmdat2, iterations = 250)   # simulated reference
tabs <- RMdimCFA(pcmdat2, cutoff = cfa_sim)            # observed vs expected
tabs$fit                                                # fit-index table
tabs$loadings                                           # per-item loading table
plots <- RMdimCFAPlot(cfa_sim, data = pcmdat2)          # list of 2 ggplots
plots$loadings                                          # observed vs expected loadings
plots$fit                                               # fit-index distributions

# DIF analysis via Andersen's LR test
grp <- factor(sample(c("A", "B"), nrow(pcmdat2), replace = TRUE))
RMdifLR(pcmdat2, dif_var = grp)

# Rasch-tree DIF with effect-size classification on continuous +
# categorical covariates simultaneously
covs <- data.frame(
  group = grp,
  band  = sample(c("low", "high"), nrow(pcmdat2), replace = TRUE)
)
RMdifTree(pcmdat2, covariates = covs)
```

## References

- Bjorner, J. B., Kreiner, S., Ware, J. E., Damsgaard, M. T., & Bech, P. (1998).
  Differential item functioning in the Danish translation of the SF-36.
  *Journal of Clinical Epidemiology, 51*(11), 1189–1202.
  <https://doi.org/10.1016/S0895-4356(98)00111-5>
- Chou, Y.-T., & Wang, W.-C. (2010). Checking dimensionality in item-response
  models with principal component analysis on standardized residuals.
  *Educational and Psychological Measurement, 70*(5), 717–731.
  <https://doi.org/10.1177/0013164410379322>
- Christensen, K. B., & Kreiner, S. (2007). A Monte Carlo approach to
  unidimensionality testing in polytomous Rasch models.
  *Applied Psychological Measurement, 31*(1), 20–30.
  <https://doi.org/10.1177/0146621605286204>
- Christensen, K. B., Makransky, G., & Horton, M. (2017). Critical values for
  Yen's Q3: Identification of local dependence in the Rasch model using
  residual correlations. *Applied Psychological Measurement, 41*(3), 178–194.
  <https://doi.org/10.1177/0146621616677520>
- Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number of
  hypotheses simultaneously: An account focusing on the control of the
  probability of at least one incorrect rejection and of the false discovery
  rate. *Journal of Statistical Theory and Practice, 19*(6).
  <https://doi.org/10.1007/s42519-024-00412-4>
- Henninger, M., Debelak, R., & Strobl, C. (2023). A new stopping criterion
  for Rasch trees based on the Mantel-Haenszel effect size measure for DIF.
  *Educational and Psychological Measurement, 83*, 181–212.
  <https://doi.org/10.1177/00131644221077135>
- Henninger, M., Radek, J., Debelak, R., & Strobl, C. (2025). Partial credit
  trees meet the partial gamma coefficient for quantifying DIF and DSF in
  polytomous items. *Behaviormetrika, 52*, 221–257.
  <https://doi.org/10.1007/s41237-024-00252-3>
- Johansson, M. (2025). Detecting item misfit in Rasch models.
  *Educational Methods & Psychometrics, 3*(18).
  <https://doi.org/10.61186/emp.2025.5>
- Kreiner, S. (2011). A note on item-restscore association in Rasch models.
  *Applied Psychological Measurement, 35*(7), 557–561.
  <https://doi.org/10.1177/0146621611410227>
- Müller, M. (2020). Item fit statistics for Rasch analysis: Can we trust them?
  *Journal of Statistical Distributions and Applications, 7*(1), 5.
  <https://doi.org/10.1186/s40488-020-00108-7>
- Philipp, M., Rusch, T., Hornik, K., & Strobl, C. (2018). Measuring the
  stability of results from supervised statistical learning. *Journal of
  Computational and Graphical Statistics, 27*, 685–700.
  <https://doi.org/10.1080/10618600.2018.1473779>
- Rosseel, Y. (2012). lavaan: An R package for structural equation modeling.
  *Journal of Statistical Software, 48*(2), 1–36.
  <https://doi.org/10.18637/jss.v048.i02>
- Sinharay, S. (2016). Assessment of person fit using resampling-based
  approaches. *Journal of Educational Measurement, 53*(1), 63–85.
  <https://doi.org/10.1111/jedm.12101>
- Strobl, C., Kopf, J., & Zeileis, A. (2015). Rasch trees: A new method for
  detecting DIF in the Rasch model. *Psychometrika, 80*, 289–316.
  <https://doi.org/10.1007/s11336-013-9388-3>


## Credits

As mentioned earlier, this is based on my `easyRasch` package, and I am using
Claude Opus/Fable to rewrite functions to this more properly formatted package.
While it uses my earlier code, most of the code in this package is produced by
the LLM and tested and bug fixed by me.

`RMdifTree()` adapts MIT-licensed code from Mirka Henninger and Jan Radek's
[`raschtreeMH`](https://github.com/mirka-henninger/raschtreeMH) and
[`effecttree`](https://github.com/mirka-henninger/effecttree) packages for
the effect-size and ETS-classification algorithms.

[Magnus Johansson](https://ki.se/en/people/magnus-johansson-3) is a licensed psychologist with a PhD in behavior analysis. He works as a research specialist focused on psychometrics and statistics at Karolinska Institutet, Department of Clinical Neuroscience, Center for Psychiatry Research.

- ORCID: [0000-0003-1669-592X](https://orcid.org/0000-0003-1669-592X)
- Bluesky: [@pgmj.bsky.social](https://bsky.app/profile/pgmj.bsky.social) 

## License

GPL (>= 3)

# Internal simulation helpers for parametric bootstrap functions

#' Simulate responses for a single polytomous item
#'
#' @param deltas Numeric vector of threshold parameters.
#' @param thetas Numeric vector of person parameters.
#' @return Integer vector of simulated responses (0-indexed).
#' @noRd
sim_poly_item <- function(deltas, thetas) {
  k <- length(deltas) + 1L
  n <- length(thetas)
  Y <- outer(thetas, deltas, "-")
  cumsums <- t(rbind(rep(0, times = n), apply(X = Y, MARGIN = 1, FUN = cumsum)))
  expcumsums <- exp(cumsums)
  norms <- apply(X = expcumsums, MARGIN = 1, FUN = sum)
  z <- expcumsums / norms
  vapply(X = seq_len(n), FUN = function(x) {
    sample(x = 0L:(k - 1L), size = 1L, replace = TRUE, prob = z[x, ])
  }, FUN.VALUE = 1L)
}

#' Simulate partial score (polytomous) response matrix
#'
#' @param deltaslist List of threshold parameter vectors, one per item.
#' @param thetavec Numeric vector of person parameters.
#' @return Integer matrix of simulated responses (rows = persons, cols = items).
#' @noRd
sim_partial_score <- function(deltaslist, thetavec) {
  deltaslist <- lapply(X = deltaslist, FUN = unlist)
  sapply(X = deltaslist, FUN = sim_poly_item, thetas = thetavec)
}

# CML item thresholds are now obtained via `.fit_cml_thresholds()` (psychotools,
# grand-mean centred; see utils-theta.R), which returns the per-item threshold
# list the bootstrap generators consume directly. The former eRm-based
# `extract_item_thresholds()` matrix helper has been removed.

test_that("sim_poly_item returns integer vector of correct length", {
  deltas <- c(-1, 0, 1)
  thetas <- rnorm(50)
  result <- easyRasch2:::sim_poly_item(deltas, thetas)
  expect_length(result, 50)
  expect_true(all(result >= 0L & result <= length(deltas)))
})

test_that("sim_partial_score returns matrix with correct dimensions", {
  set.seed(1)
  deltaslist <- list(c(-1, 1), c(-0.5, 0.5), c(-2, 0, 2))
  thetavec <- rnorm(30)
  result <- easyRasch2:::sim_partial_score(deltaslist, thetavec)
  expect_equal(dim(result), c(30L, 3L))
})

test_that("sim_poly_item responses are within valid range", {
  set.seed(42)
  deltas <- c(-1, 1)   # 3-category item
  thetas <- rnorm(100)
  result <- easyRasch2:::sim_poly_item(deltas, thetas)
  expect_true(all(result %in% 0:2))
})

test_that(".fit_cml_thresholds returns a per-item threshold list (poly)", {
  set.seed(7)
  poly_data <- as.data.frame(
    matrix(sample(0:2, 120, replace = TRUE), nrow = 30, ncol = 4)
  )
  thr <- easyRasch2:::.fit_cml_thresholds(poly_data)
  expect_type(thr, "list")
  # One element per item, each with (categories - 1) = 2 thresholds
  expect_length(thr, 4L)
  expect_true(all(vapply(thr, length, integer(1L)) == 2L))
  # Grand-mean centred
  expect_equal(mean(unlist(thr)), 0, tolerance = 1e-8)
})

# Tree-based DIF analysis with effect-size classification and optional
# stability assessment.
#
# Acknowledgement
# ----------------
# Several internal helpers below (Mantel-Haenszel effect-size with
# iterative purification, partial-gamma effect-size with iterative
# purification, ETS A/B/C classification, partial-gamma A/B/C
# classification, walking the partykit tree to obtain per-split group
# assignments) follow the algorithms implemented by
# Mirka Henninger and Jan Radek in the GitHub-only packages
#   * raschtreeMH (https://github.com/mirka-henninger/raschtreeMH)
#   * effecttree  (https://github.com/mirka-henninger/effecttree)
# both released under the MIT licence:
#
#   Copyright (c) Mirka Henninger and Jan Radek
#
#   Permission is hereby granted, free of charge, to any person
#   obtaining a copy of this software and associated documentation
#   files (the "Software"), to deal in the Software without
#   restriction, including without limitation the rights to use, copy,
#   modify, merge, publish, distribute, sublicense, and/or sell copies
#   of the Software, and to permit persons to whom the Software is
#   furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be
#   included in all copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND.
#
# The MIT-licensed code from those packages has been adapted here for
# easyRasch2's API and combined with the rest of the package, which is
# distributed under GPL (>= 3) (the combined work is therefore GPL).
#
# References
#  - Henninger, M., Debelak, R., & Strobl, C. (2023). A new stopping
#    criterion for Rasch trees based on the Mantel-Haenszel effect size
#    measure for DIF. Educational Psychological Measurement, 83, 181-212.
#    doi:10.1177/00131644221077135
#  - Henninger, M., Radek, J., Debelak, R., & Strobl, C. (2025). Partial
#    credit trees meet the partial gamma coefficient for quantifying DIF
#    and DSF in polytomous items. Behaviormetrika, 52, 221-257.
#  - Strobl, C., Kopf, J., & Zeileis, A. (2015). Rasch trees: A new
#    method for detecting differential item functioning in the Rasch
#    model. Psychometrika, 80, 289-316.
#  - Komboz, B., Strobl, C., & Zeileis, A. (2018). Tree-based global
#    model tests for polytomous Rasch models. Educational and
#    Psychological Measurement, 78, 128-166.
#  - Philipp, M., Rusch, T., Hornik, K., & Strobl, C. (2018). Measuring
#    the stability of results from supervised statistical learning.
#    Journal of Computational and Graphical Statistics, 27, 685-700.

#' Tree-based DIF analysis with effect-size classification
#'
#' Detects Differential Item Functioning (DIF) by recursively splitting
#' the sample on one or more covariates using \code{psychotree::raschtree()}
#' (dichotomous data) or \code{psychotree::pctree()} (polytomous data),
#' then computes per-split effect-size measures for every item -- the
#' Mantel-Haenszel odds-ratio (Holland & Thayer, 1986), on the Delta scale
#' developed at the Educational Testing Service (ETS; Zwick, 2012), for
#' dichotomous data, or the partial gamma coefficient for polytomous data -- and
#' classifies them into ETS A/B/C categories (Bjorner et al., 1998). Optionally,
#' an iterative purification step (Henninger et al., 2025) is applied, and tree
#' stability across resamples can be assessed via
#' \code{stablelearner::stabletree()} (Philipp et al., 2018).
#'
#' Continuous covariates (e.g., age in years) and interactions among
#' multiple covariates are handled natively by the model-based
#' recursive-partitioning machinery of \code{partykit} / \code{psychotree}.
#'
#' @param data A data.frame or matrix of item responses (non-negative
#'   integers, 0-based). One column per item. Items only -- pass
#'   covariates separately via \code{covariates}.
#' @param covariates A data.frame with \code{nrow(data)} rows giving the
#'   DIF covariates. Columns may be numeric (continuous), factor, ordered
#'   factor, or logical. At least one column is required.
#' @param model One of \code{"auto"} (default), \code{"PCM"}, or
#'   \code{"RM"}. \code{"auto"} picks RM when the data are dichotomous
#'   (max response = 1) and PCM otherwise.
#' @param effect_size One of \code{"auto"} (default), \code{"MH"}
#'   (Mantel-Haenszel), or \code{"pgamma"} (partial gamma). \code{"auto"}
#'   uses MH for RM and partial gamma for PCM.
#' @param purification \code{"none"} (default) or \code{"iterative"}.
#'   Iterative purification recomputes the effect size while excluding
#'   items already classified as DIF from the matching score (Bjorner et
#'   al., 1998).
#' @param p_adj Multiple-testing adjustment for the partial-gamma
#'   classification: \code{"none"} (default), \code{"fdr"} (Benjamini &
#'   Hochberg), or \code{"bonferroni"}. Ignored for MH (which uses the
#'   ETS test directly).
#' @param thresholds Numeric length-2 vector with the partial-gamma
#'   B/C boundaries (default \code{c(0.21, 0.31)}). Ignored for MH (the
#'   ETS Delta-scale boundaries 1.0 / 1.5 are used).
#' @param alpha Significance level for the A/C tests. Default
#'   \code{0.05}.
#' @param prune_negligible Logical. If \code{TRUE}, splits whose every
#'   item is classified as A are pruned from the tree. Default
#'   \code{FALSE}.
#' @param stability Logical. If \code{TRUE}, runs
#'   \code{stablelearner::stabletree()} on the fitted tree to assess
#'   variable-selection and cutpoint stability across resamples.
#'   Default \code{FALSE}.
#' @param stability_B Integer. Number of resamples for stability
#'   assessment. Default \code{100}.
#' @param stability_sampler One of \code{"subsampling"} (default) or
#'   \code{"bootstrap"}. Subsampling is recommended for tree stability
#'   because bootstrap resamples can drop levels of categorical
#'   covariates, breaking the model fit on small subsamples.
#' @param min_n_per_level Integer. Minimum count required for any factor
#'   level in \code{covariates}. Default \code{20}. Set to \code{0} to
#'   disable.
#' @param on_rescale One of \code{"message"} (default), \code{"warning"},
#'   or \code{"stop"}. Controls how the function reports the situation
#'   in which one or more items have no responses in their lowest
#'   category within a terminal node of the tree (psychotree silently
#'   rescales those items per node, which leaves the per-node item
#'   parameters on a different metric). \emph{The MH and partial-gamma
#'   effect sizes reported here are computed on the raw responses and
#'   are not affected.} The diagnostic concerns only the per-node item
#'   parameters that are visible via \code{plot(tree)}.
#' @param output One of \code{"kable"} (default), \code{"dataframe"},
#'   \code{"tree"}, \code{"plot"}, or \code{"list"}. \code{"kable"}
#'   renders the
#'   per-split per-item effect-size table as a \code{knitr::kable()};
#'   \code{"dataframe"} returns the underlying tidy data.frame;
#'   \code{"tree"} returns the augmented partykit tree object;
#'   \code{"plot"} returns the partykit tree plot with item names on
#'   the terminal-node x-axis (\code{tp_args = list(names = TRUE)});
#'   \code{"list"} returns both views from a single fit, as
#'   \code{list(table = <data.frame>, tree = <tree object>)}.
#'   For full control over the plot, use \code{output = "tree"} and
#'   call \code{plot()} on the result with your own \code{tp_args}.
#'   When
#'   \code{stability = TRUE}, the stability summary is attached as
#'   \code{attr(result, "stability")} (a small data.frame) and
#'   \code{attr(result, "stability_kable")} (pre-rendered kable).
#' @param ... Additional arguments forwarded to
#'   \code{psychotree::raschtree()} or \code{psychotree::pctree()}, e.g.
#'   \code{minsize}, \code{alpha} for the parameter-instability test.
#'   Note: this \code{alpha} (the tree-fitting argument) is independent
#'   of the \code{alpha} for ETS classification above; pass it via
#'   \code{...}.
#'
#' @return Depending on \code{output}:
#' \describe{
#'   \item{\code{"kable"}}{A \code{knitr_kable} object with one row per
#'     (split node x item), grouped by node via \code{pack_rows}-style
#'     section headers. The caption summarises model, effect-size
#'     measure, purification, and (if requested) stability.}
#'   \item{\code{"dataframe"}}{A data.frame with one row per (split node
#'     x item): columns \code{NodeID}, \code{Split} (human-readable
#'     description of the split), \code{Variable}, \code{Direction},
#'     \code{Item}, \code{EffectSize}, \code{SE}, \code{Class} (A/B/C),
#'     \code{Flagged} (TRUE for B or C), \code{n_left}, \code{n_right}.}
#'   \item{\code{"tree"}}{The fitted partykit tree object with class
#'     \code{c("RMdifTree", ...)}, with effect-size results stored at
#'     \code{tree$info$effectsize}.}
#'   \item{\code{"plot"}}{A plotted partykit tree.}
#'   \item{\code{"list"}}{A list with both views from a single fit:
#'     \code{table} (the \code{"dataframe"} result, with its attributes)
#'     and \code{tree} (the \code{"tree"} result).}
#' }
#'
#' Stability results (when \code{stability = TRUE}) are attached to the
#' return value as \code{attr(result, "stability")} (data.frame) and
#' \code{attr(result, "stability_kable")} (pre-rendered kable).
#'
#' @details
#' \strong{ETS Delta-scale Mantel-Haenszel.} The MH common odds ratio
#' \eqn{\hat{\alpha}_{MH}} is mapped to the ETS Delta scale via
#' \eqn{\Delta_{MH} = -2.35 \log \hat{\alpha}_{MH}}. The classification
#' rules (Holland & Thayer, 1988; Zwick, 2012) are:
#' \itemize{
#'   \item Class \strong{A}: \eqn{|\Delta_{MH}| < 1} or test of
#'     \eqn{\Delta_{MH} = 0} not rejected at \code{alpha}.
#'   \item Class \strong{C}: \eqn{|\Delta_{MH}| \ge 1.5} and test of
#'     \eqn{|\Delta_{MH}| \le 1} rejected at \code{alpha}.
#'   \item Class \strong{B}: otherwise.
#' }
#' easyRasch2 uses the sign convention of \code{effecttree}: positive
#' \eqn{\Delta} indicates that the item is more difficult for the second
#' (reference) group.
#'
#' \strong{Partial gamma.} \code{iarm::partgam_DIF()} provides the gamma
#' estimate and SE per item. ETS-style classification follows Bjorner et
#' al. (1998): A if \eqn{|\gamma| < 0.21} or the test of \eqn{\gamma = 0}
#' is not rejected at \code{alpha}; C if \eqn{|\gamma| > 0.31} and the
#' test of \eqn{|\gamma| \le 0.21} is rejected at \code{alpha}; B
#' otherwise.
#'
#' \strong{Rule-of-thumb caveat.} The A/B/C boundaries (Delta 1.0 / 1.5;
#' gamma 0.21 / 0.31) are conventions carried over from large-scale
#' educational testing, not values calibrated to the sample and items at
#' hand -- in contrast to the simulation-based cutoffs used elsewhere in
#' this package -- so the classification is best read as a rough
#' magnitude guide rather than a calibrated test. For a sample-calibrated
#' partial-gamma DIF test, see [RMdifGamma()] with an
#' [RMdifGammaCutoff()] object (optionally with `p_value = TRUE`).
#'
#' \strong{Continuous and interaction effects.} The recursive
#' partitioning step automatically handles continuous covariates (the
#' parameter-instability test searches for the optimal cutpoint) and
#' multivariate interactions (later splits are conditional on earlier
#' ones). To assess sensitivity of variable selection and cutpoints to
#' the particular sample, set \code{stability = TRUE}.
#'
#' \strong{Stability assessment.} When \code{stability = TRUE}, the same
#' tree-fitting call is replayed on \code{stability_B} resamples of the
#' data. The returned \code{stabletree} object reports, per covariate,
#' the proportion of resamples in which it was selected at any split,
#' and (for continuous covariates) the empirical distribution of
#' cutpoints. Stability is independent of effect-size classification --
#' a stable split with a negligible effect is still negligible, and a
#' large effect on an unstable split should be interpreted with care.
#' Cost is roughly \code{stability_B} times the original fitting time.
#'
#' \strong{Acknowledgement.} The effect-size machinery -- MH and partial
#' gamma per node, iterative purification, ETS A/B/C classification --
#' follows the implementations by Henninger & Radek in the GitHub
#' packages \emph{raschtreeMH} and \emph{effecttree}. The relevant code
#' has been adapted here under the MIT licence (see file header).
#'
#' @references
#' Bjorner, J. B., Kreiner, S., Ware, J. E., Damsgaard, M. T., & Bech,
#' P. (1998). Differential item functioning in the Danish translation
#' of the SF-36. \emph{Journal of Clinical Epidemiology, 51}(11),
#' 1189-1202. \doi{10.1016/S0895-4356(98)00111-5}
#'
#' Henninger, M., Debelak, R., & Strobl, C. (2023). A new stopping
#' criterion for Rasch trees based on the Mantel-Haenszel effect size
#' measure for DIF. \emph{Educational and Psychological Measurement,
#' 83}, 181-212. \doi{10.1177/00131644221077135}
#'
#' Henninger, M., Radek, J., Debelak, R., & Strobl, C. (2025). Partial
#' credit trees meet the partial gamma coefficient for quantifying DIF
#' and DSF in polytomous items. \emph{Behaviormetrika, 52}, 221-257.
#' \doi{10.1007/s41237-024-00252-3}
#'
#' Holland, P. W., & Thayer, D. T. (1986). Differential item performance
#' and the Mantel-Haenszel procedure.
#' \emph{ETS Research Report Series}(2).
#' \doi{10.1002/j.2330-8516.1986.tb00186.x}
#'
#' Philipp, M., Rusch, T., Hornik, K., & Strobl, C. (2018). Measuring
#' the stability of results from supervised statistical learning.
#' \emph{Journal of Computational and Graphical Statistics, 27},
#' 685-700. \doi{10.1080/10618600.2018.1473779}
#'
#' Asamoah, N. A. B., Turner, R. C., Lo, W.-J., Crawford, B. L., & Jozkowski,
#' K. N. (2025). Impacts of DIF Item Balance and Effect Size Incorporation With
#' the Rasch Tree. \emph{Educational and Psychological Measurement}.
#' \doi{10.1177/00131644251370605}
#'
#' Zwick, R. (2012). A review of ETS differential item functioning
#' assessment procedures: Flagging rules, minimum sample size
#' requirements, and criterion refinement. \emph{ETS Research Report
#' Series}(1). \doi{10.1002/j.2333-8504.2012.tb02290.x}
#'
#' @seealso \code{\link{RMdifLR}}, \code{\link{RMdifGamma}}
#'
#' @examples
#' \donttest{
#' if (requireNamespace("psychotree", quietly = TRUE) &&
#'     requireNamespace("partykit", quietly = TRUE) &&
#'     requireNamespace("difR", quietly = TRUE) &&
#'     requireNamespace("iarm", quietly = TRUE)) {
#'   data("DIFSimPC", package = "psychotree")
#'
#'   items <- as.data.frame(as.matrix(DIFSimPC$resp))
#'   covs  <- DIFSimPC[, c("age", "gender", "motivation")]
#'
#'   # Default: kable of per-split effect sizes
#'   RMdifTree(items, covariates = covs)
#'
#'   # Tidy data.frame -- one row per (split node x item)
#'   df <- RMdifTree(items, covariates = covs, output = "dataframe")
#'   df[df$Flagged, ]
#'
#'   # Tree object (for plotting via partykit::plot)
#'   tree <- RMdifTree(items, covariates = covs, output = "tree")
#'   plot(tree)
#'
#'   # Stability assessment refits the tree on B resamples.
#'   # (use more resamples, e.g. 100+, in real analyses)
#'   if (requireNamespace("stablelearner", quietly = TRUE)) {
#'     kbl <- RMdifTree(items, covariates = covs,
#'                      purification = "iterative", p_adj = "fdr",
#'                      stability = TRUE, stability_B = 25)
#'     kbl                          # main effect-size kable
#'     attr(kbl, "stability_kable") # pre-rendered stability kable
#'     attr(kbl, "stability")       # raw stability data.frame
#'   }
#' }
#' }
#'
#' @export
RMdifTree <- function(
  data,
  covariates,
  model = c("auto", "PCM", "RM"),
  effect_size = c("auto", "MH", "pgamma"),
  purification = c("none", "iterative"),
  p_adj = c("none", "fdr", "bonferroni"),
  thresholds = c(0.21, 0.31),
  alpha = 0.05,
  prune_negligible = FALSE,
  stability = FALSE,
  stability_B = 100L,
  stability_sampler = c("subsampling", "bootstrap"),
  min_n_per_level = 20L,
  on_rescale = c("message", "warning", "stop"),
  output = c("kable", "dataframe", "tree", "plot", "list"),
  ...
) {
  # Capture the unevaluated `covariates` expression so we can recover a
  # readable column name when the user passes a single vector like
  # RMdifTree(d, dif$age) instead of a data.frame.
  covariates_expr <- substitute(covariates)

  model <- match.arg(model)
  effect_size <- match.arg(effect_size)
  purification <- match.arg(purification)
  p_adj <- match.arg(p_adj)
  stability_sampler <- match.arg(stability_sampler)
  on_rescale <- match.arg(on_rescale)
  output <- match.arg(output)

  # ---------------------------------------------------------------------
  # Package availability
  # ---------------------------------------------------------------------
  if (!requireNamespace("psychotree", quietly = TRUE)) {
    stop(
      "Package 'psychotree' is required for RMdifTree(). ",
      "Install it with: install.packages(\"psychotree\")",
      call. = FALSE
    )
  }
  if (!requireNamespace("partykit", quietly = TRUE)) {
    stop("Package 'partykit' is required for RMdifTree().", call. = FALSE)
  }
  if (!requireNamespace("difR", quietly = TRUE)) {
    stop(
      "Package 'difR' is required for RMdifTree() (Mantel-Haenszel ",
      "effect size). Install it with: install.packages(\"difR\")",
      call. = FALSE
    )
  }
  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required for RMdifTree() (partial-gamma ",
      "effect size). Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }
  if (stability && !requireNamespace("stablelearner", quietly = TRUE)) {
    stop(
      "Package 'stablelearner' is required for stability = TRUE. ",
      "Install it with: install.packages(\"stablelearner\")",
      call. = FALSE
    )
  }

  # ---------------------------------------------------------------------
  # Validate items
  # ---------------------------------------------------------------------
  validate_response_data(data)
  data <- as.data.frame(data)
  n_total_dt <- nrow(data)

  # ---------------------------------------------------------------------
  # Validate covariates
  # ---------------------------------------------------------------------
  if (missing(covariates) || is.null(covariates)) {
    stop(
      "`covariates` is required: a data.frame of one or more DIF ",
      "covariates with nrow(data) rows.",
      call. = FALSE
    )
  }
  if (!is.data.frame(covariates)) {
    if (is.atomic(covariates) || is.list(covariates)) {
      derived_name <- derive_covariate_name(covariates_expr)
      covariates <- data.frame(x = covariates, stringsAsFactors = FALSE)
      names(covariates) <- derived_name
    } else {
      stop("`covariates` must be a data.frame.", call. = FALSE)
    }
  }
  if (nrow(covariates) != nrow(data)) {
    stop(
      "`covariates` must have the same number of rows as `data` (",
      nrow(data),
      "). Got ",
      nrow(covariates),
      ".",
      call. = FALSE
    )
  }
  if (ncol(covariates) < 1L) {
    stop("`covariates` must contain at least one column.", call. = FALSE)
  }
  validate_covariates(covariates, min_n_per_level = min_n_per_level)

  # ---------------------------------------------------------------------
  # Resolve model and effect_size
  # ---------------------------------------------------------------------
  is_polytomous <- max(as.matrix(data), na.rm = TRUE) > 1L

  resolved_model <- if (model == "auto") {
    if (is_polytomous) "PCM" else "RM"
  } else {
    if (model == "RM" && is_polytomous) {
      stop(
        "`model = \"RM\"` requires dichotomous data; use \"PCM\" or ",
        "\"auto\".",
        call. = FALSE
      )
    }
    model
  }

  resolved_es <- if (effect_size == "auto") {
    if (resolved_model == "RM") "MH" else "pgamma"
  } else {
    if (effect_size == "MH" && resolved_model == "PCM") {
      stop(
        "`effect_size = \"MH\"` is only defined for dichotomous data ",
        "(model = \"RM\"). Use \"pgamma\" or \"auto\".",
        call. = FALSE
      )
    }
    effect_size
  }

  # ---------------------------------------------------------------------
  # Validate effect-size args
  # ---------------------------------------------------------------------
  if (
    !is.numeric(thresholds) ||
      length(thresholds) != 2L ||
      any(!is.finite(thresholds)) ||
      thresholds[1L] >= thresholds[2L] ||
      thresholds[1L] <= 0
  ) {
    stop(
      "`thresholds` must be a length-2 numeric vector with ",
      "0 < thresholds[1] < thresholds[2].",
      call. = FALSE
    )
  }
  if (!is.numeric(alpha) || length(alpha) != 1L || alpha <= 0 || alpha >= 1) {
    stop("`alpha` must be a single numeric in (0, 1).", call. = FALSE)
  }
  if (
    !is.numeric(stability_B) || length(stability_B) != 1L || stability_B < 1
  ) {
    stop("`stability_B` must be a positive integer.", call. = FALSE)
  }
  stability_B <- as.integer(stability_B)

  # ---------------------------------------------------------------------
  # Build the combined data.frame in the form psychotree expects:
  # one column called `.items` of class matrix on the LHS, covariates on
  # the RHS. Drop rows where any covariate is NA (psychotree handles
  # item NAs itself).
  # ---------------------------------------------------------------------
  na_cov <- !stats::complete.cases(covariates)
  if (any(na_cov)) {
    message(sum(na_cov), " row(s) with NA in `covariates` dropped.")
    data <- data[!na_cov, , drop = FALSE]
    covariates <- covariates[!na_cov, , drop = FALSE]
  }
  if (nrow(data) < 30L) {
    stop(
      "Need at least 30 complete cases after NA handling. Got ",
      nrow(data),
      ".",
      call. = FALSE
    )
  }

  items_mat <- as.matrix(data)
  storage.mode(items_mat) <- "integer"
  combined <- covariates
  combined$.items <- items_mat

  # Backtick-protect the covariate names so they are matched as literal
  # column names in `combined`. Without this, a non-syntactic name (e.g.
  # "phq9[, 10]", produced when the user passes a single column like
  # `covariates = phq9[, 10]`) would be parsed by reformulate() as an R
  # expression and re-evaluated against the *original* data inside
  # model.frame(), triggering a "variable lengths differ" error.
  fmla <- stats::reformulate(
    termlabels = sprintf("`%s`", names(covariates)),
    response = ".items"
  )

  # ---------------------------------------------------------------------
  # Fit the tree. Use do.call so stablelearner::stabletree() can later
  # update() the call cleanly.
  # ---------------------------------------------------------------------
  fit_fun <- if (resolved_model == "RM") {
    psychotree::raschtree
  } else {
    psychotree::pctree
  }

  tree_call <- as.call(c(
    list(
      if (resolved_model == "RM") {
        quote(psychotree::raschtree)
      } else {
        quote(psychotree::pctree)
      },
      fmla,
      quote(combined)
    ),
    list(...)
  ))
  names(tree_call)[2:3] <- c("formula", "data")

  # The "Minimum score is not zero" warning fires repeatedly during the
  # tree's parameter-instability search (psychotree refits pcmodel many
  # times). Capturing it here just to muffle the noise -- we inspect the
  # *final* tree's terminal nodes below to detect the actual condition.
  tree <- withCallingHandlers(
    eval(tree_call, envir = list(combined = combined)),
    warning = function(w) {
      if (
        grepl("Minimum score is not zero", conditionMessage(w), fixed = TRUE)
      ) {
        invokeRestart("muffleWarning")
      }
      # Non-matching warnings: let them propagate normally.
    }
  )

  # ---------------------------------------------------------------------
  # Compute per-split effect sizes
  # ---------------------------------------------------------------------
  effect_info <- compute_tree_effectsize(
    tree = tree,
    items = items_mat,
    es_method = resolved_es,
    purification = purification,
    p_adj = p_adj,
    thresholds = thresholds,
    alpha = alpha
  )

  tree$info$effectsize <- effect_info

  # ---------------------------------------------------------------------
  # Optional pruning of negligible (all-A) splits
  # ---------------------------------------------------------------------
  if (prune_negligible && length(effect_info$nodes) > 0L) {
    to_prune <- vapply(
      effect_info$nodes,
      function(nd) {
        all(nd$Class == "A", na.rm = TRUE)
      },
      logical(1L)
    )
    prune_ids <- as.integer(names(effect_info$nodes)[to_prune])
    if (length(prune_ids) > 0L) {
      tree <- partykit::nodeprune(tree, ids = prune_ids)
      # Recompute on the pruned tree -- splits and ids may have shifted
      effect_info <- compute_tree_effectsize(
        tree = tree,
        items = items_mat,
        es_method = resolved_es,
        purification = purification,
        p_adj = p_adj,
        thresholds = thresholds,
        alpha = alpha
      )
      tree$info$effectsize <- effect_info
    }
  }

  # ---------------------------------------------------------------------
  # Diagnostic: detect items rescaled within terminal nodes
  # ---------------------------------------------------------------------
  rescale_info <- detect_rescaled_items(tree, items_mat)
  if (rescale_info$n_terminal_cells > 0L) {
    msg <- format_rescale_message(rescale_info)
    switch(
      on_rescale,
      message = message(msg),
      warning = warning(msg, call. = FALSE),
      stop = stop(msg, call. = FALSE)
    )
  }

  # ---------------------------------------------------------------------
  # Stability assessment
  # ---------------------------------------------------------------------
  # Resample-level fits inside stabletree() can produce three flavours of
  # noise that we don't want to leak to the user (each fires repeatedly
  # during the parameter-instability search but is not separately
  # actionable -- the user has already been told about rescaling in the
  # *original* tree above):
  #
  #   1. "Minimum score is not zero ..."   -- pcmodel rescaling warning
  #   2. "items with null categories ..."  -- interior categories empty
  #   3. "Error in chol.default(meat) ..." -- M-fluctuation test cannot
  #      invert a rank-deficient score covariance (printed via try()
  #      inside partykit::mob; non-fatal, the affected split is skipped).
  #
  # We muffle the warnings via withCallingHandlers and (when safe) capture
  # stderr via sink() to swallow the printed try() error text. After the
  # call returns we emit a single summary message so the user knows what
  # happened during the resample fits.
  #
  # Caveat on the message sink: R allows only one message-stream sink at a
  # time and offers no way to save and restore a pre-existing one. Some
  # front-ends (notably RStudio) redirect the console's stderr at startup,
  # so unconditionally calling sink(type = "message") here -- and resetting
  # it with sink(NULL) afterwards -- would clobber that front-end sink and
  # silence all subsequent console output (the symptom: RMdifTree() returns
  # but prints nothing). We therefore only redirect when the message stream
  # is still at its default (connection 2); otherwise we skip the capture
  # (and the resample-error count) and let any rare try() text show.
  stability_result <- NULL
  if (stability) {
    sampler <- if (stability_sampler == "subsampling") {
      stablelearner::subsampling
    } else {
      stablelearner::bootstrap
    }

    n_min_zero <- 0L
    n_null_cat <- 0L

    capture_stderr <- sink.number(type = "message") == 2L
    err_file <- NULL
    err_con <- NULL
    if (capture_stderr) {
      err_file <- tempfile(fileext = ".log")
      err_con <- file(err_file, open = "wt")
      sink(err_con, type = "message")
    }

    stability_result <- tryCatch(
      withCallingHandlers(
        stablelearner::stabletree(tree, B = stability_B, sampler = sampler),
        warning = function(w) {
          msg <- conditionMessage(w)
          if (grepl("Minimum score is not zero", msg, fixed = TRUE)) {
            n_min_zero <<- n_min_zero + 1L
            invokeRestart("muffleWarning")
          } else if (grepl("items with null categories", msg, fixed = TRUE)) {
            n_null_cat <<- n_null_cat + 1L
            invokeRestart("muffleWarning")
          }
        }
      ),
      finally = {
        if (capture_stderr) {
          sink(NULL, type = "message")
          close(err_con)
        }
      }
    )

    n_resample_errors <- 0L
    if (capture_stderr) {
      err_lines <- tryCatch(
        readLines(err_file, warn = FALSE),
        error = function(e) character()
      )
      unlink(err_file)
      n_resample_errors <- sum(grepl("^Error", err_lines))
    }

    # Single user-facing summary, only if anything noteworthy happened.
    summary_parts <- character()
    if (n_min_zero > 0L) {
      summary_parts <- c(
        summary_parts,
        paste0(n_min_zero, " 'minimum score not zero' rescaling warning(s)")
      )
    }
    if (n_null_cat > 0L) {
      summary_parts <- c(
        summary_parts,
        paste0(n_null_cat, " 'items with null categories' warning(s)")
      )
    }
    if (n_resample_errors > 0L) {
      summary_parts <- c(
        summary_parts,
        paste0(
          n_resample_errors,
          " resample fit(s) errored out (rank-deficient score ",
          "covariance in the M-fluctuation test); affected splits ",
          "skipped"
        )
      )
    }
    if (length(summary_parts) > 0L) {
      message(
        "Stability assessment notes (suppressed during fitting): ",
        paste(summary_parts, collapse = "; "),
        ". These reflect sparsity in some resamples and are normal ",
        "sampling variability; selection-frequency and cutpoint summaries ",
        "are based on resamples that completed successfully."
      )
    }

    attr(tree, "stability") <- stability_result
  }

  class(tree) <- unique(c("RMdifTree", class(tree)))

  # ---------------------------------------------------------------------
  # Build the per-split per-item dataframe (used by all non-tree outputs)
  # ---------------------------------------------------------------------
  df <- effectsize_to_dataframe(
    effect_info,
    item_names = colnames(items_mat),
    tree = tree,
    rescaled_terminals = rescale_info$per_terminal
  )

  stab_df <- NULL
  stab_kbl <- NULL
  if (!is.null(stability_result)) {
    stab_df <- stabletree_summary_df(stability_result, covariates)
    stab_kbl <- stabletree_summary_kable(stab_df)
  }

  attr(df, "model") <- resolved_model
  attr(df, "effect_size") <- resolved_es
  if (!is.null(stab_df)) {
    attr(df, "stability") <- stab_df
  }
  if (!is.null(stab_kbl)) {
    attr(df, "stability_kable") <- stab_kbl
  }
  if (rescale_info$n_terminal_cells > 0L) {
    attr(df, "rescaled_terminal_items") <- rescale_info$per_terminal
  }

  # ---------------------------------------------------------------------
  # Dispatch on output
  # ---------------------------------------------------------------------
  if (output %in% c("tree", "list")) {
    if (!is.null(stab_df)) {
      attr(tree, "stability") <- stab_df
    }
    if (!is.null(stab_kbl)) {
      attr(tree, "stability_kable") <- stab_kbl
    }
    if (rescale_info$n_terminal_cells > 0L) {
      attr(tree, "rescaled_terminal_items") <- rescale_info$per_terminal
    }
    if (output == "list") {
      # One fit, both views: the tidy per-split table (with its
      # attributes) and the augmented tree object (for plotting).
      return(list(table = df, tree = tree))
    }
    return(tree)
  }
  if (output == "plot") {
    # Show item names (rather than 1..I numeric indices) on the
    # terminal-node x-axis. Users wanting full control can use
    # `output = "tree"` and call `plot()` themselves with custom
    # `tp_args`.
    return(graphics::plot(tree, tp_args = list(names = TRUE)))
  }
  if (output == "dataframe") {
    return(df)
  }

  # output == "kable"
  # Display rounding (the dataframe output above stays unrounded)
  df <- .round_display(df, c(EffectSize = 4, SE = 4))
  kbl <- render_effectsize_kable(
    df,
    n_persons = nrow(combined),
    n_total = n_total_dt,
    n_items = ncol(items_mat),
    es_method = resolved_es,
    purification = purification,
    p_adj = p_adj,
    thresholds = thresholds,
    alpha = alpha,
    is_polytomous = is_polytomous,
    has_stability = !is.null(stab_df),
    n_rescaled_items = length(unique(rescale_info$per_terminal$Item))
  )
  if (!is.null(stab_df)) {
    attr(kbl, "stability") <- stab_df
  }
  if (!is.null(stab_kbl)) {
    attr(kbl, "stability_kable") <- stab_kbl
  }
  if (rescale_info$n_terminal_cells > 0L) {
    attr(kbl, "rescaled_terminal_items") <- rescale_info$per_terminal
  }
  attr(kbl, "model") <- resolved_model
  attr(kbl, "effect_size") <- resolved_es
  kbl
}

# =====================================================================
# Internal helpers
# =====================================================================

#' Derive a readable column name from an unevaluated `covariates` expression
#'
#' When the user passes a single vector -- e.g.
#' \code{RMdifTree(d, dif$age)} -- \code{as.data.frame(covariates)} would
#' name the column "covariates" (the parameter name). To keep the plot,
#' kable, and dataframe outputs informative we recover the user's
#' expression here and produce a clean column name:
#' * a bare symbol \code{age} -> "age"
#' * \code{dif$age} or \code{dif[["age"]]} -> "age"
#' * \code{dif[, "age"]} -> "age"
#' * anything else -> the deparsed expression as-is.
#'
#' If the resulting name would collide with R reserved syntax, a
#' fallback "covariate" is used.
#'
#' @keywords internal
#' @noRd
derive_covariate_name <- function(expr) {
  bad_fallback <- "covariate"
  if (is.symbol(expr)) {
    nm <- as.character(expr)
    return(if (nzchar(nm)) nm else bad_fallback)
  }
  if (is.call(expr) && length(expr) >= 3L) {
    op <- expr[[1L]]
    # x$name
    if (identical(op, as.name("$"))) {
      return(as.character(expr[[3L]]))
    }
    # x[["name"]] or x[[i]]
    if (identical(op, as.name("[["))) {
      key <- expr[[3L]]
      if (is.character(key)) {
        return(key)
      }
      if (is.symbol(key)) return(as.character(key))
    }
    # x[, "name"] or x[i]
    if (identical(op, as.name("["))) {
      # Last index argument is the column selector
      n <- length(expr)
      key <- expr[[n]]
      if (is.character(key)) {
        return(key)
      }
      if (is.symbol(key)) return(as.character(key))
    }
  }
  out <- tryCatch(deparse(expr, width.cutoff = 60L)[1L], error = function(e) {
    bad_fallback
  })
  if (is.null(out) || !nzchar(out)) bad_fallback else out
}

#' Validate the covariates data.frame
#'
#' Catches all-NA columns, zero-variance columns, and (for factors) too
#' few cases per level.
#'
#' @keywords internal
#' @noRd
validate_covariates <- function(covariates, min_n_per_level) {
  for (nm in names(covariates)) {
    col <- covariates[[nm]]
    if (all(is.na(col))) {
      stop("Covariate `", nm, "` is all NA.", call. = FALSE)
    }
    if (is.character(col)) {
      covariates[[nm]] <- as.factor(col)
      col <- covariates[[nm]]
    }
    if (is.logical(col)) {
      covariates[[nm]] <- as.factor(col)
      col <- covariates[[nm]]
    }
    if (is.factor(col)) {
      col <- droplevels(col)
      tab <- table(col, useNA = "no")
      if (length(tab) < 2L) {
        stop(
          "Covariate `",
          nm,
          "` has fewer than 2 non-NA levels.",
          call. = FALSE
        )
      }
      if (min_n_per_level > 0L && any(tab < min_n_per_level)) {
        small <- names(tab)[tab < min_n_per_level]
        stop(
          "Covariate `",
          nm,
          "` has level(s) with fewer than ",
          min_n_per_level,
          " cases: ",
          paste(sprintf("%s (n = %d)", small, tab[small]), collapse = ", "),
          ". Combine sparse levels or set `min_n_per_level = 0`.",
          call. = FALSE
        )
      }
    } else if (is.numeric(col)) {
      if (length(unique(stats::na.omit(col))) < 2L) {
        stop(
          "Covariate `",
          nm,
          "` has fewer than 2 unique values.",
          call. = FALSE
        )
      }
    } else {
      stop(
        "Covariate `",
        nm,
        "` has unsupported type ",
        paste(class(col), collapse = "/"),
        ". Use numeric, factor, ordered factor, or logical.",
        call. = FALSE
      )
    }
  }
  invisible(TRUE)
}

#' Compute effect sizes for every inner (split) node of a tree
#'
#' Walks the partykit tree, gets the binary group assignment defined by
#' the descendants of each inner node, and computes per-item effect
#' sizes on that split.
#'
#' Adapted from \code{effecttree::get_effectsize()} (MIT, Henninger &
#' Radek), with simplifications: we only support the binary-split case
#' that \code{psychotree::raschtree}/\code{pctree} produce, and we
#' return a flat list keyed by node id.
#'
#' @keywords internal
#' @noRd
compute_tree_effectsize <- function(
  tree,
  items,
  es_method,
  purification,
  p_adj,
  thresholds,
  alpha
) {
  inner_ids <- setdiff(
    partykit::nodeids(tree, terminal = FALSE),
    partykit::nodeids(tree, terminal = TRUE)
  )

  if (length(inner_ids) == 0L) {
    return(list(
      es_method = es_method,
      purification = purification,
      p_adj = p_adj,
      thresholds = thresholds,
      alpha = alpha,
      nodes = list(),
      n_splits = 0L
    ))
  }

  dat_fitted <- partykit::data_party(tree)
  fitted_id <- dat_fitted[["(fitted)"]]
  root_node <- partykit::node_party(tree)

  split_groups <- lapply(inner_ids, function(id) {
    nd <- find_node_by_id(root_node, id)
    if (is.null(nd)) {
      stop("Internal: could not locate node ", id, " in tree.", call. = FALSE)
    }
    children <- partykit::kids_node(nd)
    left_terminals <- collect_terminal_ids(children[[1L]])
    right_terminals <- collect_terminal_ids(children[[2L]])
    grp <- rep(NA_integer_, length(fitted_id))
    grp[fitted_id %in% left_terminals] <- 0L
    grp[fitted_id %in% right_terminals] <- 1L
    grp
  })
  names(split_groups) <- as.character(inner_ids)

  sums <- rowSums(items, na.rm = TRUE)

  per_node <- lapply(seq_along(inner_ids), function(i) {
    nid <- inner_ids[i]
    grp <- split_groups[[i]]
    split_meta <- describe_split(tree, nid)

    result <- if (es_method == "MH") {
      one_split_mh(
        items = items,
        group = grp,
        sums = sums,
        purification = purification,
        alpha = alpha
      )
    } else {
      one_split_pgamma(
        items = items,
        group = grp,
        purification = purification,
        p_adj = p_adj,
        thresholds = thresholds,
        alpha = alpha
      )
    }

    list(
      NodeID = nid,
      Variable = split_meta$variable,
      Direction = split_meta$direction,
      Split = split_meta$split,
      EffectSize = result$es,
      SE = result$se,
      Class = result$class,
      n_left = sum(grp == 0L, na.rm = TRUE),
      n_right = sum(grp == 1L, na.rm = TRUE),
      purification_iterations = result$purification_iterations
    )
  })
  names(per_node) <- as.character(inner_ids)

  list(
    es_method = es_method,
    purification = purification,
    p_adj = p_adj,
    thresholds = thresholds,
    alpha = alpha,
    nodes = per_node,
    n_splits = length(per_node)
  )
}

#' Recursively collect terminal node ids descended from a node
#'
#' Adapted from \code{effecttree::get_terminal_nodes} (MIT).
#'
#' @keywords internal
#' @noRd
collect_terminal_ids <- function(node) {
  kids <- partykit::kids_node(node)
  if (is.null(kids)) {
    return(partykit::id_node(node))
  }
  unlist(lapply(kids, collect_terminal_ids), use.names = FALSE)
}

#' Detect which (terminal node x item) cells had no responses in
#' category 0 in the final fitted tree
#'
#' \code{psychotree::pcmodel} silently rescales such items to start at
#' the new lowest observed category, producing per-node parameters on a
#' shifted metric. We detect this post-hoc by scanning each terminal
#' node's data.
#'
#' @return A list with three components:
#'   * \code{per_terminal}: data.frame with columns \code{NodeID}
#'     (terminal-node id) and \code{Item}, one row per affected cell.
#'   * \code{per_item}: named integer vector -- for each affected item,
#'     how many terminal nodes had it rescaled.
#'   * \code{n_terminal_cells}: total count of (terminal x item)
#'     affected cells.
#'
#' @keywords internal
#' @noRd
detect_rescaled_items <- function(tree, items_mat) {
  dat_fitted <- partykit::data_party(tree)
  fitted_id <- dat_fitted[["(fitted)"]]
  terminal_ids <- partykit::nodeids(tree, terminal = TRUE)
  item_names <- colnames(items_mat)
  if (is.null(item_names)) {
    item_names <- paste0("V", seq_len(ncol(items_mat)))
  }

  affected_rows <- list()
  for (tid in terminal_ids) {
    rows <- which(fitted_id == tid)
    if (length(rows) == 0L) {
      next
    }
    sub <- items_mat[rows, , drop = FALSE]
    item_min <- apply(sub, 2L, function(x) {
      x <- x[!is.na(x)]
      if (length(x) == 0L) NA_real_ else min(x)
    })
    bad_items <- item_names[!is.na(item_min) & item_min > 0]
    if (length(bad_items) > 0L) {
      affected_rows[[length(affected_rows) + 1L]] <- data.frame(
        NodeID = rep(tid, length(bad_items)),
        Item = bad_items,
        stringsAsFactors = FALSE
      )
    }
  }

  if (length(affected_rows) == 0L) {
    return(list(
      per_terminal = data.frame(
        NodeID = integer(),
        Item = character(),
        stringsAsFactors = FALSE
      ),
      per_item = integer(0),
      n_terminal_cells = 0L
    ))
  }

  per_terminal <- do.call(rbind, affected_rows)
  rownames(per_terminal) <- NULL
  per_item <- table(per_terminal$Item)

  list(
    per_terminal = per_terminal,
    per_item = stats::setNames(as.integer(per_item), names(per_item)),
    n_terminal_cells = nrow(per_terminal)
  )
}

#' Build a single user-facing message about rescaled items
#'
#' Item-level: "Item X was rescaled in K of T terminal nodes".
#'
#' @keywords internal
#' @noRd
format_rescale_message <- function(rescale_info) {
  pi <- rescale_info$per_item
  T_n <- length(unique(rescale_info$per_terminal$NodeID))
  n_aff_terms <- length(unique(rescale_info$per_terminal$NodeID))
  per_item_lines <- vapply(
    seq_along(pi),
    function(i) {
      paste0(
        "  - ",
        names(pi)[i],
        ": rescaled in ",
        pi[i],
        " terminal node",
        if (pi[i] != 1L) "s" else ""
      )
    },
    character(1L)
  )

  paste0(
    "Some items have no responses in their lowest category within ",
    n_aff_terms,
    " terminal node",
    if (n_aff_terms != 1L) "s" else "",
    " of the fitted tree. ",
    "psychotree silently rescales those items per node, so the ",
    "per-node item parameters (visible via plot(tree)) are on a ",
    "shifted metric for the affected cells. The MH / partial-gamma ",
    "effect sizes reported here are computed on raw responses and ",
    "are NOT affected.\n",
    "Affected items:\n",
    paste(per_item_lines, collapse = "\n"),
    "\nDiagnose with: RMplotTile(data, group = <covariate>) for each ",
    "covariate. To suppress, increase `minsize`. To downgrade or ",
    "escalate this notice see `?RMdifTree` argument `on_rescale`."
  )
}

#' Walk a partykit node tree to find the node with the given id
#'
#' \code{tree[[id]]} on a fitted partykit tree returns a *renumbered*
#' subtree (terminals start at 2, 3, ...), so we cannot use it to map
#' fitted ids back to subtree membership. Walking the root node
#' directly preserves the original ids.
#'
#' @keywords internal
#' @noRd
find_node_by_id <- function(node, id) {
  if (partykit::id_node(node) == id) {
    return(node)
  }
  kids <- partykit::kids_node(node)
  if (is.null(kids)) {
    return(NULL)
  }
  for (kid in kids) {
    found <- find_node_by_id(kid, id)
    if (!is.null(found)) return(found)
  }
  NULL
}

#' Describe a split (variable name + cutpoint or level set)
#'
#' @keywords internal
#' @noRd
describe_split <- function(tree, node_id) {
  na_out <- list(
    variable = NA_character_,
    direction = NA_character_,
    split = NA_character_
  )
  nd <- find_node_by_id(partykit::node_party(tree), node_id)
  if (is.null(nd)) {
    return(na_out)
  }
  sp <- partykit::split_node(nd)
  if (is.null(sp)) {
    return(na_out)
  }

  varid <- sp$varid
  varname <- names(partykit::data_party(tree))[varid]
  bp <- sp$breaks
  idx <- sp$index

  if (!is.null(bp)) {
    cut_str <- signif(bp, 4)
    return(list(
      variable = varname,
      direction = paste0("<= ", cut_str),
      split = paste0(varname, " <= ", cut_str)
    ))
  }
  if (!is.null(idx)) {
    var_col <- partykit::data_party(tree)[[varid]]
    levs <- levels(var_col)
    if (is.null(levs)) {
      return(list(
        variable = varname,
        direction = "split",
        split = paste0(varname, ": split")
      ))
    }
    left <- levs[idx == 1L]
    right <- levs[idx == 2L]
    left_str <- if (length(left) == 1L) {
      left
    } else {
      paste0("{", paste(left, collapse = ", "), "}")
    }
    right_str <- if (length(right) == 1L) {
      right
    } else {
      paste0("{", paste(right, collapse = ", "), "}")
    }
    return(list(
      variable = varname,
      direction = paste0("= ", left_str),
      split = paste0(varname, ": ", left_str, " vs ", right_str)
    ))
  }
  na_out
}

#' Mantel-Haenszel effect-size for one binary split
#'
#' Adapted from \code{effecttree::calculate_mantelhaenszel} (MIT) with
#' the easyRasch2-style alpha argument.
#'
#' @keywords internal
#' @noRd
one_split_mh <- function(items, group, sums, purification, alpha) {
  keep <- !is.na(group)
  dat <- as.data.frame(items[keep, , drop = FALSE])
  grp <- group[keep]
  smm <- sums[keep]

  res <- difR::mantelHaenszel(data = dat, member = grp, match = smm)
  delta <- -2.35 * log(res$resAlpha)
  delta_sd <- 2.35 * sqrt(res$varLambda)
  iters <- 0L

  if (purification == "iterative") {
    which_dif <- which(abs(delta) >= 1)
    if (length(which_dif) > 0L) {
      repeat {
        iters <- iters + 1L
        pur <- purify_mh_step(dat, grp, delta)
        delta <- pur$delta
        delta_sd <- pur$delta_sd
        new_dif <- which(abs(delta) >= 1)
        if (setequal(which_dif, new_dif) || iters >= ncol(dat)) {
          break
        }
        which_dif <- new_dif
      }
    }
  }

  cls <- ets_classify(delta, delta_sd, alpha = alpha)
  list(
    es = unname(delta),
    se = unname(delta_sd),
    class = cls,
    purification_iterations = iters
  )
}

#' One step of Holland-Thayer iterative purification for MH
#'
#' Items currently classified as DIF are excluded from the matching
#' total when re-computing MH for the non-DIF items. For DIF items,
#' the purified MH includes the item itself in the matching total
#' (Bjorner et al., 1998, p. 1191; Henninger et al., 2023).
#'
#' Adapted from \code{effecttree::purify_MH} (MIT).
#'
#' @keywords internal
#' @noRd
purify_mh_step <- function(dat, group, delta) {
  no_dif <- which(abs(delta) < 1)
  dif_set <- which(abs(delta) >= 1)

  if (length(no_dif) < 2L) {
    return(list(delta = delta, delta_sd = rep(NA_real_, length(delta))))
  }

  base_match <- rowSums(dat[, no_dif, drop = FALSE], na.rm = TRUE)
  res <- difR::mantelHaenszel(data = dat, member = group, match = base_match)
  new_delta <- -2.35 * log(res$resAlpha)
  new_sd <- 2.35 * sqrt(res$varLambda)

  # Recompute DIF items adding their own score back in
  for (i in dif_set) {
    smm <- dat[, i] + base_match
    res_i <- difR::mantelHaenszel(
      data = dat[, i, drop = FALSE],
      member = group,
      match = smm
    )
    new_delta[i] <- -2.35 * log(res_i$resAlpha)
    new_sd[i] <- 2.35 * sqrt(res_i$varLambda)
  }

  list(delta = new_delta, delta_sd = new_sd)
}

#' ETS A/B/C classification for MH Delta-scale effect sizes
#'
#' Two non-central chi-square tests on \eqn{(\Delta/SD)^2}:
#' tau = 0 for the A-test, tau = 1 for the C-test (Paek & Holland,
#' 2015).
#'
#' Adapted from \code{effecttree::get_ETS_classification} (MIT).
#'
#' @keywords internal
#' @noRd
ets_classify <- function(delta, delta_sd, alpha = 0.05) {
  pval_at <- function(d, sd, tau) {
    lambda <- (tau / sd)^2
    psi <- (d / sd)^2
    1 - stats::pchisq(psi, df = 1, ncp = lambda)
  }
  p_A <- pval_at(delta, delta_sd, tau = 0)
  p_C <- pval_at(delta, delta_sd, tau = 1)
  ifelse(
    abs(delta) < 1 | p_A >= alpha,
    "A",
    ifelse(abs(delta) >= 1.5 & p_C < alpha, "C", "B")
  )
}

#' Partial-gamma effect size for one binary split
#'
#' Adapted from \code{effecttree::calculate_pgamma} (MIT).
#'
#' @keywords internal
#' @noRd
one_split_pgamma <- function(
  items,
  group,
  purification,
  p_adj,
  thresholds,
  alpha
) {
  keep <- !is.na(group)
  dat <- as.data.frame(items[keep, , drop = FALSE])
  grp <- group[keep]

  res <- quiet_call(iarm::partgam_DIF(dat.items = dat, dat.exo = grp))
  gam <- res$gamma
  sse <- res$se
  iters <- 0L

  if (purification == "iterative") {
    which_dif <- which(abs(gam) >= thresholds[1L])
    if (length(which_dif) > 0L) {
      repeat {
        iters <- iters + 1L
        pur <- purify_pgamma_step(dat, grp, gam, sse, thresholds)
        if (is.null(pur)) {
          break
        }
        gam <- pur$gamma
        sse <- pur$se
        new_dif <- which(abs(gam) >= thresholds[1L])
        if (setequal(which_dif, new_dif) || iters >= ncol(dat)) {
          break
        }
        which_dif <- new_dif
      }
    }
  }

  cls <- pgamma_classify(
    gam,
    sse,
    p_adj = p_adj,
    thresholds = thresholds,
    alpha = alpha
  )
  list(
    es = unname(gam),
    se = unname(sse),
    class = cls,
    purification_iterations = iters
  )
}

#' One step of iterative purification for partial gamma
#'
#' Adapted from \code{effecttree::purify_pgamma} (MIT). Returns NULL if
#' fewer than 2 non-DIF items remain (purification ill-defined).
#'
#' @keywords internal
#' @noRd
purify_pgamma_step <- function(dat, group, gam, sse, thresholds) {
  no_dif <- which(abs(gam) < thresholds[1L])
  dif_set <- which(abs(gam) >= thresholds[1L])
  if (length(no_dif) < 2L) {
    return(NULL)
  }

  new_gam <- gam
  new_sse <- sse

  base <- quiet_call(
    iarm::partgam_DIF(dat[, no_dif, drop = FALSE], group)
  )
  new_gam[no_dif] <- base$gamma
  new_sse[no_dif] <- base$se

  for (i in dif_set) {
    cols <- sort(unique(c(i, no_dif)))
    pos <- match(i, cols)
    res_i <- quiet_call(iarm::partgam_DIF(dat[, cols, drop = FALSE], group))
    new_gam[i] <- res_i$gamma[pos]
    new_sse[i] <- res_i$se[pos]
  }

  list(gamma = new_gam, se = new_sse)
}

#' A/B/C classification for partial gamma
#'
#' Adapted from \code{effecttree::get_ABC_classification} (MIT).
#'
#' @keywords internal
#' @noRd
pgamma_classify <- function(gam, sse, p_adj, thresholds, alpha = 0.05) {
  z_A <- gam / sse
  p_A <- ifelse(gam > 0, 2 * (1 - stats::pnorm(z_A)), 2 * stats::pnorm(z_A))
  shifted <- ifelse(gam >= 0, gam - thresholds[1L], gam + thresholds[1L])
  z_C <- shifted / sse
  p_C <- ifelse(shifted > 0, 2 * (1 - stats::pnorm(z_C)), 2 * stats::pnorm(z_C))
  if (p_adj != "none") {
    p_A <- stats::p.adjust(p_A, method = p_adj)
    p_C <- stats::p.adjust(p_C, method = p_adj)
  }
  ifelse(
    abs(gam) < thresholds[1L] | p_A >= alpha,
    "A",
    ifelse(abs(gam) > thresholds[2L] & p_C < alpha, "C", "B")
  )
}

#' Convert per-node effect-size results to a tidy data.frame
#'
#' One row per (node x item).
#'
#' @keywords internal
#' @noRd
effectsize_to_dataframe <- function(
  effect_info,
  item_names,
  tree = NULL,
  rescaled_terminals = NULL
) {
  empty_cols <- c(
    "NodeID",
    "Split",
    "Variable",
    "Direction",
    "Item",
    "EffectSize",
    "SE",
    "Class",
    "Flagged",
    "Rescaled",
    "n_left",
    "n_right"
  )
  if (effect_info$n_splits == 0L) {
    out <- data.frame(matrix(ncol = length(empty_cols), nrow = 0L))
    names(out) <- empty_cols
    return(out)
  }

  rows <- lapply(effect_info$nodes, function(nd) {
    n_items <- length(nd$EffectSize)

    # Rescaled at this split for this item: TRUE if item was rescaled
    # in any terminal node descended from EITHER child of this inner
    # node. Computed only when we have the tree + per-terminal table.
    rescaled <- rep(FALSE, n_items)
    if (
      !is.null(tree) &&
        !is.null(rescaled_terminals) &&
        nrow(rescaled_terminals) > 0L
    ) {
      nd_obj <- find_node_by_id(partykit::node_party(tree), nd$NodeID)
      if (!is.null(nd_obj)) {
        descend_terms <- collect_terminal_ids(nd_obj)
        bad <- rescaled_terminals$Item[
          rescaled_terminals$NodeID %in% descend_terms
        ]
        rescaled <- item_names %in% bad
      }
    }

    data.frame(
      NodeID = rep(nd$NodeID, n_items),
      Split = rep(nd$Split, n_items),
      Variable = rep(nd$Variable, n_items),
      Direction = rep(nd$Direction, n_items),
      Item = item_names,
      EffectSize = nd$EffectSize,
      SE = nd$SE,
      Class = nd$Class,
      Flagged = nd$Class %in% c("B", "C"),
      Rescaled = rescaled,
      n_left = rep(nd$n_left, n_items),
      n_right = rep(nd$n_right, n_items),
      stringsAsFactors = FALSE
    )
  })
  out <- do.call(rbind, rows)
  rownames(out) <- NULL
  out
}

#' Render the per-split effect-size dataframe as a kable
#'
#' Flagged rows (Class B/C) render in bold. One section per inner node,
#' delimited by plain section-header rows (no kableExtra dependency).
#'
#' @keywords internal
#' @noRd
render_effectsize_kable <- function(
  df,
  n_persons,
  n_items,
  es_method,
  purification,
  p_adj,
  thresholds,
  alpha,
  is_polytomous,
  has_stability,
  n_rescaled_items = 0L,
  n_total = n_persons
) {
  caption <- build_es_caption(
    df = df,
    n_persons = n_persons,
    n_total = n_total,
    n_items = n_items,
    es_method = es_method,
    purification = purification,
    p_adj = p_adj,
    thresholds = thresholds,
    alpha = alpha,
    is_polytomous = is_polytomous,
    has_stability = has_stability,
    n_rescaled_items = n_rescaled_items
  )

  if (nrow(df) == 0L) {
    empty <- data.frame(
      Note = "No splits -- model is invariant across covariates.",
      stringsAsFactors = FALSE
    )
    return(knitr::kable(
      empty,
      format = "pipe",
      row.names = FALSE,
      caption = caption
    ))
  }

  display <- df
  flag <- isTRUE_logical(display$Flagged)
  display$Flagged <- ifelse(flag, "yes", "no")
  body_cols <- c("Item", "EffectSize", "SE", "Class", "Flagged")
  if (any(flag)) {
    for (cc in body_cols) {
      col <- display[[cc]]
      if (is.numeric(col)) {
        col <- format(col, nsmall = 3, trim = TRUE)
      }
      display[[cc]] <- ifelse(flag, paste0("**", col, "**"), as.character(col))
    }
  }

  # Build one section per node with a header row showing the split.
  # Pull n_left / n_right / Split from the original `df` to avoid the
  # bold-decoration applied to `display`.
  ids <- unique(df$NodeID)
  parts <- lapply(ids, function(nid) {
    raw <- df[df$NodeID == nid, , drop = FALSE]
    split_lbl <- raw$Split[1L]
    n_l <- raw$n_left[1L]
    n_r <- raw$n_right[1L]
    header <- paste0(
      "Node ",
      nid,
      " -- ",
      split_lbl,
      "  (n left = ",
      n_l,
      ", n right = ",
      n_r,
      ")"
    )
    body <- display[display$NodeID == nid, body_cols, drop = FALSE]
    list(header = header, body = body)
  })

  # Concatenate as a single kable by interleaving header lines using
  # pipe-format manually.
  header_line <- "| Item | EffectSize | SE | Class | Flagged |"
  divider <- "|:-----|-----------:|---:|:------|:--------|"

  body_chunks <- lapply(parts, function(p) {
    bl <- knitr::kable(p$body, format = "pipe", row.names = FALSE)
    # Drop the kable's own header line (first 2 lines) and prepend our
    # section header instead.
    bl_lines <- as.character(bl)
    if (length(bl_lines) >= 3L) {
      data_lines <- bl_lines[3:length(bl_lines)]
    } else {
      data_lines <- bl_lines
    }
    c(paste0("**", p$header, "**"), "", header_line, divider, data_lines, "")
  })

  out_lines <- c(
    unlist(body_chunks, use.names = FALSE),
    "",
    paste0(": ", caption)
  )

  structure(out_lines, format = "pipe", class = "knitr_kable")
}

#' Build the caption string for the effect-size kable
#'
#' @keywords internal
#' @noRd
build_es_caption <- function(
  df,
  n_persons,
  n_items,
  es_method,
  purification,
  p_adj,
  thresholds,
  alpha,
  is_polytomous,
  has_stability,
  n_rescaled_items = 0L,
  n_total = n_persons
) {
  model_name <- if (is_polytomous) "Partial Credit Tree" else "Rasch Tree"
  es_name <- if (es_method == "MH") {
    "Mantel-Haenszel effect size (ETS Delta scale)"
  } else {
    paste0(
      "partial gamma (B/C thresholds = ",
      thresholds[1L],
      " / ",
      thresholds[2L],
      ")"
    )
  }
  pur_part <- if (purification == "iterative") {
    " Iterative purification applied (Bjorner et al. 1998)."
  } else {
    ""
  }
  padj_part <- if (es_method == "pgamma" && p_adj != "none") {
    paste0(" p-values adjusted by ", p_adj, ".")
  } else {
    ""
  }
  flag_part <- if (nrow(df) > 0L) {
    paste0(
      " Flagged (Class B or C): ",
      sum(df$Flagged),
      " / ",
      nrow(df),
      " (item x split combinations).",
      " **Bold** rows are flagged."
    )
  } else {
    ""
  }
  stab_part <- if (has_stability) {
    " Stability summary attached as `attr(result, \"stability\")`."
  } else {
    ""
  }
  rescale_part <- if (n_rescaled_items > 0L) {
    paste0(
      " ",
      n_rescaled_items,
      " item(s) were rescaled in at least one terminal ",
      "node -- see `attr(result, \"rescaled_terminal_items\")`. ",
      "Effect sizes are unaffected."
    )
  } else {
    ""
  }
  paste0(
    model_name,
    " (",
    n_items,
    " items). ",
    .n_caption(
      n_persons,
      n_total,
      if (n_persons < n_total) "complete DIF covariates" else character()
    ),
    ". Effect size: ",
    es_name,
    ". alpha = ",
    alpha,
    ".",
    pur_part,
    padj_part,
    flag_part,
    stab_part,
    rescale_part
  )
}

#' Robust isTRUE for character/logical Flagged columns
#'
#' @keywords internal
#' @noRd
isTRUE_logical <- function(x) {
  if (is.logical(x)) {
    return(!is.na(x) & x)
  }
  vapply(x, isTRUE, logical(1L))
}

#' Summarise a stabletree object as a per-variable data.frame
#'
#' Selection frequency = proportion of resamples in which the variable
#' was selected at any split. For continuous variables, also report
#' mean and SD of cutpoint when used.
#'
#' @keywords internal
#' @noRd
stabletree_summary_df <- function(stb, covariates) {
  vsel <- stb$vs
  if (is.null(vsel)) {
    out <- data.frame(
      Variable = character(),
      Type = character(),
      Selected_orig = logical(),
      Selection_freq_pct = numeric(),
      Cutpoint_mean = numeric(),
      Cutpoint_sd = numeric(),
      stringsAsFactors = FALSE
    )
    attr(out, "B") <- if (!is.null(stb$B)) stb$B else NA_integer_
    attr(out, "sampler") <- "unknown"
    attr(out, "caption") <- "No variables available for stability assessment."
    return(out)
  }

  vars <- colnames(vsel)
  freq <- colMeans(vsel > 0)
  vs0 <- if (!is.null(stb$vs0)) stb$vs0[vars] > 0 else rep(NA, length(vars))

  # Per-variable cutpoint summary across resamples (numeric covariates only;
  # for factors the "cutpoint" is a level-index pair that does not have a
  # meaningful mean/SD interpretation, so we report NA).
  cp_mean <- rep(NA_real_, length(vars))
  cp_sd <- rep(NA_real_, length(vars))
  br_list <- stb$br
  if (!is.null(br_list)) {
    for (i in seq_along(vars)) {
      v <- vars[i]
      if (!is.numeric(covariates[[v]]) || is.factor(covariates[[v]])) {
        next
      }
      brks <- br_list[[v]]
      if (is.null(brks) || length(brks) == 0L) {
        next
      }
      if (is.list(brks)) {
        all_breaks <- unlist(
          lapply(brks, function(rec) {
            if (
              is.list(rec) && !is.null(rec$breaks) && is.numeric(rec$breaks)
            ) {
              as.numeric(rec$breaks)
            } else if (is.numeric(rec)) {
              as.numeric(rec)
            } else {
              numeric(0)
            }
          }),
          use.names = FALSE
        )
      } else if (is.numeric(brks)) {
        all_breaks <- as.numeric(brks)
      } else {
        all_breaks <- numeric(0)
      }
      all_breaks <- all_breaks[is.finite(all_breaks)]
      if (length(all_breaks) > 0L) {
        cp_mean[i] <- mean(all_breaks)
      }
      if (length(all_breaks) >= 2L) cp_sd[i] <- stats::sd(all_breaks)
    }
  }

  type_of <- function(v) {
    col <- covariates[[v]]
    if (is.ordered(col)) {
      "ordered"
    } else if (is.factor(col)) {
      "factor"
    } else if (is.logical(col)) {
      "logical"
    } else if (is.numeric(col)) {
      "numeric"
    } else {
      class(col)[1L]
    }
  }

  out <- data.frame(
    Variable = vars,
    Type = vapply(vars, type_of, character(1L)),
    Selected_orig = unname(vs0),
    Selection_freq_pct = unname(freq) * 100,
    Cutpoint_mean = cp_mean,
    Cutpoint_sd = cp_sd,
    stringsAsFactors = FALSE,
    row.names = NULL
  )
  # Sort: selected-in-original first, then by frequency descending
  out <- out[order(!out$Selected_orig, -out$Selection_freq_pct), , drop = FALSE]
  rownames(out) <- NULL

  attr(out, "B") <- nrow(vsel)
  attr(out, "sampler") <- if (!is.null(stb$sampler)) {
    attr(stb$sampler, "method") %||% "subsampling"
  } else {
    "subsampling"
  }
  attr(out, "caption") <- paste0(
    "Tree-stability assessment across ",
    nrow(vsel),
    " resamples (",
    attr(out, "sampler"),
    "). ",
    "Selection_freq_pct: percent of resamples in which the covariate was ",
    "selected at any split. Selected_orig: TRUE if selected in the original tree. ",
    "Cutpoint_mean / Cutpoint_sd: mean and SD of the chosen cutpoint across ",
    "resamples (numeric covariates only)."
  )

  out
}

#' Render the stability summary as a kable
#'
#' @keywords internal
#' @noRd
stabletree_summary_kable <- function(stab_df) {
  if (nrow(stab_df) == 0L) {
    return(NULL)
  }
  # Display rounding (the dataframe / attribute output stays unrounded)
  stab_df <- .round_display(stab_df, c(
    Selection_freq_pct = 1, Cutpoint_mean = 4, Cutpoint_sd = 4
  ))
  cap <- attr(stab_df, "caption")
  knitr::kable(stab_df, format = "pipe", row.names = FALSE, caption = cap)
}

# null-coalesce helper
`%||%` <- function(a, b) if (is.null(a)) b else a

#' Suppress messages and printed output from a single call
#'
#' Equivalent to SimDesign::quiet() but without the dependency.
#'
#' @keywords internal
#' @noRd
quiet_call <- function(expr) {
  utils::capture.output(
    {
      val <- suppressMessages(suppressWarnings(force(expr)))
    },
    file = nullfile()
  )
  val
}

# Tests for RMdifTree()
#
# Covers:
#   - Input validation (covariates, model, min_n_per_level)
#   - Auto model + effect-size selection (RM/MH vs PCM/pgamma)
#   - Output structures: kable (default), dataframe, tree, plot
#   - Flagged + Rescaled columns
#   - on_rescale modes (message / warning / stop)
#   - Stability assessment via stablelearner
#   - Covariate-name propagation (bare symbol, dif$age, dif[["age"]])
#
# All tests use small data so each fit completes in well under 1s.

# ---------------------------------------------------------------------
# Helpers
# ---------------------------------------------------------------------
need_tree_pkgs <- function() {
  skip_if_not_installed("psychotree")
  skip_if_not_installed("partykit")
  skip_if_not_installed("difR")
  skip_if_not_installed("iarm")
}

# Dichotomous data with strong gender DIF: items 1-2 favour F, items 3-4
# favour M. Guarantees the tree splits on gender at small minsize.
make_dif_dichotomous <- function(n = 400, seed = 1L) {
  set.seed(seed)
  gender <- factor(rep(c("F", "M"), length.out = n))
  age    <- runif(n, 18, 80)
  theta  <- stats::rnorm(n)
  dif_item <- function(easy_for) {
    b <- ifelse(gender == easy_for, -1, 1)
    stats::rbinom(n, 1, plogis(theta - b))
  }
  items <- data.frame(
    I1 = dif_item("F"),
    I2 = dif_item("F"),
    I3 = dif_item("M"),
    I4 = dif_item("M"),
    I5 = stats::rbinom(n, 1, plogis(theta)),
    I6 = stats::rbinom(n, 1, plogis(theta + 0.3))
  )
  list(items = items,
       covs  = data.frame(age = age, gender = gender))
}

# Polytomous data with a forced rescaling case: item I1 has no category-0
# responses when age > 50, so any age-based split rescales I1 in the old
# subgroup.
make_polytomous_with_rescale <- function(n = 300, seed = 7L) {
  set.seed(seed)
  age   <- runif(n, 18, 80)
  theta <- stats::rnorm(n)
  bake  <- function(theta, b) {
    pmax(0L, pmin(2L, as.integer(round(
      theta - b + stats::rnorm(length(theta), 0, 0.5) + 1
    ))))
  }
  items <- data.frame(
    I1 = bake(theta + (age - 50) * 0.05, 0),
    I2 = bake(theta, 0),
    I3 = bake(theta, 0.3),
    I4 = bake(theta, -0.3)
  )
  # Force no zeros for I1 in the older subgroup → guaranteed rescale.
  items$I1[age > 50 & items$I1 == 0L] <- 1L
  list(items = items,
       covs  = data.frame(age = age))
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMdifTree errors when covariates is missing", {
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  expect_error(RMdifTree(d$items), regexp = "covariates")
})

test_that("RMdifTree errors when covariates row count mismatches data", {
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  expect_error(
    RMdifTree(d$items, covariates = d$covs[seq_len(nrow(d$items) - 1L), ]),
    regexp = "same number of rows"
  )
})

test_that("RMdifTree errors when factor covariate has tiny levels", {
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  d$covs$tiny <- factor(c(rep("a", nrow(d$items) - 2L),
                          rep("b", 2L)))
  expect_error(
    RMdifTree(d$items, covariates = d$covs, min_n_per_level = 20),
    regexp = "fewer than 20"
  )
})

test_that("RMdifTree errors when model = 'RM' on polytomous data", {
  need_tree_pkgs()
  d <- make_polytomous_with_rescale()
  expect_error(
    suppressMessages(
      RMdifTree(d$items, covariates = d$covs, model = "RM",
                minsize = 50)
    ),
    regexp = "dichotomous"
  )
})

# ---------------------------------------------------------------------
# Output structures
# ---------------------------------------------------------------------
test_that("RMdifTree default output is a knitr_kable", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  out <- RMdifTree(d$items, covariates = d$covs, minsize = 50)
  expect_s3_class(out, "knitr_kable")
  expect_equal(attr(out, "model"),       "RM")
  expect_equal(attr(out, "effect_size"), "MH")
})

test_that("RMdifTree output = 'dataframe' on dichotomous data picks RM/MH", {
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  df <- RMdifTree(d$items, covariates = d$covs,
                  minsize = 50, output = "dataframe")
  expect_s3_class(df, "data.frame")
  expect_true(all(c("NodeID", "Split", "Variable", "Direction",
                    "Item", "EffectSize", "SE", "Class",
                    "Flagged", "Rescaled",
                    "n_left", "n_right") %in% names(df)))
  expect_true(nrow(df) > 0L,
              info = "tree should have at least one split given strong DIF")
  expect_equal(attr(df, "model"),       "RM")
  expect_equal(attr(df, "effect_size"), "MH")
})

test_that("RMdifTree output = 'dataframe' on polytomous data picks PCM/pgamma", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_polytomous_with_rescale()
  set.seed(1)
  df <- suppressMessages(
    RMdifTree(d$items, covariates = d$covs,
              minsize = 50, output = "dataframe")
  )
  expect_s3_class(df, "data.frame")
  expect_equal(attr(df, "model"),       "PCM")
  expect_equal(attr(df, "effect_size"), "pgamma")
})

test_that("RMdifTree output = 'tree' returns an RMdifTree-classed partykit tree", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  tree <- RMdifTree(d$items, covariates = d$covs,
                    minsize = 50, output = "tree")
  expect_s3_class(tree, "RMdifTree")
  expect_s3_class(tree, "party")
  expect_true(!is.null(tree$info$effectsize))
  expect_true(is.list(tree$info$effectsize$nodes))
})

# ---------------------------------------------------------------------
# Flagged + Rescaled columns
# ---------------------------------------------------------------------
test_that("Flagged is logical and TRUE only for Class B/C", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  df <- suppressWarnings(
    RMdifTree(d$items, covariates = d$covs,
              minsize = 50, output = "dataframe")
  )
  expect_type(df$Flagged, "logical")
  # Class is one of A/B/C; rare NA permitted on sparse / unstable cells.
  expect_true(all(df$Class %in% c("A", "B", "C", NA)))
  # `Flagged` mirrors B/C (NA in Class → FALSE in Flagged).
  expected_flag <- !is.na(df$Class) & df$Class %in% c("B", "C")
  expect_equal(df$Flagged, expected_flag)
})

test_that("Rescaled column flags item × split cells affected by terminal-node rescaling", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_polytomous_with_rescale()
  set.seed(1)
  df <- suppressMessages(
    RMdifTree(d$items, covariates = d$covs,
              minsize = 50, output = "dataframe")
  )
  expect_type(df$Rescaled, "logical")
  rt <- attr(df, "rescaled_terminal_items")
  expect_s3_class(rt, "data.frame")
  expect_true("I1" %in% rt$Item)
  expect_true(any(df$Rescaled[df$Item == "I1"]))
})

# ---------------------------------------------------------------------
# on_rescale modes
# ---------------------------------------------------------------------
test_that("on_rescale = 'message' (default) emits message but does not error", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_polytomous_with_rescale()
  set.seed(1)
  expect_message(
    df <- RMdifTree(d$items, covariates = d$covs,
                    minsize = 50, output = "dataframe"),
    regexp = "no responses in their lowest category"
  )
  expect_s3_class(df, "data.frame")
})

test_that("on_rescale = 'warning' raises a warning instead", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_polytomous_with_rescale()
  set.seed(1)
  expect_warning(
    suppressMessages(
      RMdifTree(d$items, covariates = d$covs,
                minsize = 50, on_rescale = "warning",
                output = "dataframe")
    ),
    regexp = "no responses in their lowest category"
  )
})

test_that("on_rescale = 'stop' raises an error instead", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_polytomous_with_rescale()
  set.seed(1)
  expect_error(
    suppressMessages(
      RMdifTree(d$items, covariates = d$covs,
                minsize = 50, on_rescale = "stop",
                output = "dataframe")
    ),
    regexp = "no responses in their lowest category"
  )
})

# ---------------------------------------------------------------------
# Stability assessment
# ---------------------------------------------------------------------
test_that("stability = TRUE attaches a stability summary attribute", {
  skip_on_cran()
  need_tree_pkgs()
  skip_if_not_installed("stablelearner")
  d <- make_dif_dichotomous()
  set.seed(1)
  df <- RMdifTree(d$items, covariates = d$covs,
                  minsize = 50,
                  stability = TRUE, stability_B = 5L,
                  output = "dataframe")
  stab <- attr(df, "stability")
  expect_s3_class(stab, "data.frame")
  expect_true(all(c("Variable", "Type", "Selected_orig",
                    "Selection_freq_pct") %in% names(stab)))
  expect_setequal(stab$Variable, names(d$covs))
  # Pre-rendered kable also attached
  expect_s3_class(attr(df, "stability_kable"), "knitr_kable")
})

test_that("stability = TRUE does not clobber a pre-existing message sink", {
  skip_on_cran()
  need_tree_pkgs()
  skip_if_not_installed("stablelearner")
  d <- make_dif_dichotomous()
  # Mimic a front-end (e.g. RStudio) that has redirected the message
  # stream before the call. The function must leave that sink in place;
  # resetting it to the default (connection 2) silences console output.
  con <- file(tempfile(), open = "wt")
  withr::defer({
    if (sink.number(type = "message") != 2L) sink(NULL, type = "message")
    close(con)
  })
  sink(con, type = "message")
  before <- sink.number(type = "message")
  set.seed(1)
  invisible(RMdifTree(d$items, covariates = d$covs, minsize = 50,
                      stability = TRUE, stability_B = 5L,
                      output = "dataframe"))
  expect_equal(sink.number(type = "message"), before)
})

# ---------------------------------------------------------------------
# Covariate-name propagation
# ---------------------------------------------------------------------
test_that("Variable column reflects user's expression when covariates is passed as dif$col", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  df <- RMdifTree(d$items, d$covs$gender,
                  minsize = 50, output = "dataframe")
  expect_true("gender" %in% df$Variable)
})

test_that("Variable column reflects user's expression when covariates is a bare symbol", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  my_gender <- d$covs$gender
  set.seed(1)
  df <- RMdifTree(d$items, my_gender,
                  minsize = 50, output = "dataframe")
  expect_true("my_gender" %in% df$Variable)
})

test_that("Existing data.frame column names are preserved", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  df <- RMdifTree(d$items, covariates = d$covs,
                  minsize = 50, output = "dataframe")
  expect_true(all(df$Variable %in% names(d$covs)))
})

test_that("single covariate passed by numeric index (non-syntactic name) works", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  # d$covs[, 2] drops to a vector whose derived name is the non-syntactic
  # deparse "d$covs[, 2]"; the formula term must be matched as a literal
  # column name, not re-evaluated as an expression.
  df <- RMdifTree(d$items, covariates = d$covs[, 2],
                  minsize = 50, output = "dataframe")
  expect_s3_class(df, "data.frame")
  expect_true("d$covs[, 2]" %in% df$Variable)
})

# ---------------------------------------------------------------------
# Iterative purification, kable/plot output, pruning
# ---------------------------------------------------------------------

# Polytomous data with strong group DIF on I1/I2 so pctree() splits on the
# group and the partial-gamma effect sizes flag those items (exercising the
# purification loop).
make_dif_poly <- function(n = 400, seed = 11L) {
  set.seed(seed)
  grp   <- factor(rep(c("A", "B"), length.out = n))
  theta <- stats::rnorm(n)
  mk <- function(shift) {
    lin <- theta + ifelse(grp == "B", shift, 0)
    pmax(0L, pmin(2L, as.integer(round(lin + stats::rnorm(n, 0, 0.4) + 1))))
  }
  list(
    items = data.frame(I1 = mk(1.6), I2 = mk(1.6), I3 = mk(0),
                       I4 = mk(0), I5 = mk(0), I6 = mk(0)),
    covs  = data.frame(grp = grp)
  )
}

test_that("purification = 'iterative' (MH) runs on dichotomous DIF data", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  df <- RMdifTree(d$items, covariates = d$covs, minsize = 50,
                  purification = "iterative", output = "dataframe")
  expect_s3_class(df, "data.frame")
  expect_equal(attr(df, "effect_size"), "MH")
  expect_true(all(df$Class %in% c("A", "B", "C")))
  expect_true(any(df$Flagged))
})

test_that("purification = 'iterative' (partial gamma) with p_adj on polytomous data", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_poly()
  set.seed(1)
  df <- RMdifTree(d$items, covariates = d$covs, minsize = 50,
                  purification = "iterative", p_adj = "fdr",
                  output = "dataframe")
  expect_s3_class(df, "data.frame")
  expect_equal(attr(df, "effect_size"), "pgamma")
  expect_true(all(df$Class %in% c("A", "B", "C")))
})

test_that("default kable output renders per-split effect sizes", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  kbl <- RMdifTree(d$items, covariates = d$covs, minsize = 50,
                   p_adj = "bonferroni")
  expect_s3_class(kbl, "knitr_kable")
})

test_that("output = 'plot' draws the tree without error", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  grDevices::pdf(tempfile(fileext = ".pdf"))
  on.exit(grDevices::dev.off(), add = TRUE)
  expect_no_error(
    RMdifTree(d$items, covariates = d$covs, minsize = 50, output = "plot")
  )
})

test_that("prune_negligible = TRUE runs and returns a tree", {
  skip_on_cran()
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  set.seed(1)
  tr <- RMdifTree(d$items, covariates = d$covs, minsize = 50,
                  prune_negligible = TRUE, output = "tree")
  expect_s3_class(tr, "RMdifTree")
})

test_that("effect_size = 'MH' errors on polytomous data", {
  need_tree_pkgs()
  d <- make_dif_poly()
  expect_error(
    RMdifTree(d$items, covariates = d$covs, effect_size = "MH", minsize = 50),
    regexp = "dichotomous"
  )
})

test_that("invalid thresholds, alpha, and stability_B are rejected", {
  need_tree_pkgs()
  d <- make_dif_dichotomous()
  expect_error(
    RMdifTree(d$items, covariates = d$covs, thresholds = 0.5),
    regexp = "thresholds"
  )
  expect_error(
    RMdifTree(d$items, covariates = d$covs, alpha = 1.5),
    regexp = "alpha"
  )
  expect_error(
    RMdifTree(d$items, covariates = d$covs, stability_B = 0),
    regexp = "stability_B"
  )
})

# ---------------------------------------------------------------------
# output = "list"
# ---------------------------------------------------------------------
test_that("output = 'list' returns table + tree from one fit, matching the other outputs", {
  need_tree_pkgs()
  dd <- make_dif_dichotomous()

  res <- suppressMessages(RMdifTree(dd$items, dd$covs, output = "list"))
  expect_named(res, c("table", "tree"))
  expect_s3_class(res$table, "data.frame")
  expect_s3_class(res$tree, "RMdifTree")

  df <- suppressMessages(RMdifTree(dd$items, dd$covs, output = "dataframe"))
  expect_identical(res$table, df)
  expect_identical(attr(res$table, "effect_size"), "MH")
})

#' Conditional Item Characteristic Curves
#'
#' Faceted panel of Conditional Item Characteristic Curves (cICCs), one per
#' item. Each panel shows the model-expected conditional item score
#' \eqn{E[X_i \mid R = r]} against the total score \eqn{r}, together with the
#' average observed item score within class intervals of the total score and
#' their confidence intervals. When a `dif_var` is supplied, observed averages
#' are computed separately per group, turning each panel into a visual DIF
#' check, and the partial-gamma DIF magnitude is reported per item.
#'
#' The model curve and its conditional variance come from the shared CML
#' engine (CML item thresholds via `psychotools` and the exact conditional
#' distribution of each item score given the total score, via elementary
#' symmetric functions). The conditional-ICC approach follows Buchardt,
#' Christensen & Jensen (2023) and their `RASCHplot` package; the implementation
#' here is native to easyRasch2's CML/WLE engine.
#'
#' @param data A data.frame or matrix of item responses (non-negative
#'   integers, 0-based). One column per item, one row per person.
#' @param dif_var Optional vector of length `nrow(data)` (factor or coercible
#'   to factor) defining a DIF grouping variable. When supplied, observed
#'   averages are shown separately per group and the partial-gamma DIF
#'   coefficient (`iarm::partgam_DIF()`, as in [RMdifGamma()]) is annotated
#'   per panel. Default `NULL` (no DIF).
#' @param method How the observed means are grouped along the total score
#'   (see \emph{Class intervals} in Details). One of `"quantile"` (default):
#'   `class_intervals` groups of approximately equal numbers of respondents;
#'   `"width"`: `class_intervals` equal-width intervals over the observed
#'   total-score range; `"score"`: every observed total score is its own
#'   group; or `"manual"`: groups defined by `score_breaks`. `"cut"` is
#'   accepted as a legacy alias for `"quantile"`.
#' @param class_intervals Integer >= 2. Number of class intervals for
#'   `method = "quantile"` and `method = "width"`. Default `4`. Ignored
#'   (without an error) by `method = "score"` and `method = "manual"`.
#' @param score_breaks Integer vector for `method = "manual"`: the total
#'   scores at which a new group \emph{starts}, in increasing order. For
#'   example, `score_breaks = c(3, 6, 8)` on a 0-9 scale defines the groups
#'   0-2, 3-5, 6-7, and 8-9 (the endpoints are added automatically).
#'   Serves the same purpose as `lower.groups` in RASCHplot's `CICCplot()`,
#'   without the leading zero. Default `NULL`.
#' @param ci Logical. Draw confidence intervals (error bars) on the observed
#'   class-interval means. Default `TRUE`.
#' @param error_band Logical. Add a shaded band around the model-expected
#'   curve: the model's interval for the \emph{observed mean} at each total
#'   score, \eqn{E \pm z \sqrt{\mathrm{Var}/n_r}} (`n_r` = number of
#'   respondents at that total score), as in RASCHplot's `CICCplot`.
#'   Complementary to `ci`, not a replacement -- see Details. Default
#'   `FALSE`.
#' @param conf_level Numeric in (0, 1). Confidence level for the observed
#'   error bars and the model band. Default `0.95`.
#' @param min_n Integer. A group-by-interval cell needs at least this many
#'   respondents to contribute an observed point + CI; sparser cells are
#'   skipped. Default `8` (the package's per-cell stability floor).
#' @param items Optional character or integer vector selecting which items to
#'   plot. The model is always fitted on all items; only the rendering is
#'   filtered. Default `NULL` (all items).
#' @param output One of `"patchwork"` (default) -- composite patchwork figure
#'   -- or `"list"` -- a named list of per-item `ggplot` objects.
#'
#' @return Either a `patchwork`/`ggplot` composite (default) or a named list
#'   of per-item `ggplot` objects (`output = "list"`). In DIF mode the per-item
#'   partial-gamma table is attached as `attr(., "dif_gamma")`.
#'
#' @details
#' \strong{Conditioning.} The expected curve is the exact conditional
#' expectation of the item score given the total score (it accounts for the
#' item being part of the total), not a marginal ICC.
#'
#' \strong{Class intervals.} Following Buchardt et al. (2023), the empirical
#' item means can be shown for each value of the total score or for each
#' value of the \emph{grouped} total score; all grouping happens on the
#' total-score scale (never on an estimated latent score). Four grouping
#' rules are available. `method = "quantile"` (the default) splits the
#' observed total scores into `class_intervals` bins of approximately equal
#' numbers of respondents, using common boundaries so DIF groups share the
#' x-axis -- the same approach as `iarm::ICCplot()`'s class intervals
#' (`Hmisc::cut2(g = ...)`). `method = "width"` instead splits the observed
#' score range into `class_intervals` intervals of equal width, which reads
#' naturally on the score axis but can leave sparse intervals (pruned by
#' `min_n`). `method = "score"` uses every observed total score as its own
#' group -- the maximal-resolution display, sensible for short scales, with
#' `min_n` pruning thinly-populated scores. `method = "manual"` places the
#' group boundaries exactly where you say via `score_breaks` (RASCHplot's
#' `lower.groups` concept). Each bin contributes one observed point at its
#' mean total score. If the score distribution is too sparse to form
#' `class_intervals` distinct bins, the quantile and width methods fall
#' back to score-level points.
#'
#' \strong{Confidence intervals.} Observed error bars use the normal
#' approximation \eqn{\bar{x}_l \pm z \sqrt{\mathrm{var}(x_l) / n_l}} within
#' each (group, interval) cell, clamped to the item's score range; cells with
#' fewer than `min_n` respondents are dropped. In DIF mode this makes sparse
#' group differences visibly uncertain rather than over-interpreted.
#'
#' \strong{The model band (`error_band`).} The shaded band is drawn around
#' the model-expected curve at \eqn{E_{ri} \pm z \sqrt{V_{ri}/n_r}}, where
#' \eqn{E_{ri}} and \eqn{V_{ri}} are the exact conditional mean and variance
#' of the item score given total score \eqn{r} (model quantities), and
#' \eqn{n_r} is the \emph{observed} number of respondents at that total
#' score. If the Rasch model is true, this is where the empirical mean item
#' score at total score \eqn{r} should fall, given how many people actually
#' sit at that score; an observed diamond outside the band is localized
#' graphical misfit. The band is deliberately the standard error of the
#' \emph{mean} (\eqn{\sqrt{V/n_r}}), not the far wider individual-response
#' SD band, which would flag nothing. Its width therefore tracks the data:
#' narrow where many respondents sit (a strict test), wide in sparse score
#' regions (a lenient one), with gaps where \eqn{n_r = 0} and collapsing to
#' the curve at the deterministic extremes (total score 0 and the maximum).
#' It is complementary to, not a replacement for, the `ci` error bars: the
#' band is the model-implied uncertainty of the observed mean per raw
#' score; the error bars are the sample-based uncertainty of the observed
#' group means.
#'
#' \strong{DIF magnitude.} The annotated partial gamma (Bjorner et al., 1998)
#' is the association between the item score and the group conditional on the
#' rest score -- a non-parametric effect size, complementary to the
#' total-score-conditional visual. It is the same statistic as [RMdifGamma()].
#'
#' @references
#' Buchardt, A.-S., Christensen, K. B., & Jensen, S. N. (2023). Visualizing
#' Rasch item fit using conditional item characteristic curves in R.
#' *Psychological Test and Assessment Modeling, 65*(2), 206-219.
#'
#' Andersen, E. B. (1995). Polytomous Rasch models and their estimation. In
#' G. H. Fischer & I. W. Molenaar (Eds.), *Rasch Models: Foundations, Recent
#' Developments, and Applications* (pp. 271-291). Springer. (Conditional
#' expectation of item scores given the total score; formula 15.22.)
#'
#' Bjorner, J. B., Kreiner, S., Ware, J. E., Damsgaard, M. T., & Bech, P.
#' (1998). Differential item functioning in the Danish translation of the
#' SF-36. *Journal of Clinical Epidemiology, 51*(11), 1189-1202.
#'
#' @seealso \code{\link{RMdifGamma}}, \code{\link{RMitemRestscore}}
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE) &&
#'     requireNamespace("patchwork", quietly = TRUE) &&
#'     requireNamespace("iarm", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:2, 200 * 4, replace = TRUE), nrow = 200, ncol = 4)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:4)
#'
#'   # Default: quantile groups (~equal numbers of respondents per group)
#'   RMitemICCPlot(sim_data)
#'
#'   # Equal-width intervals on the total-score scale, with the model band
#'   RMitemICCPlot(sim_data, method = "width", error_band = TRUE)
#'
#'   # Every total score as its own group (short scales)
#'   RMitemICCPlot(sim_data, method = "score")
#'
#'   # Manual grouping: new groups start at total scores 3, 5, and 7
#'   RMitemICCPlot(sim_data, method = "manual", score_breaks = c(3, 5, 7))
#' }
#' }
#'
#' @importFrom rlang .data
#' @export
RMitemICCPlot <- function(data,
                          dif_var         = NULL,
                          method          = c("quantile", "width", "score",
                                              "manual", "cut"),
                          class_intervals = 4,
                          score_breaks    = NULL,
                          ci              = TRUE,
                          error_band      = FALSE,
                          conf_level      = 0.95,
                          min_n           = 8L,
                          items           = NULL,
                          output          = c("patchwork", "list")) {

  method <- match.arg(method)
  # "cut" is the pre-1.1.0 name of the quantile grouping, kept as a silent
  # legacy alias so existing code keeps working.
  if (method == "cut") {
    method <- "quantile"
  }
  output <- match.arg(output)

  if (!is.null(score_breaks)) {
    if (method != "manual") {
      stop(
        "`score_breaks` is only used with method = 'manual'.",
        call. = FALSE
      )
    }
    if (!is.numeric(score_breaks) || length(score_breaks) < 1L ||
        anyNA(score_breaks) || any(score_breaks != as.integer(score_breaks)) ||
        is.unsorted(score_breaks, strictly = TRUE) || any(score_breaks < 1)) {
      stop(
        "`score_breaks` must be a strictly increasing vector of positive ",
        "integers (each value starts a new total-score group).",
        call. = FALSE
      )
    }
    score_breaks <- as.integer(score_breaks)
  } else if (method == "manual") {
    stop(
      "method = 'manual' requires `score_breaks` (the total scores at ",
      "which a new group starts, e.g. c(3, 6, 8)).",
      call. = FALSE
    )
  }

  validate_response_data(data)
  data <- as.data.frame(data)
  if (ncol(data) < 2L) {
    stop("RMitemICCPlot() requires at least 2 items.", call. = FALSE)
  }
  if (method %in% c("quantile", "width")) {
    if (!is.numeric(class_intervals) || length(class_intervals) != 1L ||
        !is.finite(class_intervals) || class_intervals < 2L) {
      stop("`class_intervals` must be a single integer >= 2.", call. = FALSE)
    }
    class_intervals <- as.integer(class_intervals)
  }
  if (!is.numeric(conf_level) || length(conf_level) != 1L ||
      conf_level <= 0 || conf_level >= 1) {
    stop("`conf_level` must be a single number in (0, 1).", call. = FALSE)
  }
  if (!is.numeric(min_n) || length(min_n) != 1L || min_n < 1) {
    stop("`min_n` must be a single positive integer.", call. = FALSE)
  }
  min_n <- as.integer(min_n)

  # --- DIF variable + joint NA handling --------------------------------------
  dif_mode   <- !is.null(dif_var)
  dif_factor <- NULL
  if (dif_mode) {
    if (length(dif_var) != nrow(data)) {
      stop("`dif_var` must have the same length as `nrow(data)` (",
           nrow(data), "). Got ", length(dif_var), ".", call. = FALSE)
    }
    dif_factor <- droplevels(as.factor(dif_var))
  }
  keep <- stats::complete.cases(data)
  if (dif_mode) keep <- keep & !is.na(dif_factor)
  data <- data[keep, , drop = FALSE]
  if (dif_mode) dif_factor <- droplevels(dif_factor[keep])
  if (nrow(data) == 0L) stop("No complete cases in `data`.", call. = FALSE)
  if (dif_mode && nlevels(dif_factor) < 2L) {
    stop("`dif_var` must have at least 2 distinct non-missing levels ",
         "after NA filtering.", call. = FALSE)
  }

  # --- Which items to render (model is always fitted on all) -----------------
  item_names <- names(data)
  show_items <- if (is.null(items)) item_names else {
    if (is.numeric(items)) {
      if (any(items < 1L | items > ncol(data))) {
        stop("`items` indices out of range.", call. = FALSE)
      }
      item_names[as.integer(items)]
    } else {
      unknown <- setdiff(items, item_names)
      if (length(unknown) > 0L) {
        stop("Unknown item(s) in `items`: ", paste(unknown, collapse = ", "),
             call. = FALSE)
      }
      items
    }
  }

  # --- Plotting packages (checked after input validation) --------------------
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for RMitemICCPlot(). ",
         "Install with: install.packages(\"ggplot2\")", call. = FALSE)
  }
  if (output == "patchwork" && !requireNamespace("patchwork", quietly = TRUE)) {
    stop("Package 'patchwork' is required for output = \"patchwork\". ",
         "Install with: install.packages(\"patchwork\")", call. = FALSE)
  }

  # --- Model curve + conditional variance (shared CML engine) ----------------
  data_mat <- as.matrix(data)
  .sparsity_warning(data, is_poly = max(data_mat, na.rm = TRUE) > 1L)
  thr_list <- .fit_cml_thresholds(data_mat)
  mom      <- .cond_moments(thr_list)              # $E, $V by total score; $max_score
  max_sc   <- mom$max_score
  steps    <- vapply(thr_list, length, integer(1L))   # per-item max score
  rs         <- rowSums(data_mat)
  n_by_score <- tabulate(rs + 1L, nbins = max_sc + 1L)  # respondents per total score
  z          <- stats::qnorm(1 - (1 - conf_level) / 2)

  # --- Common class-interval assignment --------------------------------------
  if (method == "manual" && any(score_breaks > max_sc)) {
    stop(
      "`score_breaks` contains values above the maximum possible total ",
      "score (", max_sc, ").",
      call. = FALSE
    )
  }
  bin <- .cicc_class_bins(rs, method, class_intervals, score_breaks)
  grp <- if (dif_mode) dif_factor else factor(rep("all", nrow(data_mat)))

  # Dodge width for per-group points/bars: scales with the number of DIF groups,
  # but capped at the gap between adjacent class-interval centres so groups stay
  # within their interval (more groups -> wider spread; 0 when not in DIF mode).
  n_grp       <- nlevels(grp)
  bin_centres <- sort(unique(as.numeric(tapply(rs, bin, mean))))
  bin_spacing <- if (length(bin_centres) > 1L) stats::median(diff(bin_centres)) else 1
  dodge_width <- if (dif_mode && n_grp > 1L) min(n_grp * 1, 0.9 * bin_spacing) else 0

  # --- Partial-gamma DIF magnitude (per item; engine-invariant via iarm) -----
  gamma_tab <- NULL
  if (dif_mode) {
    gamma_tab <- tryCatch(
      RMdifGamma(data, dif_var = dif_factor, output = "dataframe"),
      error = function(e) NULL
    )
  }

  # --- Build per-item panels -------------------------------------------------
  panels <- lapply(show_items, function(nm) {
    i     <- match(nm, item_names)
    curve <- .cicc_curve_df(mom, i, steps[i], z, n_by_score)
    obs   <- .cicc_obs_df(data_mat[, i], rs, grp, bin, min_n, z, steps[i])
    gl    <- .cicc_gamma_label(gamma_tab, nm)
    .cicc_panel(curve, obs, nm, steps[i], error_band, ci, dif_mode, gl, dodge_width)
  })
  names(panels) <- show_items

  if (output == "list") {
    if (!is.null(gamma_tab)) attr(panels, "dif_gamma") <- gamma_tab
    return(panels)
  }

  out <- patchwork::wrap_plots(panels, axes = "collect", guides = "collect") +
    patchwork::plot_annotation(
      title   = "Conditional Item Characteristic Curves",
      caption = er2_caption(.cicc_caption(dif_mode, conf_level)),
      theme   = er2_plot_caption()
    )
  if (!is.null(gamma_tab)) attr(out, "dif_gamma") <- gamma_tab
  out
}

# ===========================================================================
# Internal helpers
# ===========================================================================

#' Assign respondents to total-score class intervals
#' @keywords internal
#' @noRd
.cicc_class_bins <- function(rs, method, class_intervals,
                             score_breaks = NULL) {
  if (method == "score") return(factor(rs))

  if (method == "manual") {
    # Each value in score_breaks starts a new group: breaks c(3, 6, 8) on a
    # 0-9 scale give the groups 0-2, 3-5, 6-7, 8-9. Groups left empty by the
    # data simply contribute no observed point (min_n prunes them anyway).
    lows   <- c(0L, score_breaks)
    highs  <- c(score_breaks - 1L, max(rs, score_breaks))
    labels <- ifelse(lows == highs, as.character(lows),
                     paste0(lows, "-", highs))
    return(cut(rs,
               breaks = c(-0.5, score_breaks - 0.5, max(rs, score_breaks) + 0.5),
               labels = labels, include.lowest = TRUE))
  }

  if (method == "width") {
    # Equal-width intervals over the observed total-score range.
    breaks <- unique(seq(min(rs) - 0.5, max(rs) + 0.5,
                         length.out = class_intervals + 1L))
    if (length(breaks) < 3L) {
      # Too few distinct total scores to form bins: fall back to score level.
      return(factor(rs))
    }
    return(cut(rs, breaks = breaks, include.lowest = TRUE))
  }

  # method == "quantile": equal-count bins over the observed total scores.
  qs <- unique(stats::quantile(
    rs, probs = seq(0, 1, length.out = class_intervals + 1L), na.rm = TRUE))
  if (length(qs) < 3L) {
    # Too few distinct total scores to form bins: fall back to score level.
    return(factor(rs))
  }
  cut(rs, breaks = qs, include.lowest = TRUE)
}

#' Expected conditional item score (and SE-of-mean band) per total score
#'
#' The optional band is the model 95\% interval for the *observed mean* at each
#' total score, \eqn{E \pm z \sqrt{\mathrm{Var}/n_r}}, with `n_r` the number of
#' respondents at that total score (as in RASCHplot's `CICCplot`) -- not the
#' much wider individual-response SD. It is `NA` where no respondent has that
#' score and collapses to the point estimate at the degenerate extremes
#' (`Var = 0`).
#' @keywords internal
#' @noRd
.cicc_curve_df <- function(mom, i, max_y, z, n_by_score) {
  max_sc   <- mom$max_score
  expected <- numeric(max_sc + 1L)
  v        <- numeric(max_sc + 1L)
  expected[1L] <- 0; expected[max_sc + 1L] <- max_y      # degenerate extremes
  v[1L] <- 0; v[max_sc + 1L] <- 0
  if (max_sc >= 2L) {
    interior <- 2L:max_sc                                # total scores 1..max-1
    expected[interior] <- mom$E[interior, i]
    v[interior]        <- mom$V[interior, i]
  }
  # Standard error of the conditional *mean* at each total score (variance
  # divided by the count at that score), not the individual-response SD.
  se <- sqrt(pmax(v, 0) / n_by_score)
  se[!is.finite(se)] <- NA_real_                         # no respondents at score
  se[v == 0]         <- 0                                # deterministic extremes
  data.frame(
    score    = 0:max_sc,
    expected = expected,
    band_lo  = pmax(0,     expected - z * se),
    band_hi  = pmin(max_y, expected + z * se),
    stringsAsFactors = FALSE
  )
}

#' Observed class-interval means + CIs for one item (per group)
#' @keywords internal
#' @noRd
.cicc_obs_df <- function(y, rs, grp, bin, min_n, z, max_y) {
  # Common x per bin (shared across groups) so per-group points/error bars at
  # the same interval can be separated cleanly with position_dodge().
  bin_x <- tapply(rs, bin, mean)
  key   <- interaction(grp, bin, drop = TRUE)
  parts <- split(seq_along(y), key)
  rows  <- lapply(parts, function(idx) {
    if (length(idx) < min_n) return(NULL)
    yy <- y[idx]
    m  <- mean(yy)
    se <- sqrt(stats::var(yy) / length(idx))
    data.frame(
      grp  = grp[idx][1L],
      x    = as.numeric(bin_x[as.character(bin[idx][1L])]),
      mean = m,
      lo   = max(0,     m - z * se),
      hi   = min(max_y, m + z * se),
      stringsAsFactors = FALSE
    )
  })
  rows <- Filter(Negate(is.null), rows)
  if (length(rows) == 0L) return(NULL)
  do.call(rbind, rows)
}

#' Per-item partial-gamma DIF annotation string (or NULL)
#' @keywords internal
#' @noRd
.cicc_gamma_label <- function(gamma_tab, nm) {
  if (is.null(gamma_tab)) return(NULL)
  row <- gamma_tab[gamma_tab$Item == nm, , drop = FALSE]
  if (nrow(row) == 0L) return(NULL)
  row <- row[1L, ]
  # Short label for the in-plot annotation; `p` is the BH-adjusted DIF p-value.
  # \u03b3 is the Greek small letter gamma (escaped to keep R source ASCII).
  sprintf("\u03b3 = %.2f [%.2f, %.2f]\np = %.3f",
          row$gamma, row$lower, row$upper, row$padj_bh)
}

#' Composite caption explaining the markers and the DIF annotation
#' @keywords internal
#' @noRd
.cicc_caption <- function(dif_mode, conf_level) {
  pct <- round(conf_level * 100)
  g   <- intToUtf8(947L)   # Greek small letter gamma (keeps R source ASCII)
  cap <- sprintf(paste0(
    "Black line: model-expected conditional item score. Diamonds: observed mean ",
    "item score per total-score group with %d%% confidence-interval error ",
    "bars."), pct)
  if (dif_mode) {
    cap <- paste0(cap, sprintf(paste0(
      " %s is Goodman-Kruskal's gamma (partial gamma DIF coefficient). The ",
      "bracketed values are its %d%% confidence interval and p is the ",
      "Benjamini-Hochberg adjusted p-value."), g, pct))
  }
  cap
}

#' Build one cICC ggplot panel
#' @keywords internal
#' @noRd
.cicc_panel <- function(curve, obs, item_name, max_y, error_band, ci, dif_mode,
                        gamma_label, dodge_width) {
  curve$item <- item_name                     # single facet -> grey strip label
  dodge      <- ggplot2::position_dodge(width = dodge_width)

  p <- ggplot2::ggplot()
  if (error_band) {
    p <- p + ggplot2::geom_ribbon(
      data = curve,
      ggplot2::aes(x = .data$score, ymin = .data$band_lo, ymax = .data$band_hi),
      fill = "grey85", alpha = 0.6
    )
  }
  p <- p + ggplot2::geom_line(
    data = curve, ggplot2::aes(x = .data$score, y = .data$expected),
    colour = "black", linewidth = 0.8
  )
  if (!is.null(obs)) {
    obs$item <- item_name
    if (dif_mode) {
      if (ci) {
        p <- p + ggplot2::geom_errorbar(
          data = obs,
          ggplot2::aes(x = .data$x, ymin = .data$lo, ymax = .data$hi,
                       colour = .data$grp),
          width = 0, linewidth = 0.75, position = dodge
        )
      }
      p <- p + ggplot2::geom_point(
        data = obs,
        ggplot2::aes(x = .data$x, y = .data$mean, colour = .data$grp),
        shape = 18, size = 2.9, position = dodge
      )
    } else {
      if (ci) {
        p <- p + ggplot2::geom_errorbar(
          data = obs, ggplot2::aes(x = .data$x, ymin = .data$lo, ymax = .data$hi),
          width = 0, linewidth = 0.75, colour = "sienna2"
        )
      }
      p <- p + ggplot2::geom_point(
        data = obs, ggplot2::aes(x = .data$x, y = .data$mean),
        shape = 18, size = 2.9, colour = "sienna2"
      )
    }
  }
  # DIF magnitude annotated top-left inside the panel (like iarm's cICC).
  if (!is.null(gamma_label)) {
    p <- p + ggplot2::annotate(
      "text", x = 0, y = max_y, label = gamma_label,
      hjust = 0, vjust = 1, size = 3, colour = "grey20"
    )
  }
  xmax  <- max(curve$score)
  xstep <- if (xmax <= 15) 1L else if (xmax <= 40) 3L else 5L
  p +
    ggplot2::facet_wrap(~ item) +
    ggplot2::scale_x_continuous(breaks = seq(0L, xmax, by = xstep)) +
    ggplot2::labs(x = "Total score", y = "Item score", colour = NULL) +
    ggplot2::theme_bw(base_size = 11) +
    ggplot2::theme(
      strip.background = ggplot2::element_rect(fill = "grey95", colour = "grey70"),
      strip.text       = ggplot2::element_text(face = "bold")
    ) +
    er2_axis_margins()
}

# Tests for RMitemICCPlot()

# Polytomous data gives enough populated total-score levels for class
# intervals and DIF groups at modest n.
make_polytomous <- function(n = 200, k = 5, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:3, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation (reachable without the plotting packages)
# ---------------------------------------------------------------------
test_that("RMitemICCPlot errors when data has non-zero minimum", {
  df <- make_polytomous() + 1L
  expect_error(RMitemICCPlot(df), regexp = "scored starting at 0")
})

test_that("RMitemICCPlot errors when fewer than 2 items", {
  df <- make_polytomous()[, 1L, drop = FALSE]
  expect_error(RMitemICCPlot(df), regexp = "at least 2 items")
})

test_that("RMitemICCPlot errors when class_intervals < 2", {
  df <- make_polytomous()
  expect_error(RMitemICCPlot(df, class_intervals = 1),
               regexp = "class_intervals")
})

test_that("RMitemICCPlot errors when conf_level is out of range", {
  df <- make_polytomous()
  expect_error(RMitemICCPlot(df, conf_level = 1.2), regexp = "conf_level")
})

test_that("RMitemICCPlot errors when dif_var length mismatches nrow(data)", {
  df  <- make_polytomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df) - 1L))
  expect_error(RMitemICCPlot(df, dif_var = grp), regexp = "same length")
})

test_that("RMitemICCPlot errors when dif_var has fewer than 2 levels", {
  df  <- make_polytomous()
  grp <- factor(rep("A", nrow(df)))
  expect_error(RMitemICCPlot(df, dif_var = grp), regexp = "at least 2 distinct")
})

# ---------------------------------------------------------------------
# Output structures (no iarm needed for the non-DIF CML plot)
# ---------------------------------------------------------------------
test_that("RMitemICCPlot default output is a patchwork/ggplot composite", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("patchwork")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMitemICCPlot(df, class_intervals = 4L)
  expect_s3_class(p, "patchwork")
  expect_s3_class(p, "ggplot")
})

test_that("RMitemICCPlot output = 'list' returns one ggplot per item, named", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  plist <- RMitemICCPlot(df, class_intervals = 4L, output = "list")
  expect_type(plist, "list")
  expect_equal(length(plist), ncol(df))
  expect_equal(names(plist), names(df))
  for (p in plist) expect_s3_class(p, "ggplot")
})

test_that("RMitemICCPlot method = 'score' and error_band return a composite", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("patchwork")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  expect_s3_class(RMitemICCPlot(df, method = "score"), "patchwork")
  expect_s3_class(RMitemICCPlot(df, error_band = TRUE, ci = FALSE), "patchwork")
})

test_that("RMitemICCPlot `items` subsets the rendered panels", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  plist <- RMitemICCPlot(df, items = c("I2", "I4"), output = "list")
  expect_equal(names(plist), c("I2", "I4"))
})

# ---------------------------------------------------------------------
# DIF mode (needs iarm for the partial-gamma magnitude)
# ---------------------------------------------------------------------
test_that("RMitemICCPlot DIF mode returns a composite with a gamma table attr", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("iarm")
  skip_if_not_installed("patchwork")
  skip_if_not_installed("ggplot2")
  df  <- make_polytomous(n = 300)
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  p <- RMitemICCPlot(df, class_intervals = 4L, dif_var = grp)
  expect_s3_class(p, "patchwork")
  g <- attr(p, "dif_gamma")
  expect_s3_class(g, "data.frame")
  expect_true(all(c("Item", "gamma", "lower", "upper", "padj_bh") %in% names(g)))
})

# ---------------------------------------------------------------------
# NA handling
# ---------------------------------------------------------------------
test_that("RMitemICCPlot drops rows with NA in data or dif_var", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("iarm")
  skip_if_not_installed("patchwork")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous(n = 300)
  df[1:5, 1] <- NA_integer_
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  grp[6:10] <- NA
  expect_s3_class(
    RMitemICCPlot(df, class_intervals = 4L, dif_var = grp), "patchwork"
  )
})

# ---------------------------------------------------------------------
# Grouping methods (quantile / width / score / manual, "cut" alias)
# ---------------------------------------------------------------------
test_that("all grouping methods run and 'cut' aliases 'quantile'", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("patchwork")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 150 * 4, replace = TRUE), nrow = 150))
  colnames(df) <- paste0("i", 1:4)

  for (m in c("quantile", "width", "score")) {
    p <- suppressWarnings(RMitemICCPlot(df, method = m))
    expect_s3_class(p, "patchwork")
  }
  # legacy alias gives the same grouping as "quantile"
  p_cut <- suppressWarnings(RMitemICCPlot(df, method = "cut", output = "list"))
  p_q   <- suppressWarnings(RMitemICCPlot(df, method = "quantile", output = "list"))
  expect_identical(p_cut[[1]]$data, p_q[[1]]$data)
})

test_that("manual grouping via score_breaks works and validates", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("patchwork")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 150 * 4, replace = TRUE), nrow = 150))
  colnames(df) <- paste0("i", 1:4)

  p <- suppressWarnings(
    RMitemICCPlot(df, method = "manual", score_breaks = c(3, 5, 7)))
  expect_s3_class(p, "patchwork")

  # validation
  expect_error(RMitemICCPlot(df, method = "manual"),
               "requires `score_breaks`")
  expect_error(RMitemICCPlot(df, method = "quantile", score_breaks = c(3)),
               "only used with method")
  expect_error(RMitemICCPlot(df, method = "manual", score_breaks = c(5, 3)),
               "strictly increasing")
  expect_error(RMitemICCPlot(df, method = "manual", score_breaks = c(3, 99)),
               "maximum possible total")
})

#' Martin-Löf Test of Unidimensionality
#'
#' Likelihood-ratio test of unidimensionality against an *a priori* specified
#' multidimensional alternative, generalised to polytomous Rasch / partial
#' credit models (Christensen, Bjorner, Kreiner, & Petersen, 2002). The
#' p-value is obtained by parametric-bootstrap (Monte Carlo) sampling under
#' the unidimensional null, following Christensen & Kreiner (2007), because
#' the asymptotic chi-square approximation is biased toward conservatism for
#' realistic sample sizes -- especially with polytomous items, where the
#' degrees of freedom can be very large.
#'
#' This is **not** a routine screening tool. The test requires an *a priori*
#' partition of items into subscales; using it post-hoc on, e.g., the
#' partition suggested by `RMdimResidualPCA()`'s PC1 sign would inflate the
#' Type-I error rate. Both source papers state this explicitly.
#'
#' @param data A data.frame or matrix of item responses (0-based,
#'   non-negative integers). Rows with any `NA` are dropped.
#' @param partition The hypothesised partition of items into subscales. One of:
#'   * a **list** of column-name or column-index vectors, e.g.
#'     `list(c("I1","I2","I3"), c("I4","I5","I6"))`;
#'   * a **vector** of length `ncol(data)` indicating each item's subscale
#'     (factor, character, or integer), e.g. `c(1,1,1,2,2,2)`.
#'   Each subscale must contain at least two items. Subscales must not
#'   overlap; items not assigned to any subscale are dropped with a warning.
#' @param iterations Integer. Maximum number of Monte Carlo iterations
#'   (default `1000`).
#' @param stopping Character. `"none"` (default) runs all iterations.
#'   `"sequential"` uses Besag & Clifford's (1991) sequential rule: stop as
#'   soon as `h` simulated statistics have exceeded the observed value. The
#'   sequential strategy substantially reduces compute time when H0 holds
#'   but cannot be parallelised.
#' @param h Integer. Sequential-stopping count threshold (default `50`).
#'   Ignored when `stopping = "none"`.
#' @param alpha Numeric in (0, 1). Nominal significance level used only for
#'   the `rejected` flag in the result; default `0.05`.
#' @param parallel Logical. Use parallel processing via `mirai` (default
#'   `TRUE`). Ignored when `stopping = "sequential"`.
#' @param n_cores Integer or `NULL`. Number of parallel workers. When `NULL`,
#'   `getOption("mc.cores")` is checked first; if neither is set, falls back
#'   to sequential with a warning.
#' @param verbose Logical. Show a progress bar (default `FALSE`).
#' @param seed Integer or `NULL`. Random seed for reproducibility. Items
#'   are processed internally in a fixed (alphabetical) order, so the same
#'   seed reproduces the same p-value regardless of how the data's columns
#'   are arranged.
#'
#' @return A list with components:
#' \describe{
#'   \item{`T_obs`}{Observed Martin-Löf likelihood-ratio statistic.}
#'   \item{`p_value`}{Monte Carlo p-value with `(n_exceed + 1) / (n + 1)`
#'     correction. The attainable p-values are `k / (n + 1)` for
#'     `k = 1, ..., n + 1`, so the p-value's resolution is limited by the
#'     number of iterations: with 100 iterations the smallest attainable
#'     value is `1/101 = 0.0099`. A reported p-value *equal to the floor*
#'     (see `p_value_floor`) means no simulated statistic reached the
#'     observed one and should be read as "p < floor" -- the true p-value
#'     may be much smaller; increase `iterations` for finer resolution.}
#'   \item{`p_value_floor`}{The smallest attainable p-value,
#'     `1 / (actual_iterations + 1)`.}
#'   \item{`actual_iterations`}{Number of successful MC iterations
#'     completed.}
#'   \item{`rejected`}{Logical: is `p_value < alpha`?}
#'   \item{`partition`}{Normalised partition (list of integer indices).}
#'   \item{`n_subscales`}{Number of subscales.}
#'   \item{`is_polytomous`}{Whether a PCM was fitted.}
#'   \item{`sample_n`}{Number of complete cases analysed.}
#'   \item{`n_items`}{Number of items.}
#'   \item{`stopping`}{The stopping strategy used.}
#'   \item{`h`}{The sequential-stopping count, or `NA` for `stopping = "none"`.}
#'   \item{`T_rep`}{Numeric vector of successful MC test statistics.}
#'   \item{`wle_scores`}{data.frame with one row per person and one column
#'     per subscale (`subscale_1_wle`, ..., `subscale_D_wle`), giving Warm's
#'     Weighted Likelihood Estimate of theta from a CML fit on each subscale
#'     alone. Persons whose subscore equals the minimum or maximum on a
#'     subscale produce non-finite WLEs (`Inf` / `-Inf`) and are excluded
#'     from `wle_correlation` pairwise.}
#'   \item{`wle_correlation`}{data.frame of pairwise Pearson correlations
#'     between subscale WLEs, with columns `subscale_a`, `subscale_b`, `r`,
#'     `ci_lower`, `ci_upper` (95% CI from `stats::cor.test`), `p_value`,
#'     and `n` (number of persons with finite WLEs on both subscales). One
#'     row per pair; for D = 2, a single row. Useful as an effect-size
#'     companion to `p_value` -- a rejected test with `r` near 1 indicates a
#'     small effect; `r` clearly below 1 indicates substantive
#'     multidimensionality.}
#' }
#'
#' @details
#' **Test statistic.** With items partitioned into D subscales, total score
#' \eqn{t} and subscores \eqn{(t_1, \ldots, t_D)} (Christensen et al. 2002,
#' eq. 22):
#' \deqn{T = 2\Bigl[\sum_{t_1, \ldots, t_D}
#'   n_{t_1, \ldots, t_D}\log(n_{t_1, \ldots, t_D}/N)
#'   - \sum_t n_t\log(n_t/N)
#'   - \ell_C(\hat{\epsilon}) + \sum_d \ell_C(\hat{\epsilon}^{(d)})\Bigr]}
#' where \eqn{\ell_C} is the conditional log-likelihood and the
#' \eqn{\hat{\epsilon}^{(d)}} are CML estimates on the d-th subscale alone.
#' CML fits use `psychotools::raschmodel()` (RM) or `psychotools::pcmodel()`
#' (PCM) for speed.
#'
#' **Monte Carlo sampling under H0.** Following Christensen & Kreiner (2007):
#' (a) sample N total scores from the empirical score distribution
#' \eqn{n_t/N}; (b) for each sampled score, sample an item-response vector
#' from the conditional distribution \eqn{p(x \mid t, \hat{\epsilon})} given
#' by eq. 4 of the paper. For dichotomous data the fast algorithm of
#' Christensen & Kreiner (2007, p. 23) is used (sample without replacement
#' weighted by item easinesses). For polytomous data the recursive
#' \eqn{\gamma}-function approach is used, with each item's response sampled
#' conditional on the remaining items' joint score distribution (computed via
#' `psychotools::elementary_symmetric_functions()`).
#'
#' Iterations that fail (e.g., simulated dataset has an empty category for an
#' item) are silently dropped.
#'
#' Item parameters are estimated once on the observed data and held fixed
#' across MC iterations. Christensen & Kreiner (2007) use the extended
#' likelihood function (Tjur, 1982) with the empirical score distribution as
#' a non-parametric estimate of the latent distribution, so no distributional
#' assumption about \eqn{\theta} is needed.
#'
#' @references
#' Christensen, K. B., Bjorner, J. B., Kreiner, S., & Petersen, J. H. (2002).
#' Testing unidimensionality in polytomous Rasch models. *Psychometrika,
#' 67*(4), 563-574. \doi{10.1007/BF02295132}
#'
#' Christensen, K. B., & Kreiner, S. (2007). A Monte Carlo approach to
#' unidimensionality testing in polytomous Rasch models. *Applied
#' Psychological Measurement, 31*(1), 20-30.
#' \doi{10.1177/0146621605286204}
#'
#' Besag, J., & Clifford, P. (1991). Sequential Monte Carlo p-values.
#' *Biometrika, 78*(2), 301-304. \doi{10.1093/biomet/78.2.301}
#'
#' @seealso \code{\link{RMdimResidualPCA}}, \code{\link{RMdimResidualPCACutoff}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' # Build 2-dimensional polytomous data: 4 items per subscale, 5 categories
#' n     <- 400
#' theta1 <- rnorm(n)
#' theta2 <- 0.6 * theta1 + sqrt(1 - 0.6^2) * rnorm(n)
#' make_pcm <- function(theta, n_items, taus) {
#'   sapply(seq_len(n_items), function(j) {
#'     # ... toy simulation here
#'     sample(0:4, n, replace = TRUE)
#'   })
#' }
#' dat <- cbind(make_pcm(theta1, 4, NULL), make_pcm(theta2, 4, NULL))
#' colnames(dat) <- paste0("I", 1:8)
#'
#' # Few iterations for a fast example; use 1000+ in real analyses
#' RMdimMartinLof(dat,
#'             partition = list(c("I1","I2","I3","I4"),
#'                              c("I5","I6","I7","I8")),
#'             iterations = 100, parallel = FALSE, seed = 1)
#'
#' # Sequential stopping: stop as soon as h = 25 simulated statistics exceed
#' # the observed one (cuts compute time under H0).
#' RMdimMartinLof(dat,
#'             partition = c(1,1,1,1,2,2,2,2),
#'             iterations = 200, stopping = "sequential", h = 25,
#'             seed = 1)
#' }
RMdimMartinLof <- function(
  data,
  partition,
  iterations = 1000L,
  stopping = c("none", "sequential"),
  h = 50L,
  alpha = 0.05,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL
) {
  stopping <- match.arg(stopping)

  validate_response_data(data)

  data <- stats::na.omit(as.data.frame(data))
  if (nrow(data) < 30L) {
    stop("Need at least 30 complete cases.", call. = FALSE)
  }

  # Normalise partition
  partition_list <- normalize_ml_partition(partition, data)
  D <- length(partition_list)
  if (D < 2L) {
    stop("Partition must specify >= 2 subscales.", call. = FALSE)
  }
  if (any(lengths(partition_list) < 2L)) {
    stop("Each subscale must contain at least 2 items.", call. = FALSE)
  }

  # Restrict data to items used in the partition (drop unassigned items)
  used_idx <- sort(unique(unlist(partition_list)))
  if (length(used_idx) < ncol(data)) {
    data <- data[, used_idx, drop = FALSE]
    # Re-index partition relative to the restricted column set
    remap <- stats::setNames(seq_along(used_idx), used_idx)
    partition_list <- lapply(partition_list, function(idx) {
      unname(remap[as.character(idx)])
    })
  }

  # Canonical item order (alphabetical by column name): the Monte Carlo
  # sampler consumes the RNG per item in column order, so a fixed internal
  # order makes results reproducible under the same seed regardless of how
  # the caller's columns happen to be arranged. The test statistic and the
  # subscale WLEs are order-invariant anyway.
  canon <- order(colnames(data))
  if (!identical(canon, seq_along(canon))) {
    remap <- stats::setNames(seq_along(canon), canon)
    data <- data[, canon, drop = FALSE]
    partition_list <- lapply(partition_list, function(idx) {
      sort(unname(remap[as.character(idx)]))
    })
  }

  data_mat <- as.matrix(data)
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L
  N <- nrow(data)

  # rgl workaround (in case any helper triggers it)
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # Item parameters for sampling under H0 (fixed at observed-data CML estimates)
  sampling_params <- extract_ml_sampling_params(data, is_polytomous)

  # Observed test statistic (also returns log-likelihood components)
  T_obs <- compute_ml_statistic(data, partition_list, is_polytomous)

  # ---- Per-subscale WLE thetas + pairwise correlations ---------------------
  # Diagnostic to interpret rejection / non-rejection: under H0 the WLE
  # estimates from each subscale should be highly correlated; clearly < 1
  # values point to substantive multidimensionality.
  wle_compute <- compute_subscale_wle_and_correlations(
    data,
    partition_list,
    is_polytomous
  )
  wle_scores <- wle_compute$wle_scores
  wle_correlation <- wle_compute$wle_correlation

  # Empirical total-score distribution for sampling
  total_scores <- rowSums(data)
  score_table <- table(total_scores)
  score_values <- as.integer(names(score_table))
  score_probs <- as.numeric(score_table) / N

  # Per-iteration seed list
  if (!is.null(seed)) {
    set.seed(seed)
  }
  sim_seeds <- sample.int(.Machine$integer.max, iterations)

  sim_data_list <- list(
    N = N,
    is_polytomous = is_polytomous,
    sampling_params = sampling_params,
    score_values = score_values,
    score_probs = score_probs,
    item_names = colnames(data),
    partition_list = partition_list
  )

  # ---- Run iterations -------------------------------------------------------
  if (stopping == "none") {
    use_parallel <- parallel && requireNamespace("mirai", quietly = TRUE)
    if (parallel && !use_parallel) {
      message(
        "Install 'mirai' for parallel processing: install.packages(\"mirai\")"
      )
      message("Running sequentially...")
    }
    if (use_parallel) {
      if (is.null(n_cores)) {
        n_cores <- getOption("mc.cores")
      }
      if (is.null(n_cores)) {
        warning(
          "For parallel processing, specify n_cores or set options(mc.cores = N).\n",
          "Falling back to sequential.",
          call. = FALSE
        )
        use_parallel <- FALSE
      } else {
        n_cores <- min(n_cores, iterations)
      }
    }

    results_raw <- if (use_parallel) {
      run_ml_sim_parallel(
        iterations,
        sim_seeds,
        sim_data_list,
        n_cores,
        verbose
      )
    } else {
      run_ml_sim_sequential(iterations, sim_seeds, sim_data_list, verbose)
    }

    T_rep <- vapply(
      results_raw,
      function(x) {
        if (is.numeric(x) && length(x) == 1L) as.numeric(x) else NA_real_
      },
      numeric(1L)
    )
    T_rep <- T_rep[is.finite(T_rep)]
    actual_iterations <- length(T_rep)
    if (actual_iterations < 2L) {
      stop(
        "Fewer than 2 successful Monte Carlo iterations; cannot estimate p-value.",
        call. = FALSE
      )
    }
    n_exceed <- sum(T_rep >= T_obs)
    p_value <- (n_exceed + 1) / (actual_iterations + 1)
  } else {
    # Sequential stopping (Besag & Clifford 1991)
    if (verbose) {
      pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
    }
    n_exceed <- 0L
    actual_iterations <- 0L
    T_rep <- numeric(0)
    for (i in seq_len(iterations)) {
      result <- run_single_ml_iteration(sim_seeds[i], sim_data_list)
      if (is.numeric(result) && length(result) == 1L && is.finite(result)) {
        actual_iterations <- actual_iterations + 1L
        T_rep <- c(T_rep, result)
        if (result >= T_obs) {
          n_exceed <- n_exceed + 1L
          if (n_exceed >= h) break
        }
      }
      if (verbose) utils::setTxtProgressBar(pb, i)
    }
    if (verbose) {
      close(pb)
      message("")
    }

    if (actual_iterations < 2L) {
      stop(
        "Fewer than 2 successful Monte Carlo iterations; cannot estimate p-value.",
        call. = FALSE
      )
    }
    p_value <- (n_exceed + 1) / (actual_iterations + 1)
  }

  list(
    T_obs = T_obs,
    p_value = p_value,
    p_value_floor = 1 / (actual_iterations + 1),
    actual_iterations = actual_iterations,
    rejected = p_value < alpha,
    partition = partition_list,
    n_subscales = D,
    is_polytomous = is_polytomous,
    sample_n = N,
    n_items = ncol(data),
    stopping = stopping,
    h = if (stopping == "sequential") h else NA_integer_,
    T_rep = T_rep,
    wle_scores = wle_scores,
    wle_correlation = wle_correlation
  )
}

# ===========================================================================
# Internal helpers
# ===========================================================================

#' Normalise a partition argument to a list of integer column indices
#'
#' @keywords internal
#' @noRd
normalize_ml_partition <- function(partition, data) {
  n_items <- ncol(data)
  item_names <- colnames(data)

  if (is.list(partition)) {
    partition_list <- lapply(partition, function(p) {
      if (is.numeric(p)) {
        idx <- as.integer(p)
        if (any(idx < 1L) || any(idx > n_items)) {
          stop("Partition indices out of range.", call. = FALSE)
        }
        idx
      } else if (is.character(p)) {
        idx <- match(p, item_names)
        if (any(is.na(idx))) {
          missing <- p[is.na(idx)]
          stop(
            "Items not found in data: ",
            paste(missing, collapse = ", "),
            call. = FALSE
          )
        }
        idx
      } else {
        stop(
          "Each partition list element must be numeric or character.",
          call. = FALSE
        )
      }
    })
  } else if (
    is.factor(partition) || is.character(partition) || is.numeric(partition)
  ) {
    if (length(partition) != n_items) {
      stop(
        "Partition vector must have length equal to ncol(data) (",
        n_items,
        "); got ",
        length(partition),
        ".",
        call. = FALSE
      )
    }
    partition_list <- split(seq_len(n_items), as.character(partition))
    partition_list <- partition_list[lengths(partition_list) > 0L]
    names(partition_list) <- NULL
  } else {
    stop(
      "Invalid partition format. Provide a list of column vectors or a ",
      "vector of length ncol(data).",
      call. = FALSE
    )
  }

  used_items <- unlist(partition_list)
  if (length(used_items) != length(unique(used_items))) {
    stop("Partition has overlapping items.", call. = FALSE)
  }
  if (length(used_items) < n_items) {
    not_used <- setdiff(seq_len(n_items), used_items)
    warning(
      "Items not assigned to any subscale (dropped): ",
      paste(item_names[not_used], collapse = ", "),
      call. = FALSE
    )
  }

  partition_list
}

#' Extract item parameters for conditional sampling under H0
#'
#' For dichotomous data, returns a numeric vector of \eqn{\epsilon_{i,1} =
#' -\beta_i} (item easinesses). For polytomous data, returns a list with one
#' element per item: a numeric vector \eqn{(\epsilon_{i,1}, \ldots,
#' \epsilon_{i,m_i})} with \eqn{\epsilon_{i,x} = -\sum_{k=1}^x \tau_{i,k}}
#' (cumulative negated Andrich thresholds), suitable for direct use with
#' `psychotools::elementary_symmetric_functions()`.
#'
#' @keywords internal
#' @noRd
extract_ml_sampling_params <- function(data, is_polytomous) {
  if (is_polytomous) {
    # CML Andrich thresholds via psychotools (grand-mean centred). The
    # Martin-Löf conditional Monte Carlo samples patterns given the total
    # score, so it is invariant to this overall location. Parameters are
    # stored in the psychotools convention -- cumulative thresholds
    # (item-category difficulties) -- because
    # elementary_symmetric_functions() weights category k of item j by
    # exp(-par_jk); the sampler negates for its numerator weights.
    thr_list <- .fit_cml_thresholds(as.matrix(data))
    params_list <- lapply(thr_list, function(taus) cumsum(as.numeric(taus)))
    names(params_list) <- colnames(as.matrix(data))
    params_list
  } else {
    rasch_fit <- psychotools::raschmodel(data, hessian = FALSE)
    # `coef(raschmodel)` returns K-1 parameters under the sum-to-zero
    # constraint, but the sampler needs all K item parameters. Use
    # `itempar()` which returns the full K-vector with proper names.
    # Difficulties (psychotools convention); the sampler negates to get
    # easiness weights.
    as.numeric(psychotools::itempar(rasch_fit))
  }
}

#' Compute the Martin-Lof likelihood-ratio statistic
#'
#' @keywords internal
#' @noRd
compute_ml_statistic <- function(data, partition_list, is_polytomous) {
  N <- nrow(data)

  # Full-scale CML log-likelihood
  full_fit <- if (is_polytomous) {
    psychotools::pcmodel(data, hessian = FALSE)
  } else {
    psychotools::raschmodel(data, hessian = FALSE)
  }
  ll_full <- as.numeric(stats::logLik(full_fit))

  # Sum of subscale CML log-likelihoods
  ll_subs <- 0
  for (idx in partition_list) {
    sub_data <- data[, idx, drop = FALSE]
    sub_fit <- if (is_polytomous) {
      psychotools::pcmodel(sub_data, hessian = FALSE)
    } else {
      psychotools::raschmodel(sub_data, hessian = FALSE)
    }
    ll_subs <- ll_subs + as.numeric(stats::logLik(sub_fit))
  }

  # Score and joint subscore frequencies
  total_scores <- rowSums(data)
  n_t <- as.numeric(table(total_scores))

  subscores <- vapply(
    partition_list,
    function(idx) rowSums(data[, idx, drop = FALSE]),
    numeric(N)
  )
  if (is.matrix(subscores)) {
    subscore_keys <- apply(subscores, 1L, function(x) paste(x, collapse = ","))
  } else {
    subscore_keys <- as.character(subscores)
  }
  n_combo <- as.numeric(table(subscore_keys))

  # 0 log 0 = 0 (and table() never returns 0, so safe to use logs directly)
  log_term_combo <- sum(n_combo * log(n_combo / N))
  log_term_total <- sum(n_t * log(n_t / N))

  2 * (log_term_combo - log_term_total - ll_full + ll_subs)
}

#' Per-subscale WLE thetas and pairwise correlations
#'
#' For each subscale, fits a CML Rasch / PCM via `psychotools::raschmodel()`
#' / `psychotools::pcmodel()` and extracts Warm's Weighted Likelihood
#' Estimate of theta per person via the patched
#' iarm helper that handles boundary scores cleanly. Returns the WLE
#' matrix and pairwise Pearson correlations with 95% CIs. Robust to
#' subscale fit failures (returns NA WLEs for that subscale).
#'
#' @keywords internal
#' @noRd
compute_subscale_wle_and_correlations <- function(
  data,
  partition_list,
  is_polytomous
) {
  D <- length(partition_list)
  N <- nrow(data)

  wle_per_subscale <- lapply(seq_along(partition_list), function(d) {
    idx <- partition_list[[d]]
    sub_data <- data[, idx, drop = FALSE]
    tryCatch(
      {
        sub_fit <- if (is_polytomous) {
          psychotools::pcmodel(sub_data, hessian = FALSE)
        } else {
          psychotools::raschmodel(sub_data, hessian = FALSE)
        }
        pe <- iarm_person_estimates(sub_fit, allperson = TRUE)
        as.numeric(pe[, "WLE"])
      },
      error = function(e) {
        warning(
          sprintf(
            "WLE estimation failed for subscale %d: %s",
            d,
            conditionMessage(e)
          ),
          call. = FALSE
        )
        rep(NA_real_, N)
      }
    )
  })

  wle_scores <- as.data.frame(do.call(cbind, wle_per_subscale))
  names(wle_scores) <- paste0("subscale_", seq_len(D), "_wle")
  rownames(wle_scores) <- NULL

  # Pairwise correlations
  pairs_idx <- if (D == 2L) {
    matrix(c(1L, 2L), ncol = 2L)
  } else {
    t(utils::combn(D, 2L))
  }

  rows <- lapply(seq_len(nrow(pairs_idx)), function(i) {
    a <- pairs_idx[i, 1L]
    b <- pairs_idx[i, 2L]
    x <- wle_scores[[a]]
    y <- wle_scores[[b]]
    finite <- is.finite(x) & is.finite(y)
    if (sum(finite) < 3L) {
      return(data.frame(
        subscale_a = a,
        subscale_b = b,
        r = NA_real_,
        ci_lower = NA_real_,
        ci_upper = NA_real_,
        p_value = NA_real_,
        n = sum(finite),
        stringsAsFactors = FALSE,
        row.names = NULL
      ))
    }
    test <- tryCatch(
      stats::cor.test(x[finite], y[finite], method = "pearson"),
      error = function(e) NULL
    )
    if (is.null(test)) {
      data.frame(
        subscale_a = a,
        subscale_b = b,
        r = NA_real_,
        ci_lower = NA_real_,
        ci_upper = NA_real_,
        p_value = NA_real_,
        n = sum(finite),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    } else {
      data.frame(
        subscale_a = a,
        subscale_b = b,
        r = unname(test$estimate),
        ci_lower = test$conf.int[1L],
        ci_upper = test$conf.int[2L],
        p_value = test$p.value,
        n = sum(finite),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    }
  })
  wle_correlation <- do.call(rbind, rows)
  rownames(wle_correlation) <- NULL

  list(wle_scores = wle_scores, wle_correlation = wle_correlation)
}

#' Sample a dichotomous response vector with target total score
#'
#' Uses the fast algorithm of Christensen & Kreiner (2007, p. 23): pick `s`
#' items without replacement, weighted by easiness. `item_params` are
#' difficulties (psychotools convention), so the weights are
#' `exp(-item_params)`.
#'
#' @keywords internal
#' @noRd
sample_dichotomous_at_score <- function(s, item_params) {
  n_items <- length(item_params)
  if (s == 0L) {
    return(integer(n_items))
  }
  if (s == n_items) {
    return(rep(1L, n_items))
  }
  weights <- exp(-item_params)
  picked <- sample.int(n_items, size = s, prob = weights, replace = FALSE)
  out <- integer(n_items)
  out[picked] <- 1L
  out
}

#' Sample a polytomous response vector with target total score
#'
#' Iteratively samples one item's response at a time. For each item j in the
#' remaining pool, the conditional probability of \eqn{x_j} given the
#' remaining target score \eqn{t'} is proportional to
#' \eqn{\exp(\epsilon_{j,x_j}) \cdot \gamma^{(j)}_{t' - x_j}} where
#' \eqn{\gamma^{(j)}} is the elementary symmetric function over the
#' remaining items excluding item j (Andersen, 1995, eq. 15.22, 15.26).
#' \eqn{\gamma^{(j)}} is recomputed via
#' `psychotools::elementary_symmetric_functions()` for each step.
#'
#' @keywords internal
#' @noRd
sample_polytomous_at_score <- function(t, params_list) {
  n_items <- length(params_list)
  m_i <- vapply(params_list, length, integer(1L)) # max category per item
  M_total <- sum(m_i)

  if (t == 0L) {
    return(integer(n_items))
  }
  if (t == M_total) {
    return(m_i)
  }

  x <- integer(n_items)
  remaining_pos <- seq_len(n_items) # original positions still in pool
  remaining_params <- params_list
  remaining_max <- m_i
  remaining_score <- t

  while (length(remaining_pos) > 0L) {
    if (length(remaining_pos) == 1L) {
      x[remaining_pos] <- remaining_score
      break
    }
    j <- 1L
    item_pos <- remaining_pos[j]
    other_params <- remaining_params[-j]
    M_other <- sum(remaining_max[-j])

    # gamma^{(j)}: ESF over the items remaining minus item j
    gamma_other <- if (length(other_params) == 0L) {
      1
    } else {
      esf <- psychotools::elementary_symmetric_functions(
        other_params,
        order = 0L
      )
      if (is.list(esf)) esf[[1L]] else esf
    }

    # Conditional log-probabilities for x_j in 0:m_j. The stored
    # parameters follow the psychotools convention (cumulative
    # difficulties, category weight exp(-par)), so the log-weights
    # epsilon are their negation -- matching the exp(-par) weights that
    # elementary_symmetric_functions() uses for gamma_other above.
    eps_j <- c(0, -remaining_params[[j]]) # epsilon_{j,0}=0 prepended
    max_xj <- remaining_max[j]
    log_probs <- rep(-Inf, max_xj + 1L)
    for (xj in 0L:max_xj) {
      target_other <- remaining_score - xj
      if (target_other >= 0L && target_other <= M_other) {
        log_probs[xj + 1L] <- eps_j[xj + 1L] +
          log(gamma_other[target_other + 1L])
      }
    }
    log_probs <- log_probs - max(log_probs)
    probs <- exp(log_probs)
    if (sum(probs) <= 0 || !is.finite(sum(probs))) {
      # Numerical fallback: should be rare. Pick the most likely value.
      sampled_xj <- which.max(log_probs) - 1L
    } else {
      probs <- probs / sum(probs)
      sampled_xj <- sample(0L:max_xj, size = 1L, prob = probs)
    }
    x[item_pos] <- sampled_xj
    remaining_score <- remaining_score - sampled_xj
    remaining_pos <- remaining_pos[-j]
    remaining_params <- remaining_params[-j]
    remaining_max <- remaining_max[-j]
  }
  x
}

#' Run a single Monte Carlo iteration of the Martin-Lof test
#'
#' @keywords internal
#' @noRd
run_single_ml_iteration <- function(seed, sim_data) {
  set.seed(seed)

  N <- sim_data$N
  scores <- sample(
    sim_data$score_values,
    size = N,
    replace = TRUE,
    prob = sim_data$score_probs
  )

  sim_responses <- matrix(0L, nrow = N, ncol = length(sim_data$item_names))
  if (sim_data$is_polytomous) {
    for (i in seq_len(N)) {
      sim_responses[i, ] <- sample_polytomous_at_score(
        scores[i],
        sim_data$sampling_params
      )
    }
  } else {
    for (i in seq_len(N)) {
      sim_responses[i, ] <- sample_dichotomous_at_score(
        scores[i],
        sim_data$sampling_params
      )
    }
  }

  sim_df <- as.data.frame(sim_responses)
  colnames(sim_df) <- sim_data$item_names

  tryCatch(
    {
      compute_ml_statistic(
        sim_df,
        sim_data$partition_list,
        sim_data$is_polytomous
      )
    },
    error = function(e) NA_real_
  )
}

#' Parallel runner for ML iterations (mirai)
#'
#' @keywords internal
#' @noRd
run_ml_sim_parallel <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  n_cores,
  verbose = FALSE
) {
  mirai::daemons(n_cores)
  on.exit(mirai::daemons(0), add = TRUE)

  if (verbose) {
    message(sprintf("Starting %d daemons...", n_cores))
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  tasks <- lapply(seq_len(iterations), function(i) {
    mirai::mirai(
      {
        run_single_ml_iteration(seed, data_list)
      },
      seed = sim_seeds[i],
      data_list = sim_data_list,
      run_single_ml_iteration = run_single_ml_iteration,
      compute_ml_statistic = compute_ml_statistic,
      sample_dichotomous_at_score = sample_dichotomous_at_score,
      sample_polytomous_at_score = sample_polytomous_at_score
    )
  })

  results <- vector("list", iterations)
  for (i in seq_len(iterations)) {
    res <- mirai::call_mirai(tasks[[i]])$data
    results[[i]] <- if (inherits(res, "errorValue")) NA_real_ else res
    if (verbose) utils::setTxtProgressBar(pb, i)
  }

  if (verbose) {
    close(pb)
    message("")
  }
  results
}

#' Sequential runner for ML iterations
#'
#' @keywords internal
#' @noRd
run_ml_sim_sequential <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  verbose = FALSE
) {
  if (verbose) {
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }
  results <- vector("list", iterations)
  for (i in seq_len(iterations)) {
    results[[i]] <- run_single_ml_iteration(sim_seeds[i], sim_data_list)
    if (verbose) utils::setTxtProgressBar(pb, i)
  }
  if (verbose) {
    close(pb)
    message("")
  }
  results
}

# ===========================================================================
# RMdimMartinLofResiduals: standardised residuals from the joint subscore table
# ===========================================================================

#' Standardised Residuals from the Joint Subscore Distribution
#'
#' Diagnostic accompanying \code{\link{RMdimMartinLof}}: per-cell standardised
#' residuals from the joint distribution of subscores under unidimensionality
#' (Christensen, Bjorner, Kreiner, & Petersen, 2002, eq. 13). Useful for
#' identifying *where* a partition deviates from the unidimensional null
#' rather than just whether it does (which `RMdimMartinLof()` answers).
#'
#' For each cell of the joint subscore table (indexed by
#' \eqn{(t_1, \ldots, t_D)}), the conditional probability under H0 given the
#' total score \eqn{t = \sum_d t_d} is
#' \deqn{p(t_1, \ldots, t_D \mid t) = \prod_d \gamma^{(d)}_{t_d} / \gamma_t,}
#' the expected count is \eqn{e = n_t \cdot p}, and the residual is
#' \eqn{(o - e) / \sqrt{n_t \cdot p \cdot (1 - p)}}. CML estimates from the
#' unidimensional model are used for the \eqn{\gamma}-functions.
#'
#' Reading the table (D = 2): under the unidimensional null, residuals
#' should be patternless and roughly N(0, 1). Multidimensionality with
#' positively correlated dimensions typically shows up as **positive**
#' residuals at the corners of each antidiagonal (high `t1` + low `t2`,
#' low `t1` + high `t2`) and **negative** residuals near the antidiagonal
#' centre (matched subscores). Negatively correlated dimensions show
#' positive residuals at the table corners (high/low and low/high) and
#' negative residuals at high/high and low/low. See Christensen et al.
#' (2002, section7) for a worked example.
#'
#' Cells where the total score has no observed cases (`n_t = 0`) are
#' uninformative and are dropped from the output.
#'
#' @param data A data.frame or matrix of item responses (0-based,
#'   non-negative integers). Rows with any `NA` are dropped.
#' @param partition Same format as in \code{\link{RMdimMartinLof}}: a list of
#'   item-name/index vectors, or a length-`ncol(data)` vector of group
#'   labels. Each subscale must contain at least 2 items.
#' @param output Character. `"kable"` (default) for a 2-D pipe-format
#'   residual table when D = 2 (long-format kable when D > 2), `"dataframe"`
#'   for the underlying long-format data.frame, or `"ggplot"` for a
#'   diverging-fill heatmap (with `facet_wrap` over `t3` for D = 3, error
#'   for D > 3).
#' @param flag_threshold Numeric. Cells with `|residual| > flag_threshold`
#'   are flagged: marked as `**bold**` in the kable, shown in the `flagged`
#'   column of the dataframe. Default `2`.
#' @param color_by Character. For `output = "ggplot"`, what the tile fill
#'   colour encodes. `"residual"` (default) -- diverging red-white-blue scale
#'   centred at 0, the most directly diagnostic. `"n"` -- sequential blue
#'   scale on the observed cell count, useful for spotting whether
#'   large-magnitude residuals are driven by sparse cells. Either way, the
#'   numeric residual is printed inside each cell.
#' @param color_limits Numeric length-2 vector or `NULL`. Caps the colour
#'   scale for `output = "ggplot"`. Default `c(-5, 5)` when
#'   `color_by = "residual"` (sparse-cell residuals from
#'   `(o-e)/sqrt(n*p*(1-p))` can be enormous when expected counts are tiny;
#'   capping the scale stops outliers from compressing the rest of the
#'   plot). The unclipped residual values still appear as cell labels.
#'   `NULL` for `color_by = "n"`, which uses the natural data range.
#' @param min_expected Numeric or `NULL`. If set, cells with `expected <
#'   min_expected` have their residual set to `NA` (they appear grey in the
#'   heatmap and are not flagged). Default `NULL` (no filtering). Setting
#'   `min_expected = 1` (or `5`) is the analogue of the Cochran rule for
#'   sparse-cell chi-square contributions and removes residuals whose
#'   asymptotic standard normal approximation is unreliable.
#'
#' @return
#' * `output = "kable"`: a `knitr_kable` object. For D = 2, a wide table
#'   with rows = `t1`, columns = `t2`, cells = standardised residual
#'   (`**bold**` if flagged, em-dash if NA). For D > 2, long-format with
#'   one row per cell.
#' * `output = "dataframe"`: a long-format data.frame with columns `t1`,
#'   ..., `tD`, `total`, `observed`, `expected`, `residual`, `flagged`.
#' * `output = "ggplot"`: a `geom_tile()` heatmap (D = 2 or 3 only).
#'
#' @references
#' Christensen, K. B., Bjorner, J. B., Kreiner, S., & Petersen, J. H. (2002).
#' Testing unidimensionality in polytomous Rasch models. *Psychometrika,
#' 67*(4), 563-574. \doi{10.1007/BF02295132}
#'
#' @seealso \code{\link{RMdimMartinLof}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' dat <- as.data.frame(matrix(sample(0:1, 400 * 8, replace = TRUE),
#'                             nrow = 400, ncol = 8))
#' colnames(dat) <- paste0("I", 1:8)
#'
#' # Wide kable table for D = 2
#' RMdimMartinLofResiduals(dat,
#'                      partition = list(c("I1","I2","I3","I4"),
#'                                       c("I5","I6","I7","I8")))
#'
#' # Heatmap
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   RMdimMartinLofResiduals(dat,
#'                        partition = c(1,1,1,1,2,2,2,2),
#'                        output = "ggplot")
#' }
#'
#' # Underlying data.frame for custom analysis
#' df <- RMdimMartinLofResiduals(dat,
#'                            partition = c(1,1,1,1,2,2,2,2),
#'                            output = "dataframe")
#' df[df$flagged, ]
#' }
RMdimMartinLofResiduals <- function(
  data,
  partition,
  output = c("kable", "dataframe", "ggplot"),
  flag_threshold = 2,
  color_by = c("residual", "n"),
  color_limits = NULL,
  min_expected = NULL
) {
  output <- match.arg(output)
  color_by <- match.arg(color_by)

  if (
    !is.null(min_expected) &&
      (!is.numeric(min_expected) ||
        length(min_expected) != 1L ||
        !is.finite(min_expected) ||
        min_expected < 0)
  ) {
    stop(
      "`min_expected` must be NULL or a non-negative numeric scalar.",
      call. = FALSE
    )
  }
  if (
    !is.null(color_limits) &&
      (!is.numeric(color_limits) ||
        length(color_limits) != 2L ||
        any(!is.finite(color_limits)) ||
        color_limits[1L] >= color_limits[2L])
  ) {
    stop(
      "`color_limits` must be NULL or a length-2 numeric vector with ",
      "lower < upper.",
      call. = FALSE
    )
  }

  validate_response_data(data)

  data <- as.data.frame(data)
  n_total_ml <- nrow(data)
  has_na_ml <- anyNA(data)
  data <- stats::na.omit(data)
  if (nrow(data) < 30L) {
    stop("Need at least 30 complete cases.", call. = FALSE)
  }

  partition_list <- normalize_ml_partition(partition, data)
  D <- length(partition_list)
  if (D < 2L) {
    stop("Partition must specify >= 2 subscales.", call. = FALSE)
  }
  if (any(lengths(partition_list) < 2L)) {
    stop("Each subscale must contain at least 2 items.", call. = FALSE)
  }
  if (output == "ggplot" && D > 3L) {
    stop(
      "output = \"ggplot\" supports D = 2 or 3 subscales only. ",
      "Use output = \"dataframe\" for higher D.",
      call. = FALSE
    )
  }

  # Restrict to items used in the partition; reindex
  used_idx <- sort(unique(unlist(partition_list)))
  if (length(used_idx) < ncol(data)) {
    data <- data[, used_idx, drop = FALSE]
    remap <- stats::setNames(seq_along(used_idx), used_idx)
    partition_list <- lapply(partition_list, function(idx) {
      unname(remap[as.character(idx)])
    })
  }

  data_mat <- as.matrix(data)
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L
  N <- nrow(data)

  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # CML item parameters (vector for RM, list for PCM)
  full_params <- extract_ml_sampling_params(data, is_polytomous)

  # gamma_full over all items in the partition
  gamma_full <- compute_esf_gamma(full_params)

  # gamma per subscale
  gamma_per_sub <- lapply(partition_list, function(idx) {
    sub_params <- if (is_polytomous) full_params[idx] else full_params[idx]
    compute_esf_gamma(sub_params)
  })
  M_d <- vapply(gamma_per_sub, function(g) length(g) - 1L, integer(1L))

  # Observed subscores
  subscores <- vapply(
    partition_list,
    function(idx) rowSums(data[, idx, drop = FALSE]),
    numeric(N)
  )
  if (!is.matrix(subscores)) {
    subscores <- matrix(subscores, nrow = N, ncol = D)
  }

  total_scores <- rowSums(subscores)
  n_t_table <- table(total_scores)
  observable_t <- as.integer(names(n_t_table))

  # All possible cells (cartesian product of 0:M_d)
  cell_indices <- expand.grid(
    lapply(M_d, function(M) 0:M),
    KEEP.OUT.ATTRS = FALSE
  )
  colnames(cell_indices) <- paste0("t", seq_len(D))
  cell_totals <- rowSums(cell_indices)

  # Drop cells with totals not observed (n_t = 0)
  keep_cell <- cell_totals %in% observable_t
  cell_indices <- cell_indices[keep_cell, , drop = FALSE]
  cell_totals <- cell_totals[keep_cell]

  # Observed counts via key-based lookup
  obs_keys <- apply(subscores, 1L, function(x) paste(x, collapse = "."))
  cell_keys <- apply(cell_indices, 1L, function(x) paste(x, collapse = "."))
  obs_table <- table(obs_keys)
  observed <- as.numeric(obs_table[cell_keys])
  observed[is.na(observed)] <- 0

  # n_t per cell
  n_t_per_cell <- as.numeric(n_t_table[as.character(cell_totals)])

  # Conditional probability p(t1,...,tD | t) under H0
  log_p <- numeric(nrow(cell_indices))
  for (d in seq_len(D)) {
    log_p <- log_p + log(gamma_per_sub[[d]][cell_indices[[d]] + 1L])
  }
  log_p <- log_p - log(gamma_full[cell_totals + 1L])
  p <- exp(log_p)
  p[!is.finite(p)] <- 0

  # Expected and standardised residuals
  expected <- n_t_per_cell * p
  se_denom <- sqrt(n_t_per_cell * p * (1 - p))
  residual <- (observed - expected) / se_denom
  residual[!is.finite(residual)] <- NA_real_

  # Optional Cochran-style filter on sparse cells (asymptotic standard
  # normal approximation breaks down when expected is tiny).
  if (!is.null(min_expected)) {
    residual[expected < min_expected] <- NA_real_
  }

  # Result data.frame
  result_df <- data.frame(
    cell_indices,
    total = as.integer(cell_totals),
    observed = as.integer(observed),
    expected = expected,
    residual = residual,
    flagged = !is.na(residual) & abs(residual) > flag_threshold,
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  # Sort by total then by t1, t2, ...
  ord_cols <- c("total", paste0("t", seq_len(D)))
  result_df <- result_df[do.call(order, result_df[ord_cols]), , drop = FALSE]
  rownames(result_df) <- NULL

  if (output == "dataframe") {
    return(result_df)
  }

  if (output == "kable") {
    if (D == 2L) {
      t1_vals <- 0:M_d[1L]
      t2_vals <- 0:M_d[2L]
      mat <- matrix(
        "",
        nrow = length(t1_vals),
        ncol = length(t2_vals),
        dimnames = list(as.character(t1_vals), as.character(t2_vals))
      )
      for (k in seq_len(nrow(result_df))) {
        r1 <- result_df$t1[k] + 1L
        r2 <- result_df$t2[k] + 1L
        v <- result_df$residual[k]
        if (is.na(v)) {
          mat[r1, r2] <- "--"
        } else {
          formatted <- sprintf("%.2f", v)
          if (isTRUE(result_df$flagged[k])) {
            formatted <- paste0("**", formatted, "**")
          }
          mat[r1, r2] <- formatted
        }
      }
      kable_df <- as.data.frame(mat, stringsAsFactors = FALSE)
      kable_df <- cbind(`t1\\t2` = rownames(mat), kable_df)
      caption <- paste0(
        "Standardised residuals (Christensen et al. 2002, eq. 13). ",
        "Rows = subscale 1 score, columns = subscale 2 score. ",
        "**Bold** = |residual| > ",
        flag_threshold,
        ". -- = uncomputable. ",
        .n_caption(
          N,
          n_total_ml,
          if (has_na_ml) "complete cases" else character()
        ),
        "."
      )
      return(knitr::kable(
        kable_df,
        format = "pipe",
        row.names = FALSE,
        caption = caption
      ))
    } else {
      caption <- paste0(
        "Standardised residuals (Christensen et al. 2002, eq. 13) for D = ",
        D,
        " subscales. ",
        .n_caption(
          N,
          n_total_ml,
          if (has_na_ml) "complete cases" else character()
        ),
        ". |residual| > ",
        flag_threshold,
        " indicates potential ",
        "dimensionality issue."
      )
      cols <- c(
        paste0("t", seq_len(D)),
        "total",
        "observed",
        "expected",
        "residual",
        "flagged"
      )
      # Display rounding (the dataframe output above stays unrounded)
      result_df <- .round_display(result_df, c(expected = 3, residual = 3))
      return(knitr::kable(
        result_df[, cols],
        format = "pipe",
        caption = caption
      ))
    }
  }

  # ggplot
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for output = \"ggplot\".",
      call. = FALSE
    )
  }

  plot_df <- result_df
  plot_df$t1 <- as.integer(plot_df$t1)
  plot_df$t2 <- as.integer(plot_df$t2)

  # Stash the chosen fill values in a dedicated column. Avoids any
  # `.data$col` / quosure / aes-evaluation surprises that can swap branches
  # in some ggplot2 setups -- the column we map onto fill is unambiguous.
  plot_df[["fill_value"]] <- if (identical(color_by, "residual")) {
    plot_df[["residual"]]
  } else {
    as.numeric(plot_df[["observed"]])
  }

  # Resolve colour-scale limits and clip fill_value BEFORE building the
  # plot (mutating plot_df after ggplot() captures it has no effect, since
  # ggplot has its own copy of the data).
  if (identical(color_by, "residual")) {
    fill_limits <- if (is.null(color_limits)) c(-5, 5) else color_limits
    plot_df$fill_value <- pmin(
      pmax(plot_df$fill_value, fill_limits[1L]),
      fill_limits[2L]
    )
  } else {
    fill_limits <- color_limits # NULL -> use natural range below
    if (!is.null(fill_limits)) {
      plot_df$fill_value <- pmin(
        pmax(plot_df$fill_value, fill_limits[1L]),
        fill_limits[2L]
      )
    }
  }

  base <- ggplot2::ggplot(plot_df) +
    ggplot2::geom_tile(
      mapping = ggplot2::aes(
        x = .data[["t2"]],
        y = .data[["t1"]],
        fill = .data[["fill_value"]]
      ),
      colour = "grey90"
    )

  if (identical(color_by, "residual")) {
    base <- base +
      ggplot2::scale_fill_gradient2(
        low = "#2166AC",
        mid = "white",
        high = "#B2182B",
        midpoint = 0,
        na.value = "grey95",
        limits = fill_limits,
        name = "Std.\nresidual"
      )
  } else {
    base <- base +
      ggplot2::scale_fill_gradient(
        low = "#F7FBFF",
        high = "#08306B",
        na.value = "grey95",
        limits = fill_limits,
        name = "Observed\ncount"
      )
  }

  # Always print residual values inside cells, regardless of fill choice.
  # `inherit.aes = FALSE` so the text layer's mapping is independent of
  # the tile's fill aesthetic.
  base <- base +
    ggplot2::geom_text(
      data = plot_df[!is.na(plot_df$residual), ],
      mapping = ggplot2::aes(
        x = .data[["t2"]],
        y = .data[["t1"]],
        label = sprintf("%.1f", .data[["residual"]])
      ),
      size = 3,
      colour = "grey20",
      inherit.aes = FALSE
    ) +
    ggplot2::scale_x_continuous(
      breaks = seq(0L, max(plot_df$t2), by = 1L),
      expand = c(0, 0)
    ) +
    ggplot2::scale_y_continuous(
      breaks = seq(0L, max(plot_df$t1), by = 1L),
      expand = c(0, 0)
    ) +
    ggplot2::coord_equal() +
    ggplot2::theme_minimal(base_size = 12) +
    ggplot2::theme(
      panel.grid = ggplot2::element_blank()
    ) +
    er2_axis_margins() +
    er2_plot_caption()

  if (D == 2L) {
    return(
      base +
        ggplot2::labs(
          x = expression("Subscale 2 score (" * t[2] * ")"),
          y = expression("Subscale 1 score (" * t[1] * ")"),
          caption = er2_caption(paste0(
            "Standardised residuals (Christensen et al. 2002, eq. 13). ",
            .n_caption(
              N,
              n_total_ml,
              if (has_na_ml) "complete cases" else character()
            ),
            ". Cells with |residual| > ",
            flag_threshold,
            " indicate potential dimensionality issues."
          ))
        )
    )
  }

  # D == 3: facet by t3
  base +
    ggplot2::facet_wrap(~ .data$t3, labeller = ggplot2::label_both) +
    ggplot2::labs(
      x = expression("Subscale 2 score (" * t[2] * ")"),
      y = expression("Subscale 1 score (" * t[1] * ")"),
      caption = er2_caption(paste0(
        "Standardised residuals (Christensen et al. 2002, eq. 13). ",
        .n_caption(
          N,
          n_total_ml,
          if (has_na_ml) "complete cases" else character()
        ),
        ". Faceted by subscale 3 score (",
        expression(t[3]),
        "). |residual| > ",
        flag_threshold,
        " flagged."
      ))
    )
}

# ---------------------------------------------------------------------------
# Internal: gamma-function computation that handles both vector (RM) and list
# (PCM) parameter formats and returns a numeric vector gamma_0, ..., gamma_M.
# ---------------------------------------------------------------------------

#' @keywords internal
#' @noRd
compute_esf_gamma <- function(params) {
  if (is.list(params) && length(params) == 0L) {
    return(1)
  }
  if (is.numeric(params) && length(params) == 0L) {
    return(1)
  }
  esf <- psychotools::elementary_symmetric_functions(params, order = 0L)
  if (is.list(esf)) esf[[1L]] else esf
}

# Tests for RMdimMartinLof() and RMdimMartinLofResiduals()

make_polytomous <- function(n = 200, k = 8, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:3, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

make_dichotomous <- function(n = 300, k = 8, seed = 2L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMdimMartinLof errors when partition has fewer than 2 subscales", {
  df <- make_polytomous()
  expect_error(
    RMdimMartinLof(df, partition = rep(1L, ncol(df)),
                iterations = 5L, parallel = FALSE, seed = 1L),
    regexp = ">= 2 subscales"
  )
})

test_that("RMdimMartinLof errors when a subscale has fewer than 2 items", {
  df <- make_polytomous()
  partition <- c(1L, rep(2L, ncol(df) - 1L))
  expect_error(
    RMdimMartinLof(df, partition = partition,
                iterations = 5L, parallel = FALSE, seed = 1L),
    regexp = "at least 2 items"
  )
})

# ---------------------------------------------------------------------
# Output structure (polytomous)
# ---------------------------------------------------------------------
test_that("RMdimMartinLof returns expected fields on polytomous data", {
  skip_on_cran()
  skip_if_not_installed("psychotools")
  df <- make_polytomous()
  res <- RMdimMartinLof(
    df, partition = list(c("I1","I2","I3","I4"), c("I5","I6","I7","I8")),
    iterations = 20L, parallel = FALSE, seed = 1L,
    stopping = "sequential", h = 10L
  )
  expect_type(res, "list")
  expect_true(all(c("T_obs", "p_value", "actual_iterations", "rejected",
                    "partition", "n_subscales", "is_polytomous",
                    "sample_n", "n_items", "stopping", "h",
                    "T_rep", "wle_scores", "wle_correlation") %in% names(res)))
  expect_true(res$is_polytomous)
  expect_equal(res$n_subscales, 2L)
  expect_true(is.finite(res$T_obs) && res$T_obs >= 0)
  expect_true(res$p_value >= 0 && res$p_value <= 1)
})

# ---------------------------------------------------------------------
# Output structure (dichotomous)
# ---------------------------------------------------------------------
test_that("RMdimMartinLof works on dichotomous data (Rasch model path)", {
  skip_on_cran()
  skip_if_not_installed("psychotools")
  df <- make_dichotomous()
  res <- RMdimMartinLof(
    df, partition = c(1,1,1,1,2,2,2,2),
    iterations = 20L, parallel = FALSE, seed = 1L,
    stopping = "sequential", h = 10L
  )
  expect_type(res, "list")
  expect_false(res$is_polytomous)
  expect_true(is.finite(res$T_obs))
})

test_that("RMdimMartinLof p_value is reproducible with the same seed", {
  skip_on_cran()
  skip_if_not_installed("psychotools")
  df <- make_polytomous(n = 150)
  r1 <- RMdimMartinLof(df, partition = c(1,1,1,1,2,2,2,2),
                    iterations = 10L, parallel = FALSE, seed = 42L,
                    stopping = "sequential", h = 5L)
  r2 <- RMdimMartinLof(df, partition = c(1,1,1,1,2,2,2,2),
                    iterations = 10L, parallel = FALSE, seed = 42L,
                    stopping = "sequential", h = 5L)
  expect_equal(r1$T_obs,  r2$T_obs)
  expect_equal(r1$T_rep,  r2$T_rep)
})

# ---------------------------------------------------------------------
# RMdimMartinLofResiduals
# ---------------------------------------------------------------------
test_that("RMdimMartinLofResiduals output = 'dataframe' returns expected columns", {
  skip_if_not_installed("psychotools")
  df <- make_polytomous()
  res <- RMdimMartinLofResiduals(
    df, partition = list(c("I1","I2","I3","I4"), c("I5","I6","I7","I8")),
    output = "dataframe"
  )
  expect_s3_class(res, "data.frame")
  expect_true(all(c("t1", "t2", "total", "observed", "expected",
                    "residual", "flagged") %in% names(res)))
  expect_type(res$flagged, "logical")
})

test_that("RMdimMartinLofResiduals output = 'kable' returns a kable-like object", {
  skip_if_not_installed("psychotools")
  skip_if_not_installed("knitr")
  df  <- make_polytomous()
  out <- RMdimMartinLofResiduals(
    df, partition = c(1,1,1,1,2,2,2,2),
    output = "kable"
  )
  expect_true(inherits(out, "knitr_kable") || inherits(out, "knit_asis"))
})

test_that("RMdimMartinLofResiduals output = 'ggplot' returns a ggplot", {
  skip_if_not_installed("psychotools")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p  <- RMdimMartinLofResiduals(
    df, partition = c(1,1,1,1,2,2,2,2),
    output = "ggplot"
  )
  expect_s3_class(p, "ggplot")
})

# ---------------------------------------------------------------------
# stopping = "none" Monte Carlo runner (the default; existing tests above
# all use stopping = "sequential")
# ---------------------------------------------------------------------
test_that("RMdimMartinLof stopping = 'none' returns a valid p-value", {
  skip_on_cran()
  skip_if_not_installed("psychotools")
  df <- make_dichotomous()
  res <- RMdimMartinLof(df, partition = c(1,1,1,1,2,2,2,2),
                        iterations = 20L, stopping = "none",
                        parallel = FALSE, seed = 1L)
  expect_equal(res$stopping, "none")
  expect_equal(res$actual_iterations, 20L)
  expect_true(res$p_value > 0 && res$p_value <= 1)
})

test_that("RMdimMartinLof verbose = TRUE runs the sequential progress path", {
  skip_on_cran()
  skip_if_not_installed("psychotools")
  df <- make_dichotomous()
  suppressMessages(invisible(utils::capture.output(
    res <- RMdimMartinLof(df, partition = c(1,1,1,1,2,2,2,2),
                          iterations = 10L, stopping = "none",
                          parallel = FALSE, seed = 1L, verbose = TRUE)
  )))
  expect_true(res$p_value <= 1)
})

test_that("RMdimMartinLof parallel path returns a valid result", {
  skip_on_cran()
  skip_if_not_installed("psychotools")
  skip_if_not_installed("mirai")
  df <- make_dichotomous()
  res <- suppressMessages(
    RMdimMartinLof(df, partition = c(1,1,1,1,2,2,2,2),
                   iterations = 8L, stopping = "none",
                   parallel = TRUE, n_cores = 2L, seed = 1L)
  )
  expect_true(res$p_value > 0 && res$p_value <= 1)
})

test_that("RMdimMartinLof parallel without n_cores/mc.cores warns and falls back", {
  skip_on_cran()
  skip_if_not_installed("psychotools")
  skip_if_not_installed("mirai")
  withr::local_options(mc.cores = NULL)
  df <- make_dichotomous()
  expect_warning(
    res <- RMdimMartinLof(df, partition = c(1,1,1,1,2,2,2,2),
                          iterations = 6L, stopping = "none",
                          parallel = TRUE, seed = 1L),
    regexp = "Falling back to sequential"
  )
  expect_true(res$p_value <= 1)
})

test_that("RMdimMartinLof errors with fewer than 30 complete cases", {
  skip_if_not_installed("psychotools")
  df <- make_dichotomous()[1:20, ]
  expect_error(
    RMdimMartinLof(df, partition = c(1,1,1,1,2,2,2,2),
                   iterations = 5L, parallel = FALSE),
    regexp = "at least 30 complete cases"
  )
})

# ---------------------------------------------------------------------
# Partition normalisation branches
# ---------------------------------------------------------------------
test_that("RMdimMartinLof warns and drops items not assigned to a subscale", {
  skip_on_cran()
  skip_if_not_installed("psychotools")
  df <- make_dichotomous()  # 8 items
  expect_warning(
    res <- RMdimMartinLof(df, partition = list(c(1, 2, 3), c(4, 5, 6)),
                          iterations = 10L, parallel = FALSE, seed = 1L),
    regexp = "not assigned"
  )
  expect_equal(res$n_items, 6L)
})

test_that("RMdimMartinLof errors on overlapping partition", {
  skip_if_not_installed("psychotools")
  df <- make_dichotomous()
  expect_error(
    RMdimMartinLof(df, partition = list(c(1, 2, 3), c(3, 4, 5)),
                   iterations = 5L, parallel = FALSE),
    regexp = "overlapping"
  )
})

test_that("RMdimMartinLof errors on an invalid partition format", {
  skip_if_not_installed("psychotools")
  df <- make_dichotomous()
  expect_error(
    RMdimMartinLof(df, partition = TRUE, iterations = 5L, parallel = FALSE),
    regexp = "Invalid partition format"
  )
})

# ---------------------------------------------------------------------
# RMdimMartinLofResiduals: option branches
# ---------------------------------------------------------------------
test_that("RMdimMartinLofResiduals min_expected blanks sparse cells", {
  skip_if_not_installed("psychotools")
  df  <- make_polytomous()
  res <- RMdimMartinLofResiduals(df, partition = c(1,1,1,1,2,2,2,2),
                                 output = "dataframe", min_expected = 1)
  expect_true(all(is.na(res$residual[res$expected < 1])))
})

test_that("RMdimMartinLofResiduals ggplot color_by = 'n' returns a ggplot", {
  skip_if_not_installed("psychotools")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p  <- RMdimMartinLofResiduals(df, partition = c(1,1,1,1,2,2,2,2),
                                output = "ggplot", color_by = "n",
                                color_limits = c(0, 50))
  expect_s3_class(p, "ggplot")
})

test_that("RMdimMartinLofResiduals supports D = 3 for kable and ggplot", {
  skip_if_not_installed("psychotools")
  skip_if_not_installed("knitr")
  df <- make_polytomous(k = 6)
  k  <- RMdimMartinLofResiduals(df, partition = c(1,1,2,2,3,3), output = "kable")
  expect_true(inherits(k, "knitr_kable") || inherits(k, "knit_asis"))
  skip_if_not_installed("ggplot2")
  p <- RMdimMartinLofResiduals(df, partition = c(1,1,2,2,3,3), output = "ggplot")
  expect_s3_class(p, "ggplot")
})

test_that("RMdimMartinLofResiduals rejects bad min_expected / color_limits / D>3 ggplot", {
  skip_if_not_installed("psychotools")
  df <- make_polytomous()
  expect_error(
    RMdimMartinLofResiduals(df, partition = c(1,1,1,1,2,2,2,2),
                            min_expected = -1),
    regexp = "min_expected"
  )
  expect_error(
    RMdimMartinLofResiduals(df, partition = c(1,1,1,1,2,2,2,2),
                            color_limits = c(5, 1)),
    regexp = "color_limits"
  )
  expect_error(
    RMdimMartinLofResiduals(df, partition = c(1,1,2,2,3,3,4,4),
                            output = "ggplot"),
    regexp = "D = 2 or 3"
  )
})

test_that("p_value_floor reports the smallest attainable MC p-value", {
  set.seed(11)
  dat <- as.data.frame(replicate(6, sample(0:2, 120, replace = TRUE)))
  names(dat) <- paste0("q", 1:6)
  res <- RMdimMartinLof(dat, partition = c(1, 1, 1, 2, 2, 2),
                        iterations = 100, parallel = FALSE, seed = 3)
  expect_equal(res$p_value_floor, 1 / (res$actual_iterations + 1))
  expect_gte(res$p_value, res$p_value_floor)
})

test_that("polytomous sampler draws the exact conditional pattern distribution", {
  # Regression for the 1.0.0 sign-convention bug: the numerator weights and
  # the psychotools gamma functions must use the same convention
  # (psychotools weights category k by exp(-par_k)).
  taus <- list(c(-0.8, 0.4), c(0.2, 0.9), c(-0.3, -0.1))
  params <- lapply(taus, cumsum)
  psi <- lapply(params, function(p) exp(-c(0, p)))
  pats <- expand.grid(x1 = 0:2, x2 = 0:2, x3 = 0:2)
  pats$w <- psi[[1]][pats$x1 + 1] * psi[[2]][pats$x2 + 1] *
    psi[[3]][pats$x3 + 1]
  pats$t <- pats$x1 + pats$x2 + pats$x3
  sub <- pats[pats$t == 3L, ]
  exact <- sub$w / sum(sub$w)

  set.seed(1)
  draws <- t(replicate(20000, sample_polytomous_at_score(3L, params)))
  key_d <- apply(draws, 1, paste, collapse = "")
  key_e <- paste0(sub$x1, sub$x2, sub$x3)
  freq <- as.numeric(table(factor(key_d, levels = key_e)) / nrow(draws))
  expect_lt(max(abs(freq - exact)), 0.015)

  # and invariant (in distribution) to item order
  set.seed(1)
  draws_p <- t(replicate(20000, sample_polytomous_at_score(
    3L, params[c(3, 1, 2)])))
  key_p <- apply(draws_p[, c(2, 3, 1)], 1, paste, collapse = "")
  freq_p <- as.numeric(table(factor(key_p, levels = key_e)) / nrow(draws_p))
  expect_lt(max(abs(freq_p - exact)), 0.015)
})

test_that("same seed gives the same p-value regardless of column arrangement", {
  set.seed(9)
  n <- 150
  th <- rnorm(n)
  dat <- as.data.frame(sapply(1:6, function(j)
    findInterval(plogis(th + rnorm(n, 0, 0.8)), c(0.3, 0.6, 0.85))))
  names(dat) <- paste0("w", 1:6)
  s1 <- paste0("w", 1:3); s2 <- paste0("w", 4:6)

  r1 <- RMdimMartinLof(dat, partition = list(s1, s2), iterations = 60,
                       parallel = FALSE, seed = 42)
  r2 <- RMdimMartinLof(dat[, c(s2, s1)], partition = list(s2, s1),
                       iterations = 60, parallel = FALSE, seed = 42)
  expect_identical(r1$p_value, r2$p_value)
  expect_equal(r1$T_obs, r2$T_obs)
  expect_equal(r1$T_rep, r2$T_rep, tolerance = 1e-9)
})

test_that("residual expected counts match brute-force enumeration", {
  set.seed(3)
  n <- 200
  th <- rnorm(n)
  dat <- as.data.frame(sapply(1:4, function(j)
    findInterval(plogis(th + rnorm(n, 0, 0.8)), c(0.3, 0.6, 0.85))))
  names(dat) <- paste0("v", 1:4)
  part <- list(c("v1", "v2"), c("v3", "v4"))
  res_df <- suppressWarnings(RMdimMartinLofResiduals(
    dat, partition = part, output = "dataframe"))

  fp <- extract_ml_sampling_params(dat, TRUE)
  g1 <- compute_esf_gamma(fp[1:2])
  g2 <- compute_esf_gamma(fp[3:4])
  gT <- compute_esf_gamma(fp)
  tot <- rowSums(dat)
  row_pick <- res_df[which.max(res_df$observed), ]
  E_bf <- sum(tot == row_pick$t1 + row_pick$t2) *
    g1[row_pick$t1 + 1] * g2[row_pick$t2 + 1] /
    gT[row_pick$t1 + row_pick$t2 + 1]
  expect_equal(row_pick$expected, as.numeric(E_bf), tolerance = 1e-6)
})

# Internal helpers for item- and person-parameter estimation.
#
# These low-level routines are shared by RMitemParameters() and
# RMpersonParameters(). They operate on a normalised item-parameter
# representation: a list `thr_list` with one element per item, each a
# numeric vector of Andrich thresholds (the category-boundary locations
# on the logit difficulty scale). A dichotomous item is simply an item
# with a single threshold equal to its difficulty, so one set of
# polytomous routines handles both the Rasch and Partial Credit cases.

# ---------------------------------------------------------------------
# Category-response probabilities (Partial Credit / Rasch)
# ---------------------------------------------------------------------

#' Category probabilities for one item at a given theta
#'
#' @param theta Numeric person location.
#' @param thr Numeric vector of Andrich thresholds (length = n_cat - 1).
#' @return Numeric vector of length `length(thr) + 1` giving the
#'   probability of scoring in categories 0, 1, ..., m. Computed with a
#'   log-sum-exp shift for numerical stability.
#' @keywords internal
#' @noRd
.pcm_cat_probs <- function(theta, thr) {
  cum_eta <- c(0, cumsum(theta - thr))
  shift   <- max(cum_eta)
  exp(cum_eta - (shift + log(sum(exp(cum_eta - shift)))))
}

#' Centre item thresholds to grand mean zero
#'
#' The standard Rasch identification: the mean of all thresholds is set
#' to zero so that person locations are on a common, comparable scale
#' across functions (matching the convention in [RMitemParameters()] and
#' the score-to-logit table).
#'
#' @param thr_list List of threshold vectors, one per item.
#' @return The list with every threshold shifted by the grand mean.
#' @keywords internal
#' @noRd
.center_thresholds <- function(thr_list) {
  shift <- mean(unlist(thr_list))
  lapply(thr_list, function(x) x - shift)
}

# ---------------------------------------------------------------------
# Per-pattern WLE (Warm, 1989) with analytic SEM
# ---------------------------------------------------------------------

#' Warm's weighted likelihood estimate for a single response pattern
#'
#' Newton-Raphson with adaptive step damping on the Warm-corrected score
#' equation. Only the items the person actually answered (non-`NA`)
#' contribute, so partial missingness is handled directly on the response
#' pattern rather than via a sum-score lookup.
#'
#' @param resp Numeric response vector for one person (may contain `NA`).
#' @param thr_list List of threshold vectors, one per item, aligned with
#'   `resp`.
#' @param theta_range Length-2 numeric search/boundary range.
#' @return Named numeric vector `c(theta, sem)`. Minimum and maximum
#'   possible scores yield finite WLE estimates (Warm's bias correction
#'   keeps the weighted likelihood equation solvable at the boundaries,
#'   unlike MLE); only if the root falls outside `theta_range` is the
#'   boundary returned with `NA` SEM.
#' @keywords internal
#' @noRd
.theta_wle <- function(resp, thr_list, theta_range, tol = 1e-6) {
  ok <- !is.na(resp)
  if (!any(ok)) return(c(theta = NA_real_, sem = NA_real_))

  resp     <- resp[ok]
  thr_list <- thr_list[ok]
  r         <- sum(resp)

  # Information at theta (also used for the SEM).
  info_at <- function(theta) {
    s <- 0
    for (i in seq_along(thr_list)) {
      cats <- 0:length(thr_list[[i]])
      p    <- .pcm_cat_probs(theta, thr_list[[i]])
      E    <- sum(cats * p)
      s    <- s + sum((cats - E)^2 * p)
    }
    s
  }

  # Warm's weighted-likelihood score function: l'(theta) + J(theta) / (2 I(theta)),
  # with J the summed third central moment of the item scores. Solving this for
  # zero gives the WLE, which is finite even at extreme scores.
  score <- function(theta) {
    dll <- r; info <- 0; third <- 0
    for (i in seq_along(thr_list)) {
      cats <- 0:length(thr_list[[i]])
      p    <- .pcm_cat_probs(theta, thr_list[[i]])
      E    <- sum(cats * p)
      dll   <- dll  - E
      info  <- info + sum((cats - E)^2 * p)
      third <- third + sum((cats - E)^3 * p)
    }
    dll + third / (2 * info)
  }

  lo <- theta_range[1L]; hi <- theta_range[2L]
  g_lo <- score(lo); g_hi <- score(hi)

  if (is.finite(g_lo) && is.finite(g_hi) && g_lo * g_hi < 0) {
    theta <- stats::uniroot(score, c(lo, hi), tol = tol)$root
    info  <- info_at(theta)
    sem   <- if (info < 1e-12) NA_real_ else 1 / sqrt(info)
  } else {
    # Root outside the search range: clamp to the nearer boundary.
    theta <- if (r <= sum(vapply(thr_list, length, integer(1))) / 2) lo else hi
    sem   <- NA_real_
  }
  c(theta = theta, sem = sem)
}

# ---------------------------------------------------------------------
# Quadrature machinery for EAP and prior-SD estimation
# ---------------------------------------------------------------------

#' Per-item log-probability tables over a quadrature grid
#'
#' Precomputes, for each item, a `length(grid) x n_cat` matrix of
#' log category probabilities. This lets per-person EAP and the marginal
#' likelihood reuse the same tables instead of recomputing probabilities.
#'
#' @param thr_list List of threshold vectors, one per item.
#' @param grid Numeric vector of quadrature nodes.
#' @return List of log-probability matrices, one per item.
#' @keywords internal
#' @noRd
.logp_tables <- function(thr_list, grid) {
  lapply(thr_list, function(thr) {
    tab <- vapply(grid, function(th) log(.pcm_cat_probs(th, thr)),
                  numeric(length(thr) + 1L))
    t(tab)   # rows = grid nodes, cols = categories 0..m
  })
}

#' Person-by-node log-likelihood matrix
#'
#' Sums the per-item log probabilities of each person's observed
#' responses across the quadrature grid, skipping missing responses.
#'
#' @param data_mat Numeric response matrix (persons x items).
#' @param logp_tabs List of per-item log-probability tables from
#'   `.logp_tables()`.
#' @param grid Numeric vector of quadrature nodes.
#' @return Numeric matrix (persons x nodes) of log-likelihoods.
#' @keywords internal
#' @noRd
.grid_loglik <- function(data_mat, logp_tabs, grid) {
  n_persons <- nrow(data_mat)
  loglik    <- matrix(0, n_persons, length(grid))
  for (i in seq_along(logp_tabs)) {
    x   <- data_mat[, i]
    obs <- !is.na(x)
    if (!any(obs)) next
    # category index (1-based) for each observed response on item i
    cat_idx <- x[obs] + 1L
    loglik[obs, ] <- loglik[obs, ] + t(logp_tabs[[i]][, cat_idx, drop = FALSE])
  }
  loglik
}

#' Marginal-maximum-likelihood estimate of the prior SD
#'
#' Holds the item parameters fixed and finds the normal-prior SD that
#' maximises the marginal likelihood of the data over the quadrature
#' grid. Used as the default EAP prior when the user does not supply one.
#'
#' @param loglik Person-by-node log-likelihood matrix from `.grid_loglik()`.
#' @param grid Numeric quadrature nodes.
#' @param prior_mean Numeric prior mean.
#' @return Numeric scalar: the estimated prior SD.
#' @keywords internal
#' @noRd
.estimate_prior_sd <- function(loglik, grid, prior_mean) {
  dgrid <- if (length(grid) > 1L) mean(diff(grid)) else 1
  neg_marg_ll <- function(sigma) {
    lprior <- stats::dnorm(grid, mean = prior_mean, sd = sigma, log = TRUE)
    # log integral over nodes for each person, via log-sum-exp
    M  <- sweep(loglik, 2L, lprior, `+`)
    mx <- apply(M, 1L, max)
    person_ll <- mx + log(rowSums(exp(M - mx)) * dgrid)
    -sum(person_ll[is.finite(person_ll)])
  }
  stats::optimize(neg_marg_ll, interval = c(0.05, 5))$minimum
}

#' EAP point estimates and posterior SDs from a grid log-likelihood
#'
#' @param loglik Person-by-node log-likelihood matrix.
#' @param grid Numeric quadrature nodes.
#' @param prior_mean,prior_sd Normal-prior parameters.
#' @return data.frame with columns `theta` (EAP) and `sem` (posterior SD).
#'   Persons with no answered items return `NA`.
#' @keywords internal
#' @noRd
.theta_eap <- function(loglik, grid, prior_mean, prior_sd) {
  lprior <- stats::dnorm(grid, mean = prior_mean, sd = prior_sd, log = TRUE)
  M      <- sweep(loglik, 2L, lprior, `+`)
  mx     <- apply(M, 1L, max)
  w      <- exp(M - mx)
  wsum   <- rowSums(w)

  eap  <- as.numeric((w %*% grid) / wsum)
  eap2 <- as.numeric((w %*% grid^2) / wsum)
  sd   <- sqrt(pmax(eap2 - eap^2, 0))
  # rows with no information (all-missing) -> NA
  empty <- !is.finite(eap)
  eap[empty] <- NA_real_
  sd[empty]  <- NA_real_
  data.frame(theta = eap, sem = sd)
}

# ---------------------------------------------------------------------
# Shared CML/WLE estimation engine (used by the Q3 local-dependence
# functions; intended as the common entry point for migrating other
# functions off mirt MML/EAP and eRm MLE).
# ---------------------------------------------------------------------

#' Conditional-maximum-likelihood item thresholds via psychotools
#'
#' Fits a Rasch model (dichotomous) or Partial Credit Model (polytomous)
#' by CML using `psychotools::raschmodel()` / `psychotools::pcmodel()`
#' and returns a list of grand-mean-centred Andrich threshold vectors,
#' one per item --- the normalised representation consumed by
#' `.pcm_cat_probs()`, `.theta_wle()`, and the residual routines.
#' `psychotools` is used (rather than `eRm`) because its CML fitter is
#' an order of magnitude faster, handles missing data without the
#' per-NA-pattern slowdown of `eRm`, and returns numerically identical
#' parameters after centring.
#'
#' @param data Numeric response matrix or data.frame (items scored from 0).
#' @return List of centred Andrich threshold vectors, one per item.
#' @keywords internal
#' @noRd
.fit_cml_thresholds <- function(data) {
  data <- as.matrix(data)
  is_poly <- max(data, na.rm = TRUE) > 1L
  # A dichotomous item is a 2-category PCM, so pcmodel() handles both types and
  # gives the identical CML estimate (same log-likelihood, difficulties match to
  # 0). raschmodel() is used for the dichotomous case only because it is ~2x
  # faster, which matters in the per-iteration bootstrap loops that call this.
  # The one-time `.rasch_fit_cml()` (item_parameters.R) uses pcmodel() for both,
  # where that speed difference is immaterial.
  fit <- if (is_poly) psychotools::pcmodel(data) else psychotools::raschmodel(data)
  thr_list <- lapply(psychotools::threshpar(fit), as.numeric)
  .center_thresholds(thr_list)
}

#' Estimate person locations under a fixed item model
#'
#' Shared person-estimation entry point. Given centred item thresholds,
#' returns a per-respondent location estimate (and its SEM) by Warm's
#' weighted likelihood (`"WLE"`, the default) or expected a posteriori
#' (`"EAP"`). WLE depends only on the sufficient statistic --- the
#' answered-item set and the partial raw score over it --- so estimates
#' are solved once per distinct (answered-set, score) key and mapped
#' back to respondents. On complete data this collapses to one solve
#' per distinct raw score.
#'
#' @param data Numeric response matrix or data.frame (items from 0; `NA`
#'   allowed).
#' @param thr_list List of centred Andrich threshold vectors, one per
#'   item, aligned with the columns of `data` (e.g. from
#'   `.fit_cml_thresholds()`).
#' @param method `"WLE"` (default) or `"EAP"`.
#' @param theta_range Length-2 search/boundary range for WLE and the EAP
#'   quadrature grid. Person-reporting callers (`RMpersonParameters()`,
#'   `RMpersonFit()`) pass the wider `c(-10, 10)` to locate extreme scorers;
#'   the default `c(-6, 6)` is used by the parametric-bootstrap theta pools
#'   (via `.wle_theta_pool()`), where it is the validated/calibrated range and
#'   only the resampled extremes are affected.
#' @param prior_mean,prior_sd Normal-prior parameters for `"EAP"`. When
#'   `prior_sd` is `NULL` it is estimated from the data by marginal
#'   maximum likelihood.
#' @param n_nodes Number of EAP quadrature nodes (default 121; the single
#'   source for every EAP caller so the same data yields the same estimate).
#' @return data.frame with columns `theta` and `sem`, one row per
#'   respondent (in input order). For `method = "EAP"` the result carries
#'   `attr(., "prior_mean")` and `attr(., "prior_sd")` (the prior actually
#'   used).
#' @keywords internal
#' @noRd
.estimate_thetas <- function(data, thr_list, method = c("WLE", "EAP"),
                             theta_range = c(-6, 6), prior_mean = 0,
                             prior_sd = NULL, n_nodes = 121L) {
  method <- match.arg(method)
  data <- as.matrix(data)

  if (method == "EAP") {
    grid   <- seq(theta_range[1L], theta_range[2L], length.out = n_nodes)
    logp   <- .logp_tables(thr_list, grid)
    loglik <- .grid_loglik(data, logp, grid)
    if (is.null(prior_sd)) prior_sd <- .estimate_prior_sd(loglik, grid, prior_mean)
    out <- .theta_eap(loglik, grid, prior_mean, prior_sd)
    # Expose the prior used so callers (e.g. RMpersonParameters) can report it.
    attr(out, "prior_mean") <- prior_mean
    attr(out, "prior_sd")   <- prior_sd
    return(out)
  }

  # WLE: cache by sufficient statistic (answered-set, partial score).
  pat <- apply(!is.na(data), 1L, function(z) paste0(which(z), collapse = ","))
  rs  <- rowSums(data, na.rm = TRUE)
  key <- paste(pat, rs, sep = "|")
  rep_idx <- which(!duplicated(key))
  est <- vapply(rep_idx, function(i) .theta_wle(data[i, ], thr_list, theta_range),
                numeric(2L))                       # rows: theta, sem; cols: reps
  m <- match(key, key[rep_idx])
  data.frame(theta = est["theta", m], sem = est["sem", m])
}

#' Finite WLE person-location pool for parametric-bootstrap DGPs
#'
#' Shared setup for the simulation-cutoff functions: fits CML item thresholds
#' and returns them together with the finite WLE person locations used as the
#' resampling pool. Centralises the block repeated across the `*Cutoff()`
#' functions (Q3, infit, CFA, gamma DIF/LD).
#'
#' @param data Numeric response matrix or data.frame (items from 0).
#' @return A list with `thr_list` (centred CML thresholds) and `thetas` (the
#'   finite WLE person locations).
#' @keywords internal
#' @noRd
.wle_theta_pool <- function(data) {
  data     <- as.matrix(data)
  thr_list <- .fit_cml_thresholds(data)
  thetas   <- .estimate_thetas(data, thr_list, method = "WLE")$theta
  list(thr_list = thr_list, thetas = thetas[is.finite(thetas)])
}

#' Standardized Rasch/PCM residual matrix (CML items + chosen person estimator)
#'
#' Computes the model standardized residuals \eqn{(x - E)/\sqrt{Var}} for
#' every person-by-item cell, using CML item thresholds and WLE (or EAP)
#' person locations. This is the residual object underlying Yen's Q3
#' (its column-wise correlation matrix) and is reusable for residual-based
#' diagnostics. Non-finite residuals (e.g. zero-variance cells) are set
#' to `NA`.
#'
#' @param data Numeric response matrix or data.frame (items from 0; `NA`
#'   allowed).
#' @param method Person estimator passed to `.estimate_thetas()`.
#' @return Numeric matrix (persons x items) of standardized residuals,
#'   with the item names of `data` as column names.
#' @keywords internal
#' @noRd
.rasch_std_residuals <- function(data, method = "WLE") {
  data <- as.matrix(data)
  thr_list <- .fit_cml_thresholds(data)
  th <- .estimate_thetas(data, thr_list, method = method)$theta
  N <- nrow(data); K <- ncol(data)
  E <- W <- matrix(NA_real_, N, K)
  for (i in seq_len(K)) {
    cats <- 0:length(thr_list[[i]])
    P  <- vapply(th, function(z) .pcm_cat_probs(z, thr_list[[i]]),
                 numeric(length(cats)))            # (n_cat x N)
    Ei <- as.numeric(crossprod(cats, P))
    W[, i] <- as.numeric(crossprod(cats^2, P)) - Ei^2
    E[, i] <- Ei
  }
  R <- (data - E) / sqrt(W)
  R[!is.finite(R)] <- NA
  colnames(R) <- colnames(data)
  R
}

test_that("RMlocdepQ3 no longer errors when cutoff is missing", {
  skip_if_not_installed("mirt")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5))
  # NULL cutoff should return a knitr_kable, not an error
  expect_s3_class(RMlocdepQ3(df), "knitr_kable")
})

test_that("RMlocdepQ3 with cutoff = NULL returns raw Q3 kable", {
  skip_if_not_installed("mirt")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5))
  result <- RMlocdepQ3(df, cutoff = NULL)
  expect_s3_class(result, "knitr_kable")
  # Caption should mention "Raw Q3"
  cap <- attr(result, "caption")
  if (is.null(cap)) cap <- paste(as.character(result), collapse = "\n")
  expect_true(grepl("Raw Q3", cap))
})

test_that("RMlocdepQ3 with cutoff = NULL returns raw Q3 dataframe", {
  skip_if_not_installed("mirt")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5))
  result <- RMlocdepQ3(df, cutoff = NULL, output = "dataframe")
  expect_s3_class(result, "data.frame")
  expect_true(all(is.na(diag(as.matrix(result)))))
  expect_true(all(is.na(result[upper.tri(result)])))
  expect_false(all(is.na(result[lower.tri(result)])))
})

test_that("RMlocdepQ3 errors when data has non-zero minimum", {
  df <- as.data.frame(matrix(sample(1:3, 100, replace = TRUE), nrow = 20, ncol = 5))
  expect_error(RMlocdepQ3(df, cutoff = 0.2), regexp = "scored starting at 0")
})

test_that("RMlocdepQ3 errors when data is not a data.frame or matrix", {
  expect_error(RMlocdepQ3(list(a = 1:5), cutoff = 0.2), regexp = "data.frame or matrix")
})

test_that("RMlocdepQ3 returns a data.frame when output = 'dataframe'", {
  skip_if_not_installed("mirt")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5))
  result <- RMlocdepQ3(df, cutoff = 0.2, output = "dataframe")
  expect_s3_class(result, "data.frame")
  # Lower triangle: off-diagonal lower should be numeric, upper + diag should be NA
  item_cols <- setdiff(names(result), "above_cutoff")
  expect_true(all(is.na(diag(as.matrix(result[item_cols])))))
  expect_true(all(is.na(result[item_cols][upper.tri(result[item_cols])])))
  expect_false(all(is.na(result[lower.tri(result)])))
})

test_that("RMlocdepQ3 returns a knitr_kable object when output = 'kable'", {
  skip_if_not_installed("mirt")
  set.seed(2)
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5))
  result <- RMlocdepQ3(df, cutoff = 0.2, output = "kable")
  expect_s3_class(result, "knitr_kable")
  # Caption should mention dynamic cut-off
  cap <- attr(result, "caption")
  if (is.null(cap)) cap <- paste(as.character(result), collapse = "\n")
  expect_true(grepl("Dynamic cut-off", cap))
})

# ---------------------------------------------------------------------
# RMlocdepQ3Cutoff -- small iterations
# ---------------------------------------------------------------------
test_that("RMlocdepQ3Cutoff returns the expected list structure", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("eRm")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200 * 8, replace = TRUE), 200, 8))
  colnames(df) <- paste0("I", 1:8)
  res <- RMlocdepQ3Cutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)
  expect_type(res, "list")
  expect_true(all(c("results", "p95", "p99", "p995", "p999",
                    "suggested_cutoff", "actual_iterations",
                    "sample_n") %in% names(res)))
  expect_true(res$actual_iterations >= 1L)
  expect_true(is.finite(res$suggested_cutoff))
})

test_that("RMlocdepQ3Cutoff result is consumable by RMlocdepQ3", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("eRm")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200 * 8, replace = TRUE), 200, 8))
  colnames(df) <- paste0("I", 1:8)
  cu  <- RMlocdepQ3Cutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)
  out <- RMlocdepQ3(df, cutoff = cu$suggested_cutoff)
  expect_s3_class(out, "knitr_kable")
})

test_that("RMlocdepQ3Cutoff is reproducible with the same seed", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("eRm")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200 * 8, replace = TRUE), 200, 8))
  colnames(df) <- paste0("I", 1:8)
  r1 <- RMlocdepQ3Cutoff(df, iterations = 5L, parallel = FALSE, seed = 42L)
  r2 <- RMlocdepQ3Cutoff(df, iterations = 5L, parallel = FALSE, seed = 42L)
  expect_equal(r1$results, r2$results)
})

# ---------------------------------------------------------------------
# Per-pair additions (pair_results / pair_cutoffs / item_names)
# ---------------------------------------------------------------------
test_that("RMlocdepQ3Cutoff returns pair_results / pair_cutoffs / item_names", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggdist")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200 * 8, replace = TRUE), 200, 8))
  colnames(df) <- paste0("I", 1:8)
  res <- RMlocdepQ3Cutoff(df, iterations = 5L, parallel = FALSE, seed = 42L)

  # New top-level slots
  expect_true(all(c("pair_results", "pair_cutoffs", "item_names",
                    "cutoff_method", "hdci_width") %in% names(res)))

  # pair_results: long format, one row per pair per successful iteration
  expect_s3_class(res$pair_results, "data.frame")
  expect_true(all(c("Item1", "Item2", "Q3", "iteration") %in%
                  names(res$pair_results)))
  expect_equal(nrow(res$pair_results),
               choose(ncol(df), 2L) * res$actual_iterations)
  expect_setequal(unique(c(res$pair_results$Item1, res$pair_results$Item2)),
                  colnames(df))

  # pair_cutoffs: one row per unordered pair
  expect_s3_class(res$pair_cutoffs, "data.frame")
  expect_true(all(c("Item1", "Item2", "Q3_low", "Q3_high") %in%
                  names(res$pair_cutoffs)))
  expect_equal(nrow(res$pair_cutoffs), choose(ncol(df), 2L))
  expect_true(all(res$pair_cutoffs$Q3_low <= res$pair_cutoffs$Q3_high))

  # item_names preserves the data colnames
  expect_equal(res$item_names, colnames(df))
})

test_that("RMlocdepQ3Cutoff cutoff_method = 'quantile' works without ggdist", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("eRm")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200 * 6, replace = TRUE), 200, 6))
  colnames(df) <- paste0("I", 1:6)
  res <- RMlocdepQ3Cutoff(df, iterations = 5L, parallel = FALSE, seed = 1L,
                          cutoff_method = "quantile")
  expect_equal(res$cutoff_method, "quantile")
  expect_true(all(c("Q3_low", "Q3_high") %in% names(res$pair_cutoffs)))
})

# ---------------------------------------------------------------------
# RMlocdepQ3 accepts the full simfit object
# ---------------------------------------------------------------------
test_that("full simfit returns list whose $matrix matches the scalar cutoff", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggdist")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200 * 6, replace = TRUE), 200, 6))
  colnames(df) <- paste0("I", 1:6)
  cu  <- RMlocdepQ3Cutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)
  # Full object -> list($matrix, $pairs); $matrix matches the scalar-cutoff matrix
  a <- RMlocdepQ3(df, cutoff = cu,                  output = "dataframe")
  b <- RMlocdepQ3(df, cutoff = cu$suggested_cutoff, output = "dataframe")
  expect_named(a, c("matrix", "pairs"))
  expect_equal(a$matrix, b)
})

# ---------------------------------------------------------------------
# RMlocdepQ3Plot
# ---------------------------------------------------------------------
test_that("RMlocdepQ3Plot returns a $matrix/$pairs list", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("scales")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200 * 6, replace = TRUE), 200, 6))
  colnames(df) <- paste0("I", 1:6)
  cu <- RMlocdepQ3Cutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)

  # Case 1: no data -> per-pair plot only, $matrix is NULL
  p1 <- suppressMessages(RMlocdepQ3Plot(cu))
  expect_named(p1, c("matrix", "pairs"))
  expect_null(p1$matrix)
  expect_s3_class(p1$pairs, "ggplot")

  # Case 2: observed data -> both the heatmap and the per-pair plot
  p2 <- RMlocdepQ3Plot(cu, data = df)
  expect_named(p2, c("matrix", "pairs"))
  expect_s3_class(p2$matrix, "ggplot")
  expect_s3_class(p2$pairs, "ggplot")
})

test_that("RMlocdepQ3Plot n_pairs trims and orders by deviation", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("scales")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200 * 6, replace = TRUE), 200, 6))
  colnames(df) <- paste0("I", 1:6)
  cu <- RMlocdepQ3Cutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)

  p <- RMlocdepQ3Plot(cu, data = df, n_pairs = 3L)
  expect_s3_class(p$pairs, "ggplot")
  # Pair factor should have exactly 3 levels
  expect_equal(nlevels(p$pairs$data$Pair), 3L)
})

test_that("RMlocdepQ3Plot validates inputs", {
  skip_on_cran()
  skip_if_not_installed("mirt")
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggdist")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200 * 6, replace = TRUE), 200, 6))
  colnames(df) <- paste0("I", 1:6)
  cu <- RMlocdepQ3Cutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)

  # Missing required simfit slots
  expect_error(RMlocdepQ3Plot(list(results = data.frame())),
               regexp = "missing required components")
  # n_pairs validation
  expect_error(RMlocdepQ3Plot(cu, n_pairs = 0L),  regexp = "positive integer")
  expect_error(RMlocdepQ3Plot(cu, n_pairs = 1.5), regexp = "positive integer")
  # items validation
  expect_error(RMlocdepQ3Plot(cu, items = "no_such_item"),
               regexp = "Unknown item")
  expect_error(RMlocdepQ3Plot(cu, items = c("I1")),
               regexp = "at least 2 item names")
})

# ---------------------------------------------------------------------
# RMlocdepQ3() bootstrap p-values (per item pair)
# ---------------------------------------------------------------------

q3_null_data <- function(n = 300, J = 7, seed = 11L) {
  set.seed(seed)
  theta <- rnorm(n, 0, 1.3); beta <- seq(-1.5, 1.5, length.out = J)
  df <- as.data.frame(sapply(seq_len(J), function(j) rbinom(n, 1, plogis(theta - beta[j]))))
  colnames(df) <- paste0("I", seq_len(J)); df
}

test_that("full cutoff object returns list($matrix, $pairs, $plot)", {
  skip_on_cran()
  skip_if_not_installed("mirt"); skip_if_not_installed("ggdist")
  df  <- q3_null_data()
  sim <- RMlocdepQ3Cutoff(df, iterations = 300, parallel = FALSE, seed = 1)
  res <- RMlocdepQ3(df, cutoff = sim, output = "dataframe")
  expect_named(res, c("matrix", "pairs"))
  expect_named(res$pairs, c("Item1", "Item2", "Observed", "Low", "High", "Flagged"))
  expect_equal(nrow(res$pairs), choose(ncol(df), 2L))          # one row per pair
  expect_true(all(res$pairs$Flagged %in% c("above", "below", "")))
})

test_that("n_pairs truncates the $pairs table", {
  skip_on_cran()
  skip_if_not_installed("mirt"); skip_if_not_installed("ggdist")
  df  <- q3_null_data()
  sim <- RMlocdepQ3Cutoff(df, iterations = 200, parallel = FALSE, seed = 1)
  res <- RMlocdepQ3(df, cutoff = sim, n_pairs = 3, output = "dataframe")
  expect_equal(nrow(res$pairs), 3L)
})

test_that("p_value = TRUE adds p columns to $pairs and flags on padj", {
  skip_on_cran()
  skip_if_not_installed("mirt"); skip_if_not_installed("ggdist")
  df  <- q3_null_data()
  sim <- RMlocdepQ3Cutoff(df, iterations = 300, parallel = FALSE, seed = 1)
  res <- suppressWarnings(
    RMlocdepQ3(df, cutoff = sim, p_value = TRUE, output = "dataframe")
  )
  expect_true(all(c("p_q3", "padj_q3") %in% names(res$pairs)))
  expect_true(all(res$pairs$p_q3 >= 0 & res$pairs$p_q3 <= 1, na.rm = TRUE))
  expect_true(all(res$pairs$padj_q3 >= res$pairs$p_q3 - 1e-9, na.rm = TRUE))
})

test_that("RMlocdepQ3 p_value = TRUE errors without the full cutoff object", {
  skip_if_not_installed("mirt"); skip_if_not_installed("ggdist")
  df  <- q3_null_data()
  sim <- RMlocdepQ3Cutoff(df, iterations = 200, parallel = FALSE, seed = 1)
  expect_error(RMlocdepQ3(df, p_value = TRUE), regexp = "full RMlocdepQ3Cutoff")
  expect_error(RMlocdepQ3(df, cutoff = sim$suggested_cutoff, p_value = TRUE),
               regexp = "full RMlocdepQ3Cutoff")
})

test_that("NULL cutoff is unchanged (single square matrix)", {
  skip_if_not_installed("mirt")
  df  <- q3_null_data()
  res <- RMlocdepQ3(df, output = "dataframe")
  expect_false(is.list(res) && all(c("matrix", "pairs") %in% names(res)))
  expect_equal(nrow(res), ncol(df))                            # square matrix
})

test_that("RMlocdepQ3 detects an injected locally dependent pair", {
  skip_on_cran()
  skip_if_not_installed("mirt"); skip_if_not_installed("ggdist")
  set.seed(3); n <- 400; J <- 7
  theta <- rnorm(n, 0, 1.3); beta <- seq(-1.5, 1.5, length.out = J)
  m <- sapply(seq_len(J), function(j) rbinom(n, 1, plogis(theta - beta[j])))
  cp <- sample(n, 0.7 * n); m[cp, 2] <- m[cp, 1]              # I1, I2 dependent
  df <- as.data.frame(m); colnames(df) <- paste0("I", seq_len(J))
  sim <- RMlocdepQ3Cutoff(df, iterations = 500, parallel = FALSE, seed = 4)
  res <- RMlocdepQ3(df, cutoff = sim, output = "dataframe")$pairs
  dep <- res[(res$Item1 == "I1" & res$Item2 == "I2") |
             (res$Item1 == "I2" & res$Item2 == "I1"), ]
  expect_identical(dep$Flagged, "above")
  # the injected pair has the largest departure -> sorted to the top row
  expect_true((res$Item1[1] == "I1" && res$Item2[1] == "I2") ||
              (res$Item1[1] == "I2" && res$Item2[1] == "I1"))
})

# ---------------------------------------------------------------------
# Estimator engine (CML/WLE default, MML/EAP optional)
# ---------------------------------------------------------------------
test_that("CML is the default estimator and MML stays available", {
  skip_on_cran()
  skip_if_not_installed("psychotools")
  set.seed(3)
  df <- as.data.frame(matrix(sample(0:3, 250 * 6, replace = TRUE), 250, 6))
  colnames(df) <- paste0("I", 1:6)

  cml <- RMlocdepQ3(df, output = "dataframe")                    # default
  expect_s3_class(cml, "data.frame")
  expect_equal(dim(cml), c(6L, 6L))

  skip_if_not_installed("mirt")
  mml <- RMlocdepQ3(df, output = "dataframe", estimator = "MML")
  # Same statistic, different engine -> correlated but not identical. The bound
  # is loose because this is unstructured (noise) data; on real Rasch data the
  # CML/MML off-diagonal correlation is typically > 0.95.
  expect_gt(stats::cor(as.matrix(cml)[lower.tri(cml)],
                       as.matrix(mml)[lower.tri(mml)]), 0.6)
})

test_that("cutoff object stores its estimator and RMlocdepQ3 honours it", {
  skip_on_cran()
  skip_if_not_installed("psychotools"); skip_if_not_installed("ggdist")
  skip_if_not_installed("mirt")
  set.seed(4)
  df <- as.data.frame(matrix(sample(0:1, 200 * 6, replace = TRUE), 200, 6))
  colnames(df) <- paste0("I", 1:6)

  cu <- RMlocdepQ3Cutoff(df, iterations = 10, parallel = FALSE, seed = 1)
  expect_identical(cu$estimator, "CML")

  # Asking for MML while the cut-off is CML must warn and fall back to CML.
  expect_warning(
    RMlocdepQ3(df, cutoff = cu, estimator = "MML", output = "dataframe"),
    "overriding estimator"
  )
})

test_that("shared person-estimation helper returns theta + sem for WLE and EAP", {
  skip_if_not_installed("psychotools")
  set.seed(5)
  df <- matrix(sample(0:2, 150 * 5, replace = TRUE), 150, 5)
  thr <- .fit_cml_thresholds(df)
  wle <- .estimate_thetas(df, thr, method = "WLE")
  eap <- .estimate_thetas(df, thr, method = "EAP")
  expect_named(wle, c("theta", "sem"))
  expect_named(eap, c("theta", "sem"))
  expect_equal(nrow(wle), nrow(df))
  expect_gt(stats::cor(wle$theta, eap$theta, use = "complete.obs"), 0.95)
})

# ---------------------------------------------------------------------
# Conditional DGP option
# ---------------------------------------------------------------------
test_that("dgp = 'conditional' runs and is consumable downstream", {
  skip_on_cran()
  skip_if_not_installed("psychotools"); skip_if_not_installed("ggdist")
  set.seed(6)
  df <- as.data.frame(matrix(sample(0:1, 200 * 6, replace = TRUE), 200, 6))
  colnames(df) <- paste0("I", 1:6)

  cu <- RMlocdepQ3Cutoff(df, iterations = 20, parallel = FALSE, seed = 1,
                         dgp = "conditional")
  expect_identical(cu$dgp, "conditional")
  expect_true(is.finite(cu$suggested_cutoff))
  expect_equal(nrow(cu$pair_cutoffs), choose(ncol(df), 2L))

  # default is still "resample"
  cu_def <- RMlocdepQ3Cutoff(df, iterations = 5, parallel = FALSE, seed = 1)
  expect_identical(cu_def$dgp, "resample")

  # the conditional object flows through RMlocdepQ3()
  res <- RMlocdepQ3(df, cutoff = cu, output = "dataframe")
  expect_named(res, c("matrix", "pairs"))
})

test_that("RMlocdepQ3(estimator='CML') warns on sparse/zero-variance items", {
  skip_if_not_installed("eRm")
  df <- as.data.frame(matrix(sample(0:1, 200 * 6, replace = TRUE), 200, 6))
  colnames(df) <- paste0("I", 1:6)
  df[[1]] <- 0L   # constant item -> destabilises CML; should warn
  expect_warning(try(RMlocdepQ3(df, estimator = "CML"), silent = TRUE),
                 regexp = "[Ss]parse|zero-variance")
})

# ---------------------------------------------------------------------
# All-NA respondents (regression: psychotools cannot fit all-NA rows;
# pcmodel errors and raschmodel segfaults, so they must be dropped
# up front)
# ---------------------------------------------------------------------
test_that("RMlocdepQ3Cutoff drops all-NA respondents instead of erroring", {
  skip_if_not_installed("ggdist")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 60 * 6, replace = TRUE), nrow = 60))
  colnames(df) <- paste0("i", 1:6)
  df[3, ] <- NA

  expect_message(
    res <- RMlocdepQ3Cutoff(df, iterations = 5, parallel = FALSE, seed = 1),
    "no responses dropped"
  )
  expect_equal(res$sample_n, 59L)
  expect_equal(res$sample_n_total, 60L)
  expect_true(res$sample_has_na)
  # The full object stays consumable by RMlocdepQ3() (which drops the same row)
  out <- suppressMessages(RMlocdepQ3(df, cutoff = res, output = "dataframe"))
  expect_named(out, c("matrix", "pairs"))
})

test_that("RMlocdepQ3Plot observed overlay drops all-NA respondents", {
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("scales")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 60 * 6, replace = TRUE), nrow = 60))
  colnames(df) <- paste0("i", 1:6)
  simfit <- RMlocdepQ3Cutoff(df, iterations = 5, parallel = FALSE, seed = 1)
  df[3, ] <- NA

  expect_message(
    p <- RMlocdepQ3Plot(simfit, data = df),
    "no responses dropped"
  )
  expect_s3_class(p$pairs, "ggplot")
})

# Guard against the roxygen "Could not resolve link to topic" warnings that
# devtools::document() emits when a cross-reference link points at an internal
# helper. By package convention every dot-prefixed function (e.g.
# `.estimate_thetas`, `.q3_residual_matrix`) is internal and tagged `@noRd`, so
# it generates no Rd topic and a link to it can never resolve. Reference such
# helpers as plain code -- `.estimate_thetas()` -- never as a link
# (`[.estimate_thetas()]`, `[`.estimate_thetas()`]`, or `\link{.estimate_thetas}`).
#
# This is the blind spot R CMD check does not cover: it validates \link{} targets
# only for documented (exported) topics, so broken links to @noRd helpers slip
# through as document()-time warnings.

test_that("roxygen comments never link to internal (dot-prefixed) helpers", {
  r_dir <- test_path("..", "..", "R")
  skip_if_not(dir.exists(r_dir))
  r_files <- list.files(r_dir, pattern = "\\.R$", full.names = TRUE)

  # Matches a roxygen markdown auto-link or an Rd \link{} whose target begins
  # with a dot: "[.name", "[`.name", or "\\link{.name".
  link_to_dotted <- "\\[`?\\.[A-Za-z]|\\\\link\\{\\.[A-Za-z]"

  offenders <- character()
  for (f in r_files) {
    lines    <- readLines(f, warn = FALSE)
    is_roxy  <- grepl("^\\s*#'", lines)
    hit_rows <- which(is_roxy & grepl(link_to_dotted, lines))
    if (length(hit_rows)) {
      offenders <- c(offenders, sprintf("%s:%d: %s", basename(f), hit_rows,
                                        trimws(lines[hit_rows])))
    }
  }

  if (length(offenders)) {
    fail(paste0(
      "Roxygen links to internal (@noRd) helpers warn in devtools::document() ",
      "(\"Could not resolve link to topic\"). Use plain `code()` instead of a ",
      "[link]:\n", paste(offenders, collapse = "\n")
    ))
  }
  succeed()
})

# Tests for RMitemParameters() and RMpersonParameters()

make_polytomous <- function(n = 250, k = 5, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:2, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

make_dichotomous <- function(n = 250, k = 6, seed = 2L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# Simulate data with genuine latent structure (for recovery-style checks).
make_pcm_signal <- function(n = 600, k = 6, person_sd = 1.4, seed = 7L) {
  set.seed(seed)
  theta <- rnorm(n, 0, person_sd)
  thr <- lapply(seq_len(k), function(j) sort(rnorm(2, 0, 1)))
  pcm_p <- function(th, t) {
    ce <- c(0, cumsum(th - t))
    exp(ce - max(ce)) / sum(exp(ce - max(ce)))
  }
  m <- sapply(seq_len(k), function(j) {
    sapply(theta, function(th) sample(0:2, 1, prob = pcm_p(th, thr[[j]])))
  })
  df <- as.data.frame(m)
  colnames(df) <- paste0("I", seq_len(k))
  list(data = df, theta = theta, person_sd = person_sd)
}

# =====================================================================
# RMitemParameters
# =====================================================================

test_that("RMitemParameters errors on non-zero minimum", {
  expect_error(
    RMitemParameters(make_dichotomous() + 1L),
    regexp = "scored starting at 0"
  )
})

test_that("RMitemParameters errors on invalid ci_level", {
  skip_if_not_installed("eRm")
  expect_error(
    RMitemParameters(make_dichotomous(), ci_level = 1.2),
    regexp = "ci_level"
  )
})

test_that("long format has expected columns and rows (polytomous)", {
  skip_if_not_installed("eRm")
  res <- RMitemParameters(make_polytomous(), output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_named(
    res,
    c("item", "threshold", "location", "se", "ci_lower", "ci_upper")
  )
  # 5 items x 2 thresholds
  expect_equal(nrow(res), 10L)
  expect_true(all(res$ci_lower < res$location & res$location < res$ci_upper))
})

test_that("se = FALSE drops SE/CI columns", {
  skip_if_not_installed("eRm")
  res <- RMitemParameters(make_polytomous(), se = FALSE, output = "dataframe")
  expect_named(res, c("item", "threshold", "location"))
})

test_that("wide format returns one row per item with threshold columns", {
  skip_if_not_installed("eRm")
  res <- RMitemParameters(
    make_polytomous(),
    format = "wide",
    se = FALSE,
    output = "dataframe"
  )
  expect_equal(nrow(res), 5L)
  expect_true(all(c("item", "t1", "t2", "location") %in% names(res)))
})

test_that("wide format with SE appends se_t* columns", {
  skip_if_not_installed("eRm")
  res <- RMitemParameters(
    make_polytomous(),
    format = "wide",
    se = TRUE,
    output = "dataframe"
  )
  expect_true(all(c("se_t1", "se_t2") %in% names(res)))
})

test_that("dichotomous data yields one row per item", {
  skip_if_not_installed("eRm")
  res <- RMitemParameters(make_dichotomous(), output = "dataframe")
  expect_equal(nrow(res), 6L)
  expect_true(all(res$threshold == 1L))
})

test_that("kable output returns knitr_kable", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("knitr")
  expect_s3_class(RMitemParameters(make_polytomous()), "knitr_kable")
})

test_that("all-NA respondents are dropped with a message and n is reported", {
  skip_if_not_installed("knitr")
  df <- make_polytomous(n = 100)
  df[100, ] <- NA
  # message + drop (also prevents the CML fit from erroring on the all-NA row)
  expect_message(
    kbl <- RMitemParameters(df),
    "1 respondent\\(s\\) with no responses dropped"
  )
  out <- paste(capture.output(print(kbl)), collapse = " ")
  expect_match(
    out,
    "n = 99 of 100 respondents \\(incomplete responses retained\\)"
  )
  # RMpersonParameters drops the row from its per-respondent output
  pp <- suppressMessages(RMpersonParameters(df, output = "dataframe"))
  expect_equal(nrow(pp), 99L)
})

test_that("caption qualifiers are conditional on missingness", {
  skip_if_not_installed("knitr")
  # Complete data: no "of Y", no parenthetical.
  complete <- paste(
    capture.output(print(RMitemParameters(make_polytomous(n = 100)))),
    collapse = " "
  )
  expect_match(complete, "n = 100 respondents\\.")
  expect_no_match(complete, "incomplete responses retained")
  # Partial (retained) missingness: parenthetical, but still no "of Y".
  df <- make_polytomous(n = 100)
  df[1, 1] <- NA
  partial <- paste(capture.output(print(RMitemParameters(df))), collapse = " ")
  expect_match(
    partial,
    "n = 100 respondents \\(incomplete responses retained\\)"
  )
})

test_that("output = 'file' writes the dataframe to CSV and returns it invisibly", {
  skip_if_not_installed("eRm")
  path <- withr::local_tempfile(fileext = ".csv")
  df <- RMitemParameters(make_polytomous(), output = "dataframe")
  ret <- suppressMessages(
    RMitemParameters(make_polytomous(), output = "file", filename = path)
  )
  expect_equal(ret, df)
  back <- read.csv(path)
  expect_equal(names(back), names(df))
  expect_equal(nrow(back), nrow(df))
  expect_equal(back$location, df$location, tolerance = 1e-6)
})

test_that("output = 'file' errors early when filename is missing", {
  skip_if_not_installed("eRm")
  expect_snapshot(
    RMitemParameters(make_polytomous(), output = "file"),
    error = TRUE
  )
})

test_that("wider ci_level gives wider intervals", {
  skip_if_not_installed("eRm")
  narrow <- RMitemParameters(
    make_polytomous(),
    ci_level = 0.90,
    output = "dataframe"
  )
  wide <- RMitemParameters(
    make_polytomous(),
    ci_level = 0.99,
    output = "dataframe"
  )
  expect_true(all(
    (wide$ci_upper - wide$ci_lower) > (narrow$ci_upper - narrow$ci_lower)
  ))
})

test_that("MML estimator agrees with CML on item locations", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("mirt")
  sim <- make_pcm_signal()
  cml <- RMitemParameters(sim$data, estimator = "CML", output = "dataframe")
  mml <- RMitemParameters(sim$data, estimator = "MML", output = "dataframe")
  ord <- match(paste(cml$item, cml$threshold), paste(mml$item, mml$threshold))
  expect_gt(cor(cml$location, mml$location[ord]), 0.98)
  expect_true(all(is.finite(mml$se)))
})

test_that("CML threshold SEs are recovered (delta method sane)", {
  skip_if_not_installed("eRm")
  res <- RMitemParameters(make_pcm_signal()$data, output = "dataframe")
  expect_true(all(res$se > 0 & res$se < 1))
})

# =====================================================================
# RMpersonParameters
# =====================================================================

test_that("RMpersonParameters errors on bad theta_range", {
  expect_error(
    RMpersonParameters(make_polytomous(), theta_range = c(5, -5)),
    regexp = "theta_range"
  )
})

test_that("WLE dataframe has one row per person and expected columns", {
  skip_if_not_installed("eRm")
  df <- make_polytomous(n = 200)
  res <- RMpersonParameters(df, output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_equal(nrow(res), 200L)
  expect_named(res, c("theta", "sem", "sum_score", "n_answered", "extreme"))
})

test_that("WLE gives finite extrapolated estimates for extreme scores", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous(n = 200, k = 6)
  df[1, ] <- 0L # minimum score
  df[2, ] <- 1L # maximum score
  res <- RMpersonParameters(df, output = "dataframe")
  expect_true(res$extreme[1] && res$extreme[2])
  # WLE is finite at the boundaries (Warm), well inside theta_range,
  # and a sensible step beyond the next-most-extreme estimate.
  expect_true(is.finite(res$theta[1]) && is.finite(res$theta[2]))
  expect_true(res$theta[1] > -10 && res$theta[2] < 10)
  second_low <- min(res$theta[res$sum_score == 1])
  second_high <- max(res$theta[res$sum_score == 5])
  expect_lt(res$theta[1], second_low) # zero score below the 1-score
  expect_gt(res$theta[2], second_high) # perfect score above the 5-score
})

test_that("WLE satisfies Warm's weighted-likelihood score equation", {
  beta <- c(-1.5, -0.8, -0.2, 0.3, 0.9, 1.6)
  thr <- as.list(beta)
  # Warm score: l'(theta) + J(theta) / (2 I(theta)); zero at the WLE.
  warm_score <- function(theta, x) {
    p <- plogis(theta - beta)
    dll <- sum(x) - sum(p)
    I <- sum(p * (1 - p))
    J <- sum(p * (1 - p) * (1 - 2 * p))
    dll + J / (2 * I)
  }
  for (r in 0:length(beta)) {
    x <- c(rep(1L, r), rep(0L, length(beta) - r))
    th <- easyRasch2:::.theta_wle(x, thr, c(-10, 10))[["theta"]]
    expect_lt(abs(warm_score(th, x)), 1e-4) # solves the equation
  }
})

test_that("partial missingness is handled per-pattern", {
  skip_if_not_installed("eRm")
  df <- make_polytomous(n = 200)
  df[cbind(sample(200, 40), sample(5, 40, replace = TRUE))] <- NA
  res <- RMpersonParameters(df, output = "dataframe")
  expect_equal(nrow(res), 200L)
  expect_true(any(res$n_answered < 5L))
  # non-extreme persons get a finite estimate even with missingness
  expect_true(all(is.finite(res$theta[!res$extreme])))
})

test_that("EAP returns finite estimates at extreme scores", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous(n = 200, k = 6)
  df[1, ] <- 0L
  df[2, ] <- 1L
  res <- RMpersonParameters(df, method = "EAP", output = "dataframe")
  expect_true(all(is.finite(res$theta)))
  expect_true(all(is.finite(res$sem)))
})

test_that("EAP attaches the prior used; WLE does not", {
  skip_if_not_installed("eRm")
  df <- make_pcm_signal()$data
  eap <- RMpersonParameters(df, method = "EAP", output = "dataframe")
  wle <- RMpersonParameters(df, method = "WLE", output = "dataframe")
  pr <- attr(eap, "prior")
  expect_false(is.null(pr))
  expect_identical(unname(pr["estimated"]), 1) # estimated
  expect_null(attr(wle, "prior"))
})

test_that("fixed prior_sd is respected and reported as not estimated", {
  skip_if_not_installed("eRm")
  df <- make_pcm_signal()$data
  eap <- RMpersonParameters(
    df,
    method = "EAP",
    prior_sd = 1,
    output = "dataframe"
  )
  pr <- attr(eap, "prior")
  expect_equal(unname(pr["sd"]), 1)
  expect_equal(unname(pr["estimated"]), 0)
})

test_that("marginal-ML prior SD recovers the simulated person SD", {
  skip_if_not_installed("eRm")
  sim <- make_pcm_signal(n = 800, k = 8, person_sd = 1.4)
  eap <- RMpersonParameters(sim$data, method = "EAP", output = "dataframe")
  est_sd <- unname(attr(eap, "prior")["sd"])
  expect_equal(est_sd, sim$person_sd, tolerance = 0.15)
})

test_that("invalid prior_sd is rejected", {
  skip_if_not_installed("eRm")
  expect_error(
    RMpersonParameters(make_polytomous(), method = "EAP", prior_sd = -1),
    regexp = "prior_sd"
  )
})

test_that("supplied item_params (list) bypasses model fitting", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous(n = 150, k = 4)
  ip <- list(I1 = 0.2, I2 = -0.3, I3 = 0.1, I4 = 0.0)
  res <- RMpersonParameters(df, item_params = ip, output = "dataframe")
  expect_equal(nrow(res), 150L)
  expect_true(all(is.finite(res$theta[!res$extreme])))
})

test_that("supplied item_params (data.frame from RMitemParameters) works", {
  skip_if_not_installed("eRm")
  df <- make_polytomous(n = 150)
  ip <- RMitemParameters(df, output = "dataframe")[, c(
    "item",
    "threshold",
    "location"
  )]
  res <- RMpersonParameters(df, item_params = ip, output = "dataframe")
  expect_equal(nrow(res), 150L)
})

test_that("kable and ggplot outputs return the right classes", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("knitr")
  df <- make_polytomous(n = 150)
  expect_s3_class(RMpersonParameters(df), "knitr_kable")
  skip_if_not_installed("ggplot2")
  expect_s3_class(RMpersonParameters(df, output = "ggplot"), "ggplot")
})

test_that("RMpersonParameters output = 'file' writes a CSV with one row per person", {
  skip_if_not_installed("eRm")
  df <- make_polytomous(n = 150)
  path <- withr::local_tempfile(fileext = ".csv")
  ddf <- RMpersonParameters(df, output = "dataframe")
  ret <- suppressMessages(
    RMpersonParameters(df, output = "file", filename = path)
  )
  expect_equal(ret, ddf)
  back <- read.csv(path)
  expect_equal(nrow(back), nrow(df))
  expect_equal(back$theta, ddf$theta, tolerance = 1e-6)
})

test_that("RMitemParameters dataframe output is unrounded and wide location = mean of thresholds", {
  # Regression: the dataframe/file path used to round to 4 decimals
  # (missed in the unrounded-dataframe sweep), which also made the wide
  # format's mean `location` differ from the mean of its threshold
  # columns in the 5th decimal.
  set.seed(42)
  dat <- as.data.frame(replicate(4, sample(0:2, 120, replace = TRUE)))
  names(dat) <- paste0("q", 1:4)

  w <- RMitemParameters(dat, format = "wide", se = FALSE,
                        output = "dataframe")
  t_cols <- grep("^t[0-9]+$", names(w), value = TRUE)
  expect_equal(w$location, rowMeans(w[, t_cols]))

  l <- RMitemParameters(dat, format = "long", se = TRUE,
                        output = "dataframe")
  # unrounded: exact agreement between long locations and wide columns
  expect_identical(sort(l$location), sort(unlist(w[, t_cols], use.names = FALSE)))
  expect_false(all(l$se == round(l$se, 4)))
})

#' DIF analysis via Andersen's likelihood-ratio test
#'
#' Splits a Rasch model by an external grouping variable using
#' \code{eRm::LRtest()} and reports per-group item locations (or per-group
#' threshold locations) together with their standard errors. A single
#' function replaces the four legacy helpers (`RIdifTableLR`,
#' `RIdifThreshTblLR`, `RIdifFigureLR`, `RIdifThreshFigLR`) by exposing the
#' two underlying axes -- \code{level} (item or threshold) and \code{output}
#' (data.frame, kable, or ggplot) -- as arguments. The same data
#' preparation pipeline feeds all six combinations.
#'
#' The Partial Credit Model (PCM) is fitted by default for polytomous data
#' and the dichotomous Rasch Model (RM) is fitted when all responses are
#' 0/1; this can be overridden via \code{model}.
#'
#' @param data A data.frame or matrix of item responses (non-negative
#'   integers, 0-based). One column per item, one row per person. Person
#'   IDs and grouping variables must not be included -- pass the grouping
#'   variable separately via \code{dif_var}.
#' @param dif_var Vector of length \code{nrow(data)} (factor, character,
#'   or numeric) defining the DIF grouping variable. Coerced to factor;
#'   unused levels are dropped. Rows where \code{dif_var} is \code{NA} are
#'   dropped with a message. Must result in at least 2 groups after
#'   cleaning.
#' @param model One of \code{"auto"} (default), \code{"PCM"}, or
#'   \code{"RM"}. \code{"auto"} fits \code{eRm::RM()} when the data are
#'   dichotomous (max response = 1) and \code{eRm::PCM()} otherwise.
#'   \code{"RM"} errors on polytomous data.
#' @param level One of \code{"item"} (default) or \code{"threshold"}.
#'   \code{"item"} reports each item's mean threshold location per group;
#'   \code{"threshold"} reports each individual threshold per group. For
#'   dichotomous (RM) data the two views are equivalent (one threshold per
#'   item).
#' @param output One of \code{"kable"} (default), \code{"dataframe"}, or
#'   \code{"ggplot"}. The data.frame view always carries a
#'   \code{Flagged} logical column; the kable view bolds flagged rows; the
#'   ggplot view shows confidence intervals.
#' @param cutoff Numeric or \code{NULL}. Threshold (in logits) for the
#'   \code{Flagged} column: a row is flagged when
#'   \code{MaxDiff > cutoff}, where \code{MaxDiff} is the difference
#'   between the largest and smallest per-group location for that item
#'   (or threshold). Set to \code{NULL} to suppress flagging. Default
#'   \code{0.5}.
#' @param conf Numeric in (0, 1). Confidence level used for the ggplot
#'   error bars. Default \code{0.95}.
#' @param sort Logical. \code{kable} output only: sort rows by
#'   \code{MaxDiff} (descending). Default \code{FALSE}.
#'
#' @return A data.frame, a \code{knitr_kable} object, or a \code{ggplot}
#'   object, depending on \code{output}.
#'
#'   The data.frame has one row per item (\code{level = "item"}) or per
#'   item x threshold (\code{level = "threshold"}), with columns
#'   \code{Item} (and \code{Threshold} at threshold level), one numeric
#'   column per group level, an \code{All} column for the unsplit fit,
#'   \code{MaxDiff}, \code{Flagged} (when \code{cutoff} is non-\code{NULL}),
#'   and matching \code{SE_*} columns.
#'
#'   The Andersen LR test result is attached as
#'   \code{attr(result, "lr_test")} on the data.frame, in the kable
#'   footnote, and in the ggplot caption (LR \eqn{\chi^2}, df, p-value).
#'
#' @details
#' For the data.frame and kable outputs, locations are reported on the
#' centred eRm parameterisation returned by \code{eRm::thresholds()}.
#' Per-group fits come from \code{eRm::LRtest(..., splitcr = dif_var)};
#' the unsplit fit (\code{All} column) is the model fitted to the full
#' dataset. The Andersen LR statistic, df, and p-value reported as the
#' \code{lr_test} attribute / caption come directly from
#' \code{LRtest()}'s return value.
#'
#' \code{cell_spec()}-style HTML cell colouring used in the legacy
#' easyRasch package has been dropped in favour of a logical
#' \code{Flagged} column (and bold rendering in the kable output), so the
#' kable renders correctly in HTML, LaTeX, and pipe/markdown.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("eRm", quietly = TRUE)) {
#'   set.seed(1)
#'   data("pcmdat2", package = "eRm")
#'   grp <- factor(sample(c("A", "B"), nrow(pcmdat2), replace = TRUE))
#'
#'   # Default: kable of per-group item locations
#'   RMdifLR(pcmdat2, dif_var = grp)
#'
#'   # ggplot panel of item locations with 95% CIs
#'   RMdifLR(pcmdat2, dif_var = grp, output = "ggplot")
#'
#'   # Threshold-level kable, sorted by MaxDiff
#'   RMdifLR(pcmdat2, dif_var = grp, level = "threshold", sort = TRUE)
#'
#'   # Tidy data.frame for downstream use
#'   df <- RMdifLR(pcmdat2, dif_var = grp, output = "dataframe")
#'   attr(df, "lr_test")
#'   df[df$Flagged, ]
#' }
#' }
#'
#' @importFrom rlang .data
#' @export
RMdifLR <- function(
  data,
  dif_var,
  model = c("auto", "PCM", "RM"),
  level = c("item", "threshold"),
  output = c("kable", "dataframe", "ggplot"),
  cutoff = 0.5,
  conf = 0.95,
  sort = FALSE
) {
  model <- match.arg(model)
  level <- match.arg(level)
  output <- match.arg(output)

  if (output == "ggplot" && !requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for output = \"ggplot\".",
      call. = FALSE
    )
  }
  if (output == "kable" && !requireNamespace("knitr", quietly = TRUE)) {
    stop("Package 'knitr' is required for output = \"kable\".", call. = FALSE)
  }
  if (!requireNamespace("eRm", quietly = TRUE)) {
    stop("Package 'eRm' is required for RMdifLR().", call. = FALSE)
  }

  if (!is.null(cutoff)) {
    if (
      !is.numeric(cutoff) ||
        length(cutoff) != 1L ||
        !is.finite(cutoff) ||
        cutoff < 0
    ) {
      stop(
        "`cutoff` must be a single non-negative numeric value, or NULL.",
        call. = FALSE
      )
    }
  }
  if (
    !is.numeric(conf) ||
      length(conf) != 1L ||
      !is.finite(conf) ||
      conf <= 0 ||
      conf >= 1
  ) {
    stop("`conf` must be a single numeric in (0, 1).", call. = FALSE)
  }

  validate_response_data(data)
  data <- as.data.frame(data)
  n_total_lr <- nrow(data)
  has_na_lr <- anyNA(data) || anyNA(dif_var)

  # ---------------------------------------------------------------------
  # Resolve dif_var and drop NAs (jointly with `data`)
  # ---------------------------------------------------------------------
  if (length(dif_var) != nrow(data)) {
    stop(
      "`dif_var` must have the same length as nrow(data) (",
      nrow(data),
      "). Got ",
      length(dif_var),
      ".",
      call. = FALSE
    )
  }
  dif_factor <- droplevels(as.factor(dif_var))

  na_mask <- is.na(dif_factor)
  if (any(na_mask)) {
    message(sum(na_mask), " row(s) with NA in `dif_var` dropped.")
    data <- data[!na_mask, , drop = FALSE]
    dif_factor <- droplevels(dif_factor[!na_mask])
  }

  # Respondents with no responses at all contribute nothing and make
  # eRm::LRtest() error; drop them jointly with `dif_factor` (the standard
  # .drop_empty_respondents() treatment, inlined to keep the rows aligned
  # with the grouping variable).
  empty <- rowSums(!is.na(as.matrix(data))) == 0L
  if (any(empty)) {
    message(sum(empty), " respondent(s) with no responses dropped.")
    data <- data[!empty, , drop = FALSE]
    dif_factor <- droplevels(dif_factor[!empty])
  }

  if (nlevels(dif_factor) < 2L) {
    stop(
      "`dif_var` must have at least 2 distinct non-missing levels.",
      call. = FALSE
    )
  }
  groups <- levels(dif_factor)
  nr_groups <- length(groups)

  # ---------------------------------------------------------------------
  # Pick the model
  # ---------------------------------------------------------------------
  data_mat <- as.matrix(data)
  storage.mode(data_mat) <- "integer"
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L

  resolved_model <- if (model == "auto") {
    if (is_polytomous) "PCM" else "RM"
  } else {
    if (model == "RM" && is_polytomous) {
      stop(
        "`model = \"RM\"` requires dichotomous (0/1) data; use ",
        "\"PCM\" or \"auto\" for polytomous responses.",
        call. = FALSE
      )
    }
    model
  }

  # ---------------------------------------------------------------------
  # Fit, run LRtest, extract per-group + overall thresholds and SEs
  # ---------------------------------------------------------------------
  fit_full <- if (resolved_model == "PCM") eRm::PCM(data) else eRm::RM(data)

  lrt <- tryCatch(
    eRm::LRtest(fit_full, splitcr = dif_factor),
    error = function(e) {
      stop(
        "eRm::LRtest() failed: ",
        conditionMessage(e),
        "\nThis often indicates an empty response category in one ",
        "subgroup. Inspect with RMplotTile(data, group = dif_var).",
        call. = FALSE
      )
    }
  )

  per_group <- extract_lr_locations(lrt, groups, names(data))
  overall <- extract_overall_locations(fit_full, names(data))

  # Combine into a long-form table: Item, Threshold, DIFgroup, Location, SE
  long_df <- rbind(per_group, overall)
  long_df$Item <- factor(long_df$Item, levels = names(data))

  # ---------------------------------------------------------------------
  # Aggregate to the requested level
  # ---------------------------------------------------------------------
  if (level == "item") {
    agg <- stats::aggregate(
      cbind(Location = long_df$Location, SE = long_df$SE),
      by = list(Item = long_df$Item, DIFgroup = long_df$DIFgroup),
      FUN = function(x) mean(x, na.rm = TRUE)
    )
    agg$Item <- factor(agg$Item, levels = names(data))
    table_df <- build_wide_table(agg, groups, level = "item")
    plot_df <- agg
    plot_df$Threshold <- NA_character_
  } else {
    table_df <- build_wide_table(long_df, groups, level = "threshold")
    plot_df <- long_df
  }

  # ---------------------------------------------------------------------
  # Flagged column
  # ---------------------------------------------------------------------
  if (!is.null(cutoff)) {
    table_df$Flagged <- !is.na(table_df$MaxDiff) &
      table_df$MaxDiff > cutoff
  }

  # ---------------------------------------------------------------------
  # Reorder columns: Item [Threshold] groups All MaxDiff [Flagged] SE_*
  # ---------------------------------------------------------------------
  id_cols <- if (level == "item") "Item" else c("Item", "Threshold")
  loc_cols <- c(
    groups,
    "All",
    "MaxDiff",
    if (!is.null(cutoff)) "Flagged" else NULL
  )
  se_cols <- paste0("SE_", c(groups, "All"))
  table_df <- table_df[, c(id_cols, loc_cols, se_cols), drop = FALSE]
  rownames(table_df) <- NULL

  # ---------------------------------------------------------------------
  # LR test summary (Andersen)
  # ---------------------------------------------------------------------
  lr_summary <- list(
    LR = unname(lrt$LR),
    df = unname(lrt$df),
    p_value = unname(lrt$pvalue),
    n_groups = nr_groups,
    groups = groups,
    model = resolved_model,
    n_persons = nrow(data),
    n_items = ncol(data)
  )
  attr(table_df, "lr_test") <- lr_summary

  # ---------------------------------------------------------------------
  # Dispatch on output
  # ---------------------------------------------------------------------
  if (output == "dataframe") {
    if (sort) {
      table_df <- table_df[order(-table_df$MaxDiff), , drop = FALSE]
      rownames(table_df) <- NULL
    }
    return(table_df)
  }

  if (output == "kable") {
    # Display rounding (3 dp; the dataframe output above stays unrounded)
    num_cols <- c(groups, "All", "MaxDiff", se_cols)
    for (cc in num_cols) {
      if (is.numeric(table_df[[cc]])) {
        table_df[[cc]] <- round(table_df[[cc]], 3)
      }
    }
    n_clause <- .n_caption(
      lr_summary$n_persons,
      n_total_lr,
      if (has_na_lr) "complete cases" else character()
    )
    return(render_lr_kable(
      table_df,
      lr_summary,
      level = level,
      cutoff = cutoff,
      sort = sort,
      is_polytomous = is_polytomous,
      n_clause = n_clause
    ))
  }

  # output == "ggplot"
  render_lr_ggplot(
    plot_df,
    lr_summary,
    level = level,
    groups = groups,
    conf = conf,
    is_polytomous = is_polytomous
  )
}

# =====================================================================
# Internal helpers
# =====================================================================

#' Extract per-group threshold locations + SEs from an LRtest fit
#'
#' Returns a long-format data.frame with columns
#' Item, Threshold, DIFgroup, Location, SE.
#'
#' @keywords internal
#' @noRd
extract_lr_locations <- function(lrt, groups, item_names) {
  parts <- lapply(seq_along(groups), function(g) {
    fit_g <- lrt$fitobj[[g]]
    one_fit_long(fit_g, dif_group = groups[g], item_names = item_names)
  })
  out <- do.call(rbind, parts)
  rownames(out) <- NULL
  out
}

#' Extract threshold locations + SEs from the unsplit fit
#'
#' @keywords internal
#' @noRd
extract_overall_locations <- function(fit, item_names) {
  one_fit_long(fit, dif_group = "All", item_names = item_names)
}

#' Pull thresholds + SEs out of one eRm `Rm` fit, as a long data.frame
#'
#' Both \code{eRm::PCM()} and \code{eRm::RM()} fits are supported via
#' \code{eRm::thresholds()}, which returns the centred threshold matrix
#' and matching SEs. For RM fits there is exactly one threshold per item.
#'
#' @keywords internal
#' @noRd
one_fit_long <- function(fit, dif_group, item_names) {
  is_rm <- isTRUE(fit$model == "RM")

  if (is_rm) {
    # eRm::thresholds() is polytomous-only. For RM the "threshold" is
    # the item difficulty, available as -betapar with matching se.beta.
    beta <- fit$betapar
    se <- fit$se.beta
    raw_names <- sub("^beta\\s+", "", names(beta))
    if (!all(raw_names %in% item_names)) {
      stop(
        "Could not match RM item parameter names to data columns: ",
        paste(utils::head(setdiff(raw_names, item_names), 5L), collapse = ", "),
        call. = FALSE
      )
    }
    return(data.frame(
      Item = raw_names,
      Threshold = "1",
      DIFgroup = dif_group,
      Location = unname(-as.numeric(beta)),
      SE = unname(as.numeric(se)),
      stringsAsFactors = FALSE
    ))
  }

  thr <- eRm::thresholds(fit)
  loc <- thr$threshpar
  se <- thr$se.thresh
  if (is.list(loc)) {
    loc <- unlist(loc, use.names = TRUE)
  }
  if (is.list(se)) {
    se <- unlist(se, use.names = TRUE)
  }

  parsed <- parse_threshold_names(names(loc), item_names)

  data.frame(
    Item = parsed$Item,
    Threshold = parsed$Threshold,
    DIFgroup = dif_group,
    Location = unname(as.numeric(loc)),
    SE = unname(as.numeric(se)),
    stringsAsFactors = FALSE
  )
}

#' Parse "beta Item.Tk" style names into Item / Threshold columns
#'
#' Tolerates name shapes seen across eRm versions:
#'   "beta I1.c1", "beta I1.T1", "I1.c1", "I1".
#'
#' @keywords internal
#' @noRd
parse_threshold_names <- function(nm, item_names) {
  bare <- sub("^(thresh\\s+)?beta\\s+", "", nm)

  # Match the longest item name as a prefix -- needed because item names
  # themselves may legitimately contain dots ("Q1.a").
  matched_item <- vapply(
    bare,
    function(b) {
      hits <- item_names[
        startsWith(b, item_names) |
          startsWith(b, paste0(item_names, "."))
      ]
      if (length(hits) == 0L) {
        NA_character_
      } else {
        hits[which.max(nchar(hits))]
      }
    },
    character(1L)
  )

  thr_part <- mapply(
    function(b, it) {
      if (is.na(it)) {
        return(NA_character_)
      }
      rest <- substr(b, nchar(it) + 1L, nchar(b))
      rest <- sub("^\\.", "", rest)
      if (!nzchar(rest)) "1" else rest
    },
    bare,
    matched_item,
    USE.NAMES = FALSE
  )

  if (any(is.na(matched_item))) {
    bad <- nm[is.na(matched_item)]
    stop(
      "Could not match threshold parameter name(s) to items: ",
      paste(utils::head(bad, 5L), collapse = ", "),
      if (length(bad) > 5L) ", ..." else "",
      call. = FALSE
    )
  }

  list(Item = unname(matched_item), Threshold = unname(thr_part))
}

#' Reshape long -> wide on DIFgroup, add MaxDiff, attach SE columns
#'
#' @keywords internal
#' @noRd
build_wide_table <- function(long_df, groups, level) {
  id_cols <- if (level == "item") "Item" else c("Item", "Threshold")

  loc_wide <- stats::reshape(
    long_df[, c(id_cols, "DIFgroup", "Location"), drop = FALSE],
    idvar = id_cols,
    timevar = "DIFgroup",
    direction = "wide"
  )
  names(loc_wide) <- sub("^Location\\.", "", names(loc_wide))

  se_wide <- stats::reshape(
    long_df[, c(id_cols, "DIFgroup", "SE"), drop = FALSE],
    idvar = id_cols,
    timevar = "DIFgroup",
    direction = "wide"
  )
  names(se_wide) <- sub("^SE\\.", "SE_", names(se_wide))

  out <- merge(loc_wide, se_wide, by = id_cols, sort = FALSE)

  # MaxDiff across the per-group columns (excludes "All")
  group_mat <- as.matrix(out[, groups, drop = FALSE])
  out$MaxDiff <- apply(group_mat, 1L, function(x) {
    x <- x[is.finite(x)]
    if (length(x) < 2L) NA_real_ else max(x) - min(x)
  })

  ord <- if (level == "threshold") {
    order(out$Item, out$Threshold)
  } else {
    order(out$Item)
  }
  out <- out[ord, , drop = FALSE]
  rownames(out) <- NULL
  out
}

#' Render the wide table as a kable
#'
#' @keywords internal
#' @noRd
render_lr_kable <- function(
  table_df,
  lr_summary,
  level,
  cutoff,
  sort,
  is_polytomous,
  n_clause = NULL
) {
  if (sort) {
    table_df <- table_df[order(-table_df$MaxDiff), , drop = FALSE]
    rownames(table_df) <- NULL
  }

  display <- table_df

  # Bold flagged rows by wrapping each visible cell in **...**
  if ("Flagged" %in% names(display)) {
    flag <- isTRUE_vec(display$Flagged)
    display$Flagged <- ifelse(flag, "yes", "no")
    if (any(flag)) {
      data_cols <- setdiff(
        names(display),
        if (level == "item") "Item" else c("Item", "Threshold")
      )
      for (cc in data_cols) {
        col <- display[[cc]]
        if (is.numeric(col)) {
          col <- format(col, nsmall = 3, trim = TRUE)
        }
        display[[cc]] <- ifelse(
          flag,
          paste0("**", col, "**"),
          as.character(col)
        )
      }
    }
  }

  caption <- paste0(
    if (is_polytomous) "Partial Credit Model" else "Rasch Model",
    " split by DIF variable (",
    lr_summary$n_groups,
    " groups). ",
    "Andersen LR \u03c7\u00b2 = ",
    round(lr_summary$LR, 3),
    ", df = ",
    lr_summary$df,
    ", ",
    .format_p(lr_summary$p_value),
    ". ",
    n_clause,
    ", ",
    lr_summary$n_items,
    " items.",
    if (!is.null(cutoff)) {
      paste0(" **Bold** = MaxDiff > ", cutoff, " logits.")
    } else {
      ""
    }
  )

  knitr::kable(display, format = "pipe", row.names = FALSE, caption = caption)
}

#' Coerce maybe-character flag column back to logical for rendering
#'
#' @keywords internal
#' @noRd
isTRUE_vec <- function(x) {
  if (is.logical(x)) {
    return(!is.na(x) & x)
  }
  vapply(x, isTRUE, logical(1L))
}

#' Render a ggplot panel of per-group locations with confidence bars
#'
#' @keywords internal
#' @noRd
render_lr_ggplot <- function(
  plot_df,
  lr_summary,
  level,
  groups,
  conf,
  is_polytomous
) {
  z <- stats::qnorm(1 - (1 - conf) / 2)

  plot_df$DIFgroup <- factor(plot_df$DIFgroup, levels = c(groups, "All"))
  group_only <- plot_df[plot_df$DIFgroup %in% groups, , drop = FALSE]
  all_only <- plot_df[plot_df$DIFgroup == "All", , drop = FALSE]

  group_only$DIFgroup <- droplevels(group_only$DIFgroup)
  diamond_x <- (length(groups) + 1) / 2 + 0.15

  caption <- er2_caption(paste0(
    "Andersen LR \u03c7\u00b2 = ",
    round(lr_summary$LR, 3),
    ", df = ",
    lr_summary$df,
    ", ",
    .format_p(lr_summary$p_value),
    ". Error bars: ",
    round(conf * 100),
    "% CI. ",
    "Black diamond = whole-sample location."
  ))

  if (level == "item") {
    p <- ggplot2::ggplot(
      group_only,
      ggplot2::aes(
        x = .data$DIFgroup,
        y = .data$Location,
        group = .data$Item,
        colour = .data$Item
      )
    ) +
      ggplot2::geom_line() +
      ggplot2::geom_point() +
      ggplot2::geom_errorbar(
        ggplot2::aes(
          ymin = .data$Location - z * .data$SE,
          ymax = .data$Location + z * .data$SE
        ),
        width = 0.1
      ) +
      ggplot2::geom_point(
        data = all_only,
        ggplot2::aes(x = diamond_x, y = .data$Location),
        shape = 18,
        colour = "black",
        size = 3,
        alpha = 0.6,
        inherit.aes = FALSE
      ) +
      ggplot2::facet_wrap(~Item) +
      ggplot2::labs(
        title = "DIF: item locations by group",
        subtitle = "Item locations are means of threshold locations",
        x = "DIF group",
        y = "Item location (logits)",
        caption = caption
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = "none",
        panel.spacing = ggplot2::unit(0.7, "cm")
      ) +
      er2_axis_margins() +
      er2_plot_caption()
    return(p)
  }

  # level == "threshold"
  p <- ggplot2::ggplot(
    group_only,
    ggplot2::aes(
      x = .data$DIFgroup,
      y = .data$Location,
      group = .data$Threshold,
      colour = .data$Threshold
    )
  ) +
    ggplot2::geom_line() +
    ggplot2::geom_point(alpha = 0.9) +
    ggplot2::geom_errorbar(
      ggplot2::aes(
        ymin = .data$Location - z * .data$SE,
        ymax = .data$Location + z * .data$SE
      ),
      width = 0.1
    ) +
    ggplot2::geom_errorbar(
      data = all_only,
      ggplot2::aes(
        x = diamond_x,
        ymin = .data$Location - z * .data$SE,
        ymax = .data$Location + z * .data$SE
      ),
      width = 0.1,
      colour = "darkgrey",
      inherit.aes = FALSE
    ) +
    ggplot2::geom_point(
      data = all_only,
      ggplot2::aes(x = diamond_x, y = .data$Location),
      shape = 18,
      colour = "black",
      size = 3,
      alpha = 0.6,
      inherit.aes = FALSE
    ) +
    ggplot2::facet_wrap(~Item) +
    ggplot2::labs(
      title = "DIF: threshold locations by group",
      x = "DIF group",
      y = "Threshold location (logits)",
      caption = caption
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      legend.position = "none",
      panel.spacing = ggplot2::unit(0.7, "cm")
    ) +
    er2_axis_margins() +
    er2_plot_caption()

  p
}

# Tests for RMdifLR()
#
# Covers:
#   - Input validation (data, dif_var, cutoff, conf, model)
#   - Auto-detect of model (PCM vs RM) by data shape
#   - Output structures: "dataframe", "kable", "ggplot"
#   - Flagged column / MaxDiff
#   - LR test attribute
#   - NA handling in dif_var
#   - sort = TRUE for kable

# ---------------------------------------------------------------------
# Helpers
# ---------------------------------------------------------------------
make_dichotomous <- function(n = 80, k = 5, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(
    sample(0:1, n * k, replace = TRUE),
    nrow = n,
    ncol = k
  ))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

make_polytomous <- function(n = 80, k = 5, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(
    sample(0:2, n * k, replace = TRUE),
    nrow = n,
    ncol = k
  ))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMdifLR errors when data has non-zero minimum", {
  df <- make_dichotomous() + 1L # min = 1
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))
  expect_error(RMdifLR(df, dif_var = grp), regexp = "scored starting at 0")
})

test_that("RMdifLR errors when dif_var length does not match nrow(data)", {
  df <- make_dichotomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df) - 1L))
  expect_error(RMdifLR(df, dif_var = grp), regexp = "same length as nrow")
})

test_that("RMdifLR errors when dif_var has fewer than 2 distinct levels", {
  df <- make_dichotomous()
  grp <- factor(rep("A", nrow(df)))
  expect_error(RMdifLR(df, dif_var = grp), regexp = "at least 2 distinct")
})

test_that("RMdifLR errors when model = 'RM' on polytomous data", {
  df <- make_polytomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))
  expect_error(
    RMdifLR(df, dif_var = grp, model = "RM"),
    regexp = "dichotomous"
  )
})

test_that("RMdifLR errors on invalid cutoff or conf", {
  df <- make_dichotomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))
  expect_error(RMdifLR(df, dif_var = grp, cutoff = -1), regexp = "non-negative")
  expect_error(
    RMdifLR(df, dif_var = grp, cutoff = c(0.1, 0.2)),
    regexp = "non-negative"
  )
  expect_error(RMdifLR(df, dif_var = grp, conf = 0), regexp = "in \\(0, 1\\)")
  expect_error(RMdifLR(df, dif_var = grp, conf = 1), regexp = "in \\(0, 1\\)")
})

# ---------------------------------------------------------------------
# Output structure (dataframe)
# ---------------------------------------------------------------------
test_that("RMdifLR output = 'dataframe' on dichotomous data picks RM and returns one row per item", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))

  out <- RMdifLR(df, dif_var = grp, output = "dataframe")

  expect_s3_class(out, "data.frame")
  expect_equal(nrow(out), ncol(df)) # one row per item
  expect_true(all(
    c("Item", "A", "B", "All", "MaxDiff", "SE_A", "SE_B", "SE_All") %in%
      names(out)
  ))
  expect_equal(as.character(out$Item), names(df))

  lr <- attr(out, "lr_test")
  expect_type(lr, "list")
  expect_true(all(c("LR", "df", "p_value", "model") %in% names(lr)))
  expect_equal(lr$model, "RM") # auto-picked
})

test_that("RMdifLR level = 'threshold' on polytomous data returns one row per item × threshold", {
  skip_if_not_installed("eRm")
  df <- make_polytomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))

  out <- RMdifLR(df, dif_var = grp, level = "threshold", output = "dataframe")

  expect_s3_class(out, "data.frame")
  expect_true(all(
    c("Item", "Threshold", "A", "B", "All", "MaxDiff") %in% names(out)
  ))
  # 0/1/2 data → 2 thresholds per item
  expect_equal(nrow(out), ncol(df) * 2L)
  expect_equal(attr(out, "lr_test")$model, "PCM") # auto-picked
})

# ---------------------------------------------------------------------
# Output structure (kable / ggplot)
# ---------------------------------------------------------------------
test_that("RMdifLR output = 'kable' returns a knitr_kable", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("knitr")
  df <- make_dichotomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))
  out <- RMdifLR(df, dif_var = grp, output = "kable")
  expect_s3_class(out, "knitr_kable")
})

test_that("RMdifLR defaults to kable (consistent with other kable/ggplot functions)", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("knitr")
  df <- make_dichotomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))
  expect_s3_class(RMdifLR(df, dif_var = grp), "knitr_kable")
})

test_that("RMdifLR output = 'ggplot' returns a ggplot", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_dichotomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))
  out <- RMdifLR(df, dif_var = grp, output = "ggplot")
  expect_s3_class(out, "ggplot")
})

# ---------------------------------------------------------------------
# Flagged column / MaxDiff
# ---------------------------------------------------------------------
test_that("Flagged column is logical and absent when cutoff = NULL", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))

  with_cutoff <- RMdifLR(df, dif_var = grp, output = "dataframe", cutoff = 0.5)
  expect_true("Flagged" %in% names(with_cutoff))
  expect_type(with_cutoff$Flagged, "logical")
  expect_true(all(with_cutoff$MaxDiff[with_cutoff$Flagged] > 0.5, na.rm = TRUE))

  no_cutoff <- RMdifLR(df, dif_var = grp, output = "dataframe", cutoff = NULL)
  expect_false("Flagged" %in% names(no_cutoff))
})

# ---------------------------------------------------------------------
# NA handling in dif_var
# ---------------------------------------------------------------------
test_that("RMdifLR drops NA rows in dif_var with a message", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))
  grp[c(1L, 5L, 9L)] <- NA

  expect_message(
    out <- RMdifLR(df, dif_var = grp, output = "dataframe"),
    regexp = "3 row\\(s\\) with NA"
  )
  expect_equal(attr(out, "lr_test")$n_persons, nrow(df) - 3L)
})

# ---------------------------------------------------------------------
# Sorting (kable)
# ---------------------------------------------------------------------
test_that("sort = TRUE in dataframe output sorts by MaxDiff descending", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))
  out <- RMdifLR(df, dif_var = grp, output = "dataframe", sort = TRUE)
  expect_true(all(diff(out$MaxDiff) <= 0))
})

# ---------------------------------------------------------------------
# p-value formatting in captions
# ---------------------------------------------------------------------
test_that("captions format p as plain numbers, never scientific notation", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_dichotomous()
  grp <- rep(c("A", "B"), length.out = nrow(df))

  p <- RMdifLR(df, dif_var = grp, output = "ggplot")
  expect_match(p$labels$caption, "p (=|<) 0?\\.\\d{3}")
  expect_no_match(p$labels$caption, "e-0")

  kbl <- RMdifLR(df, dif_var = grp)
  expect_match(paste(kbl, collapse = "\n"), "p (=|<) 0?\\.\\d{3}")
  expect_no_match(paste(kbl, collapse = "\n"), "e-0")
})

test_that(".format_p follows APA-style thresholds", {
  expect_equal(.format_p(0.0000497), "p < 0.001")
  expect_equal(.format_p(0.001), "p = 0.001")
  expect_equal(.format_p(0.004975), "p = 0.005")
  expect_equal(.format_p(0.71638), "p = 0.716")
  expect_equal(.format_p(NA_real_), "p = NA")
})

test_that("RMdifLR drops all-NA respondents instead of erroring", {
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 80 * 5, replace = TRUE), nrow = 80))
  colnames(df) <- paste0("i", 1:5)
  df[3, ] <- NA
  grp <- factor(rep(c("a", "b"), 40))

  expect_message(k <- RMdifLR(df, dif_var = grp), "no responses dropped")
  expect_s3_class(k, "knitr_kable")
})

#' Plot Distribution of Simulated \eqn{Q_3} Residual Correlations
#'
#' Visualises the distribution of simulation-based Yen's \eqn{Q_3} residual
#' correlations per item pair from \code{\link{RMlocdepQ3Cutoff}},
#' optionally overlaying observed \eqn{Q_3} values computed from real data via
#' \code{mirt::residuals(..., type = "Q3")}.
#'
#' Uses `ggdist::stat_dotsinterval()` (when `data` is not supplied) or
#' `ggdist::stat_dots()` (when `data` is supplied) with
#' `point_interval = "median_hdci"` and `.width = c(0.66, 0.95, 0.99)`.
#'
#' @param simfit The return value of \code{\link{RMlocdepQ3Cutoff}} (a list
#'   with components `pair_results`, `pair_cutoffs`, `actual_iterations`,
#'   `sample_n`, and `item_names`).
#' @param data Optional. A data.frame or matrix of item responses for
#'   computing and overlaying observed \eqn{Q_3} values. Items must be scored
#'   starting at 0 (non-negative integers). When provided, the plot
#'   includes orange diamond markers for the observed \eqn{Q_3} alongside the
#'   simulated distribution, plus segment summaries from the cutoff
#'   intervals.
#' @param items Optional character vector of item names to include in the
#'   plot. Only item pairs where **both** items are in this vector will be
#'   shown. When `NULL` (default), all item pairs are plotted.
#' @param n_pairs Optional positive integer. When supplied, only the
#'   `n_pairs` item pairs with the largest **deviation** from the simulated
#'   null are plotted, sorted by `|observed Q3 - median(simulated Q3 per
#'   pair)|` descending when `data` is supplied, or by
#'   `|median(simulated Q3 per pair)|` otherwise. Applied *after* the
#'   `items` filter when both are supplied. Values larger than the number
#'   of available pairs are silently capped.
#'
#' @return A named list of two `ggplot` objects (mirroring the `$matrix` /
#'   `$pairs` structure of \code{\link{RMlocdepQ3}}'s table output):
#'   \describe{
#'     \item{`$pairs`}{the per-pair plot described below (always returned).}
#'     \item{`$matrix`}{a lower-triangle tile heatmap of the **observed** \eqn{Q_3}
#'       matrix, with pairs above the global dynamic cut-off outlined. This
#'       needs the observed data, so it is `NULL` (with a message) when `data`
#'       is not supplied. When `items` is given, the heatmap is subset to those
#'       items; `n_pairs` does not apply to it.}
#'   }
#'
#' @details
#' The `$pairs` plot shows one row per item pair (labelled as "Item1 - Item2").
#' Only
#' the upper triangle of the \eqn{Q_3} matrix is plotted (pairs are unordered
#' under symmetric \eqn{Q_3}, unlike partial gamma which is direction-dependent).
#'
#' When `data` is **not** supplied, the function plots the simulated Q3
#' distributions as dot-interval plots using `ggdist::stat_dotsinterval()`
#' with median and Highest Density Continuous Interval (HDCI) summaries.
#'
#' When `data` **is** supplied, the function:
#' \enumerate{
#'   \item Computes observed \eqn{Q_3} residual correlations under the same
#'     estimator used to build `simfit` (its `$estimator`: CML/WLE by
#'     default, or MML via `mirt`).
#'   \item Overlays observed \eqn{Q_3} values as orange diamond markers on the
#'     simulated distributions.
#'   \item Shows per-pair cutoff intervals (from `simfit$pair_cutoffs`)
#'     as black line segments, with thicker segments for the 66\%
#'     interval and black dots for the median.
#' }
#'
#' The `ggplot2`, `ggdist`, `mirt`, and `scales` packages must be
#' installed (most are in Suggests, not Imports).
#'
#' @seealso \code{\link{RMlocdepQ3}}, \code{\link{RMlocdepQ3Cutoff}},
#'   \code{\link{RMlocdepGammaPlot}}
#'
#' @importFrom rlang .data
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE) &&
#'     requireNamespace("ggdist", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:10)
#'
#'   # Run simulation (use more iterations, e.g. 500+, in real analyses)
#'   cutoff_res <- RMlocdepQ3Cutoff(sim_data, iterations = 50,
#'                                  parallel = FALSE, seed = 42)
#'
#'   # Simulated distribution only
#'   RMlocdepQ3Plot(cutoff_res)
#'
#'   # With observed Q3 overlaid
#'   RMlocdepQ3Plot(cutoff_res, data = sim_data)
#'
#'   # Top 10 pairs by departure from null
#'   RMlocdepQ3Plot(cutoff_res, data = sim_data, n_pairs = 10)
#' }
#' }
RMlocdepQ3Plot <- function(simfit, data, items = NULL, n_pairs = NULL) {
  # --- Check required packages ------------------------------------------------
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for RMlocdepQ3Plot().", call. = FALSE)
  }
  if (!requireNamespace("ggdist", quietly = TRUE)) {
    stop("Package 'ggdist' is required for RMlocdepQ3Plot().", call. = FALSE)
  }
  if (!requireNamespace("scales", quietly = TRUE)) {
    stop("Package 'scales' is required for RMlocdepQ3Plot().", call. = FALSE)
  }

  # --- Validate simfit --------------------------------------------------------
  required_names <- c(
    "pair_results",
    "pair_cutoffs",
    "actual_iterations",
    "sample_n",
    "item_names"
  )
  missing_names <- setdiff(required_names, names(simfit))
  if (length(missing_names) > 0L) {
    stop(
      "`simfit` is missing required components: ",
      paste(missing_names, collapse = ", "),
      ".\nExpected the return value of RMlocdepQ3Cutoff() (this requires ",
      "easyRasch2 >= 0.7; rerun the cutoff function if you have an older ",
      "cached result).",
      call. = FALSE
    )
  }

  results_df <- simfit$pair_results
  pair_cutoffs <- simfit$pair_cutoffs
  actual_iterations <- simfit$actual_iterations
  sample_n <- simfit$sample_n
  item_names <- simfit$item_names
  # Match the estimator used to build the cut-off (older objects predate it).
  estimator <- if (is.null(simfit$estimator)) "CML" else simfit$estimator

  # Standard sample-size clause. The Q3 cutoff retains incomplete responses
  # (matching RMlocdepQ3); `sample_n_total` / `sample_has_na` are absent in
  # cutoff objects made by older versions, so fall back to the plain count.
  sample_clause <- .n_caption(
    sample_n,
    if (is.null(simfit$sample_n_total)) sample_n else simfit$sample_n_total,
    if (isTRUE(simfit$sample_has_na)) {
      "incomplete responses retained"
    } else {
      character()
    }
  )

  # --- Validate items parameter -----------------------------------------------
  if (!is.null(items)) {
    unknown_items <- setdiff(items, item_names)
    if (length(unknown_items) > 0L) {
      stop(
        "Unknown item(s) in `items`: ",
        paste(unknown_items, collapse = ", "),
        ".\nAvailable items: ",
        paste(item_names, collapse = ", "),
        call. = FALSE
      )
    }
    if (length(items) < 2L) {
      stop(
        "`items` must contain at least 2 item names to form a pair.",
        call. = FALSE
      )
    }
  }

  # --- Validate n_pairs parameter ---------------------------------------------
  if (!is.null(n_pairs)) {
    if (
      !is.numeric(n_pairs) ||
        length(n_pairs) != 1L ||
        !is.finite(n_pairs) ||
        n_pairs < 1 ||
        n_pairs != as.integer(n_pairs)
    ) {
      stop(
        "`n_pairs` must be a single positive integer or NULL.",
        call. = FALSE
      )
    }
    n_pairs <- as.integer(n_pairs)
  }

  # --- Create pair labels -----------------------------------------------------
  results_df$Pair <- paste(results_df$Item1, "-", results_df$Item2)
  pair_cutoffs$Pair <- paste(pair_cutoffs$Item1, "-", pair_cutoffs$Item2)

  # --- Filter to selected items -----------------------------------------------
  if (!is.null(items)) {
    keep <- results_df$Item1 %in% items & results_df$Item2 %in% items
    results_df <- results_df[keep, , drop = FALSE]
    keep_cut <- pair_cutoffs$Item1 %in% items & pair_cutoffs$Item2 %in% items
    pair_cutoffs <- pair_cutoffs[keep_cut, , drop = FALSE]

    if (nrow(results_df) == 0L) {
      stop("No item pairs remain after filtering by `items`.", call. = FALSE)
    }
  }

  # --- Compute observed Q3 (when data supplied) -------------------------------
  observed_df <- NULL
  if (!missing(data)) {
    if (estimator == "MML" && !requireNamespace("mirt", quietly = TRUE)) {
      stop(
        "Package 'mirt' is required to compute observed Q3 with ",
        "estimator = \"MML\" but is not installed.\n",
        "Install it with: install.packages(\"mirt\")",
        call. = FALSE
      )
    }
    validate_response_data(data)

    # Respondents with no responses at all break the CML fit inside
    # .q3_residual_matrix() (psychotools errors on all-NA rows); drop them,
    # as RMlocdepQ3() and RMlocdepQ3Cutoff() do.
    data <- .drop_empty_respondents(data)

    # Same estimator as the simulated cut-off (stored in simfit$estimator).
    q3_mat <- .q3_residual_matrix(data, estimator = estimator)
    item_names_q3 <- colnames(q3_mat)
    if (is.null(item_names_q3)) {
      item_names_q3 <- item_names
    }
    upper_idx <- which(upper.tri(q3_mat), arr.ind = TRUE)
    observed_df <- data.frame(
      Item1 = item_names_q3[upper_idx[, "row"]],
      Item2 = item_names_q3[upper_idx[, "col"]],
      observed_Q3 = q3_mat[upper_idx],
      stringsAsFactors = FALSE
    )
    observed_df$Pair <- paste(observed_df$Item1, "-", observed_df$Item2)

    if (!is.null(items)) {
      keep_obs <- observed_df$Item1 %in% items & observed_df$Item2 %in% items
      observed_df <- observed_df[keep_obs, , drop = FALSE]
    }
  }

  # --- Apply n_pairs filter (rank by |observed - median(sim)| or |median|) ---
  if (!is.null(n_pairs)) {
    # Per-pair simulated median (used by either ranking metric below)
    pair_names_all <- unique(results_df$Pair)
    med_sim <- vapply(
      pair_names_all,
      function(pp) {
        stats::median(results_df$Q3[results_df$Pair == pp], na.rm = TRUE)
      },
      numeric(1L)
    )
    names(med_sim) <- pair_names_all

    if (!is.null(observed_df)) {
      # Rank by |observed Q3 − median(simulated Q3 per pair)|
      obs_lookup <- stats::setNames(observed_df$observed_Q3, observed_df$Pair)
      deviation <- abs(obs_lookup[pair_names_all] - med_sim[pair_names_all])
    } else {
      # Rank by |median simulated Q3 per pair|
      deviation <- abs(med_sim[pair_names_all])
    }
    ord <- order(deviation, decreasing = TRUE)
    keep_n <- min(n_pairs, length(pair_names_all))
    keep_pairs <- pair_names_all[ord[seq_len(keep_n)]]

    results_df <- results_df[results_df$Pair %in% keep_pairs, , drop = FALSE]
    pair_cutoffs <- pair_cutoffs[
      pair_cutoffs$Pair %in% keep_pairs,
      ,
      drop = FALSE
    ]
    if (!is.null(observed_df)) {
      observed_df <- observed_df[
        observed_df$Pair %in% keep_pairs,
        ,
        drop = FALSE
      ]
    }
    # Y-axis order: largest deviation at the top
    pair_levels <- rev(keep_pairs)
  } else {
    pair_levels <- rev(unique(results_df$Pair))
  }

  # --- Per-pair summary intervals for segment overlays ------------------------
  pair_names <- unique(results_df$Pair)
  lo_hi <- do.call(
    rbind,
    lapply(pair_names, function(pair) {
      sub <- results_df[results_df$Pair == pair, ]
      data.frame(
        Pair = pair,
        min_Q3 = stats::quantile(sub$Q3, 0.005, na.rm = TRUE),
        max_Q3 = stats::quantile(sub$Q3, 0.995, na.rm = TRUE),
        p66lo_Q3 = stats::quantile(sub$Q3, 0.167, na.rm = TRUE),
        p66hi_Q3 = stats::quantile(sub$Q3, 0.833, na.rm = TRUE),
        median_Q3 = stats::median(sub$Q3, na.rm = TRUE),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    })
  )
  rownames(lo_hi) <- NULL

  # --- Case 1: no observed data, show simulation distribution only ------------
  if (missing(data)) {
    results_plot <- data.frame(
      Pair = results_df$Pair,
      Value = results_df$Q3,
      stringsAsFactors = FALSE
    )
    results_plot$Pair <- factor(results_plot$Pair, levels = pair_levels)

    p <- ggplot2::ggplot(
      results_plot,
      ggplot2::aes(x = .data$Value, y = .data$Pair)
    ) +
      ggdist::stat_dotsinterval(
        ggplot2::aes(slab_fill = ggplot2::after_stat(.data$level)),
        quantiles = actual_iterations,
        point_interval = "median_hdci",
        layout = "weave",
        slab_color = NA,
        .width = c(0.66, 0.95, 0.99)
      ) +
      ggplot2::geom_vline(
        xintercept = 0,
        linetype = "dashed",
        color = "grey50",
        linewidth = 0.4
      ) +
      ggplot2::labs(
        x = "Q3 residual correlation",
        y = "Item pair",
        caption = er2_caption(paste0(
          "Results from ",
          actual_iterations,
          " simulated datasets (no true local dependence). ",
          sample_clause,
          " per dataset."
        ))
      ) +
      ggplot2::scale_color_manual(
        values = scales::brewer_pal()(3),
        aesthetics = "slab_fill",
        guide = "none"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(panel.spacing = ggplot2::unit(0.7, "cm")) +
      er2_axis_margins() +
      er2_plot_caption()

    # The $matrix tile heatmap needs the observed Q3 matrix, which requires
    # `data`; without it only the per-pair distribution ($pairs) is available.
    message(
      "The Q3 tile heatmap ($matrix) requires `data`; ",
      "returning $matrix = NULL."
    )
    return(list(matrix = NULL, pairs = p))
  }

  # --- Case 2: observed data supplied -----------------------------------------
  Q3_sim <- data.frame(
    Pair = results_df$Pair,
    Value = results_df$Q3,
    stringsAsFactors = FALSE
  )
  Q3_sim <- merge(
    Q3_sim,
    observed_df[, c("Pair", "observed_Q3")],
    by = "Pair",
    sort = FALSE
  )
  Q3_sim$Pair <- factor(Q3_sim$Pair, levels = pair_levels)

  lo_hi$Pair_f <- factor(lo_hi$Pair, levels = pair_levels)

  caption_text <- er2_caption(paste0(
    "Results from ",
    actual_iterations,
    " simulated datasets. ",
    sample_clause,
    " per dataset.\n",
    "Orange diamonds indicate observed Q3 residual correlation. ",
    "Black dots indicate median Q3 from simulations."
  ))

  p <- ggplot2::ggplot(
    Q3_sim,
    ggplot2::aes(x = .data$Value, y = .data$Pair)
  ) +
    ggdist::stat_dots(
      ggplot2::aes(slab_fill = ggplot2::after_stat(.data$level)),
      quantiles = actual_iterations,
      layout = "weave",
      slab_color = NA,
      .width = c(0.66, 0.95, 0.99)
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$min_Q3,
        xend = .data$max_Q3,
        y = .data$Pair_f,
        yend = .data$Pair_f
      ),
      color = "black",
      linewidth = 0.7
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$p66lo_Q3,
        xend = .data$p66hi_Q3,
        y = .data$Pair_f,
        yend = .data$Pair_f
      ),
      color = "black",
      linewidth = 1.2
    ) +
    ggplot2::geom_point(
      data = lo_hi,
      ggplot2::aes(
        x = .data$median_Q3,
        y = .data$Pair_f
      ),
      size = 3.6
    ) +
    ggplot2::geom_point(
      ggplot2::aes(x = .data$observed_Q3),
      color = "sienna2",
      shape = 18,
      position = ggplot2::position_nudge(y = -0.1),
      size = 4
    ) +
    ggplot2::geom_vline(
      xintercept = 0,
      linetype = "dashed",
      color = "grey50",
      linewidth = 0.4
    ) +
    ggplot2::labs(
      x = "Q3 residual correlation",
      y = "Item pair",
      caption = caption_text
    ) +
    ggplot2::scale_color_manual(
      values = scales::brewer_pal()(3),
      aesthetics = "slab_fill",
      guide = "none"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(panel.spacing = ggplot2::unit(0.7, "cm")) +
    er2_axis_margins() +
    er2_plot_caption()

  # --- $matrix: tile heatmap of the observed Q3 matrix ------------------------
  # Same heatmap earlier versions returned from RMlocdepQ3()$plot, now here so
  # the table and plot outputs share the $matrix / $pairs structure. The global
  # dynamic cut-off mirrors RMlocdepQ3(): mean off-diagonal Q3 + suggested
  # cut-off. The `items` filter subsets the matrix; `n_pairs` does not apply.
  q3_tile <- if (is.null(items)) q3_mat else q3_mat[items, items, drop = FALSE]
  dyn_cutoff <- if (!is.null(simfit$suggested_cutoff)) {
    mean(q3_tile, na.rm = TRUE) + as.numeric(simfit$suggested_cutoff)
  } else {
    NULL
  }
  matrix_plot <- .q3_tile_plot(
    q3_tile,
    dyn_cutoff = dyn_cutoff,
    estimator = estimator,
    actual_iterations = actual_iterations
  )

  list(matrix = matrix_plot, pairs = p)
}

#' Person Locations for a Rasch / Partial Credit Model
#'
#' Estimates a person location (theta) and its standard error of
#' measurement (SEM) for every respondent. Estimation is performed
#' directly on each person's observed response pattern, so partial
#' missingness is handled correctly: two respondents with the same sum
#' score on different subsets of items receive different estimates. This
#' avoids the sum-score lookup used by [RMscoreSE()], which assumes
#' complete data.
#'
#' @param data A data.frame or matrix of item responses. Items must be
#'   scored starting at 0 (non-negative integers). Missing values (`NA`)
#'   are allowed.
#' @param method Character. `"WLE"` (default) for Warm's Weighted
#'   Likelihood Estimate (lower bias than MLE; Warm, 1989), or `"EAP"`
#'   for the Expected A Posteriori estimate under a normal prior.
#' @param item_params Optional pre-specified item parameters, e.g. for
#'   anchoring or equating. Either a named list of Andrich-threshold
#'   vectors (one per item) or the long-format data.frame returned by
#'   [RMitemParameters()]. When `NULL` (default) item parameters are
#'   estimated from `data`.
#' @param estimator Character. How item parameters are estimated when
#'   `item_params` is `NULL`: `"CML"` (default, via \pkg{psychotools}) or
#'   `"MML"` (via \pkg{mirt}). Ignored when `item_params` is supplied.
#' @param theta_range Numeric length 2. Search range for the WLE root
#'   and bounds for the EAP quadrature grid. Default `c(-10, 10)`. Only
#'   if the WLE root falls outside this range is the boundary returned.
#' @param prior_mean Numeric. Mean of the normal prior used by
#'   `method = "EAP"`. Default `0`, the natural choice when item
#'   parameters are centred at mean difficulty zero. Ignored for WLE.
#' @param prior_sd Numeric or `NULL`. Standard deviation of the normal
#'   prior used by `method = "EAP"`. When `NULL` (default) it is
#'   estimated from the data by marginal maximum likelihood, holding the
#'   item parameters fixed. Supply a number (e.g. `1` for a standard
#'   `N(0, 1)` prior, or a value carried over from a reference sample
#'   for equating) to fix it. Ignored for WLE.
#' @param output Character. `"kable"` (default) for a formatted
#'   [knitr::kable()] table, `"dataframe"` for the underlying
#'   data.frame, `"ggplot"` for a histogram of the estimated person
#'   locations, or `"file"` to write the data.frame to a CSV at
#'   `filename` (the data.frame is also returned invisibly).
#' @param filename Character. Path to the CSV file to write when
#'   `output = "file"`. Required in that case; ignored otherwise.
#'
#' @return
#' For `output = "dataframe"`, a data.frame with one row per respondent
#' (in input order; respondents with no responses are dropped with a
#' message) and columns `theta` (the person location), `sem`
#' (standard error of measurement), `sum_score`, `n_answered` (number of
#' non-missing responses), and `extreme` (logical: a minimum or maximum
#' possible score given the items answered). For `output = "kable"`, the
#' same content as a `knitr_kable` object. For `output = "ggplot"`, a
#' histogram of `theta`.
#'
#' @details
#' Item parameters are obtained once (by CML or MML, or taken from
#' `item_params`) and treated as fixed. Each person's theta is then
#' estimated from the items they answered.
#'
#' \strong{WLE} solves Warm's weighted-likelihood score equation
#' `l'(theta) + J(theta) / (2 I(theta)) = 0` by bracketed root finding,
#' where the bias-correction term `J / (2 I)` keeps the equation solvable
#' at the boundaries. The SEM is `1 / sqrt(I(theta))`. Unlike the MLE,
#' Warm's estimator therefore yields *finite* locations for the minimum
#' and maximum possible scores (a sensible step beyond the next-most
#' extreme score rather than +/-Inf; cf. Warm, 1989), matching
#' established implementations such as \pkg{catR} and \pkg{TAM}. Such
#' scores are still marked in the `extreme` column because they are
#' extrapolated and carry large standard errors. Only if the root lies
#' outside `theta_range` is the boundary returned with `NA` SEM.
#'
#' \strong{EAP} integrates the pattern likelihood against a normal prior
#' over a quadrature grid; the estimate is the posterior mean and the
#' SEM is the posterior standard deviation. Unlike WLE, EAP is finite at
#' extreme scores because the prior shrinks them inward, at the cost of
#' depending on the assumed prior. The prior actually used -- including
#' the marginal-ML estimate of `prior_sd` when it is left `NULL` -- is
#' reported in the kable caption and attached to the result as
#' `attr(result, "prior")`.
#'
#' @references
#' Warm, T. A. (1989). Weighted likelihood estimation of ability in item
#' response theory. *Psychometrika, 54*(3), 427-450.
#' \doi{10.1007/BF02294627}
#'
#' Bock, R. D., & Mislevy, R. J. (1982). Adaptive EAP estimation of
#' ability in a microcomputer environment. *Applied Psychological
#' Measurement, 6*(4), 431-444. \doi{10.1177/014662168200600405}
#'
#' Magis, D. (2015). A note on weighted likelihood and Jeffreys modal
#' estimation of proficiency levels in polytomous item response models.
#' *Psychometrika, 80*(1), 200-204. \doi{10.1007/s11336-013-9378-5}
#'
#' @seealso [RMitemParameters()], [RMscoreSE()], [RMreliability()]
#'
#' @importFrom rlang .data
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' dat <- as.data.frame(
#'   matrix(sample(0:2, 200 * 6, replace = TRUE), nrow = 200, ncol = 6)
#' )
#' colnames(dat) <- paste0("Item", 1:6)
#' # Introduce some missingness
#' dat[cbind(sample(200, 30), sample(6, 30, replace = TRUE))] <- NA
#'
#' # Default: WLE person locations
#' RMpersonParameters(dat, output = "dataframe") |> head()
#'
#' # EAP with a data-estimated prior SD
#' eap <- RMpersonParameters(dat, method = "EAP", output = "dataframe")
#' attr(eap, "prior")
#'
#' # EAP with a fixed N(0, 1) prior
#' RMpersonParameters(dat, method = "EAP", prior_sd = 1, output = "dataframe") |>
#'   head()
#'
#' # Write the person-location table to a CSV (also returned invisibly)
#' RMpersonParameters(dat, output = "file",
#'                    filename = tempfile(fileext = ".csv"))
#' }
RMpersonParameters <- function(
  data,
  method = c("WLE", "EAP"),
  item_params = NULL,
  estimator = c("CML", "MML"),
  theta_range = c(-10, 10),
  prior_mean = 0,
  prior_sd = NULL,
  output = c("kable", "dataframe", "ggplot", "file"),
  filename = NULL
) {
  method <- match.arg(method)
  estimator <- match.arg(estimator)
  output <- match.arg(output)

  if (output == "file") {
    .validate_filename(filename)
  }

  validate_response_data(data)

  if (
    !is.numeric(theta_range) ||
      length(theta_range) != 2L ||
      theta_range[1L] >= theta_range[2L]
  ) {
    stop(
      "`theta_range` must be a numeric vector of length 2 with ",
      "theta_range[1] < theta_range[2].",
      call. = FALSE
    )
  }

  data <- as.data.frame(data)
  n_total <- nrow(data)
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)
  n_used <- nrow(data)
  n_clause <- paste0(
    " ",
    .n_caption(
      n_used,
      n_total,
      if (has_na) "incomplete responses retained" else character()
    ),
    "."
  )

  # --- Item parameters (estimated or supplied) --------------------------------
  if (is.null(item_params)) {
    fit <- if (estimator == "CML") {
      .rasch_fit_cml(data, se = FALSE)
    } else {
      .rasch_fit_mml(data, se = FALSE)
    }
    # Standard Rasch identification (mean threshold zero), so locations
    # match RMscoreSE() and RMitemParameters().
    thr_list <- .center_thresholds(fit$thr_list)
  } else {
    thr_list <- .coerce_item_params(item_params)
  }

  data <- .align_items(data, thr_list)
  data_mat <- as.matrix(data)

  # --- Person-level bookkeeping -----------------------------------------------
  n_answered <- rowSums(!is.na(data_mat))
  sum_score <- rowSums(data_mat, na.rm = TRUE)
  max_score <- vapply(
    seq_len(nrow(data_mat)),
    function(p) {
      ans <- !is.na(data_mat[p, ])
      sum(vapply(thr_list[ans], length, integer(1)))
    },
    numeric(1)
  )
  extreme <- n_answered > 0L & (sum_score == 0 | sum_score == max_score)

  # --- Estimation (shared engine: see .estimate_thetas() in utils-theta.R) -----
  prior_used <- NULL
  if (method == "WLE") {
    est <- .estimate_thetas(
      data_mat,
      thr_list,
      method = "WLE",
      theta_range = theta_range
    )
    theta <- est$theta
    sem <- est$sem
  } else {
    if (
      !is.null(prior_sd) &&
        (!is.numeric(prior_sd) || length(prior_sd) != 1L || prior_sd <= 0)
    ) {
      stop(
        "`prior_sd` must be a single positive number or NULL.",
        call. = FALSE
      )
    }
    est <- .estimate_thetas(
      data_mat,
      thr_list,
      method = "EAP",
      theta_range = theta_range,
      prior_mean = prior_mean,
      prior_sd = prior_sd
    )
    theta <- est$theta
    sem <- est$sem
    prior_used <- c(
      mean = prior_mean,
      sd = attr(est, "prior_sd"),
      estimated = as.numeric(is.null(prior_sd))
    )
  }

  out <- data.frame(
    theta = theta,
    sem = sem,
    sum_score = sum_score,
    n_answered = n_answered,
    extreme = extreme,
    stringsAsFactors = FALSE
  )
  rownames(out) <- rownames(data)
  if (!is.null(prior_used)) {
    attr(out, "prior") <- prior_used
  }

  # --- Output -----------------------------------------------------------------
  if (output %in% c("dataframe", "file")) {
    # Unrounded: rounding is a display concern (the kable path rounds a
    # display copy below).
    if (output == "file") {
      # row.names = TRUE preserves respondent identifiers (the person kable
      # also shows them), which live in the data.frame's row names.
      return(.write_output_csv(out, filename, row.names = TRUE))
    }
    return(out)
  }

  if (output == "ggplot") {
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
      stop(
        "Package 'ggplot2' is required for output = \"ggplot\". ",
        "Install it with: install.packages(\"ggplot2\")",
        call. = FALSE
      )
    }
    plot_df <- out[is.finite(out$theta), , drop = FALSE]
    return(
      ggplot2::ggplot(plot_df, ggplot2::aes(x = .data$theta)) +
        ggplot2::geom_histogram(bins = 30, fill = "grey70", colour = "white") +
        ggplot2::labs(
          x = "Person location (logits)",
          y = "Count",
          caption = er2_caption(trimws(n_clause))
        ) +
        ggplot2::theme_bw() +
        er2_axis_margins() +
        er2_plot_caption()
    )
  }

  display <- out
  display$theta <- round(display$theta, 3)
  display$sem <- round(display$sem, 3)
  caption <- if (method == "WLE") {
    paste0(
      "Person locations via Warm's WLE (",
      estimator,
      " item parameters). Extreme scores receive finite extrapolated ",
      "estimates (flagged in the extreme column).",
      n_clause
    )
  } else {
    paste0(
      "Person locations via EAP (",
      estimator,
      " item parameters; normal prior mean = ",
      round(prior_used[["mean"]], 3),
      ", sd = ",
      round(prior_used[["sd"]], 3),
      if (prior_used[["estimated"]] == 1) " [estimated]" else " [fixed]",
      ").",
      n_clause
    )
  }
  knitr::kable(display, format = "pipe", caption = caption, row.names = TRUE)
}

# ---------------------------------------------------------------------
# Internal: coerce supplied item parameters to a threshold list
# ---------------------------------------------------------------------

#' @keywords internal
#' @noRd
.coerce_item_params <- function(item_params) {
  if (is.list(item_params) && !is.data.frame(item_params)) {
    thr_list <- lapply(item_params, function(x) as.numeric(x))
    if (is.null(names(thr_list))) {
      stop(
        "When `item_params` is a list it must be named by item.",
        call. = FALSE
      )
    }
    return(thr_list)
  }
  if (is.data.frame(item_params)) {
    if (!all(c("item", "location") %in% names(item_params))) {
      stop(
        "`item_params` data.frame must have `item` and `location` ",
        "columns (as returned by RMitemParameters(format = \"long\")).",
        call. = FALSE
      )
    }
    sp <- split(
      item_params$location,
      factor(item_params$item, levels = unique(item_params$item))
    )
    return(lapply(sp, as.numeric))
  }
  stop(
    "`item_params` must be a named list of threshold vectors or a ",
    "data.frame from RMitemParameters().",
    call. = FALSE
  )
}

#' Align response-data columns with the item-parameter list
#'
#' @param data Response data.frame.
#' @param thr_list Named (or order-matched) list of threshold vectors.
#' @return `data` with columns ordered/subset to match `thr_list`.
#' @keywords internal
#' @noRd
.align_items <- function(data, thr_list) {
  if (!is.null(names(thr_list)) && all(names(thr_list) %in% colnames(data))) {
    return(data[, names(thr_list), drop = FALSE])
  }
  if (length(thr_list) != ncol(data)) {
    stop(
      "Number of items in `item_params` (",
      length(thr_list),
      ") does not match the number of columns in `data` (",
      ncol(data),
      "), and names do not align.",
      call. = FALSE
    )
  }
  data
}

# Tests for RMscoreSE()

make_polytomous <- function(n = 200, k = 5, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:3, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

make_dichotomous <- function(n = 150, k = 8, seed = 2L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMscoreSE errors when data has non-zero minimum", {
  df <- make_dichotomous() + 1L
  expect_error(RMscoreSE(df), regexp = "scored starting at 0")
})

# ---------------------------------------------------------------------
# Output structures
# ---------------------------------------------------------------------
test_that("RMscoreSE output = 'dataframe' returns one row per raw score", {
  skip_if_not_installed("eRm")
  df  <- make_polytomous()
  res <- RMscoreSE(df, output = "dataframe")
  expect_s3_class(res, "data.frame")
  # k = 5, max category = 3 -> max raw score = 15 -> 16 rows
  expect_equal(nrow(res), ncol(df) * 3L + 1L)
  expect_named(res, c("raw_score", "logit_score", "logit_se"))
  expect_equal(res$raw_score, 0:15)
})

test_that("RMscoreSE WLE on dichotomous data produces valid table", {
  skip_if_not_installed("eRm")
  df  <- make_dichotomous()
  res <- RMscoreSE(df, method = "WLE", output = "dataframe")
  expect_equal(nrow(res), ncol(df) + 1L)
  expect_true(all(is.finite(res$logit_score[2:(ncol(df))])))
})

test_that("RMscoreSE output = 'kable' returns knitr_kable", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("knitr")
  df <- make_polytomous()
  out <- RMscoreSE(df, output = "kable")
  expect_s3_class(out, "knitr_kable")
})

test_that("RMscoreSE output = 'ggplot' returns a ggplot", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p  <- RMscoreSE(df, output = "ggplot")
  expect_s3_class(p, "ggplot")
})

# WLE results must agree with RMpersonParameters (shared engine + scale)
test_that("RMscoreSE WLE matches RMpersonParameters (point and SE)", {
  skip_if_not_installed("eRm")
  for (df in list(make_dichotomous(), make_polytomous())) {
    ss <- RMscoreSE(df, method = "WLE", output = "dataframe")
    pp <- RMpersonParameters(df, output = "dataframe")
    ag <- aggregate(cbind(theta, sem) ~ sum_score, data = pp, FUN = mean)
    m  <- merge(ss, ag, by.x = "raw_score", by.y = "sum_score")
    expect_equal(m$logit_score, m$theta, tolerance = 1e-3)
    expect_equal(m$logit_se,    m$sem,   tolerance = 1e-3)
  }
})

test_that("RMscoreSE WLE SE equals the information-based 1/sqrt(I)", {
  skip_if_not_installed("eRm")
  df  <- make_dichotomous()
  res <- RMscoreSE(df, method = "WLE", output = "dataframe")
  beta <- -as.numeric(eRm::RM(df)$betapar)       # difficulties
  for (i in which(res$raw_score %in% 1:(ncol(df) - 1L))) {
    th   <- res$logit_score[i]
    info <- sum(stats::plogis(th - beta) * (1 - stats::plogis(th - beta)))
    expect_equal(res$logit_se[i], 1 / sqrt(info), tolerance = 1e-3)
  }
})

# EAP method goes through mirt; skip if not available
test_that("RMscoreSE method = 'EAP' produces a valid table when mirt is installed", {
  skip_if_not_installed("mirt")
  df  <- make_polytomous()
  res <- RMscoreSE(df, method = "EAP", output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_named(res, c("raw_score", "logit_score", "logit_se"))
  expect_true(nrow(res) > 0L)
})

#' Item Response Distribution Bar Chart
#'
#' Creates a faceted bar chart showing the response distribution for each
#' item, with counts and percentages displayed on each bar. Each item gets
#' its own panel, with response categories on the x-axis and percentage of
#' responses on the y-axis. This is a descriptive data visualization tool
#' intended for use before model fitting.
#'
#' @param data A data.frame in wide format containing only the item
#'   response columns. Each column is one item, each row is one person.
#'   All columns must be numeric (integer-valued). Response categories
#'   may be coded starting from 0 or 1. Do not include person IDs,
#'   grouping variables, or other non-item columns.
#' @param item_labels An optional character vector of descriptive labels
#'   for the items (facet strips). Must be the same length as
#'   `ncol(data)`. If `NULL` (the default), column names are used.
#'   Labels are displayed as `"column_name - label"`.
#' @param category_labels An optional character vector of labels for the
#'   response categories (x-axis). Must be the same length as the number
#'   of response categories spanning from the minimum to the maximum
#'   observed value. If `NULL` (the default), numeric category values
#'   are used.
#' @param ncol Integer. Number of columns in the faceted layout.
#'   Default is `1L`.
#' @param label_wrap Integer. Number of characters per line in facet
#'   strip labels before wrapping. Default is `25L`.
#' @param text_y Numeric. Vertical position (in percent units) for the
#'   count labels on each bar. Adjust upward if bars are tall. Default
#'   is `6`.
#' @param viridis_option Character. Viridis palette option for the
#'   count-text colour. One of `"A"` through `"H"`. Default is `"A"`.
#' @param viridis_end Numeric in \eqn{[0, 1]}. End point of the viridis
#'   colour scale for count text. Adjust if text is hard to read against
#'   the bar colours. Default is `0.9`.
#' @param font Character. Font family for all text. Default is
#'   `"sans"`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @details
#' Each item is displayed as a separate facet panel with the item label
#' in the strip on the left side. Bars are coloured by response category
#' using the viridis palette. Each bar shows the count (`n = X`) as text.
#'
#' The plot caption reports the sample in the standard
#' `n = X respondents (policy)` form; item-level `NA`s are retained --
#' each bar counts the non-missing responses for that item.
#'
#' **Input requirements:**
#' \itemize{
#'   \item All columns must be numeric (integer-valued).
#'   \item The data frame must contain at least 2 columns (items)
#'     and at least 1 row (person).
#' }
#'
#' @seealso [RMplotStackedbar()], [RMplotTile()]
#'
#' @examples
#' if (requireNamespace("eRm", quietly = TRUE) &&
#'     requireNamespace("ggplot2", quietly = TRUE)) {
#'   data(pcmdat2, package = "eRm")
#'
#'   # Basic response distribution plot
#'   RMplotBar(pcmdat2)
#'
#'   # With custom item labels
#'   RMplotBar(
#'     pcmdat2,
#'     item_labels = c("Mood", "Sleep", "Appetite", "Energy")
#'   )
#'
#'   # Two-column layout with wrapped labels
#'   RMplotBar(
#'     pcmdat2,
#'     item_labels = c(
#'       "General mood and emotional wellbeing",
#'       "Quality of sleep at night",
#'       "Appetite and eating habits",
#'       "Overall energy level during the day"
#'     ),
#'     ncol = 2, label_wrap = 20
#'   )
#'
#'   # With custom category labels
#'   RMplotBar(
#'     pcmdat2,
#'     category_labels = c("Never", "Sometimes", "Often")
#'   )
#' }
#'
#' @importFrom rlang .data
#' @export
RMplotBar <- function(
  data,
  item_labels = NULL,
  category_labels = NULL,
  ncol = 1L,
  label_wrap = 25L,
  text_y = 6,
  viridis_option = "A",
  viridis_end = 0.9,
  font = "sans"
) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for RMplotBar().", call. = FALSE)
  }

  # ---------------------------------------------------------------------
  # Input validation
  # ---------------------------------------------------------------------
  if (!is.data.frame(data)) {
    stop(
      "`data` must be a data.frame in wide format ",
      "(one column per item, one row per person).",
      call. = FALSE
    )
  }
  if (ncol(data) < 2L) {
    stop(
      "`data` must contain at least 2 columns (items). Found ",
      ncol(data),
      " column(s).",
      call. = FALSE
    )
  }
  if (nrow(data) < 1L) {
    stop(
      "`data` must contain at least 1 row (person). Found 0 rows.",
      call. = FALSE
    )
  }

  non_numeric <- names(data)[!vapply(data, is.numeric, logical(1L))]
  if (length(non_numeric) > 0L) {
    stop(
      "All columns must be numeric. Non-numeric column(s): ",
      paste(non_numeric, collapse = ", "),
      ". Remove non-item columns (e.g., person IDs, grouping ",
      "variables) before passing to RMplotBar().",
      call. = FALSE
    )
  }

  all_vals <- unlist(data, use.names = FALSE)
  all_vals <- all_vals[!is.na(all_vals)]
  if (length(all_vals) == 0L) {
    stop("`data` contains only NA values.", call. = FALSE)
  }
  if (!all(all_vals == round(all_vals))) {
    stop(
      "All response values must be integers. Found non-integer values.",
      call. = FALSE
    )
  }

  item_names <- names(data)

  if (!is.null(item_labels) && length(item_labels) != length(item_names)) {
    stop(
      "`item_labels` must have the same length as the number of ",
      "columns in `data`. Expected ",
      length(item_names),
      ", got ",
      length(item_labels),
      ".",
      call. = FALSE
    )
  }

  min_val <- min(all_vals)
  max_val <- max(all_vals)
  all_categories <- seq(min_val, max_val)
  n_categories <- length(all_categories)

  if (
    !is.null(category_labels) &&
      length(category_labels) != n_categories
  ) {
    stop(
      "`category_labels` must have the same length as the number of ",
      "response categories (",
      min_val,
      " to ",
      max_val,
      " = ",
      n_categories,
      " categories). Got ",
      length(category_labels),
      ".",
      call. = FALSE
    )
  }

  # ---------------------------------------------------------------------
  # Prepare data
  # ---------------------------------------------------------------------
  facet_labels <- if (!is.null(item_labels)) {
    paste0(item_names, " - ", item_labels)
  } else {
    item_names
  }

  count_list <- vector("list", length(item_names))
  for (i in seq_along(item_names)) {
    item <- item_names[i]
    item_vals <- data[[item]]
    item_vals <- item_vals[!is.na(item_vals)]
    tab <- table(factor(item_vals, levels = all_categories))
    total <- sum(tab)

    count_list[[i]] <- data.frame(
      item_name = item,
      facet_label = facet_labels[i],
      category = as.integer(names(tab)),
      n = as.integer(tab),
      percent = if (total > 0L) {
        round(as.integer(tab) / total * 100, 1)
      } else {
        rep(0, length(tab))
      },
      stringsAsFactors = FALSE
    )
  }
  plot_df <- do.call(rbind, count_list)
  rownames(plot_df) <- NULL

  plot_df$facet_label <- factor(plot_df$facet_label, levels = facet_labels)
  plot_df$category_fct <- factor(plot_df$category, levels = all_categories)

  if (!is.null(category_labels)) {
    levels(plot_df$category_fct) <- category_labels
  }

  # ---------------------------------------------------------------------
  # Build plot
  # ---------------------------------------------------------------------
  # Standard sample-size caption; item-level NAs are retained -- each bar
  # counts the non-missing responses for that item.
  caption_text <- er2_caption(paste0(
    .n_caption(
      nrow(data),
      nrow(data),
      if (anyNA(data)) "incomplete responses retained" else character()
    ),
    "."
  ))

  ggplot2::ggplot(
    plot_df,
    ggplot2::aes(x = .data$category_fct, y = .data$percent)
  ) +
    ggplot2::geom_col(
      ggplot2::aes(fill = .data$category_fct)
    ) +
    ggplot2::geom_text(
      ggplot2::aes(
        label = paste0("n = ", .data$n),
        color = .data$category_fct
      ),
      y = text_y
    ) +
    ggplot2::facet_wrap(
      ~facet_label,
      ncol = ncol,
      strip.position = "left",
      labeller = ggplot2::labeller(
        facet_label = ggplot2::label_wrap_gen(label_wrap)
      )
    ) +
    ggplot2::scale_fill_viridis_d(guide = "none") +
    ggplot2::scale_color_viridis_d(
      guide = "none",
      direction = -1,
      option = viridis_option,
      end = viridis_end
    ) +
    ggplot2::scale_y_continuous(position = "right") +
    ggplot2::labs(
      x = "Response category",
      y = "% of responses",
      caption = caption_text
    ) +
    ggplot2::theme_bw(base_family = font) +
    ggplot2::theme(
      legend.position = "none",
      strip.text.y.left = ggplot2::element_text(angle = 0)
    ) +
    er2_axis_margins() +
    er2_plot_caption()
}


#' Stacked Bar Chart of Item Response Distributions
#'
#' Creates a horizontal stacked bar chart showing the response
#' distribution for all items. Each bar represents one item, with segments
#' coloured by response category. Counts are displayed as text labels
#' within each segment. This is a descriptive data visualization tool
#' intended for use before model fitting.
#'
#' @param data A data.frame in wide format containing only the item
#'   response columns. Each column is one item, each row is one person.
#'   All columns must be numeric (integer-valued). Response categories
#'   may be coded starting from 0 or 1. Do not include person IDs,
#'   grouping variables, or other non-item columns.
#' @param item_labels An optional character vector of descriptive labels
#'   for the items (y-axis). Must be the same length as `ncol(data)`.
#'   If `NULL` (the default), column names are used.
#' @param category_labels An optional character vector of labels for the
#'   response categories (legend). Must be the same length as the number
#'   of response categories spanning from the minimum to the maximum
#'   observed value, ordered from lowest to highest category. If `NULL`
#'   (the default), numeric category values are used.
#' @param show_n Logical. If `TRUE` (the default), the count of responses
#'   is displayed as a text label inside each bar segment.
#' @param show_percent Logical. If `TRUE`, the percentage of responses is
#'   displayed instead of (or in addition to) counts. Default is `FALSE`.
#' @param text_color Character. Colour for the count/percentage labels.
#'   Default is `"sienna1"`.
#' @param text_size Numeric. Size of the count/percentage labels.
#'   Default is `3`.
#' @param min_label_n Integer. Minimum count required for a label to be
#'   displayed within a bar segment. Segments with fewer responses are
#'   left unlabelled to avoid clutter. Default is `0L` (all segments
#'   labelled).
#' @param viridis_option Character. Viridis palette option. One of
#'   `"A"` through `"H"`. Default is `"D"`.
#' @param viridis_end Numeric in \eqn{[0, 1]}. End point of the viridis
#'   colour scale. Default is `0.99`.
#' @param title Character. Plot title. Default is `"Item responses"`.
#'
#' @return A [ggplot2::ggplot] object.
#'
#' @details
#' Items are displayed on the y-axis in the same order as the columns
#' in `data` (first column at the top). Each bar is divided into
#' segments representing response categories, with the lowest category
#' on the left and the highest on the right. The total bar length
#' equals the number of non-missing responses for that item.
#'
#' Categories with zero responses still appear in the legend but produce
#' no visible bar segment, which helps identify gaps in the response
#' distribution.
#'
#' The plot caption reports the sample in the standard
#' `n = X respondents (policy)` form; item-level `NA`s are retained --
#' each bar counts the non-missing responses for that item.
#'
#' **Input requirements:**
#' \itemize{
#'   \item All columns must be numeric (integer-valued).
#'   \item The data frame must contain at least 2 columns (items)
#'     and at least 1 row (person).
#' }
#'
#' @seealso [RMplotBar()], [RMplotTile()]
#'
#' @examples
#' if (requireNamespace("eRm", quietly = TRUE) &&
#'     requireNamespace("ggplot2", quietly = TRUE)) {
#'   data(pcmdat2, package = "eRm")
#'
#'   # Basic stacked bar chart
#'   RMplotStackedbar(pcmdat2)
#'
#'   # With custom item and category labels
#'   RMplotStackedbar(
#'     pcmdat2,
#'     item_labels     = c("Mood", "Sleep", "Appetite", "Energy"),
#'     category_labels = c("Never", "Sometimes", "Often")
#'   )
#'
#'   # Show percentages, suppress small segments
#'   RMplotStackedbar(
#'     pcmdat2,
#'     show_percent = TRUE,
#'     show_n       = FALSE,
#'     min_label_n  = 5
#'   )
#' }
#'
#' @importFrom rlang .data
#' @export
RMplotStackedbar <- function(
  data,
  item_labels = NULL,
  category_labels = NULL,
  show_n = TRUE,
  show_percent = FALSE,
  text_color = "sienna1",
  text_size = 3,
  min_label_n = 0L,
  viridis_option = "D",
  viridis_end = 0.99,
  title = "Item responses"
) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required for RMplotStackedbar().", call. = FALSE)
  }

  # ---------------------------------------------------------------------
  # Input validation
  # ---------------------------------------------------------------------
  if (!is.data.frame(data)) {
    stop(
      "`data` must be a data.frame in wide format ",
      "(one column per item, one row per person).",
      call. = FALSE
    )
  }
  if (ncol(data) < 2L) {
    stop(
      "`data` must contain at least 2 columns (items). Found ",
      ncol(data),
      " column(s).",
      call. = FALSE
    )
  }
  if (nrow(data) < 1L) {
    stop(
      "`data` must contain at least 1 row (person). Found 0 rows.",
      call. = FALSE
    )
  }

  non_numeric <- names(data)[!vapply(data, is.numeric, logical(1L))]
  if (length(non_numeric) > 0L) {
    stop(
      "All columns must be numeric. Non-numeric column(s): ",
      paste(non_numeric, collapse = ", "),
      ". Remove non-item columns (e.g., person IDs, grouping ",
      "variables) before passing to RMplotStackedbar().",
      call. = FALSE
    )
  }

  all_vals <- unlist(data, use.names = FALSE)
  all_vals <- all_vals[!is.na(all_vals)]
  if (length(all_vals) == 0L) {
    stop("`data` contains only NA values.", call. = FALSE)
  }
  if (!all(all_vals == round(all_vals))) {
    stop(
      "All response values must be integers. Found non-integer values.",
      call. = FALSE
    )
  }

  item_names <- names(data)

  if (!is.null(item_labels) && length(item_labels) != length(item_names)) {
    stop(
      "`item_labels` must have the same length as the number of ",
      "columns in `data`. Expected ",
      length(item_names),
      ", got ",
      length(item_labels),
      ".",
      call. = FALSE
    )
  }

  min_val <- min(all_vals)
  max_val <- max(all_vals)
  all_categories <- seq(min_val, max_val)
  n_categories <- length(all_categories)

  if (
    !is.null(category_labels) &&
      length(category_labels) != n_categories
  ) {
    stop(
      "`category_labels` must have the same length as the number of ",
      "response categories (",
      min_val,
      " to ",
      max_val,
      " = ",
      n_categories,
      " categories). Got ",
      length(category_labels),
      ".",
      call. = FALSE
    )
  }

  # ---------------------------------------------------------------------
  # Prepare data
  # ---------------------------------------------------------------------
  y_labels <- if (!is.null(item_labels)) {
    stats::setNames(item_labels, item_names)
  } else {
    stats::setNames(item_names, item_names)
  }

  count_list <- vector("list", length(item_names))
  for (i in seq_along(item_names)) {
    item <- item_names[i]
    item_vals <- data[[item]]
    item_vals <- item_vals[!is.na(item_vals)]
    tab <- table(factor(item_vals, levels = all_categories))
    total <- sum(tab)

    count_list[[i]] <- data.frame(
      item_name = item,
      category = as.integer(names(tab)),
      n = as.integer(tab),
      percent = if (total > 0L) {
        round(as.integer(tab) / total * 100, 1)
      } else {
        rep(0, length(tab))
      },
      stringsAsFactors = FALSE
    )
  }
  plot_df <- do.call(rbind, count_list)
  rownames(plot_df) <- NULL

  # Items: reversed so the first column appears at the top of the y-axis
  plot_df$item_fct <- factor(
    plot_df$item_name,
    levels = rev(item_names),
    labels = rev(y_labels[item_names])
  )

  # Categories: reversed for stacking order (lowest category on the left)
  plot_df$category_fct <- factor(
    plot_df$category,
    levels = rev(all_categories)
  )

  # Build text labels
  plot_df$label <- ""
  show_mask <- plot_df$n >= min_label_n

  if (show_n && show_percent) {
    plot_df$label[show_mask] <- paste0(
      plot_df$n[show_mask],
      " (",
      plot_df$percent[show_mask],
      "%)"
    )
  } else if (show_n) {
    plot_df$label[show_mask] <- as.character(plot_df$n[show_mask])
  } else if (show_percent) {
    plot_df$label[show_mask] <- paste0(plot_df$percent[show_mask], "%")
  }

  # Suppress labels for zero-count segments
  plot_df$label[plot_df$n == 0L] <- ""

  # Legend labels: lowest -> highest (reverse of stacking factor)
  legend_labels <- if (!is.null(category_labels)) {
    stats::setNames(rev(category_labels), rev(as.character(all_categories)))
  } else {
    stats::setNames(
      rev(as.character(all_categories)),
      rev(as.character(all_categories))
    )
  }

  # ---------------------------------------------------------------------
  # Build plot
  # ---------------------------------------------------------------------
  # Standard sample-size caption; item-level NAs are retained -- each bar
  # counts the non-missing responses for that item.
  caption_text <- er2_caption(paste0(
    .n_caption(
      nrow(data),
      nrow(data),
      if (anyNA(data)) "incomplete responses retained" else character()
    ),
    "."
  ))

  p <- ggplot2::ggplot(
    plot_df,
    ggplot2::aes(
      x = .data$n,
      y = .data$item_fct,
      fill = .data$category_fct
    )
  ) +
    ggplot2::geom_col() +
    ggplot2::scale_fill_viridis_d(
      "Response\ncategory",
      direction = -1,
      option = viridis_option,
      end = viridis_end,
      labels = legend_labels
    ) +
    ggplot2::labs(
      title = title,
      x = "Number of responses",
      y = NULL,
      caption = caption_text
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(legend.position = "right") +
    er2_axis_margins() +
    er2_plot_caption()

  if (show_n || show_percent) {
    p <- p +
      ggplot2::geom_text(
        ggplot2::aes(label = .data$label),
        color = text_color,
        size = text_size,
        position = ggplot2::position_stack(vjust = 0.5)
      )
  }

  p
}

# Tests for RMplotBar() and RMplotStackedbar()

make_polytomous <- function(n = 200, k = 4, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:3, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# RMplotBar --- input validation
# ---------------------------------------------------------------------
test_that("RMplotBar errors on non-data.frame input", {
  skip_if_not_installed("ggplot2")
  expect_error(RMplotBar(matrix(0:3, 4, 4)), regexp = "data.frame")
})

test_that("RMplotBar errors when fewer than 2 items", {
  skip_if_not_installed("ggplot2")
  expect_error(
    RMplotBar(make_polytomous(k = 1L)),
    regexp = "at least 2 columns"
  )
})

test_that("RMplotBar errors on non-numeric columns", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  df$group <- letters[seq_len(nrow(df))]
  expect_error(RMplotBar(df), regexp = "must be numeric")
})

test_that("RMplotBar errors on non-integer values", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  df[1, 1] <- 0.5
  expect_error(RMplotBar(df), regexp = "integers")
})

test_that("RMplotBar errors when item_labels length mismatches", {
  skip_if_not_installed("ggplot2")
  expect_error(
    RMplotBar(make_polytomous(), item_labels = c("a", "b")),
    regexp = "same length"
  )
})

test_that("RMplotBar errors when category_labels length mismatches", {
  skip_if_not_installed("ggplot2")
  expect_error(
    RMplotBar(make_polytomous(), category_labels = c("Low", "High")),
    regexp = "same length"
  )
})

# ---------------------------------------------------------------------
# RMplotBar --- output
# ---------------------------------------------------------------------
test_that("RMplotBar returns a ggplot", {
  skip_if_not_installed("ggplot2")
  expect_s3_class(RMplotBar(make_polytomous()), "ggplot")
})

test_that("RMplotBar accepts custom item_labels", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMplotBar(df, item_labels = paste("Item", seq_len(ncol(df))))
  expect_s3_class(p, "ggplot")
})

test_that("RMplotBar accepts custom category_labels", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMplotBar(df, category_labels = c("None", "Some", "More", "Most"))
  expect_s3_class(p, "ggplot")
})

test_that("RMplotBar handles 0/1 dichotomous data", {
  skip_if_not_installed("ggplot2")
  set.seed(2)
  df <- as.data.frame(matrix(sample(0:1, 200 * 5, replace = TRUE), 200, 5))
  colnames(df) <- paste0("I", 1:5)
  expect_s3_class(RMplotBar(df), "ggplot")
})

test_that("RMplotBar tolerates NA values", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  df[1:5, 1] <- NA
  expect_s3_class(RMplotBar(df), "ggplot")
})

# ---------------------------------------------------------------------
# RMplotStackedbar --- input validation
# ---------------------------------------------------------------------
test_that("RMplotStackedbar errors on non-data.frame input", {
  skip_if_not_installed("ggplot2")
  expect_error(RMplotStackedbar(matrix(0:3, 4, 4)), regexp = "data.frame")
})

test_that("RMplotStackedbar errors when fewer than 2 items", {
  skip_if_not_installed("ggplot2")
  expect_error(
    RMplotStackedbar(make_polytomous(k = 1L)),
    regexp = "at least 2 columns"
  )
})

test_that("RMplotStackedbar errors on non-numeric columns", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  df$group <- letters[seq_len(nrow(df))]
  expect_error(RMplotStackedbar(df), regexp = "must be numeric")
})

test_that("RMplotStackedbar errors on non-integer values", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  df[1, 1] <- 0.5
  expect_error(RMplotStackedbar(df), regexp = "integers")
})

test_that("RMplotStackedbar errors when item_labels length mismatches", {
  skip_if_not_installed("ggplot2")
  expect_error(
    RMplotStackedbar(make_polytomous(), item_labels = c("a", "b")),
    regexp = "same length"
  )
})

test_that("RMplotStackedbar errors when category_labels length mismatches", {
  skip_if_not_installed("ggplot2")
  expect_error(
    RMplotStackedbar(make_polytomous(), category_labels = c("Low", "High")),
    regexp = "same length"
  )
})

# ---------------------------------------------------------------------
# RMplotStackedbar --- output
# ---------------------------------------------------------------------
test_that("RMplotStackedbar returns a ggplot", {
  skip_if_not_installed("ggplot2")
  expect_s3_class(RMplotStackedbar(make_polytomous()), "ggplot")
})

test_that("RMplotStackedbar accepts custom item & category labels", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMplotStackedbar(
    df,
    item_labels = paste("Item", seq_len(ncol(df))),
    category_labels = c("None", "Some", "More", "Most")
  )
  expect_s3_class(p, "ggplot")
})

test_that("RMplotStackedbar supports show_percent and min_label_n", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMplotStackedbar(
    df,
    show_percent = TRUE,
    show_n = FALSE,
    min_label_n = 5L
  )
  expect_s3_class(p, "ggplot")
})

test_that("RMplotStackedbar can suppress all labels", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMplotStackedbar(df, show_n = FALSE, show_percent = FALSE)
  expect_s3_class(p, "ggplot")
})

test_that("RMplotStackedbar handles 0/1 dichotomous data", {
  skip_if_not_installed("ggplot2")
  set.seed(3)
  df <- as.data.frame(matrix(sample(0:1, 200 * 5, replace = TRUE), 200, 5))
  colnames(df) <- paste0("I", 1:5)
  expect_s3_class(RMplotStackedbar(df), "ggplot")
})

# ---------------------------------------------------------------------
# Sample-size captions
# ---------------------------------------------------------------------
test_that("RMplotBar caption reports n, with qualifier only when NAs exist", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMplotBar(df)
  expect_match(p$labels$caption, "n = 200 respondents")
  expect_no_match(p$labels$caption, "incomplete responses retained")

  df[1:3, 1] <- NA
  p_na <- RMplotBar(df)
  expect_match(p_na$labels$caption, "n = 200 respondents")
  expect_match(p_na$labels$caption, "incomplete responses retained")
})

test_that("RMplotStackedbar caption reports n, with qualifier only when NAs exist", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMplotStackedbar(df)
  expect_match(p$labels$caption, "n = 200 respondents")
  expect_no_match(p$labels$caption, "incomplete responses retained")

  df[1:3, 1] <- NA
  p_na <- RMplotStackedbar(df)
  expect_match(p_na$labels$caption, "incomplete responses retained")
})

#' PCA of Standardized Rasch Residuals
#'
#' Fits a Rasch model by CML via `psychotools` (a dichotomous item is a
#' 2-category PCM), extracts the standardized residuals \eqn{(x - E)/\sqrt{Var}}
#' at WLE person locations, and runs an unrotated principal-component
#' analysis on those residuals via `stats::prcomp()`. The function reports
#' the top `n_components` eigenvalues and their proportions of unexplained
#' variance, and optionally compares the first-contrast eigenvalue against
#' a simulation-based bound from \code{\link{RMdimResidualPCACutoff}}.
#'
#' Rule-of-thumb thresholds for the first-contrast eigenvalue (e.g., the
#' "> 2" heuristic occasionally cited from Winsteps documentation) are not
#' reliable indicators of multidimensionality; the first-contrast eigenvalue
#' under a correctly fitting unidimensional model varies systematically with
#' sample size, test length, and item-parameter spread. Empirical (simulated)
#' bounds tailored to the data structure should be used instead — see
#' \code{\link{RMdimResidualPCACutoff}}, and Chou & Wang (2010) for the underlying
#' simulation argument.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Rows with any `NA` are dropped
#'   before PCA, since `prcomp()` does not accept missing values.
#' @param cutoff Optional. The list returned by \code{\link{RMdimResidualPCACutoff}} (its
#'   `suggested_cutoff` is used), or a single numeric value to use as the
#'   cutoff directly. When provided, the result includes a `Flagged` column
#'   (logical: is the eigenvalue above the simulated bound?) and the kable
#'   caption notes the cutoff.
#' @param p_value Logical. When `TRUE`, adds a one-sided bootstrap p-value for
#'   the first-contrast eigenvalue: the proportion of simulated first-contrast
#'   eigenvalues at least as large as the observed one,
#'   `(1 + #\{lambda* >= lambda\}) / (B + 1)`. Requires the **full**
#'   \code{\link{RMdimResidualPCACutoff}} object as `cutoff` (it carries the
#'   simulated eigenvalues in `$results`); a bare numeric cutoff is not
#'   sufficient. The simulated null is the distribution of the *largest*
#'   eigenvalue, so the p-value applies to PC1 only (`NA` for the other
#'   components). This is a single test — no multiplicity correction is
#'   involved. Default `FALSE`.
#' @param n_components Integer. Number of eigenvalues to report. Capped at the
#'   number of items. Default `5`.
#' @param output Character. `"kable"` (default) for a formatted
#'   `knitr::kable()` table, `"dataframe"` for the underlying data.frame, or
#'   `"ggplot"` for a ggplot of PC1 loadings against item locations.
#'   `"loadings"` is accepted as a backward-compatible alias for `"ggplot"`.
#'
#' @return
#' * If `output = "kable"`: a `knitr_kable` object with columns Component,
#'   Eigenvalue, Proportion of variance (and `Flagged` when `cutoff` is
#'   provided; `p` when `p_value = TRUE`). The caption gives the variance
#'   partition (% of total observed variance explained by measures vs.
#'   unexplained), the model fitted, sample size, and cutoff/p-value metadata
#'   if applicable.
#' * If `output = "dataframe"`: a data.frame with columns `Component`,
#'   `Eigenvalue`, `Proportion_of_variance` (and `Flagged` when `cutoff` is
#'   provided; `p` when `p_value = TRUE`, non-`NA` for PC1 only). The
#'   variance partition is attached as the
#'   `"variance_partition"` attribute — a list with elements `total`,
#'   `explained`, `unexplained`, `pct_explained`, `pct_unexplained`,
#'   `n_persons`. Access via
#'   `attr(result, "variance_partition")`.
#' * If `output = "ggplot"`: a ggplot showing each item's PC1 loading on
#'   the x-axis and Rasch item location on the y-axis, with dashed reference
#'   lines at zero, and the variance partition in the figure caption. Item
#'   names are labelled via `ggrepel::geom_text_repel()` when `ggrepel` is
#'   installed; otherwise plain `geom_text()`.
#'
#' @details
#' The PCA is performed on the standardized residuals
#' \eqn{(x - E)/\sqrt{Var}} from the shared CML/WLE engine (CML item
#' parameters via `psychotools`, WLE person locations). The reported
#' eigenvalues are unrotated; rotation is appropriate for *interpreting* a
#' multidimensional solution but obscures the dominant first contrast that
#' dimensionality assessment is concerned with.
#'
#' Item locations on the loadings plot are the per-item mean of the CML
#' Andrich thresholds.
#'
#' The variance partition follows Linacre's convention: per-item observed
#' variance is compared to per-item *expected* variance under the fitted
#' model, summed across items. Expected scores are computed from the CML item
#' parameters and WLE person locations. WLE is finite at extreme scores, so
#' all persons are retained (the previous MLE partition dropped extreme-score
#' cases).
#'
#' \strong{Bootstrap p-value.} When `p_value = TRUE`, the observed
#' first-contrast eigenvalue is compared against the simulated null
#' distribution of largest eigenvalues (from `cutoff$results`), giving the
#' one-sided Monte-Carlo p-value `(1 + #\{lambda* >= lambda\}) / (B + 1)`.
#' Because the maximum eigenvalue is a single family-wise statistic, no
#' multiplicity correction applies. The p-value is model-conditional and
#' sample-size-sensitive; it is reported alongside the simulated cutoff, not
#' in place of it, and can be no smaller than `1 / (B + 1)`.
#'
#' @references
#' Chou, Y.-T., & Wang, W.-C. (2010). Checking dimensionality in item
#' response models with principal component analysis on standardized
#' residuals. *Educational and Psychological Measurement, 70*(5), 717-731.
#' \doi{10.1177/0013164410379322}
#'
#' @seealso \code{\link{RMdimResidualPCACutoff}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' dat <- as.data.frame(
#'   matrix(sample(0:1, 200 * 12, replace = TRUE), nrow = 200, ncol = 12)
#' )
#' colnames(dat) <- paste0("I", 1:12)
#'
#' # Default kable output
#' RMdimResidualPCA(dat)
#'
#' # PC1 loadings vs item location plot
#' if (requireNamespace("ggplot2", quietly = TRUE) &&
#'     requireNamespace("ggrepel", quietly = TRUE)) {
#'   RMdimResidualPCA(dat, output = "ggplot")
#' }
#'
#' # Simulation-based cutoff (use 250+ iterations in real analyses)
#' bound <- RMdimResidualPCACutoff(dat, iterations = 50, parallel = FALSE, seed = 1)
#' RMdimResidualPCA(dat, cutoff = bound)
#'
#' # With the one-sided bootstrap p-value for the first contrast
#' RMdimResidualPCA(dat, cutoff = bound, p_value = TRUE)
#' }
RMdimResidualPCA <- function(
  data,
  cutoff = NULL,
  p_value = FALSE,
  n_components = 5L,
  output = "kable"
) {
  output <- match.arg(output, c("kable", "dataframe", "ggplot", "loadings"))
  # "loadings" is a backward-compatible alias for "ggplot" (the canonical value,
  # consistent with the other single-ggplot functions).
  if (output == "loadings") {
    output <- "ggplot"
  }

  validate_response_data(data)

  # Drop incomplete rows (prcomp can't handle NA in standardized residuals)
  data <- as.data.frame(data)
  n_total <- nrow(data)
  has_na <- anyNA(data)
  data <- stats::na.omit(data)
  if (nrow(data) == 0L) {
    stop("No complete cases in `data` after na.omit.", call. = FALSE)
  }

  # --- Normalise the cutoff argument -----------------------------------------
  cutoff_value <- NULL
  cutoff_n_iter <- NULL
  cutoff_percentile_pct <- NULL
  cutoff_full <- NULL # full object (carries simulated $results for the p-value)
  if (!is.null(cutoff)) {
    if (
      is.list(cutoff) &&
        !is.data.frame(cutoff) &&
        "suggested_cutoff" %in% names(cutoff)
    ) {
      cutoff_full <- cutoff
      cutoff_value <- cutoff$suggested_cutoff
      cutoff_n_iter <- cutoff$actual_iterations
      cutoff_percentile_pct <- cutoff$suggested_cutoff_percentile
    } else if (is.numeric(cutoff) && length(cutoff) == 1L) {
      cutoff_value <- as.numeric(cutoff)
    } else {
      stop(
        "`cutoff` must be NULL, the return value of RMdimResidualPCACutoff(), or a ",
        "single numeric value.",
        call. = FALSE
      )
    }
  }

  # --- p-value prerequisites --------------------------------------------------
  if (p_value) {
    if (is.null(cutoff_full) || is.null(cutoff_full$results)) {
      stop(
        "`p_value = TRUE` requires the full RMdimResidualPCACutoff() object ",
        "(it carries the simulated eigenvalues in $results); a NULL cutoff ",
        "or a bare numeric cutoff is not sufficient.",
        call. = FALSE
      )
    }
  }

  # rgl workaround
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # --- Fit Rasch model and get residuals -------------------------------------
  data_mat <- as.matrix(data)
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L

  # CML (psychotools) can choke or destabilise on sparse / zero-variance
  # response categories; warn before fitting, consistent with RMitemParameters().
  .sparsity_warning(data, is_poly = is_polytomous)

  # CML item thresholds (psychotools) and WLE-based standardized residuals,
  # consistent with the rest of the package. Item locations are per-item mean
  # thresholds; the standardized residuals (x - E)/sqrt(Var) come from the
  # shared CML/WLE engine. (Previously eRm CML + eRm::itemfit() MLE residuals.)
  thr_list <- .fit_cml_thresholds(data)
  item_locations <- vapply(thr_list, mean, numeric(1L))
  names(item_locations) <- names(data)

  st_resids <- .rasch_std_residuals(data, method = "WLE")
  if (anyNA(st_resids)) {
    keep_rows <- stats::complete.cases(st_resids)
    st_resids <- st_resids[keep_rows, , drop = FALSE]
  }

  # --- Variance partition (Linacre-style; CML items, WLE thetas) -------------
  # Compares Var(observed) to Var(model-expected) per item, summed across
  # items. WLE locations are finite at extreme scores, so all persons are
  # retained (unlike the MLE partition, which dropped extreme-score cases).
  thetas_all <- .estimate_thetas(data_mat, thr_list, method = "WLE")$theta
  finite_thetas <- is.finite(thetas_all)

  if (sum(finite_thetas) >= 2L) {
    th <- thetas_all[finite_thetas]
    expected_mat <- vapply(
      seq_along(thr_list),
      function(i) {
        cats <- 0:length(thr_list[[i]])
        P <- vapply(
          th,
          function(z) .pcm_cat_probs(z, thr_list[[i]]),
          numeric(length(cats))
        )
        as.numeric(crossprod(cats, P)) # E[score] per person
      },
      numeric(length(th))
    )
    data_finite <- data_mat[finite_thetas, , drop = FALSE]
    var_total <- sum(apply(data_finite, 2L, stats::var, na.rm = TRUE))
    var_explained <- sum(apply(expected_mat, 2L, stats::var, na.rm = TRUE))
    var_unexplained <- max(var_total - var_explained, 0)
    pct_explained <- if (var_total > 0) var_explained / var_total else NA_real_
    pct_unexplained <- if (var_total > 0) {
      var_unexplained / var_total
    } else {
      NA_real_
    }
    n_partition <- sum(finite_thetas)
    partition_avail <- TRUE
  } else {
    var_total <- var_explained <- var_unexplained <- NA_real_
    pct_explained <- pct_unexplained <- NA_real_
    n_partition <- 0L
    partition_avail <- FALSE
  }

  partition_text <- if (partition_avail) {
    paste0(
      "Total observed variance: ",
      round(pct_explained * 100, 1),
      "% explained by measures, ",
      round(pct_unexplained * 100, 1),
      "% unexplained.",
      # The headline n clause already reports the complete-case sample; only
      # note the partition basis when it is smaller (some non-finite WLE
      # locations), to avoid restating the same n twice.
      if (n_partition < nrow(data)) {
        paste0(
          " Variance partition based on ",
          n_partition,
          " respondents with finite locations."
        )
      } else {
        ""
      }
    )
  } else {
    "Variance partition unavailable (too few persons with finite theta MLEs)."
  }

  # --- Run unrotated PCA -----------------------------------------------------
  pca_fit <- stats::prcomp(st_resids)
  eigvals <- pca_fit$sdev^2
  total <- sum(eigvals)
  prop_var <- eigvals / total

  k_show <- min(as.integer(n_components), length(eigvals))

  # Unrounded values; the kable path rounds a display copy before rendering.
  result_df <- data.frame(
    Component = paste0("PC", seq_len(k_show)),
    Eigenvalue = eigvals[seq_len(k_show)],
    Proportion_of_variance = prop_var[seq_len(k_show)],
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  if (!is.null(cutoff_value)) {
    result_df$Flagged <- result_df$Eigenvalue > cutoff_value
  }

  # --- One-sided bootstrap p-value for the first contrast ---------------------
  # The simulated null is the distribution of the *largest* eigenvalue, so the
  # p-value applies to PC1 only. Reuses the shared helper (studentisation is
  # monotone per column, so it does not change the single-column p-value);
  # correction = "none" because the max eigenvalue is one family-wise test.
  p_pc1 <- NULL
  if (p_value) {
    sim_eig <- matrix(
      cutoff_full$results$eigenvalue,
      ncol = 1L,
      dimnames = list(NULL, "PC1")
    )
    pv <- .bootstrap_pvalues(
      c(PC1 = eigvals[1L]),
      sim_eig,
      correction = "none",
      tail = "upper"
    )
    p_pc1 <- pv$p[1L]
    result_df$p <- c(p_pc1, rep(NA_real_, nrow(result_df) - 1L))
  }

  # Variance partition is metadata on the dataframe rather than a separate
  # list element. Access via attr(result, "variance_partition").
  attr(result_df, "variance_partition") <- list(
    total = var_total,
    explained = var_explained,
    unexplained = var_unexplained,
    pct_explained = pct_explained,
    pct_unexplained = pct_unexplained,
    n_persons = n_partition
  )

  # --- Loadings plot ---------------------------------------------------------
  if (output == "ggplot") {
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
      stop(
        "Package 'ggplot2' is required for output = \"ggplot\".",
        call. = FALSE
      )
    }
    loadings <- as.data.frame(pca_fit$rotation)
    loadings$Item <- rownames(loadings)
    loadings$Location <- as.numeric(item_locations[loadings$Item])

    p <- ggplot2::ggplot(
      loadings,
      ggplot2::aes(x = .data$PC1, y = .data$Location, label = .data$Item)
    ) +
      ggplot2::geom_vline(xintercept = 0, linetype = 2, colour = "grey40") +
      ggplot2::geom_hline(yintercept = 0, linetype = 2, colour = "grey40") +
      ggplot2::geom_point(size = 2.5) +
      ggplot2::scale_x_continuous(limits = c(-1, 1))

    if (requireNamespace("ggrepel", quietly = TRUE)) {
      p <- p + ggrepel::geom_text_repel(size = 3.5, max.overlaps = Inf)
    } else {
      p <- p + ggplot2::geom_text(nudge_y = 0.05, size = 3.5)
    }

    p <- p +
      ggplot2::labs(
        x = "Loading on first residual contrast (PC1)",
        y = "Item location (logit scale)",
        caption = er2_caption(paste0(
          partition_text,
          " ",
          .n_caption(
            nrow(data),
            n_total,
            if (has_na) "complete cases" else character()
          ),
          "."
        ))
      ) +
      ggplot2::theme_bw(base_size = 13) +
      er2_axis_margins() +
      er2_plot_caption()

    return(p)
  }

  if (output == "dataframe") {
    return(result_df)
  }

  # --- kable -----------------------------------------------------------------
  # Display rounding (the dataframe output above stays unrounded)
  result_df <- .round_display(result_df, c(
    Eigenvalue = 3, Proportion_of_variance = 3, p = 4
  ))
  n_clause <- .n_caption(
    nrow(data),
    n_total,
    if (has_na) "complete cases" else character()
  )
  caption_parts <- c(
    paste0(
      if (is_polytomous) "Partial Credit Model" else "Rasch model",
      " (",
      ncol(data),
      " items), ",
      n_clause,
      "."
    ),
    partition_text
  )
  if (!is.null(cutoff_value)) {
    iter_part <- if (!is.null(cutoff_n_iter)) {
      paste0(" based on ", cutoff_n_iter, " simulation iterations")
    } else {
      ""
    }
    pct_part <- if (!is.null(cutoff_percentile_pct)) {
      paste0(" (", cutoff_percentile_pct, "th percentile)")
    } else {
      ""
    }
    caption_parts <- c(
      caption_parts,
      paste0(
        "First-contrast cutoff = ",
        round(cutoff_value, 3),
        iter_part,
        pct_part,
        "."
      )
    )
  }
  if (p_value) {
    caption_parts <- c(
      caption_parts,
      paste0(
        "One-sided bootstrap p-value for the first contrast (single test, ",
        "no multiplicity correction); it cannot be smaller than 1/(",
        cutoff_n_iter,
        "+1) = ",
        round(1 / (cutoff_n_iter + 1), 4),
        "."
      )
    )
  }

  knitr::kable(
    result_df,
    format = "pipe",
    col.names = c(
      "Component",
      "Eigenvalue",
      "Proportion of variance",
      if (!is.null(cutoff_value)) "Flagged",
      if (p_value) "p"
    ),
    caption = paste(caption_parts, collapse = " ")
  )
}

# ===========================================================================

#' Simulation-based Cutoff for First-Contrast Eigenvalue
#'
#' Parametric bootstrap producing an empirical upper percentile for the
#' largest eigenvalue from a PCA of standardized residuals under a correctly
#' fitting Rasch model. For each iteration the function resamples theta
#' values from the data-based estimates, simulates response data, refits the
#' appropriate Rasch model, and extracts the first-contrast eigenvalue.
#' Several upper-tail percentiles of the resulting distribution are returned;
#' the 99th percentile is reported as the suggested cutoff (matching the
#' convention used by \code{\link{RMlocdepQ3Cutoff}}).
#'
#' Rule-of-thumb cutoffs for the first-contrast eigenvalue depend strongly on
#' sample size, test length, and item-parameter spread; simulation-based
#' cutoffs tailored to the data are more defensible (Chou & Wang, 2010).
#'
#' @param data A data.frame or matrix of item responses (0-based,
#'   non-negative integers).
#' @param iterations Integer. Number of simulation iterations. Default `250`.
#' @param parallel Logical. Use parallel processing via `mirai`. Default
#'   `TRUE`.
#' @param n_cores Integer or `NULL`. Number of parallel workers. When `NULL`,
#'   `getOption("mc.cores")` is checked first; if neither is set,
#'   `parallel = TRUE` falls back to sequential with a warning.
#' @param verbose Logical. Show a progress bar. Default `FALSE`.
#' @param seed Integer or `NULL`. Random seed for reproducibility.
#'
#' @return A list with components:
#' \describe{
#'   \item{`results`}{data.frame: `iteration`, `eigenvalue`.}
#'   \item{`p95`, `p99`, `p995`, `p999`}{Empirical percentiles of
#'     `eigenvalue`.}
#'   \item{`max`}{The largest simulated eigenvalue.}
#'   \item{`suggested_cutoff`}{The 99th percentile (`p99`) — pass this list
#'     back into \code{\link{RMdimResidualPCA}} via `cutoff = `, or use
#'     `suggested_cutoff` directly.}
#'   \item{`suggested_cutoff_percentile`}{The percentile used for
#'     `suggested_cutoff`, currently always `99`.}
#'   \item{`actual_iterations`}{Number of successful iterations.}
#'   \item{`sample_n`}{Number of complete cases used.}
#'   \item{`item_names`}{Character vector of item names.}
#' }
#'
#' @details
#' The generating model uses CML item parameters (`psychotools`) and WLE person
#' locations. Per iteration: theta values are sampled with replacement from the
#' WLE estimates; response data are simulated under the model
#' (`psychotools::rrm` for dichotomous, partial-credit simulator for
#' polytomous); CML/WLE standardized residuals are computed (the same engine as
#' the observed analysis); `prcomp()` is run; the first-contrast eigenvalue is
#' recorded.
#'
#' Iterations that fail (e.g., due to a degenerate simulated dataset where
#' some category isn't represented) are silently dropped. The `iarm` package
#' is **not** required.
#'
#' @references
#' Chou, Y.-T., & Wang, W.-C. (2010). Checking dimensionality in item
#' response models with principal component analysis on standardized
#' residuals. *Educational and Psychological Measurement, 70*(5), 717-731.
#' \doi{10.1177/0013164410379322}
#'
#' @seealso \code{\link{RMdimResidualPCA}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' dat <- as.data.frame(
#'   matrix(sample(0:1, 200 * 12, replace = TRUE), nrow = 200, ncol = 12)
#' )
#' colnames(dat) <- paste0("I", 1:12)
#'
#' # Few iterations for a fast example; use 250+ in real analyses
#' bound <- RMdimResidualPCACutoff(dat, iterations = 50, parallel = FALSE, seed = 1)
#' bound$suggested_cutoff
#'
#' RMdimResidualPCA(dat, cutoff = bound)
#' }
RMdimResidualPCACutoff <- function(
  data,
  iterations = 250,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL
) {
  validate_response_data(data)

  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  data <- stats::na.omit(as.data.frame(data))
  if (nrow(data) == 0L) {
    stop("No complete cases in `data`.", call. = FALSE)
  }

  use_parallel <- parallel && requireNamespace("mirai", quietly = TRUE)
  if (parallel && !use_parallel) {
    message(
      "Install 'mirai' for parallel processing: install.packages(\"mirai\")"
    )
    message("Running sequentially...")
  }
  if (use_parallel) {
    if (is.null(n_cores)) {
      n_cores <- getOption("mc.cores")
    }
    if (is.null(n_cores)) {
      warning(
        "For parallel processing, specify n_cores or set options(mc.cores = N).\n",
        "Falling back to sequential.",
        call. = FALSE
      )
      use_parallel <- FALSE
    } else {
      n_cores <- min(n_cores, iterations)
    }
  }

  if (!is.null(seed)) {
    set.seed(seed)
  }
  sim_seeds <- sample.int(.Machine$integer.max, iterations)

  data_mat <- as.matrix(data)
  sample_n <- nrow(data_mat)
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L
  item_names_vec <- colnames(data_mat)

  # Generating model: CML item thresholds (psychotools) + WLE person locations,
  # consistent with the rest of the package. The DGP is unchanged: thetas are
  # resampled with replacement and data simulated parametrically.
  pool <- .wle_theta_pool(data_mat)
  thr_list <- pool$thr_list
  wle_thetas <- pool$thetas

  sim_data_list <- list(
    type = if (is_polytomous) "polytomous" else "dichotomous",
    thetas = wle_thetas,
    n_items = ncol(data_mat),
    sample_n = sample_n,
    item_names = item_names_vec
  )
  if (is_polytomous) {
    sim_data_list$deltaslist <- thr_list
  } else {
    sim_data_list$item_params <- unlist(thr_list, use.names = FALSE)
  }

  if (use_parallel) {
    results_raw <- run_pca_sim_parallel(
      iterations,
      sim_seeds,
      sim_data_list,
      n_cores,
      verbose
    )
  } else {
    results_raw <- run_pca_sim_sequential(
      iterations,
      sim_seeds,
      sim_data_list,
      verbose
    )
  }

  ok <- vapply(results_raw, is.numeric, logical(1L))
  successful <- results_raw[ok]

  if (length(successful) == 0L) {
    failed_msgs <- unlist(results_raw[!ok])
    sample_msg <- if (length(failed_msgs) > 0L) {
      unique(failed_msgs)[1L]
    } else {
      "(no message captured)"
    }
    stop(
      "All simulation iterations failed. Example: ",
      sample_msg,
      call. = FALSE
    )
  }

  actual_iterations <- length(successful)
  eig_vec <- as.numeric(unlist(successful))

  results_df <- data.frame(
    iteration = seq_len(actual_iterations),
    eigenvalue = eig_vec,
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  p95 <- as.numeric(stats::quantile(eig_vec, 0.95, na.rm = TRUE))
  p99 <- as.numeric(stats::quantile(eig_vec, 0.99, na.rm = TRUE))
  p995 <- as.numeric(stats::quantile(eig_vec, 0.995, na.rm = TRUE))
  p999 <- as.numeric(stats::quantile(eig_vec, 0.999, na.rm = TRUE))

  list(
    results = results_df,
    p95 = p95,
    p99 = p99,
    p995 = p995,
    p999 = p999,
    max = max(eig_vec, na.rm = TRUE),
    suggested_cutoff = p99,
    suggested_cutoff_percentile = 99,
    actual_iterations = actual_iterations,
    sample_n = sample_n,
    item_names = item_names_vec
  )
}

# ===========================================================================
# Internal: single iteration
# ===========================================================================

#' Run a single PCA-eigenvalue simulation iteration
#'
#' @keywords internal
#' @noRd
run_single_pca_sim <- function(seed, data_list) {
  set.seed(seed)
  thetas_res <- sample(
    data_list$thetas,
    size = data_list$sample_n,
    replace = TRUE
  )

  tryCatch(
    {
      if (data_list$type == "dichotomous") {
        sim_mat <- psychotools::rrm(
          theta = thetas_res,
          beta = data_list$item_params
        )
        sim_df <- as.data.frame(sim_mat$data)
        colnames(sim_df) <- data_list$item_names

        pos_counts <- colSums(sim_df, na.rm = TRUE)
        if (any(pos_counts < 8L)) {
          return(
            "validation_failed: fewer than 8 positive responses in at least one item"
          )
        }
        neg_counts <- nrow(sim_df) - pos_counts
        if (any(neg_counts < 8L)) {
          return(
            "validation_failed: fewer than 8 negative responses in at least one item"
          )
        }
      } else {
        sim_mat <- sim_partial_score(data_list$deltaslist, thetas_res)
        sim_df <- as.data.frame(sim_mat)
        colnames(sim_df) <- data_list$item_names

        n_cats <- vapply(
          data_list$deltaslist,
          function(d) length(d) + 1L,
          integer(1L)
        )
        for (j in seq_len(ncol(sim_df))) {
          tab <- tabulate(sim_df[[j]] + 1L, nbins = n_cats[j])
          if (any(tab == 0L)) {
            return("validation_failed: not all categories represented")
          }
        }
      }

      # CML/WLE standardized residuals (same engine as the observed analysis).
      st_resids <- .rasch_std_residuals(sim_df, method = "WLE")

      if (anyNA(st_resids)) {
        keep <- stats::complete.cases(st_resids)
        st_resids <- st_resids[keep, , drop = FALSE]
      }
      if (nrow(st_resids) < ncol(st_resids)) {
        return("validation_failed: too few rows in residual matrix for PCA")
      }

      pca_fit <- stats::prcomp(st_resids)
      as.numeric(pca_fit$sdev[1L]^2)
    },
    error = function(e) as.character(conditionMessage(e))
  )
}

# ===========================================================================
# Internal: parallel runner
# ===========================================================================

#' @keywords internal
#' @noRd
run_pca_sim_parallel <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  n_cores,
  verbose = FALSE
) {
  mirai::daemons(n_cores)
  on.exit(mirai::daemons(0), add = TRUE)

  if (verbose) {
    message(sprintf("Starting %d daemons...", n_cores))
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  tasks <- lapply(seq_len(iterations), function(i) {
    mirai::mirai(
      {
        run_single_pca_sim(seed, data_list)
      },
      seed = sim_seeds[i],
      data_list = sim_data_list,
      run_single_pca_sim = run_single_pca_sim,
      sim_partial_score = sim_partial_score,
      sim_poly_item = sim_poly_item,
      # CML/WLE residual engine used by .rasch_std_residuals() in the daemon.
      .rasch_std_residuals = .rasch_std_residuals,
      .fit_cml_thresholds = .fit_cml_thresholds,
      .estimate_thetas = .estimate_thetas,
      .theta_wle = .theta_wle,
      .pcm_cat_probs = .pcm_cat_probs,
      .center_thresholds = .center_thresholds
    )
  })

  results <- vector("list", iterations)
  for (i in seq_len(iterations)) {
    res <- mirai::call_mirai(tasks[[i]])$data
    results[[i]] <- if (inherits(res, "errorValue")) "mirai_error" else res
    if (verbose) utils::setTxtProgressBar(pb, i)
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# ===========================================================================
# Internal: sequential runner
# ===========================================================================

# ===========================================================================
# Internal: PCM expected-score matrix (for the variance partition)
# ===========================================================================

#' Expected scores per (person, item) under a Partial Credit Model
#'
#' Computes \deqn{E[X_ij | theta_i, tau_j]} under the PCM:
#' \deqn{P(X = x | theta, tau) = exp(x*theta - sum_{l=1}^x tau_l) /
#'                              sum_{m=0}^M exp(m*theta - sum_{l=1}^m tau_l)}
#' Loops over items but is vectorised across persons within an item.
#'
#' @param thetas Numeric vector of person locations (length n).
#' @param thresh_mat Numeric matrix of thresholds (n_items rows; columns are
#'   threshold positions, NA-padded for items with fewer thresholds).
#'
#' @return Numeric matrix (n persons x n_items) of expected scores.
#'
#' @keywords internal
#' @noRd
.pcm_expected_scores <- function(thetas, thresh_mat) {
  n <- length(thetas)
  n_items <- nrow(thresh_mat)
  expected <- matrix(NA_real_, nrow = n, ncol = n_items)

  for (j in seq_len(n_items)) {
    taus <- thresh_mat[j, !is.na(thresh_mat[j, ])]
    K_j <- length(taus)
    cumsum_taus <- c(0, cumsum(taus)) # length K_j + 1
    log_num <- outer(thetas, 0:K_j) -
      matrix(cumsum_taus, nrow = n, ncol = K_j + 1L, byrow = TRUE)
    # Numerical-stability shift before exponentiating
    log_num <- log_num - apply(log_num, 1L, max)
    probs <- exp(log_num)
    probs <- probs / rowSums(probs)
    expected[, j] <- as.numeric(probs %*% (0:K_j))
  }
  expected
}

# ===========================================================================

#' @keywords internal
#' @noRd
run_pca_sim_sequential <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  verbose = FALSE
) {
  if (verbose) {
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  results <- vector("list", iterations)
  for (i in seq_len(iterations)) {
    results[[i]] <- run_single_pca_sim(sim_seeds[i], sim_data_list)
    if (verbose) utils::setTxtProgressBar(pb, i)
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# Tests for RMdimResidualPCA() and RMdimResidualPCACutoff()

make_dichotomous <- function(n = 200, k = 10, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

make_polytomous <- function(n = 200, k = 6, seed = 2L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:2, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# RMdimResidualPCA -- input validation
# ---------------------------------------------------------------------
test_that("RMdimResidualPCA errors when data has non-zero minimum", {
  df <- make_dichotomous() + 1L
  expect_error(RMdimResidualPCA(df), regexp = "scored starting at 0")
})

test_that("RMdimResidualPCA errors when cutoff is the wrong type", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  expect_error(RMdimResidualPCA(df, cutoff = "abc"),
               regexp = "cutoff")
})

# ---------------------------------------------------------------------
# RMdimResidualPCA -- output structure
# ---------------------------------------------------------------------
test_that("RMdimResidualPCA output = 'dataframe' returns eigenvalue table + variance_partition attr", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  res <- RMdimResidualPCA(df, output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_named(res, c("Component", "Eigenvalue", "Proportion_of_variance"))
  expect_equal(nrow(res), 5L)  # default n_components
  expect_true(all(res$Eigenvalue > 0))
  expect_true(all(res$Proportion_of_variance > 0 & res$Proportion_of_variance <= 1))

  vp <- attr(res, "variance_partition")
  expect_type(vp, "list")
  expect_true(all(c("total", "explained", "unexplained",
                    "pct_explained", "pct_unexplained",
                    "n_persons") %in% names(vp)))
})

test_that("RMdimResidualPCA output = 'kable' returns knitr_kable", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("knitr")
  df  <- make_dichotomous()
  out <- RMdimResidualPCA(df, output = "kable")
  expect_s3_class(out, "knitr_kable")
})

test_that("RMdimResidualPCA output = 'ggplot' returns a ggplot", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_dichotomous()
  p  <- RMdimResidualPCA(df, output = "ggplot")
  expect_s3_class(p, "ggplot")
})

test_that("RMdimResidualPCA output = 'loadings' is a backward-compatible alias for 'ggplot'", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_dichotomous()
  p  <- RMdimResidualPCA(df, output = "loadings")
  expect_s3_class(p, "ggplot")
})

test_that("RMdimResidualPCA auto-picks PCM on polytomous data", {
  skip_if_not_installed("eRm")
  df <- make_polytomous()
  res <- RMdimResidualPCA(df, output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_true(nrow(res) >= 1L)
})

test_that("RMdimResidualPCA accepts numeric cutoff and adds Flagged column", {
  skip_if_not_installed("eRm")
  df  <- make_dichotomous()
  res <- RMdimResidualPCA(df, cutoff = 2.0, output = "dataframe")
  expect_true("Flagged" %in% names(res))
  expect_type(res$Flagged, "logical")
})

test_that("RMdimResidualPCA n_components controls number of rows", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  res <- RMdimResidualPCA(df, n_components = 3L, output = "dataframe")
  expect_equal(nrow(res), 3L)
})

# ---------------------------------------------------------------------
# RMdimResidualPCACutoff -- small iterations to keep tests fast
# ---------------------------------------------------------------------
test_that("RMdimResidualPCACutoff returns a list with simulated eigenvalues + percentile cutoffs", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  df  <- make_dichotomous()
  res <- RMdimResidualPCACutoff(df, iterations = 10L, parallel = FALSE, seed = 1L)
  expect_type(res, "list")
  expect_true(all(c("results", "p95", "p99", "p995", "p999",
                    "suggested_cutoff", "actual_iterations",
                    "sample_n", "item_names") %in% names(res)))
  expect_s3_class(res$results, "data.frame")
  expect_true(res$actual_iterations >= 1L)
  expect_true(res$actual_iterations <= 10L)
  expect_true(is.finite(res$suggested_cutoff))
})

test_that("RMdimResidualPCA accepts an RMdimResidualPCACutoff result for cutoff arg", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  bound <- RMdimResidualPCACutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)
  res   <- RMdimResidualPCA(df, cutoff = bound, output = "dataframe")
  expect_true("Flagged" %in% names(res))
})

test_that("RMdimResidualPCA p_value requires the full cutoff object", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  expect_error(
    RMdimResidualPCA(df, cutoff = 1.8, p_value = TRUE),
    regexp = "full RMdimResidualPCACutoff"
  )
  expect_error(
    RMdimResidualPCA(df, p_value = TRUE),
    regexp = "full RMdimResidualPCACutoff"
  )
})

test_that("RMdimResidualPCA p_value adds a one-sided p for PC1 only", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  bound <- RMdimResidualPCACutoff(df, iterations = 10L, parallel = FALSE,
                                  seed = 1L)
  res <- RMdimResidualPCA(df, cutoff = bound, p_value = TRUE,
                          output = "dataframe")
  expect_true("p" %in% names(res))
  # p is rounded to 4 dp in the table, so compare against the rounded floor
  B <- bound$actual_iterations
  expect_gte(res$p[1L], round(1 / (B + 1), 4))
  expect_lte(res$p[1L], 1)
  expect_true(all(is.na(res$p[-1L])))
  # kable output carries the extra column without erroring
  kbl <- RMdimResidualPCA(df, cutoff = bound, p_value = TRUE)
  expect_s3_class(kbl, "knitr_kable")
})

test_that("RMdimResidualPCACutoff is reproducible with the same seed", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  r1 <- RMdimResidualPCACutoff(df, iterations = 5L, parallel = FALSE, seed = 42L)
  r2 <- RMdimResidualPCACutoff(df, iterations = 5L, parallel = FALSE, seed = 42L)
  expect_equal(r1$results, r2$results)
})

test_that("RMdimResidualPCA warns on sparse/zero-variance items (CML)", {
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  df[[1]] <- 0L   # constant item -> destabilises CML; should warn, not fail silently
  expect_warning(try(RMdimResidualPCA(df, output = "dataframe"), silent = TRUE),
                 regexp = "[Ss]parse|zero-variance")
})

# Internal theme helpers shared across easyRasch2 plotting functions.

#' Standard easyRasch2 axis-title margins
#'
#' Returns the ggplot2 theme tweak used throughout easyRasch2 to add a
#' little extra breathing room around the x and y axis titles. Add to
#' any ggplot via `p + er2_axis_margins()`.
#'
#' Implemented as a `ggplot2::theme()` object that sets only the
#' `margin` slot of `axis.title.x` and `axis.title.y` --- other axis-
#' title attributes (size, face, family, colour) inherit from the
#' parent theme and are not disturbed.
#'
#' @return A `ggplot2::theme()` object.
#' @keywords internal
#' @noRd
er2_axis_margins <- function() {
  ggplot2::theme(
    axis.title.x = ggplot2::element_text(margin = ggplot2::margin(t = 12)),
    axis.title.y = ggplot2::element_text(margin = ggplot2::margin(r = 12))
  )
}

#' APA-style plot.caption theme element
#'
#' Returns a `ggplot2::theme()` object that sets `plot.caption` so it
#' renders left-aligned with the APA-conventional italic "Note." prefix
#' followed by roman body text. When `ggtext` is installed
#' (recommended; in Suggests), the caption is rendered via
#' `ggtext::element_markdown()` which lets the italic "*Note.*" prefix
#' coexist with roman body text in a single caption. When `ggtext` is
#' not installed, falls back to a plain `element_text(face = "italic")`
#' --- the whole caption goes italic, and the prefix collapses to a
#' plain "Note. " (no asterisks leaked).
#'
#' Pair with \code{\link{er2_caption}} when building caption strings,
#' so the "Note." prefix matches whichever renderer is active.
#'
#' @return A `ggplot2::theme()` object.
#' @keywords internal
#' @noRd
er2_plot_caption <- function() {
  ggplot2::theme(
    plot.caption = if (requireNamespace("ggtext", quietly = TRUE)) {
      ggtext::element_markdown(hjust = 0, size = 9)
    } else {
      ggplot2::element_text(hjust = 0, face = "italic", size = 9)
    }
  )
}

#' APA-style "Note." caption-text prefix with line wrapping
#'
#' Build a plot caption with the standard "Note." prefix and wrap the
#' body text at `width` characters so long captions don't run off the
#' right edge of the plot. The body text is wrapped first (via
#' [strwrap()]) then prefixed --- so the prefix never gets broken by a
#' line break, and the markdown asterisks (when `ggtext` is installed)
#' stay together.
#'
#' When `ggtext` is installed, returns `"*Note.* <wrapped body>"`
#' (markdown italic on "Note."); otherwise `"Note. <wrapped body>"` (plain
#' text, with the whole caption set italic by \code{\link{er2_plot_caption}}).
#'
#' Wrapped lines are joined with the CommonMark "hard line break" token --- two
#' trailing spaces before a newline (`"  \\n"`). This is honoured as a line
#' break by **both** caption renderers: `ggtext::element_markdown()` (markdown,
#' where a bare `"\\n"` would instead collapse to a space) and the plain
#' `element_text()` fallback. So wrapping survives even on a plot that forgets
#' to apply \code{\link{er2_plot_caption}}; that pairing then only governs the
#' italic "Note." styling, not the wrapping.
#'
#' @param text Character. The caption body text (everything after the
#'   "Note." prefix). Pre-existing `\\n` in `text` is treated as
#'   ordinary whitespace by `strwrap()`; the wrapped output reflows
#'   to fit `width`.
#' @param width Integer. Maximum characters per line, passed to
#'   [strwrap()] after subtracting the prefix length. Default `90L`
#'   (suits the package's default 8-inch plot at 144 dpi).
#' @return A character string ready to pass to
#'   `ggplot2::labs(caption = ...)`.
#' @keywords internal
#' @noRd
er2_caption <- function(text, width = 90L) {
  prefix     <- if (requireNamespace("ggtext", quietly = TRUE)) "*Note.* " else "Note. "
  body_width <- max(20L, width - nchar(prefix))
  # "  \n" = CommonMark hard line break: rendered as a break by both
  # element_markdown() (markdown) and element_text() (plain). A bare "\n"
  # would be collapsed to a space by element_markdown().
  body <- paste(strwrap(text, width = body_width), collapse = "  \n")
  paste0(prefix, body)
}

#' Simulation-Based Infit MSQ Cutoff Determination
#'
#' Uses parametric bootstrap simulation to determine appropriate cutoff values
#' for \code{\link{RMitemInfit}}. This function simulates data from a correctly fitting
#' Rasch model that mimics your data and returns per-item empirical cutoffs.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Only complete cases (rows without
#'   any `NA`) are used.
#' @param iterations Integer. Number of simulation iterations (default 250).
#' @param parallel Logical. Use parallel processing via `mirai` if available
#'   (default `TRUE`).
#' @param n_cores Integer or `NULL`. Number of parallel workers. When `NULL`,
#'   `getOption("mc.cores")` is checked first. If neither is set and
#'   `parallel = TRUE`, a warning is issued and execution falls back to
#'   sequential (single core) processing.
#' @param verbose Logical. Show a progress bar (default `FALSE`).
#' @param seed Integer or `NULL`. Random seed for reproducibility.
#' @param cutoff_method Character string specifying how cutoff intervals are
#'   computed. Either `"hdci"` (default) for the Highest Density Interval via
#'   `ggdist::hdci()`, or `"quantile"` for the 2.5th/97.5th percentiles via
#'   `stats::quantile()`.
#' @param hdci_width Numeric. Width of the HDCI when `cutoff_method = "hdci"`.
#'   Default is `0.999` (99.9% HDCI). Ignored when `cutoff_method = "quantile"`.
#' @param dgp Character. Data-generating process for the parametric bootstrap.
#'   `"resample"` (default) resamples WLE person locations with replacement and
#'   simulates responses under the model (a *marginal* null). `"conditional"`
#'   simulates each respondent's pattern from the exact Rasch conditional
#'   distribution given their observed total score, item parameters fixed (a
#'   *conditional* null). Because the conditional infit/outfit statistic is
#'   itself conditional on the total score, `"conditional"` is its naturally
#'   matched null. \strong{Experimental.}
#'
#' @return A list with components:
#' \describe{
#'   \item{`results`}{data.frame with columns `iteration`, `Item`,
#'     `InfitMSQ`, `OutfitMSQ` (one row per item per successful iteration).}
#'   \item{`item_cutoffs`}{data.frame with per-item cutoff summaries: `Item`,
#'     `infit_low`, `infit_high`, `outfit_low`, `outfit_high`. Bounds are
#'     computed using the method specified by `cutoff_method`.}
#'   \item{`actual_iterations`}{Number of successful iterations.}
#'   \item{`sample_n`}{Number of complete cases used.}
#'   \item{`sample_n_total`}{Number of respondents in the raw input data,
#'     before the complete-case filter.}
#'   \item{`sample_has_na`}{Logical. Whether the raw input data contained
#'     any missing values.}
#'   \item{`sample_summary`}{Summary statistics of estimated person parameters.}
#'   \item{`item_names`}{Character vector of item names from data.}
#'   \item{`cutoff_method`}{The method used to compute cutoffs (`"hdci"` or
#'     `"quantile"`).}
#'   \item{`hdci_width`}{The HDCI width used (only meaningful when
#'     `cutoff_method = "hdci"`).}
#'   \item{`dgp`}{The data-generating process used (`"resample"` or
#'     `"conditional"`).}
#' }
#'
#' @details
#' The generating model is CML item parameters (via `psychotools`) with WLE
#' person locations. For each iteration a dataset is simulated under the chosen
#' `dgp`, the model is refitted by CML (`psychotools::pcmodel()`, which handles
#' dichotomous and polytomous data and is accepted by `iarm`), and conditional
#' infit and outfit MSQ are computed via `iarm::out_infit()`. The distribution
#' of these statistics across iterations provides empirical critical values per
#' item. Failed iterations (e.g., degenerate simulated data) are silently
#' discarded.
#'
#' Parallel processing is provided by the `mirai` package (optional). Install
#' it with `install.packages("mirai")` to enable parallelisation.
#'
#' The `iarm` package must be installed (it is in Suggests, not Imports).
#'
#' @seealso \code{\link{RMitemInfit}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE) &&
#'     requireNamespace("ggdist", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:10)
#'
#'   # Run 100 iterations sequentially for a quick demo
#'   cutoff_res <- RMitemInfitCutoff(sim_data, iterations = 100,
#'                                   parallel = FALSE, seed = 42)
#'   cutoff_res$item_cutoffs
#'
#'   # Use the cutoffs in RMitemInfit()
#'   RMitemInfit(sim_data)
#' }
#' }
RMitemInfitCutoff <- function(
  data,
  iterations = 250,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  cutoff_method = "hdci",
  hdci_width = 0.999,
  dgp = c("resample", "conditional")
) {
  dgp <- match.arg(dgp)
  cutoff_method <- match.arg(cutoff_method, c("hdci", "quantile"))

  if (cutoff_method == "hdci" && !requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required when cutoff_method = \"hdci\" but is not installed.\n",
      "Install it with: install.packages(\"ggdist\")\n",
      "Alternatively, use cutoff_method = \"quantile\" to avoid this dependency.",
      call. = FALSE
    )
  }

  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required for RMitemInfitCutoff() but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }

  validate_response_data(data)

  # rgl workaround
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # Only complete cases (matching RMitemInfit behaviour). Record the raw
  # total and whether anything was dropped so callers (e.g. RMitemInfitPlot)
  # can report the sample in the standard `n = X of Y respondents` form.
  n_total <- nrow(as.data.frame(data))
  has_na <- anyNA(data)
  data <- stats::na.omit(data)
  if (nrow(data) == 0L) {
    stop(
      "No complete cases in data. All rows contain at least one NA.",
      call. = FALSE
    )
  }

  use_parallel <- parallel && requireNamespace("mirai", quietly = TRUE)

  if (parallel && !use_parallel) {
    message(
      "Install 'mirai' package for parallel processing: install.packages(\"mirai\")"
    )
    message("Running sequentially...")
  }

  if (use_parallel) {
    if (is.null(n_cores)) {
      n_cores <- getOption("mc.cores")
    }
    if (is.null(n_cores)) {
      warning(
        paste0(
          "For parallel processing, specify n_cores or set options(mc.cores = N).\n",
          "(Use `parallel::detectCores()` to see how many cores are available.)\n",
          "Falling back to sequential (single core) processing."
        ),
        call. = FALSE
      )
      use_parallel <- FALSE
    } else {
      n_cores <- min(n_cores, iterations)
    }
  }

  if (!is.null(seed)) {
    set.seed(seed)
  }

  # Generate per-iteration seeds
  sim_seeds <- sample.int(.Machine$integer.max, iterations)

  data_mat <- as.matrix(data)
  sample_n <- nrow(data_mat)
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L

  item_names_vec <- colnames(data_mat)

  # Generating model: CML item thresholds (psychotools), computed once. The
  # conditional infit statistic is conditional on the total score, so the
  # "conditional" DGP (simulate each respondent's pattern given their observed
  # score) is the matched null. The "resample" DGP draws WLE person locations
  # with replacement and simulates parametrically (a marginal null).
  pool <- .wle_theta_pool(data_mat)
  thr_list <- pool$thr_list
  wle_thetas <- pool$thetas

  sim_data_list <- list(
    dgp = dgp,
    type = if (is_polytomous) "polytomous" else "dichotomous",
    thr_list = thr_list,
    n_items = ncol(data_mat),
    sample_n = sample_n,
    item_names = item_names_vec
  )
  if (dgp == "resample") {
    sim_data_list$thetas <- wle_thetas
    if (is_polytomous) {
      sim_data_list$deltaslist <- thr_list
    } else {
      sim_data_list$item_params <- unlist(thr_list, use.names = FALSE)
    }
  } else {
    sim_data_list$cond_groups <- .cond_groups(data_mat, thr_list)
  }

  if (use_parallel) {
    results_raw <- run_infit_sim_parallel(
      iterations,
      sim_seeds,
      sim_data_list,
      n_cores,
      verbose
    )
  } else {
    results_raw <- run_infit_sim_sequential(
      iterations,
      sim_seeds,
      sim_data_list,
      verbose
    )
  }

  # Filter out failures (character strings indicate errors)
  ok <- vapply(results_raw, is.list, logical(1L))
  successful <- results_raw[ok]

  if (length(successful) == 0L) {
    stop("All simulation iterations failed. Check your data.", call. = FALSE)
  }

  actual_iterations <- length(successful)

  # Combine per-iteration data.frames
  iter_dfs <- lapply(seq_along(successful), function(i) {
    df <- successful[[i]]
    df$iteration <- i
    df
  })
  results_df <- do.call(rbind, iter_dfs)
  rownames(results_df) <- NULL

  # Compute per-item cutoffs
  item_names <- unique(results_df$Item)
  item_cutoffs <- do.call(
    rbind,
    lapply(item_names, function(item) {
      sub <- results_df[results_df$Item == item, ]
      if (cutoff_method == "hdci") {
        # ggdist::hdci() returns a matrix with ncol = 2: column 1 is the lower
        # bound, column 2 is the upper bound. Row 1 contains the continuous
        # interval.
        infit_interval <- ggdist::hdci(sub$InfitMSQ, .width = hdci_width)
        outfit_interval <- ggdist::hdci(sub$OutfitMSQ, .width = hdci_width)
        data.frame(
          Item = item,
          infit_low = infit_interval[1L, 1L],
          infit_high = infit_interval[1L, 2L],
          outfit_low = outfit_interval[1L, 1L],
          outfit_high = outfit_interval[1L, 2L],
          stringsAsFactors = FALSE,
          row.names = NULL
        )
      } else {
        data.frame(
          Item = item,
          infit_low = stats::quantile(sub$InfitMSQ, 0.025, na.rm = TRUE),
          infit_high = stats::quantile(sub$InfitMSQ, 0.975, na.rm = TRUE),
          outfit_low = stats::quantile(sub$OutfitMSQ, 0.025, na.rm = TRUE),
          outfit_high = stats::quantile(sub$OutfitMSQ, 0.975, na.rm = TRUE),
          stringsAsFactors = FALSE,
          row.names = NULL
        )
      }
    })
  )
  rownames(item_cutoffs) <- NULL

  list(
    results = results_df,
    item_cutoffs = item_cutoffs,
    actual_iterations = actual_iterations,
    sample_n = sample_n,
    sample_n_total = n_total,
    sample_has_na = has_na,
    sample_summary = summary(wle_thetas),
    item_names = item_names_vec,
    cutoff_method = cutoff_method,
    hdci_width = hdci_width,
    dgp = dgp
  )
}

# ---------------------------------------------------------------------------
# Internal: single simulation iteration
# ---------------------------------------------------------------------------

#' Run a single infit simulation iteration
#'
#' @param seed Integer seed for reproducibility.
#' @param data_list List produced inside [RMitemInfitCutoff()].
#' @return A data.frame with columns `Item`, `InfitMSQ`, `OutfitMSQ`, or a
#'   character string on failure.
#' @keywords internal
run_single_infit_sim <- function(seed, data_list) {
  set.seed(seed)

  tryCatch(
    {
      # --- Generate one simulated dataset under the chosen DGP -----------------
      if (identical(data_list$dgp, "conditional")) {
        sim_df <- .sim_cond_dataset(data_list)
      } else if (data_list$type == "dichotomous") {
        thetas_res <- sample(
          data_list$thetas,
          size = data_list$sample_n,
          replace = TRUE
        )
        sim_df <- as.data.frame(
          psychotools::rrm(
            theta = thetas_res,
            beta = data_list$item_params
          )$data
        )
      } else {
        thetas_res <- sample(
          data_list$thetas,
          size = data_list$sample_n,
          replace = TRUE
        )
        sim_df <- as.data.frame(sim_partial_score(
          data_list$deltaslist,
          thetas_res
        ))
      }
      colnames(sim_df) <- data_list$item_names

      # --- Validate the simulated dataset (estimable refit) --------------------
      if (data_list$type == "dichotomous") {
        if (any(colSums(sim_df, na.rm = TRUE) < 8L)) {
          return(
            "validation_failed: fewer than 8 positive responses in at least one item"
          )
        }
      } else {
        n_cats <- vapply(
          data_list$thr_list,
          function(d) length(d) + 1L,
          integer(1L)
        )
        for (j in seq_len(ncol(sim_df))) {
          tab <- tabulate(sim_df[[j]] + 1L, nbins = n_cats[j])
          if (any(tab == 0L)) {
            return("validation_failed: not all categories represented")
          }
        }
      }

      # Conditional infit/outfit from a CML refit. psychotools::pcmodel() handles
      # both polytomous and dichotomous (a 2-category PCM is the Rasch model) and
      # is accepted by iarm::out_infit(); it matches eRm to ~1e-6 but is faster.
      model_fit <- psychotools::pcmodel(sim_df)
      cfit <- iarm::out_infit(model_fit)

      data.frame(
        Item = data_list$item_names,
        InfitMSQ = round(cfit$Infit, 3),
        OutfitMSQ = round(cfit$Outfit, 3),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    },
    error = function(e) {
      as.character(conditionMessage(e))
    }
  )
}

# ---------------------------------------------------------------------------
# Internal: parallel runner
# ---------------------------------------------------------------------------

#' Run infit simulations in parallel using mirai
#'
#' @param iterations Number of iterations.
#' @param sim_seeds Integer vector of per-iteration seeds.
#' @param sim_data_list List of data passed to each worker.
#' @param n_cores Number of mirai daemons.
#' @param verbose Show progress bar.
#' @return List of raw results (one element per iteration).
#' @keywords internal
run_infit_sim_parallel <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  n_cores,
  verbose = FALSE
) {
  mirai::daemons(n_cores)
  on.exit(mirai::daemons(0), add = TRUE)

  if (verbose) {
    message(sprintf("Starting %d daemons...", n_cores))
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
    completed <- 0L
  }

  # Submit all tasks
  tasks <- lapply(seq_len(iterations), function(sim) {
    mirai::mirai(
      {
        run_single_infit_sim(seed, data_list)
      },
      seed = sim_seeds[sim],
      data_list = sim_data_list,
      run_single_infit_sim = run_single_infit_sim,
      sim_partial_score = sim_partial_score,
      sim_poly_item = sim_poly_item,
      # Conditional-DGP generators (shared with the Q3 cutoff).
      .sim_cond_dataset = .sim_cond_dataset,
      .sim_conditional = .sim_conditional,
      .esf_convolve = .esf_convolve
    )
  })

  # Collect results
  results <- vector("list", iterations)
  for (sim in seq_len(iterations)) {
    result <- mirai::call_mirai(tasks[[sim]])$data
    if (!inherits(result, "errorValue")) {
      results[[sim]] <- result
    } else {
      results[[sim]] <- "mirai_error"
    }
    if (verbose) {
      completed <- completed + 1L
      utils::setTxtProgressBar(pb, completed)
    }
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# ---------------------------------------------------------------------------
# Internal: sequential runner
# ---------------------------------------------------------------------------

#' Run infit simulations sequentially
#'
#' @param iterations Number of iterations.
#' @param sim_seeds Integer vector of per-iteration seeds.
#' @param sim_data_list List of data passed to each worker.
#' @param verbose Show progress bar.
#' @return List of raw results (one element per iteration).
#' @keywords internal
run_infit_sim_sequential <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  verbose = FALSE
) {
  if (verbose) {
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  results <- vector("list", iterations)
  for (sim in seq_len(iterations)) {
    results[[sim]] <- run_single_infit_sim(sim_seeds[sim], sim_data_list)
    if (verbose) {
      utils::setTxtProgressBar(pb, sim)
    }
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

test_that("RMitemInfit errors when iarm is not installed", {
  skip_if(
    requireNamespace("iarm", quietly = TRUE),
    "iarm is installed; skipping missing-package error path test"
  )
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5))
  expect_error(RMitemInfit(df), regexp = "iarm")
})

test_that("RMitemInfit errors when data does not start at 0", {
  skip_if_not_installed("iarm")
  df <- as.data.frame(matrix(sample(1:3, 200, replace = TRUE), nrow = 40, ncol = 5))
  expect_error(RMitemInfit(df), regexp = "scored starting at 0")
})

test_that("RMitemInfit errors when data is all NA", {
  skip_if_not_installed("iarm")
  df <- as.data.frame(matrix(NA_integer_, nrow = 10, ncol = 4))
  expect_error(RMitemInfit(df), regexp = "No complete|no non-missing")
})

test_that("RMitemInfit errors when data is not a data.frame or matrix", {
  skip_if_not_installed("iarm")
  expect_error(RMitemInfit(list(a = 1:5)), regexp = "data.frame or matrix")
})

test_that("RMitemInfit errors when no complete cases", {
  skip_if_not_installed("iarm")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5))
  # Make all rows have at least one NA by cycling through columns
  df[cbind(seq_len(nrow(df)), ((seq_len(nrow(df)) - 1L) %% ncol(df)) + 1L)] <- NA
  expect_error(RMitemInfit(df), regexp = "No complete cases")
})

test_that("RMitemInfit output = 'dataframe' returns correct structure (dichotomous)", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  result <- RMitemInfit(df, output = "dataframe")

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 5L)
  expect_equal(names(result), c("Item", "Infit_MSQ", "Relative_location"))
  expect_equal(result$Item, colnames(df))
  expect_true(all(result$Infit_MSQ > 0))
})

test_that("RMitemInfit output = 'kable' returns knitr_kable with complete-cases caption", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)
  df[1, 1] <- NA # one incomplete row so the caption shows the drop + label

  result <- RMitemInfit(df, output = "kable")
  expect_s3_class(result, "knitr_kable")
  cap <- attr(result, "caption")
  if (is.null(cap)) cap <- paste(as.character(result), collapse = "\n")
  expect_true(grepl("of 40 respondents \\(complete cases\\)", cap))
})

test_that("RMitemInfit sort = 'infit' sorts by Infit_MSQ descending", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  result_sorted   <- RMitemInfit(df, output = "dataframe", sort = "infit")
  result_unsorted <- RMitemInfit(df, output = "dataframe")

  expect_true(
    all(diff(result_sorted$Infit_MSQ) <= 0),
    info = "Rows should be in descending order of Infit_MSQ"
  )
  expect_setequal(result_sorted$Item, result_unsorted$Item)
})

# ---------------------------------------------------------------------------
# cutoff parameter tests
# ---------------------------------------------------------------------------

test_that("RMitemInfit cutoff = NULL (default) behaves as before", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  result <- RMitemInfit(df, cutoff = NULL, output = "dataframe")
  expect_equal(names(result), c("Item", "Infit_MSQ", "Relative_location"))
})

test_that("RMitemInfit with mock cutoff data.frame adds expected columns", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  mock_cutoff <- data.frame(
    Item        = paste0("Item", 1:5),
    infit_low   = rep(0.7, 5),
    infit_high  = rep(1.3, 5),
    outfit_low  = rep(0.7, 5),
    outfit_high = rep(1.3, 5),
    stringsAsFactors = FALSE
  )

  result <- RMitemInfit(df, cutoff = mock_cutoff, output = "dataframe")
  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 5L)
  expect_equal(
    names(result),
    c("Item", "Infit_MSQ", "Infit_low", "Infit_high", "Flagged", "Relative_location")
  )
  expect_type(result$Flagged, "character")
  expect_true(all(result$Flagged %in% c("overfit", "underfit", "")))
  expect_true(all(result$Infit_low  == 0.7))
  expect_true(all(result$Infit_high == 1.3))
})

test_that("RMitemInfit with cutoff list (from RMitemInfitCutoff shape) extracts item_cutoffs", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  mock_cutoff_list <- list(
    item_cutoffs = data.frame(
      Item        = paste0("Item", 1:5),
      infit_low   = rep(0.7, 5),
      infit_high  = rep(1.3, 5),
      outfit_low  = rep(0.7, 5),
      outfit_high = rep(1.3, 5),
      stringsAsFactors = FALSE
    ),
    actual_iterations = 100L
  )

  result_df <- RMitemInfit(df, cutoff = mock_cutoff_list, output = "dataframe")
  expect_equal(
    names(result_df),
    c("Item", "Infit_MSQ", "Infit_low", "Infit_high", "Flagged", "Relative_location")
  )

  result_kable <- RMitemInfit(df, cutoff = mock_cutoff_list, output = "kable")
  expect_s3_class(result_kable, "knitr_kable")
  cap <- attr(result_kable, "caption")
  if (is.null(cap)) cap <- paste(as.character(result_kable), collapse = "\n")
  expect_true(grepl("100 simulation iterations", cap))
})

test_that("RMitemInfit caption includes HDCI info when cutoff_method = 'hdci'", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  mock_cutoff_hdci <- list(
    item_cutoffs = data.frame(
      Item        = paste0("Item", 1:5),
      infit_low   = rep(0.7, 5),
      infit_high  = rep(1.3, 5),
      outfit_low  = rep(0.7, 5),
      outfit_high = rep(1.3, 5),
      stringsAsFactors = FALSE
    ),
    actual_iterations = 200L,
    cutoff_method     = "hdci",
    hdci_width         = 0.999
  )

  result_kable <- RMitemInfit(df, cutoff = mock_cutoff_hdci, output = "kable")
  cap <- attr(result_kable, "caption")
  if (is.null(cap)) cap <- paste(as.character(result_kable), collapse = "\n")
  expect_true(grepl("200 simulation iterations", cap))
  expect_true(grepl("HDCI", cap))
  expect_true(grepl("99.9", cap))
})

test_that("RMitemInfit caption includes quantile info when cutoff_method = 'quantile'", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  mock_cutoff_quantile <- list(
    item_cutoffs = data.frame(
      Item        = paste0("Item", 1:5),
      infit_low   = rep(0.7, 5),
      infit_high  = rep(1.3, 5),
      outfit_low  = rep(0.7, 5),
      outfit_high = rep(1.3, 5),
      stringsAsFactors = FALSE
    ),
    actual_iterations = 150L,
    cutoff_method     = "quantile",
    hdci_width         = 0.999
  )

  result_kable <- RMitemInfit(df, cutoff = mock_cutoff_quantile, output = "kable")
  cap <- attr(result_kable, "caption")
  if (is.null(cap)) cap <- paste(as.character(result_kable), collapse = "\n")
  expect_true(grepl("150 simulation iterations", cap))
  expect_true(grepl("percentile", cap))
})

test_that("RMitemInfitCutoff errors when cutoff_method = 'hdci' but ggdist is not installed", {
  skip_if(
    requireNamespace("ggdist", quietly = TRUE),
    "ggdist is installed; skipping missing-package error path test"
  )
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5))
  colnames(df) <- paste0("Item", 1:5)
  expect_error(
    RMitemInfitCutoff(df, iterations = 5, parallel = FALSE, seed = 42,
                  cutoff_method = "hdci"),
    regexp = "ggdist"
  )
})

test_that("RMitemInfitCutoff cutoff_method = 'quantile' returns valid cutoffs", {
  skip_on_cran()
  # RMitemInfitCutoff() fits via psychotools (Imports) + iarm; no eRm needed.
  skip_if_not_installed("iarm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 1000, replace = TRUE), nrow = 200, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  res <- RMitemInfitCutoff(df, iterations = 20, parallel = FALSE, seed = 42,
                       cutoff_method = "quantile")

  expect_equal(res$cutoff_method, "quantile")
  # hdci_width is always stored regardless of method; verify it keeps its default
  expect_equal(res$hdci_width, 0.999)
  expect_s3_class(res$item_cutoffs, "data.frame")
  expect_equal(nrow(res$item_cutoffs), 5L)
  expect_true(all(res$item_cutoffs$infit_low < res$item_cutoffs$infit_high))
  expect_true(all(res$item_cutoffs$outfit_low < res$item_cutoffs$outfit_high))
})

test_that("RMitemInfitCutoff cutoff_method = 'hdci' returns valid cutoffs", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 1000, replace = TRUE), nrow = 200, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  res <- RMitemInfitCutoff(df, iterations = 20, parallel = FALSE, seed = 42,
                       cutoff_method = "hdci", hdci_width = 0.999)

  expect_equal(res$cutoff_method, "hdci")
  expect_equal(res$hdci_width, 0.999)
  expect_s3_class(res$item_cutoffs, "data.frame")
  expect_equal(nrow(res$item_cutoffs), 5L)
  expect_true(all(res$item_cutoffs$infit_low < res$item_cutoffs$infit_high))
  expect_true(all(res$item_cutoffs$outfit_low < res$item_cutoffs$outfit_high))
})

test_that("RMitemInfitCutoff dgp = 'conditional' runs; default is 'resample'", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  set.seed(6)
  df <- as.data.frame(matrix(sample(0:1, 200 * 6, replace = TRUE), 200, 6))
  colnames(df) <- paste0("I", 1:6)

  cu <- RMitemInfitCutoff(df, iterations = 20, parallel = FALSE, seed = 1,
                          dgp = "conditional")
  expect_identical(cu$dgp, "conditional")
  expect_s3_class(cu$item_cutoffs, "data.frame")
  expect_equal(nrow(cu$item_cutoffs), 6L)
  expect_true(all(cu$item_cutoffs$infit_low < cu$item_cutoffs$infit_high))

  # default DGP is unchanged
  cu_def <- RMitemInfitCutoff(df, iterations = 5, parallel = FALSE, seed = 1)
  expect_identical(cu_def$dgp, "resample")

  # the conditional object flows through RMitemInfit()
  skip_if_not_installed("eRm")
  res <- RMitemInfit(df, cutoff = cu, output = "dataframe")
  expect_true(is.data.frame(res))
})

test_that("RMitemInfit with cutoff data.frame (no actual_iterations) uses generic caption", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  mock_cutoff <- data.frame(
    Item        = paste0("Item", 1:5),
    infit_low   = rep(0.7, 5),
    infit_high  = rep(1.3, 5),
    stringsAsFactors = FALSE
  )

  result_kable <- RMitemInfit(df, cutoff = mock_cutoff, output = "kable")
  expect_s3_class(result_kable, "knitr_kable")
  cap <- attr(result_kable, "caption")
  if (is.null(cap)) cap <- paste(as.character(result_kable), collapse = "\n")
  expect_true(grepl("Simulation-based cutoff values applied", cap))
})

test_that("RMitemInfit errors when cutoff item names do not match data", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  bad_cutoff <- data.frame(
    Item        = paste0("X", 1:5),
    infit_low   = rep(0.7, 5),
    infit_high  = rep(1.3, 5),
    stringsAsFactors = FALSE
  )

  expect_error(
    RMitemInfit(df, cutoff = bad_cutoff),
    regexp = "Item names in `cutoff` do not match"
  )
})

test_that("RMitemInfit errors when cutoff data.frame has missing required columns", {
  skip_if_not_installed("iarm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  bad_cutoff <- data.frame(
    Item      = paste0("Item", 1:5),
    infit_low = rep(0.7, 5),
    stringsAsFactors = FALSE
  )

  expect_error(
    RMitemInfit(df, cutoff = bad_cutoff),
    regexp = "missing required columns"
  )
})

test_that("RMitemInfit Flagged column correctly identifies values outside cutoff range", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  # Compute actual infit values to build tight vs. loose cutoffs
  base_result <- RMitemInfit(df, output = "dataframe")
  infit_vals  <- base_result$Infit_MSQ

  # Tight cutoff: all items should be flagged (narrow window around each)
  tight_cutoff <- data.frame(
    Item        = paste0("Item", 1:5),
    infit_low   = infit_vals + 0.1,
    infit_high  = infit_vals + 0.2,
    stringsAsFactors = FALSE
  )
  tight_result <- RMitemInfit(df, cutoff = tight_cutoff, output = "dataframe")
  # each MSQ is below its (MSQ + 0.1) lower bound -> all flagged "overfit"
  expect_true(all(tight_result$Flagged == "overfit"))

  # Loose cutoff: no items should be flagged (window encompasses all values)
  loose_cutoff <- data.frame(
    Item        = paste0("Item", 1:5),
    infit_low   = rep(0.0, 5),
    infit_high  = rep(10.0, 5),
    stringsAsFactors = FALSE
  )
  loose_result <- RMitemInfit(df, cutoff = loose_cutoff, output = "dataframe")
  expect_true(all(loose_result$Flagged == ""))
})

test_that("RMitemInfit drops all-NA respondents instead of erroring", {
  skip_if_not_installed("iarm")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 60 * 6, replace = TRUE), nrow = 60))
  colnames(df) <- paste0("i", 1:6)
  df[3, ] <- NA

  expect_message(k <- RMitemInfit(df), "no responses dropped")
  # Caption keeps the raw total: complete cases of 60 respondents
  expect_match(paste(as.character(k), collapse = "\n"), "of 60 respondents")
})

test_that("dataframe output is unrounded (kable-only display rounding)", {
  skip_if_not_installed("iarm")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 60 * 6, replace = TRUE), nrow = 60))
  colnames(df) <- paste0("i", 1:6)

  result <- RMitemInfit(df, output = "dataframe")
  # Must match the raw iarm statistic exactly, not its 3-dp rounding
  fit <- psychotools::pcmodel(as.matrix(df))
  expect_equal(result$Infit_MSQ, as.numeric(iarm::out_infit(fit)$Infit))
})

# Tests for the bootstrap p-value layer of RMitemInfit() and the shared
# multiplicity helper .bootstrap_pvalues().

sim_rasch_null <- function(n = 300, J = 8, seed = 1L) {
  set.seed(seed)
  theta <- rnorm(n, 0, 1.2); beta <- seq(-1.5, 1.5, length.out = J)
  m <- sapply(seq_len(J), function(j) rbinom(n, 1, plogis(theta - beta[j])))
  df <- as.data.frame(m); colnames(df) <- paste0("I", seq_len(J))
  df
}

# ---------------------------------------------------------------------
# .bootstrap_pvalues() unit tests (no model fitting needed)
# ---------------------------------------------------------------------

test_that(".bootstrap_pvalues returns valid p-values and FWER >= marginal", {
  set.seed(1)
  k <- 6; B <- 2000
  sim <- matrix(rnorm(B * k, mean = 1, sd = 0.1), nrow = B)
  colnames(sim) <- paste0("I", seq_len(k))
  observed <- setNames(c(1.0, 1.0, 1.0, 1.0, 1.0, 1.6), colnames(sim))  # I6 extreme
  res <- easyRasch2:::.bootstrap_pvalues(observed, sim, correction = "fwer")
  expect_named(res, c("name", "p", "padj"))
  expect_true(all(res$p >= 0 & res$p <= 1))
  expect_true(all(res$padj >= res$p - 1e-9))          # FWER no smaller than marginal
  expect_lt(res$padj[res$name == "I6"], 0.05)         # the extreme item is detected
  expect_true(all(res$padj[res$name != "I6"] > 0.05)) # the null items are not
})

test_that(".bootstrap_pvalues FDR detects strong signals and is valid", {
  set.seed(2)
  k <- 8; B <- 2000
  sim <- matrix(rnorm(B * k, 1, 0.1), nrow = B); colnames(sim) <- paste0("I", 1:k)
  observed <- setNames(c(1.5, 1.45, rep(1.0, k - 2)), colnames(sim))
  bh <- easyRasch2:::.bootstrap_pvalues(observed, sim, "fdr_bh")
  expect_true(all(bh$padj >= 0 & bh$padj <= 1))
  expect_true(all(bh$padj[1:2] < 0.05))               # the two real effects detected
  expect_true(all(bh$padj[3:k] > 0.05))               # null comparisons not flagged
})

test_that(".bootstrap_pvalues handles zero-variance comparisons as NA", {
  sim <- cbind(I1 = rnorm(500, 1, 0.1), I2 = rep(1, 500))
  observed <- setNames(c(1.3, 1.0), c("I1", "I2"))
  res <- easyRasch2:::.bootstrap_pvalues(observed, sim, "fwer")
  expect_true(is.na(res$p[res$name == "I2"]))
  expect_false(is.na(res$p[res$name == "I1"]))
})

# ---------------------------------------------------------------------
# RMitemInfit() integration
# ---------------------------------------------------------------------

test_that("p_value = TRUE adds p columns and bases Flagged on padj", {
  skip_on_cran()
  skip_if_not_installed("eRm"); skip_if_not_installed("iarm")
  df  <- sim_rasch_null()
  sim <- RMitemInfitCutoff(df, iterations = 300, parallel = FALSE, seed = 1)
  res <- suppressWarnings(
    RMitemInfit(df, cutoff = sim, p_value = TRUE, output = "dataframe")
  )
  expect_true(all(c("p_infit", "padj_infit", "Flagged") %in% names(res)))
  expect_true(all(res$p_infit >= 0 & res$p_infit <= 1, na.rm = TRUE))
  expect_true(all(res$padj_infit >= res$p_infit - 1e-9, na.rm = TRUE))
})

test_that("p_value = TRUE errors without the full cutoff object", {
  skip_if_not_installed("eRm"); skip_if_not_installed("iarm")
  df  <- sim_rasch_null()
  sim <- RMitemInfitCutoff(df, iterations = 200, parallel = FALSE, seed = 1)
  expect_error(RMitemInfit(df, p_value = TRUE), regexp = "full RMitemInfitCutoff")
  expect_error(RMitemInfit(df, cutoff = sim$item_cutoffs, p_value = TRUE),
               regexp = "full RMitemInfitCutoff")
})

test_that("p_value = FALSE leaves the output unchanged (backward compatible)", {
  skip_on_cran()
  skip_if_not_installed("eRm"); skip_if_not_installed("iarm")
  df  <- sim_rasch_null()
  sim <- RMitemInfitCutoff(df, iterations = 200, parallel = FALSE, seed = 1)
  res <- RMitemInfit(df, cutoff = sim, output = "dataframe")
  expect_named(res, c("Item", "Infit_MSQ", "Infit_low", "Infit_high",
                      "Flagged", "Relative_location"))
})

test_that("all correction methods run and return the expected columns", {
  skip_on_cran()
  skip_if_not_installed("eRm"); skip_if_not_installed("iarm")
  df  <- sim_rasch_null()
  sim <- RMitemInfitCutoff(df, iterations = 300, parallel = FALSE, seed = 1)
  for (corr in c("fwer", "fdr_bh", "fdr_by", "none")) {
    res <- suppressWarnings(
      RMitemInfit(df, cutoff = sim, p_value = TRUE, correction = corr,
                  output = "dataframe")
    )
    expect_true(all(res$padj_infit >= 0 & res$padj_infit <= 1, na.rm = TRUE))
  }
})

test_that("low iteration count triggers a warning", {
  skip_on_cran()
  skip_if_not_installed("eRm"); skip_if_not_installed("iarm")
  df  <- sim_rasch_null()
  sim <- RMitemInfitCutoff(df, iterations = 200, parallel = FALSE, seed = 1)
  expect_warning(RMitemInfit(df, cutoff = sim, p_value = TRUE,
                             output = "dataframe"),
                 regexp = "iterations")
})

test_that("invalid alpha is rejected", {
  skip_if_not_installed("eRm"); skip_if_not_installed("iarm")
  df <- sim_rasch_null()
  expect_error(RMitemInfit(df, alpha = 1.5), regexp = "alpha")
})

test_that("kable output works with p-values", {
  skip_on_cran()
  skip_if_not_installed("eRm"); skip_if_not_installed("iarm")
  skip_if_not_installed("knitr")
  df  <- sim_rasch_null()
  sim <- RMitemInfitCutoff(df, iterations = 300, parallel = FALSE, seed = 1)
  expect_s3_class(
    suppressWarnings(RMitemInfit(df, cutoff = sim, p_value = TRUE)),
    "knitr_kable"
  )
})

# Tests for RMitemInfitCutoff() branches not exercised in
# test-conditional_infit.R (which covers the dichotomous
# quantile/hdci/conditional/resample paths): the polytomous DGP paths, the
# parallel runner, verbose progress, and error/fallback branches.

make_poly <- function(n = 200, k = 5, seed = 3L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:2, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

make_dich <- function(n = 200, k = 5, seed = 3L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

test_that("polytomous data (resample DGP) returns per-item cutoffs", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  res <- RMitemInfitCutoff(make_poly(), iterations = 15, parallel = FALSE,
                           seed = 1, cutoff_method = "quantile")
  expect_equal(nrow(res$item_cutoffs), 5L)
  expect_equal(res$actual_iterations, 15L)
  expect_true(all(res$item_cutoffs$infit_low < res$item_cutoffs$infit_high))
  expect_true(all(res$item_cutoffs$outfit_low < res$item_cutoffs$outfit_high))
})

test_that("polytomous data with conditional DGP runs", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  res <- RMitemInfitCutoff(make_poly(), iterations = 15, parallel = FALSE,
                           seed = 1, dgp = "conditional",
                           cutoff_method = "quantile")
  expect_identical(res$dgp, "conditional")
  expect_equal(nrow(res$item_cutoffs), 5L)
})

test_that("verbose = TRUE runs the sequential progress path", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  suppressMessages(invisible(utils::capture.output(
    res <- RMitemInfitCutoff(make_dich(), iterations = 5, parallel = FALSE,
                             seed = 1, cutoff_method = "quantile",
                             verbose = TRUE)
  )))
  expect_equal(res$actual_iterations, 5L)
})

test_that("errors when no complete cases remain after na.omit", {
  skip_if_not_installed("iarm")
  d <- data.frame(a = c(0, NA, 1), b = c(NA, 1, NA))
  expect_snapshot(
    RMitemInfitCutoff(d, iterations = 3, parallel = FALSE),
    error = TRUE
  )
})

test_that("parallel runner produces the same-shaped result", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("mirai")
  res <- suppressMessages(
    RMitemInfitCutoff(make_dich(), iterations = 6, parallel = TRUE,
                      n_cores = 2, seed = 1, cutoff_method = "quantile")
  )
  expect_equal(nrow(res$item_cutoffs), 5L)
  expect_equal(res$actual_iterations, 6L)
})

test_that("parallel = TRUE without n_cores or mc.cores warns and falls back", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("mirai")
  withr::local_options(mc.cores = NULL)
  expect_snapshot(
    res <- RMitemInfitCutoff(make_dich(), iterations = 4, parallel = TRUE,
                             seed = 1, cutoff_method = "quantile")
  )
  expect_equal(res$actual_iterations, 4L)
})

#' Item-Threshold Hierarchy Plot
#'
#' Visualises item and threshold locations on the logit scale for a
#' Partial Credit Model. Items are sorted by their (mean-threshold)
#' location; each item shows its location as a black diamond and its
#' individual thresholds as coloured dots with confidence-interval error
#' bars.
#'
#' Threshold locations are centred at the grand mean of all thresholds
#' across items, so the dashed reference line at 0 represents the mean
#' threshold location across the scale.
#'
#' Confidence intervals around thresholds are 84% by default
#' (`sem_multiplier = 1.405`), following Payton, Greenstone, & Schenker
#' (2003) -- non-overlap of 84% intervals approximately corresponds to a
#' two-sample significance test at \eqn{\alpha = 0.05}. Use
#' `sem_multiplier = 1.96` for 95% intervals.
#'
#' @param data A data.frame or matrix of polytomous item responses
#'   (non-negative integers, 0-based, max value > 1). One column per
#'   item, one row per person.
#' @param show_numbers Logical. When `TRUE` (default), prints the
#'   numerical item location and threshold values next to the points on
#'   the plot. When `FALSE`, only the threshold labels (`T1`, `T2`, ...)
#'   are shown.
#' @param sem_multiplier Numeric multiplier for the threshold SE used to
#'   draw the error bars. Default `1.405` (84% CI). Common alternatives:
#'   `1.96` (95% CI), `2.576` (99% CI).
#' @param item_labels Optional character vector of length `ncol(data)`
#'   providing descriptive labels for items. Default `NULL` uses the
#'   column names of `data`. Labels are appended to the column names on
#'   the y-axis (`name - label`) and wrapped at 36 characters via base-R
#'   `strwrap()`.
#' @param output One of `"ggplot"` (default) -- a faceted hierarchy
#'   figure -- or `"dataframe"` -- the long-format underlying data with
#'   one row per (item × threshold).
#'
#' @return Either a `ggplot` (default) or a data.frame with columns
#'   `Item`, `ItemLabel`, `Threshold`, `ThresholdLocation`,
#'   `ThresholdSE`, and `ItemLocation` (the per-item mean of the centred
#'   thresholds).
#'
#' @details
#' \strong{Polytomous only.} Dichotomous items have a single threshold
#' that coincides with the item location; the hierarchy plot is
#' visually degenerate in that case. For dichotomous data use
#' [RMtargeting()] or [RMscoreSE()] instead.
#'
#' \strong{Centring convention.} The CML PCM thresholds (estimated via
#' `psychotools::pcmodel()`) are shifted so that their grand mean is zero;
#' each item's location is then the mean of its centred thresholds.
#' The dashed horizontal reference line on the plot marks this zero --
#' i.e., the average threshold across all items.
#'
#' @references
#' Payton, M. E., Greenstone, M. H., & Schenker, N. (2003). Overlapping
#' confidence intervals or standard error intervals: What do they mean
#' in terms of statistical significance? \emph{Journal of Insect Science,
#' 3}(34), 1-6. \doi{10.1093/jis/3.1.34}
#'
#' @seealso [RMtargeting()], [RMscoreSE()], [RMitemICCPlot()]
#'
#' @examples
#' \donttest{
#' if (requireNamespace("eRm", quietly = TRUE)) {
#'   data("pcmdat2", package = "eRm")
#'   RMitemHierarchy(pcmdat2)
#'
#'   # 95% CI instead of 84%
#'   RMitemHierarchy(pcmdat2, sem_multiplier = 1.96)
#'
#'   # Underlying data.frame
#'   RMitemHierarchy(pcmdat2, output = "dataframe")
#' }
#' }
#'
#' @importFrom rlang .data
#' @export
RMitemHierarchy <- function(
  data,
  show_numbers = TRUE,
  sem_multiplier = 1.405,
  item_labels = NULL,
  output = c("ggplot", "dataframe")
) {
  output <- match.arg(output)

  if (output == "ggplot" && !requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for output = \"ggplot\". ",
      "Install with: install.packages(\"ggplot2\")",
      call. = FALSE
    )
  }

  # --- Data validation ----------------------------------------------------
  validate_response_data(data)
  data <- as.data.frame(data)
  if (ncol(data) < 2L) {
    stop("RMitemHierarchy() requires at least 2 items.", call. = FALSE)
  }

  data_mat <- as.matrix(data)
  if (max(data_mat, na.rm = TRUE) <= 1L) {
    stop(
      "RMitemHierarchy() requires polytomous data (max response > 1). ",
      "For dichotomous items the threshold coincides with the item ",
      "location, so the hierarchy plot is degenerate. Use ",
      "RMtargeting() or RMscoreSE() instead.",
      call. = FALSE
    )
  }

  if (
    !is.numeric(sem_multiplier) ||
      length(sem_multiplier) != 1L ||
      !is.finite(sem_multiplier) ||
      sem_multiplier <= 0
  ) {
    stop(
      "`sem_multiplier` must be a positive numeric value. Common ",
      "choices: 1.405 (84% CI), 1.96 (95%), 2.576 (99%).",
      call. = FALSE
    )
  }

  labels_supplied <- !is.null(item_labels)
  if (is.null(item_labels)) {
    item_labels <- names(data)
  } else {
    if (length(item_labels) != ncol(data)) {
      stop(
        "`item_labels` must have the same length as ncol(data) (",
        ncol(data),
        "). Got ",
        length(item_labels),
        ".",
        call. = FALSE
      )
    }
    item_labels <- as.character(item_labels)
  }

  data <- as.data.frame(data)
  n_total <- nrow(data)
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)
  n_used <- nrow(data)

  # --- Fit PCM and extract centred thresholds + SEs -----------------------
  # rgl workaround for any iarm-related dependency chains
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # CML Andrich thresholds + their SEs via psychotools, consistent with the
  # rest of the package. threshpar() returns a per-item list (in data column
  # order); with vcov = TRUE the threshold-parameter covariance aligns with the
  # flattened threshold order, so SEs are the square-root of its diagonal.
  # Threshold locations match the previous eRm values after grand-mean
  # centring; the reported SEs differ slightly (psychotools vs eRm vcov).
  fit <- psychotools::pcmodel(data)
  tp <- psychotools::threshpar(fit, vcov = TRUE)
  se_all <- sqrt(diag(attr(tp, "vcov")))

  item_per_thr <- rep(names(tp), lengths(tp))
  thr_index <- unlist(lapply(tp, seq_along), use.names = FALSE)
  thresh_raw <- unlist(lapply(tp, as.numeric), use.names = FALSE)

  # Centre thresholds at the grand mean
  thresh_centred <- thresh_raw - mean(thresh_raw, na.rm = TRUE)

  long_df <- data.frame(
    Item = item_per_thr,
    ItemLabel = item_labels[match(item_per_thr, names(data))],
    Threshold = paste0("T", thr_index),
    ThresholdLocation = thresh_centred,
    ThresholdSE = as.numeric(se_all),
    stringsAsFactors = FALSE
  )

  # Per-item Location = mean of that item's centred thresholds
  item_loc <- stats::aggregate(
    ThresholdLocation ~ Item,
    data = long_df,
    FUN = function(x) mean(x, na.rm = TRUE)
  )
  names(item_loc)[2L] <- "ItemLocation"
  long_df <- merge(long_df, item_loc, by = "Item", sort = FALSE)

  # Order items by item location (lowest first, so they appear at the
  # bottom of the y-axis after coord_flip)
  item_order <- item_loc$Item[order(item_loc$ItemLocation)]
  label_order <- item_labels[match(item_order, names(data))]
  long_df$Item <- factor(long_df$Item, levels = item_order)

  rownames(long_df) <- NULL

  if (output == "dataframe") {
    return(long_df)
  }

  # --- Plot ---------------------------------------------------------------
  # Axis labels wrapped at 36 chars (base R; matches the original's str_wrap
  # usage). Only append " - label" when item_labels were actually supplied;
  # otherwise the default labels are the column names and would be repeated
  # ("I1 - I1").
  axis_base <- if (labels_supplied) {
    paste0(item_order, " - ", label_order)
  } else {
    item_order
  }
  axis_labels <- vapply(
    axis_base,
    function(s) paste(strwrap(s, width = 36L), collapse = "\n"),
    character(1L)
  )

  # `scale_x_discrete(labels = ...)` does a named lookup against the
  # factor levels (= `item_order`), so we must name the vector
  # accordingly. If we don't, the level names slip through unchanged
  # and the `item_labels` argument silently has no effect.
  names(axis_labels) <- item_order

  caption_base <-
    "Note. Item locations are indicated by black diamond shapes"
  caption_thr <-
    "Item threshold locations are indicated by coloured dots and labels."
  caption_ci <- sprintf(
    "Horizontal error bars indicate +/-%.3f x SE around each threshold.",
    sem_multiplier
  )
  caption_n <- paste0(
    .n_caption(
      n_used,
      n_total,
      if (has_na) "incomplete responses retained" else character()
    ),
    "."
  )
  caption_text <- er2_caption(paste(
    if (show_numbers) {
      paste0(caption_base, " and black text.")
    } else {
      paste0(caption_base, ".")
    },
    caption_thr,
    caption_ci,
    caption_n,
    sep = "\n"
  ))

  p <- ggplot2::ggplot(
    long_df,
    ggplot2::aes(x = .data$Item, colour = .data$Threshold)
  ) +
    # Item location: black diamond
    ggplot2::geom_point(
      ggplot2::aes(y = .data$ItemLocation),
      size = 4,
      shape = 18,
      colour = "black"
    ) +
    # Threshold locations: coloured dots + 84%-style CI error bars
    ggplot2::geom_point(ggplot2::aes(y = .data$ThresholdLocation)) +
    ggplot2::geom_errorbar(
      ggplot2::aes(
        ymin = .data$ThresholdLocation - sem_multiplier * .data$ThresholdSE,
        ymax = .data$ThresholdLocation + sem_multiplier * .data$ThresholdSE
      ),
      width = 0.11
    ) +
    # Threshold labels (T1, T2, ...) below each coloured dot
    ggplot2::geom_text(
      ggplot2::aes(y = .data$ThresholdLocation, label = .data$Threshold),
      size = 3,
      hjust = 0.5,
      vjust = 1.5,
      show.legend = FALSE
    )

  if (show_numbers) {
    p <- p +
      ggplot2::geom_text(
        ggplot2::aes(
          y = .data$ItemLocation,
          label = sprintf("%.2f", .data$ItemLocation)
        ),
        hjust = 0.5,
        vjust = 2.2,
        colour = "black",
        size = 3,
        show.legend = FALSE
      ) +
      ggplot2::geom_text(
        ggplot2::aes(
          y = .data$ThresholdLocation,
          label = sprintf("%.2f", .data$ThresholdLocation)
        ),
        hjust = 0.5,
        vjust = -1.1,
        size = 3,
        show.legend = FALSE
      )
  }

  p +
    ggplot2::geom_hline(
      yintercept = mean(long_df$ItemLocation, na.rm = TRUE),
      linetype = "dashed",
      colour = "darkgrey"
    ) +
    ggplot2::geom_rug(
      ggplot2::aes(y = .data$ThresholdLocation),
      colour = "darkgrey",
      sides = "l",
      length = ggplot2::unit(0.02, "npc")
    ) +
    ggplot2::scale_x_discrete(labels = axis_labels) +
    ggplot2::coord_flip() +
    ggplot2::scale_colour_viridis_d(guide = "none", option = "H") +
    ggplot2::labs(y = "Logit location", x = NULL, caption = caption_text) +
    ggplot2::theme_bw(base_size = 13) +
    ggplot2::theme(
      legend.position = "none"
    ) +
    er2_axis_margins() +
    er2_plot_caption()
}

# Tests for RMitemHierarchy()

make_polytomous <- function(n = 200, k = 5, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:3, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMitemHierarchy errors when data has non-zero minimum", {
  df <- make_polytomous() + 1L
  expect_error(RMitemHierarchy(df), regexp = "scored starting at 0")
})

test_that("RMitemHierarchy errors when fewer than 2 items", {
  df <- make_polytomous()[, 1L, drop = FALSE]
  expect_error(RMitemHierarchy(df), regexp = "at least 2 items")
})

test_that("RMitemHierarchy errors on dichotomous data with helpful pointer", {
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:1, 200 * 5, replace = TRUE), 200, 5))
  colnames(df) <- paste0("I", 1:5)
  expect_error(RMitemHierarchy(df),
               regexp = "polytomous data")
})

test_that("RMitemHierarchy errors on invalid sem_multiplier", {
  df <- make_polytomous()
  expect_error(RMitemHierarchy(df, sem_multiplier = -1),
               regexp = "positive numeric")
  expect_error(RMitemHierarchy(df, sem_multiplier = c(1, 2)),
               regexp = "positive numeric")
})

test_that("RMitemHierarchy errors when item_labels length mismatches ncol(data)", {
  df <- make_polytomous()
  expect_error(RMitemHierarchy(df, item_labels = c("a", "b")),
               regexp = "same length")
})

# ---------------------------------------------------------------------
# Output structures
# ---------------------------------------------------------------------
test_that("RMitemHierarchy default returns a ggplot", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMitemHierarchy(df)
  expect_s3_class(p, "ggplot")
})

test_that("RMitemHierarchy show_numbers = FALSE returns a ggplot", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMitemHierarchy(df, show_numbers = FALSE)
  expect_s3_class(p, "ggplot")
})

test_that("RMitemHierarchy with custom sem_multiplier returns a ggplot", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMitemHierarchy(df, sem_multiplier = 1.96)
  expect_s3_class(p, "ggplot")
})

test_that("RMitemHierarchy with custom item_labels returns a ggplot", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMitemHierarchy(df, item_labels = paste("Item", seq_len(ncol(df))))
  expect_s3_class(p, "ggplot")
})

test_that("RMitemHierarchy actually applies item_labels to the x-axis", {
  # Regression test: scale_x_discrete(labels = ...) does a named lookup
  # against the factor levels, so axis_labels must be named by the
  # column names (item_order). Otherwise the labels silently revert to
  # the raw column names. (#item_labels-bug, 2026)
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  custom <- paste("Label", seq_len(ncol(df)))
  p <- RMitemHierarchy(df, item_labels = custom)

  sc <- ggplot2::ggplot_build(p)$plot$scales$get_scales("x")
  expect_true(all(names(sc$labels) %in% names(df)))
  # Every displayed label must contain its custom suffix
  for (i in seq_along(custom)) {
    expect_true(any(grepl(custom[i], sc$labels, fixed = TRUE)),
                info = paste("custom label not found:", custom[i]))
  }
})

test_that("RMitemHierarchy does not duplicate the column name when item_labels is unset", {
  # Regression: with no item_labels the y-axis label should be just the column
  # name, not "I1 - I1".
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p  <- RMitemHierarchy(df)
  sc <- ggplot2::ggplot_build(p)$plot$scales$get_scales("x")
  expect_setequal(names(sc$labels), names(df))
  expect_equal(unname(sc$labels[names(df)]), names(df))
  expect_false(any(grepl(" - ", sc$labels, fixed = TRUE)))
})

# ---------------------------------------------------------------------
# Dataframe output
# ---------------------------------------------------------------------
test_that("RMitemHierarchy output = 'dataframe' returns long-format data", {
  skip_if_not_installed("eRm")
  df <- make_polytomous()  # 5 items × 3 thresholds (categories 0..3)
  res <- RMitemHierarchy(df, output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_equal(nrow(res), ncol(df) * 3L)  # one row per item × threshold
  expect_named(res, c("Item", "ItemLabel", "Threshold",
                      "ThresholdLocation", "ThresholdSE", "ItemLocation"))
  # Item is a factor ordered by ItemLocation
  expect_s3_class(res$Item, "factor")
  # ThresholdLocations centred at the grand mean -> mean should be ~ 0
  expect_lt(abs(mean(res$ThresholdLocation, na.rm = TRUE)), 1e-8)
  # SE values should all be positive and finite
  expect_true(all(res$ThresholdSE > 0 & is.finite(res$ThresholdSE)))
  # Per-item Location = mean of that item's centred thresholds
  per_item <- tapply(res$ThresholdLocation, res$Item, mean, na.rm = TRUE)
  expect_equal(
    as.numeric(per_item),
    as.numeric(tapply(res$ItemLocation, res$Item, unique)),
    tolerance = 1e-8
  )
})

test_that("RMitemHierarchy item factor levels are ordered by ItemLocation", {
  skip_if_not_installed("eRm")
  df  <- make_polytomous()
  res <- RMitemHierarchy(df, output = "dataframe")
  # Pull the unique (Item, ItemLocation) pairs in factor-level order
  ord_locs <- tapply(res$ItemLocation, res$Item, unique)
  expect_true(all(diff(ord_locs) >= 0),
              info = "Item factor levels should be in ascending ItemLocation order")
})

#' Item Category Probability Curves
#'
#' Plots model-implied response-category probability curves for each item
#' as a function of the latent trait \eqn{\theta}. Item parameters are
#' estimated by conditional maximum likelihood via `psychotools::pcmodel()`
#' (a dichotomous item is a 2-category PCM). Each item gets
#' its own facet panel, with one curve per response category coloured
#' from low to high using the viridis palette. Comparable in scope to
#' `eRm::plotICC()` and `mirt`'s trace plots, with a `ggplot2` /
#' viridis output and optional descriptive labels for items and
#' categories.
#'
#' @param data A data.frame or matrix of item responses. Items must be
#'   scored starting at 0 (non-negative integers). Missing values
#'   (`NA`) are allowed --- the model fit handles them.
#' @param item_labels Optional character vector of descriptive item
#'   labels (facet strip titles). Must be the same length as
#'   `ncol(data)`. If `NULL` (the default), column names are used.
#' @param category_labels Optional character vector of labels for the
#'   response categories (legend). Must be the same length as the
#'   number of categories spanning from 0 to the maximum observed
#'   value. If `NULL` (the default), numeric category values are used.
#' @param theta_range Numeric length 2. Range of the latent trait
#'   \eqn{\theta} (logits) plotted along the x-axis. When `NULL`
#'   (default) the range is `c(-4, 4)` for `label_curves = "legend"`
#'   and a wider `c(-5, 5)` for `label_curves = "path"` --- the
#'   wider range gives `geomtextpath` more horizontal room to place
#'   per-curve labels at their peaks. Pass an explicit
#'   `c(lo, hi)` to override.
#' @param n_points Integer. Number of evenly-spaced \eqn{\theta} values
#'   at which the curves are evaluated. Default `200L`.
#' @param viridis_option Character. Viridis palette identifier. One of
#'   `"A"` through `"H"`. Default `"D"` (viridis green).
#' @param viridis_end Numeric in (0, 1]. Upper end of the viridis
#'   palette range; lower values keep the palette inside its mid-tones
#'   (avoids the very bright yellow at `1.0`). Default `0.95`.
#' @param facet_ncol Optional integer. Number of columns in the facet
#'   layout. Default `NULL` (`ggplot2::facet_wrap()` auto-layout).
#' @param label_wrap Integer. Characters per line for facet-strip
#'   label wrapping. Default `25L`.
#' @param line_width Numeric. Line width for the probability curves.
#'   Default `0.9`.
#' @param font Character. Font family for all text. Default `"sans"`.
#' @param output Character. Either `"ggplot"` (default) for a
#'   `ggplot` object, or `"dataframe"` for the underlying long-format
#'   probability table.
#' @param label_curves Character. How response categories are
#'   identified. `"legend"` (default) draws lines and a separate
#'   colour legend with one swatch per category --- the standard
#'   multi-item presentation. `"path"` uses
#'   `geomtextpath::geom_textpath()` to write each category's label
#'   *along* its own curve (a la classic IRT trace plots) and
#'   suppresses the legend; valid for **a single item at a time**
#'   because narrow facets do not give path labels enough horizontal
#'   room to render legibly. The model is still fit on the full
#'   multi-item `data` (PCM thresholds require multi-item input for
#'   CML estimation); the `item` argument then selects which item's
#'   curves to plot. Requires the optional `geomtextpath` package.
#' @param item Character or integer. Used only when
#'   `label_curves = "path"`. Either an item name (matching a column
#'   in `data`) or a column index, identifying which item's category
#'   curves to plot. Required for path mode.
#' @param text_size Numeric. Used only when `label_curves = "path"`.
#'   Size of the path labels in mm. Default `4`.
#'
#' @return
#' * If `output = "ggplot"`: a [ggplot2::ggplot] object, one facet per
#'   item.
#' * If `output = "dataframe"`: a long-format data.frame with columns
#'   `Item` (factor in column order), `Category` (integer), and
#'   `Theta`, `Probability` (numeric), one row per item × category ×
#'   theta gridpoint.
#'
#' @details
#' For each polytomous item *i* with response categories
#' \eqn{0, 1, \ldots, K_i} and threshold parameters
#' \eqn{\delta_{i,1}, \ldots, \delta_{i,K_i}} (CML, `psychotools`),
#' the PCM category probability is
#'
#' \deqn{P(X_i = k \mid \theta) = \frac{\exp(\sum_{j=1}^{k}
#'   (\theta - \delta_{i,j}))}{\sum_{k'=0}^{K_i} \exp(\sum_{j=1}^{k'}
#'   (\theta - \delta_{i,j}))},}
#'
#' with the empty sum (when \eqn{k = 0}) taken as zero. For
#' dichotomous items the function fits a Rasch model and treats
#' the item difficulty \eqn{\delta_i = -\beta_i} as the single
#' threshold, recovering the standard two-category logistic ICC.
#'
#' The colour mapping uses `scale_color_viridis_c()` against the
#' integer category value, so the natural ordering of response
#' categories is preserved visually --- low categories at one end of
#' the palette, high categories at the other. When `category_labels`
#' is provided, the legend uses those labels (e.g., "Never" /
#' "Sometimes" / "Often") while the colour mapping stays on the
#' integer category value.
#'
#' Items with fewer response categories than the maximum (e.g., an
#' otherwise four-category scale with one three-category item)
#' contribute only the categories they actually have to their own
#' facet --- the y-axis still spans \eqn{[0, 1]}.
#'
#' @seealso [RMitemICCPlot()] for conditional ICCs binned by total
#'   score, [RMitemHierarchy()] for item threshold locations on the
#'   logit scale.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("eRm", quietly = TRUE) &&
#'     requireNamespace("ggplot2", quietly = TRUE)) {
#'   data(pcmdat2, package = "eRm")
#'
#'   # Default plot
#'   RMitemCatProb(pcmdat2)
#'
#'   # Custom item and category labels
#'   RMitemCatProb(
#'     pcmdat2,
#'     item_labels     = c("Mood", "Sleep", "Appetite", "Energy"),
#'     category_labels = c("Never", "Sometimes", "Often")
#'   )
#'
#'   # Underlying probability data
#'   df <- RMitemCatProb(pcmdat2, output = "dataframe")
#'   head(df)
#'
#'   # Single-item plot with labels written along each curve
#'   # (classic IRT trace-plot style). Model is still fit on all four
#'   # items; `item` picks which one to plot.
#'   if (requireNamespace("geomtextpath", quietly = TRUE)) {
#'     RMitemCatProb(
#'       pcmdat2,
#'       category_labels = c("Never", "Sometimes", "Often"),
#'       label_curves    = "path",
#'       item            = "I1"
#'     )
#'   }
#' }
#' }
#'
#' @importFrom rlang .data
#' @export
RMitemCatProb <- function(
  data,
  item_labels = NULL,
  category_labels = NULL,
  theta_range = NULL,
  n_points = 200L,
  viridis_option = "D",
  viridis_end = 0.95,
  facet_ncol = NULL,
  label_wrap = 25L,
  line_width = 0.9,
  font = "sans",
  output = c("ggplot", "dataframe"),
  label_curves = c("legend", "path"),
  item = NULL,
  text_size = 4
) {
  output <- match.arg(output)
  label_curves <- match.arg(label_curves)

  # Resolve default theta_range based on label_curves: wider for path mode
  # so geom_textpath has enough horizontal room to place labels at peaks.
  if (is.null(theta_range)) {
    theta_range <- if (label_curves == "path") c(-5, 5) else c(-4, 4)
  }

  if (output == "ggplot" && !requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for output = \"ggplot\".",
      call. = FALSE
    )
  }
  if (!requireNamespace("eRm", quietly = TRUE)) {
    stop("Package 'eRm' is required for RMitemCatProb().", call. = FALSE)
  }
  if (
    label_curves == "path" &&
      !requireNamespace("geomtextpath", quietly = TRUE)
  ) {
    stop(
      "Package 'geomtextpath' is required when `label_curves = \"path\"`.\n",
      "Install it with: install.packages(\"geomtextpath\")",
      call. = FALSE
    )
  }

  # ---------------------------------------------------------------------
  # Input validation
  # ---------------------------------------------------------------------
  validate_response_data(data)

  # PCM / RM require multi-item data for CML estimation, so `data` must
  # always have >= 2 items. Path mode uses the `item` argument to filter
  # which item's curves to plot AFTER the model is fit on all items.
  if (ncol(data) < 2L) {
    stop(
      "`data` must contain at least 2 columns (items). Found ",
      ncol(data),
      " column(s).",
      call. = FALSE
    )
  }

  if (
    !is.numeric(text_size) ||
      length(text_size) != 1L ||
      !is.finite(text_size) ||
      text_size <= 0
  ) {
    stop("`text_size` must be a single positive number.", call. = FALSE)
  }

  if (
    !is.numeric(theta_range) ||
      length(theta_range) != 2L ||
      theta_range[1L] >= theta_range[2L]
  ) {
    stop(
      "`theta_range` must be a numeric vector of length 2 with ",
      "theta_range[1] < theta_range[2].",
      call. = FALSE
    )
  }

  if (
    !is.numeric(n_points) ||
      length(n_points) != 1L ||
      n_points < 2L ||
      n_points != as.integer(n_points)
  ) {
    stop("`n_points` must be a single integer >= 2.", call. = FALSE)
  }
  n_points <- as.integer(n_points)

  if (
    !is.numeric(viridis_end) ||
      length(viridis_end) != 1L ||
      viridis_end <= 0 ||
      viridis_end > 1
  ) {
    stop("`viridis_end` must be a single number in (0, 1].", call. = FALSE)
  }

  item_names <- colnames(data)
  if (is.null(item_names)) {
    item_names <- paste0("V", seq_len(ncol(data)))
  }
  n_items <- ncol(data)

  if (!is.null(item_labels) && length(item_labels) != n_items) {
    stop(
      "`item_labels` must have the same length as ncol(data) (",
      n_items,
      "). Got ",
      length(item_labels),
      ".",
      call. = FALSE
    )
  }

  # Resolve `item` for path mode. Accept either a single item name or a
  # column index; produce a single character item name.
  selected_item <- NULL
  if (label_curves == "path") {
    if (is.null(item)) {
      stop(
        "`label_curves = \"path\"` requires an `item` argument (a single ",
        "item name or column index) identifying which item's category ",
        "curves to plot. Available items: ",
        paste(item_names, collapse = ", "),
        ".",
        call. = FALSE
      )
    }
    if (length(item) != 1L) {
      stop("`item` must identify exactly one item (length 1).", call. = FALSE)
    }
    if (is.numeric(item)) {
      if (
        !is.finite(item) ||
          item != as.integer(item) ||
          item < 1L ||
          item > n_items
      ) {
        stop(
          "`item` index must be an integer in 1:",
          n_items,
          ".",
          call. = FALSE
        )
      }
      selected_item <- item_names[as.integer(item)]
    } else if (is.character(item)) {
      if (!(item %in% item_names)) {
        stop(
          "`item` \"",
          item,
          "\" not found in data. ",
          "Available items: ",
          paste(item_names, collapse = ", "),
          ".",
          call. = FALSE
        )
      }
      selected_item <- item
    } else {
      stop(
        "`item` must be a character item name or a numeric index.",
        call. = FALSE
      )
    }
  }

  data <- as.data.frame(data)
  n_total <- nrow(data)
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)
  n_used <- nrow(data)

  data_mat <- as.matrix(data)
  max_score <- max(data_mat, na.rm = TRUE)
  if (!is.finite(max_score) || max_score < 1L) {
    stop(
      "`data` must contain at least one item with maximum score >= 1.",
      call. = FALSE
    )
  }
  n_categories <- max_score + 1L

  if (
    !is.null(category_labels) &&
      length(category_labels) != n_categories
  ) {
    stop(
      "`category_labels` must have the same length as the number of ",
      "response categories (0 to ",
      max_score,
      " = ",
      n_categories,
      " categories). Got ",
      length(category_labels),
      ".",
      call. = FALSE
    )
  }

  # ---------------------------------------------------------------------
  # Fit model + extract per-item threshold vectors
  # ---------------------------------------------------------------------
  is_polytomous <- max_score > 1L

  # Per-item thresholds (a list, one numeric vector per item) by CML via
  # psychotools, on the grand-mean-zero scale used across the package. A
  # dichotomous item is a 2-category PCM. The curve shapes are unchanged; for
  # polytomous data the common scale can shift the absolute theta position
  # relative to the earlier eRm parameterisation.
  item_thresholds <- .fit_cml_thresholds(data)[item_names]

  # ---------------------------------------------------------------------
  # Compute category probability curves on the theta grid
  # ---------------------------------------------------------------------
  theta <- seq(theta_range[1L], theta_range[2L], length.out = n_points)

  # For one item: returns a length(theta) x (K+1) probability matrix.
  pcm_probs <- function(thresholds, theta) {
    K <- length(thresholds)
    # Cumulative sums of (theta - delta_j) for j = 0, 1, ..., K
    numerators <- matrix(0, nrow = length(theta), ncol = K + 1L)
    for (k in seq_len(K)) {
      numerators[, k + 1L] <- numerators[, k] + (theta - thresholds[k])
    }
    exp_num <- exp(numerators)
    exp_num / rowSums(exp_num)
  }

  # Build long-format probability data.frame
  per_item_dfs <- lapply(seq_along(item_thresholds), function(i) {
    item <- item_names[i]
    thr <- item_thresholds[[i]]
    probs <- pcm_probs(thr, theta)
    K_i <- length(thr)
    out <- data.frame(
      Item = rep(item, times = length(theta) * (K_i + 1L)),
      Category = rep(0L:K_i, each = length(theta)),
      Theta = rep(theta, times = K_i + 1L),
      Probability = as.numeric(probs),
      stringsAsFactors = FALSE,
      row.names = NULL
    )
    out
  })
  plot_df <- do.call(rbind, per_item_dfs)
  rownames(plot_df) <- NULL

  # Apply item labels to facet titles (preserve input column order)
  facet_labels <- if (!is.null(item_labels)) {
    paste0(item_names, " - ", item_labels)
  } else {
    item_names
  }
  facet_lookup <- stats::setNames(facet_labels, item_names)
  plot_df$Item <- factor(facet_lookup[plot_df$Item], levels = facet_labels)

  # In path mode, subset to the chosen item AFTER the model has been
  # fit on all items (so thresholds are CML-estimated against the full
  # multi-item dataset).
  if (label_curves == "path") {
    facet_label_selected <- facet_lookup[[selected_item]]
    plot_df <- plot_df[
      as.character(plot_df$Item) == facet_label_selected,
      ,
      drop = FALSE
    ]
    rownames(plot_df) <- NULL
  }

  # ---------------------------------------------------------------------
  # Return early for dataframe output
  # ---------------------------------------------------------------------
  if (output == "dataframe") {
    return(plot_df)
  }

  # ---------------------------------------------------------------------
  # Build ggplot
  # ---------------------------------------------------------------------
  cat_breaks <- 0L:max_score
  cat_labels <- if (!is.null(category_labels)) {
    category_labels
  } else {
    as.character(cat_breaks)
  }

  # Attach a human-readable label per row for use in path mode.
  plot_df$CategoryLabel <- cat_labels[plot_df$Category + 1L]

  # Common scale + axis components for both modes
  base_scales <- list(
    ggplot2::scale_y_continuous(
      limits = c(0, 1),
      breaks = c(0, 0.25, 0.5, 0.75, 1)
    ),
    ggplot2::scale_x_continuous(
      limits = theta_range,
      breaks = seq(ceiling(theta_range[1L]), floor(theta_range[2L]), by = 1)
    ),
    ggplot2::labs(
      x = expression(paste("Latent trait ", theta, " (logits)")),
      y = "Category probability",
      caption = er2_caption(paste0(
        .n_caption(
          n_used,
          n_total,
          if (has_na) "incomplete responses retained" else character()
        ),
        "."
      ))
    ),
    ggplot2::theme_bw(base_family = font),
    ggplot2::theme(
      panel.grid.minor = ggplot2::element_blank(),
      strip.background = ggplot2::element_rect(fill = "grey95", colour = NA),
      strip.text = ggplot2::element_text(face = "bold")
    ),
    er2_axis_margins()
  )

  if (label_curves == "legend") {
    # -----------------------------------------------------------------
    # Mode 1: multi-item faceted plot with a separate colour legend
    # -----------------------------------------------------------------
    p <- ggplot2::ggplot(
      plot_df,
      ggplot2::aes(
        x = .data$Theta,
        y = .data$Probability,
        group = .data$Category,
        color = .data$Category
      )
    ) +
      ggplot2::geom_line(linewidth = line_width) +
      ggplot2::scale_color_viridis_c(
        name = "Response\ncategory",
        breaks = cat_breaks,
        labels = cat_labels,
        limits = c(0, max_score),
        option = viridis_option,
        end = viridis_end,
        # guide_legend gives one swatch per category (not a continuous bar),
        # so all `n_categories` labels are visible. ggplot's default places
        # the smallest break at the top of a vertical legend, which is the
        # natural reading order for ordinal "least -> most" categories
        # (e.g. PHQ-9: "Not at all" -> "Nearly every day").
        guide = ggplot2::guide_legend(
          override.aes = list(linewidth = 2)
        )
      ) +
      ggplot2::facet_wrap(
        ~Item,
        ncol = facet_ncol,
        labeller = ggplot2::labeller(
          Item = ggplot2::label_wrap_gen(label_wrap)
        )
      ) +
      base_scales
  } else {
    # -----------------------------------------------------------------
    # Mode 2: single-item plot with labels along each curve.
    #
    # For each category we compute the *modal theta* (the theta value
    # at which that category has its highest probability) and convert
    # that to a numeric `hjust` fraction along the path. This puts the
    # label exactly where the curve peaks:
    #   - Monotone-decreasing extreme (Category 0): peak at the left
    #     edge of the plot.
    #   - Bell-shaped middle categories: peak at the modal theta
    #     between the surrounding thresholds.
    #   - Monotone-increasing extreme (Category K): peak at the right
    #     edge of the plot.
    #
    # Why not `hjust = "auto"`? geomtextpath's "auto" mode picks the
    # *smoothest* path segment, which for sharp bell curves means the
    # long, flat near-zero tails --- so labels get pushed to the plot
    # edges, not the peaks. Computing modal theta directly is robust.
    #
    # A small margin (3% of the plotted range) clamps `hjust` away
    # from 0 and 1 so the extreme-category labels don't get clipped
    # at the plot edges.
    # -----------------------------------------------------------------
    p <- ggplot2::ggplot(
      plot_df,
      ggplot2::aes(
        x = .data$Theta,
        y = .data$Probability,
        group = .data$Category,
        color = .data$Category,
        label = .data$CategoryLabel
      )
    ) +
      ggplot2::scale_color_viridis_c(
        breaks = cat_breaks,
        limits = c(0, max_score),
        option = viridis_option,
        end = viridis_end,
        guide = "none"
      ) +
      ggplot2::ggtitle(as.character(plot_df$Item[1L])) +
      base_scales

    edge_margin <- 0.03 * diff(theta_range)
    hjust_lo_clip <- edge_margin / diff(theta_range)
    hjust_hi_clip <- 1 - hjust_lo_clip

    for (cat in sort(unique(plot_df$Category))) {
      sub_cat <- plot_df[plot_df$Category == cat, , drop = FALSE]
      modal_theta <- sub_cat$Theta[which.max(sub_cat$Probability)]
      hjust_val <- (modal_theta - theta_range[1L]) / diff(theta_range)
      hjust_val <- max(hjust_lo_clip, min(hjust_hi_clip, hjust_val))
      p <- p +
        geomtextpath::geom_textpath(
          data = sub_cat,
          hjust = hjust_val,
          vjust = -0.08,
          linewidth = line_width,
          size = text_size
        )
    }
  }

  p
}

# Tests for RMitemCatProb()

make_polytomous <- function(n = 200, k = 4, seed = 1L, max_score = 3L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:max_score, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

make_dichotomous <- function(n = 200, k = 5, seed = 2L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMitemCatProb errors on min != 0", {
  skip_if_not_installed("eRm")
  df <- make_polytomous() + 1L
  expect_error(RMitemCatProb(df), regexp = "scored starting at 0")
})

test_that("RMitemCatProb errors when fewer than 2 items", {
  skip_if_not_installed("eRm")
  df <- make_polytomous()[, 1L, drop = FALSE]
  expect_error(RMitemCatProb(df), regexp = "at least 2 columns")
})

test_that("RMitemCatProb errors on invalid theta_range", {
  skip_if_not_installed("eRm")
  df <- make_polytomous()
  expect_error(RMitemCatProb(df, theta_range = c(2, -2)),
               regexp = "theta_range\\[1\\] < theta_range\\[2\\]")
  expect_error(RMitemCatProb(df, theta_range = 1),
               regexp = "length 2")
})

test_that("RMitemCatProb errors on bad n_points", {
  skip_if_not_installed("eRm")
  df <- make_polytomous()
  expect_error(RMitemCatProb(df, n_points = 1L), regexp = "integer >= 2")
  expect_error(RMitemCatProb(df, n_points = 50.5), regexp = "integer >= 2")
})

test_that("RMitemCatProb errors on bad viridis_end", {
  skip_if_not_installed("eRm")
  df <- make_polytomous()
  expect_error(RMitemCatProb(df, viridis_end = 0),  regexp = "in \\(0, 1\\]")
  expect_error(RMitemCatProb(df, viridis_end = 1.5), regexp = "in \\(0, 1\\]")
})

test_that("RMitemCatProb errors when item_labels length mismatches", {
  skip_if_not_installed("eRm")
  df <- make_polytomous()
  expect_error(RMitemCatProb(df, item_labels = c("a", "b")),
               regexp = "same length")
})

test_that("RMitemCatProb errors when category_labels length mismatches", {
  skip_if_not_installed("eRm")
  df <- make_polytomous(max_score = 3L)  # max=3 -> 4 categories
  expect_error(RMitemCatProb(df, category_labels = c("low", "high")),
               regexp = "same length")
})

# ---------------------------------------------------------------------
# Polytomous output
# ---------------------------------------------------------------------
test_that("RMitemCatProb returns a ggplot for polytomous data", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  p <- RMitemCatProb(df)
  expect_s3_class(p, "ggplot")
})

test_that("RMitemCatProb dataframe output has correct structure + sums to 1", {
  skip_if_not_installed("eRm")
  df <- make_polytomous(k = 4L, max_score = 3L)
  res <- RMitemCatProb(df, n_points = 50L, output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_named(res, c("Item", "Category", "Theta", "Probability"))
  # 4 items * 4 categories (0..3) * 50 theta points = 800 rows
  expect_equal(nrow(res), 4L * 4L * 50L)
  expect_s3_class(res$Item, "factor")
  # Per-(Item, Theta) cell, probabilities sum to exactly 1
  sums <- tapply(res$Probability, list(res$Item, res$Theta), sum)
  expect_true(all(abs(sums - 1) < 1e-10))
  # Probabilities in [0, 1]
  expect_true(all(res$Probability >= 0 & res$Probability <= 1))
})

test_that("RMitemCatProb item_labels are applied to facet titles", {
  skip_if_not_installed("eRm")
  df <- make_polytomous()
  labs <- paste("Custom", seq_len(ncol(df)))
  res <- RMitemCatProb(df, item_labels = labs, output = "dataframe")
  # Each item facet level should contain the custom label
  uniq_levels <- levels(res$Item)
  expect_true(all(grepl("Custom", uniq_levels)))
})

# ---------------------------------------------------------------------
# Dichotomous fallback (RM)
# ---------------------------------------------------------------------
test_that("RMitemCatProb handles dichotomous data via RM", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  df <- make_dichotomous()
  p <- RMitemCatProb(df)
  expect_s3_class(p, "ggplot")
  res <- RMitemCatProb(df, n_points = 30L, output = "dataframe")
  # 5 items * 2 categories * 30 theta points = 300 rows
  expect_equal(nrow(res), 5L * 2L * 30L)
  # Sums to 1 per (Item, Theta)
  sums <- tapply(res$Probability, list(res$Item, res$Theta), sum)
  expect_true(all(abs(sums - 1) < 1e-10))
})

# ---------------------------------------------------------------------
# Monotonic boundaries: at the extreme low theta, P(Category 0) -> 1
# At the extreme high theta, P(Category max) -> 1
# ---------------------------------------------------------------------
# ---------------------------------------------------------------------
# Path mode (geomtextpath single-item branch)
# ---------------------------------------------------------------------
test_that("RMitemCatProb path mode requires the `item` argument", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("geomtextpath")
  df <- make_polytomous()
  expect_error(
    RMitemCatProb(df, label_curves = "path"),
    regexp = "requires an `item` argument"
  )
})

test_that("RMitemCatProb path mode rejects unknown item name", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("geomtextpath")
  df <- make_polytomous()
  expect_error(
    RMitemCatProb(df, label_curves = "path", item = "Z9"),
    regexp = "not found in data"
  )
})

test_that("RMitemCatProb path mode rejects out-of-range item index", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("geomtextpath")
  df <- make_polytomous()
  expect_error(
    RMitemCatProb(df, label_curves = "path", item = 999L),
    regexp = "index must be an integer"
  )
  expect_error(
    RMitemCatProb(df, label_curves = "path", item = 1.5),
    regexp = "index must be an integer"
  )
})

test_that("RMitemCatProb path mode returns a ggplot for polytomous data", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("geomtextpath")
  df <- make_polytomous()
  p <- RMitemCatProb(df, label_curves = "path", item = "I1")
  expect_s3_class(p, "ggplot")
})

test_that("RMitemCatProb path mode also works for dichotomous (no middle tier)", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("geomtextpath")
  df <- make_dichotomous()
  p <- RMitemCatProb(df, label_curves = "path", item = "I1")
  expect_s3_class(p, "ggplot")
})

test_that("RMitemCatProb path mode dataframe output is subset to the chosen item", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("geomtextpath")
  df  <- make_polytomous(k = 4L, max_score = 3L)
  res <- RMitemCatProb(df, label_curves = "path", item = "I2",
                       n_points = 30L, output = "dataframe")
  # 1 item * 4 categories * 30 theta points
  expect_equal(nrow(res), 1L * 4L * 30L)
  expect_true(all(grepl("^I2", as.character(res$Item))))
})

test_that("RMitemCatProb path mode default theta_range widens to (-5, 5)", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("geomtextpath")
  df  <- make_polytomous()
  res <- RMitemCatProb(df, label_curves = "path", item = "I1",
                       output = "dataframe")
  expect_equal(range(res$Theta), c(-5, 5))

  # User-supplied theta_range should still be honoured
  res2 <- RMitemCatProb(df, label_curves = "path", item = "I1",
                        theta_range = c(-3, 3), output = "dataframe")
  expect_equal(range(res2$Theta), c(-3, 3))
})

test_that("RMitemCatProb path mode validates text_size", {
  skip_if_not_installed("eRm")
  skip_if_not_installed("geomtextpath")
  df <- make_polytomous()
  expect_error(RMitemCatProb(df, label_curves = "path", item = "I1",
                             text_size = -1),
               regexp = "positive number")
  expect_error(RMitemCatProb(df, label_curves = "path", item = "I1",
                             text_size = c(3, 4)),
               regexp = "positive number")
})

test_that("RMitemCatProb path mode produces one geom_textpath layer per category", {
  # Sanity check the new per-category hjust strategy: there should be
  # exactly (max_score + 1) geom_textpath layers, one per response
  # category. This catches regressions if the layer loop is ever
  # changed back to a single layer.
  skip_if_not_installed("eRm")
  skip_if_not_installed("geomtextpath")
  df <- make_polytomous(max_score = 3L)  # categories 0..3 -> 4 layers
  p  <- RMitemCatProb(df, label_curves = "path", item = "I1")
  geom_classes <- vapply(
    p$layers,
    function(l) class(l$geom)[1L],
    character(1L)
  )
  expect_equal(sum(geom_classes == "GeomTextpath"), 4L)
})

test_that("RMitemCatProb boundary probabilities behave correctly", {
  skip_if_not_installed("eRm")
  df  <- make_polytomous(max_score = 3L)
  res <- RMitemCatProb(df, theta_range = c(-10, 10), n_points = 100L,
                       output = "dataframe")
  # At theta = -10, category 0 should dominate; at theta = +10, category 3 should
  low  <- res[res$Theta == min(res$Theta), ]
  high <- res[res$Theta == max(res$Theta), ]
  # Per item: argmax of probability at low end should be category 0
  for (it in unique(low$Item)) {
    sub <- low[low$Item == it, ]
    expect_equal(sub$Category[which.max(sub$Probability)], 0L,
                 info = paste("Item:", it))
  }
  for (it in unique(high$Item)) {
    sub <- high[high$Item == it, ]
    expect_equal(sub$Category[which.max(sub$Probability)], 3L,
                 info = paste("Item:", it))
  }
})

#' Person-Item Targeting Plot (Wright Map)
#'
#' Produces a three-panel targeting plot with a shared logit scale x-axis:
#' \enumerate{
#'   \item \strong{Top}: Histogram of person location estimates, with a
#'     reference line for the mean (or median) and shading for ±1 SD
#'     (or ±1 MAD).
#'   \item \strong{Middle}: Inverted histogram of item threshold locations,
#'     with the same summary annotations.
#'   \item \strong{Bottom}: Dot-and-whisker plot of individual item thresholds
#'     with confidence intervals based on threshold standard errors.
#' }
#'
#' Together, the top and middle panels form a back-to-back histogram that
#' makes it easy to assess whether the test is well-targeted to the sample.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Missing values (`NA`) are allowed.
#' @param robust Logical. If `FALSE` (the default), histogram annotations use
#'   mean ± SD. If `TRUE`, median ± MAD is used instead.
#' @param sort_items Character string controlling item ordering on the y-axis
#'   of the bottom panel. `"data"` (the default) preserves the column order in
#'   `data` (first item at top). `"location"` sorts items by their average
#'   threshold location (easiest at top, hardest at bottom).
#' @param bins Integer. Number of bins for both histograms. Default is number of
#'   unique scores divided by 2, but no less than 11.
#' @param xlim Numeric vector of length 2. Initial lower and upper limits for
#'   the shared x-axis. Automatically expanded if any person or item threshold
#'   values fall outside these limits.
#' @param ci_level Numeric. Confidence level for the item threshold error bars.
#'   Default is `0.95` (95% CI). Set to `NULL` to hide error bars.
#' @param person_fill Fill colour for the person histogram. Default
#'   `"#0072B2"` (blue).
#' @param threshold_fill Fill colour for the item threshold histogram. Default
#'   `"#D55E00"` (vermillion).
#' @param height_ratios Numeric vector of length 3 specifying the relative
#'   heights of the top (person), middle (threshold), and bottom (dot-whisker)
#'   panels. Default `c(3, 2, 5)`.
#' @param output Character string. `"patchwork"` (the default) returns the
#'   combined patchwork plot. `"list"` returns a named list of the three
#'   ggplot objects (`p1`, `p2`, `p3`) for further customisation.
#'
#' @return
#' * If `output = "patchwork"`: a `patchwork` object (combined `ggplot`).
#' * If `output = "list"`: a named list with elements `p1` (person histogram),
#'   `p2` (threshold histogram), and `p3` (item threshold dot-whisker plot).
#'
#' @details
#' \strong{Estimation method selection.}
#' The function checks whether any item response category has fewer than 3
#' observations. If all categories have at least 3 responses, item threshold
#' locations and their standard errors are estimated via Conditional Maximum
#' Likelihood (CML) using `psychotools::pcmodel()` (a dichotomous item is a
#' 2-category PCM). If any category has fewer than 3 responses, the function
#' falls back to Marginal Maximum Likelihood (MML) estimation via
#' `mirt::mirt()` with `itemtype = "Rasch"` and `SE = TRUE`, which is more
#' numerically stable under sparse-category conditions. A message is emitted
#' when the MML fallback is used.
#'
#' In both cases, item threshold locations are centered (shifted so the grand
#' mean of all thresholds equals zero).
#'
#' \strong{Person estimates} are obtained by Warm's weighted likelihood (WLE)
#' from the fitted item thresholds, consistent with the rest of the package.
#' WLE is finite at extreme scores, so all-zero and perfect responders are
#' located rather than dropped.
#'
#' \strong{Confidence intervals} for item thresholds are based on Wald-type
#' intervals: threshold estimate ± z × SE, where z is the standard normal
#' quantile corresponding to `ci_level`.
#'
#' The `ggplot2` and `patchwork` packages must be installed (they are in
#' Suggests, not Imports).
#'
#' @references
#' Wright, B. D. & Stone, M. H. (1979). *Best Test Design*. MESA Press.
#'
#' @seealso [psychotools::pcmodel()], [mirt::mirt()]
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggplot2", quietly = TRUE) &&
#'     requireNamespace("patchwork", quietly = TRUE)) {
#'   # Polytomous example
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:3, 200 * 8, replace = TRUE), nrow = 200, ncol = 8)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:8)
#'
#'   # Default: mean/SD, data order, 95% CI
#'   RMtargeting(sim_data)
#'
#'   # Robust (median/MAD), sorted by location, 84% CI
#'   RMtargeting(sim_data, robust = TRUE, sort_items = "location",
#'               ci_level = 0.84)
#'
#'   # Get list of sub-plots for customisation
#'   plots <- RMtargeting(sim_data, output = "list")
#'   plots$p1 + ggplot2::ggtitle("My custom title")
#'
#'   # Dichotomous example
#'   sim_bin <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_bin) <- paste0("Item", 1:10)
#'   RMtargeting(sim_bin)
#' }
#' }
RMtargeting <- function(
  data,
  robust = FALSE,
  sort_items = c("data", "location"),
  bins,
  xlim = c(-4, 4),
  ci_level = 0.95,
  person_fill = "#0072B2",
  threshold_fill = "#D55E00",
  height_ratios = c(3, 2, 5),
  output = "patchwork"
) {
  # --- Check required packages ------------------------------------------------
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for RMtargeting() but is not installed.\n",
      "Install it with: install.packages(\"ggplot2\")",
      call. = FALSE
    )
  }
  if (!requireNamespace("patchwork", quietly = TRUE)) {
    stop(
      "Package 'patchwork' is required for RMtargeting() but is not installed.\n",
      "Install it with: install.packages(\"patchwork\")",
      call. = FALSE
    )
  }

  # if no manual value was selected for number of bins, use the number of
  # unique scores divided by 2, but no less than 10
  if (missing(bins)) {
    bins <- max(rowSums(data, na.rm = TRUE)) / 2
    if (bins < 11) {
      bins <- 11
    }
  }

  sort_items <- match.arg(sort_items)
  output <- match.arg(output, c("patchwork", "list"))

  validate_response_data(data)

  data <- as.data.frame(data)
  n_total <- nrow(data)
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)
  n_used <- nrow(data)

  data_mat <- as.matrix(data)
  max_score <- max(data_mat, na.rm = TRUE)
  is_dicho <- max_score == 1L
  item_names <- names(data)
  n_items <- ncol(data)

  # --- Check for sparse categories -------------------------------------------
  sparse <- .has_sparse_categories(data, min_n = 3L)

  # --- Estimate item thresholds -----------------------------------------------
  if (sparse) {
    message(
      "Some response categories have fewer than 3 observations. ",
      "Using MML estimation (mirt) for item thresholds."
    )
    thresh_info <- .estimate_thresholds_mml(data, is_dicho)
  } else {
    thresh_info <- .estimate_thresholds_cml(data, is_dicho)
  }

  # data.frame: Item, Threshold, Location, SE
  item_thresholds <- thresh_info$thresholds

  # --- Person estimates: WLE on the fitted item thresholds --------------------
  # Reconstruct the per-item threshold list from the (centred) locations and
  # estimate person locations by Warm's WLE, consistent with the rest of the
  # package and finite at extreme scores. Persons and items share a scale by
  # construction, so the targeting overlap is unaffected by the centring.
  thr_list <- lapply(
    split(
      item_thresholds$Location,
      factor(item_thresholds$Item, levels = names(data))
    ),
    as.numeric
  )
  person_theta <- .estimate_thetas(
    as.matrix(data),
    thr_list,
    method = "WLE"
  )$theta
  person_theta <- person_theta[is.finite(person_theta)]

  # --- Compute CI bounds for thresholds ---------------------------------------
  show_ci <- !is.null(ci_level) && all(!is.na(item_thresholds$SE))
  if (show_ci) {
    z_val <- stats::qnorm(1 - (1 - ci_level) / 2)
    item_thresholds$CI_low <- item_thresholds$Location -
      z_val * item_thresholds$SE
    item_thresholds$CI_high <- item_thresholds$Location +
      z_val * item_thresholds$SE
  }

  # --- Auto-expand xlim -------------------------------------------------------
  all_values <- c(person_theta, item_thresholds$Location)
  if (show_ci) {
    all_values <- c(all_values, item_thresholds$CI_low, item_thresholds$CI_high)
  }
  if (max(all_values, na.rm = TRUE) > xlim[2]) {
    xlim[2] <- ceiling(max(all_values, na.rm = TRUE))
  }
  if (min(all_values, na.rm = TRUE) < xlim[1]) {
    xlim[1] <- floor(min(all_values, na.rm = TRUE))
  }

  # --- Summary statistics -----------------------------------------------------
  if (robust) {
    p_center <- stats::median(person_theta, na.rm = TRUE)
    p_spread <- stats::mad(person_theta, na.rm = TRUE)
    t_center <- stats::median(item_thresholds$Location, na.rm = TRUE)
    t_spread <- stats::mad(item_thresholds$Location, na.rm = TRUE)
    center_label <- "Median"
    spread_label <- "MAD"
  } else {
    p_center <- mean(person_theta, na.rm = TRUE)
    p_spread <- stats::sd(person_theta, na.rm = TRUE)
    t_center <- mean(item_thresholds$Location, na.rm = TRUE)
    t_spread <- stats::sd(item_thresholds$Location, na.rm = TRUE)
    center_label <- "Mean"
    spread_label <- "SD"
  }

  # --- Item ordering for bottom panel -----------------------------------------
  if (sort_items == "location") {
    item_means <- stats::aggregate(
      Location ~ Item,
      data = item_thresholds,
      FUN = mean,
      na.rm = TRUE
    )
    # Easiest at top = lowest location at top.
    # ggplot places first factor level at bottom, so reverse.
    item_order <- rev(item_means$Item[order(item_means$Location)])
  } else {
    # Data order: first column at top -> reverse for ggplot
    item_order <- rev(item_names)
  }

  # ============================================================================
  # TOP PANEL: Person histogram
  # ============================================================================
  person_df <- data.frame(theta = person_theta)

  p1 <- ggplot2::ggplot(person_df, ggplot2::aes(x = .data$theta)) +
    ggplot2::geom_histogram(
      bins = bins,
      fill = person_fill,
      colour = "white",
      alpha = 0.85
    ) +
    ggplot2::annotate(
      "rect",
      xmin = p_center - p_spread,
      xmax = p_center + p_spread,
      ymin = -Inf,
      ymax = Inf,
      fill = person_fill,
      alpha = 0.12
    ) +
    ggplot2::geom_vline(
      xintercept = p_center,
      linewidth = 0.8,
      linetype = "dashed",
      colour = "grey20"
    ) +
    ggplot2::annotate(
      "text",
      x = p_center,
      y = Inf,
      vjust = -0.5,
      label = paste0(
        center_label,
        " = ",
        round(p_center, 2),
        ", ",
        spread_label,
        " = ",
        round(p_spread, 2)
      ),
      size = 3.2,
      colour = "grey20"
    ) +
    ggplot2::scale_y_continuous(
      breaks = function(lim) {
        seq(0, floor(lim[2]), by = max(1, round(lim[2] / 6)))
      }
    ) +
    ggplot2::scale_x_continuous(breaks = seq(xlim[1], xlim[2], by = 1)) +
    ggplot2::coord_cartesian(xlim = xlim, clip = "off") +
    ggplot2::labs(x = NULL, y = "Persons") +
    ggplot2::theme_bw() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_blank(),
      axis.ticks.x = ggplot2::element_blank(),
      plot.margin = ggplot2::margin(5, 5, 0, 5)
    ) +
    er2_axis_margins()

  # ============================================================================
  # MIDDLE PANEL: Inverted threshold histogram
  # ============================================================================
  thresh_hist_df <- data.frame(location = item_thresholds$Location)

  p2 <- ggplot2::ggplot(thresh_hist_df, ggplot2::aes(x = .data$location)) +
    ggplot2::geom_histogram(
      bins = bins,
      fill = threshold_fill,
      colour = "white",
      alpha = 0.85
    ) +
    ggplot2::annotate(
      "rect",
      xmin = t_center - t_spread,
      xmax = t_center + t_spread,
      ymin = -Inf,
      ymax = Inf,
      fill = threshold_fill,
      alpha = 0.12
    ) +
    ggplot2::geom_vline(
      xintercept = t_center,
      linewidth = 0.8,
      linetype = "dashed",
      colour = "grey20"
    ) +
    ggplot2::annotate(
      "text",
      x = t_center,
      y = -Inf,
      vjust = 1.5,
      label = paste0(
        center_label,
        " = ",
        round(t_center, 2),
        ", ",
        spread_label,
        " = ",
        round(t_spread, 2)
      ),
      size = 3.2,
      colour = "grey20"
    ) +
    ggplot2::scale_y_reverse(
      minor_breaks = NULL,
      breaks = function(lim) {
        max_val <- abs(floor(lim[1]))
        seq(0, max_val, by = max(1, round(max_val / 4)))
      }
    ) +
    ggplot2::scale_x_continuous(breaks = seq(xlim[1], xlim[2], by = 1)) +
    ggplot2::coord_cartesian(xlim = xlim, clip = "off") +
    ggplot2::labs(x = NULL, y = "Thresholds") +
    ggplot2::theme_bw() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_blank(),
      axis.ticks.x = ggplot2::element_blank(),
      axis.ticks.y = ggplot2::element_blank(),
      plot.margin = ggplot2::margin(0, 5, 0, 5)
    ) +
    er2_axis_margins()

  # ============================================================================
  # BOTTOM PANEL: Item threshold dot-whisker plot
  # ============================================================================
  item_thresholds$Item <- factor(item_thresholds$Item, levels = item_order)

  # Build CI caption fragment
  ci_caption <- ""
  if (show_ci) {
    ci_caption <- paste0(
      "Error bars show ",
      round(ci_level * 100),
      "% confidence intervals."
    )
  }

  caption_text <- er2_caption(paste0(
    "Person location ",
    tolower(center_label),
    ": ",
    round(p_center, 2),
    " (",
    spread_label,
    " ",
    round(p_spread, 2),
    "). Item threshold location ",
    tolower(center_label),
    ": ",
    round(t_center, 2),
    " (",
    spread_label,
    " ",
    round(t_spread, 2),
    "). ",
    .n_caption(
      n_used,
      n_total,
      if (has_na) "incomplete responses retained" else character()
    ),
    ".\n",
    ci_caption
  ))

  if (is_dicho) {
    p3 <- ggplot2::ggplot(
      item_thresholds,
      ggplot2::aes(x = .data$Location, y = .data$Item)
    ) +
      ggplot2::geom_point(size = 3, colour = threshold_fill)

    if (show_ci) {
      p3 <- p3 +
        ggplot2::geom_errorbar(
          ggplot2::aes(xmin = .data$CI_low, xmax = .data$CI_high),
          width = 0.25,
          linewidth = 0.5,
          colour = threshold_fill,
          orientation = "y"
        )
    }

    p3 <- p3 +
      ggplot2::coord_cartesian(xlim = xlim) +
      ggplot2::labs(
        x = "Location (logit scale)",
        y = NULL,
        caption = caption_text
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        plot.margin = ggplot2::margin(0, 5, 5, 5)
      ) +
      er2_axis_margins() +
      er2_plot_caption()
  } else {
    p3 <- ggplot2::ggplot(
      item_thresholds,
      ggplot2::aes(
        x = .data$Location,
        y = .data$Item,
        colour = .data$Threshold
      )
    )

    if (show_ci) {
      p3 <- p3 +
        ggplot2::geom_errorbar(
          ggplot2::aes(xmin = .data$CI_low, xmax = .data$CI_high),
          width = 0.25,
          linewidth = 0.5,
          position = ggplot2::position_dodge(width = 0.4),
          orientation = "y"
        )
    }

    p3 <- p3 +
      ggplot2::geom_point(
        size = 2.5,
        position = ggplot2::position_dodge(width = 0.4)
      ) +
      ggplot2::scale_colour_viridis_d(end = 0.9) +
      ggplot2::scale_x_continuous(breaks = seq(xlim[1], xlim[2], by = 1)) +
      ggplot2::coord_cartesian(xlim = xlim) +
      ggplot2::labs(
        x = "Location (logit scale)",
        y = NULL,
        colour = "Threshold",
        caption = caption_text
      ) +
      ggplot2::theme_bw() +
      ggplot2::theme(
        legend.position = "bottom",
        plot.margin = ggplot2::margin(0, 5, 5, 5)
      ) +
      er2_axis_margins() +
      er2_plot_caption()
  }

  # --- Return -----------------------------------------------------------------
  if (output == "list") {
    return(list(p1 = p1, p2 = p2, p3 = p3))
  }

  p1 / p2 / p3 + patchwork::plot_layout(heights = height_ratios)
}


# ── Internal helpers ─────────────────────────────────────────────────────────

#' Check for sparse response categories
#'
#' Returns `TRUE` if any item has a response category (within the item's
#' observed range 0..max) with fewer than `min_n` responses.
#'
#' @param data A data.frame of item responses.
#' @param min_n Minimum count threshold.
#' @return Logical scalar.
#' @noRd
.has_sparse_categories <- function(data, min_n = 3L) {
  for (j in seq_len(ncol(data))) {
    col_vals <- data[[j]]
    col_vals <- col_vals[!is.na(col_vals)]
    if (length(col_vals) == 0L) {
      next
    }
    max_cat <- max(col_vals)
    counts <- tabulate(col_vals + 1L, nbins = max_cat + 1L)
    if (any(counts < min_n)) return(TRUE)
  }
  FALSE
}


#' Estimate item thresholds via CML (eRm)
#'
#' Returns a list with `$thresholds` (data.frame: Item, Threshold, Location,
#' SE) and `$erm_out` (the fitted eRm model object, needed for person
#' parameter estimation).
#'
#' @param data A data.frame of item responses.
#' @param is_dicho Logical. `TRUE` for dichotomous data.
#' @return Named list.
#' @noRd
.estimate_thresholds_cml <- function(data, is_dicho) {
  # CML Andrich thresholds + SEs via psychotools (a dichotomous item is a
  # 2-category PCM), on the grand-mean-zero scale. Locations match the previous
  # eRm values; SEs differ slightly (psychotools vs eRm vcov). Person locations
  # are estimated downstream by WLE from these thresholds.
  fit <- psychotools::pcmodel(data)
  tp <- psychotools::threshpar(fit, vcov = TRUE)
  se_all <- sqrt(diag(attr(tp, "vcov")))
  loc <- unlist(lapply(tp, as.numeric), use.names = FALSE)
  loc <- loc - mean(loc, na.rm = TRUE)

  thresh_df <- data.frame(
    Item = rep(names(tp), lengths(tp)),
    Threshold = paste0("T", unlist(lapply(tp, seq_along), use.names = FALSE)),
    Location = loc,
    SE = as.numeric(se_all),
    stringsAsFactors = FALSE
  )
  rownames(thresh_df) <- NULL
  list(thresholds = thresh_df, erm_out = NULL)
}


#' Estimate item thresholds via MML (mirt) for sparse-category data
#'
#' Uses `mirt::mirt()` with `itemtype = "Rasch"` and `SE = TRUE` for threshold
#' estimation (more stable with sparse categories). Person locations are
#' estimated downstream by WLE from these thresholds.
#'
#' @param data A data.frame of item responses.
#' @param is_dicho Logical. `TRUE` for dichotomous data.
#' @return Named list with `$thresholds` and `$erm_out`.
#' @noRd
.estimate_thresholds_mml <- function(data, is_dicho) {
  mirt_out <- mirt::mirt(
    data,
    model = 1,
    itemtype = "Rasch",
    SE = TRUE,
    verbose = FALSE
  )

  # Extract per-item coefficients with SEs in IRT parameterisation
  item_coefs <- mirt::coef(
    mirt_out,
    IRTpars = TRUE,
    printSE = TRUE,
    simplify = FALSE
  )
  # item_coefs is a list: one element per item + "GroupPars"
  # Each item element is a matrix with rows: "par", "SE" and cols: "a", "b" (or "b1","b2",...)
  n_items <- ncol(data)

  thresh_list <- vector("list", n_items)

  for (i in seq_len(n_items)) {
    item_mat <- item_coefs[[i]]
    b_cols <- grep("^b\\d*$", colnames(item_mat), value = TRUE)
    if (length(b_cols) == 0L) {
      b_cols <- "b"
    }
    b_vals <- item_mat["par", b_cols]
    b_ses <- item_mat["SE", b_cols]

    thresh_list[[i]] <- data.frame(
      Item = names(data)[i],
      Threshold = paste0("T", seq_along(b_cols)),
      Location = as.numeric(b_vals),
      SE = as.numeric(b_ses),
      stringsAsFactors = FALSE
    )
  }

  thresh_df <- do.call(rbind, thresh_list)
  rownames(thresh_df) <- NULL

  # Center thresholds (SEs are invariant to location shift)
  grand_mean <- mean(thresh_df$Location, na.rm = TRUE)
  thresh_df$Location <- thresh_df$Location - grand_mean

  # Person locations are estimated downstream by WLE from these thresholds.
  list(thresholds = thresh_df, erm_out = NULL)
}

# Tests for RMtargeting()

make_polytomous <- function(n = 200, k = 6, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:3, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

make_dichotomous <- function(n = 200, k = 8, seed = 2L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMtargeting errors when data has non-zero minimum", {
  df <- make_dichotomous() + 1L
  expect_error(RMtargeting(df), regexp = "scored starting at 0")
})

# ---------------------------------------------------------------------
# Output structures
# ---------------------------------------------------------------------
test_that("RMtargeting default returns a patchwork/ggplot figure on polytomous data", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("patchwork")
  skip_if_not_installed("eRm")
  df <- make_polytomous()
  p  <- RMtargeting(df)
  # patchwork-composed plots also inherit ggplot
  expect_s3_class(p, "ggplot")
})

test_that("RMtargeting default returns a figure on dichotomous data", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("patchwork")
  skip_if_not_installed("eRm")
  df <- make_dichotomous()
  p  <- RMtargeting(df)
  expect_s3_class(p, "ggplot")
})

test_that("RMtargeting output = 'list' returns sub-plots", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("eRm")
  df <- make_polytomous()
  res <- RMtargeting(df, output = "list")
  expect_type(res, "list")
  expect_true(length(res) >= 1L)
})

test_that("RMtargeting accepts robust = TRUE and sort_items = 'location'", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("patchwork")
  skip_if_not_installed("eRm")
  df <- make_polytomous()
  p <- RMtargeting(df, robust = TRUE, sort_items = "location",
                   ci_level = 0.84)
  expect_s3_class(p, "ggplot")
})

#' Partial Gamma DIF Analysis
#'
#' Computes partial gamma coefficients for Differential Item Functioning (DIF)
#' using \code{iarm::partgam_DIF()}. Each item is tested for association with
#' a single categorical DIF variable, controlling for the total score.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Missing values (`NA`) are allowed,
#'   but at least one complete case must exist after combining `data` and
#'   `dif_var`.
#' @param dif_var A vector (factor or character) of the same length as
#'   `nrow(data)`, representing the grouping variable for DIF analysis.
#' @param cutoff Optional. Default `NULL` (no cutoff applied). Can be:
#'   * The return value of \code{\link{RMdifGammaCutoff}} (a list with
#'     `$item_cutoffs`): the data.frame is extracted automatically and
#'     simulation metadata is included in the kable caption.
#'   * The `$item_cutoffs` data.frame from \code{\link{RMdifGammaCutoff}}
#'     directly: must have columns `Item`, `gamma_low`, `gamma_high`.
#'   When provided, adds columns `Gamma_low`, `Gamma_high`, and `Flagged`
#'   (logical; `TRUE` when the observed partial gamma falls outside the
#'   credible range) to the result.
#' @param p_value Logical. When `TRUE`, adds two-sided bootstrap p-values
#'   (`p_gamma`, `padj_gamma`) comparing each item's observed partial gamma
#'   against its simulated null distribution, and `flagged` reflects
#'   `padj_gamma < alpha` instead of the credible range. The asymptotic
#'   BH-adjusted p-value and star columns from `iarm::partgam_DIF()` are
#'   **dropped** in this mode (two p-value families in one table would invite
#'   double-reading); the simulated `gamma_low` / `gamma_high` band is kept as
#'   the effect-size reference. Requires the **full**
#'   \code{\link{RMdifGammaCutoff}} object as `cutoff` (it carries the
#'   simulated distributions in `$results`). Default `FALSE`.
#' @param correction Character. Multiplicity correction for the bootstrap
#'   p-values: `"fwer"` (default; Westfall-Young studentised-max step-down),
#'   `"fdr_bh"`, `"fdr_by"`, or `"none"`. Ignored when `p_value = FALSE`.
#' @param alpha Numeric in (0, 1). Significance level used to flag items on
#'   the corrected p-value. Default `0.05`. Ignored when `p_value = FALSE`.
#' @param output Character string controlling the return value. Either
#'   `"kable"` (default) for a formatted `knitr::kable()` table, or
#'   `"dataframe"` for the underlying data.frame.
#'
#' @return
#' * If `output = "kable"`: a `knitr_kable` object with columns "Item",
#'   "Partial gamma", "SE", "Lower CI", "Upper CI", "Adj. p-value (BH)",
#'   and "p-value sign." (a star-string indicator from
#'   `iarm::partgam_DIF()`). When `cutoff` is provided, additional columns
#'   "Gamma low", "Gamma high", and "Flagged" are included. With
#'   `p_value = TRUE`, the asymptotic p-value columns are replaced by
#'   bootstrap "p" and "p (adj)".
#' * If `output = "dataframe"`: a data.frame with columns `Item`, `gamma`,
#'   `se`, `lower`, `upper`, `padj_bh`, `Significance`. When `cutoff` is
#'   provided, columns `gamma_low`, `gamma_high`, and `flagged` are also
#'   included. With `p_value = TRUE`, `padj_bh` and `Significance` are
#'   replaced by `p_gamma` and `padj_gamma`.
#'
#' @details
#' Partial gamma (Bjorner et al., 1998) measures the association between item
#' response and an exogenous grouping variable, controlling for the total
#' score. Values near 0 indicate no DIF. Recommended interpretive thresholds
#' (Bjorner et al., 1998):
#'
#' * **No or negligible DIF**: gamma within \eqn{[-0.21, 0.21]}, *or* gamma
#'   not significantly different from 0.
#' * **Slight to moderate DIF**: gamma within \eqn{[-0.31, 0.31]} (and outside
#'   \eqn{[-0.21, 0.21]}), *or* not significantly outside \eqn{[-0.21, 0.21]}.
#' * **Moderate to large DIF**: gamma outside \eqn{[-0.31, 0.31]}, **and**
#'   significantly outside \eqn{[-0.21, 0.21]}.
#'
#' The `iarm` package must be installed (it is in Suggests, not Imports).
#'
#' \strong{Bootstrap p-values.} When `p_value = TRUE`, each item's observed
#' partial gamma is compared against its simulated null distribution (from
#' `cutoff$results`, where the DIF variable is random by construction). The
#' per-item statistic is the residual studentised by the bootstrap mean and
#' SD; the marginal p-value is the two-sided Monte-Carlo p-value
#' `(1 + #\{|t*| >= |t|\}) / (B + 1)`, so it can be no smaller than
#' `1 / (B + 1)`. `correction = "fwer"` uses the Westfall-Young
#' studentised-max step-down, which exploits the bootstrap dependence among
#' items (Ferreira, 2024); it is liberal when the simulation is small, so at
#' least 1000 `iterations` in [RMdifGammaCutoff()] are recommended (a warning
#' is issued below that). Unlike the asymptotic p-values from
#' `iarm::partgam_DIF()`, these are calibrated against the *simulated Rasch
#' null* rather than the asymptotic SE; they are model-conditional and
#' sample-size-sensitive, and are reported alongside the simulated
#' effect-size band, not in place of it.
#'
#' @inheritSection RMitemInfit Multiple comparisons
#'
#' @references
#' Bjorner, J. B., Kreiner, S., Ware, J. E., Damsgaard, M. T., &
#' Bech, P. (1998). Differential item functioning in the Danish translation
#' of the SF-36. *Journal of Clinical Epidemiology, 51*(11), 1189--1202.
#' \doi{10.1016/S0895-4356(98)00111-5}
#'
#' Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number
#' of hypotheses simultaneously. *Journal of Statistical Theory and Practice,
#' 19*(6). \doi{10.1007/s42519-024-00412-4}
#'
#' Westfall, P. H., & Young, S. S. (1993). *Resampling-Based Multiple Testing*.
#' Wiley.
#'
#' @seealso \code{\link{RMdifGammaCutoff}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:10)
#'   dif_group <- factor(sample(c("A", "B"), 200, replace = TRUE))
#'
#'   # Default kable output
#'   RMdifGamma(sim_data, dif_group)
#'
#'   # Return as data.frame
#'   RMdifGamma(sim_data, dif_group, output = "dataframe")
#'
#'   # Simulation-based cutoffs (100 Monte-Carlo iterations)
#'   if (requireNamespace("ggdist", quietly = TRUE)) {
#'     cutoff_res <- RMdifGammaCutoff(sim_data, dif_var = dif_group,
#'                                    iterations = 100, parallel = FALSE,
#'                                    seed = 42)
#'     RMdifGamma(sim_data, dif_group, cutoff = cutoff_res)
#'
#'     # Bootstrap p-values with family-wise (Westfall-Young) correction
#'     # (use iterations >= 1000 in real analyses for stable p-values)
#'     RMdifGamma(sim_data, dif_group, cutoff = cutoff_res, p_value = TRUE,
#'                output = "dataframe")
#'   }
#' }
#' }
RMdifGamma <- function(
  data,
  dif_var,
  cutoff = NULL,
  p_value = FALSE,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  alpha = 0.05,
  output = "kable"
) {
  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required for RMdifGamma() but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }

  output <- match.arg(output, c("kable", "dataframe"))
  correction <- match.arg(correction)
  if (!is.numeric(alpha) || length(alpha) != 1L || alpha <= 0 || alpha >= 1) {
    stop("`alpha` must be a single number in (0, 1).", call. = FALSE)
  }

  validate_response_data(data)

  # --- Validate dif_var -------------------------------------------------------
  if (length(dif_var) != nrow(data)) {
    stop(
      "`dif_var` must have the same length as `nrow(data)` (",
      nrow(data),
      "), but has length ",
      length(dif_var),
      ".",
      call. = FALSE
    )
  }

  # Require at least 2 levels
  dif_levels <- length(unique(stats::na.omit(dif_var)))
  if (dif_levels < 2L) {
    stop(
      "`dif_var` must have at least 2 non-missing levels, but has ",
      dif_levels,
      ".",
      call. = FALSE
    )
  }

  # --- Validate and normalise cutoff ------------------------------------------
  cutoff_n_iter <- NULL
  cutoff_method <- NULL
  cutoff_hdci_width <- NULL
  cutoff_full <- NULL # full object (carries simulated $results for p-values)
  if (!is.null(cutoff)) {
    if (
      is.list(cutoff) &&
        !is.data.frame(cutoff) &&
        "item_cutoffs" %in% names(cutoff)
    ) {
      cutoff_full <- cutoff
      cutoff_n_iter <- cutoff$actual_iterations
      cutoff_method <- cutoff$cutoff_method
      cutoff_hdci_width <- cutoff$hdci_width
      cutoff <- cutoff$item_cutoffs
    }
    if (!is.data.frame(cutoff)) {
      stop(
        "`cutoff` must be NULL, the return value of RMdifGammaCutoff(), or its ",
        "$item_cutoffs data.frame.",
        call. = FALSE
      )
    }
    required_cols <- c("Item", "gamma_low", "gamma_high")
    missing_cols <- setdiff(required_cols, names(cutoff))
    if (length(missing_cols) > 0L) {
      stop(
        "`cutoff` data.frame is missing required columns: ",
        paste(missing_cols, collapse = ", "),
        ".",
        call. = FALSE
      )
    }
  }

  # --- p-value prerequisites --------------------------------------------------
  if (p_value) {
    if (is.null(cutoff_full) || is.null(cutoff_full$results)) {
      stop(
        "`p_value = TRUE` requires the full RMdifGammaCutoff() object (it ",
        "carries the simulated distributions in $results); a NULL cutoff or ",
        "the bare $item_cutoffs data.frame is not sufficient.",
        call. = FALSE
      )
    }
    if (!is.null(cutoff_n_iter) && cutoff_n_iter < 1000L) {
      warning(
        "Bootstrap p-values are based on only ",
        cutoff_n_iter,
        " simulation iterations. With few iterations the studentised-max ",
        "(FWER) correction is liberal and small p-values are imprecise; ",
        "use iterations >= 1000 in RMdifGammaCutoff() for reliable p-values.",
        call. = FALSE
      )
    }
  }

  # --- rgl workaround ---------------------------------------------------------
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # --- Compute partial gamma DIF via iarm -------------------------------------
  sink(nullfile())
  #on.exit(sink(), add = TRUE)
  pgam_raw <- iarm::partgam_DIF(as.data.frame(data), dif_var)
  sink()

  # pgam_raw is a data.frame with character columns that need numeric conversion
  pgam_df <- data.frame(
    Item = as.character(pgam_raw$Item),
    gamma = as.numeric(pgam_raw$gamma),
    se = as.numeric(pgam_raw$se),
    pvalue = as.numeric(pgam_raw$pvalue),
    padj_bh = as.numeric(pgam_raw[[6]]),
    Significance = trimws(as.character(pgam_raw$sig)),
    lower = as.numeric(pgam_raw$lower),
    upper = as.numeric(pgam_raw$upper),
    stringsAsFactors = FALSE
  )

  # Keep output columns (unrounded; the kable path rounds a display copy)
  result_df <- pgam_df[, c(
    "Item",
    "gamma",
    "se",
    "lower",
    "upper",
    "padj_bh",
    "Significance"
  )]

  # --- Apply cutoff if provided -----------------------------------------------
  if (!is.null(cutoff)) {
    data_items <- result_df$Item
    cutoff_items <- cutoff$Item
    if (!setequal(data_items, cutoff_items)) {
      stop(
        "Item names in `cutoff` do not match item names in `data`.\n",
        "  data items  : ",
        paste(data_items, collapse = ", "),
        "\n",
        "  cutoff items: ",
        paste(cutoff_items, collapse = ", "),
        call. = FALSE
      )
    }
    cutoff_sub <- cutoff[, c("Item", "gamma_low", "gamma_high")]
    result_df <- merge(result_df, cutoff_sub, by = "Item", sort = FALSE)
    # Restore original row order
    result_df <- result_df[match(data_items, result_df$Item), ]
    rownames(result_df) <- NULL
    result_df$flagged <- result_df$gamma < result_df$gamma_low |
      result_df$gamma > result_df$gamma_high

    if (p_value) {
      # Compare observed gamma (unrounded, from iarm) to its simulated null
      # (cutoff_full$results). Two-sided: DIF in either direction matters.
      sim_items <- unique(cutoff_full$results$Item)
      if (!setequal(data_items, sim_items)) {
        stop(
          "Item names in the cutoff simulations ($results) do not match ",
          "`data`.",
          call. = FALSE
        )
      }
      sim_mat <- tapply(
        cutoff_full$results$gamma,
        list(cutoff_full$results$iteration, cutoff_full$results$Item),
        function(x) x[1L]
      )
      observed <- stats::setNames(
        as.numeric(pgam_raw$gamma),
        as.character(pgam_raw$Item)
      )
      pv <- .bootstrap_pvalues(
        observed,
        sim_mat,
        correction = correction,
        tail = "two.sided"
      )
      idx <- match(result_df$Item, pv$name)
      result_df$p_gamma <- pv$p[idx]
      result_df$padj_gamma <- pv$padj[idx]
      result_df$flagged <- !is.na(result_df$padj_gamma) &
        result_df$padj_gamma < alpha
      # Drop the asymptotic p-value pair; the bootstrap p-values replace it.
      result_df <- result_df[, c(
        "Item",
        "gamma",
        "se",
        "lower",
        "upper",
        "gamma_low",
        "gamma_high",
        "p_gamma",
        "padj_gamma",
        "flagged"
      )]
    } else {
      # Reorder columns
      result_df <- result_df[, c(
        "Item",
        "gamma",
        "se",
        "lower",
        "upper",
        "padj_bh",
        "Significance",
        "gamma_low",
        "gamma_high",
        "flagged"
      )]
    }
  }

  # --- Return -----------------------------------------------------------------
  if (output == "dataframe") {
    return(result_df)
  }

  # Kable display rounding (the dataframe output above stays unrounded)
  result_df <- .round_display(result_df, c(
    gamma = 3, se = 3, lower = 3, upper = 3, padj_bh = 3,
    gamma_low = 3, gamma_high = 3, p_gamma = 4, padj_gamma = 4
  ))

  # Build caption
  n_complete <- sum(stats::complete.cases(cbind(
    as.data.frame(data),
    dif_var = dif_var
  )))
  has_na <- anyNA(as.data.frame(data)) || anyNA(dif_var)
  n_clause <- .n_caption(
    n_complete,
    nrow(as.data.frame(data)),
    if (has_na) "complete cases" else character()
  )
  if (p_value) {
    caption_text <- paste0(
      "Partial gamma DIF analysis. ",
      n_clause,
      ". Two-sided bootstrap p-values from ",
      cutoff_n_iter,
      " iterations (replacing the asymptotic BH p-values); multiplicity ",
      "correction: ",
      .correction_label(correction),
      "; flagged at padj < ",
      alpha,
      ". p-values cannot be smaller than 1/(",
      cutoff_n_iter,
      "+1) = ",
      round(1 / (cutoff_n_iter + 1), 4),
      ". Positive gamma indicates higher scores in higher DIF group levels."
    )
  } else if (is.null(cutoff)) {
    caption_text <- paste0(
      "Partial gamma DIF analysis. ",
      n_clause,
      ". Positive gamma indicates higher scores in higher DIF group levels."
    )
  } else if (!is.null(cutoff_n_iter)) {
    method_label <- .format_gamma_cutoff_method_label(
      cutoff_method,
      cutoff_hdci_width
    )
    iter_part <- paste0(cutoff_n_iter, " simulation iterations")
    caption_text <- paste0(
      "Partial gamma DIF analysis. ",
      n_clause,
      ". Cutoff values based on ",
      if (!is.null(method_label)) {
        paste0(iter_part, " (", method_label, ").")
      } else {
        paste0(iter_part, ".")
      }
    )
  } else {
    caption_text <- paste0(
      "Partial gamma DIF analysis. ",
      n_clause,
      ". Simulation-based cutoff values applied."
    )
  }

  knitr::kable(
    result_df,
    format = "pipe",
    col.names = if (p_value) {
      c(
        "Item",
        "Partial gamma",
        "SE",
        "Lower CI",
        "Upper CI",
        "Gamma low",
        "Gamma high",
        "p",
        "p (adj)",
        "Flagged"
      )
    } else if (is.null(cutoff)) {
      c(
        "Item",
        "Partial gamma",
        "SE",
        "Lower CI",
        "Upper CI",
        "Adj. p-value (BH)",
        "p-value sign."
      )
    } else {
      c(
        "Item",
        "Partial gamma",
        "SE",
        "Lower CI",
        "Upper CI",
        "Adj. p-value (BH)",
        "p-value sign.",
        "Gamma low",
        "Gamma high",
        "Flagged"
      )
    },
    caption = caption_text
  )
}

# ---------------------------------------------------------------------------
# Internal helper
# ---------------------------------------------------------------------------

#' Format a human-readable label for the gamma cutoff method
#'
#' @param cutoff_method Character. `"hdci"`, `"quantile"`, or `NULL`.
#' @param hdci_width Numeric or `NULL`. HDCI width (e.g., `0.99`).
#' @return A character label, or `NULL` if the method is unknown/unset.
#' @keywords internal
.format_gamma_cutoff_method_label <- function(cutoff_method, hdci_width) {
  if (is.null(cutoff_method)) {
    return(NULL)
  }
  switch(
    cutoff_method,
    quantile = "2.5th/97.5th percentile",
    hdci = if (!is.null(hdci_width)) {
      paste0(hdci_width * 100, "% HDCI")
    } else {
      "HDCI"
    },
    NULL
  )
}


#' Simulation-Based Partial Gamma DIF Cutoff Determination
#'
#' Uses parametric bootstrap simulation to determine appropriate cutoff values
#' for partial gamma DIF analysis via \code{\link[iarm]{partgam_DIF}}. Under
#' a correctly fitting Rasch model where the DIF variable is unrelated to item
#' responses (i.e., no true DIF), this function generates the expected
#' distribution of absolute partial gamma values per item, providing empirical
#' critical values.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Only complete cases (rows without
#'   any `NA`) are used.
#' @param dif_var A vector (factor, character, or integer) defining group
#'   membership for DIF analysis. Must have the same length as `nrow(data)`.
#'   The actual group labels are used to determine the number of groups and
#'   their relative sizes; during simulation, respondents are randomly assigned
#'   to groups with the same proportions, so there is no true DIF by
#'   construction.
#' @param iterations Integer. Number of simulation iterations (default 250).
#' @param parallel Logical. Use parallel processing via `mirai` if available
#'   (default `TRUE`).
#' @param n_cores Integer or `NULL`. Number of parallel workers. When `NULL`,
#'   `getOption("mc.cores")` is checked first. If neither is set and
#'   `parallel = TRUE`, a warning is issued and execution falls back to
#'   sequential (single core) processing.
#' @param verbose Logical. Show a progress bar (default `FALSE`).
#' @param seed Integer or `NULL`. Random seed for reproducibility.
#' @param cutoff_method Character string specifying how cutoff intervals are
#'   computed. Either `"hdci"` (default) for the Highest Density Interval via
#'   `ggdist::hdci()`, or `"quantile"` for the 2.5th/97.5th percentiles via
#'   `stats::quantile()`.
#' @param hdci_width Numeric. Width of the HDCI when `cutoff_method = "hdci"`.
#'   Default is `0.99` (99\% HDCI). Ignored when
#'   `cutoff_method = "quantile"`.
#'
#' @return A list with components:
#' \describe{
#'   \item{`results`}{data.frame with columns `iteration`, `Item`, and
#'     `gamma` (one row per item per successful iteration).}
#'   \item{`item_cutoffs`}{data.frame with per-item cutoff summaries: `Item`,
#'     `gamma_low`, `gamma_high`. Bounds are computed using the method
#'     specified by `cutoff_method`.}
#'   \item{`actual_iterations`}{Number of successful iterations.}
#'   \item{`sample_n`}{Number of complete cases used.}
#'   \item{`sample_n_total`}{Number of respondents in the raw input data,
#'     before removing rows with `NA` in `data` or `dif_var`.}
#'   \item{`sample_has_na`}{Logical. Whether `data` or `dif_var` contained
#'     any missing values.}
#'   \item{`sample_summary`}{Summary statistics of estimated person
#'     parameters.}
#'   \item{`item_names`}{Character vector of item names from data.}
#'   \item{`dif_group_sizes`}{Named integer vector of group sizes used in the
#'     simulation (matches proportions in the observed `dif_var`).}
#'   \item{`cutoff_method`}{The method used to compute cutoffs (`"hdci"` or
#'     `"quantile"`).}
#'   \item{`hdci_width`}{The HDCI width used (only meaningful when
#'     `cutoff_method = "hdci"`).}
#' }
#'
#' @details
#' For each simulation iteration the function:
#' \enumerate{
#'   \item Resamples person parameters (thetas) with replacement from the
#'     WLE person locations.
#'   \item Simulates item response data under a Rasch model (dichotomous via
#'     `psychotools::rrm()` or polytomous via an internal partial credit
#'     simulator).
#'   \item Creates a random DIF variable by sampling group labels with the
#'     same proportions as the observed `dif_var`, so there is **no true DIF**
#'     by construction.
#'   \item Computes partial gamma DIF statistics via
#'     `iarm::partgam_DIF()`.
#' }
#'
#' The distribution of partial gamma values across iterations provides
#' empirical critical values per item. Values from real data that fall
#' outside these bounds suggest DIF that exceeds what would be expected by
#' chance under a correctly fitting Rasch model. Failed iterations (e.g.,
#' due to convergence issues or degenerate data) are silently discarded.
#'
#' The generating model uses CML item thresholds via `psychotools::pcmodel()`
#' (a dichotomous item is a 2-category PCM) and WLE person locations,
#' consistent with the rest of the package; responses are simulated with
#' `psychotools::rrm()` (dichotomous) or an internal partial credit score
#' simulator (polytomous).
#'
#' Parallel processing is provided by the `mirai` package (optional). Install
#' it with `install.packages("mirai")` to enable parallelisation.
#'
#' The `iarm` package must be installed (it is in Suggests, not Imports).
#'
#' @references
#' Bjorner, J. B., Kreiner, S., Ware, J. E., Damsgaard, M. T., &
#' Bech, P. (1998). Differential item functioning in the Danish translation
#' of the SF-36. *Journal of Clinical Epidemiology, 51*(11), 1189--1202.
#' \doi{10.1016/S0895-4356(98)00111-5}
#'
#' Henninger, M., Radek, J., Debelak, R., & Strobl, C. (2025).
#' Partial credit trees meet the partial gamma coefficient for quantifying
#' DIF and DSF in polytomous items. *Behaviormetrika, 52*, 221--257.
#' \doi{10.1007/s41237-024-00252-3}
#'
#' @seealso \code{\link[iarm]{partgam_DIF}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE) &&
#'     requireNamespace("ggdist", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:10)
#'   dif_sex <- sample(c("male", "female"), 200, replace = TRUE)
#'
#'   # Run 100 iterations sequentially for a quick demo
#'   cutoff_res <- RMdifGammaCutoff(sim_data, dif_var = dif_sex,
#'                                  iterations = 100, parallel = FALSE,
#'                                  seed = 42)
#'   cutoff_res$item_cutoffs
#' }
#' }
RMdifGammaCutoff <- function(
  data,
  dif_var,
  iterations = 250,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  cutoff_method = "hdci",
  hdci_width = 0.99
) {
  cutoff_method <- match.arg(cutoff_method, c("hdci", "quantile"))

  if (cutoff_method == "hdci" && !requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required when cutoff_method = \"hdci\" but is not installed.\n",
      "Install it with: install.packages(\"ggdist\")\n",
      "Alternatively, use cutoff_method = \"quantile\" to avoid this dependency.",
      call. = FALSE
    )
  }

  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required for RMdifGammaCutoff() but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }

  validate_response_data(data)

  # --- Validate dif_var -------------------------------------------------------
  if (missing(dif_var) || is.null(dif_var)) {
    stop("`dif_var` must be provided.", call. = FALSE)
  }
  if (length(dif_var) != nrow(data)) {
    stop(
      "`dif_var` must have the same length as `nrow(data)` (",
      nrow(data),
      "), but has length ",
      length(dif_var),
      ".",
      call. = FALSE
    )
  }

  # rgl workaround (iarm depends on vcdExtra -> rgl)
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # Only complete cases (both data and dif_var). Record the raw total and
  # whether anything was dropped so callers (e.g. RMdifGammaPlot) can report
  # the sample in the standard `n = X of Y respondents` form.
  n_total <- nrow(data)
  has_na <- anyNA(data) || anyNA(dif_var)
  complete_idx <- stats::complete.cases(data) & !is.na(dif_var)
  data <- data[complete_idx, , drop = FALSE]
  dif_var <- dif_var[complete_idx]

  if (nrow(data) == 0L) {
    stop(
      "No complete cases in data after removing rows with NA in data or dif_var.",
      call. = FALSE
    )
  }

  # Determine DIF group structure (labels and proportions)
  dif_table <- table(dif_var)
  dif_levels <- names(dif_table)
  dif_proportions <- as.numeric(dif_table) / sum(dif_table)
  n_dif_groups <- length(dif_levels)

  if (n_dif_groups < 2L) {
    stop(
      "`dif_var` must have at least 2 distinct groups, but has ",
      n_dif_groups,
      ".",
      call. = FALSE
    )
  }

  # --- Parallel setup ---------------------------------------------------------
  use_parallel <- parallel && requireNamespace("mirai", quietly = TRUE)

  if (parallel && !use_parallel) {
    message(
      "Install 'mirai' package for parallel processing: install.packages(\"mirai\")"
    )
    message("Running sequentially...")
  }

  if (use_parallel) {
    if (is.null(n_cores)) {
      n_cores <- getOption("mc.cores")
    }
    if (is.null(n_cores)) {
      warning(
        paste0(
          "For parallel processing, specify n_cores or set options(mc.cores = N).\n",
          "(Use `parallel::detectCores()` to see how many cores are available.)\n",
          "Falling back to sequential (single core) processing."
        ),
        call. = FALSE
      )
      use_parallel <- FALSE
    } else {
      n_cores <- min(n_cores, iterations)
    }
  }

  if (!is.null(seed)) {
    set.seed(seed)
  }

  # Generate per-iteration seeds
  sim_seeds <- sample.int(.Machine$integer.max, iterations)

  data_mat <- as.matrix(data)
  sample_n <- nrow(data_mat)
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L

  item_names_vec <- colnames(data_mat)

  # Generating model: CML item thresholds (psychotools) + WLE person locations,
  # consistent with the rest of the package, replacing eRm CML +
  # eRm::person.parameter() (MLE). Thetas form the pool resampled with
  # replacement to build each simulated dataset; a dichotomous item is a
  # 2-category PCM, so its centred threshold is the item difficulty for rrm().
  pool <- .wle_theta_pool(data_mat)
  thr_list <- pool$thr_list
  thetas <- pool$thetas

  if (is_polytomous) {
    sim_data_list <- list(
      type = "polytomous",
      thetas = thetas,
      deltaslist = thr_list,
      n_items = ncol(data_mat),
      sample_n = sample_n,
      item_names = item_names_vec,
      dif_levels = dif_levels,
      dif_proportions = dif_proportions
    )
  } else {
    sim_data_list <- list(
      type = "dichotomous",
      thetas = thetas,
      item_params = unlist(thr_list, use.names = FALSE),
      n_items = ncol(data_mat),
      sample_n = sample_n,
      item_names = item_names_vec,
      dif_levels = dif_levels,
      dif_proportions = dif_proportions
    )
  }

  if (use_parallel) {
    results_raw <- run_partgam_sim_parallel(
      iterations,
      sim_seeds,
      sim_data_list,
      n_cores,
      verbose
    )
  } else {
    results_raw <- run_partgam_sim_sequential(
      iterations,
      sim_seeds,
      sim_data_list,
      verbose
    )
  }

  # Filter out failures (character strings indicate errors)
  ok <- vapply(results_raw, is.data.frame, logical(1L))
  successful <- results_raw[ok]

  if (length(successful) == 0L) {
    stop("All simulation iterations failed. Check your data.", call. = FALSE)
  }

  actual_iterations <- length(successful)

  # Combine per-iteration data.frames
  iter_dfs <- lapply(seq_along(successful), function(i) {
    df <- successful[[i]]
    df$iteration <- i
    df
  })
  results_df <- do.call(rbind, iter_dfs)
  rownames(results_df) <- NULL

  # Compute per-item cutoffs
  item_names <- unique(results_df$Item)
  item_cutoffs <- do.call(
    rbind,
    lapply(item_names, function(item) {
      sub <- results_df[results_df$Item == item, ]
      if (cutoff_method == "hdci") {
        gamma_interval <- ggdist::hdci(sub$gamma, .width = hdci_width)
        data.frame(
          Item = item,
          gamma_low = gamma_interval[1L, 1L],
          gamma_high = gamma_interval[1L, 2L],
          stringsAsFactors = FALSE,
          row.names = NULL
        )
      } else {
        data.frame(
          Item = item,
          gamma_low = stats::quantile(sub$gamma, 0.025, na.rm = TRUE),
          gamma_high = stats::quantile(sub$gamma, 0.975, na.rm = TRUE),
          stringsAsFactors = FALSE,
          row.names = NULL
        )
      }
    })
  )
  rownames(item_cutoffs) <- NULL

  list(
    results = results_df,
    item_cutoffs = item_cutoffs,
    actual_iterations = actual_iterations,
    sample_n = sample_n,
    sample_n_total = n_total,
    sample_has_na = has_na,
    sample_summary = summary(thetas),
    item_names = item_names_vec,
    dif_group_sizes = as.integer(dif_table),
    cutoff_method = cutoff_method,
    hdci_width = hdci_width
  )
}

# ---------------------------------------------------------------------------
# Internal: single simulation iteration
# ---------------------------------------------------------------------------

#' Run a single partial gamma DIF simulation iteration
#'
#' @param seed Integer seed for reproducibility.
#' @param data_list List produced inside [RMdifGammaCutoff()].
#' @return A data.frame with columns `Item` and `gamma`, or a character string
#'   on failure.
#' @keywords internal
run_single_partgam_sim <- function(seed, data_list) {
  set.seed(seed)

  thetas_res <- sample(
    data_list$thetas,
    size = data_list$sample_n,
    replace = TRUE
  )

  tryCatch(
    {
      if (data_list$type == "dichotomous") {
        sim_mat <- psychotools::rrm(
          theta = thetas_res,
          beta = data_list$item_params
        )
        sim_df <- as.data.frame(sim_mat$data)
        colnames(sim_df) <- data_list$item_names

        pos_counts <- colSums(sim_df, na.rm = TRUE)
        if (any(pos_counts < 8L)) {
          return(
            "validation_failed: fewer than 8 positive responses in at least one item"
          )
        }
      } else {
        sim_mat <- sim_partial_score(data_list$deltaslist, thetas_res)
        sim_df <- as.data.frame(sim_mat)
        colnames(sim_df) <- data_list$item_names

        n_cats <- vapply(
          data_list$deltaslist,
          function(d) length(d) + 1L,
          integer(1L)
        )
        for (j in seq_len(ncol(sim_df))) {
          tab <- tabulate(sim_df[[j]] + 1L, nbins = n_cats[j])
          if (any(tab == 0L)) {
            return("validation_failed: not all categories represented")
          }
        }
      }

      # Create a random DIF variable with the same group proportions
      # but no actual relationship to item responses (no true DIF)
      random_dif <- sample(
        data_list$dif_levels,
        size = data_list$sample_n,
        replace = TRUE,
        prob = data_list$dif_proportions
      )

      # Compute partial gamma DIF via iarm.
      # iarm::partgam_DIF() prints its result table to stdout on every call,
      # which floods sequential runs (e.g. vignettes); silence it. `finally`
      # restores the sink even if the call errors (the outer tryCatch then
      # reports the failure as usual).
      sink(nullfile())
      pgam <- tryCatch(
        iarm::partgam_DIF(sim_df, random_dif),
        finally = sink()
      )

      # pgam is a data.frame with columns including "item" and "gamma"
      # (column names may vary slightly; use positional or clean names)
      pgam_df <- as.data.frame(pgam)

      data.frame(
        Item = data_list$item_names,
        gamma = as.numeric(pgam_df[seq_along(data_list$item_names), "gamma"]),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    },
    error = function(e) {
      as.character(conditionMessage(e))
    }
  )
}

# ---------------------------------------------------------------------------
# Internal: parallel runner
# ---------------------------------------------------------------------------

#' Run partial gamma DIF simulations in parallel using mirai
#'
#' @param iterations Number of iterations.
#' @param sim_seeds Integer vector of per-iteration seeds.
#' @param sim_data_list List of data passed to each worker.
#' @param n_cores Number of mirai daemons.
#' @param verbose Show progress bar.
#' @return List of raw results (one element per iteration).
#' @keywords internal
run_partgam_sim_parallel <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  n_cores,
  verbose = FALSE
) {
  mirai::daemons(n_cores)
  on.exit(mirai::daemons(0), add = TRUE)

  if (verbose) {
    message(sprintf("Starting %d daemons...", n_cores))
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
    completed <- 0L
  }

  # Submit all tasks
  tasks <- lapply(seq_len(iterations), function(sim) {
    mirai::mirai(
      {
        run_single_partgam_sim(seed, data_list)
      },
      seed = sim_seeds[sim],
      data_list = sim_data_list,
      run_single_partgam_sim = run_single_partgam_sim,
      sim_partial_score = sim_partial_score,
      sim_poly_item = sim_poly_item
    )
  })

  # Collect results
  results <- vector("list", iterations)
  for (sim in seq_len(iterations)) {
    result <- mirai::call_mirai(tasks[[sim]])$data
    if (!inherits(result, "errorValue")) {
      results[[sim]] <- result
    } else {
      results[[sim]] <- "mirai_error"
    }
    if (verbose) {
      completed <- completed + 1L
      utils::setTxtProgressBar(pb, completed)
    }
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# ---------------------------------------------------------------------------
# Internal: sequential runner
# ---------------------------------------------------------------------------

#' Run partial gamma DIF simulations sequentially
#'
#' @param iterations Number of iterations.
#' @param sim_seeds Integer vector of per-iteration seeds.
#' @param sim_data_list List of data passed to each worker.
#' @param verbose Show progress bar.
#' @return List of raw results (one element per iteration).
#' @keywords internal
run_partgam_sim_sequential <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  verbose = FALSE
) {
  if (verbose) {
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  results <- vector("list", iterations)
  for (sim in seq_len(iterations)) {
    results[[sim]] <- run_single_partgam_sim(sim_seeds[sim], sim_data_list)
    if (verbose) {
      utils::setTxtProgressBar(pb, sim)
    }
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}


#' Plot Distribution of Simulated Partial Gamma DIF Values
#'
#' Visualises the distribution of simulation-based partial gamma DIF values
#' from \code{\link{RMdifGammaCutoff}}, optionally overlaying observed partial
#' gamma values computed from real data via \code{\link[iarm]{partgam_DIF}}.
#'
#' Uses `ggdist::stat_dotsinterval()` (when `data` is not supplied) or
#' `ggdist::stat_dots()` (when `data` is supplied) with
#' `point_interval = "median_hdci"` and `.width = c(0.66, 0.95, 0.99)`.
#'
#' @param simfit The return value of \code{\link{RMdifGammaCutoff}} (a list with
#'   components `results`, `item_cutoffs`, `actual_iterations`, `sample_n`, and
#'   `item_names`).
#' @param data Optional. A data.frame or matrix of item responses for computing
#'   and overlaying observed partial gamma values. Items must be scored starting
#'   at 0 (non-negative integers). When provided, the plot includes orange
#'   diamond markers for the observed partial gamma alongside the simulated
#'   distribution, plus segment summaries from the cutoff intervals.
#' @param dif_var Required when `data` is supplied. A vector (factor, character,
#'   or integer) defining group membership for the DIF analysis. Must have the
#'   same length as `nrow(data)`.
#'
#' @return A `ggplot` object.
#'
#' @details
#' When `data` is **not** supplied, the function plots the simulated partial
#' gamma distributions as dot-interval plots using
#' `ggdist::stat_dotsinterval()` with median and Highest Density Continuous
#' Interval (HDCI) summaries.
#'
#' When `data` **is** supplied (along with `dif_var`), the function:
#' \enumerate{
#'   \item Computes observed partial gamma values via
#'     `iarm::partgam_DIF()`.
#'   \item Overlays observed gamma values as orange diamond markers on the
#'     simulated distributions.
#'   \item Shows per-item cutoff intervals (from `simfit$item_cutoffs`) as
#'     black line segments, with thicker segments for the 66\% interval and
#'     black dots for the median.
#' }
#'
#' The `ggplot2`, `ggdist`, and optionally `iarm` packages must be installed
#' (they are in Suggests, not Imports).
#'
#' @seealso \code{\link{RMdifGammaCutoff}}, \code{\link{RMdifGamma}}
#'
#' @importFrom rlang .data
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE) &&
#'     requireNamespace("ggdist", quietly = TRUE) &&
#'     requireNamespace("ggplot2", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:10)
#'   dif_group <- factor(sample(c("A", "B"), 200, replace = TRUE))
#'
#'   # Run simulation
#'   cutoff_res <- RMdifGammaCutoff(sim_data, dif_var = dif_group,
#'                                  iterations = 100, parallel = FALSE,
#'                                  seed = 42)
#'
#'   # Simulated distribution only
#'   RMdifGammaPlot(cutoff_res)
#'
#'   # With observed partial gamma overlaid
#'   RMdifGammaPlot(cutoff_res, data = sim_data, dif_var = dif_group)
#' }
#' }
RMdifGammaPlot <- function(simfit, data, dif_var) {
  # --- Check required packages ------------------------------------------------
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for RMdifGammaPlot() but is not installed.\n",
      "Install it with: install.packages(\"ggplot2\")",
      call. = FALSE
    )
  }
  if (!requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required for RMdifGammaPlot() but is not installed.\n",
      "Install it with: install.packages(\"ggdist\")",
      call. = FALSE
    )
  }

  # --- Validate simfit --------------------------------------------------------
  required_names <- c(
    "results",
    "item_cutoffs",
    "actual_iterations",
    "sample_n",
    "item_names"
  )
  missing_names <- setdiff(required_names, names(simfit))
  if (length(missing_names) > 0L) {
    stop(
      "`simfit` is missing required components: ",
      paste(missing_names, collapse = ", "),
      ".\nExpected the return value of RMdifGammaCutoff().",
      call. = FALSE
    )
  }

  results_df <- simfit$results
  item_cutoffs <- simfit$item_cutoffs
  actual_iterations <- simfit$actual_iterations
  sample_n <- simfit$sample_n
  item_names <- simfit$item_names

  # Standard sample-size clause for the (complete-case) simulation sample.
  # `sample_n_total` / `sample_has_na` are absent in cutoff objects made by
  # older versions, so fall back to the plain count.
  sample_clause <- .n_caption(
    sample_n,
    if (is.null(simfit$sample_n_total)) sample_n else simfit$sample_n_total,
    if (isTRUE(simfit$sample_has_na)) "complete cases" else character()
  )

  # Item factor levels (reversed for plotting top-to-bottom)
  item_levels <- rev(item_names)

  # --- Compute per-item summary intervals for segment overlays ----------------
  lo_hi <- do.call(
    rbind,
    lapply(item_names, function(item) {
      sub <- results_df[results_df$Item == item, ]
      data.frame(
        Item = item,
        min_gamma = stats::quantile(sub$gamma, 0.005, na.rm = TRUE),
        max_gamma = stats::quantile(sub$gamma, 0.995, na.rm = TRUE),
        p66lo_gamma = stats::quantile(sub$gamma, 0.167, na.rm = TRUE),
        p66hi_gamma = stats::quantile(sub$gamma, 0.833, na.rm = TRUE),
        median_gamma = stats::median(sub$gamma, na.rm = TRUE),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    })
  )
  rownames(lo_hi) <- NULL

  # --- Case 1: no observed data, show simulation distribution only ------------
  if (missing(data)) {
    results_plot <- data.frame(
      Item = results_df$Item,
      Value = results_df$gamma,
      stringsAsFactors = FALSE
    )
    results_plot$Item <- factor(results_plot$Item, levels = item_levels)

    p <- ggplot2::ggplot(
      results_plot,
      ggplot2::aes(
        x = .data$Value,
        y = .data$Item
      )
    ) +
      ggdist::stat_dotsinterval(
        ggplot2::aes(slab_fill = ggplot2::after_stat(.data$level)),
        quantiles = actual_iterations,
        point_interval = "median_hdci",
        layout = "weave",
        slab_color = NA,
        .width = c(0.66, 0.95, 0.99)
      ) +
      ggplot2::labs(
        x = "Partial gamma",
        y = "Item",
        caption = er2_caption(paste0(
          "Results from ",
          actual_iterations,
          " simulated datasets. ",
          sample_clause,
          " per dataset."
        ))
      ) +
      ggplot2::scale_color_manual(
        values = scales::brewer_pal()(4)[-1],
        aesthetics = "slab_fill",
        guide = "none"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(panel.spacing = ggplot2::unit(0.7, "cm")) +
      er2_axis_margins() +
      er2_plot_caption()

    return(p)
  }

  # --- Case 2: observed data supplied -----------------------------------------
  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required to compute observed partial gamma but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }

  if (missing(dif_var)) {
    stop(
      "`dif_var` must be supplied when `data` is provided.",
      call. = FALSE
    )
  }

  validate_response_data(data)

  if (length(dif_var) != nrow(data)) {
    stop(
      "`dif_var` must have the same length as `nrow(data)` (",
      nrow(data),
      "), but has length ",
      length(dif_var),
      ".",
      call. = FALSE
    )
  }

  # rgl workaround
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  sink(nullfile())
  pgam_raw <- iarm::partgam_DIF(as.data.frame(data), dif_var)
  sink()

  observed_df <- data.frame(
    Item = as.character(pgam_raw$Item),
    observed_gamma = as.numeric(pgam_raw$gamma),
    stringsAsFactors = FALSE
  )

  # --- Build plot data --------------------------------------------------------
  gamma_sim <- data.frame(
    Item = results_df$Item,
    Value = results_df$gamma,
    stringsAsFactors = FALSE
  )
  gamma_sim <- merge(gamma_sim, observed_df, by = "Item", sort = FALSE)
  gamma_sim$Item <- factor(gamma_sim$Item, levels = item_levels)

  lo_hi$Item_f <- factor(lo_hi$Item, levels = item_levels)

  caption_text <- er2_caption(paste0(
    "Results from ",
    actual_iterations,
    " simulated datasets. ",
    sample_clause,
    " per dataset.\n",
    "Orange diamonds indicate observed partial gamma DIF. ",
    "Black dots indicate median gamma from simulations."
  ))

  p <- ggplot2::ggplot(
    gamma_sim,
    ggplot2::aes(
      x = .data$Value,
      y = .data$Item
    )
  ) +
    ggdist::stat_dots(
      ggplot2::aes(slab_fill = ggplot2::after_stat(.data$level)),
      quantiles = actual_iterations,
      layout = "weave",
      slab_color = NA,
      .width = c(0.66, 0.95, 0.99)
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$min_gamma,
        xend = .data$max_gamma,
        y = .data$Item_f,
        yend = .data$Item_f
      ),
      color = "black",
      linewidth = 0.7
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$p66lo_gamma,
        xend = .data$p66hi_gamma,
        y = .data$Item_f,
        yend = .data$Item_f
      ),
      color = "black",
      linewidth = 1.2
    ) +
    ggplot2::geom_point(
      data = lo_hi,
      ggplot2::aes(
        x = .data$median_gamma,
        y = .data$Item_f
      ),
      size = 3.6
    ) +
    ggplot2::geom_point(
      ggplot2::aes(x = .data$observed_gamma),
      color = "sienna2",
      shape = 18,
      position = ggplot2::position_nudge(y = -0.1),
      size = 4
    ) +
    ggplot2::geom_vline(
      xintercept = 0,
      linetype = "dashed",
      color = "grey50",
      linewidth = 0.4
    ) +
    ggplot2::labs(
      x = "Partial gamma",
      y = "Item",
      caption = caption_text
    ) +
    ggplot2::scale_color_manual(
      values = scales::brewer_pal()(4)[-1],
      aesthetics = "slab_fill",
      guide = "none"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(panel.spacing = ggplot2::unit(0.7, "cm")) +
    er2_axis_margins() +
    er2_plot_caption()

  p
}

# Tests for RMdifGamma(), RMdifGammaCutoff(), and RMdifGammaPlot()

make_dichotomous <- function(n = 200, k = 10, seed = 42L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("Item", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMdifGamma errors when iarm is not installed", {
  skip_if(
    requireNamespace("iarm", quietly = TRUE),
    "iarm is installed; skipping missing-package error path"
  )
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  expect_error(RMdifGamma(df, grp), regexp = "iarm")
})

test_that("RMdifGamma errors when dif_var length is wrong", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df) - 1L, replace = TRUE))
  expect_error(RMdifGamma(df, grp), regexp = "same length")
})

test_that("RMdifGamma errors when dif_var has fewer than 2 levels", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  grp <- factor(rep("A", nrow(df)))
  expect_error(RMdifGamma(df, grp), regexp = "at least 2")
})

# ---------------------------------------------------------------------
# RMdifGamma output structure
# ---------------------------------------------------------------------
test_that("RMdifGamma output = 'dataframe' returns one row per item", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  res <- RMdifGamma(df, grp, output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_equal(nrow(res), ncol(df))
  expect_true(all(
    c("Item", "gamma", "se", "lower", "upper", "padj_bh", "Significance") %in%
      names(res)
  ))
  expect_true(all(res$Significance %in% c("", ".", "*", "**", "***")))
})

test_that("RMdifGamma output = 'kable' returns a knitr_kable", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("knitr")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  out <- RMdifGamma(df, grp, output = "kable")
  expect_s3_class(out, "knitr_kable")
})

# ---------------------------------------------------------------------
# RMdifGammaCutoff -- small iterations
# ---------------------------------------------------------------------
test_that("RMdifGammaCutoff returns a list with item_cutoffs + actual_iterations", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  res <- RMdifGammaCutoff(
    df,
    grp,
    iterations = 10L,
    parallel = FALSE,
    seed = 1L
  )
  expect_type(res, "list")
  expect_true("item_cutoffs" %in% names(res))
  expect_s3_class(res$item_cutoffs, "data.frame")
  expect_equal(nrow(res$item_cutoffs), ncol(df))
  expect_true(all(
    c("Item", "gamma_low", "gamma_high") %in%
      names(res$item_cutoffs)
  ))
  expect_true(res$actual_iterations >= 1L)
})

test_that("RMdifGamma accepts an RMdifGammaCutoff result and adds Flagged column", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  cuts <- RMdifGammaCutoff(
    df,
    grp,
    iterations = 5L,
    parallel = FALSE,
    seed = 1L
  )
  res <- RMdifGamma(df, grp, cutoff = cuts, output = "dataframe")
  expect_true("flagged" %in% names(res))
  expect_type(res$flagged, "logical")
})

test_that("RMdifGammaCutoff is reproducible with the same seed", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  r1 <- RMdifGammaCutoff(df, grp, iterations = 5L, parallel = FALSE, seed = 42L)
  r2 <- RMdifGammaCutoff(df, grp, iterations = 5L, parallel = FALSE, seed = 42L)
  expect_equal(r1$item_cutoffs, r2$item_cutoffs)
})

# ---------------------------------------------------------------------
# RMdifGammaPlot
# ---------------------------------------------------------------------
test_that("RMdifGammaPlot returns a ggplot from an RMdifGammaCutoff result", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  cuts <- RMdifGammaCutoff(
    df,
    grp,
    iterations = 5L,
    parallel = FALSE,
    seed = 1L
  )
  p <- RMdifGammaPlot(cuts)
  expect_s3_class(p, "ggplot")
})

test_that("RMdifGammaPlot with observed data overlay returns a ggplot", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  cuts <- RMdifGammaCutoff(
    df,
    grp,
    iterations = 5L,
    parallel = FALSE,
    seed = 1L
  )
  p <- RMdifGammaPlot(cuts, data = df, dif_var = grp)
  expect_s3_class(p, "ggplot")
})

# ---------------------------------------------------------------------
# RMdifGamma -- cutoff validation and band flagging
# ---------------------------------------------------------------------
test_that("RMdifGamma rejects malformed cutoff arguments", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  expect_error(RMdifGamma(df, grp, cutoff = "abc"), regexp = "cutoff")
  expect_error(
    RMdifGamma(df, grp, cutoff = data.frame(Item = "Item1", gamma_low = -1)),
    regexp = "missing required columns"
  )
})

test_that("RMdifGamma errors when cutoff item names do not match data", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  cuts <- RMdifGammaCutoff(
    df,
    grp,
    iterations = 5L,
    parallel = FALSE,
    seed = 1L
  )
  bad <- cuts$item_cutoffs
  bad$Item[1L] <- "NotAnItem"
  expect_error(RMdifGamma(df, grp, cutoff = bad), regexp = "do not match")
})

test_that("RMdifGamma band flag matches gamma vs bounds", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  cuts <- RMdifGammaCutoff(
    df,
    grp,
    iterations = 10L,
    parallel = FALSE,
    seed = 1L
  )
  res <- RMdifGamma(df, grp, cutoff = cuts, output = "dataframe")
  expect_equal(
    res$flagged,
    res$gamma < res$gamma_low | res$gamma > res$gamma_high
  )
})

test_that("RMdifGamma handles missing values in data and dif_var", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  df[1:3, 1] <- NA
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  grp[4:5] <- NA
  res <- RMdifGamma(df, grp, output = "dataframe")
  expect_equal(nrow(res), ncol(df))
  kbl <- RMdifGamma(df, grp)
  expect_match(paste(kbl, collapse = "\n"), "complete cases")
})

# ---------------------------------------------------------------------
# RMdifGamma -- bootstrap p-values
# ---------------------------------------------------------------------
test_that("RMdifGamma p_value requires the full cutoff object", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  expect_error(
    RMdifGamma(df, grp, p_value = TRUE),
    regexp = "full RMdifGammaCutoff"
  )
  cuts <- RMdifGammaCutoff(
    df,
    grp,
    iterations = 5L,
    parallel = FALSE,
    seed = 1L
  )
  expect_error(
    RMdifGamma(df, grp, cutoff = cuts$item_cutoffs, p_value = TRUE),
    regexp = "full RMdifGammaCutoff"
  )
})

test_that("RMdifGamma p_value replaces asymptotic p columns and flags at alpha", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  cuts <- RMdifGammaCutoff(
    df,
    grp,
    iterations = 10L,
    parallel = FALSE,
    seed = 1L
  )
  expect_warning(
    res <- RMdifGamma(
      df,
      grp,
      cutoff = cuts,
      p_value = TRUE,
      output = "dataframe"
    ),
    regexp = "only 10 simulation iterations"
  )
  expect_named(
    res,
    c(
      "Item",
      "gamma",
      "se",
      "lower",
      "upper",
      "gamma_low",
      "gamma_high",
      "p_gamma",
      "padj_gamma",
      "flagged"
    )
  )
  expect_false(any(c("padj_bh", "Significance") %in% names(res)))
  # p is rounded to 4 dp, so compare against the rounded floor
  B <- cuts$actual_iterations
  p_floor <- round(1 / (B + 1), 4)
  expect_true(all(res$p_gamma >= p_floor & res$p_gamma <= 1, na.rm = TRUE))
  expect_true(all(res$padj_gamma >= res$p_gamma, na.rm = TRUE))
  expect_equal(res$flagged, !is.na(res$padj_gamma) & res$padj_gamma < 0.05)

  # kable renders with the correction label
  suppressWarnings(
    kbl <- RMdifGamma(df, grp, cutoff = cuts, p_value = TRUE)
  )
  expect_s3_class(kbl, "knitr_kable")
  expect_match(paste(kbl, collapse = "\n"), "Westfall-Young")
})

test_that("RMdifGamma p_value correction variants run and 'none' is marginal", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  cuts <- RMdifGammaCutoff(
    df,
    grp,
    iterations = 10L,
    parallel = FALSE,
    seed = 1L
  )
  suppressWarnings({
    none <- RMdifGamma(
      df,
      grp,
      cutoff = cuts,
      p_value = TRUE,
      correction = "none",
      output = "dataframe"
    )
    bh <- RMdifGamma(
      df,
      grp,
      cutoff = cuts,
      p_value = TRUE,
      correction = "fdr_bh",
      output = "dataframe"
    )
  })
  expect_equal(none$p_gamma, none$padj_gamma)
  expect_true(all(bh$padj_gamma >= bh$p_gamma, na.rm = TRUE))
})

test_that("RMdifGamma validates alpha", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  grp <- factor(sample(c("A", "B"), nrow(df), replace = TRUE))
  expect_error(RMdifGamma(df, grp, alpha = 0), regexp = "alpha")
  expect_error(RMdifGamma(df, grp, alpha = 1.2), regexp = "alpha")
})

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end

## Suppress R CMD check NOTEs for variables used in mirai expressions
utils::globalVariables(c("seed", "data_list"))

NULL

# Internal patched copies of two iarm functions used by RMscoreSE().
#
# The upstream `iarm::person_estimates()` and `iarm::persons.mle()` (iarm 0.4.x)
# converge poorly for extreme raw scores: the default `sthetarange` and `maxval`
# can leave the boundary scores with NA / non-finite SEs. The functions below
# are direct copies of the iarm code with these defaults widened so that score-
# to-logit tables produced for typical Rasch analyses come out clean at the
# tails.
#
# Original source: https://github.com/cran/iarm/blob/master/R/Person-Fit.R
# License: GPL (>= 2), as iarm.

#' Patched iarm::person_estimates()
#'
#' Score-to-theta lookup with MLE and WLE columns plus optional precision
#' properties (SEM, bias, RMSE, score-SEM). Used internally by [RMscoreSE()].
#'
#' @param object An eRm `Rm` object or a psychotools `raschmodel` / `pcmodel`.
#' @param properties Logical. Return precision properties.
#' @param allperson Logical. If TRUE, return one row per respondent rather than
#'   per raw score.
#' @param sthetarange Numeric length 2. Theta search range for boundary scores.
#'
#' @return A matrix or list, depending on `properties` / `allperson`.
#'
#' @keywords internal
#' @noRd
iarm_person_estimates <- function(object, properties = FALSE, allperson = FALSE,
                                  sthetarange = c(-10, 10)) {

  if (!any("Rm" %in% class(object),
           class(object) %in% c("raschmodel", "pcmodel"))) {
    stop("`object` must be of class Rm, raschmodel, or pcmodel.",
         call. = FALSE)
  }
  if (class(object)[1L] == "pcmodel")    object$model <- "pcmodel"
  if (class(object)[1L] == "raschmodel") object$model <- "raschmodel"

  X <- if (object$model %in% c("raschmodel", "pcmodel")) object$data else object$X

  if (object$model %in% c("RM", "raschmodel")) {
    k <- dim(X)[2L]
    coeff <- if (object$model == "RM") (-1) * stats::coef(object) else psychotools::itempar(object)
    m <- k
    respm <- rbind(rep(0, k), lower.tri(matrix(1, k, k)) + diag(k))
  } else {
    if (object$model == "PCM") {
      thr <- eRm::thresholds(object)[[3L]][[1L]][, -1L]
      coeff <- thr - mean(thr, na.rm = TRUE)
    } else {
      coeff <- stats::coef(psychotools::threshpar(object), type = "matrix")
    }
    k  <- dim(X)[2L]
    mi <- apply(X, 2L, max, na.rm = TRUE)
    m  <- sum(mi)
    respm <- matrix(0, ncol = k, nrow = m + 1L)
    respm[, 1L] <- c(0:mi[1L], rep(mi[1L], nrow(respm) - mi[1L] - 1L))
    for (i in 2:k) {
      respm[, i] <- c(rep(0, cumsum(mi)[i - 1L] + 1L),
                      1:mi[i],
                      rep(mi[i], nrow(respm) - cumsum(mi)[i] - 1L))
    }
  }

  mode <- if (object$model == "pcmodel") {
    "PCM"
  } else if (object$model == "raschmodel") {
    "RM"
  } else {
    object$model
  }

  mm <- cbind(
    0:m,
    iarm_persons_mle(respm, coeff, model = mode, type = "MLE")[, 1L],
    iarm_persons_mle(respm, coeff, model = mode, type = "WLE")[, 1L]
  )
  rownames(mm) <- rep(" ", m + 1L)
  colnames(mm) <- c("Raw Score", "MLE", "WLE")

  if (allperson) {
    rv <- rowSums(X, na.rm = TRUE)
    return(mm[rv + 1L, ])
  }

  if (!properties) return(mm)

  if (object$model %in% c("RM", "raschmodel")) {
    koeff <- as.list(coeff)
  } else {
    koeff <- lapply(as.list(as.data.frame(t(coeff))),
                    function(x) cumsum(stats::na.omit(x)))
  }
  gr <- psychotools::elementary_symmetric_functions(koeff)[[1L]]

  s.theta <- function(r) {
    function(x) {
      ((exp(x * (0:m)) * gr) / as.vector(exp(x * (0:m)) %*% gr)) %*% (0:m) - r
    }
  }

  if (object$model %in% c("pcmodel", "raschmodel")) {
    mm[1L, 2L] <- NA
  } else {
    mm[1L, 2L] <- eRm::person.parameter(object)$pred.list[[1L]]$y[1L]
  }
  try(mm[1L, 2L]      <- stats::uniroot(s.theta(0.25),     sthetarange)$root,
      silent = TRUE)
  try(mm[m + 1L, 2L]  <- stats::uniroot(s.theta(m - 0.25), sthetarange)$root,
      silent = TRUE)

  rvec <- 0:m
  pers_prop <- function(x, persons) {
    pr        <- (exp(x[2L] * rvec) * gr) / as.vector(exp(x[2L] * rvec) %*% gr)
    bias      <- pr %*% persons - x[2L]
    sem       <- sqrt((persons - as.vector(pr %*% persons))^2 %*% pr)
    rsem      <- sqrt((persons - x[2L])^2 %*% pr)
    scoresem  <- sqrt((rvec - x[1L])^2 %*% pr)
    c(SEM = sem, Bias = bias, RMSE = rsem, Score.SEM = scoresem)
  }

  list(
    cbind(mm[, 1:2], t(apply(mm[, c(1L, 2L)], 1L, pers_prop, persons = mm[, 2L]))),
    cbind(mm[, c(1L, 3L)], t(apply(mm[, c(1L, 3L)], 1L, pers_prop, persons = mm[, 3L])))
  )
}

#' Patched iarm::persons.mle()
#'
#' Newton-style WLE / MLE estimation given a response matrix and item-threshold
#' matrix. `maxval = 9` retains finite WLE values across a wide raw-score range.
#'
#' @keywords internal
#' @noRd
iarm_persons_mle <- function(respm, thresh,
                             model = c("RM", "PCM"),
                             theta = rep(0, dim(respm)[1L]),
                             type  = c("MLE", "WLE"),
                             extreme = TRUE,
                             maxit = 20L, maxdelta = 3, tol = 1e-04, maxval = 9) {

  n  <- dim(respm)[1L]
  k  <- dim(respm)[2L]
  rv <- rowSums(respm, na.rm = TRUE)
  mode <- match.arg(model)
  typ  <- match.arg(type)

  cll.rasch <- function(theta) {
    ksi <- exp(theta)
    mm  <- outer(ksi, 1 / exp(thresh))
    mm[is.na(respm)] <- 0
    dll  <- rv - rowSums(mm / (1 + mm))
    d2ll <- -rowSums(mm / (1 + mm)^2)
    d3ll <- 0
    if (typ == "WLE") {
      d3ll <- -rowSums((mm * (1 - mm)) / (1 + mm)^3)
      if (!extreme) {
        d3ll[rv == 0L]    <- 0
        d3ll[rv == maxr]  <- 0
      }
    }
    list(dll = dll, d2ll = d2ll, d3ll = d3ll)
  }

  cll.pcm <- function(theta) {
    dlogki <- function(i) {
      mmn <- exp(outer(theta, 1:mi[i]) +
                   matrix(psi.l[[i]], ncol = mi[i], nrow = n, byrow = TRUE))
      kd  <- 1 + rowSums(mmn)
      kd1 <- rowSums(matrix(1:mi[i],         ncol = mi[i], nrow = n, byrow = TRUE) * mmn)
      kd2 <- rowSums(matrix((1:mi[i])^2,     ncol = mi[i], nrow = n, byrow = TRUE) * mmn)
      kd3 <- rowSums(matrix((1:mi[i])^3,     ncol = mi[i], nrow = n, byrow = TRUE) * mmn)
      cbind(
        dlli  = kd1 / kd,
        d2lli = kd2 / kd - (kd1 / kd)^2,
        d3lli = -kd3 / kd + 3 * kd2 * kd1 / (kd^2) - 2 * (kd1 / kd)^3
      )
    }
    mm <- sapply(1:k, dlogki)
    mm[is.na(rbind(respm, respm, respm))] <- 0
    dll  <- rv -  rowSums(mm[1:n, ])
    d2ll <- -rowSums(mm[(n + 1):(2 * n), ])
    d3ll <- 0
    if (typ == "WLE") {
      d3ll <- rowSums(mm[(2 * n + 1):(3 * n), ])
      if (!extreme) {
        d3ll[rv == 0L]   <- 0
        d3ll[rv == maxr] <- 0
      }
    }
    list(dll = dll, d2ll = d2ll, d3ll = d3ll)
  }

  iter <- 1L
  conv <- 1
  if (mode == "RM") {
    maxr <- apply(respm, 1L, function(x) sum(!is.na(x)))
    clog <- cll.rasch
  } else {
    thresh.l <- apply(thresh, 1L, function(x) as.vector(stats::na.omit(x)),
                      simplify = FALSE)
    psi.l    <- lapply(thresh.l, function(x) (-1) * cumsum(x))
    mi       <- sapply(psi.l, length)
    maxr     <- apply(respm, 1L, function(x) sum(mi[!is.na(x)]))
    clog     <- cll.pcm
  }

  while ((conv > tol) & (iter <= maxit)) {
    theta0 <- theta
    fn     <- clog(theta)
    delta  <- -fn[[1L]] / fn[[2L]]
    if (typ == "WLE") {
      delta <- -fn[[1L]] / fn[[2L]] - fn[[3L]] / (2 * fn[[2L]]^2)
    }
    maxdelta <- maxdelta / 1.05
    delta    <- ifelse(abs(delta) > maxdelta, sign(delta) * maxdelta, delta)
    theta    <- theta + delta
    theta    <- ifelse(abs(theta) > maxval,   sign(theta) * maxval,    theta)
    conv     <- max(abs(theta - theta0))
    iter     <- iter + 1L
  }

  se    <- sqrt(abs(-1 / fn[[2L]]))
  se    <- ifelse(abs(theta) == maxval, NA, se)
  theta <- ifelse(theta ==  maxval,  Inf,
           ifelse(theta == -maxval, -Inf, theta))

  structure(data.frame(est = theta, se = se),
            model = mode, type = typ)
}

#' Bootstrap Item-Restscore Misfit Detection
#'
#' Non-parametric bootstrap of item-restscore fit using
#' `iarm::item_restscore()`. For each iteration, a sample of size `samplesize`
#' is drawn from `data` with replacement, the appropriate Rasch model is
#' refitted, and item-restscore results are classified as `"overfit"`,
#' `"underfit"`, or `"no misfit"` based on the BH-adjusted p-value (< .05) and
#' the sign of `expected - observed`. The function returns the percentage of
#' iterations in which each item is flagged.
#'
#' Useful with large samples, where the asymptotic test underlying
#' \code{\link{RMitemRestscore}} can flag items that are not practically
#' misfitting; bootstrapping gives a more nuanced view of the probability of
#' an item actually being misfit.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers).
#' @param iterations Integer. Number of bootstrap samples (default 200).
#' @param samplesize Integer. Size of each bootstrap sample (default 600). Must
#'   not exceed `nrow(data)`.
#' @param parallel Logical. Use parallel processing via `mirai` if available
#'   (default `TRUE`).
#' @param n_cores Integer or `NULL`. Number of parallel workers. When `NULL`,
#'   `getOption("mc.cores")` is checked first. If neither is set and
#'   `parallel = TRUE`, a warning is issued and execution falls back to
#'   sequential processing.
#' @param cutoff Numeric. Items flagged in fewer than this percentage of
#'   iterations are excluded from the kable output (default 5). Has no effect
#'   when `output = "dataframe"` or `output = "raw"`.
#' @param verbose Logical. Show a progress bar (default `FALSE`).
#' @param seed Integer or `NULL`. Random seed for reproducibility.
#' @param output Either `"kable"` (default) for a formatted `knitr::kable()`
#'   table, `"dataframe"` for the per-item summary data.frame, or `"raw"` for
#'   the per-iteration long data.frame (useful for custom plotting).
#'
#' @return
#' * If `output = "kable"`: a `knitr_kable` object listing items flagged in
#'   more than `cutoff`% of iterations, with columns Item, Item-restscore
#'   result, % of iterations, Conditional MSQ infit, and Relative item
#'   location, and a caption noting iteration count, bootstrap size, and
#'   number of complete cases.
#' * If `output = "dataframe"`: a data.frame with one row per item ×
#'   classification combination (`Item`, `item_restscore`, `n`, `percent`,
#'   `Infit_MSQ`, `Relative_location`), including `"no misfit"` rows.
#' * If `output = "raw"`: a long data.frame with one row per item ×
#'   successful iteration (`iteration`, `Item`, `item_restscore`, `diff`,
#'   `diff_abs`), where `diff = expected - observed`.
#'
#' @details
#' The full-sample model is fitted by CML via `psychotools::pcmodel()` (a
#' dichotomous item is a 2-category partial credit model), and item locations
#' (mean of the grand-mean-centred CML Andrich thresholds) and the mean WLE
#' person location are computed from it — consistent with
#' \code{\link{RMitemRestscore}} and the rest of the package. Each bootstrap
#' iteration draws a sample of size `samplesize` with replacement and refits via
#' `psychotools::pcmodel(..., hessian = FALSE)` for speed;
#' `iarm::item_restscore()` accepts the fitted model.
#'
#' Conditional infit MSQ (computed once on the full sample via
#' `iarm::out_infit()`) and relative item locations are reported alongside the
#' bootstrap percentages for context. The item-restscore classification and the
#' infit statistic are conditional and engine-invariant; only the relative-item
#' location shifts slightly relative to the previous `eRm` implementation,
#' because it now uses the WLE person mean (finite at extreme scores) rather
#' than the `eRm` MLE mean.
#'
#' Iterations that fail (e.g., due to convergence issues on a degenerate
#' bootstrap sample) are silently discarded; the caption / `actual_iterations`
#' reflects only successful runs.
#'
#' Parallel processing is provided by the `mirai` package (optional). Install
#' it with `install.packages("mirai")` to enable parallelisation.
#'
#' The `iarm` package must be installed (it is in Suggests, not Imports).
#'
#' @references
#' Kreiner, S. (2011). A Note on Item-Restscore Association in Rasch Models.
#' *Applied Psychological Measurement, 35*(7), 557-561.
#' \doi{10.1177/0146621611410227}
#'
#' @seealso \code{\link{RMitemRestscore}}, \code{\link{RMitemInfit}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE)) {
#' set.seed(42)
#' sim_data <- as.data.frame(
#'   matrix(sample(0:1, 400 * 8, replace = TRUE), nrow = 400, ncol = 8)
#' )
#' colnames(sim_data) <- paste0("Item", 1:8)
#'
#' # Few iterations for a fast example; use 100+ in real analyses
#' # Default kable output (only items flagged > cutoff%)
#' RMitemRestscoreBoot(sim_data, iterations = 50, samplesize = 300,
#'                 parallel = FALSE, seed = 1)
#'
#' # Per-item summary data.frame (all classifications, including "no misfit")
#' summary_df <- RMitemRestscoreBoot(sim_data, iterations = 50, samplesize = 300,
#'                               parallel = FALSE, seed = 1,
#'                               output = "dataframe")
#'
#' # Per-iteration long data for custom plotting
#' raw_df <- RMitemRestscoreBoot(sim_data, iterations = 50, samplesize = 300,
#'                           parallel = FALSE, seed = 1, output = "raw")
#'
#' # Distribution of (expected - observed) across iterations, per item
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   library(ggplot2)
#'   ggplot(raw_df, aes(x = Item, y = diff)) +
#'     geom_hline(yintercept = 0, linetype = "dashed", colour = "grey50") +
#'     geom_violin(fill = "grey90", colour = NA) +
#'     geom_jitter(aes(colour = item_restscore),
#'                 width = 0.15, alpha = 0.5, size = 0.8) +
#'     scale_colour_manual(values = c("overfit"   = "#377eb8",
#'                                    "underfit"  = "#e41a1c",
#'                                    "no misfit" = "grey60")) +
#'     labs(y = "Expected - observed restscore correlation",
#'          colour = NULL) +
#'     theme_minimal() +
#'     theme(axis.text.x = element_text(angle = 45, hjust = 1))
#' }
#' }
#' }
RMitemRestscoreBoot <- function(
  data,
  iterations = 200,
  samplesize = 600,
  parallel = TRUE,
  n_cores = NULL,
  cutoff = 5,
  verbose = FALSE,
  seed = NULL,
  output = "kable"
) {
  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required for RMitemRestscoreBoot() but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }

  output <- match.arg(output, c("kable", "dataframe", "raw"))

  validate_response_data(data)

  # Respondents with no responses at all contribute nothing, break the CML
  # fit (psychotools errors on all-NA rows), and would poison the bootstrap
  # resampling pool; drop them, keeping the raw totals for the caption.
  n_total <- nrow(as.data.frame(data))
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)

  if (samplesize > nrow(data)) {
    stop(
      paste0(
        "`samplesize` (",
        samplesize,
        ") cannot be larger than the number of rows in `data` (",
        nrow(data),
        ")."
      ),
      call. = FALSE
    )
  }

  # rgl workaround for iarm dependency chain (vcdExtra -> rgl)
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  data_mat <- as.matrix(data)
  item_names <- colnames(data_mat)
  n_items <- ncol(data_mat)

  # --- Full-sample model: locations + conditional infit ----------------------
  # CML item parameters (psychotools; a dichotomous item is a 2-category PCM)
  # and WLE person locations, consistent with RMitemRestscore() and the rest of
  # the package. The item-restscore and infit statistics from iarm are
  # conditional and engine-invariant; only the relative-location reference
  # shifts slightly (WLE vs eRm MLE person mean).
  fit_full <- psychotools::pcmodel(data)
  thr_list <- .center_thresholds(lapply(
    psychotools::threshpar(fit_full),
    as.numeric
  ))
  item_avg_locations <- vapply(thr_list, mean, numeric(1L))
  person_avg_location <- mean(
    .estimate_thetas(data_mat, thr_list, method = "WLE")$theta,
    na.rm = TRUE
  )
  relative_item_avg_locations <- item_avg_locations - person_avg_location

  cfit <- iarm::out_infit(fit_full)
  n_complete <- nrow(stats::na.omit(data))

  # --- Parallel setup --------------------------------------------------------
  use_parallel <- parallel && requireNamespace("mirai", quietly = TRUE)
  if (parallel && !use_parallel) {
    message(
      "Install 'mirai' package for parallel processing: install.packages(\"mirai\")"
    )
    message("Running sequentially...")
  }
  if (use_parallel) {
    if (is.null(n_cores)) {
      n_cores <- getOption("mc.cores")
    }
    if (is.null(n_cores)) {
      warning(
        paste0(
          "For parallel processing, specify n_cores or set options(mc.cores = N).\n",
          "(Use `parallel::detectCores()` to see how many cores are available.)\n",
          "Falling back to sequential (single core) processing."
        ),
        call. = FALSE
      )
      use_parallel <- FALSE
    } else {
      n_cores <- min(n_cores, iterations)
    }
  }

  # --- Per-iteration seeds for reproducibility -------------------------------
  if (!is.null(seed)) {
    set.seed(seed)
  }
  boot_seeds <- sample.int(.Machine$integer.max, iterations)

  boot_data_list <- list(
    data = data,
    samplesize = samplesize,
    item_names = item_names
  )

  # --- Run bootstrap ---------------------------------------------------------
  if (use_parallel) {
    results_raw <- run_boot_restscore_parallel(
      iterations,
      boot_seeds,
      boot_data_list,
      n_cores,
      verbose
    )
  } else {
    results_raw <- run_boot_restscore_sequential(
      iterations,
      boot_seeds,
      boot_data_list,
      verbose
    )
  }

  ok <- vapply(results_raw, is.data.frame, logical(1L))
  successful <- results_raw[ok]

  if (length(successful) == 0L) {
    stop("All bootstrap iterations failed. Check your data.", call. = FALSE)
  }
  actual_iterations <- length(successful)

  # Tag with iteration index and stack
  iter_dfs <- lapply(seq_along(successful), function(i) {
    df <- successful[[i]]
    df$iteration <- i
    df[, c("iteration", "Item", "item_restscore", "diff", "diff_abs")]
  })
  fit_all <- do.call(rbind, iter_dfs)
  rownames(fit_all) <- NULL

  # --- Return raw per-iteration long data ------------------------------------
  if (output == "raw") {
    return(fit_all)
  }

  # --- Per-item classification counts ----------------------------------------
  classes <- c("overfit", "underfit", "no misfit")
  combos <- expand.grid(
    Item = item_names,
    item_restscore = classes,
    KEEP.OUT.ATTRS = FALSE,
    stringsAsFactors = FALSE
  )
  counts <- as.data.frame(
    table(
      Item = factor(fit_all$Item, levels = item_names),
      item_restscore = factor(fit_all$item_restscore, levels = classes)
    ),
    responseName = "n",
    stringsAsFactors = FALSE
  )
  counts$Item <- as.character(counts$Item)
  counts$item_restscore <- as.character(counts$item_restscore)

  fit_tbl <- merge(
    combos,
    counts,
    by = c("Item", "item_restscore"),
    all.x = TRUE,
    sort = FALSE
  )
  fit_tbl$n[is.na(fit_tbl$n)] <- 0L
  per_item_total <- tapply(fit_tbl$n, fit_tbl$Item, sum)
  # Unrounded values; the kable path rounds a display copy before rendering.
  fit_tbl$percent <- fit_tbl$n * 100 / per_item_total[fit_tbl$Item]

  cfit_df <- data.frame(
    Item = item_names,
    Infit_MSQ = as.numeric(cfit$Infit),
    Relative_location = as.numeric(relative_item_avg_locations),
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  result_df <- merge(fit_tbl, cfit_df, by = "Item", sort = FALSE)
  result_df <- result_df[
    order(match(result_df$Item, item_names), result_df$item_restscore),
  ]
  rownames(result_df) <- NULL

  # --- Return per-item summary dataframe -------------------------------------
  if (output == "dataframe") {
    return(result_df)
  }

  # Kable display rounding (the dataframe output above stays unrounded); done
  # before the percent filter below so the kable keeps its previous behaviour
  # (filtering on the displayed, rounded percentage).
  result_df <- .round_display(result_df, c(
    percent = 1, Infit_MSQ = 2, Relative_location = 2
  ))

  # --- Kable: only flagged items above cutoff --------------------------------
  show <- result_df[
    result_df$item_restscore != "no misfit" &
      result_df$percent > cutoff,
    ,
    drop = FALSE
  ]

  if (nrow(show) == 0L) {
    message(paste0(
      "No item indicates misfit in more than ",
      cutoff,
      "% of iterations."
    ))
    return(invisible(NULL))
  }

  show <- show[order(show$item_restscore, -show$percent), ]
  rownames(show) <- NULL
  show <- show[, c(
    "Item",
    "item_restscore",
    "percent",
    "Infit_MSQ",
    "Relative_location"
  )]

  knitr::kable(
    show,
    format = "pipe",
    col.names = c(
      "Item",
      "Item-restscore result",
      "% of iterations",
      "Conditional MSQ infit",
      "Relative item location"
    ),
    caption = paste0(
      "Conditional mean-square infit from ",
      actual_iterations,
      " successful bootstrap iterations (resample size ",
      samplesize,
      ", ",
      n_items,
      " items). ",
      .n_caption(
        n_complete,
        n_total,
        if (has_na) "complete cases" else character()
      ),
      "."
    )
  )
}

# ---------------------------------------------------------------------------
# Internal: single bootstrap iteration
# ---------------------------------------------------------------------------

#' Run a single item-restscore bootstrap iteration
#'
#' @param seed Integer seed for reproducibility.
#' @param data_list List produced inside [RMitemRestscoreBoot()] containing `data`,
#'   `samplesize`, `item_names`.
#' @return A data.frame with columns `Item`, `item_restscore`, `diff`,
#'   `diff_abs`, or a character string on failure.
#' @keywords internal
run_single_boot_restscore <- function(seed, data_list) {
  set.seed(seed)
  idx <- sample.int(nrow(data_list$data), data_list$samplesize, replace = TRUE)
  d <- data_list$data[idx, , drop = FALSE]

  tryCatch(
    {
      # CML refit via psychotools::pcmodel() for both dichotomous (2-category PCM)
      # and polytomous data; accepted by iarm::item_restscore() and faster than
      # eRm. hessian = FALSE skips the (unused) SE computation.
      model_fit <- psychotools::pcmodel(d, hessian = FALSE)

      i1 <- as.data.frame(iarm::item_restscore(model_fit))
      res_mat <- i1[[1L]]
      n_items <- length(data_list$item_names)

      observed <- as.numeric(res_mat[seq_len(n_items), 1L])
      expected <- as.numeric(res_mat[seq_len(n_items), 2L])
      p_adj <- as.numeric(res_mat[seq_len(n_items), 5L])
      diff_val <- expected - observed

      cls <- ifelse(
        p_adj < 0.05 & diff_val < 0,
        "overfit",
        ifelse(p_adj < 0.05 & diff_val > 0, "underfit", "no misfit")
      )

      data.frame(
        Item = data_list$item_names,
        item_restscore = cls,
        diff = diff_val,
        diff_abs = abs(diff_val),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    },
    error = function(e) as.character(conditionMessage(e))
  )
}

# ---------------------------------------------------------------------------
# Internal: parallel runner (mirai)
# ---------------------------------------------------------------------------

#' Run item-restscore bootstrap iterations in parallel using mirai
#'
#' @param iterations Number of iterations.
#' @param boot_seeds Integer vector of per-iteration seeds.
#' @param boot_data_list List of data passed to each worker.
#' @param n_cores Number of mirai daemons.
#' @param verbose Show progress bar.
#' @return List of raw results (one element per iteration).
#' @keywords internal
run_boot_restscore_parallel <- function(
  iterations,
  boot_seeds,
  boot_data_list,
  n_cores,
  verbose = FALSE
) {
  mirai::daemons(n_cores)
  on.exit(mirai::daemons(0), add = TRUE)

  if (verbose) {
    message(sprintf("Starting %d daemons...", n_cores))
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  tasks <- lapply(seq_len(iterations), function(i) {
    mirai::mirai(
      {
        run_single_boot_restscore(seed, data_list)
      },
      seed = boot_seeds[i],
      data_list = boot_data_list,
      run_single_boot_restscore = run_single_boot_restscore
    )
  })

  results <- vector("list", iterations)
  for (i in seq_len(iterations)) {
    res <- mirai::call_mirai(tasks[[i]])$data
    results[[i]] <- if (inherits(res, "errorValue")) "mirai_error" else res
    if (verbose) utils::setTxtProgressBar(pb, i)
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# ---------------------------------------------------------------------------
# Internal: sequential runner
# ---------------------------------------------------------------------------

#' Run item-restscore bootstrap iterations sequentially
#'
#' @param iterations Number of iterations.
#' @param boot_seeds Integer vector of per-iteration seeds.
#' @param boot_data_list List of data passed to each worker.
#' @param verbose Show progress bar.
#' @return List of raw results (one element per iteration).
#' @keywords internal
run_boot_restscore_sequential <- function(
  iterations,
  boot_seeds,
  boot_data_list,
  verbose = FALSE
) {
  if (verbose) {
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  results <- vector("list", iterations)
  for (i in seq_len(iterations)) {
    results[[i]] <- run_single_boot_restscore(boot_seeds[i], boot_data_list)
    if (verbose) utils::setTxtProgressBar(pb, i)
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# Tests for RMitemRestscoreBoot()

make_dichotomous <- function(n = 200, k = 8, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("Item", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMitemRestscoreBoot errors when data has non-zero minimum", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous() + 1L
  expect_error(RMitemRestscoreBoot(df), regexp = "scored starting at 0")
})

# ---------------------------------------------------------------------
# Output structures
# ---------------------------------------------------------------------
test_that("RMitemRestscoreBoot output = 'dataframe' returns one row per item × classification", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  res <- RMitemRestscoreBoot(df, iterations = 10L, samplesize = 100L,
                         parallel = FALSE, seed = 1L,
                         output = "dataframe")
  expect_s3_class(res, "data.frame")
  # 8 items × 3 classifications (overfit / underfit / no misfit) = 24 rows
  expect_equal(nrow(res), ncol(df) * 3L)
  expect_true(all(c("Item", "item_restscore", "n", "percent",
                    "Infit_MSQ", "Relative_location") %in% names(res)))
})

test_that("RMitemRestscoreBoot output = 'raw' returns per-iteration data", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  res <- RMitemRestscoreBoot(df, iterations = 10L, samplesize = 100L,
                         parallel = FALSE, seed = 1L,
                         output = "raw")
  expect_s3_class(res, "data.frame")
  expect_true(nrow(res) > 0L)
})

test_that("RMitemRestscoreBoot output = 'kable' returns a knitr_kable", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("knitr")
  df  <- make_dichotomous()
  out <- RMitemRestscoreBoot(df, iterations = 10L, samplesize = 100L,
                         parallel = FALSE, seed = 1L,
                         output = "kable")
  expect_s3_class(out, "knitr_kable")
})

test_that("RMitemRestscoreBoot is reproducible with the same seed", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  r1 <- RMitemRestscoreBoot(df, iterations = 5L, samplesize = 100L,
                        parallel = FALSE, seed = 42L,
                        output = "dataframe")
  r2 <- RMitemRestscoreBoot(df, iterations = 5L, samplesize = 100L,
                        parallel = FALSE, seed = 42L,
                        output = "dataframe")
  expect_equal(r1, r2)
})

test_that("RMitemRestscoreBoot drops all-NA respondents instead of erroring", {
  skip_if_not_installed("iarm")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 60 * 6, replace = TRUE), nrow = 60))
  colnames(df) <- paste0("i", 1:6)
  df[3, ] <- NA

  expect_message(
    res <- RMitemRestscoreBoot(df, iterations = 5, samplesize = 40,
                               parallel = FALSE, seed = 1,
                               output = "dataframe"),
    "no responses dropped"
  )
  expect_s3_class(res, "data.frame")
})

#' Plot Distribution of Simulated Infit and Outfit MSQ Values
#'
#' Visualises the distribution of simulation-based conditional item fit values
#' from \code{\link{RMitemInfitCutoff}}, optionally overlaying observed item fit
#' from the original data.
#'
#' Uses `ggdist::stat_dotsinterval()` (when `data` is not supplied) or
#' `ggdist::stat_dots()` (when `data` is supplied) with
#' `point_interval = "median_hdci"` and `.width = c(0.66, 0.999)`.
#'
#' @param simfit The return value of \code{\link{RMitemInfitCutoff}} (a list with
#'   components `results`, `item_cutoffs`, `actual_iterations`, `sample_n`, and
#'   `item_names`).
#' @param data Optional. A data.frame or matrix of item responses for computing
#'   and overlaying observed conditional item fit values. Items must be scored
#'   starting at 0 (non-negative integers). When provided, the plot includes
#'   orange diamond markers for the observed infit/outfit MSQ alongside the
#'   simulated distribution, plus segment summaries from the cutoff intervals.
#' @param statistic Character string. Either `"infit"` (default) to show only the
#'   infit panel, `"outfit"` to show only the outfit panel, or `"both"` to show
#'   infit and outfit side by side (requires the `patchwork` package when
#'   `data` is supplied).
#'
#' @return A `ggplot` object (or a `patchwork` object when `statistic = "both"`
#'   and `data` is supplied).
#'
#' @details
#' When `data` is **not** supplied, the function plots the simulated MSQ
#' distributions as dot-interval plots using `ggdist::stat_dotsinterval()` with
#' median and Highest Density Continuous Interval (HDCI) summaries, faceted by
#' statistic (InfitMSQ / OutfitMSQ).
#'
#' When `data` **is** supplied, the function:
#' \enumerate{
#'   \item Fits a Rasch / Partial Credit model by CML via
#'     `psychotools::pcmodel()` (a dichotomous item is a 2-category PCM)
#'     and computes observed conditional infit and outfit MSQ via
#'     `iarm::out_infit()`.
#'   \item Overlays observed fit values as orange diamond markers on the
#'     simulated distributions.
#'   \item Shows per-item cutoff intervals (from `simfit$item_cutoffs`) as
#'     black line segments, with thicker segments for the 66% HDCI range and
#'     black dots for the median.
#' }
#'
#' The `ggplot2`, `ggdist`, and optionally `patchwork` packages must be
#' installed (they are in Suggests, not Imports).
#'
#' @seealso \code{\link{RMitemInfitCutoff}}, \code{\link{RMitemInfit}}
#'
#' @importFrom rlang .data
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE) &&
#'     requireNamespace("ggdist", quietly = TRUE) &&
#'     requireNamespace("ggplot2", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:10)
#'
#'   # Run simulation
#'   cutoff_res <- RMitemInfitCutoff(sim_data, iterations = 100,
#'                                   parallel = FALSE, seed = 42)
#'
#'   # Simulated distribution only (infit + outfit faceted)
#'   RMitemInfitPlot(cutoff_res)
#'
#'   # With observed fit overlaid (infit only, the default)
#'   RMitemInfitPlot(cutoff_res, data = sim_data)
#'
#'   # Both infit and outfit panels side by side
#'   if (requireNamespace("patchwork", quietly = TRUE)) {
#'     RMitemInfitPlot(cutoff_res, data = sim_data, statistic = "both")
#'   }
#' }
#' }
RMitemInfitPlot <- function(simfit, data, statistic = "infit") {
  # --- Check required packages ------------------------------------------------
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for RMitemInfitPlot() but is not installed.\n",
      "Install it with: install.packages(\"ggplot2\")",
      call. = FALSE
    )
  }
  if (!requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required for RMitemInfitPlot() but is not installed.\n",
      "Install it with: install.packages(\"ggdist\")",
      call. = FALSE
    )
  }

  statistic <- match.arg(statistic, c("infit", "outfit", "both"))

  # --- Validate simfit --------------------------------------------------------
  required_names <- c(
    "results",
    "item_cutoffs",
    "actual_iterations",
    "sample_n",
    "item_names"
  )
  missing_names <- setdiff(required_names, names(simfit))
  if (length(missing_names) > 0L) {
    stop(
      "`simfit` is missing required components: ",
      paste(missing_names, collapse = ", "),
      ".\nExpected the return value of RMitemInfitCutoff().",
      call. = FALSE
    )
  }

  results_df <- simfit$results
  item_cutoffs <- simfit$item_cutoffs
  actual_iterations <- simfit$actual_iterations
  sample_n <- simfit$sample_n
  item_names <- simfit$item_names

  # Standard sample-size clause for the (complete-case) simulation sample.
  # `sample_n_total` / `sample_has_na` are absent in cutoff objects made by
  # older versions, so fall back to the plain count.
  sample_clause <- .n_caption(
    sample_n,
    if (is.null(simfit$sample_n_total)) sample_n else simfit$sample_n_total,
    if (isTRUE(simfit$sample_has_na)) "complete cases" else character()
  )

  # Item factor levels (reversed for plotting top-to-bottom)
  item_levels <- rev(item_names)

  # --- Compute per-item summary intervals for segment overlays ----------------
  lo_hi <- do.call(
    rbind,
    lapply(item_names, function(item) {
      sub <- results_df[results_df$Item == item, ]
      data.frame(
        Item = item,
        min_infit_msq = stats::quantile(sub$InfitMSQ, 0.001, na.rm = TRUE),
        max_infit_msq = stats::quantile(sub$InfitMSQ, 0.999, na.rm = TRUE),
        p66lo_infit_msq = stats::quantile(sub$InfitMSQ, 0.167, na.rm = TRUE),
        p66hi_infit_msq = stats::quantile(sub$InfitMSQ, 0.833, na.rm = TRUE),
        median_infit = stats::median(sub$InfitMSQ, na.rm = TRUE),
        min_outfit_msq = stats::quantile(sub$OutfitMSQ, 0.001, na.rm = TRUE),
        max_outfit_msq = stats::quantile(sub$OutfitMSQ, 0.999, na.rm = TRUE),
        p66lo_outfit_msq = stats::quantile(sub$OutfitMSQ, 0.167, na.rm = TRUE),
        p66hi_outfit_msq = stats::quantile(sub$OutfitMSQ, 0.833, na.rm = TRUE),
        median_outfit = stats::median(sub$OutfitMSQ, na.rm = TRUE),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    })
  )
  rownames(lo_hi) <- NULL

  # --- Case 1: no observed data, show simulation distribution only ------------
  if (missing(data)) {
    # Pivot results to long format
    infit_long <- data.frame(
      Item = results_df$Item,
      statistic = "InfitMSQ",
      Value = results_df$InfitMSQ,
      stringsAsFactors = FALSE
    )
    outfit_long <- data.frame(
      Item = results_df$Item,
      statistic = "OutfitMSQ",
      Value = results_df$OutfitMSQ,
      stringsAsFactors = FALSE
    )
    results_long <- rbind(infit_long, outfit_long)
    results_long$Item <- factor(results_long$Item, levels = item_levels)

    p <- ggplot2::ggplot(
      results_long,
      ggplot2::aes(
        x = .data$Value,
        y = .data$Item
      )
    ) +
      ggdist::stat_dotsinterval(
        ggplot2::aes(slab_fill = ggplot2::after_stat(.data$level)),
        quantiles = actual_iterations,
        point_interval = "median_hdci",
        layout = "weave",
        slab_color = NA,
        .width = c(0.66, 0.999)
      ) +
      ggplot2::labs(
        x = "Conditional MSQ",
        y = "Item",
        caption = er2_caption(paste0(
          "Results from ",
          actual_iterations,
          " simulated datasets. ",
          sample_clause,
          " per dataset."
        ))
      ) +
      ggplot2::scale_color_manual(
        values = scales::brewer_pal()(3)[-1],
        aesthetics = "slab_fill",
        guide = "none"
      ) +
      ggplot2::facet_wrap(~statistic, ncol = 2) +
      ggplot2::scale_x_continuous(
        breaks = seq(0.5, 1.5, 0.1),
        minor_breaks = NULL
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(panel.spacing = ggplot2::unit(0.7, "cm")) +
      er2_axis_margins() +
      er2_plot_caption()

    return(p)
  }

  # --- Case 2: observed data supplied -----------------------------------------
  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required to compute observed item fit but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }
  validate_response_data(data)

  # Respondents with no responses at all break the CML fit below
  # (psychotools errors on all-NA rows); drop them, as in RMitemInfit().
  data <- .drop_empty_respondents(data)

  # rgl workaround
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # CML fit via psychotools (a dichotomous item is a 2-category PCM);
  # iarm::out_infit() accepts pcmodel objects and matches the former
  # eRm::RM()/PCM() route to ~1e-6 (same pairing as RMitemInfitCutoff()).
  model_fit <- psychotools::pcmodel(data)
  cfit <- iarm::out_infit(model_fit)

  observed_df <- data.frame(
    Item = names(data),
    observed_infit = cfit$Infit,
    observed_outfit = cfit$Outfit,
    stringsAsFactors = FALSE
  )

  # --- Build infit data -------------------------------------------------------
  infit_sim <- data.frame(
    Item = results_df$Item,
    Value = results_df$InfitMSQ,
    stringsAsFactors = FALSE
  )
  infit_sim <- merge(
    infit_sim,
    observed_df[, c("Item", "observed_infit")],
    by = "Item",
    sort = FALSE
  )
  infit_sim$Item <- factor(infit_sim$Item, levels = item_levels)

  lo_hi$Item_f <- factor(lo_hi$Item, levels = item_levels)

  caption_text <- er2_caption(paste0(
    "Results from ",
    actual_iterations,
    " simulated datasets. ",
    sample_clause,
    " per dataset.\n",
    "Orange dots indicate observed conditional item fit. ",
    "Black dots indicate median fit from simulations."
  ))

  infit_p <- ggplot2::ggplot(
    infit_sim,
    ggplot2::aes(
      x = .data$Value,
      y = .data$Item
    )
  ) +
    ggdist::stat_dots(
      ggplot2::aes(slab_fill = ggplot2::after_stat(.data$level)),
      quantiles = actual_iterations,
      layout = "weave",
      slab_color = NA,
      .width = c(0.666, 0.999)
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$min_infit_msq,
        xend = .data$max_infit_msq,
        y = .data$Item_f,
        yend = .data$Item_f
      ),
      color = "black",
      linewidth = 0.7
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$p66lo_infit_msq,
        xend = .data$p66hi_infit_msq,
        y = .data$Item_f,
        yend = .data$Item_f
      ),
      color = "black",
      linewidth = 1.2
    ) +
    ggplot2::geom_point(
      data = lo_hi,
      ggplot2::aes(
        x = .data$median_infit,
        y = .data$Item_f
      ),
      size = 3.6
    ) +
    ggplot2::geom_point(
      ggplot2::aes(x = .data$observed_infit),
      color = "sienna2",
      shape = 18,
      position = ggplot2::position_nudge(y = -0.1),
      size = 4
    ) +
    ggplot2::labs(
      x = "Conditional Infit MSQ",
      y = "Item"
    ) +
    ggplot2::scale_color_manual(
      values = scales::brewer_pal()(3)[-1],
      aesthetics = "slab_fill",
      guide = "none"
    ) +
    ggplot2::scale_x_continuous(
      breaks = seq(0.5, 1.5, 0.1),
      minor_breaks = NULL
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(panel.spacing = ggplot2::unit(0.7, "cm")) +
    er2_axis_margins() +
    er2_plot_caption()

  # --- Build outfit data ------------------------------------------------------
  outfit_sim <- data.frame(
    Item = results_df$Item,
    Value = results_df$OutfitMSQ,
    stringsAsFactors = FALSE
  )
  outfit_sim <- merge(
    outfit_sim,
    observed_df[, c("Item", "observed_outfit")],
    by = "Item",
    sort = FALSE
  )
  outfit_sim$Item <- factor(outfit_sim$Item, levels = item_levels)

  outfit_p <- ggplot2::ggplot(
    outfit_sim,
    ggplot2::aes(
      x = .data$Value,
      y = .data$Item
    )
  ) +
    ggdist::stat_dots(
      ggplot2::aes(slab_fill = ggplot2::after_stat(.data$level)),
      quantiles = actual_iterations,
      layout = "weave",
      slab_color = NA,
      .width = c(0.666, 0.999)
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$min_outfit_msq,
        xend = .data$max_outfit_msq,
        y = .data$Item_f,
        yend = .data$Item_f
      ),
      color = "black",
      linewidth = 0.7
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$p66lo_outfit_msq,
        xend = .data$p66hi_outfit_msq,
        y = .data$Item_f,
        yend = .data$Item_f
      ),
      color = "black",
      linewidth = 1.2
    ) +
    ggplot2::geom_point(
      data = lo_hi,
      ggplot2::aes(
        x = .data$median_outfit,
        y = .data$Item_f
      ),
      size = 3.6
    ) +
    ggplot2::geom_point(
      ggplot2::aes(x = .data$observed_outfit),
      color = "sienna2",
      shape = 18,
      position = ggplot2::position_nudge(y = -0.1),
      size = 4
    ) +
    ggplot2::labs(
      x = "Conditional Outfit MSQ",
      y = "Item",
      caption = caption_text
    ) +
    ggplot2::scale_color_manual(
      values = scales::brewer_pal()(3)[-1],
      aesthetics = "slab_fill",
      guide = "none"
    ) +
    ggplot2::scale_x_continuous(
      breaks = seq(0.5, 1.5, 0.1),
      minor_breaks = NULL
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(panel.spacing = ggplot2::unit(0.7, "cm")) +
    er2_axis_margins() +
    er2_plot_caption()

  # --- Return the requested panel(s) ------------------------------------------
  if (statistic == "both") {
    if (!requireNamespace("patchwork", quietly = TRUE)) {
      stop(
        "Package 'patchwork' is required for statistic = \"both\" but is not installed.\n",
        "Install it with: install.packages(\"patchwork\")",
        call. = FALSE
      )
    }
    return(infit_p + outfit_p)
  } else if (statistic == "outfit") {
    return(outfit_p)
  } else {
    # default: infit only, add caption to infit panel
    infit_p <- infit_p +
      ggplot2::labs(caption = caption_text)
    return(infit_p)
  }
}

#' @rdname RMitemInfitPlot
#' @param ... Arguments passed on to \code{RMitemInfitPlot()}.
#' @details
#' `RMitemInfitCutoffPlot()` is a deprecated alias for `RMitemInfitPlot()`,
#' retained for backward compatibility with code written against easyRasch2
#' 0.8.0. It warns and forwards to `RMitemInfitPlot()`.
#' @export
RMitemInfitCutoffPlot <- function(...) {
  .Deprecated("RMitemInfitPlot")
  RMitemInfitPlot(...)
}

# Tests for RMitemInfitPlot() (and its deprecated alias RMitemInfitCutoffPlot())

make_dichotomous <- function(n = 200, k = 8, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Output structures: simulated-only distribution
# ---------------------------------------------------------------------
test_that("RMitemInfitPlot returns a ggplot from a simfit result (no data overlay)", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  df     <- make_dichotomous()
  simfit <- RMitemInfitCutoff(df, iterations = 10L, parallel = FALSE, seed = 1L)
  p <- RMitemInfitPlot(simfit)
  expect_s3_class(p, "ggplot")
})

# ---------------------------------------------------------------------
# Output structures: with observed-data overlay
# ---------------------------------------------------------------------
test_that("RMitemInfitPlot with observed data overlay returns a ggplot", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  df     <- make_dichotomous()
  simfit <- RMitemInfitCutoff(df, iterations = 10L, parallel = FALSE, seed = 1L)
  p <- RMitemInfitPlot(simfit, df)  # default statistic = "infit"
  expect_s3_class(p, "ggplot")
})

test_that("RMitemInfitPlot statistic = 'outfit' returns a ggplot", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  df     <- make_dichotomous()
  simfit <- RMitemInfitCutoff(df, iterations = 10L, parallel = FALSE, seed = 1L)
  p <- RMitemInfitPlot(simfit, df, statistic = "outfit")
  expect_s3_class(p, "ggplot")
})

test_that("RMitemInfitPlot statistic = 'both' returns a (patchwork) ggplot", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("patchwork")
  df     <- make_dichotomous()
  simfit <- RMitemInfitCutoff(df, iterations = 10L, parallel = FALSE, seed = 1L)
  p <- RMitemInfitPlot(simfit, df, statistic = "both")
  # patchwork compositions also inherit ggplot
  expect_s3_class(p, "ggplot")
})

# ---------------------------------------------------------------------
# Deprecated alias
# ---------------------------------------------------------------------
test_that("RMitemInfitCutoffPlot is a deprecated alias forwarding to RMitemInfitPlot", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  df     <- make_dichotomous()
  simfit <- RMitemInfitCutoff(df, iterations = 10L, parallel = FALSE, seed = 1L)
  expect_snapshot(p <- RMitemInfitCutoffPlot(simfit))
  expect_s3_class(p, "ggplot")
})

test_that("RMitemInfitPlot observed overlay drops all-NA respondents", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("ggdist")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 60 * 6, replace = TRUE), nrow = 60))
  colnames(df) <- paste0("i", 1:6)
  simfit <- suppressWarnings(
    RMitemInfitCutoff(df, iterations = 5, parallel = FALSE, seed = 1)
  )
  df[3, ] <- NA

  expect_message(p <- RMitemInfitPlot(simfit, data = df),
                 "no responses dropped")
  expect_s3_class(p, "ggplot")
})

#' Tile Plot of Item Response Distributions
#'
#' Creates a tile (heat map) plot showing the distribution of responses
#' across all items and response categories. Each cell displays the count
#' (or percentage) of responses, with optional conditional highlighting
#' for cells with low counts. Optional faceting by a grouping variable
#' is provided for inspecting subgroup response distributions before DIF
#' analyses -- particularly useful for spotting empty categories or
#' under-represented subgroups before fitting Rasch models per group.
#'
#' Adapted from `easyRaschBayes::plot_tile()` and extended with the
#' `group` parameter for faceted display.
#'
#' @param data A data.frame in wide format containing only the item
#'   response columns. Each column is one item, each row is one person.
#'   All columns must be numeric (integer-valued). Response categories
#'   may be coded starting from 0 or 1. Do not include person IDs,
#'   grouping variables, or other non-item columns -- supply the grouping
#'   variable separately via `group`.
#' @param group Optional vector of length `nrow(data)` (factor, character,
#'   or numeric) defining a grouping variable. When provided, the plot is
#'   faceted by group and counts / percentages are computed within each
#'   group. Default `NULL` (no faceting). Persons with `NA` group are
#'   excluded, with a `message()` reporting how many rows were dropped.
#' @param cutoff Integer. Cells with counts below this value are
#'   highlighted (when `highlight = TRUE`). Default `10`.
#' @param highlight Logical. If `TRUE` (default), cell labels with counts
#'   below `cutoff` are displayed in red. This includes empty cells
#'   (`n = 0`), useful for identifying gaps in the response distribution.
#' @param percent Logical. If `TRUE`, cell labels show percentages
#'   instead of raw counts. Percentages are computed within item (and
#'   within group, when `group` is supplied). Default `FALSE`.
#' @param text_color Character. Colour for non-highlighted cell labels.
#'   Default `"orange"`.
#' @param text_size Numeric. Size of the cell labels (passed to
#'   `ggplot2::geom_text()`). Default `4`, matching [RMitemCatProb()].
#' @param zero_fill Colour for cells with zero responses, or `NULL` to
#'   keep them on the fill scale. Default `"white"`: empty cells are
#'   shown unfilled with a light grey outline, so the *absence* of data
#'   reads as absence rather than as the darkest end of the colour
#'   scale (where `n = 0` is nearly indistinguishable from `n = 1`).
#' @param item_labels Optional character vector of descriptive labels
#'   for the items (y-axis), same length as `ncol(data)`. Default `NULL`
#'   uses the column names.
#' @param category_labels Optional character vector of labels for the
#'   response categories (x-axis), same length as the number of
#'   categories spanning `min` to `max` observed value. Default `NULL`
#'   uses the numeric category values.
#' @param group_labels Optional character vector of length
#'   `nlevels(as.factor(group))` to override the displayed facet labels.
#'   Order corresponds to `levels(as.factor(group))`.
#' @param facet_ncol Integer or `NULL`. Number of columns in the facet
#'   grid when `group` is supplied. Default `NULL` (ggplot2 chooses).
#' @param output Character. `"ggplot"` (default) returns the plot;
#'   `"dataframe"` returns the underlying per-cell counts (one row per
#'   item x category, plus group when supplied).
#'
#' @return Either a `ggplot` object or a data.frame, depending on
#'   `output`.
#'
#' @details
#' Items are placed on the y-axis (in the same order as the columns of
#' `data`, top to bottom) and response categories on the x-axis. Cell
#' shading represents the count of responses (darker = more responses).
#' Categories with zero responses are explicitly shown (`n = 0`), which
#' helps identify gaps in the response distribution -- one of the primary
#' purposes of the plot, especially before DIF analyses where
#' under-represented categories within a subgroup can break model
#' fitting on that subgroup.
#'
#' When `group` is supplied, percentages and the highlight cutoff are
#' applied within each group, so a cell labelled "5" in the group-A
#' facet contains the count for group A only.
#'
#' The plot caption reports the sample in the standard
#' `n = X of Y respondents (policy)` form: rows with `NA` group are
#' dropped (and counted in `Y` only), while item-level `NA`s are retained
#' -- each cell simply counts the non-missing responses for that item.
#'
#' @examples
#' \donttest{
#' if (requireNamespace("eRm", quietly = TRUE)) {
#'   data("pcmdat2", package = "eRm")
#'
#'   # Basic tile plot
#'   RMplotTile(pcmdat2)
#'
#'   # With percentages
#'   RMplotTile(pcmdat2, percent = TRUE)
#'
#'   # Faceted by an external grouping variable
#'   set.seed(1)
#'   grp <- sample(c("A", "B"), nrow(pcmdat2), replace = TRUE)
#'   RMplotTile(pcmdat2, group = grp)
#'
#'   # With custom labels and tighter cutoff
#'   RMplotTile(pcmdat2,
#'              group = grp,
#'              group_labels = c("Female", "Male"),
#'              cutoff = 5,
#'              facet_ncol = 2)
#'
#'   # Underlying counts as a data.frame
#'   RMplotTile(pcmdat2, group = grp, output = "dataframe")
#' }
#' }
#'
#' @importFrom rlang .data
#' @export
RMplotTile <- function(
  data,
  group = NULL,
  cutoff = 10,
  highlight = TRUE,
  percent = FALSE,
  text_color = "orange",
  text_size = 4,
  zero_fill = "white",
  item_labels = NULL,
  category_labels = NULL,
  group_labels = NULL,
  facet_ncol = NULL,
  output = c("ggplot", "dataframe")
) {
  output <- match.arg(output)

  if (output == "ggplot" && !requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for output = \"ggplot\".",
      call. = FALSE
    )
  }

  # ---------------------------------------------------------------------
  # Input validation
  # ---------------------------------------------------------------------
  if (!is.data.frame(data)) {
    stop(
      "`data` must be a data.frame in wide format ",
      "(one column per item, one row per person).",
      call. = FALSE
    )
  }
  if (ncol(data) < 2L) {
    stop(
      "`data` must contain at least 2 columns (items). Found ",
      ncol(data),
      " column(s).",
      call. = FALSE
    )
  }
  if (nrow(data) < 1L) {
    stop(
      "`data` must contain at least 1 row (person). Found 0 rows.",
      call. = FALSE
    )
  }

  non_numeric <- names(data)[!vapply(data, is.numeric, logical(1L))]
  if (length(non_numeric) > 0L) {
    stop(
      "All columns must be numeric. Non-numeric column(s): ",
      paste(non_numeric, collapse = ", "),
      ". Remove non-item columns (e.g., person IDs, grouping ",
      "variables) before passing to RMplotTile() -- pass groups via ",
      "the `group` argument instead.",
      call. = FALSE
    )
  }

  all_vals <- unlist(data, use.names = FALSE)
  all_vals <- all_vals[!is.na(all_vals)]
  if (length(all_vals) == 0L) {
    stop("`data` contains only NA values.", call. = FALSE)
  }
  if (!all(all_vals == round(all_vals))) {
    stop(
      "All response values must be integers. Found non-integer values.",
      call. = FALSE
    )
  }

  if (!is.null(item_labels) && length(item_labels) != ncol(data)) {
    stop(
      "`item_labels` must have the same length as the number of ",
      "columns in `data`. Expected ",
      ncol(data),
      ", got ",
      length(item_labels),
      ".",
      call. = FALSE
    )
  }

  min_val <- min(all_vals)
  max_val <- max(all_vals)
  all_categories <- seq(min_val, max_val)
  n_categories <- length(all_categories)

  if (
    !is.null(category_labels) &&
      length(category_labels) != n_categories
  ) {
    stop(
      "`category_labels` must have the same length as the number of ",
      "response categories (",
      min_val,
      " to ",
      max_val,
      " = ",
      n_categories,
      " categories). Got ",
      length(category_labels),
      ".",
      call. = FALSE
    )
  }

  # Group validation
  group_factor <- NULL
  n_group_na <- 0L
  if (!is.null(group)) {
    if (length(group) != nrow(data)) {
      stop(
        "`group` must have the same length as nrow(data) (",
        nrow(data),
        "). Got ",
        length(group),
        ".",
        call. = FALSE
      )
    }
    group_factor <- as.factor(group)
    if (nlevels(droplevels(group_factor)) < 2L) {
      stop(
        "`group` must have at least 2 distinct non-missing values.",
        call. = FALSE
      )
    }
    if (!is.null(group_labels)) {
      if (length(group_labels) != nlevels(group_factor)) {
        stop(
          "`group_labels` must have the same length as the number of ",
          "group levels (",
          nlevels(group_factor),
          "). Got ",
          length(group_labels),
          ".",
          call. = FALSE
        )
      }
      levels(group_factor) <- as.character(group_labels)
    }
    # Rows with NA group cannot be shown in any facet; surface the drop
    # (matching the RMdifLR / RMdifTree NA-drop messaging).
    n_group_na <- sum(is.na(group_factor))
    if (n_group_na > 0L) {
      message(n_group_na, " row(s) with NA in `group` dropped.")
    }
  }

  # ---------------------------------------------------------------------
  # Prepare data
  # ---------------------------------------------------------------------
  item_names <- names(data)
  label_map <- if (!is.null(item_labels)) {
    stats::setNames(item_labels, item_names)
  } else {
    stats::setNames(item_names, item_names)
  }

  # Per-item x category counts; optionally per group too
  build_counts <- function(sub_data, group_label = NULL) {
    rows <- lapply(item_names, function(item) {
      vals <- sub_data[[item]]
      vals <- vals[!is.na(vals)]
      tab <- table(factor(vals, levels = all_categories))
      data.frame(
        item = item,
        category = as.integer(names(tab)),
        n = as.integer(tab),
        stringsAsFactors = FALSE
      )
    })
    out <- do.call(rbind, rows)
    if (!is.null(group_label)) {
      out$group <- group_label
    }
    rownames(out) <- NULL
    out
  }

  if (is.null(group_factor)) {
    count_df <- build_counts(data)
    agg_keys <- "item"
  } else {
    parts <- lapply(levels(group_factor), function(g) {
      mask <- !is.na(group_factor) & group_factor == g
      build_counts(data[mask, , drop = FALSE], group_label = g)
    })
    count_df <- do.call(rbind, parts)
    rownames(count_df) <- NULL
    # Preserve factor ordering for downstream facet order
    count_df$group <- factor(count_df$group, levels = levels(group_factor))
    agg_keys <- c("item", "group")
  }

  # Per-(item [x group]) totals -> percentages
  totals <- stats::aggregate(
    count_df$n,
    by = count_df[, agg_keys, drop = FALSE],
    FUN = sum
  )
  colnames(totals)[ncol(totals)] <- "total"
  count_df <- merge(count_df, totals, by = agg_keys, sort = FALSE)
  count_df$percentage <- round(count_df$n / count_df$total * 100, 1)

  # Item labels with column-order ordering (top item = first column)
  count_df$item_label <- label_map[count_df$item]
  count_df$item_label <- factor(
    count_df$item_label,
    levels = rev(label_map[item_names])
  )

  # ---------------------------------------------------------------------
  # Output: dataframe
  # ---------------------------------------------------------------------
  if (output == "dataframe") {
    keep <- c(
      if (!is.null(group_factor)) "group" else NULL,
      "item",
      "item_label",
      "category",
      "n",
      "percentage"
    )
    out_df <- count_df[, keep, drop = FALSE]
    rownames(out_df) <- NULL
    return(out_df)
  }

  # ---------------------------------------------------------------------
  # Output: ggplot
  # ---------------------------------------------------------------------
  # Standard sample-size caption: rows with NA group are dropped (also
  # surfaced via message above); item-level NAs are retained -- each cell
  # counts non-missing responses only.
  n_total <- nrow(data)
  n_used <- n_total - n_group_na
  has_zero_cells <- any(count_df$n == 0L)
  use_zero_fill <- !is.null(zero_fill) && has_zero_cells
  caption_text <- er2_caption(paste0(
    .n_caption(
      n_used,
      n_total,
      c(
        if (anyNA(data)) "incomplete responses retained",
        if (n_group_na > 0L) "complete group data"
      )
    ),
    ".",
    if (use_zero_fill) " Outlined cells without fill: no responses." else ""
  ))

  # Cells with zero responses are taken off the fill scale (NA fill ->
  # `zero_fill`) and outlined, so missing data reads as *absence* rather
  # than as the darkest end of the colour scale.
  if (!is.null(zero_fill)) {
    count_df$fill_n <- ifelse(count_df$n == 0L, NA_integer_, count_df$n)
  } else {
    count_df$fill_n <- count_df$n
  }

  p <- ggplot2::ggplot(
    count_df,
    ggplot2::aes(x = .data$category, y = .data$item_label,
                 fill = .data$fill_n)
  ) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_viridis_c(
      expression(italic(n)),
      limits = c(0, NA),
      na.value = if (is.null(zero_fill)) "grey50" else zero_fill
    ) +
    ggplot2::labs(y = "Items", caption = caption_text) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(size = 8),
      panel.grid = ggplot2::element_blank(),
      panel.spacing = ggplot2::unit(0.7, "cm")
    ) +
    er2_axis_margins() +
    er2_plot_caption()

  if (use_zero_fill) {
    p <- p +
      ggplot2::geom_tile(
        data = count_df[count_df$n == 0L, , drop = FALSE],
        fill = zero_fill,
        color = "grey70",
        linewidth = 0.4,
        # Explicit cell size: ggplot otherwise derives the tile width from
        # this layer's own x-resolution, which overshoots when the empty
        # cells are non-adjacent categories.
        width = 1,
        height = 1
      )
  }

  # X-axis: numeric breaks or category labels
  if (!is.null(category_labels)) {
    p <- p +
      ggplot2::scale_x_continuous(
        "Response category",
        expand = c(0, 0),
        breaks = all_categories,
        labels = category_labels
      )
  } else {
    p <- p +
      ggplot2::scale_x_continuous(
        "Response category",
        expand = c(0, 0),
        breaks = all_categories
      )
  }

  # Cell labels (count or percent), with optional <-cutoff highlighting
  if (percent) {
    if (highlight) {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(
            label = paste0(.data$percentage, "%"),
            color = ifelse(.data$n < cutoff, "red", text_color)
          ),
          size = text_size
        ) +
        ggplot2::guides(color = "none") +
        ggplot2::scale_color_identity()
    } else {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(label = paste0(.data$percentage, "%")),
          color = text_color,
          size = text_size
        )
    }
  } else {
    if (highlight) {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(
            label = .data$n,
            color = ifelse(.data$n < cutoff, "red", text_color)
          ),
          size = text_size
        ) +
        ggplot2::guides(color = "none") +
        ggplot2::scale_color_identity()
    } else {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(label = .data$n),
          color = text_color,
          size = text_size
        )
    }
  }

  # Faceting
  if (!is.null(group_factor)) {
    p <- p + ggplot2::facet_wrap(~group, ncol = facet_ncol)
  }

  p
}

# Tests for RMplotTile()

make_dichotomous <- function(n = 80, k = 4, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

make_polytomous <- function(n = 80, k = 4, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:3, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMplotTile errors when data is not a data.frame", {
  m <- matrix(0:1, nrow = 4, ncol = 4)
  expect_error(RMplotTile(m), regexp = "data\\.frame")
})

test_that("RMplotTile errors when fewer than 2 columns", {
  df <- data.frame(I1 = sample(0:1, 20, replace = TRUE))
  expect_error(RMplotTile(df), regexp = "at least 2 columns")
})

test_that("RMplotTile errors when all values are NA", {
  df <- data.frame(I1 = NA_integer_, I2 = NA_integer_)
  expect_error(RMplotTile(df), regexp = "only NA")
})

test_that("RMplotTile errors when non-numeric column present", {
  df <- data.frame(I1 = sample(0:1, 20, replace = TRUE), I2 = letters[1:20])
  expect_error(RMplotTile(df), regexp = "numeric")
})

test_that("RMplotTile errors when group length mismatches nrow(data)", {
  df <- make_dichotomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df) - 1L))
  expect_error(RMplotTile(df, group = grp), regexp = "same length as nrow")
})

test_that("RMplotTile errors when item_labels length mismatches ncol(data)", {
  df <- make_dichotomous()
  expect_error(
    RMplotTile(df, item_labels = c("a", "b")),
    regexp = "same length as the number of"
  )
})

# ---------------------------------------------------------------------
# Output: dataframe
# ---------------------------------------------------------------------
test_that("RMplotTile output = 'dataframe' returns one row per item x category", {
  df <- make_polytomous() # values 0..3 -> 4 categories
  out <- RMplotTile(df, output = "dataframe")
  expect_s3_class(out, "data.frame")
  expect_equal(nrow(out), ncol(df) * 4L)
  expect_true(all(c("item", "category", "n", "percentage") %in% names(out)))
  expect_true(all(out$n >= 0))
  expect_equal(sum(out$n), nrow(df) * ncol(df))
})

test_that("RMplotTile dataframe has group column when group is supplied", {
  df <- make_polytomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))
  out <- RMplotTile(df, group = grp, output = "dataframe")
  expect_true("group" %in% names(out))
  # 4 items × 4 categories × 2 groups = 32 rows
  expect_equal(nrow(out), 4L * 4L * 2L)
})

# ---------------------------------------------------------------------
# Output: ggplot
# ---------------------------------------------------------------------
test_that("RMplotTile output = 'ggplot' returns a ggplot", {
  skip_if_not_installed("ggplot2")
  df <- make_dichotomous()
  p <- RMplotTile(df) # default output
  expect_s3_class(p, "ggplot")
})

test_that("RMplotTile with group + percent + custom cutoff returns ggplot", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  grp <- factor(rep(c("A", "B"), length.out = nrow(df)))
  p <- RMplotTile(df, group = grp, percent = TRUE, cutoff = 5)
  expect_s3_class(p, "ggplot")
})

# ---------------------------------------------------------------------
# Missingness reporting (message + caption)
# ---------------------------------------------------------------------
test_that("RMplotTile messages when group has NA and reports n in caption", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  grp <- rep(c("A", "B"), length.out = nrow(df))
  grp[1:5] <- NA

  expect_message(
    p <- RMplotTile(df, group = grp),
    regexp = "5 row\\(s\\) with NA in `group` dropped"
  )
  expect_match(p$labels$caption, "n = 75 of 80 respondents")
  expect_match(p$labels$caption, "complete group data")
})

test_that("RMplotTile is silent with complete group and plain caption", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  grp <- rep(c("A", "B"), length.out = nrow(df))

  expect_no_message(p <- RMplotTile(df, group = grp))
  expect_match(p$labels$caption, "n = 80 respondents")
  expect_no_match(p$labels$caption, "of 80")
})

test_that("RMplotTile caption notes retained incomplete responses", {
  skip_if_not_installed("ggplot2")
  df <- make_polytomous()
  df[1:3, 1] <- NA
  p <- RMplotTile(df)
  expect_match(p$labels$caption, "n = 80 respondents")
  expect_match(p$labels$caption, "incomplete responses retained")
})

test_that("RMplotTile group-NA message also fires for dataframe output", {
  df <- make_polytomous()
  grp <- rep(c("A", "B"), length.out = nrow(df))
  grp[1L] <- NA
  expect_message(
    RMplotTile(df, group = grp, output = "dataframe"),
    regexp = "1 row\\(s\\) with NA in `group` dropped"
  )
})

test_that("zero cells are taken off the fill scale and text_size applies", {
  skip_if_not_installed("ggplot2")
  set.seed(1)
  df <- as.data.frame(matrix(sample(0:2, 60 * 3, replace = TRUE), nrow = 60))
  colnames(df) <- paste0("i", 1:3)
  df$i1[df$i1 == 2] <- 1   # force an empty category on item 1

  p <- RMplotTile(df, text_size = 5)
  expect_s3_class(p, "ggplot")
  # zero cells have NA fill (mapped to zero_fill via na.value)
  expect_true(anyNA(p$data$fill_n))
  expect_true(all(is.na(p$data$fill_n[p$data$n == 0])))
  # caption mentions the convention
  expect_match(p$labels$caption, "no responses")

  # opting out restores the old fill mapping
  p0 <- RMplotTile(df, zero_fill = NULL)
  expect_false(anyNA(p0$data$fill_n))
})

#' Conditional Item Infit MSQ for Multiply Imputed Data
#'
#' Extends \code{\link{RMitemInfit}} to work with multiply imputed datasets
#' produced by the `mice` package. Computes conditional infit MSQ on each
#' imputed dataset and pools the results using Rubin's rules.
#'
#' @param mids_object A `mids` object (multiply imputed dataset) as returned by
#'   `mice::mice()`. Each completed dataset must contain only the item response
#'   columns to be analysed (i.e., no ID or grouping variables). Items must be
#'   scored starting at 0 (non-negative integers).
#' @param cutoff Optional. Default `NULL` (no cutoff applied). Can be:
#'   * The return value of \code{\link{RMitemInfitCutoff}} or
#'     \code{\link{RMitemInfitCutoffMI}} (a list with `$item_cutoffs`): the
#'     data.frame is extracted automatically and metadata is included in the
#'     kable caption.
#'   * The `$item_cutoffs` data.frame directly: must have columns `Item`,
#'     `infit_low`, and `infit_high`.
#'   When provided, adds columns `Infit_low`, `Infit_high`, and `Flagged` to
#'   the result. `Flagged` is a character column labelling the misfit
#'   direction: `"overfit"` (pooled infit below the range), `"underfit"`
#'   (above), or `""` (within range).
#' @param output Character string controlling the return value. Either
#'   `"kable"` (default) for a formatted `knitr::kable()` table, or
#'   `"dataframe"` for the underlying data.frame.
#' @param sort Optional character string. When `sort = "infit"`, rows are
#'   sorted by `Infit_MSQ` in descending order before output.
#'
#' @return
#' * If `output = "kable"`: a `knitr_kable` object (plain text table via
#'   `format = "pipe"`) with columns "Item", "Infit MSQ", "Infit SE",
#'   "Relative location", and a caption noting the number of imputations and
#'   complete cases. When `cutoff` is provided, columns "Infit low",
#'   "Infit high", and "Flagged" are also included.
#' * If `output = "dataframe"`: a data.frame with columns `Item`,
#'   `Infit_MSQ`, `Infit_SE`, and `Relative_location`. When `cutoff` is
#'   provided, columns `Infit_low`, `Infit_high`, and `Flagged` are also
#'   included (inserted after `Infit_SE`, before `Relative_location`).
#'   `Flagged` is a character column (`"overfit"` / `"underfit"` / `""`),
#'   not the previous logical.
#'
#' @details
#' For each of the `m` imputed datasets, the function:
#' \enumerate{
#'   \item Fits a Rasch model by CML via `psychotools::pcmodel()` (a
#'     dichotomous item is a 2-category partial credit model), consistent with
#'     \code{\link{RMitemInfit}} and the rest of the package.
#'   \item Computes conditional infit MSQ and its standard error via
#'     `iarm::out_infit()`.
#'   \item Computes item locations (mean of the grand-mean-centred CML
#'     Andrich thresholds) and the mean WLE person location.
#' }
#'
#' The per-imputation estimates are then pooled using Rubin's rules:
#' \describe{
#'   \item{Pooled MSQ}{The mean of the `m` infit MSQ point estimates.}
#'   \item{Within-imputation variance}{The mean of the `m` squared standard
#'     errors.}
#'   \item{Between-imputation variance}{The sample variance of the `m` point
#'     estimates.}
#'   \item{Total variance}{Within + (1 + 1/m) * Between.}
#'   \item{Pooled SE}{The square root of the total variance.}
#' }
#'
#' Relative item location is the mean of per-imputation relative locations
#' (item location minus sample mean person location).
#'
#' \strong{Caveat on the pooled SE.} The within-imputation variance is the
#' squared conditional infit SE from `iarm::out_infit()`. Müller (2020) showed
#' that this asymptotic SE is an unreliable measure of uncertainty for the
#' conditional infit statistic; Rubin's pooled SE inherits that limitation, so
#' the `Infit_SE`/`Infit SE` column should be read as an approximate indication
#' of imputation-related variability rather than a trustworthy inferential
#' standard error. For item misfit decisions, prefer the simulation-based
#' cutoffs from \code{\link{RMitemInfitCutoffMI}}.
#'
#' Imputed datasets that cause model convergence failures are dropped with a
#' warning. If all imputations fail, the function stops with an error. At
#' least two successful imputations are required to estimate between-imputation
#' variance.
#'
#' The `mice` and `iarm` packages must be installed (they are in Suggests, not
#' Imports).
#'
#' @references
#' Müller, M. (2020). Item fit statistics for Rasch analysis: Can we trust
#' them? *Journal of Statistical Distributions and Applications*, 7(5).
#' \doi{10.1186/s40488-020-00108-7}
#'
#' @seealso \code{\link{RMitemInfit}}, \code{\link{RMitemInfitCutoffMI}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("mice", quietly = TRUE) &&
#'     requireNamespace("iarm", quietly = TRUE) &&
#'     requireNamespace("ggdist", quietly = TRUE)) {
#'   # Create example data with ~10% MCAR missingness
#'   set.seed(42)
#'   mat <- matrix(sample(0:1, 200 * 8, replace = TRUE), nrow = 200, ncol = 8)
#'   mat[sample(length(mat), round(0.10 * length(mat)))] <- NA
#'   sim_data <- as.data.frame(mat)
#'   colnames(sim_data) <- paste0("Item", 1:8)
#'
#'   # mice's ordinal method (`polr`) requires the items to be ordered
#'   # factors, so code them as such before imputing. RMitemInfitMI()
#'   # converts the completed factors back to numeric internally.
#'   sim_data[] <- lapply(sim_data, function(x) factor(x, ordered = TRUE))
#'
#'   # Impute (use more imputations, e.g. m = 5+, in real analyses)
#'   imp <- mice::mice(sim_data, m = 2, method = "polr", seed = 123,
#'                     printFlag = FALSE)
#'
#'   # Pooled infit table (no cutoffs)
#'   RMitemInfitMI(imp)
#'
#'   # With simulation-based cutoffs
#'   # (use more iterations, e.g. 250+, in real analyses)
#'   cutoff_mi <- RMitemInfitCutoffMI(imp, iterations = 50, parallel = FALSE,
#'                                 seed = 42)
#'   RMitemInfitMI(imp, cutoff = cutoff_mi)
#'
#'   # As data.frame
#'   df <- RMitemInfitMI(imp, cutoff = cutoff_mi, output = "dataframe")
#' }
#' }
RMitemInfitMI <- function(mids_object, cutoff = NULL, output = "kable", sort) {
  # --- Check required packages ------------------------------------------------
  if (!requireNamespace("mice", quietly = TRUE)) {
    stop(
      "Package 'mice' is required for RMitemInfitMI() but is not installed.\n",
      "Install it with: install.packages(\"mice\")",
      call. = FALSE
    )
  }

  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required for RMitemInfitMI() but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }

  if (!inherits(mids_object, "mids")) {
    stop(
      "`mids_object` must be a 'mids' object as returned by mice::mice().",
      call. = FALSE
    )
  }

  output <- match.arg(output, c("kable", "dataframe"))

  # --- Validate and normalise cutoff ------------------------------------------
  cutoff_n_iter <- NULL
  cutoff_method <- NULL
  cutoff_hdci_width <- NULL
  cutoff_n_imp <- NULL
  if (!is.null(cutoff)) {
    if (
      is.list(cutoff) &&
        !is.data.frame(cutoff) &&
        "item_cutoffs" %in% names(cutoff)
    ) {
      cutoff_n_iter <- cutoff$actual_iterations
      cutoff_method <- cutoff$cutoff_method
      cutoff_hdci_width <- cutoff$hdci_width
      cutoff_n_imp <- cutoff$n_imputations
      cutoff <- cutoff$item_cutoffs
    }
    if (!is.data.frame(cutoff)) {
      stop(
        "`cutoff` must be NULL, the return value of RMitemInfitCutoff() / ",
        "RMitemInfitCutoffMI(), or its $item_cutoffs data.frame.",
        call. = FALSE
      )
    }
    required_cols <- c("Item", "infit_low", "infit_high")
    missing_cols <- setdiff(required_cols, names(cutoff))
    if (length(missing_cols) > 0L) {
      stop(
        "`cutoff` data.frame is missing required columns: ",
        paste(missing_cols, collapse = ", "),
        ".",
        call. = FALSE
      )
    }
  }

  # --- rgl workaround ---------------------------------------------------------
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  m <- mids_object$m

  # --- Compute infit on each imputed dataset ----------------------------------
  per_imp <- vector("list", m)
  n_failed <- 0L
  n_complete_first <- NULL

  for (i in seq_len(m)) {
    completed_data <- .mi_completed_to_numeric(
      mice::complete(mids_object, action = i)
    )

    result_i <- tryCatch(
      {
        validate_response_data(completed_data)

        data_mat <- as.matrix(completed_data)
        n_complete <- nrow(completed_data)

        # CML item parameters (psychotools; a dichotomous item is a 2-category
        # PCM) and WLE person locations, consistent with RMitemInfit() and the
        # rest of the package. The conditional infit/outfit statistic from iarm
        # is engine-invariant; only the relative-location reference shifts
        # slightly (WLE vs eRm MLE person mean).
        fit <- psychotools::pcmodel(completed_data)
        thr_list <- .center_thresholds(lapply(
          psychotools::threshpar(fit),
          as.numeric
        ))
        item_avg_locations <- vapply(thr_list, mean, numeric(1L))
        person_avg_location <- mean(
          .estimate_thetas(data_mat, thr_list, method = "WLE")$theta,
          na.rm = TRUE
        )

        relative_locations <- item_avg_locations - person_avg_location

        # Conditional infit
        cfit <- iarm::out_infit(fit)

        list(
          infit_msq = cfit$Infit,
          infit_se = cfit$Infit.se,
          relative_location = as.numeric(relative_locations),
          n_complete = n_complete
        )
      },
      error = function(e) {
        warning(
          sprintf(
            "Model fitting failed for imputation %d: %s",
            i,
            conditionMessage(e)
          ),
          call. = FALSE
        )
        NULL
      }
    )

    if (is.null(result_i)) {
      n_failed <- n_failed + 1L
      next
    }

    per_imp[[i]] <- result_i

    if (is.null(n_complete_first)) {
      n_complete_first <- result_i$n_complete
    }
  }

  if (n_failed == m) {
    stop(
      "Model fitting failed for all ",
      m,
      " imputed datasets. ",
      "Check your data.",
      call. = FALSE
    )
  }

  successful <- per_imp[!vapply(per_imp, is.null, logical(1L))]
  m_ok <- length(successful)

  if (m_ok < 2L) {
    stop(
      "Only ",
      m_ok,
      " imputation(s) succeeded. At least 2 are required ",
      "to estimate between-imputation variance.",
      call. = FALSE
    )
  }

  if (n_failed > 0L) {
    warning(
      sprintf(
        "%d of %d imputed datasets failed and were excluded.",
        n_failed,
        m
      ),
      call. = FALSE
    )
  }

  # --- Pool using Rubin's rules -----------------------------------------------
  item_names <- names(mice::complete(mids_object, action = 1L))
  n_items <- length(item_names)

  # Collect per-imputation vectors into matrices (items x imputations)
  msq_mat <- matrix(NA_real_, nrow = n_items, ncol = m_ok)
  se_mat <- matrix(NA_real_, nrow = n_items, ncol = m_ok)
  loc_mat <- matrix(NA_real_, nrow = n_items, ncol = m_ok)

  for (j in seq_len(m_ok)) {
    msq_mat[, j] <- successful[[j]]$infit_msq
    se_mat[, j] <- successful[[j]]$infit_se
    loc_mat[, j] <- successful[[j]]$relative_location
  }

  # Pooled point estimate: mean across imputations
  pooled_msq <- rowMeans(msq_mat)

  # Within-imputation variance: mean of squared SEs
  within_var <- rowMeans(se_mat^2)

  # Between-imputation variance: sample variance of point estimates
  between_var <- apply(msq_mat, 1, stats::var)

  # Total variance (Rubin's combining rule)
  total_var <- within_var + (1 + 1 / m_ok) * between_var

  # Pooled SE
  pooled_se <- sqrt(total_var)

  # Pooled relative location: simple mean

  pooled_location <- rowMeans(loc_mat)

  # --- Assemble result data.frame ---------------------------------------------
  # Unrounded values; the kable path rounds a display copy before rendering.
  item_fit_table <- data.frame(
    Item = item_names,
    Infit_MSQ = pooled_msq,
    Infit_SE = pooled_se,
    Relative_location = pooled_location,
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  # --- Apply cutoff if provided -----------------------------------------------
  if (!is.null(cutoff)) {
    data_items <- item_fit_table$Item
    cutoff_items <- cutoff$Item
    if (!setequal(data_items, cutoff_items)) {
      stop(
        "Item names in `cutoff` do not match item names in `data`.\n",
        "  data items  : ",
        paste(data_items, collapse = ", "),
        "\n",
        "  cutoff items: ",
        paste(cutoff_items, collapse = ", "),
        call. = FALSE
      )
    }
    cutoff_sub <- cutoff[, c("Item", "infit_low", "infit_high")]
    item_fit_table <- merge(
      item_fit_table,
      cutoff_sub,
      by = "Item",
      sort = FALSE
    )
    # Restore original row order
    item_fit_table <- item_fit_table[match(data_items, item_fit_table$Item), ]
    rownames(item_fit_table) <- NULL
    item_fit_table$Infit_low <- item_fit_table$infit_low
    item_fit_table$Infit_high <- item_fit_table$infit_high
    item_fit_table$infit_low <- NULL
    item_fit_table$infit_high <- NULL
    # Flagged labels the misfit direction: overfit (pooled infit below the
    # range, more predictable), underfit (above, noisier), "" within range.
    item_fit_table$Flagged <- ifelse(
      item_fit_table$Infit_MSQ < item_fit_table$Infit_low,
      "overfit",
      ifelse(
        item_fit_table$Infit_MSQ > item_fit_table$Infit_high,
        "underfit",
        ""
      )
    )
    # Reorder columns
    item_fit_table <- item_fit_table[, c(
      "Item",
      "Infit_MSQ",
      "Infit_SE",
      "Infit_low",
      "Infit_high",
      "Flagged",
      "Relative_location"
    )]
  }

  # --- Sort if requested ------------------------------------------------------
  if (!missing(sort) && identical(sort, "infit")) {
    item_fit_table <- item_fit_table[
      order(item_fit_table$Infit_MSQ, decreasing = TRUE),
    ]
    rownames(item_fit_table) <- NULL
  }

  # --- Return -----------------------------------------------------------------
  if (output == "dataframe") {
    return(item_fit_table)
  }

  # Kable display rounding (the dataframe output above stays unrounded)
  item_fit_table <- .round_display(item_fit_table, c(
    Infit_MSQ = 3, Infit_SE = 3, Infit_low = 3, Infit_high = 3,
    Relative_location = 2
  ))

  # Build caption
  base_caption <- paste0(
    "Pooled MSQ values from ",
    m_ok,
    " imputations (Rubin's rules). ",
    .n_caption(
      n_complete_first,
      n_complete_first,
      paste0("missing values imputed, m = ", m_ok)
    ),
    " per imputed dataset."
  )

  cutoff_caption <- NULL
  if (!is.null(cutoff)) {
    if (!is.null(cutoff_n_iter)) {
      method_label <- .format_cutoff_method_label(
        cutoff_method,
        cutoff_hdci_width
      )
      iter_part <- paste0(cutoff_n_iter, " total simulation iterations")
      imp_part <- if (!is.null(cutoff_n_imp)) {
        paste0(" across ", cutoff_n_imp, " imputations")
      } else {
        ""
      }
      cutoff_caption <- paste0(
        " Cutoff values based on ",
        if (!is.null(method_label)) {
          paste0(iter_part, imp_part, " (", method_label, ").")
        } else {
          paste0(iter_part, imp_part, ".")
        }
      )
    } else {
      cutoff_caption <- " Simulation-based cutoff values applied."
    }
  }

  caption <- paste0(base_caption, if (!is.null(cutoff_caption)) cutoff_caption)
  if (!is.null(cutoff)) {
    caption <- paste0(
      caption,
      " Flagged: overfit = infit below range (more predictable); ",
      "underfit = above range (noisier)."
    )
  }

  knitr::kable(
    item_fit_table,
    format = "pipe",
    col.names = if (is.null(cutoff)) {
      c("Item", "Infit MSQ", "Infit SE", "Relative location")
    } else {
      c(
        "Item",
        "Infit MSQ",
        "Infit SE",
        "Infit low",
        "Infit high",
        "Flagged",
        "Relative location"
      )
    },
    caption = caption
  )
}

# Tests for RMitemInfitCutoffMI() and RMitemInfitMI()

# Build a small `mids` (multiply-imputed dataset) object once for the file.
make_mids <- function(n = 200, k = 5, prop_na = 0.05, seed = 1L) {
  set.seed(seed)
  mat <- matrix(sample(0:1, n * k, replace = TRUE), n, k)
  mat[sample(length(mat), round(prop_na * length(mat)))] <- NA
  df <- as.data.frame(mat)
  colnames(df) <- paste0("I", seq_len(k))
  suppressWarnings(mice::mice(
    df,
    m = 3L,
    method = "pmm",
    seed = seed,
    printFlag = FALSE
  ))
}

# A `mids` built from ordered factors + mice's `polr` method, as recommended
# for ordinal items. mice::complete() then returns ordered factors, which the
# MI functions must coerce back to numeric internally.
make_mids_polr <- function(n = 200, k = 5, prop_na = 0.05, seed = 1L) {
  set.seed(seed)
  mat <- matrix(sample(0:1, n * k, replace = TRUE), n, k)
  mat[sample(length(mat), round(prop_na * length(mat)))] <- NA
  df <- as.data.frame(mat)
  colnames(df) <- paste0("I", seq_len(k))
  df[] <- lapply(df, function(x) factor(x, ordered = TRUE))
  suppressWarnings(mice::mice(
    df,
    m = 3L,
    method = "polr",
    seed = seed,
    printFlag = FALSE
  ))
}

# ---------------------------------------------------------------------
# RMitemInfitCutoffMI
# ---------------------------------------------------------------------
test_that("RMitemInfitCutoffMI accepts a polr/ordered-factor mids (coerced to numeric)", {
  skip_on_cran()
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  skip_if_not_installed("MASS")
  imp <- make_mids_polr()
  res <- RMitemInfitCutoffMI(
    imp,
    iterations = 5L,
    parallel = FALSE,
    seed = 1L,
    cutoff_method = "quantile"
  )
  expect_equal(res$n_imputations, 3L)
  expect_equal(nrow(res$item_cutoffs), 5L)
})

test_that("RMitemInfitCutoffMI returns the expected list structure", {
  skip_on_cran()
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")

  imp <- make_mids()
  res <- RMitemInfitCutoffMI(imp, iterations = 5L, parallel = FALSE, seed = 1L)
  expect_type(res, "list")
  expect_true(all(
    c(
      "results",
      "item_cutoffs",
      "actual_iterations",
      "sample_n",
      "item_names",
      "cutoff_method",
      "hdci_width",
      "n_imputations"
    ) %in%
      names(res)
  ))
  expect_equal(res$n_imputations, 3L)
  expect_s3_class(res$item_cutoffs, "data.frame")
  expect_equal(nrow(res$item_cutoffs), 5L)
  expect_true(all(
    c("Item", "infit_low", "infit_high") %in%
      names(res$item_cutoffs)
  ))
})

test_that("RMitemInfitCutoffMI is reproducible with the same seed", {
  skip_on_cran()
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")

  imp <- make_mids()
  r1 <- RMitemInfitCutoffMI(imp, iterations = 5L, parallel = FALSE, seed = 42L)
  r2 <- RMitemInfitCutoffMI(imp, iterations = 5L, parallel = FALSE, seed = 42L)
  expect_equal(r1$item_cutoffs, r2$item_cutoffs)
})

test_that("RMitemInfitCutoffMI cutoff_method = 'quantile' aggregates without ggdist", {
  skip_on_cran()
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")

  imp <- make_mids()
  res <- RMitemInfitCutoffMI(
    imp,
    iterations = 6L,
    parallel = FALSE,
    seed = 1L,
    cutoff_method = "quantile"
  )
  expect_equal(res$cutoff_method, "quantile")
  expect_equal(nrow(res$item_cutoffs), 5L)
  expect_true(all(res$item_cutoffs$infit_low < res$item_cutoffs$infit_high))
})

test_that("RMitemInfitCutoffMI verbose = TRUE runs the per-imputation progress path", {
  skip_on_cran()
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")

  imp <- make_mids()
  suppressMessages(invisible(utils::capture.output(
    res <- RMitemInfitCutoffMI(
      imp,
      iterations = 6L,
      parallel = FALSE,
      seed = 1L,
      cutoff_method = "quantile",
      verbose = TRUE
    )
  )))
  expect_equal(res$n_imputations, 3L)
})

test_that("RMitemInfitCutoffMI errors on a non-mids object", {
  skip_if_not_installed("mice")
  expect_snapshot(
    RMitemInfitCutoffMI(data.frame(a = 0:1, b = 1:0)),
    error = TRUE
  )
})

# ---------------------------------------------------------------------
# RMitemInfitMI
# ---------------------------------------------------------------------
test_that("RMitemInfitMI output = 'dataframe' returns pooled infit per item", {
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")

  imp <- make_mids()
  res <- RMitemInfitMI(imp, output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_equal(nrow(res), 5L)
  expect_true(all(c("Item", "Infit_MSQ") %in% names(res)))
})

test_that("RMitemInfitMI accepts an RMitemInfitCutoffMI result and adds Flagged column", {
  skip_on_cran()
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")

  imp <- make_mids()
  cuts <- RMitemInfitCutoffMI(imp, iterations = 5L, parallel = FALSE, seed = 1L)
  res <- RMitemInfitMI(imp, cutoff = cuts, output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_true("Flagged" %in% names(res))
  expect_type(res$Flagged, "character")
  expect_true(all(res$Flagged %in% c("overfit", "underfit", "")))
})

test_that("RMitemInfitMI output = 'kable' returns a knitr_kable", {
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  skip_if_not_installed("knitr")

  imp <- make_mids()
  out <- RMitemInfitMI(imp, output = "kable")
  expect_s3_class(out, "knitr_kable")
})

test_that("RMitemInfitMI errors on a non-mids object", {
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  expect_snapshot(
    RMitemInfitMI(data.frame(a = 0:1, b = 1:0)),
    error = TRUE
  )
})

test_that("RMitemInfitMI rejects malformed cutoff arguments", {
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  imp <- make_mids()
  expect_snapshot(
    RMitemInfitMI(imp, cutoff = "abc"),
    error = TRUE
  )
  expect_snapshot(
    RMitemInfitMI(imp, cutoff = data.frame(Item = "I1", infit_low = 0.8)),
    error = TRUE
  )
})

test_that("RMitemInfitMI errors when cutoff item names do not match", {
  skip_on_cran()
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  imp <- make_mids()
  cuts <- RMitemInfitCutoffMI(imp, iterations = 5L, parallel = FALSE, seed = 1L)
  bad <- cuts$item_cutoffs
  bad$Item[1L] <- "NotAnItem"
  expect_error(RMitemInfitMI(imp, cutoff = bad), regexp = "do not match")
})

test_that("RMitemInfitMI accepts the bare $item_cutoffs data.frame", {
  skip_on_cran()
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  imp <- make_mids()
  cuts <- RMitemInfitCutoffMI(imp, iterations = 5L, parallel = FALSE, seed = 1L)
  res <- RMitemInfitMI(imp, cutoff = cuts$item_cutoffs, output = "dataframe")
  expect_true("Flagged" %in% names(res))
  # Without the full object, the caption falls back to the generic wording
  kbl <- RMitemInfitMI(imp, cutoff = cuts$item_cutoffs)
  expect_match(
    paste(kbl, collapse = "\n"),
    "Simulation-based cutoff values applied"
  )
})

test_that("RMitemInfitMI sort = 'infit' sorts descending", {
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  imp <- make_mids()
  res <- RMitemInfitMI(imp, output = "dataframe", sort = "infit")
  expect_false(is.unsorted(rev(res$Infit_MSQ)))
})

test_that("RMitemInfitMI kable captions report pooling and cutoff metadata", {
  skip_on_cran()
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  imp <- make_mids()

  plain <- paste(RMitemInfitMI(imp), collapse = "\n")
  expect_match(plain, "Pooled MSQ values from 3 imputations")
  expect_match(plain, "missing values imputed, m = 3")
  expect_match(plain, "per imputed dataset")

  cuts <- RMitemInfitCutoffMI(imp, iterations = 5L, parallel = FALSE, seed = 1L)
  with_cut <- paste(RMitemInfitMI(imp, cutoff = cuts), collapse = "\n")
  expect_match(with_cut, "across 3 imputations")
  expect_match(with_cut, "overfit = infit below range")
})

# ---------------------------------------------------------------------
# RMitemInfitMI -- imputation-failure paths
# ---------------------------------------------------------------------

# Tamper with a mids object so that the completed dataset for the given
# imputation(s) contains a negative response, which fails
# validate_response_data() inside the per-imputation loop.
break_imputations <- function(imp, which_m) {
  col_with_na <- names(Filter(function(x) nrow(x) > 0L, imp$imp))[1L]
  for (i in which_m) {
    imp$imp[[col_with_na]][1L, i] <- -1
  }
  imp
}

test_that("RMitemInfitMI warns about failed imputations and pools the rest", {
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  imp <- break_imputations(make_mids(), which_m = 1L)
  expect_warning(
    expect_warning(
      res <- RMitemInfitMI(imp, output = "dataframe"),
      regexp = "failed for imputation 1"
    ),
    regexp = "1 of 3 imputed datasets failed"
  )
  expect_equal(nrow(res), 5L)
  expect_true(all(is.finite(res$Infit_MSQ)))
})

test_that("RMitemInfitMI errors when fewer than 2 imputations succeed", {
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  imp <- break_imputations(make_mids(), which_m = 1:2)
  expect_error(
    suppressWarnings(RMitemInfitMI(imp, output = "dataframe")),
    regexp = "At least 2 are required"
  )
})

test_that("RMitemInfitMI errors when all imputations fail", {
  skip_if_not_installed("mice")
  skip_if_not_installed("iarm")
  imp <- break_imputations(make_mids(), which_m = 1:3)
  expect_error(
    suppressWarnings(RMitemInfitMI(imp, output = "dataframe")),
    regexp = "failed for all 3 imputed datasets"
  )
})

#' Conditional Item Infit MSQ
#'
#' Computes conditional infit mean-square (MSQ) statistics for each item using
#' `iarm::out_infit()`, enriched with item locations relative to the sample
#' mean person location.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Missing values (`NA`) are allowed,
#'   but at least one complete case (row with no `NA`) must be present.
#' @param cutoff Optional. Default `NULL` (no cutoff applied, behaviour is
#'   identical to the current version). Can be:
#'   * The return value of \code{\link{RMitemInfitCutoff}} (a list with `$item_cutoffs`):
#'     the data.frame is extracted automatically and the number of simulation
#'     iterations, `cutoff_method`, and `hdci_width` are included in the kable
#'     caption.
#'   * The `$item_cutoffs` data.frame from \code{\link{RMitemInfitCutoff}} directly: must
#'     have columns `Item`, `infit_low`, and `infit_high`.
#'   When provided, adds columns `Infit_low`, `Infit_high`, and `Flagged`
#'   to the result. `Flagged` labels the misfit direction: `"overfit"`
#'   (infit below the range -- more predictable than the model expects),
#'   `"underfit"` (above the range -- noisier than expected), or `""` (within
#'   range, no misfit).
#' @param p_value Logical. If `TRUE`, bootstrap p-values are computed from the
#'   simulated null distribution and added to the output. This requires
#'   `cutoff` to be the **full** \code{\link{RMitemInfitCutoff}} object (which
#'   carries the per-item simulated values in its `$results` element); the
#'   summarised `$item_cutoffs` data.frame is not sufficient. Default `FALSE`,
#'   in which case behaviour is unchanged.
#' @param correction Character. Multiple-comparison correction applied across
#'   items when `p_value = TRUE`: `"fwer"` (default) for the Westfall-Young
#'   studentised-max step-down (family-wise error rate), `"fdr_bh"` /
#'   `"fdr_by"` for Benjamini-Hochberg / Benjamini-Yekutieli false discovery
#'   rate control, or `"none"` for uncorrected per-item p-values.
#' @param alpha Numeric in (0, 1). Significance level for the `Flagged` column
#'   when `p_value = TRUE` (an item is flagged when its corrected p-value is
#'   below `alpha`). Default `0.05`.
#' @param output Character string controlling the return value. Either
#'   `"kable"` (default) for a formatted `knitr::kable()` table, or
#'   `"dataframe"` for the underlying data.frame.
#' @param sort Optional character string. When `sort = "infit"`, rows are
#'   sorted by `Infit_MSQ` in descending order before output.
#'
#' @return
#' * If `output = "kable"`: a `knitr_kable` object (plain text table via
#'   `format = "pipe"`) with columns "Item", "Infit MSQ", and "Relative
#'   location", and a caption showing the number of complete cases. When
#'   `cutoff` is provided, columns "Infit low", "Infit high", and "Flagged"
#'   are also included, and the caption notes the simulation-based cutoffs.
#' * If `output = "dataframe"`: a data.frame with columns `Item`,
#'   `Infit_MSQ`, and `Relative_location`. When `cutoff` is provided, columns
#'   `Infit_low`, `Infit_high`, and `Flagged` are also included (inserted
#'   after `Infit_MSQ`, before `Relative_location`). `Flagged` is a character
#'   column with values `"overfit"`, `"underfit"`, or `""` (not the previous
#'   logical). When `p_value = TRUE`, columns `p_infit` (marginal two-sided
#'   p-value) and `padj_infit` (corrected p-value) are added and `Flagged`
#'   reflects items with `padj_infit < alpha` (direction from `Infit_MSQ`
#'   relative to 1).
#'
#' @details
#' Infit MSQ is a weighted fit statistic that emphasises deviations near the
#' item location. Values close to 1.0 indicate good fit. Values substantially
#' above 1.0 suggest underfit (unexpected responses), while values substantially
#' below 1.0 suggest overfit (overly predictable responses). The definition of
#' "substantially" depends on several factors such as sample size, and needs to
#' be determined by simulation using \code{\link{RMitemInfitCutoff}}. There is no general
#' rule-of-thumb value that is correct.
#'
#' Conditional infit MSQ statistics are computed via `iarm::out_infit()`, which
#' uses the conditional distribution of the sufficient statistics (Müller, 2020).
#' Only complete cases (rows without any `NA`) are used in the conditional fit
#' calculation.
#'
#' Item parameters are estimated by conditional maximum likelihood via
#' `psychotools::pcmodel()` (a dichotomous item is a 2-category PCM); the
#' conditional infit/outfit MSQ comes from `iarm::out_infit()` and is invariant
#' to the estimation engine. Per-item average locations are the means of the
#' CML thresholds, and the person-location reference is the mean of the Warm
#' WLE estimates.
#'
#' Relative item location is defined as the item's average location minus the
#' sample mean person location, providing a measure of item targeting.
#'
#' The `iarm` package must be installed (it is in Suggests, not Imports).
#'
#' \strong{Bootstrap p-values.} When `p_value = TRUE`, each item's observed
#' infit is compared against its simulated null distribution (from
#' `cutoff$results`). The per-item statistic is the residual studentised by
#' the bootstrap mean and SD -- deliberately the empirical SD rather than the
#' Wilson-Hilferty / ZSTD transform, which is uninformative for conditional
#' MSQ (Müller, 2020). The marginal p-value is the two-sided Monte-Carlo
#' p-value `(1 + #{|t*| >= |t|}) / (B + 1)`. For `correction = "fwer"` the
#' family-wise adjustment uses the Westfall-Young studentised-max step-down,
#' which exploits the bootstrap dependence among items and is more powerful
#' than Bonferroni/Holm (Ferreira, 2024); its validity rests on subset
#' pivotality. `"fdr_bh"`/`"fdr_by"` apply Benjamini-Hochberg / Benjamini-
#' Yekutieli instead. These are model-conditional, sample-size-sensitive
#' p-values and are reported alongside the simulated effect-size band, not in
#' place of it. p-values can be no smaller than `1 / (B + 1)`, and the
#' studentised-max (FWER) correction is *liberal* when the simulation is small
#' (the bootstrap mean/SD used for studentisation are then too noisy). At
#' least 1000 `iterations` in [RMitemInfitCutoff()] are recommended -- in
#' simulations the family-wise error rate is then controlled at the nominal
#' level -- and a warning is issued when the simulation is smaller. The
#' marginal p-values are well calibrated even at a few hundred iterations.
#'
#' @section Multiple comparisons:
#' The marginal p-value controls the error rate of a *single* comparison: for
#' one item (or item pair) decided on in advance it is the relevant value. But
#' scanning all *k* comparisons and flagging whichever fall below `alpha` tests
#' *k* hypotheses at once, so the chance of at least one false flag inflates to
#' roughly \eqn{1 - (1 - \alpha)^k} (e.g. about 34% for *k* = 8 at
#' `alpha = 0.05`) -- even when every marginal p-value is correctly calibrated.
#' The corrected (adjusted) p-value controls this: `correction = "fwer"` bounds
#' the probability of *any* false flag (strict, lower power), while `"fdr_bh"` /
#' `"fdr_by"` bound the expected *proportion* of false flags among those raised
#' (a more lenient middle ground). Rule of thumb: use the marginal p-value for a
#' single pre-specified comparison, and a corrected p-value when screening the
#' whole table -- the usual workflow.
#'
#' @references
#' Müller, M. (2020). Item fit statistics for Rasch analysis: Can we trust
#' them? *Journal of Statistical Distributions and Applications*, 7(5).
#' \doi{10.1186/s40488-020-00108-7}
#'
#' Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number
#' of hypotheses simultaneously. *Journal of Statistical Theory and Practice,
#' 19*(6). \doi{10.1007/s42519-024-00412-4}
#'
#' Westfall, P. H., & Young, S. S. (1993). *Resampling-Based Multiple Testing*.
#' Wiley.
#'
#' @seealso \code{\link{RMitemInfitCutoff}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE)) {
#'   # Simulate binary item response data (5 items, 40 persons)
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 40 * 5, replace = TRUE), nrow = 40, ncol = 5)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:5)
#'
#'   # Default kable output
#'   RMitemInfit(sim_data)
#'
#'   # Sorted by infit MSQ descending
#'   RMitemInfit(sim_data, sort = "infit")
#'
#'   # Return as data.frame for further processing
#'   df <- RMitemInfit(sim_data, output = "dataframe")
#'
#'   # Simulation-based cutoffs (100 Monte-Carlo iterations)
#'   if (requireNamespace("ggdist", quietly = TRUE)) {
#'     cutoff_res <- RMitemInfitCutoff(sim_data, iterations = 100,
#'                                     parallel = FALSE, seed = 42)
#'     RMitemInfit(sim_data, cutoff = cutoff_res)
#'     RMitemInfit(sim_data, cutoff = cutoff_res, output = "dataframe")
#'
#'     # Bootstrap p-values with family-wise (Westfall-Young) correction
#'     # (use iterations >= 1000 in real analyses for stable p-values)
#'     RMitemInfit(sim_data, cutoff = cutoff_res, p_value = TRUE,
#'                 output = "dataframe")
#'   }
#' }
#' }
RMitemInfit <- function(
  data,
  cutoff = NULL,
  p_value = FALSE,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  alpha = 0.05,
  output = "kable",
  sort
) {
  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required for RMitemInfit() but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }

  output <- match.arg(output, c("kable", "dataframe"))
  correction <- match.arg(correction)
  if (!is.numeric(alpha) || length(alpha) != 1L || alpha <= 0 || alpha >= 1) {
    stop("`alpha` must be a single number in (0, 1).", call. = FALSE)
  }

  # --- Validate and normalise cutoff ------------------------------------------
  cutoff_n_iter <- NULL
  cutoff_method <- NULL
  cutoff_hdci_width <- NULL
  cutoff_full <- NULL # full object (carries simulated $results for p-values)
  if (!is.null(cutoff)) {
    if (
      is.list(cutoff) &&
        !is.data.frame(cutoff) &&
        "item_cutoffs" %in% names(cutoff)
    ) {
      cutoff_full <- cutoff
      cutoff_n_iter <- cutoff$actual_iterations
      cutoff_method <- cutoff$cutoff_method
      cutoff_hdci_width <- cutoff$hdci_width
      cutoff <- cutoff$item_cutoffs
    }
    if (!is.data.frame(cutoff)) {
      stop(
        "`cutoff` must be NULL, the return value of RMitemInfitCutoff(), or its ",
        "$item_cutoffs data.frame.",
        call. = FALSE
      )
    }
    required_cols <- c("Item", "infit_low", "infit_high")
    missing_cols <- setdiff(required_cols, names(cutoff))
    if (length(missing_cols) > 0L) {
      stop(
        "`cutoff` data.frame is missing required columns: ",
        paste(missing_cols, collapse = ", "),
        ".",
        call. = FALSE
      )
    }
  }

  # --- p-value prerequisites --------------------------------------------------
  if (p_value) {
    if (is.null(cutoff_full) || is.null(cutoff_full$results)) {
      stop(
        "`p_value = TRUE` requires the full RMitemInfitCutoff() object (it ",
        "carries the simulated distributions in $results); a NULL cutoff or ",
        "the bare $item_cutoffs data.frame is not sufficient.",
        call. = FALSE
      )
    }
    if (!is.null(cutoff_n_iter) && cutoff_n_iter < 1000L) {
      warning(
        "Bootstrap p-values are based on only ",
        cutoff_n_iter,
        " simulation iterations. With few iterations the studentised-max ",
        "(FWER) correction is liberal and small p-values are imprecise; ",
        "use iterations >= 1000 in RMitemInfitCutoff() for reliable p-values.",
        call. = FALSE
      )
    }
  }

  validate_response_data(data)

  if (nrow(stats::na.omit(data)) == 0L) {
    stop(
      "No complete cases in data. All rows contain at least one NA.",
      call. = FALSE
    )
  }

  # Respondents with no responses at all contribute nothing and break the CML
  # fit (psychotools errors on all-NA rows); drop them, keeping the raw totals
  # for the caption.
  n_total <- nrow(as.data.frame(data))
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)

  data_mat <- as.matrix(data)

  # --- Fit Rasch model and compute item/person locations ----------------------
  # CML item parameters (psychotools; a dichotomous item is a 2-category PCM)
  # and WLE person locations, consistent with the rest of the package. The
  # conditional infit/outfit statistic from iarm is engine-invariant; only the
  # relative-location reference shifts slightly (WLE vs eRm MLE person mean).
  fit <- psychotools::pcmodel(data)
  thr_list <- .center_thresholds(lapply(
    psychotools::threshpar(fit),
    as.numeric
  ))
  item_avg_locations <- vapply(thr_list, mean, numeric(1L))
  names(item_avg_locations) <- names(data)
  person_avg_location <- mean(
    .estimate_thetas(data_mat, thr_list, method = "WLE")$theta,
    na.rm = TRUE
  )

  relative_item_avg_locations <- item_avg_locations - person_avg_location

  # --- rgl.useNULL workaround -------------------------------------------------
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # --- Compute conditional infit MSQ via iarm ---------------------------------
  cfit <- iarm::out_infit(fit)

  # --- Count complete cases ---------------------------------------------------
  n_complete <- nrow(stats::na.omit(data))
  n_clause <- .n_caption(
    n_complete,
    n_total,
    if (has_na) "complete cases" else character()
  )

  # --- Assemble result data.frame (unrounded; kable rounds for display) -------
  item_fit_table <- data.frame(
    Item = names(data),
    Infit_MSQ = as.numeric(cfit$Infit),
    Relative_location = as.numeric(relative_item_avg_locations),
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  # --- Apply cutoff if provided ------------------------------------------------
  if (!is.null(cutoff)) {
    data_items <- item_fit_table$Item
    cutoff_items <- cutoff$Item
    if (!setequal(data_items, cutoff_items)) {
      stop(
        "Item names in `cutoff` do not match item names in `data`.\n",
        "  data items  : ",
        paste(data_items, collapse = ", "),
        "\n",
        "  cutoff items: ",
        paste(cutoff_items, collapse = ", "),
        call. = FALSE
      )
    }
    cutoff_sub <- cutoff[, c("Item", "infit_low", "infit_high")]
    item_fit_table <- merge(
      item_fit_table,
      cutoff_sub,
      by = "Item",
      sort = FALSE
    )
    # Restore original row order (merge may reorder)
    item_fit_table <- item_fit_table[match(data_items, item_fit_table$Item), ]
    rownames(item_fit_table) <- NULL
    item_fit_table$Infit_low <- item_fit_table$infit_low
    item_fit_table$Infit_high <- item_fit_table$infit_high
    item_fit_table$infit_low <- NULL
    item_fit_table$infit_high <- NULL
    # Flagged labels the misfit direction: infit below the expected range =
    # overfit (more predictable than the model expects), above = underfit
    # (noisier than expected); "" when within range.
    item_fit_table$Flagged <- ifelse(
      item_fit_table$Infit_MSQ < item_fit_table$Infit_low,
      "overfit",
      ifelse(
        item_fit_table$Infit_MSQ > item_fit_table$Infit_high,
        "underfit",
        ""
      )
    )

    if (p_value) {
      # Compare observed infit to its simulated null (cutoff_full$results).
      sim_items <- unique(cutoff_full$results$Item)
      if (!setequal(data_items, sim_items)) {
        stop(
          "Item names in the cutoff simulations ($results) do not match ",
          "`data`.",
          call. = FALSE
        )
      }
      sim_mat <- tapply(
        cutoff_full$results$InfitMSQ,
        list(cutoff_full$results$iteration, cutoff_full$results$Item),
        function(x) x[1L]
      )
      observed <- stats::setNames(as.numeric(cfit$Infit), names(data))
      pv <- .bootstrap_pvalues(observed, sim_mat, correction = correction)
      idx <- match(item_fit_table$Item, pv$name)
      item_fit_table$p_infit <- pv$p[idx]
      item_fit_table$padj_infit <- pv$padj[idx]
      sig <- item_fit_table$padj_infit < alpha
      item_fit_table$Flagged <- ifelse(
        is.na(sig) | !sig,
        "",
        ifelse(item_fit_table$Infit_MSQ > 1, "underfit", "overfit")
      )
      item_fit_table <- item_fit_table[, c(
        "Item",
        "Infit_MSQ",
        "Infit_low",
        "Infit_high",
        "p_infit",
        "padj_infit",
        "Flagged",
        "Relative_location"
      )]
    } else {
      # Reorder: Item, Infit_MSQ, Infit_low, Infit_high, Flagged, Relative_location
      item_fit_table <- item_fit_table[, c(
        "Item",
        "Infit_MSQ",
        "Infit_low",
        "Infit_high",
        "Flagged",
        "Relative_location"
      )]
    }
  }

  # --- Sort if requested ------------------------------------------------------
  if (!missing(sort) && identical(sort, "infit")) {
    item_fit_table <- item_fit_table[
      order(item_fit_table$Infit_MSQ, decreasing = TRUE),
    ]
    rownames(item_fit_table) <- NULL
  }

  # --- Return -----------------------------------------------------------------
  if (output == "dataframe") {
    return(item_fit_table)
  }

  # Kable display rounding (the dataframe output above stays unrounded)
  item_fit_table <- .round_display(item_fit_table, c(
    Infit_MSQ = 3, Infit_low = 3, Infit_high = 3,
    p_infit = 4, padj_infit = 4, Relative_location = 2
  ))

  if (p_value) {
    kbl_colnames <- c(
      "Item",
      "Infit MSQ",
      "Infit low",
      "Infit high",
      "p",
      "p (adj)",
      "Flagged",
      "Relative location"
    )
    corr_label <- .correction_label(correction)
    kbl_caption <- paste0(
      "MSQ values based on conditional estimation. ",
      n_clause,
      ". Two-sided bootstrap p-values from ",
      cutoff_n_iter,
      " iterations; multiplicity correction: ",
      corr_label,
      "; flagged at alpha = ",
      alpha,
      ". p-values cannot be smaller than ",
      "1/(",
      cutoff_n_iter,
      "+1) = ",
      round(1 / (cutoff_n_iter + 1), 4),
      ".",
      " Flagged: underfit (infit > 1, noisier) / overfit (infit < 1, more predictable)."
    )
  } else if (is.null(cutoff)) {
    kbl_colnames <- c("Item", "Infit MSQ", "Relative location")
    kbl_caption <- paste0(
      "MSQ values based on conditional estimation. ",
      n_clause,
      "."
    )
  } else {
    kbl_colnames <- c(
      "Item",
      "Infit MSQ",
      "Infit low",
      "Infit high",
      "Flagged",
      "Relative location"
    )
    if (!is.null(cutoff_n_iter)) {
      method_label <- .format_cutoff_method_label(
        cutoff_method,
        cutoff_hdci_width
      )
      iter_part <- paste0(cutoff_n_iter, " simulation iterations")
      kbl_caption <- paste0(
        "MSQ values based on conditional estimation. ",
        n_clause,
        ". Cutoff values based on ",
        if (!is.null(method_label)) {
          paste0(iter_part, " (", method_label, ").")
        } else {
          paste0(iter_part, ".")
        }
      )
    } else {
      kbl_caption <- paste0(
        "MSQ values based on conditional estimation. ",
        n_clause,
        ". Simulation-based cutoff values applied."
      )
    }
    kbl_caption <- paste0(
      kbl_caption,
      " Flagged: overfit = infit below range (more predictable); ",
      "underfit = above range (noisier)."
    )
  }

  knitr::kable(
    item_fit_table,
    format = "pipe",
    col.names = kbl_colnames,
    caption = kbl_caption
  )
}

# ---------------------------------------------------------------------------
# Internal helper
# ---------------------------------------------------------------------------

#' Format a human-readable label for the cutoff method
#'
#' @param cutoff_method Character. `"hdci"`, `"quantile"`, or `NULL`.
#' @param hdci_width Numeric or `NULL`. HDCI width (e.g., `0.999`).
#' @return A character label, or `NULL` if the method is unknown/unset.
#' @keywords internal
.format_cutoff_method_label <- function(cutoff_method, hdci_width) {
  if (is.null(cutoff_method)) {
    return(NULL)
  }
  switch(
    cutoff_method,
    quantile = "2.5th/97.5th percentile",
    hdci = if (!is.null(hdci_width)) {
      paste0(hdci_width * 100, "% HDCI")
    } else {
      "HDCI"
    },
    NULL
  )
}

#' Raw-Score to Logit Score Transformation Table
#'
#' For a given set of items, returns the score-to-theta lookup that maps each
#' possible raw sum score to a person-location estimate (in logits) and its
#' standard error. Useful when reporting a scale's measurement properties or
#' converting raw totals to interval-scaled scores for downstream analysis.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Missing values (`NA`) are allowed;
#'   the underlying model fit handles them.
#' @param method Character string. Either `"WLE"` (default) for Warm's Weighted
#'   Likelihood Estimator computed from a CML-fitted Rasch / Partial Credit
#'   Model via `psychotools`, or `"EAP"` for Expected A Posteriori sum-score
#'   estimates from an MML-fitted model via `mirt`.
#' @param output Character string controlling the return value: `"kable"`
#'   (default) for a formatted `knitr::kable()` table, `"dataframe"` for the
#'   underlying data.frame, or `"ggplot"` for a `ggplot2` figure showing each
#'   raw score's logit estimate with `ci_multiplier`-scaled error bars.
#' @param ci_multiplier Numeric. Multiplier applied to the standard error to
#'   draw error bars on the figure. Default `1.96` (\eqn{\approx}95% CI under a Gaussian
#'   approximation). Ignored when `output != "ggplot"`.
#' @param point_size Numeric. Point size for the figure. Default `3`.
#' @param error_width Numeric. Cap width for error bars on the figure.
#'   Default `0.5`.
#' @param theta_range Numeric length 2. Theta search range used for boundary
#'   raw scores under WLE estimation. Default `c(-10, 10)`. Ignored when
#'   `method = "EAP"`.
#'
#' @return
#' * If `output = "kable"`: a `knitr_kable` object with columns "Ordinal sum
#'   score", "Logit score", and "Logit std.error", and a caption noting the
#'   estimation method.
#' * If `output = "dataframe"`: a data.frame with columns `raw_score`,
#'   `logit_score`, and `logit_se` (one row per possible raw sum score from 0
#'   to the theoretical maximum).
#' * If `output = "ggplot"`: a `ggplot` object — points at each
#'   (`logit_score`, `raw_score`) with horizontal error bars at
#'   ± `ci_multiplier × logit_se`.
#'
#' @details
#' The function automatically detects whether the data is dichotomous
#' (max score 1) or polytomous (max score > 1) and selects the appropriate
#' Rasch / Partial Credit model.
#'
#' **`method = "WLE"`** fits the model by CML with `psychotools::pcmodel()`,
#' centres the item thresholds to grand-mean-zero, and solves Warm's
#' weighted-likelihood equation for each raw score with the same engine used
#' by [RMpersonParameters()]; the two functions therefore report identical
#' locations and standard errors. Warm's bias correction yields finite
#' locations even at the minimum and maximum scores (only a root outside
#' `theta_range` is clamped to the boundary with `NA` SE). The reported
#' `logit_se` is the information-based standard error `1 / sqrt(I(theta))`,
#' matching `catR`, `TAM` and most Rasch software.
#'
#' **`method = "EAP"`** fits the model with `mirt::mirt(..., itemtype =
#' "Rasch")` (MML) and obtains sum-score-based EAP estimates and posterior
#' SDs via `mirt::fscores(method = "EAPsum", full.scores = FALSE,
#' full.scores.SE = TRUE)`. EAP estimates are finite at all score boundaries
#' (the prior shrinks them inward), but they depend on the assumed normal
#' prior on theta. Item parameters from MML differ slightly from the CML
#' values used by the WLE path; for well-behaved data the difference is small.
#'
#' @references
#' Warm, T. A. (1989). Weighted likelihood estimation of ability in item
#' response theory. *Psychometrika, 54*(3), 427-450.
#' \doi{10.1007/BF02294627}
#'
#' Bock, R. D., & Mislevy, R. J. (1982). Adaptive EAP estimation of ability
#' in a microcomputer environment. *Applied Psychological Measurement, 6*(4),
#' 431-444. \doi{10.1177/014662168200600405}
#'
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(42)
#' sim_data <- as.data.frame(
#'   matrix(sample(0:3, 200 * 6, replace = TRUE), nrow = 200, ncol = 6)
#' )
#' colnames(sim_data) <- paste0("Item", 1:6)
#'
#' # Default kable output, WLE
#' RMscoreSE(sim_data)
#'
#' # Underlying data.frame
#' RMscoreSE(sim_data, output = "dataframe")
#'
#' # ggplot figure
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   RMscoreSE(sim_data, output = "ggplot")
#' }
#'
#' # EAP via mirt
#' RMscoreSE(sim_data, method = "EAP")
#' }
RMscoreSE <- function(
  data,
  method = "WLE",
  output = "kable",
  ci_multiplier = 1.96,
  point_size = 3,
  error_width = 0.5,
  theta_range = c(-10, 10)
) {
  method <- match.arg(method, c("WLE", "EAP"))
  output <- match.arg(output, c("kable", "dataframe", "ggplot"))

  validate_response_data(data)

  if (
    !is.numeric(theta_range) ||
      length(theta_range) != 2L ||
      theta_range[1L] >= theta_range[2L]
  ) {
    stop(
      "`theta_range` must be a numeric vector of length 2 with ",
      "theta_range[1] < theta_range[2].",
      call. = FALSE
    )
  }

  data <- as.data.frame(data)
  n_total <- nrow(data)
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)
  n_used <- nrow(data)
  n_clause <- paste0(
    " ",
    .n_caption(
      n_used,
      n_total,
      if (has_na) "incomplete responses retained" else character()
    ),
    "."
  )

  # --- Compute the score-to-theta table ---------------------------------------
  score_table <- if (method == "WLE") {
    .scoreSE_wle(data, theta_range)
  } else {
    .scoreSE_eap(data)
  }

  # --- Return: dataframe ------------------------------------------------------
  if (output == "dataframe") {
    return(score_table)
  }

  # --- Return: kable ----------------------------------------------------------
  if (output == "kable") {
    display <- data.frame(
      raw_score = score_table$raw_score,
      logit_score = round(score_table$logit_score, 3),
      logit_se = round(score_table$logit_se, 3),
      stringsAsFactors = FALSE,
      row.names = NULL
    )
    caption <- paste0(
      if (method == "WLE") {
        "Person locations via Warm's WLE (CML item parameters)."
      } else {
        "Person locations via EAPsum (MML item parameters from mirt; depends on a normal theta prior)."
      },
      n_clause
    )
    return(knitr::kable(
      display,
      format = "pipe",
      col.names = c("Ordinal sum score", "Logit score", "Logit std.error"),
      caption = caption
    ))
  }

  # --- Return: ggplot ---------------------------------------------------------
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for output = \"ggplot\". ",
      "Install it with: install.packages(\"ggplot2\")",
      call. = FALSE
    )
  }

  fig_df <- score_table
  fig_df$lower <- fig_df$logit_score - ci_multiplier * fig_df$logit_se
  fig_df$upper <- fig_df$logit_score + ci_multiplier * fig_df$logit_se
  # Drop boundary rows where the SE is NA so the figure stays clean
  fig_df <- fig_df[
    is.finite(fig_df$logit_score) & !is.na(fig_df$logit_se),
    ,
    drop = FALSE
  ]

  ggplot2::ggplot(
    fig_df,
    ggplot2::aes(x = .data$logit_score, y = .data$raw_score)
  ) +
    ggplot2::geom_errorbar(
      ggplot2::aes(xmin = .data$lower, xmax = .data$upper),
      width = error_width,
      colour = "darkgrey",
      orientation = "y"
    ) +
    ggplot2::geom_point(size = point_size, shape = 18) +
    ggplot2::labs(
      x = "Logit interval score",
      y = "Ordinal sum score",
      caption = er2_caption(trimws(n_clause))
    ) +
    ggplot2::theme_bw() +
    er2_axis_margins() +
    er2_plot_caption()
}

# ---------------------------------------------------------------------------
# Internal: WLE path (shared Warm-WLE engine)
# ---------------------------------------------------------------------------

#' WLE score-to-theta lookup via the shared Warm-WLE solver
#'
#' Uses the same CML item parameters (centred to grand-mean-zero) and the
#' same `.theta_wle()` engine as [RMpersonParameters()], so the two
#' functions report identical locations and standard errors. The SE is
#' the information-based value `1 / sqrt(I(theta))`.
#'
#' @param data Validated response data.
#' @param theta_range Boundary-score search interval.
#' @return data.frame with columns `raw_score`, `logit_score`, `logit_se`.
#' @keywords internal
#' @noRd
.scoreSE_wle <- function(data, theta_range) {
  fit <- .rasch_fit_cml(data, se = FALSE)
  thr_list <- .center_thresholds(fit$thr_list)
  steps <- vapply(thr_list, length, integer(1))
  max_score <- sum(steps)

  est <- vapply(
    0:max_score,
    function(r) {
      .theta_wle(.score_pattern(r, steps), thr_list, theta_range)
    },
    numeric(2)
  )

  data.frame(
    raw_score = 0:max_score,
    logit_score = est[1L, ],
    logit_se = est[2L, ],
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

#' Build a representative response pattern summing to a given raw score
#'
#' For complete data the raw total is the sufficient statistic, so any
#' pattern with that sum yields the same WLE; this greedily fills items
#' up to their category maximum.
#'
#' @param r Target raw score.
#' @param steps Integer vector of per-item threshold counts (category max).
#' @return Integer response vector summing to `r`.
#' @keywords internal
#' @noRd
.score_pattern <- function(r, steps) {
  resp <- integer(length(steps))
  rem <- r
  for (i in seq_along(steps)) {
    take <- min(steps[i], rem)
    resp[i] <- take
    rem <- rem - take
  }
  resp
}

# ---------------------------------------------------------------------------
# Internal: EAP path (mirt EAPsum)
# ---------------------------------------------------------------------------

#' EAPsum score-to-theta lookup via mirt
#'
#' @param data Validated response data.
#' @return data.frame with columns `raw_score`, `logit_score`, `logit_se`.
#' @keywords internal
#' @noRd
.scoreSE_eap <- function(data) {
  if (!requireNamespace("mirt", quietly = TRUE)) {
    stop(
      "Package 'mirt' is required for method = \"EAP\". ",
      "Install it with: install.packages(\"mirt\")",
      call. = FALSE
    )
  }

  mirt_fit <- suppressMessages(
    mirt::mirt(
      data = as.data.frame(data),
      model = 1,
      itemtype = "Rasch",
      verbose = FALSE,
      accelerate = "squarem"
    )
  )

  sscores <- mirt::fscores(
    mirt_fit,
    method = "EAPsum",
    full.scores = FALSE,
    full.scores.SE = TRUE,
    verbose = FALSE
  )
  sscores <- as.data.frame(sscores)

  raw_col <- intersect(c("Sum.Scores", "Sum.Score"), names(sscores))[1L]
  theta_col <- intersect(c("F1", "Theta", "EAP"), names(sscores))[1L]
  se_col <- intersect(c("SE_F1", "SE", "SE_Theta"), names(sscores))[1L]

  if (is.na(raw_col) || is.na(theta_col) || is.na(se_col)) {
    stop(
      "Unexpected mirt::fscores() output structure; cannot locate ",
      "score / theta / SE columns.",
      call. = FALSE
    )
  }

  data.frame(
    raw_score = as.integer(sscores[[raw_col]]),
    logit_score = as.numeric(sscores[[theta_col]]),
    logit_se = as.numeric(sscores[[se_col]]),
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

#' PHQ-9 Depression Screener (NHANES Subsample)
#'
#' A processed subsample of the Patient Health Questionnaire 9-item (PHQ-9)
#' depression screener from the U.S. National Health and Nutrition
#' Examination Survey (NHANES), September 2024 release. Six hundred
#' respondents were drawn at random from the cycle's PHQ-9 module subject
#' to having complete responses on all nine items, while retaining a
#' realistic share of respondents with a sum-score of zero (n = 8) so
#' that floor behaviour can be illustrated in a Rasch analysis.
#'
#' @format A data frame with 600 rows and 12 variables:
#' \describe{
#'   \item{q1}{Little interest or pleasure in doing things. Integer 0--3.}
#'   \item{q2}{Feeling down, depressed, or hopeless. Integer 0--3.}
#'   \item{q3}{Trouble falling/staying asleep, or sleeping too much.
#'     Integer 0--3.}
#'   \item{q4}{Feeling tired or having little energy. Integer 0--3.}
#'   \item{q5}{Poor appetite or overeating. Integer 0--3.}
#'   \item{q6}{Feeling bad about yourself --- or that you are a failure
#'     or have let yourself or your family down. Integer 0--3.}
#'   \item{q7}{Trouble concentrating on things, such as reading the
#'     newspaper or watching television. Integer 0--3.}
#'   \item{q8}{Moving or speaking so slowly that other people could have
#'     noticed --- or the opposite, being so fidgety or restless that
#'     you have been moving around a lot more than usual. Integer 0--3.}
#'   \item{q9}{Thoughts that you would be better off dead, or of hurting
#'     yourself in some way. Integer 0--3.}
#'   \item{gender}{Self-reported gender, factor with levels
#'     `"Female"` and `"Male"` (31 respondents with missing values).}
#'   \item{age}{Age in years (integer, range 15--85).}
#'   \item{edu}{Highest educational attainment, factor with levels
#'     `"Elementary School"`, `"High school"`, `"University"`.}
#' }
#'
#' Each PHQ-9 item uses a four-point ordinal response scale, scored
#' 0 (`"Not at all"`), 1 (`"Several days"`), 2 (`"More than half the
#' days"`) and 3 (`"Nearly every day"`).
#'
#' @details
#' The dataset is a *processed subsample* intended for teaching and for
#' the package's worked example; it should not be treated as a canonical
#' NHANES microdata file. Users wishing to validate against NCHS-published
#' figures should download the original public-use microdata directly
#' from the NHANES website (see *Source*).
#'
#' @source
#' U.S. Centers for Disease Control and Prevention, National Center for
#' Health Statistics. *National Health and Nutrition Examination Survey*,
#' September 2024 release.
#' \url{https://wwwn.cdc.gov/nchs/nhanes/search/datapage.aspx?Component=Questionnaire&CycleBeginYear=2024}.
#' NHANES data are released to the public domain by the U.S. federal
#' government (\url{https://www.cdc.gov/nchs/policy/data-release-policy.html}).
#'
#' @references
#' Kroenke, K., Spitzer, R. L., & Williams, J. B. W. (2001). The PHQ-9:
#'   Validity of a brief depression severity measure. *Journal of General
#'   Internal Medicine*, 16(9), 606--613.
#'   \doi{10.1046/j.1525-1497.2001.016009606.x}
#'
#' @examples
#' data(phq9)
#' str(phq9)
#' summary(rowSums(phq9[, 1:9]))
"phq9"

#' Simulation-Based Infit MSQ Cutoff Determination for Multiply Imputed Data
#'
#' Extends \code{\link{RMitemInfitCutoff}} to work with multiply imputed datasets
#' produced by the `mice` package. Runs the parametric bootstrap simulation on
#' each imputed dataset and stacks the resulting distributions, so that the
#' final cutoff intervals reflect both sampling variability and imputation
#' uncertainty.
#'
#' @param mids_object A `mids` object (multiply imputed dataset) as returned by
#'   `mice::mice()`. Each completed dataset must contain only the item response
#'   columns to be analysed (i.e., no ID or grouping variables). Items must be
#'   scored starting at 0 (non-negative integers).
#' @param iterations Integer. Total number of simulation iterations to run
#'   across all imputations. These are distributed approximately evenly across
#'   the `m` imputed datasets (default 500).
#' @param parallel Logical. Use parallel processing via `mirai` within each
#'   imputed dataset (default `TRUE`). Passed to \code{\link{RMitemInfitCutoff}}.
#' @param n_cores Integer or `NULL`. Number of parallel workers. Passed to
#'   \code{\link{RMitemInfitCutoff}}.
#' @param verbose Logical. Show progress messages (default `FALSE`).
#' @param seed Integer or `NULL`. Master random seed for reproducibility. A
#'   unique per-imputation seed is derived from this value.
#' @param cutoff_method Character string specifying how cutoff intervals are
#'   computed from the stacked distribution. Either `"hdci"` (default) for the
#'   Highest Density Interval via `ggdist::hdci()`, or `"quantile"` for the
#'   2.5th/97.5th percentiles via `stats::quantile()`.
#' @param hdci_width Numeric. Width of the HDCI when `cutoff_method = "hdci"`.
#'   Default is `0.999` (99.9% HDCI). Ignored when
#'   `cutoff_method = "quantile"`.
#'
#' @return A list with the same structure as \code{\link{RMitemInfitCutoff}}, so
#'   that the result can be passed directly to \code{\link{RMitemInfit}},
#'   \code{\link{RMitemInfitMI}}, and \code{\link{RMitemInfitPlot}}:
#' \describe{
#'   \item{`results`}{data.frame with columns `iteration`, `imputation`,
#'     `Item`, `InfitMSQ`, `OutfitMSQ` — the stacked simulation results from
#'     all imputed datasets.}
#'   \item{`item_cutoffs`}{data.frame with per-item cutoff summaries: `Item`,
#'     `infit_low`, `infit_high`, `outfit_low`, `outfit_high`. Computed from
#'     the stacked distribution.}
#'   \item{`actual_iterations`}{Total number of successful iterations across
#'     all imputations.}
#'   \item{`sample_n`}{Number of rows (respondents) per imputed dataset.}
#'   \item{`sample_summary`}{Summary statistics of estimated person parameters
#'     from the first imputed dataset.}
#'   \item{`item_names`}{Character vector of item names.}
#'   \item{`cutoff_method`}{The method used to compute cutoffs.}
#'   \item{`hdci_width`}{The HDCI width used.}
#'   \item{`n_imputations`}{Number of imputed datasets used.}
#'   \item{`iterations_per_imputation`}{Integer vector of requested iterations
#'     per imputed dataset.}
#'   \item{`actual_iterations_per_imputation`}{Integer vector of successful
#'     iterations per imputed dataset.}
#' }
#'
#' @details
#' The function completes each of the `m` imputed datasets via
#' `mice::complete()`, then calls \code{\link{RMitemInfitCutoff}} on each one. The
#' total number of iterations is split approximately evenly across imputations
#' (i.e., each imputed dataset receives `ceiling(iterations / m)` or
#' `floor(iterations / m)` iterations). The per-imputation simulation results
#' are stacked into a single distribution from which cutoff intervals are
#' computed, naturally incorporating imputation uncertainty.
#'
#' Imputed datasets that cause model convergence failures are dropped with a
#' warning. If all imputations fail, the function stops with an error.
#'
#' The `mice` package must be installed (it is in Suggests, not Imports).
#'
#' @seealso \code{\link{RMitemInfitCutoff}}, \code{\link{RMitemInfitMI}},
#'   \code{\link{RMitemInfitPlot}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("mice", quietly = TRUE) &&
#'     requireNamespace("iarm", quietly = TRUE) &&
#'     requireNamespace("ggdist", quietly = TRUE)) {
#'   # Create example data with ~10% MCAR missingness
#'   set.seed(42)
#'   mat <- matrix(sample(0:1, 200 * 8, replace = TRUE), nrow = 200, ncol = 8)
#'   mat[sample(length(mat), round(0.10 * length(mat)))] <- NA
#'   sim_data <- as.data.frame(mat)
#'   colnames(sim_data) <- paste0("Item", 1:8)
#'
#'   # mice's ordinal method (`polr`) requires the items to be ordered
#'   # factors, so code them as such before imputing. RMitemInfitCutoffMI()
#'   # converts the completed factors back to numeric internally.
#'   sim_data[] <- lapply(sim_data, function(x) factor(x, ordered = TRUE))
#'
#'   # Impute (use more imputations, e.g. m = 5+, in real analyses)
#'   imp <- mice::mice(sim_data, m = 2, method = "polr", seed = 123,
#'                     printFlag = FALSE)
#'
#'   # Compute simulation-based cutoffs across imputations
#'   # (use more iterations, e.g. 250+, in real analyses)
#'   cutoff_mi <- RMitemInfitCutoffMI(imp, iterations = 50, parallel = FALSE,
#'                                 seed = 42)
#'   cutoff_mi$item_cutoffs
#'
#'   # Use with RMitemInfitMI()
#'   RMitemInfitMI(imp, cutoff = cutoff_mi)
#' }
#' }
RMitemInfitCutoffMI <- function(
  mids_object,
  iterations = 500,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  cutoff_method = "hdci",
  hdci_width = 0.999
) {
  # --- Check mice -----------------------------------------------------------

  if (!requireNamespace("mice", quietly = TRUE)) {
    stop(
      "Package 'mice' is required for RMitemInfitCutoffMI() but is not installed.\n",
      "Install it with: install.packages(\"mice\")",
      call. = FALSE
    )
  }

  if (!inherits(mids_object, "mids")) {
    stop(
      "`mids_object` must be a 'mids' object as returned by mice::mice().",
      call. = FALSE
    )
  }

  cutoff_method <- match.arg(cutoff_method, c("hdci", "quantile"))

  if (cutoff_method == "hdci" && !requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required when cutoff_method = \"hdci\" but is not installed.\n",
      "Install it with: install.packages(\"ggdist\")\n",
      "Alternatively, use cutoff_method = \"quantile\" to avoid this dependency.",
      call. = FALSE
    )
  }

  m <- mids_object$m

  # --- Distribute iterations across imputations -------------------------------
  base_iter <- iterations %/% m
  remainder <- iterations %% m
  iters_per_imp <- rep(base_iter, m)
  if (remainder > 0L) {
    iters_per_imp[seq_len(remainder)] <- iters_per_imp[seq_len(remainder)] + 1L
  }

  # --- Derive per-imputation seeds -------------------------------------------
  if (!is.null(seed)) {
    set.seed(seed)
  }
  imp_seeds <- sample.int(.Machine$integer.max, m)

  # --- Run RMitemInfitCutoff on each imputed dataset ------------------------------
  all_results <- vector("list", m)
  actual_per_imp <- integer(m)
  sample_summary_first <- NULL
  sample_n <- NULL
  item_names <- NULL
  n_failed <- 0L

  for (i in seq_len(m)) {
    completed_data <- .mi_completed_to_numeric(
      mice::complete(mids_object, action = i)
    )

    if (verbose) {
      message(sprintf(
        "Processing imputation %d/%d (%d iterations)...",
        i,
        m,
        iters_per_imp[i]
      ))
    }

    result_i <- tryCatch(
      {
        RMitemInfitCutoff(
          data = completed_data,
          iterations = iters_per_imp[i],
          parallel = parallel,
          n_cores = n_cores,
          verbose = verbose,
          seed = imp_seeds[i],
          cutoff_method = "quantile",
          hdci_width = hdci_width
        )
      },
      error = function(e) {
        warning(
          sprintf(
            "RMitemInfitCutoff() failed for imputation %d: %s",
            i,
            conditionMessage(e)
          ),
          call. = FALSE
        )
        NULL
      }
    )

    if (is.null(result_i)) {
      n_failed <- n_failed + 1L
      next
    }

    # Tag rows with imputation number
    res_df <- result_i$results
    res_df$imputation <- i
    all_results[[i]] <- res_df
    actual_per_imp[i] <- result_i$actual_iterations

    # Capture metadata from first successful imputation
    if (is.null(item_names)) {
      item_names <- result_i$item_names
      sample_n <- result_i$sample_n
      sample_summary_first <- result_i$sample_summary
    }
  }

  if (n_failed == m) {
    stop(
      "RMitemInfitCutoff() failed for all ",
      m,
      " imputed datasets. ",
      "Check your data.",
      call. = FALSE
    )
  }

  if (n_failed > 0L) {
    warning(
      sprintf(
        "%d of %d imputed datasets failed and were excluded.",
        n_failed,
        m
      ),
      call. = FALSE
    )
  }

  # --- Stack results ----------------------------------------------------------
  stacked_df <- do.call(
    rbind,
    all_results[!vapply(all_results, is.null, logical(1L))]
  )
  rownames(stacked_df) <- NULL

  # Re-number iterations sequentially across imputations
  stacked_df$iteration <- seq_len(nrow(stacked_df) %/% length(item_names))
  # Actually, iteration numbering per row isn't meaningful after stacking;
  # keep original per-imputation iteration numbers and add imputation column.
  # The $iteration column from each imputation is already present; we just

  # leave it as-is.

  total_actual <- sum(actual_per_imp)

  # --- Compute cutoffs from stacked distribution ------------------------------
  unique_items <- unique(stacked_df$Item)
  item_cutoffs <- do.call(
    rbind,
    lapply(unique_items, function(item) {
      sub <- stacked_df[stacked_df$Item == item, ]
      if (cutoff_method == "hdci") {
        infit_interval <- ggdist::hdci(sub$InfitMSQ, .width = hdci_width)
        outfit_interval <- ggdist::hdci(sub$OutfitMSQ, .width = hdci_width)
        data.frame(
          Item = item,
          infit_low = infit_interval[1L, 1L],
          infit_high = infit_interval[1L, 2L],
          outfit_low = outfit_interval[1L, 1L],
          outfit_high = outfit_interval[1L, 2L],
          stringsAsFactors = FALSE,
          row.names = NULL
        )
      } else {
        data.frame(
          Item = item,
          infit_low = stats::quantile(sub$InfitMSQ, 0.025, na.rm = TRUE),
          infit_high = stats::quantile(sub$InfitMSQ, 0.975, na.rm = TRUE),
          outfit_low = stats::quantile(sub$OutfitMSQ, 0.025, na.rm = TRUE),
          outfit_high = stats::quantile(sub$OutfitMSQ, 0.975, na.rm = TRUE),
          stringsAsFactors = FALSE,
          row.names = NULL
        )
      }
    })
  )
  rownames(item_cutoffs) <- NULL

  # --- Return -----------------------------------------------------------------
  list(
    results = stacked_df,
    item_cutoffs = item_cutoffs,
    actual_iterations = total_actual,
    sample_n = sample_n,
    sample_summary = sample_summary_first,
    item_names = item_names,
    cutoff_method = cutoff_method,
    hdci_width = hdci_width,
    n_imputations = m - n_failed,
    iterations_per_imputation = iters_per_imp,
    actual_iterations_per_imputation = actual_per_imp
  )
}

# Internal helpers for input validation across easyRasch2 functions

#' Validate response data for Rasch analysis
#'
#' Checks that the supplied object is a data.frame or matrix, that all values
#' are non-negative integers (or `NA`), and that the minimum non-missing value
#' is 0 (i.e., items are scored starting at 0).
#'
#' @param data A data.frame or matrix of item responses.
#'
#' @return Invisibly returns `TRUE` if validation passes; otherwise stops with
#'   an informative error message.
#'
#' @noRd
validate_response_data <- function(data) {
  if (!is.data.frame(data) && !is.matrix(data)) {
    stop("`data` must be a data.frame or matrix.", call. = FALSE)
  }

  vals <- as.vector(as.matrix(data))
  vals_nonmissing <- vals[!is.na(vals)]

  if (length(vals_nonmissing) == 0L) {
    stop("`data` contains no non-missing values.", call. = FALSE)
  }

  if (any(vals_nonmissing < 0)) {
    stop(
      "`data` contains negative values. Item responses must be non-negative integers.",
      call. = FALSE
    )
  }

  if (!all(vals_nonmissing == floor(vals_nonmissing))) {
    stop(
      "`data` contains non-integer values. Item responses must be integers.",
      call. = FALSE
    )
  }

  min_val <- min(vals_nonmissing)
  if (min_val != 0) {
    stop(
      paste0(
        "The minimum value in `data` is ",
        min_val,
        ", but Rasch analysis ",
        "requires items scored starting at 0. Please recode your data."
      ),
      call. = FALSE
    )
  }

  invisible(TRUE)
}

#' Build the standard estimation-sample-size caption clause
#'
#' Returns `"n = X respondents"`, appending ` of Y` only when respondents were
#' excluded (`X < Y`) and a ` (qualifiers)` parenthetical only when there is
#' something to qualify (both fully conditional, so complete data with no
#' exclusions reads simply `n = X respondents`). No trailing punctuation, so
#' callers append `.` or continue the sentence.
#'
#' @param n_used Number of respondents contributing to the estimate.
#' @param n_total Total respondents supplied (raw input rows).
#' @param qualifiers Character vector of parenthetical notes (e.g.
#'   `"incomplete responses retained"`, `"complete cases"`, `"extreme scores
#'   excluded"`); empty for none. Joined with `"; "`.
#' @param noun Respondent noun (default `"respondents"`; RMpersonFit uses
#'   `"respondents assessed"`).
#' @return A single caption-clause string.
#' @noRd
.n_caption <- function(
  n_used,
  n_total,
  qualifiers = character(),
  noun = "respondents"
) {
  of_total <- if (n_used < n_total) paste0(" of ", n_total) else ""
  paren <- if (length(qualifiers) > 0L) {
    paste0(" (", paste(qualifiers, collapse = "; "), ")")
  } else {
    ""
  }
  paste0("n = ", n_used, of_total, " ", noun, paren)
}

#' Round selected columns of a data.frame for kable display
#'
#' The `output = "dataframe"` contract is *unrounded* values (rounding is a
#' presentation concern); the kable path rounds a display copy just before
#' rendering via this helper. Columns absent from `df` are silently skipped,
#' so one digits spec can serve several table variants (e.g. with and
#' without p-value columns).
#'
#' @param df A data.frame.
#' @param digits Named numeric vector: column name -> decimal places.
#' @return `df` with the named columns rounded.
#' @noRd
.round_display <- function(df, digits) {
  for (nm in intersect(names(digits), names(df))) {
    df[[nm]] <- round(df[[nm]], digits[[nm]])
  }
  df
}

#' Coerce a completed multiply-imputed dataset back to numeric responses
#'
#' `mice::complete()` returns the item columns in their original type. When the
#' items were coded as ordered factors for imputation (required by mice's
#' `method = "polr"`), the completed data are ordered factors, which the
#' downstream CML routines cannot consume. This converts any factor column back
#' to its numeric level values (via `as.character()`, so the 0-based coding is
#' preserved -- not the 1-based integer codes), leaving numeric columns
#' untouched.
#'
#' @param data A completed data.frame from `mice::complete()`.
#' @return `data` with factor columns coerced to numeric.
#' @noRd
.mi_completed_to_numeric <- function(data) {
  factor_cols <- vapply(data, is.factor, logical(1L))
  if (any(factor_cols)) {
    data[factor_cols] <- lapply(
      data[factor_cols],
      function(x) as.numeric(as.character(x))
    )
  }
  data
}

#' Drop respondents with no responses (all-NA rows)
#'
#' A row that is `NA` for every item contributes nothing to any estimate and
#' breaks CML fitting (`psychotools::pcmodel()` errors on all-NA rows). Removes
#' such rows and, matching the package's existing NA-drop messaging (e.g.
#' `RMdifTree`, `RMdifLR`), emits a one-time message when any are dropped.
#'
#' @param data A data.frame or matrix of item responses.
#' @return `data` with all-NA rows removed.
#' @noRd
.drop_empty_respondents <- function(data) {
  empty <- rowSums(!is.na(as.matrix(data))) == 0L
  if (any(empty)) {
    message(sum(empty), " respondent(s) with no responses dropped.")
    data <- data[!empty, , drop = FALSE]
  }
  data
}

#' Validate a `filename` supplied for `output = "file"`
#'
#' Called early (before any model fitting) so a bad path fails fast.
#'
#' @param filename Candidate file path.
#' @return Invisibly `TRUE`; otherwise stops with an informative message.
#' @noRd
.validate_filename <- function(filename) {
  if (
    is.null(filename) ||
      !is.character(filename) ||
      length(filename) != 1L ||
      is.na(filename) ||
      !nzchar(filename)
  ) {
    stop(
      "`filename` must be a single non-empty string when ",
      "`output = \"file\"`.",
      call. = FALSE
    )
  }
  invisible(TRUE)
}

#' Write a result data.frame to CSV for `output = "file"`
#'
#' Shared by the parameter-export functions ([RMitemParameters()],
#' [RMpersonParameters()]). Writes `df` via [utils::write.csv()], emits a
#' one-line confirmation message, and returns `df` invisibly so the value is
#' still available for assignment.
#'
#' @param df Result data.frame to write.
#' @param filename Single non-empty file path (already validated).
#' @param row.names Passed to [utils::write.csv()]: `TRUE` keeps row names
#'   (e.g. respondent identifiers), `FALSE` drops them.
#' @return Invisibly, `df`.
#' @noRd
.write_output_csv <- function(df, filename, row.names = FALSE) {
  .validate_filename(filename)
  utils::write.csv(df, file = filename, row.names = row.names)
  message("Wrote ", nrow(df), " row(s) to '", filename, "'.")
  invisible(df)
}

#' Partial Gamma Local Dependence Analysis
#'
#' Computes partial gamma coefficients for Local Dependence (LD) assessment
#' using \code{iarm::partgam_LD()}. Each pair of items is tested for residual
#' association, controlling for the rest score (total score minus one of the
#' items in the pair).
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Missing values (`NA`) are allowed,
#'   but at least one complete case must exist.
#' @param cutoff Optional. Default `NULL` (no cutoff applied). Can be:
#'   * The return value of \code{\link{RMlocdepGammaCutoff}} (a list with
#'     `$pair_cutoffs`): the data.frame is extracted automatically and
#'     simulation metadata is included in the kable caption.
#'   * The `$pair_cutoffs` data.frame from \code{\link{RMlocdepGammaCutoff}}
#'     directly: must have columns `Item1`, `Item2`, `gamma_low`, `gamma_high`.
#'   When provided, adds columns `Gamma_low`, `Gamma_high`, and `Flagged`
#'   (logical; `TRUE` when the observed partial gamma falls outside the
#'   credible range) to the result.
#' @param p_value Logical. When `TRUE`, adds one-sided bootstrap p-values for
#'   *excess positive* local dependence (`p_gamma`, `padj_gamma`), matching
#'   the `p_value` semantics of \code{\link{RMlocdepQ3}}, and `flagged`
#'   reflects `padj_gamma < alpha` (positive deviations only) instead of the
#'   credible range. One test per item pair: the p-value is computed in the
#'   canonical direction (direction 1, rest score = total - Item2, the
#'   direction that was simulated) and repeated in the direction-2 table for
#'   the same pair. The asymptotic BH-adjusted p-value and star columns from
#'   `iarm::partgam_LD()` are **dropped** in this mode; the simulated
#'   `gamma_low` / `gamma_high` band is kept as the effect-size reference.
#'   Requires the **full** \code{\link{RMlocdepGammaCutoff}} object as
#'   `cutoff` (it carries the simulated distributions in `$results`).
#'   Default `FALSE`.
#' @param correction Character. Multiplicity correction for the bootstrap
#'   p-values, applied over the family of all item pairs (before any
#'   `n_pairs` display filter): `"fwer"` (default; Westfall-Young
#'   studentised-max step-down), `"fdr_bh"`, `"fdr_by"`, or `"none"`. Ignored
#'   when `p_value = FALSE`.
#' @param alpha Numeric in (0, 1). Significance level used to flag pairs on
#'   the corrected p-value. Default `0.05`. Ignored when `p_value = FALSE`.
#' @param output Character string controlling the return value. Either
#'   `"kable"` (default) for a formatted `knitr::kable()` table, or
#'   `"dataframe"` for the underlying data.frame.
#' @param n_pairs Optional positive integer. When supplied, only the
#'   `n_pairs` item pairs with the largest absolute partial-gamma values
#'   (i.e., strongest residual dependence in either direction) are
#'   retained per rest-score direction, sorted by `|gamma|` descending.
#'   When `NULL` (default), all pairs are returned in `iarm`'s native
#'   ordering. Values larger than the total number of pairs are silently
#'   capped at that total.
#'
#' @return
#' * If `output = "kable"`: an object of class `"RMlocdepGamma"`. Internally
#'   a list with two `knitr_kable` elements, `$direction1` and
#'   `$direction2`. In both, the rest score is the total score minus
#'   Item 2 (the second column); the two elements list each item pair in
#'   the two possible orders, so together they cover both rest-score
#'   directions for every pair. Each has columns "Item 1", "Item 2",
#'   "Partial gamma",
#'   "Adj. p-value (BH)", and "p-value sign." (a star-string indicator
#'   from `iarm::partgam_LD()`). When `cutoff` is provided, additional
#'   columns "Gamma low", "Gamma high", and "Flagged" are included.
#'
#'   The object has custom `print()` and `knitr::knit_print()` methods:
#'   in the R console it prints the two tables stacked vertically; in a
#'   Quarto / R Markdown chunk it renders as two distinct pipe tables.
#'   Access the individual tables explicitly as `result$direction1` and
#'   `result$direction2` if needed.
#' * If `output = "dataframe"`: a named list of two data.frames
#'   (`$direction1`, `$direction2`) with columns `Item1`, `Item2`,
#'   `gamma`, `se`, `lower`, `upper` (95% Wald CI), `padj_bh`,
#'   `Significance`. When `cutoff` is provided, columns `gamma_low`,
#'   `gamma_high`, and `flagged` are also included.
#'   With `p_value = TRUE`, `padj_bh` and `Significance` are replaced by
#'   `p_gamma` and `padj_gamma` (identical for a pair in both directions).
#'
#' @details
#' Partial gamma (Christensen, Kreiner & Mesbah, 2013) measures the residual
#' association between pairs of items after controlling for the rest score
#' (total score minus one item). Because it matters which item is subtracted,
#' calculations are done for each pair in both directions, yielding two
#' data.frames.
#'
#' Values near 0 indicate no local dependence. Large positive values suggest
#' positive LD (items share variance beyond the latent trait), while large
#' negative values suggest negative LD.
#'
#' The `iarm` package must be installed (it is in Suggests, not Imports).
#'
#' \strong{Bootstrap p-values.} When `p_value = TRUE`, each pair's observed
#' partial gamma (canonical direction) is compared against its simulated null
#' distribution (from `cutoff$results`, simulated under local independence).
#' The per-pair statistic is the residual studentised by the bootstrap mean
#' and SD; the marginal p-value is the one-sided Monte-Carlo p-value
#' `(1 + #\{t* >= t\}) / (B + 1)` for excess *positive* LD (redundancy, the
#' diagnostic target — matching \code{\link{RMlocdepQ3}}), so it can be no
#' smaller than `1 / (B + 1)`. The band still shows both bounds for
#' reference. `correction = "fwer"` uses the Westfall-Young studentised-max
#' step-down over the family of all pairs, which exploits the bootstrap
#' dependence among them (Ferreira, 2024); it is liberal when the simulation
#' is small, so at least 1000 `iterations` in [RMlocdepGammaCutoff()] are
#' recommended (a warning is issued below that). Unlike the asymptotic
#' p-values from `iarm::partgam_LD()`, these are calibrated against the
#' *simulated Rasch null* rather than the asymptotic SE; they are
#' model-conditional and sample-size-sensitive, and are reported alongside
#' the simulated effect-size band, not in place of it.
#'
#' @inheritSection RMitemInfit Multiple comparisons
#'
#' @references
#' Christensen, K. B., Kreiner, S. & Mesbah, M. (Eds.) (2013).
#' \emph{Rasch Models in Health}, pp. 133--135. ISTE & Wiley.
#' \doi{10.1002/9781118574454}
#'
#' Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number
#' of hypotheses simultaneously. *Journal of Statistical Theory and Practice,
#' 19*(6). \doi{10.1007/s42519-024-00412-4}
#'
#' Westfall, P. H., & Young, S. S. (1993). *Resampling-Based Multiple Testing*.
#' Wiley.
#'
#' @seealso \code{\link{RMlocdepGammaCutoff}}, \code{\link{RMlocdepGammaPlot}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:10)
#'
#'   # Default kable output
#'   RMlocdepGamma(sim_data)
#'
#'   # Return as data.frame list
#'   RMlocdepGamma(sim_data, output = "dataframe")
#'
#'   # Simulation-based cutoffs (slow): 100+ Monte-Carlo iterations
#'   if (requireNamespace("ggdist", quietly = TRUE)) {
#'     cutoff_res <- RMlocdepGammaCutoff(sim_data, iterations = 100,
#'                                       parallel = FALSE, seed = 42)
#'     RMlocdepGamma(sim_data, cutoff = cutoff_res)
#'
#'     # Bootstrap p-values with family-wise (Westfall-Young) correction
#'     # (use iterations >= 1000 in real analyses for stable p-values)
#'     RMlocdepGamma(sim_data, cutoff = cutoff_res, p_value = TRUE,
#'                   output = "dataframe")
#'   }
#' }
#' }
RMlocdepGamma <- function(
  data,
  cutoff = NULL,
  p_value = FALSE,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  alpha = 0.05,
  output = "kable",
  n_pairs = NULL
) {
  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required for RMlocdepGamma() but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }

  output <- match.arg(output, c("kable", "dataframe"))
  correction <- match.arg(correction)
  if (!is.numeric(alpha) || length(alpha) != 1L || alpha <= 0 || alpha >= 1) {
    stop("`alpha` must be a single number in (0, 1).", call. = FALSE)
  }

  # --- Validate n_pairs -------------------------------------------------------
  if (!is.null(n_pairs)) {
    if (
      !is.numeric(n_pairs) ||
        length(n_pairs) != 1L ||
        !is.finite(n_pairs) ||
        n_pairs < 1 ||
        n_pairs != as.integer(n_pairs)
    ) {
      stop(
        "`n_pairs` must be a single positive integer or NULL.",
        call. = FALSE
      )
    }
    n_pairs <- as.integer(n_pairs)
  }

  validate_response_data(data)

  # --- Validate and normalise cutoff ------------------------------------------
  cutoff_n_iter <- NULL
  cutoff_method <- NULL
  cutoff_hdci_width <- NULL
  cutoff_full <- NULL # full object (carries simulated $results for p-values)
  if (!is.null(cutoff)) {
    if (
      is.list(cutoff) &&
        !is.data.frame(cutoff) &&
        "pair_cutoffs" %in% names(cutoff)
    ) {
      cutoff_full <- cutoff
      cutoff_n_iter <- cutoff$actual_iterations
      cutoff_method <- cutoff$cutoff_method
      cutoff_hdci_width <- cutoff$hdci_width
      cutoff <- cutoff$pair_cutoffs
    }
    if (!is.data.frame(cutoff)) {
      stop(
        "`cutoff` must be NULL, the return value of RMlocdepGammaCutoff(), or its ",
        "$pair_cutoffs data.frame.",
        call. = FALSE
      )
    }
    required_cols <- c("Item1", "Item2", "gamma_low", "gamma_high")
    missing_cols <- setdiff(required_cols, names(cutoff))
    if (length(missing_cols) > 0L) {
      stop(
        "`cutoff` data.frame is missing required columns: ",
        paste(missing_cols, collapse = ", "),
        ".",
        call. = FALSE
      )
    }
  }

  # --- p-value prerequisites --------------------------------------------------
  if (p_value) {
    if (is.null(cutoff_full) || is.null(cutoff_full$results)) {
      stop(
        "`p_value = TRUE` requires the full RMlocdepGammaCutoff() object (it ",
        "carries the simulated per-pair distributions in $results); a NULL ",
        "cutoff or the bare $pair_cutoffs data.frame is not sufficient.",
        call. = FALSE
      )
    }
    if (!is.null(cutoff_n_iter) && cutoff_n_iter < 1000L) {
      warning(
        "Bootstrap p-values are based on only ",
        cutoff_n_iter,
        " simulation iterations. With few iterations the studentised-max ",
        "(FWER) correction is liberal and small p-values are imprecise; ",
        "use iterations >= 1000 in RMlocdepGammaCutoff() for reliable ",
        "p-values.",
        call. = FALSE
      )
    }
  }

  # --- rgl workaround ---------------------------------------------------------
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # --- Compute partial gamma LD via iarm --------------------------------------
  sink(nullfile())
  pgam_raw <- iarm::partgam_LD(as.data.frame(data))
  sink()

  # pgam_raw is a list of two data.frames (one per rest-score direction).
  # Columns from iarm::partgam_LD():
  #   1 Item1, 2 Item2, 3 gamma, 4 se, 5 pvalue, 6 padj.BH, 7 sig,
  #   8 lower, 9 upper. The `sig` column is a star string like " ***" /
  #   " **" / " *" / "  ." / "    "; we trim whitespace for display.
  process_pgam_df <- function(raw_df) {
    df <- data.frame(
      Item1 = as.character(raw_df$Item1),
      Item2 = as.character(raw_df$Item2),
      gamma = as.numeric(raw_df$gamma),
      se = as.numeric(raw_df$se),
      lower = as.numeric(raw_df$lower),
      upper = as.numeric(raw_df$upper),
      padj_bh = as.numeric(raw_df[[6]]),
      Significance = trimws(as.character(raw_df[[7]])),
      stringsAsFactors = FALSE
    )
    df
  }

  result_list <- list(
    direction1 = process_pgam_df(pgam_raw[[1]]),
    direction2 = process_pgam_df(pgam_raw[[2]])
  )

  # --- Apply cutoff if provided -----------------------------------------------
  # Cutoffs are keyed by direction-1 pairs (i < j), so for direction 2 (i > j)

  # we must look up by the canonical (sorted) pair order.
  if (!is.null(cutoff)) {
    # Build a canonical key on the cutoff table
    cutoff$canonical_key <- paste(
      pmin(cutoff$Item1, cutoff$Item2),
      pmax(cutoff$Item1, cutoff$Item2),
      sep = "___"
    )
    cutoff_sub <- cutoff[, c("canonical_key", "gamma_low", "gamma_high")]

    for (idx in seq_along(result_list)) {
      result_df <- result_list[[idx]]
      # Build the same canonical key on the result (sorted pair order)
      result_df$canonical_key <- paste(
        pmin(result_df$Item1, result_df$Item2),
        pmax(result_df$Item1, result_df$Item2),
        sep = "___"
      )

      merged <- merge(
        result_df,
        cutoff_sub,
        by = "canonical_key",
        all.x = TRUE,
        sort = FALSE
      )
      # Restore original row order
      merged <- merged[match(result_df$canonical_key, merged$canonical_key), ]
      rownames(merged) <- NULL
      merged$flagged <- !is.na(merged$gamma_low) &
        (merged$gamma < merged$gamma_low | merged$gamma > merged$gamma_high)

      # Remove helper column
      merged$canonical_key <- NULL

      merged <- merged[, c(
        "Item1",
        "Item2",
        "gamma",
        "se",
        "lower",
        "upper",
        "padj_bh",
        "Significance",
        "gamma_low",
        "gamma_high",
        "flagged"
      )]
      result_list[[idx]] <- merged
    }
    # Clean up cutoff helper column
    cutoff$canonical_key <- NULL
  }

  # --- Bootstrap p-values (one test per pair, canonical direction) ------------
  # Computed BEFORE the n_pairs display filter so the multiplicity correction
  # always runs over the full family of pairs. The simulated null holds the
  # direction-1 gammas only, so the observed statistic is the direction-1
  # (canonical) gamma; the resulting p-value is keyed by the unordered pair
  # and repeated in the direction-2 table.
  if (p_value) {
    canon_key <- function(a, b) {
      paste(pmin(a, b), pmax(a, b), sep = "___")
    }
    sim_res <- cutoff_full$results
    sim_key <- canon_key(sim_res$Item1, sim_res$Item2)
    obs_key <- canon_key(
      as.character(pgam_raw[[1L]]$Item1),
      as.character(pgam_raw[[1L]]$Item2)
    )
    if (!setequal(obs_key, unique(sim_key))) {
      stop(
        "Item pairs in the cutoff simulations ($results) do not match the ",
        "pairs in `data`.",
        call. = FALSE
      )
    }
    sim_mat <- tapply(
      sim_res$gamma,
      list(sim_res$iteration, sim_key),
      function(x) x[1L]
    )
    observed <- stats::setNames(
      as.numeric(pgam_raw[[1L]]$gamma),
      obs_key
    )
    # One-sided: excess positive LD (redundancy), matching RMlocdepQ3.
    pv <- .bootstrap_pvalues(
      observed,
      sim_mat,
      correction = correction,
      tail = "upper"
    )
    p_lookup <- stats::setNames(pv$p, pv$name)
    padj_lookup <- stats::setNames(pv$padj, pv$name)

    for (idx in seq_along(result_list)) {
      df <- result_list[[idx]]
      key <- canon_key(df$Item1, df$Item2)
      df$p_gamma <- as.numeric(p_lookup[key])
      df$padj_gamma <- as.numeric(padj_lookup[key])
      df$flagged <- !is.na(df$padj_gamma) & df$padj_gamma < alpha
      # Drop the asymptotic p-value pair; the bootstrap p-values replace it.
      df <- df[, c(
        "Item1",
        "Item2",
        "gamma",
        "se",
        "lower",
        "upper",
        "gamma_low",
        "gamma_high",
        "p_gamma",
        "padj_gamma",
        "flagged"
      )]
      result_list[[idx]] <- df
    }
  }

  # --- Top-N filter by |gamma| per direction ----------------------------------
  total_pairs <- nrow(result_list[[1L]])
  filter_applied <- FALSE
  if (!is.null(n_pairs)) {
    keep_n <- min(n_pairs, total_pairs)
    if (keep_n < total_pairs) {
      filter_applied <- TRUE
      for (idx in seq_along(result_list)) {
        df <- result_list[[idx]]
        ord <- order(abs(df$gamma), decreasing = TRUE)
        df <- df[ord[seq_len(keep_n)], , drop = FALSE]
        rownames(df) <- NULL
        result_list[[idx]] <- df
      }
    }
  }

  # --- Return -----------------------------------------------------------------
  if (output == "dataframe") {
    return(result_list)
  }

  # Kable display rounding (the dataframe output above stays unrounded).
  # The se / lower / upper columns are dataframe-only (added for
  # programmatic use, e.g. the jamovi module); the kable keeps its
  # previous column set.
  ld_digits <- c(
    gamma = 3, padj_bh = 3, gamma_low = 3, gamma_high = 3,
    p_gamma = 4, padj_gamma = 4
  )
  result_list <- lapply(result_list, function(d) {
    .round_display(d[, setdiff(names(d), c("se", "lower", "upper")),
                     drop = FALSE], digits = ld_digits)
  })

  # Build caption
  n_complete <- sum(stats::complete.cases(as.data.frame(data)))
  n_clause <- .n_caption(
    n_complete,
    nrow(as.data.frame(data)),
    if (anyNA(as.data.frame(data))) "complete cases" else character()
  )
  filter_suffix <- if (filter_applied) {
    paste0(" Showing top ", n_pairs, " of ", total_pairs, " pairs by |gamma|.")
  } else {
    ""
  }
  if (p_value) {
    caption_text <- paste0(
      "Partial gamma LD analysis. ",
      n_clause,
      ". One-sided bootstrap p-values for excess positive LD from ",
      cutoff_n_iter,
      " iterations, computed in the canonical direction (rest score = ",
      "total - Item2) and repeated for both directions (replacing the ",
      "asymptotic BH p-values); multiplicity correction: ",
      .correction_label(correction),
      "; flagged at padj < ",
      alpha,
      ". p-values cannot be smaller than 1/(",
      cutoff_n_iter,
      "+1) = ",
      round(1 / (cutoff_n_iter + 1), 4),
      ".",
      filter_suffix
    )
  } else if (is.null(cutoff)) {
    caption_text <- paste0(
      "Partial gamma LD analysis. ",
      n_clause,
      ". Positive gamma indicates positive local dependence between items.",
      filter_suffix
    )
  } else if (!is.null(cutoff_n_iter)) {
    method_label <- .format_gamma_cutoff_method_label(
      cutoff_method,
      cutoff_hdci_width
    )
    iter_part <- paste0(cutoff_n_iter, " simulation iterations")
    caption_text <- paste0(
      "Partial gamma LD analysis. ",
      n_clause,
      ". Cutoff values based on ",
      if (!is.null(method_label)) {
        paste0(iter_part, " (", method_label, ").")
      } else {
        paste0(iter_part, ".")
      },
      filter_suffix
    )
  } else {
    caption_text <- paste0(
      "Partial gamma LD analysis. ",
      n_clause,
      ". Simulation-based cutoff values applied.",
      filter_suffix
    )
  }

  col_names_no_cutoff <- c(
    "Item 1",
    "Item 2",
    "Partial gamma",
    "Adj. p-value (BH)",
    "p-value sign."
  )
  col_names_cutoff <- c(
    "Item 1",
    "Item 2",
    "Partial gamma",
    "Adj. p-value (BH)",
    "p-value sign.",
    "Gamma low",
    "Gamma high",
    "Flagged"
  )
  col_names_pvalue <- c(
    "Item 1",
    "Item 2",
    "Partial gamma",
    "Gamma low",
    "Gamma high",
    "p",
    "p (adj)",
    "Flagged"
  )
  col_names_used <- if (p_value) {
    col_names_pvalue
  } else if (is.null(cutoff)) {
    col_names_no_cutoff
  } else {
    col_names_cutoff
  }

  kable1 <- knitr::kable(
    result_list$direction1,
    format = "pipe",
    col.names = col_names_used,
    caption = paste0(caption_text, " Direction 1: rest score = total - Item2.")
  )

  kable2 <- knitr::kable(
    result_list$direction2,
    format = "pipe",
    col.names = col_names_used,
    caption = paste0(
      caption_text,
      " Direction 2: rest score = total - Item2 ",
      "(item pairs shown in the reverse order to ",
      "direction 1)."
    )
  )

  # `knitr::kable(format = "pipe")` returns a multi-line *character vector*
  # (one element per line), so `paste(kable1, "\n\n", kable2)` would do
  # element-wise concatenation, interleaving the two tables row-by-row.
  # Collapse each table to a single string first, then join with a
  # blank line between to render as two distinct kable tables.
  combined <- paste(
    paste(kable1, collapse = "\n"),
    paste(kable2, collapse = "\n"),
    sep = "\n\n"
  )

  # Return a custom-class list so the two display contexts (R console and
  # knitr chunk) can be handled separately. See print.RMlocdepGamma() and
  # knit_print.RMlocdepGamma() below.
  out <- list(
    direction1 = kable1,
    direction2 = kable2,
    .combined = combined
  )
  class(out) <- c("RMlocdepGamma", "list")
  out
}

#' Print method for RMlocdepGamma kable output
#'
#' Prints the two rest-score direction tables stacked vertically with a
#' blank line between them. Each table renders via `knitr_kable`'s own
#' print method as a clean pipe-markdown table.
#'
#' @param x An object of class `"RMlocdepGamma"` returned by
#'   \code{\link{RMlocdepGamma}} with `output = "kable"`.
#' @param ... Further arguments (currently unused).
#' @return Invisibly returns `x`.
#' @keywords internal
#' @exportS3Method base::print
print.RMlocdepGamma <- function(x, ...) {
  print(x$direction1)
  cat("\n")
  print(x$direction2)
  invisible(x)
}

#' knitr knit_print method for RMlocdepGamma kable output
#'
#' Inside a knitr / Quarto / R Markdown chunk, returns the pre-combined
#' two-table asis string so pandoc renders them as two distinct pipe
#' tables. Outside knitr, R's normal dispatch falls back to
#' `print.RMlocdepGamma()`.
#'
#' @param x An object of class `"RMlocdepGamma"`.
#' @param ... Further arguments passed to `knitr::asis_output()`.
#' @return A `knit_asis` character object.
#' @keywords internal
#' @exportS3Method knitr::knit_print
knit_print.RMlocdepGamma <- function(x, ...) {
  knitr::asis_output(x$.combined)
}

#' Simulation-Based Partial Gamma LD Cutoff Determination
#'
#' Uses parametric bootstrap simulation to determine appropriate cutoff values
#' for partial gamma Local Dependence analysis via
#' \code{\link[iarm]{partgam_LD}}. Under a correctly fitting Rasch model where
#' items are locally independent, this function generates the expected
#' distribution of partial gamma values per item pair, providing empirical
#' critical values.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Only complete cases (rows without
#'   any `NA`) are used.
#' @param iterations Integer. Number of simulation iterations (default 250).
#' @param parallel Logical. Use parallel processing via `mirai` if available
#'   (default `TRUE`).
#' @param n_cores Integer or `NULL`. Number of parallel workers. When `NULL`,
#'   `getOption("mc.cores")` is checked first. If neither is set and
#'   `parallel = TRUE`, a warning is issued and execution falls back to
#'   sequential (single core) processing.
#' @param verbose Logical. Show a progress bar (default `FALSE`).
#' @param seed Integer or `NULL`. Random seed for reproducibility.
#' @param cutoff_method Character string specifying how cutoff intervals are
#'   computed. Either `"hdci"` (default) for the Highest Density Interval via
#'   `ggdist::hdci()`, or `"quantile"` for the 2.5th/97.5th percentiles via
#'   `stats::quantile()`.
#' @param hdci_width Numeric. Width of the HDCI when `cutoff_method = "hdci"`.
#'   Default is `0.99` (99\% HDCI). Ignored when
#'   `cutoff_method = "quantile"`.
#'
#' @return A list with components:
#' \describe{
#'   \item{`results`}{data.frame with columns `iteration`, `Item1`, `Item2`,
#'     and `gamma` (one row per item pair per successful iteration). Contains
#'     results from direction 1 only (rest score = total - Item2), which is
#'     the conventional direction.}
#'   \item{`pair_cutoffs`}{data.frame with per-pair cutoff summaries: `Item1`,
#'     `Item2`, `gamma_low`, `gamma_high`. Bounds are computed using the method
#'     specified by `cutoff_method`.}
#'   \item{`actual_iterations`}{Number of successful iterations.}
#'   \item{`sample_n`}{Number of complete cases used.}
#'   \item{`sample_n_total`}{Number of respondents in the raw input data,
#'     before the complete-case filter.}
#'   \item{`sample_has_na`}{Logical. Whether the raw input data contained
#'     any missing values.}
#'   \item{`sample_summary`}{Summary statistics of estimated person
#'     parameters.}
#'   \item{`item_names`}{Character vector of item names from data.}
#'   \item{`cutoff_method`}{The method used to compute cutoffs (`"hdci"` or
#'     `"quantile"`).}
#'   \item{`hdci_width`}{The HDCI width used (only meaningful when
#'     `cutoff_method = "hdci"`).}
#' }
#'
#' @details
#' For each simulation iteration the function:
#' \enumerate{
#'   \item Resamples person parameters (thetas) with replacement from the
#'     WLE person locations.
#'   \item Simulates item response data under a Rasch model (dichotomous via
#'     `psychotools::rrm()` or polytomous via an internal partial credit
#'     simulator).
#'   \item Computes partial gamma LD statistics via
#'     `iarm::partgam_LD()`.
#' }
#'
#' Because the data are simulated under the Rasch model, items are locally
#' independent by construction. The distribution of partial gamma values
#' across iterations provides empirical critical values per item pair. Values
#' from real data that fall outside these bounds suggest local dependence that
#' exceeds what would be expected by chance. Failed iterations (e.g., due to
#' convergence issues or degenerate data) are silently discarded.
#'
#' The generating model uses CML item thresholds via `psychotools::pcmodel()`
#' (a dichotomous item is a 2-category PCM) and WLE person locations,
#' consistent with the rest of the package; responses are simulated with
#' `psychotools::rrm()` (dichotomous) or an internal partial credit score
#' simulator (polytomous).
#'
#' Parallel processing is provided by the `mirai` package (optional). Install
#' it with `install.packages("mirai")` to enable parallelisation.
#'
#' The `iarm` package must be installed (it is in Suggests, not Imports).
#'
#' @references
#' Christensen, K. B., Kreiner, S. & Mesbah, M. (Eds.) (2013).
#' \emph{Rasch Models in Health}, pp. 133--135. ISTE & Wiley.
#' \doi{10.1002/9781118574454}
#'
#' @seealso \code{\link[iarm]{partgam_LD}}, \code{\link{RMlocdepGamma}},
#'   \code{\link{RMlocdepGammaPlot}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE) &&
#'     requireNamespace("ggdist", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:10)
#'
#'   # Run 100 iterations sequentially for a quick demo
#'   cutoff_res <- RMlocdepGammaCutoff(sim_data, iterations = 100,
#'                                     parallel = FALSE, seed = 42)
#'   cutoff_res$pair_cutoffs
#' }
#' }
RMlocdepGammaCutoff <- function(
  data,
  iterations = 250,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  cutoff_method = "hdci",
  hdci_width = 0.99
) {
  cutoff_method <- match.arg(cutoff_method, c("hdci", "quantile"))

  if (cutoff_method == "hdci" && !requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required when cutoff_method = \"hdci\" but is not installed.\n",
      "Install it with: install.packages(\"ggdist\")\n",
      "Alternatively, use cutoff_method = \"quantile\" to avoid this dependency.",
      call. = FALSE
    )
  }

  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required for RMlocdepGammaCutoff() but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }

  validate_response_data(data)

  # rgl workaround (iarm depends on vcdExtra -> rgl)
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # Only complete cases. Record the raw total and whether anything was
  # dropped so callers (e.g. RMlocdepGammaPlot) can report the sample in the
  # standard `n = X of Y respondents` form.
  n_total <- nrow(data)
  has_na <- anyNA(data)
  complete_idx <- stats::complete.cases(data)
  data <- data[complete_idx, , drop = FALSE]

  if (nrow(data) == 0L) {
    stop(
      "No complete cases in data after removing rows with NA.",
      call. = FALSE
    )
  }

  # --- Parallel setup ---------------------------------------------------------
  use_parallel <- parallel && requireNamespace("mirai", quietly = TRUE)

  if (parallel && !use_parallel) {
    message(
      "Install 'mirai' package for parallel processing: install.packages(\"mirai\")"
    )
    message("Running sequentially...")
  }

  if (use_parallel) {
    if (is.null(n_cores)) {
      n_cores <- getOption("mc.cores")
    }
    if (is.null(n_cores)) {
      warning(
        paste0(
          "For parallel processing, specify n_cores or set options(mc.cores = N).\n",
          "(Use `parallel::detectCores()` to see how many cores are available.)\n",
          "Falling back to sequential (single core) processing."
        ),
        call. = FALSE
      )
      use_parallel <- FALSE
    } else {
      n_cores <- min(n_cores, iterations)
    }
  }

  if (!is.null(seed)) {
    set.seed(seed)
  }

  # Generate per-iteration seeds
  sim_seeds <- sample.int(.Machine$integer.max, iterations)

  data_mat <- as.matrix(data)
  sample_n <- nrow(data_mat)
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L

  item_names_vec <- colnames(data_mat)

  # Generating model: CML item thresholds (psychotools) + WLE person locations,
  # consistent with the rest of the package, replacing eRm CML +
  # eRm::person.parameter() (MLE). Thetas form the pool resampled with
  # replacement to build each simulated dataset; a dichotomous item is a
  # 2-category PCM, so its centred threshold is the item difficulty for rrm().
  pool <- .wle_theta_pool(data_mat)
  thr_list <- pool$thr_list
  thetas <- pool$thetas

  if (is_polytomous) {
    sim_data_list <- list(
      type = "polytomous",
      thetas = thetas,
      deltaslist = thr_list,
      n_items = ncol(data_mat),
      sample_n = sample_n,
      item_names = item_names_vec
    )
  } else {
    sim_data_list <- list(
      type = "dichotomous",
      thetas = thetas,
      item_params = unlist(thr_list, use.names = FALSE),
      n_items = ncol(data_mat),
      sample_n = sample_n,
      item_names = item_names_vec
    )
  }

  if (use_parallel) {
    results_raw <- run_partgam_LD_sim_parallel(
      iterations,
      sim_seeds,
      sim_data_list,
      n_cores,
      verbose
    )
  } else {
    results_raw <- run_partgam_LD_sim_sequential(
      iterations,
      sim_seeds,
      sim_data_list,
      verbose
    )
  }

  # Filter out failures (character strings indicate errors)
  ok <- vapply(results_raw, is.data.frame, logical(1L))
  successful <- results_raw[ok]

  if (length(successful) == 0L) {
    stop("All simulation iterations failed. Check your data.", call. = FALSE)
  }

  actual_iterations <- length(successful)

  # Combine per-iteration data.frames
  iter_dfs <- lapply(seq_along(successful), function(i) {
    df <- successful[[i]]
    df$iteration <- i
    df
  })
  results_df <- do.call(rbind, iter_dfs)
  rownames(results_df) <- NULL

  # Compute per-pair cutoffs
  pair_keys <- unique(paste(results_df$Item1, results_df$Item2, sep = "___"))
  pair_cutoffs <- do.call(
    rbind,
    lapply(pair_keys, function(pk) {
      parts <- strsplit(pk, "___", fixed = TRUE)[[1]]
      sub <- results_df[
        results_df$Item1 == parts[1] & results_df$Item2 == parts[2],
      ]
      if (cutoff_method == "hdci") {
        gamma_interval <- ggdist::hdci(sub$gamma, .width = hdci_width)
        data.frame(
          Item1 = parts[1],
          Item2 = parts[2],
          gamma_low = gamma_interval[1L, 1L],
          gamma_high = gamma_interval[1L, 2L],
          stringsAsFactors = FALSE,
          row.names = NULL
        )
      } else {
        data.frame(
          Item1 = parts[1],
          Item2 = parts[2],
          gamma_low = stats::quantile(sub$gamma, 0.025, na.rm = TRUE),
          gamma_high = stats::quantile(sub$gamma, 0.975, na.rm = TRUE),
          stringsAsFactors = FALSE,
          row.names = NULL
        )
      }
    })
  )
  rownames(pair_cutoffs) <- NULL

  list(
    results = results_df,
    pair_cutoffs = pair_cutoffs,
    actual_iterations = actual_iterations,
    sample_n = sample_n,
    sample_n_total = n_total,
    sample_has_na = has_na,
    sample_summary = summary(thetas),
    item_names = item_names_vec,
    cutoff_method = cutoff_method,
    hdci_width = hdci_width
  )
}

# ---------------------------------------------------------------------------
# Internal: single simulation iteration
# ---------------------------------------------------------------------------

#' Run a single partial gamma LD simulation iteration
#'
#' @param seed Integer seed for reproducibility.
#' @param data_list List produced inside [RMlocdepGammaCutoff()].
#' @return A data.frame with columns `Item1`, `Item2`, and `gamma`, or a
#'   character string on failure.
#' @keywords internal
run_single_partgam_LD_sim <- function(seed, data_list) {
  set.seed(seed)

  thetas_res <- sample(
    data_list$thetas,
    size = data_list$sample_n,
    replace = TRUE
  )

  tryCatch(
    {
      if (data_list$type == "dichotomous") {
        sim_mat <- psychotools::rrm(
          theta = thetas_res,
          beta = data_list$item_params
        )
        sim_df <- as.data.frame(sim_mat$data)
        colnames(sim_df) <- data_list$item_names

        pos_counts <- colSums(sim_df, na.rm = TRUE)
        if (any(pos_counts < 8L)) {
          return(
            "validation_failed: fewer than 8 positive responses in at least one item"
          )
        }
      } else {
        sim_mat <- sim_partial_score(data_list$deltaslist, thetas_res)
        sim_df <- as.data.frame(sim_mat)
        colnames(sim_df) <- data_list$item_names

        n_cats <- vapply(
          data_list$deltaslist,
          function(d) length(d) + 1L,
          integer(1L)
        )
        for (j in seq_len(ncol(sim_df))) {
          tab <- tabulate(sim_df[[j]] + 1L, nbins = n_cats[j])
          if (any(tab == 0L)) {
            return("validation_failed: not all categories represented")
          }
        }
      }

      # Compute partial gamma LD via iarm.
      # iarm::partgam_LD() prints its result tables to stdout on every call;
      # silence it. `finally` restores the sink even if the call errors (the
      # outer tryCatch then reports the failure as usual).
      sink(nullfile())
      pgam <- tryCatch(
        iarm::partgam_LD(sim_df),
        finally = sink()
      )

      # Use direction 1 (rest score = total - Item2) as the canonical direction
      pgam_df <- as.data.frame(pgam[[1]])

      data.frame(
        Item1 = as.character(pgam_df$Item1),
        Item2 = as.character(pgam_df$Item2),
        gamma = as.numeric(pgam_df$gamma),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    },
    error = function(e) {
      as.character(conditionMessage(e))
    }
  )
}

# ---------------------------------------------------------------------------
# Internal: parallel runner
# ---------------------------------------------------------------------------

#' Run partial gamma LD simulations in parallel using mirai
#'
#' @param iterations Number of iterations.
#' @param sim_seeds Integer vector of per-iteration seeds.
#' @param sim_data_list List of data passed to each worker.
#' @param n_cores Number of mirai daemons.
#' @param verbose Show progress bar.
#' @return List of raw results (one element per iteration).
#' @keywords internal
run_partgam_LD_sim_parallel <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  n_cores,
  verbose = FALSE
) {
  mirai::daemons(n_cores)
  on.exit(mirai::daemons(0), add = TRUE)

  if (verbose) {
    message(sprintf("Starting %d daemons...", n_cores))
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
    completed <- 0L
  }

  # Submit all tasks
  tasks <- lapply(seq_len(iterations), function(sim) {
    mirai::mirai(
      {
        run_single_partgam_LD_sim(seed, data_list)
      },
      seed = sim_seeds[sim],
      data_list = sim_data_list,
      run_single_partgam_LD_sim = run_single_partgam_LD_sim,
      sim_partial_score = sim_partial_score,
      sim_poly_item = sim_poly_item
    )
  })

  # Collect results
  results <- vector("list", iterations)
  for (sim in seq_len(iterations)) {
    result <- mirai::call_mirai(tasks[[sim]])$data
    if (!inherits(result, "errorValue")) {
      results[[sim]] <- result
    } else {
      results[[sim]] <- "mirai_error"
    }
    if (verbose) {
      completed <- completed + 1L
      utils::setTxtProgressBar(pb, completed)
    }
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# ---------------------------------------------------------------------------
# Internal: sequential runner
# ---------------------------------------------------------------------------

#' Run partial gamma LD simulations sequentially
#'
#' @param iterations Number of iterations.
#' @param sim_seeds Integer vector of per-iteration seeds.
#' @param sim_data_list List of data passed to each worker.
#' @param verbose Show progress bar.
#' @return List of raw results (one element per iteration).
#' @keywords internal
run_partgam_LD_sim_sequential <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  verbose = FALSE
) {
  if (verbose) {
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  results <- vector("list", iterations)
  for (sim in seq_len(iterations)) {
    results[[sim]] <- run_single_partgam_LD_sim(sim_seeds[sim], sim_data_list)
    if (verbose) {
      utils::setTxtProgressBar(pb, sim)
    }
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

#' Plot Distribution of Simulated Partial Gamma LD Values
#'
#' Visualises the distribution of simulation-based partial gamma LD values
#' from \code{\link{RMlocdepGammaCutoff}}, optionally overlaying observed partial
#' gamma values computed from real data via \code{\link[iarm]{partgam_LD}}.
#'
#' Uses `ggdist::stat_dotsinterval()` (when `data` is not supplied) or
#' `ggdist::stat_dots()` (when `data` is supplied) with
#' `point_interval = "median_hdci"` and `.width = c(0.66, 0.95, 0.99)`.
#'
#' @param simfit The return value of \code{\link{RMlocdepGammaCutoff}} (a list with
#'   components `results`, `pair_cutoffs`, `actual_iterations`, `sample_n`, and
#'   `item_names`).
#' @param data Optional. A data.frame or matrix of item responses for computing
#'   and overlaying observed partial gamma values. Items must be scored starting
#'   at 0 (non-negative integers). When provided, the plot includes orange
#'   diamond markers for the observed partial gamma alongside the simulated
#'   distribution, plus segment summaries from the cutoff intervals.
#' @param items Optional character vector of item names to include in the plot.
#'   Only item pairs where **both** items are in this vector will be shown. When
#'   `NULL` (default), all item pairs are plotted.
#' @param n_pairs Optional positive integer. When supplied, only the
#'   `n_pairs` item pairs with the largest absolute partial gamma values
#'   are plotted, sorted by `|gamma|` descending. When `data` is
#'   supplied, the ranking uses the *observed* partial gammas (the
#'   diamonds you actually want to interpret); otherwise it falls back
#'   to the per-pair median of the simulated distributions. Applied
#'   *after* the `items` filter when both are supplied. Values larger
#'   than the number of available pairs are silently capped.
#'
#' @return A `ggplot` object.
#'
#' @details
#' The plot shows one row per item pair (labelled as "Item1 - Item2"). Only
#' direction 1 (rest score = total - Item2) is plotted, matching the
#' convention used in the simulation.
#'
#' When `data` is **not** supplied, the function plots the simulated partial
#' gamma distributions as dot-interval plots using
#' `ggdist::stat_dotsinterval()` with median and Highest Density Continuous
#' Interval (HDCI) summaries.
#'
#' When `data` **is** supplied, the function:
#' \enumerate{
#'   \item Computes observed partial gamma values via
#'     `iarm::partgam_LD()`.
#'   \item Overlays observed gamma values as orange diamond markers on the
#'     simulated distributions.
#'   \item Shows per-pair cutoff intervals (from `simfit$pair_cutoffs`) as
#'     black line segments, with thicker segments for the 66\% interval and
#'     black dots for the median.
#' }
#'
#' The `ggplot2`, `ggdist`, and optionally `iarm` packages must be installed
#' (they are in Suggests, not Imports).
#'
#' @seealso \code{\link{RMlocdepGammaCutoff}}, \code{\link{RMlocdepGamma}}
#'
#' @importFrom rlang .data
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE) &&
#'     requireNamespace("ggdist", quietly = TRUE) &&
#'     requireNamespace("ggplot2", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:10)
#'
#'   # Run simulation
#'   cutoff_res <- RMlocdepGammaCutoff(sim_data, iterations = 100,
#'                                     parallel = FALSE, seed = 42)
#'
#'   # Simulated distribution only
#'   RMlocdepGammaPlot(cutoff_res)
#'
#'   # With observed partial gamma overlaid
#'   RMlocdepGammaPlot(cutoff_res, data = sim_data)
#'
#'   # Plot only a subset of items
#'   RMlocdepGammaPlot(cutoff_res, data = sim_data,
#'                     items = c("Item1", "Item2", "Item3"))
#' }
#' }
RMlocdepGammaPlot <- function(simfit, data, items = NULL, n_pairs = NULL) {
  # --- Check required packages ------------------------------------------------
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for RMlocdepGammaPlot() but is not installed.\n",
      "Install it with: install.packages(\"ggplot2\")",
      call. = FALSE
    )
  }
  if (!requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required for RMlocdepGammaPlot() but is not installed.\n",
      "Install it with: install.packages(\"ggdist\")",
      call. = FALSE
    )
  }

  # --- Validate simfit --------------------------------------------------------
  required_names <- c(
    "results",
    "pair_cutoffs",
    "actual_iterations",
    "sample_n",
    "item_names"
  )
  missing_names <- setdiff(required_names, names(simfit))
  if (length(missing_names) > 0L) {
    stop(
      "`simfit` is missing required components: ",
      paste(missing_names, collapse = ", "),
      ".\nExpected the return value of RMlocdepGammaCutoff().",
      call. = FALSE
    )
  }

  results_df <- simfit$results
  pair_cutoffs <- simfit$pair_cutoffs
  actual_iterations <- simfit$actual_iterations
  sample_n <- simfit$sample_n
  item_names <- simfit$item_names

  # Standard sample-size clause for the (complete-case) simulation sample.
  # `sample_n_total` / `sample_has_na` are absent in cutoff objects made by
  # older versions, so fall back to the plain count.
  sample_clause <- .n_caption(
    sample_n,
    if (is.null(simfit$sample_n_total)) sample_n else simfit$sample_n_total,
    if (isTRUE(simfit$sample_has_na)) "complete cases" else character()
  )

  # --- Validate items parameter -----------------------------------------------
  if (!is.null(items)) {
    unknown_items <- setdiff(items, item_names)
    if (length(unknown_items) > 0L) {
      stop(
        "Unknown item(s) in `items`: ",
        paste(unknown_items, collapse = ", "),
        ".\nAvailable items: ",
        paste(item_names, collapse = ", "),
        call. = FALSE
      )
    }
    if (length(items) < 2L) {
      stop(
        "`items` must contain at least 2 item names to form a pair.",
        call. = FALSE
      )
    }
  }

  # --- Validate n_pairs parameter ---------------------------------------------
  if (!is.null(n_pairs)) {
    if (
      !is.numeric(n_pairs) ||
        length(n_pairs) != 1L ||
        !is.finite(n_pairs) ||
        n_pairs < 1 ||
        n_pairs != as.integer(n_pairs)
    ) {
      stop(
        "`n_pairs` must be a single positive integer or NULL.",
        call. = FALSE
      )
    }
    n_pairs <- as.integer(n_pairs)
  }

  # Create pair labels
  results_df$Pair <- paste(results_df$Item1, "-", results_df$Item2)
  pair_cutoffs$Pair <- paste(pair_cutoffs$Item1, "-", pair_cutoffs$Item2)

  # --- Filter to selected items -----------------------------------------------
  if (!is.null(items)) {
    keep <- results_df$Item1 %in% items & results_df$Item2 %in% items
    results_df <- results_df[keep, , drop = FALSE]
    keep_cut <- pair_cutoffs$Item1 %in% items & pair_cutoffs$Item2 %in% items
    pair_cutoffs <- pair_cutoffs[keep_cut, , drop = FALSE]

    if (nrow(results_df) == 0L) {
      stop("No item pairs remain after filtering by `items`.", call. = FALSE)
    }
  }

  # --- Compute observed partial gammas up-front (when data supplied) ----------
  # We pre-compute here so the `n_pairs` filter below can rank by observed
  # |gamma| when data are available. The result is reused in Case 2.
  observed_df <- NULL
  if (!missing(data)) {
    if (!requireNamespace("iarm", quietly = TRUE)) {
      stop(
        "Package 'iarm' is required to compute observed partial gamma but is not installed.\n",
        "Install it with: install.packages(\"iarm\")",
        call. = FALSE
      )
    }
    validate_response_data(data)

    old_rgl <- getOption("rgl.useNULL")
    options(rgl.useNULL = TRUE)
    on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

    sink(nullfile())
    pgam_raw <- iarm::partgam_LD(as.data.frame(data))
    sink()

    observed_df <- data.frame(
      Item1 = as.character(pgam_raw[[1L]]$Item1),
      Item2 = as.character(pgam_raw[[1L]]$Item2),
      observed_gamma = as.numeric(pgam_raw[[1L]]$gamma),
      stringsAsFactors = FALSE
    )
    observed_df$Pair <- paste(observed_df$Item1, "-", observed_df$Item2)

    if (!is.null(items)) {
      keep_obs <- observed_df$Item1 %in% items & observed_df$Item2 %in% items
      observed_df <- observed_df[keep_obs, , drop = FALSE]
    }
  }

  # --- Apply n_pairs top-N filter ---------------------------------------------
  if (!is.null(n_pairs)) {
    if (!is.null(observed_df)) {
      # Rank by |observed gamma| — the diamonds the user wants to interpret
      ord <- order(abs(observed_df$observed_gamma), decreasing = TRUE)
      keep_n <- min(n_pairs, nrow(observed_df))
      keep_pairs <- observed_df$Pair[ord[seq_len(keep_n)]]
      observed_df <- observed_df[
        observed_df$Pair %in% keep_pairs,
        ,
        drop = FALSE
      ]
    } else {
      # Rank by |median simulated gamma| per pair
      pair_names_all <- unique(results_df$Pair)
      med_g <- vapply(
        pair_names_all,
        function(pp) {
          stats::median(results_df$gamma[results_df$Pair == pp], na.rm = TRUE)
        },
        numeric(1L)
      )
      ord <- order(abs(med_g), decreasing = TRUE)
      keep_n <- min(n_pairs, length(pair_names_all))
      keep_pairs <- pair_names_all[ord[seq_len(keep_n)]]
    }
    results_df <- results_df[results_df$Pair %in% keep_pairs, , drop = FALSE]
    pair_cutoffs <- pair_cutoffs[
      pair_cutoffs$Pair %in% keep_pairs,
      ,
      drop = FALSE
    ]
    # Use the ranked order for y-axis: largest |gamma| at the top
    pair_levels <- rev(keep_pairs)
  } else {
    pair_levels <- rev(unique(results_df$Pair))
  }

  # --- Compute per-pair summary intervals for segment overlays ----------------
  pair_names <- unique(results_df$Pair)
  lo_hi <- do.call(
    rbind,
    lapply(pair_names, function(pair) {
      sub <- results_df[results_df$Pair == pair, ]
      data.frame(
        Pair = pair,
        min_gamma = stats::quantile(sub$gamma, 0.005, na.rm = TRUE),
        max_gamma = stats::quantile(sub$gamma, 0.995, na.rm = TRUE),
        p66lo_gamma = stats::quantile(sub$gamma, 0.167, na.rm = TRUE),
        p66hi_gamma = stats::quantile(sub$gamma, 0.833, na.rm = TRUE),
        median_gamma = stats::median(sub$gamma, na.rm = TRUE),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    })
  )
  rownames(lo_hi) <- NULL

  # --- Case 1: no observed data, show simulation distribution only ------------
  if (missing(data)) {
    results_plot <- data.frame(
      Pair = results_df$Pair,
      Value = results_df$gamma,
      stringsAsFactors = FALSE
    )
    results_plot$Pair <- factor(results_plot$Pair, levels = pair_levels)

    p <- ggplot2::ggplot(
      results_plot,
      ggplot2::aes(
        x = .data$Value,
        y = .data$Pair
      )
    ) +
      ggdist::stat_dotsinterval(
        ggplot2::aes(slab_fill = ggplot2::after_stat(.data$level)),
        quantiles = actual_iterations,
        point_interval = "median_hdci",
        layout = "weave",
        slab_color = NA,
        .width = c(0.66, 0.95, 0.99)
      ) +
      ggplot2::geom_vline(
        xintercept = 0,
        linetype = "dashed",
        color = "grey50",
        linewidth = 0.4
      ) +
      ggplot2::labs(
        x = "Partial gamma",
        y = "Item pair",
        caption = er2_caption(paste0(
          "Results from ",
          actual_iterations,
          " simulated datasets (no true local dependence). ",
          sample_clause,
          " per dataset."
        ))
      ) +
      ggplot2::scale_color_manual(
        values = scales::brewer_pal()(3),
        aesthetics = "slab_fill",
        guide = "none"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(panel.spacing = ggplot2::unit(0.7, "cm")) +
      er2_axis_margins() +
      er2_plot_caption()

    return(p)
  }

  # --- Case 2: observed data supplied -----------------------------------------
  # (`observed_df` was pre-computed above so the n_pairs filter could rank
  # by |observed gamma|.)

  # --- Build plot data --------------------------------------------------------
  gamma_sim <- data.frame(
    Pair = results_df$Pair,
    Value = results_df$gamma,
    stringsAsFactors = FALSE
  )
  gamma_sim <- merge(
    gamma_sim,
    observed_df[, c("Pair", "observed_gamma")],
    by = "Pair",
    sort = FALSE
  )
  gamma_sim$Pair <- factor(gamma_sim$Pair, levels = pair_levels)

  lo_hi$Pair_f <- factor(lo_hi$Pair, levels = pair_levels)

  caption_text <- er2_caption(paste0(
    "Results from ",
    actual_iterations,
    " simulated datasets. ",
    sample_clause,
    " per dataset.\n",
    "Orange diamonds indicate observed partial gamma LD. ",
    "Black dots indicate median gamma from simulations."
  ))

  p <- ggplot2::ggplot(
    gamma_sim,
    ggplot2::aes(
      x = .data$Value,
      y = .data$Pair
    )
  ) +
    ggdist::stat_dots(
      ggplot2::aes(slab_fill = ggplot2::after_stat(.data$level)),
      quantiles = actual_iterations,
      layout = "weave",
      slab_color = NA,
      .width = c(0.66, 0.95, 0.99)
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$min_gamma,
        xend = .data$max_gamma,
        y = .data$Pair_f,
        yend = .data$Pair_f
      ),
      color = "black",
      linewidth = 0.7
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$p66lo_gamma,
        xend = .data$p66hi_gamma,
        y = .data$Pair_f,
        yend = .data$Pair_f
      ),
      color = "black",
      linewidth = 1.2
    ) +
    ggplot2::geom_point(
      data = lo_hi,
      ggplot2::aes(
        x = .data$median_gamma,
        y = .data$Pair_f
      ),
      size = 3.6
    ) +
    ggplot2::geom_point(
      ggplot2::aes(x = .data$observed_gamma),
      color = "sienna2",
      shape = 18,
      position = ggplot2::position_nudge(y = -0.1),
      size = 4
    ) +
    ggplot2::labs(
      x = "Partial gamma",
      y = "Item pair",
      caption = caption_text
    ) +
    ggplot2::scale_color_manual(
      values = scales::brewer_pal()(3),
      aesthetics = "slab_fill",
      guide = "none"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(panel.spacing = ggplot2::unit(0.7, "cm")) +
    er2_axis_margins() +
    er2_plot_caption()

  p
}

# Tests for RMlocdepGamma(), RMlocdepGammaCutoff(), and RMlocdepGammaPlot()

make_dichotomous <- function(n = 200, k = 10, seed = 42L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("Item", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# Input validation
# ---------------------------------------------------------------------
test_that("RMlocdepGamma errors when iarm is not installed", {
  skip_if(
    requireNamespace("iarm", quietly = TRUE),
    "iarm is installed; skipping missing-package error path"
  )
  df <- make_dichotomous()
  expect_error(RMlocdepGamma(df), regexp = "iarm")
})

test_that("RMlocdepGamma errors when data has non-zero minimum", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous() + 1L
  expect_error(RMlocdepGamma(df), regexp = "scored starting at 0")
})

# ---------------------------------------------------------------------
# RMlocdepGamma output structure
# ---------------------------------------------------------------------
test_that("RMlocdepGamma output = 'dataframe' returns a list of two long-format tables", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  res <- RMlocdepGamma(df, output = "dataframe")
  # Two perspectives: each item paired with every other item (both directions)
  expect_type(res, "list")
  expect_true(length(res) >= 1L)
  expect_s3_class(res[[1L]], "data.frame")
  expect_true(all(
    c("Item1", "Item2", "gamma", "padj_bh", "Significance") %in%
      names(res[[1L]])
  ))
  # Significance is a star-string from iarm; should be one of the valid markers
  expect_true(all(
    res[[1L]]$Significance %in%
      c("", ".", "*", "**", "***")
  ))
  # 10 items -> 45 unique pairs
  expect_equal(nrow(res[[1L]]), choose(ncol(df), 2L))
})

test_that("RMlocdepGamma output = 'kable' returns a custom-class list", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("knitr")
  df <- make_dichotomous()
  out <- RMlocdepGamma(df, output = "kable")
  expect_s3_class(out, "RMlocdepGamma")
  expect_true(all(c("direction1", "direction2", ".combined") %in% names(out)))
  expect_s3_class(out$direction1, "knitr_kable")
  expect_s3_class(out$direction2, "knitr_kable")
})

test_that("RMlocdepGamma knit_print emits two clean pipe tables", {
  # Regression test for the old asis-blob bug: an earlier implementation
  # used `paste(kable1, "\n\n", kable2)` which silently interleaved the
  # two multi-line character vectors row-by-row. The combined asis string
  # is now stored in $.combined and surfaced via knit_print.RMlocdepGamma().
  skip_if_not_installed("iarm")
  skip_if_not_installed("knitr")
  df <- make_dichotomous()
  out <- RMlocdepGamma(df, output = "kable")
  asis <- knitr::knit_print(out)
  expect_s3_class(asis, "knit_asis")
  txt <- as.character(asis)
  # Exactly two "Table:" captions (one per rest-score direction)
  expect_equal(length(gregexpr("\nTable: ", paste0("\n", txt))[[1]]), 2L)
  # Exactly two header rows "|Item 1 |Item 2 ..."
  expect_equal(length(gregexpr("|Item 1 |Item 2", txt, fixed = TRUE)[[1]]), 2L)
  # No row should contain the column header twice (interleaving symptom)
  expect_false(any(grepl("Item 1.*Item 1", strsplit(txt, "\n")[[1]])))
})

test_that("RMlocdepGamma print method emits both tables", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("knitr")
  df <- make_dichotomous()
  out <- RMlocdepGamma(df, output = "kable")
  printed <- capture.output(print(out))
  joined <- paste(printed, collapse = "\n")
  expect_equal(length(gregexpr("\nTable: ", paste0("\n", joined))[[1]]), 2L)
  expect_equal(
    length(gregexpr("|Item 1 |Item 2", joined, fixed = TRUE)[[1]]),
    2L
  )
})

# ---------------------------------------------------------------------
# RMlocdepGammaCutoff -- small iterations
# ---------------------------------------------------------------------
test_that("RMlocdepGammaCutoff returns a list with pair_cutoffs + actual_iterations", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  res <- RMlocdepGammaCutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)
  expect_type(res, "list")
  expect_true("pair_cutoffs" %in% names(res))
  expect_s3_class(res$pair_cutoffs, "data.frame")
  expect_equal(nrow(res$pair_cutoffs), choose(ncol(df), 2L))
  expect_true(all(
    c("Item1", "Item2", "gamma_low", "gamma_high") %in% names(res$pair_cutoffs)
  ))
  expect_true(res$actual_iterations >= 1L)
})

test_that("RMlocdepGamma n_pairs trims to top-N by |gamma| per direction", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous() # 10 items -> 45 unique pairs per direction
  res_all <- RMlocdepGamma(df, output = "dataframe")
  expect_equal(nrow(res_all[[1L]]), choose(ncol(df), 2L))

  res_top <- RMlocdepGamma(df, output = "dataframe", n_pairs = 5L)
  expect_equal(nrow(res_top[[1L]]), 5L)
  expect_equal(nrow(res_top[[2L]]), 5L)

  # Each direction should contain exactly the 5 pairs with the largest |gamma|
  expected_top <- res_all[[1L]][
    order(abs(res_all[[1L]]$gamma), decreasing = TRUE)[seq_len(5L)],
    c("Item1", "Item2"),
    drop = FALSE
  ]
  expect_setequal(
    paste(res_top[[1L]]$Item1, res_top[[1L]]$Item2),
    paste(expected_top$Item1, expected_top$Item2)
  )

  # |gamma| within the returned top should be non-increasing
  expect_true(all(diff(abs(res_top[[1L]]$gamma)) <= 0))
})

test_that("RMlocdepGamma n_pairs > total pairs silently returns all pairs", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  res <- RMlocdepGamma(df, output = "dataframe", n_pairs = 10000L)
  expect_equal(nrow(res[[1L]]), choose(ncol(df), 2L))
})

test_that("RMlocdepGamma validates n_pairs", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  expect_error(RMlocdepGamma(df, n_pairs = 0L), regexp = "positive integer")
  expect_error(RMlocdepGamma(df, n_pairs = -3L), regexp = "positive integer")
  expect_error(RMlocdepGamma(df, n_pairs = 1.5), regexp = "positive integer")
  expect_error(
    RMlocdepGamma(df, n_pairs = c(1, 2)),
    regexp = "positive integer"
  )
})

test_that("RMlocdepGamma accepts an RMlocdepGammaCutoff result and adds flagged column", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  cuts <- RMlocdepGammaCutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)
  res <- RMlocdepGamma(df, cutoff = cuts, output = "dataframe")
  expect_type(res, "list")
  expect_true("flagged" %in% names(res[[1L]]))
  expect_type(res[[1L]]$flagged, "logical")
})

test_that("RMlocdepGammaCutoff is reproducible with the same seed", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  r1 <- RMlocdepGammaCutoff(df, iterations = 5L, parallel = FALSE, seed = 42L)
  r2 <- RMlocdepGammaCutoff(df, iterations = 5L, parallel = FALSE, seed = 42L)
  expect_equal(r1$pair_cutoffs, r2$pair_cutoffs)
})

# ---------------------------------------------------------------------
# RMlocdepGammaPlot
# ---------------------------------------------------------------------
test_that("RMlocdepGammaPlot returns a ggplot from an RMlocdepGammaCutoff result", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  df <- make_dichotomous()
  cuts <- RMlocdepGammaCutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)
  p <- RMlocdepGammaPlot(cuts)
  expect_s3_class(p, "ggplot")
})

test_that("RMlocdepGammaPlot with observed data overlay returns a ggplot", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("ggplot2")
  df <- make_dichotomous()
  cuts <- RMlocdepGammaCutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)
  p <- RMlocdepGammaPlot(cuts, data = df)
  expect_s3_class(p, "ggplot")
})

# ---------------------------------------------------------------------
# RMlocdepGamma -- cutoff validation
# ---------------------------------------------------------------------
test_that("RMlocdepGamma rejects malformed cutoff arguments", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  expect_error(RMlocdepGamma(df, cutoff = "abc"), regexp = "cutoff")
  expect_error(
    RMlocdepGamma(df, cutoff = data.frame(Item1 = "a", Item2 = "b")),
    regexp = "missing required columns"
  )
})

test_that("RMlocdepGamma band flag matches gamma vs bounds in both directions", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  cuts <- RMlocdepGammaCutoff(df, iterations = 10L, parallel = FALSE, seed = 1L)
  res <- RMlocdepGamma(df, cutoff = cuts, output = "dataframe")
  for (dir in res) {
    expect_equal(
      dir$flagged,
      !is.na(dir$gamma_low) &
        (dir$gamma < dir$gamma_low | dir$gamma > dir$gamma_high)
    )
  }
})

test_that("RMlocdepGamma handles missing values in data", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  df[1:3, 1] <- NA
  res <- RMlocdepGamma(df, output = "dataframe")
  expect_equal(nrow(res$direction1), choose(ncol(df), 2))
  kbl <- RMlocdepGamma(df)
  expect_match(kbl$.combined, "complete cases")
})

# ---------------------------------------------------------------------
# RMlocdepGamma -- bootstrap p-values
# ---------------------------------------------------------------------
test_that("RMlocdepGamma p_value requires the full cutoff object", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  expect_error(
    RMlocdepGamma(df, p_value = TRUE),
    regexp = "full RMlocdepGammaCutoff"
  )
  cuts <- RMlocdepGammaCutoff(df, iterations = 5L, parallel = FALSE, seed = 1L)
  expect_error(
    RMlocdepGamma(df, cutoff = cuts$pair_cutoffs, p_value = TRUE),
    regexp = "full RMlocdepGammaCutoff"
  )
})

test_that("RMlocdepGamma p_value adds one-sided p per pair, mirrored across directions", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  cuts <- RMlocdepGammaCutoff(df, iterations = 10L, parallel = FALSE, seed = 1L)
  expect_warning(
    res <- RMlocdepGamma(
      df,
      cutoff = cuts,
      p_value = TRUE,
      output = "dataframe"
    ),
    regexp = "only 10 simulation iterations"
  )
  expected_cols <- c(
    "Item1",
    "Item2",
    "gamma",
    "se",
    "lower",
    "upper",
    "gamma_low",
    "gamma_high",
    "p_gamma",
    "padj_gamma",
    "flagged"
  )
  expect_named(res$direction1, expected_cols)
  expect_named(res$direction2, expected_cols)
  expect_false(any(c("padj_bh", "Significance") %in% names(res$direction1)))

  # p is rounded to 4 dp, so compare against the rounded floor
  B <- cuts$actual_iterations
  p_floor <- round(1 / (B + 1), 4)
  expect_true(all(
    res$direction1$p_gamma >= p_floor &
      res$direction1$p_gamma <= 1,
    na.rm = TRUE
  ))
  expect_true(all(
    res$direction1$padj_gamma >= res$direction1$p_gamma,
    na.rm = TRUE
  ))

  # One test per pair: p/padj identical across the two direction tables
  key <- function(d) paste(pmin(d$Item1, d$Item2), pmax(d$Item1, d$Item2))
  m <- match(key(res$direction2), key(res$direction1))
  expect_equal(res$direction2$p_gamma, res$direction1$p_gamma[m])
  expect_equal(res$direction2$padj_gamma, res$direction1$padj_gamma[m])
  expect_equal(res$direction2$flagged, res$direction1$flagged[m])

  # kable renders with the correction label and canonical-direction note
  suppressWarnings(kbl <- RMlocdepGamma(df, cutoff = cuts, p_value = TRUE))
  expect_s3_class(kbl, "RMlocdepGamma")
  expect_match(kbl$.combined, "Westfall-Young")
  expect_match(kbl$.combined, "canonical direction")
})

test_that("RMlocdepGamma p_value correction runs on the full family before n_pairs", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  cuts <- RMlocdepGammaCutoff(df, iterations = 10L, parallel = FALSE, seed = 1L)
  suppressWarnings({
    full <- RMlocdepGamma(
      df,
      cutoff = cuts,
      p_value = TRUE,
      output = "dataframe"
    )
    top3 <- RMlocdepGamma(
      df,
      cutoff = cuts,
      p_value = TRUE,
      n_pairs = 3,
      output = "dataframe"
    )
  })
  expect_equal(nrow(top3$direction1), 3L)
  key <- function(d) paste(pmin(d$Item1, d$Item2), pmax(d$Item1, d$Item2))
  m <- match(key(top3$direction1), key(full$direction1))
  expect_equal(top3$direction1$padj_gamma, full$direction1$padj_gamma[m])
})

test_that("RMlocdepGamma p_value correction variants run and 'none' is marginal", {
  skip_on_cran()
  skip_if_not_installed("iarm")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  cuts <- RMlocdepGammaCutoff(df, iterations = 10L, parallel = FALSE, seed = 1L)
  suppressWarnings({
    none <- RMlocdepGamma(
      df,
      cutoff = cuts,
      p_value = TRUE,
      correction = "none",
      output = "dataframe"
    )
    by <- RMlocdepGamma(
      df,
      cutoff = cuts,
      p_value = TRUE,
      correction = "fdr_by",
      output = "dataframe"
    )
  })
  expect_equal(none$direction1$p_gamma, none$direction1$padj_gamma)
  expect_true(all(
    by$direction1$padj_gamma >= by$direction1$p_gamma,
    na.rm = TRUE
  ))
})

test_that("RMlocdepGamma validates alpha", {
  skip_if_not_installed("iarm")
  df <- make_dichotomous()
  expect_error(RMlocdepGamma(df, alpha = -1), regexp = "alpha")
  expect_error(RMlocdepGamma(df, alpha = "x"), regexp = "alpha")
})

# Internal helpers for bootstrap p-values and multiple-comparison correction.
#
# Shared by the simulation-cutoff functions that compare an observed per-item
# (or per-pair) statistic against a parametric-bootstrap null distribution.
# The per-comparison statistic is the residual studentised by the *bootstrap*
# mean and SD, putting all comparisons on equal footing for the family-wise
# maximum (Ferreira, 2024) without invoking the Wilson-Hilferty / ZSTD
# transform criticised by Müller (2020).

#' Bootstrap p-values with optional multiplicity correction
#'
#' @param observed Named numeric vector of observed statistics (one per item
#'   or pair).
#' @param sim Numeric matrix of simulated null values, `iterations` rows by
#'   `length(observed)` columns, with column names matching `names(observed)`.
#' @param correction One of `"fwer"` (Westfall-Young studentised-max
#'   step-down), `"fdr_bh"`, `"fdr_by"`, or `"none"`.
#' @param tail `"two.sided"` (default; deviations in either direction are
#'   extreme, e.g. infit over/underfit) or `"upper"` (only large positive
#'   values are extreme, e.g. excess Q3 local dependence).
#' @return A data.frame with columns `name`, `p` (marginal Monte-Carlo
#'   p-value), and `padj` (corrected p-value). Comparisons whose simulated
#'   values have no variance return `NA`.
#' @keywords internal
#' @noRd
.bootstrap_pvalues <- function(
  observed,
  sim,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  tail = c("two.sided", "upper")
) {
  correction <- match.arg(correction)
  tail <- match.arg(tail)
  nm <- names(observed)
  sim <- sim[, nm, drop = FALSE]
  B <- nrow(sim)

  m <- colMeans(sim, na.rm = TRUE)
  s <- apply(sim, 2L, stats::sd, na.rm = TRUE)
  s[!is.finite(s) | s < 1e-8] <- NA_real_ # guard zero-variance comparisons

  # Studentise observed and simulated by the bootstrap mean / SD; the test
  # statistic is |t| (two-sided) or t (upper tail).
  t_obs <- (observed - m) / s
  t_sim <- sweep(sweep(sim, 2L, m, `-`), 2L, s, `/`)
  if (tail == "two.sided") {
    stat_obs <- abs(t_obs)
    stat_sim <- abs(t_sim)
  } else {
    stat_obs <- t_obs
    stat_sim <- t_sim
  }

  # Marginal Monte-Carlo p-value, (1 + count) / (B + 1).
  p_marg <- vapply(
    seq_along(nm),
    function(i) {
      if (is.na(stat_obs[i])) {
        return(NA_real_)
      }
      (1 + sum(stat_sim[, i] >= stat_obs[i], na.rm = TRUE)) / (B + 1)
    },
    numeric(1)
  )

  padj <- switch(
    correction,
    none = p_marg,
    fdr_bh = stats::p.adjust(p_marg, method = "BH"),
    fdr_by = stats::p.adjust(p_marg, method = "BY"),
    fwer = .wy_stepdown(stat_obs, stat_sim, B)
  )

  data.frame(
    name = nm,
    p = p_marg,
    padj = padj,
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

#' Format a p-value for prose captions
#'
#' Plain-number formatting: exact to three decimals down to 0.001, then the
#' conventional "p < 0.001". Avoids the scientific notation ("<1e-04") that
#' `format.pval()` emits, which most readers are not used to.
#'
#' @param p Numeric p-value (length 1).
#' @return A string starting with "p", e.g. "p = 0.043" or "p < 0.001".
#' @keywords internal
#' @noRd
.format_p <- function(p) {
  if (is.na(p)) {
    return("p = NA")
  }
  if (p < 0.001) "p < 0.001" else paste0("p = ", sprintf("%.3f", p))
}

#' Human-readable label for a multiplicity-correction method
#'
#' Shared by the kable captions of the p-value-reporting functions.
#'
#' @param correction One of `"fwer"`, `"fdr_bh"`, `"fdr_by"`, `"none"`.
#' @return A character label.
#' @keywords internal
#' @noRd
.correction_label <- function(correction) {
  switch(
    correction,
    fwer = "Westfall-Young step-down (FWER)",
    fdr_bh = "Benjamini-Hochberg (FDR)",
    fdr_by = "Benjamini-Yekutieli (FDR)",
    none = "uncorrected"
  )
}

#' Westfall-Young studentised-max step-down adjusted p-values
#'
#' Rejects the most extreme comparison against the maximum over all
#' comparisons, then recomputes the maximum over the remaining comparisons,
#' enforcing monotonicity. Controls the family-wise error rate while using the
#' bootstrap dependence among the statistics (Westfall & Young, 1993;
#' Ferreira, 2024). Validity of the step-down rests on subset pivotality.
#'
#' @param stat_obs Numeric vector of observed test statistics (already
#'   |studentised| or signed-studentised depending on the tail).
#' @param stat_sim Numeric matrix (`iterations` x comparisons) of the
#'   simulated test statistics.
#' @param B Number of iterations.
#' @return Numeric vector of adjusted p-values (NA for invalid comparisons).
#' @keywords internal
#' @noRd
.wy_stepdown <- function(stat_obs, stat_sim, B) {
  k <- length(stat_obs)
  padj <- rep(NA_real_, k)

  valid <- which(is.finite(stat_obs))
  if (length(valid) == 0L) {
    return(padj)
  }

  # Process valid comparisons in order of decreasing observed evidence.
  ord <- valid[order(stat_obs[valid], decreasing = TRUE)]
  prev <- 0
  for (j in seq_along(ord)) {
    remaining <- ord[j:length(ord)]
    max_star <- apply(
      stat_sim[, remaining, drop = FALSE],
      1L,
      max,
      na.rm = TRUE
    )
    p_j <- (1 + sum(max_star >= stat_obs[ord[j]], na.rm = TRUE)) / (B + 1)
    prev <- max(prev, p_j) # enforce monotonicity (step-down)
    padj[ord[j]] <- min(prev, 1)
  }
  padj
}

#' Cronbach's alpha
#'
#' Closed-form alpha from item-score and total-score variances. Complete
#' cases only (rows with any `NA` are dropped).
#'
#' @param data A data.frame or matrix of item responses.
#'
#' @return Numeric scalar; `NA_real_` if fewer than two items, fewer than
#'   two complete cases, or zero total-score variance.
#'
#' @keywords internal
#' @noRd
cronbach_alpha <- function(data) {
  d <- stats::na.omit(as.data.frame(data))
  k <- ncol(d)
  if (k < 2L || nrow(d) < 2L) {
    return(NA_real_)
  }
  total_var <- stats::var(rowSums(d))
  if (!is.finite(total_var) || total_var == 0) {
    return(NA_real_)
  }
  item_vars <- vapply(d, stats::var, numeric(1L))
  (k / (k - 1L)) * (1 - sum(item_vars) / total_var)
}

# ---------------------------------------------------------------------------

#' Relative Measurement Uncertainty (RMU)
#'
#' Bayesian-style reliability estimate (Bignardi, Kievit & Bürkner, 2025)
#' computed from a matrix of posterior or plausible-value draws. The columns of
#' `input_draws` are split at random into two halves; reliability is the
#' Pearson correlation across persons of paired columns from the two halves,
#' summarised across pairs as a posterior mean with HDCI.
#'
#' Adapted (with permission, GPL-2/3) from
#' \url{https://github.com/giac01/gbtoolbox/blob/main/R/reliability.R}.
#'
#' @param input_draws Numeric matrix or data.frame of draws. Rows are
#'   subjects; columns are draws. Must have at least two columns; ideally many.
#' @param level Numeric in (0, 1). Width of the HDCI returned. Default `0.95`.
#' @param verbose Logical. Print summary information about the input. Default
#'   `FALSE`.
#'
#' @return A 1-row data.frame with columns `rmu_estimate`, `hdci_lowerbound`,
#'   `hdci_upperbound`, plus the `.width`/`.point`/`.interval` metadata
#'   columns added by `ggdist::mean_hdci()`.
#'
#' @details
#' The function silently returns 0 for any column pair where either side has
#' zero variance (the correlation is undefined there).
#'
#' Requires the `ggdist` package (Suggests).
#'
#' @references
#' Bignardi, G., Kievit, R., & Bürkner, P. C. (2025). A general method for
#' estimating reliability using Bayesian Measurement Uncertainty. *PsyArXiv*.
#' \doi{10.31234/osf.io/h54k8_v1}
#'
#' @seealso [RMreliability()]
#'
#' @export
RMUreliability <- function(input_draws, level = 0.95, verbose = FALSE) {
  if (!requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required for RMUreliability().\n",
      "Install it with: install.packages(\"ggdist\")",
      call. = FALSE
    )
  }
  if (!is.numeric(level) || level <= 0 || level >= 1) {
    stop(
      "`level` must be a numeric value strictly between 0 and 1.",
      call. = FALSE
    )
  }

  input_draws <- as.matrix(input_draws)
  if (ncol(input_draws) < 2L) {
    stop("`input_draws` must have at least 2 columns.", call. = FALSE)
  }

  sds <- apply(input_draws, 2L, stats::sd, na.rm = TRUE)
  zero_sd_cols <- which(sds == 0)
  if (length(zero_sd_cols) > 0L) {
    warning(
      sprintf(
        "Found %d column(s) with zero standard deviation (columns: %s).",
        length(zero_sd_cols),
        paste(zero_sd_cols, collapse = ", ")
      ),
      call. = FALSE
    )
  }

  na_count <- sum(is.na(input_draws))
  if (na_count > 0L) {
    warning(
      sprintf("Found %d NA value(s) in `input_draws`.", na_count),
      call. = FALSE
    )
  }

  if (verbose) {
    message("Subjects: ", nrow(input_draws), "; draws: ", ncol(input_draws))
  }

  col_select <- sample.int(ncol(input_draws), replace = FALSE)
  half <- floor(length(col_select) / 2)
  cols_a <- col_select[seq_len(half)]
  cols_b <- col_select[(half + 1L):(2L * half)]

  draws_a <- input_draws[, cols_a, drop = FALSE]
  draws_b <- input_draws[, cols_b, drop = FALSE]

  rel_post <- vapply(
    seq_len(ncol(draws_a)),
    function(i) {
      x <- draws_a[, i]
      y <- draws_b[, i]
      if (
        stats::var(x, na.rm = TRUE) == 0 ||
          stats::var(y, na.rm = TRUE) == 0
      ) {
        return(0)
      }
      stats::cor(x, y, method = "pearson", use = "complete.obs")
    },
    numeric(1L)
  )

  hdci <- ggdist::mean_hdci(rel_post, .width = level)
  colnames(hdci)[1:3] <- c("rmu_estimate", "hdci_lowerbound", "hdci_upperbound")
  hdci
}

# ---------------------------------------------------------------------------

#' Reliability metrics for a Rasch model
#'
#' Computes three reliability indices for a Rasch / partial credit model:
#' the Person Separation Index (PSI) -- WLE-based separation reliability -- the
#' marginal reliability (native, CML test information integrated over the
#' estimated normal latent density), and Relative Measurement Uncertainty (RMU)
#' via [RMUreliability()] applied to plausible values from `mirt::fscores()`.
#'
#' Confidence intervals for **PSI** and **Marginal** reliability are obtained
#' by non-parametric bootstrap (resampling respondents; all three indices are
#' recomputed natively per resample, no model is refitted by `mirt`). The RMU
#' interval is the HDCI of correlations across plausible-value draws, averaged
#' over `rmu_iter` random splits of the draws.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers).
#' @param conf_int Numeric in (0, 1). HDCI width for both bootstrap CIs and
#'   RMU. Default `0.95`.
#' @param draws Integer. Number of plausible-value draws drawn from the mirt
#'   model for the RMU calculation. Default `1000`. More gives a more stable
#'   RMU; computational cost is mostly linear.
#' @param rmu_iter Integer. Number of times [RMUreliability()] is repeated on
#'   the same set of plausible-value draws (each repetition uses a fresh
#'   random column split). Estimates are averaged across repetitions to
#'   stabilise against split-induced variability. Default `50`.
#' @param estim Character. Theta estimator used by `mirt::fscores()` for the
#'   RMU plausible-value seed. One of `"WLE"` (default), `"EAP"`, `"MAP"`,
#'   `"ML"`. Plausible draws themselves are produced by Metropolis-Hastings.
#'   (PSI and marginal reliability are computed natively and do not use this.)
#' @param boot Logical. If `TRUE`, run a non-parametric bootstrap to obtain
#'   CIs for PSI and Marginal reliability. Default `FALSE`.
#' @param boot_iter Integer. Number of bootstrap iterations when
#'   `boot = TRUE`. Default `200`.
#' @param parallel Logical. Use parallel processing via `mirai` for the
#'   bootstrap if available. Default `TRUE`.
#' @param n_cores Integer or `NULL`. Number of parallel workers. When `NULL`,
#'   `getOption("mc.cores")` is checked first; if neither is set,
#'   bootstrapping falls back to sequential.
#' @param seed Integer or `NULL`. Master random seed for reproducibility.
#' @param verbose Logical. Print progress messages and a progress bar for
#'   the bootstrap. Default `FALSE`.
#' @param theta_range Numeric length-2 vector. Theta limits passed to
#'   `mirt::fscores()`. Default `c(-10, 10)`.
#' @param output Character. `"kable"` (default) for a formatted
#'   `knitr::kable()` table, or `"dataframe"` for the underlying data.frame.
#'
#' @return
#' * If `output = "kable"`: a `knitr_kable` object with one row per metric.
#' * If `output = "dataframe"`: a data.frame with columns `metric`,
#'   `estimate`, `lower`, `upper`, `notes`.
#'
#' @details
#' Marginal reliability is the native Green (1984) coefficient,
#' \eqn{1 - \overline{1/I(\theta)}/\sigma^2}, where the test information
#' \eqn{I(\theta)} is summed from the CML item parameters and the average error
#' variance is taken over the estimated normal latent density \eqn{N(0,
#' \sigma^2)} (\eqn{\sigma} from marginal ML). Unlike `mirt::marginal_rxx()`,
#' which assumes \eqn{N(0,1)}, this integrates over the estimated latent
#' variance, so it is correct on the Rasch logit scale (where \eqn{\sigma}
#' is typically well above 1, and the \eqn{N(0,1)} assumption
#' underestimates reliability). It is the model-based complement to the
#' sample-based PSI; a large gap between the two flags an off-target or
#' non-normal sample.
#'
#' PSI is the WLE-based separation reliability,
#' \eqn{1 - \overline{SEM^2} / \mathrm{Var}(\hat\theta)}, computed from CML item
#' thresholds (`psychotools`) and Warm's WLE person locations / analytic SEMs.
#' Respondents with extreme (min/max) raw scores are excluded -- their boundary
#' estimates would inflate the person variance and overstate reliability.
#' (Earlier versions used `eRm::SepRel()` with MLE; the values can differ, most
#' noticeably for scales with many extreme scorers, e.g. dichotomous items.)
#'
#' RMU is from Bignardi, Kievit, & Bürkner (2025), modified here to use mirt
#' plausible values rather than fully Bayesian posterior draws (see Mislevy,
#' 1991, for the plausible-values framework).
#'
#' Bootstrap iterations that fail to converge are silently dropped.
#'
#' @references
#' Bignardi, G., Kievit, R., & Bürkner, P. C. (2025). A general method for
#' estimating reliability using Bayesian Measurement Uncertainty. *PsyArXiv*.
#' \doi{10.31234/osf.io/h54k8_v1}
#'
#' Green, B. F., Bock, R. D., Humphreys, L. G., Linn, R. L., & Reckase, M. D.
#' (1984). Technical Guidelines for Assessing Computerized Adaptive Tests.
#' *Journal of Educational Measurement, 21*(4), 347–360.
#' \doi{10.1111/j.1745-3984.1984.tb01039.x}
#'
#' Mislevy, R. J. (1991). Randomization-Based Inference about Latent Variables
#' from Complex Samples. *Psychometrika, 56*(2), 177-196.
#' \doi{10.1007/BF02294457}
#'
#' Adams, R. J. (2005). Reliability as a measurement design effect.
#' *Studies in Educational Evaluation, 31*(2), 162-172.
#' \doi{10.1016/j.stueduc.2005.05.008}
#'
#' @seealso [RMUreliability()]
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggdist", quietly = TRUE) &&
#'     requireNamespace("eRm", quietly = TRUE)) {
#'   set.seed(1)
#'   RMreliability(eRm::raschdat1[, 1:20], draws = 1000)
#'
#'   # Bootstrap CI for PSI and Marginal
#'   # (use more bootstrap iterations, e.g. 200+, in real analyses)
#'   RMreliability(eRm::raschdat1[, 1:20], draws = 1000,
#'                 boot = TRUE, boot_iter = 25, parallel = FALSE, seed = 42)
#' }
#' }
RMreliability <- function(
  data,
  conf_int = 0.95,
  draws = 1000,
  rmu_iter = 50,
  estim = "WLE",
  boot = FALSE,
  boot_iter = 200,
  parallel = TRUE,
  n_cores = NULL,
  seed = NULL,
  verbose = FALSE,
  theta_range = c(-10, 10),
  output = "kable"
) {
  output <- match.arg(output, c("kable", "dataframe"))
  estim <- match.arg(estim, c("WLE", "EAP", "MAP", "ML"))

  if (!requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required for RMreliability().\n",
      "Install it with: install.packages(\"ggdist\")",
      call. = FALSE
    )
  }

  validate_response_data(data)

  if (nrow(stats::na.omit(data)) == 0L) {
    stop("No complete cases in `data`.", call. = FALSE)
  }

  # Respondents with no responses at all contribute nothing and break the CML
  # fit behind the PSI (psychotools errors on all-NA rows); drop them, keeping
  # the raw total for the caption.
  n_total <- nrow(as.data.frame(data))
  data <- .drop_empty_respondents(data)

  data_mat <- as.matrix(data)
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L
  n_persons <- nrow(data)
  n_items <- ncol(data)

  # rgl workaround for any iarm-related fallout
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # --- Full-sample fits ------------------------------------------------------
  mirt_fit <- mirt::mirt(
    data = data,
    model = 1,
    itemtype = "Rasch",
    verbose = FALSE,
    accelerate = "squarem"
  )
  # --- Marginal reliability point estimate (native CML test info) ------------
  marg_rel <- .marginal_rxx(data)

  # --- PSI point estimate (WLE-based separation reliability) -----------------
  psi <- .wle_psi(data)

  # --- Cronbach's alpha point estimate ---------------------------------------
  alpha <- cronbach_alpha(data)

  # --- Plausible values + RMU ------------------------------------------------
  if (!is.null(seed)) {
    set.seed(seed)
  }
  pvs <- mirt::fscores(
    mirt_fit,
    method = estim,
    theta_lim = theta_range,
    plausible.draws = draws,
    plausible.type = "MH",
    verbose = FALSE
  )
  rmu_input <- do.call(cbind, lapply(pvs, as.numeric))

  # mirt's MH plausible-value sampler leaves the R RNG in a nondeterministic
  # state even under set.seed() (the draws themselves are reproducible, the
  # stream advancement is not), so the RMU column splits below would differ
  # between identical calls. Re-seed to make the whole result reproducible;
  # + 2L keeps the stream distinct from the bootstrap's seed + 1L.
  if (!is.null(seed)) {
    set.seed(seed + 2L)
  }

  rmu_iter_results <- do.call(
    rbind,
    lapply(seq_len(rmu_iter), function(i) {
      RMUreliability(rmu_input, level = conf_int)[, 1:3, drop = FALSE]
    })
  )
  rmu_summary <- list(
    estimate = mean(rmu_iter_results$rmu_estimate),
    lower = mean(rmu_iter_results$hdci_lowerbound),
    upper = mean(rmu_iter_results$hdci_upperbound)
  )

  # --- Bootstrap (optional) --------------------------------------------------
  alpha_lower <- NA_real_
  alpha_upper <- NA_real_
  psi_lower <- NA_real_
  psi_upper <- NA_real_
  marg_lower <- NA_real_
  marg_upper <- NA_real_
  actual_boot <- NA_integer_

  if (isTRUE(boot)) {
    use_parallel <- parallel && requireNamespace("mirai", quietly = TRUE)
    if (parallel && !use_parallel) {
      message(
        "Install 'mirai' for parallel processing: install.packages(\"mirai\")"
      )
      message("Running sequentially...")
    }
    if (use_parallel) {
      if (is.null(n_cores)) {
        n_cores <- getOption("mc.cores")
      }
      if (is.null(n_cores)) {
        warning(
          "For parallel processing, specify n_cores or set options(mc.cores = N).\n",
          "Falling back to sequential.",
          call. = FALSE
        )
        use_parallel <- FALSE
      } else {
        n_cores <- min(n_cores, boot_iter)
      }
    }

    if (!is.null(seed)) {
      set.seed(seed + 1L)
    }
    boot_seeds <- sample.int(.Machine$integer.max, boot_iter)

    boot_args <- list(
      data = data,
      is_polytomous = is_polytomous,
      estim = estim,
      theta_range = theta_range
    )

    if (use_parallel) {
      boot_results <- run_reliability_boot_parallel(
        boot_iter,
        boot_seeds,
        boot_args,
        n_cores,
        verbose
      )
    } else {
      boot_results <- run_reliability_boot_sequential(
        boot_iter,
        boot_seeds,
        boot_args,
        verbose
      )
    }

    ok <- vapply(boot_results, is.list, logical(1L))
    successful <- boot_results[ok]
    actual_boot <- length(successful)

    if (actual_boot < 2L) {
      warning(
        "Fewer than 2 bootstrap iterations succeeded; CI not reported.",
        call. = FALSE
      )
    } else {
      alpha_vec <- vapply(successful, function(x) x$alpha, numeric(1L))
      psi_vec <- vapply(successful, function(x) x$psi, numeric(1L))
      marg_vec <- vapply(successful, function(x) x$marginal, numeric(1L))

      alpha_int <- ggdist::hdci(alpha_vec, .width = conf_int)
      psi_int <- ggdist::hdci(psi_vec, .width = conf_int)
      marg_int <- ggdist::hdci(marg_vec, .width = conf_int)

      alpha_lower <- alpha_int[1L, 1L]
      alpha_upper <- alpha_int[1L, 2L]
      psi_lower <- psi_int[1L, 1L]
      psi_upper <- psi_int[1L, 2L]
      marg_lower <- marg_int[1L, 1L]
      marg_upper <- marg_int[1L, 2L]
    }
  }

  # --- Assemble output table -------------------------------------------------
  conf_pct <- round(conf_int * 100, 1)
  boot_note <- if (isTRUE(boot)) {
    if (!is.na(actual_boot) && actual_boot >= 2L) {
      paste0(actual_boot, " bootstrap resamples")
    } else {
      "bootstrap failed"
    }
  } else {
    "no bootstrap"
  }
  rmu_note <- paste0(draws, " PVs, ", rmu_iter, " RMU iterations")

  result_df <- data.frame(
    metric = c(
      "Cronbach's alpha",
      "PSI",
      "Marginal",
      paste0("RMU (", estim, ")")
    ),
    estimate = c(alpha, psi, marg_rel, rmu_summary$estimate),
    lower = c(alpha_lower, psi_lower, marg_lower, rmu_summary$lower),
    upper = c(alpha_upper, psi_upper, marg_upper, rmu_summary$upper),
    notes = c(boot_note, boot_note, boot_note, rmu_note),
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  if (output == "dataframe") {
    return(result_df)
  }

  # Kable display rounding (the dataframe output above stays unrounded)
  result_df <- .round_display(result_df, c(estimate = 3, lower = 3, upper = 3))

  knitr::kable(
    result_df,
    format = "pipe",
    col.names = c(
      "Metric",
      "Estimate",
      paste0("Lower (", conf_pct, "% HDCI)"),
      paste0("Upper (", conf_pct, "% HDCI)"),
      "Notes"
    ),
    caption = paste0(
      "Reliability for ",
      n_items,
      " items, ",
      .n_caption(n_persons, n_total),
      ". PSI is the WLE-based separation reliability and excludes min/max ",
      "scoring respondents."
    )
  )
}

# ---------------------------------------------------------------------------
# Internal: single bootstrap iteration
# ---------------------------------------------------------------------------

#' WLE-based Person Separation Index (separation reliability)
#'
#' PSI = `1 - mean(SEM^2) / Var(theta)` on Warm's WLE person locations and their
#' analytic SEMs (CML item thresholds via `psychotools`). Extreme (minimum and
#' maximum) scorers are excluded: their boundary WLE estimates would inflate the
#' person variance and overstate reliability, so the standard
#' separation-reliability convention drops them.
#'
#' @param data A data.frame or matrix of item responses (items from 0).
#' @param thr_list Optional pre-fitted CML thresholds (else fitted here).
#' @return The PSI (numeric), or `NA` if fewer than two non-extreme persons.
#' @keywords internal
#' @noRd
.wle_psi <- function(data, thr_list = NULL) {
  data_mat <- as.matrix(data)
  if (is.null(thr_list)) {
    thr_list <- .fit_cml_thresholds(data_mat)
  }
  est <- .estimate_thetas(data_mat, thr_list, method = "WLE")
  rs <- rowSums(data_mat, na.rm = TRUE)
  max_p <- vapply(
    seq_len(nrow(data_mat)),
    function(i) {
      ans <- !is.na(data_mat[i, ])
      sum(vapply(thr_list[ans], length, integer(1L)))
    },
    integer(1L)
  )
  keep <- is.finite(est$theta) & is.finite(est$sem) & rs > 0L & rs < max_p
  if (sum(keep) < 2L) {
    return(NA_real_)
  }
  1 - mean(est$sem[keep]^2) / stats::var(est$theta[keep])
}

#' Native marginal reliability (CML test information over an assumed normal)
#'
#' Green's (1984) marginal reliability,
#' \eqn{1 - \int [1/I(\theta)]\, g(\theta)\, d\theta / \sigma^2}, with the test
#' information \eqn{I(\theta) = \sum_i \mathrm{Var}_i(\mathrm{score}\mid\theta)}
#' summed from the CML item parameters and the expectation taken over a normal
#' latent density \eqn{g(\theta) = N(0, \sigma^2)} whose SD is estimated by
#' marginal maximum likelihood. Unlike `mirt::marginal_rxx()` (which assumes
#' \eqn{N(0,1)}), this integrates over the *estimated* latent variance, so it is
#' correct on the Rasch logit scale where \eqn{\sigma \neq 1}. Floored at 0.
#'
#' @param data Response matrix/data.frame (items from 0).
#' @param thr_list Optional pre-fitted CML thresholds.
#' @param n_nodes Number of quadrature nodes.
#' @return Marginal reliability (numeric), or `NA` if the latent SD is not
#'   estimable.
#' @keywords internal
#' @noRd
.marginal_rxx <- function(data, thr_list = NULL, n_nodes = 161L) {
  data_mat <- as.matrix(data)
  if (is.null(thr_list)) {
    thr_list <- .fit_cml_thresholds(data_mat)
  }
  ge <- seq(-6, 6, length.out = 81L)
  sigma <- .estimate_prior_sd(
    .grid_loglik(data_mat, .logp_tables(thr_list, ge), ge),
    ge,
    0
  )
  if (!is.finite(sigma) || sigma <= 0) {
    return(NA_real_)
  }
  g <- seq(-6 * sigma, 6 * sigma, length.out = n_nodes)
  I <- vapply(
    g,
    function(th) {
      sum(vapply(
        thr_list,
        function(thr) {
          cats <- 0:length(thr)
          P <- .pcm_cat_probs(th, thr)
          E <- sum(cats * P)
          sum((cats - E)^2 * P)
        },
        numeric(1L)
      ))
    },
    numeric(1L)
  )
  w <- stats::dnorm(g, 0, sigma)
  w <- w / sum(w)
  max(1 - sum(w * (1 / I)) / sigma^2, 0)
}

#' Run a single reliability bootstrap iteration
#'
#' Resamples respondents with replacement and reads Cronbach's alpha
#' (closed-form), the WLE-based PSI (`.wle_psi()`), and the native marginal
#' reliability (`.marginal_rxx()`) off the resample -- no model fit by `mirt`.
#'
#' @keywords internal
#' @noRd
run_single_reliability_boot <- function(seed, data_list) {
  set.seed(seed)
  idx <- sample.int(nrow(data_list$data), nrow(data_list$data), replace = TRUE)
  dat_b <- data_list$data[idx, , drop = FALSE]

  tryCatch(
    {
      # All three indices are computed natively (CML/WLE) -- no mirt fit.
      thr_b <- .fit_cml_thresholds(dat_b)
      alpha_b <- cronbach_alpha(dat_b) # closed-form
      psi_b <- .wle_psi(dat_b, thr_list = thr_b) # WLE separation
      if (!is.finite(psi_b)) {
        return("PSI not estimable for this resample")
      }
      marg_b <- .marginal_rxx(dat_b, thr_list = thr_b) # CML marginal

      list(alpha = alpha_b, psi = psi_b, marginal = marg_b)
    },
    error = function(e) as.character(conditionMessage(e))
  )
}

# ---------------------------------------------------------------------------
# Internal: parallel runner (mirai)
# ---------------------------------------------------------------------------

#' @keywords internal
#' @noRd
run_reliability_boot_parallel <- function(
  boot_iter,
  boot_seeds,
  boot_args,
  n_cores,
  verbose = FALSE
) {
  mirai::daemons(n_cores)
  on.exit(mirai::daemons(0), add = TRUE)

  if (verbose) {
    message(sprintf("Starting %d daemons...", n_cores))
    pb <- utils::txtProgressBar(min = 0, max = boot_iter, style = 3)
  }

  tasks <- lapply(seq_len(boot_iter), function(i) {
    mirai::mirai(
      {
        run_single_reliability_boot(seed, data_list)
      },
      seed = boot_seeds[i],
      data_list = boot_args,
      run_single_reliability_boot = run_single_reliability_boot,
      cronbach_alpha = cronbach_alpha,
      # Native engine helpers used by .wle_psi() / .marginal_rxx() in the daemon.
      .wle_psi = .wle_psi,
      .marginal_rxx = .marginal_rxx,
      .fit_cml_thresholds = .fit_cml_thresholds,
      .estimate_thetas = .estimate_thetas,
      .theta_wle = .theta_wle,
      .pcm_cat_probs = .pcm_cat_probs,
      .center_thresholds = .center_thresholds,
      .estimate_prior_sd = .estimate_prior_sd,
      .logp_tables = .logp_tables,
      .grid_loglik = .grid_loglik
    )
  })

  results <- vector("list", boot_iter)
  for (i in seq_len(boot_iter)) {
    res <- mirai::call_mirai(tasks[[i]])$data
    results[[i]] <- if (inherits(res, "errorValue")) "mirai_error" else res
    if (verbose) utils::setTxtProgressBar(pb, i)
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# ---------------------------------------------------------------------------
# Internal: sequential runner
# ---------------------------------------------------------------------------

#' @keywords internal
#' @noRd
run_reliability_boot_sequential <- function(
  boot_iter,
  boot_seeds,
  boot_args,
  verbose = FALSE
) {
  if (verbose) {
    pb <- utils::txtProgressBar(min = 0, max = boot_iter, style = 3)
  }

  results <- vector("list", boot_iter)
  for (i in seq_len(boot_iter)) {
    results[[i]] <- run_single_reliability_boot(boot_seeds[i], boot_args)
    if (verbose) utils::setTxtProgressBar(pb, i)
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# Tests for RMreliability() and RMUreliability()

make_dichotomous <- function(n = 200, k = 10, seed = 1L) {
  set.seed(seed)
  df <- as.data.frame(matrix(sample(0:1, n * k, replace = TRUE), n, k))
  colnames(df) <- paste0("I", seq_len(k))
  df
}

# ---------------------------------------------------------------------
# RMreliability() -- input validation
# ---------------------------------------------------------------------
test_that("RMreliability errors when data has non-zero minimum", {
  df <- make_dichotomous() + 1L
  expect_error(RMreliability(df), regexp = "scored starting at 0")
})

# ---------------------------------------------------------------------
# RMreliability() -- output structure (no bootstrap path)
# ---------------------------------------------------------------------
test_that("RMreliability output = 'dataframe' returns four reliability rows", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  skip_if_not_installed("mirt")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous()
  res <- RMreliability(df, draws = 50, rmu_iter = 20,
                       output = "dataframe", seed = 1)
  expect_s3_class(res, "data.frame")
  # Four metrics: alpha, PSI, Empirical, RMU
  expect_true(nrow(res) >= 4L)
  expect_true(all(c("metric", "estimate") %in% names(res)))
  # All point estimates should be finite numerics
  expect_true(all(is.finite(res$estimate)))
  # All estimates should be in plausible reliability range
  expect_true(all(res$estimate >= -1 & res$estimate <= 1))
})

test_that("RMreliability output = 'kable' returns knitr_kable", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  skip_if_not_installed("mirt")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("knitr")
  df <- make_dichotomous()
  out <- RMreliability(df, draws = 30, rmu_iter = 10,
                       output = "kable", seed = 1)
  expect_s3_class(out, "knitr_kable")
})

# ---------------------------------------------------------------------
# RMUreliability() -- input validation
# ---------------------------------------------------------------------
test_that("RMUreliability errors when fewer than 2 columns", {
  skip_if_not_installed("ggdist")
  m <- matrix(rnorm(50), nrow = 50, ncol = 1)
  expect_error(RMUreliability(m), regexp = "at least 2 columns")
})

test_that("RMUreliability errors when level is out of range", {
  skip_if_not_installed("ggdist")
  m <- matrix(rnorm(200), nrow = 50, ncol = 4)
  expect_error(RMUreliability(m, level = 0), regexp = "between 0 and 1")
  expect_error(RMUreliability(m, level = 1), regexp = "between 0 and 1")
})

# ---------------------------------------------------------------------
# RMUreliability() -- output structure
# ---------------------------------------------------------------------
test_that("RMUreliability on synthetic high-reliability draws returns plausible structure", {
  skip_if_not_installed("ggdist")
  # Build draws with a strong common signal -> high reliability expected
  set.seed(42)
  n_subj  <- 100
  n_draws <- 20
  theta   <- stats::rnorm(n_subj, mean = 0, sd = 1)
  draws   <- matrix(NA_real_, nrow = n_subj, ncol = n_draws)
  for (j in seq_len(n_draws)) {
    draws[, j] <- theta + stats::rnorm(n_subj, sd = 0.3)
  }
  res <- RMUreliability(draws)
  expect_s3_class(res, "data.frame")
  expect_equal(nrow(res), 1L)
  expect_true(all(c("rmu_estimate", "hdci_lowerbound",
                    "hdci_upperbound") %in% names(res)))
  expect_true(is.finite(res$rmu_estimate))
  # Strong signal should give reliability well above 0.5
  expect_gt(res$rmu_estimate, 0.5)
})

# ---------------------------------------------------------------------
# RMreliability() -- bootstrap path (sequential)
# ---------------------------------------------------------------------
test_that("RMreliability with boot = TRUE (sequential) returns finite CIs", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  skip_if_not_installed("mirt")
  skip_if_not_installed("ggdist")
  df <- make_dichotomous(n = 150, k = 8)
  res <- RMreliability(df, draws = 30, rmu_iter = 5,
                       boot = TRUE, boot_iter = 10, parallel = FALSE,
                       seed = 42, output = "dataframe")
  expect_s3_class(res, "data.frame")
  psi_row  <- res[res$metric == "PSI", ]
  marg_row <- res[res$metric == "Marginal", ]
  expect_true(is.finite(psi_row$lower)  && is.finite(psi_row$upper))
  expect_true(is.finite(marg_row$lower) && is.finite(marg_row$upper))
  expect_match(psi_row$notes, "bootstrap resamples")
})

# ---------------------------------------------------------------------
# RMreliability() -- bootstrap path (parallel via mirai)
# ---------------------------------------------------------------------
test_that("RMreliability with boot = TRUE (parallel) runs via mirai", {
  skip_on_cran()
  skip_if_not_installed("eRm")
  skip_if_not_installed("mirt")
  skip_if_not_installed("ggdist")
  skip_if_not_installed("mirai")
  df <- make_dichotomous(n = 120, k = 6)
  res <- RMreliability(df, draws = 20, rmu_iter = 3,
                       boot = TRUE, boot_iter = 4, parallel = TRUE,
                       n_cores = 2, seed = 7, output = "dataframe")
  expect_s3_class(res, "data.frame")
  expect_true(nrow(res) >= 4L)
})

# ---------------------------------------------------------------------
# cronbach_alpha() -- internal edge cases
# ---------------------------------------------------------------------
test_that("cronbach_alpha returns NA for <2 items or zero total variance", {
  one_col <- data.frame(a = sample(0:1, 20, replace = TRUE))
  expect_true(is.na(easyRasch2:::cronbach_alpha(one_col)))
  const <- as.data.frame(matrix(1L, nrow = 30, ncol = 5))
  expect_true(is.na(easyRasch2:::cronbach_alpha(const)))
})

# ---------------------------------------------------------------------
# RMUreliability() -- warnings / verbose / degenerate columns
# ---------------------------------------------------------------------
test_that("RMUreliability warns on zero-SD columns", {
  skip_if_not_installed("ggdist")
  set.seed(1)
  m <- cbind(rnorm(60), rnorm(60), rnorm(60), 5)  # last column constant
  expect_warning(RMUreliability(m), regexp = "zero standard deviation")
})

test_that("RMUreliability warns on NA values", {
  skip_if_not_installed("ggdist")
  set.seed(2)
  m <- matrix(rnorm(60 * 4), nrow = 60, ncol = 4)
  m[1, 1] <- NA
  expect_warning(RMUreliability(m), regexp = "NA value")
})

test_that("RMUreliability prints input summary when verbose = TRUE", {
  skip_if_not_installed("ggdist")
  set.seed(3)
  m <- matrix(rnorm(60 * 4), nrow = 60, ncol = 4)
  expect_message(RMUreliability(m, verbose = TRUE), regexp = "Subjects")
})

test_that("RMreliability drops all-NA respondents instead of erroring", {
  skip_if_not_installed("ggdist")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 60 * 6, replace = TRUE), nrow = 60))
  colnames(df) <- paste0("i", 1:6)
  df[3, ] <- NA

  expect_message(k <- RMreliability(df), "no responses dropped")
  expect_match(paste(as.character(k), collapse = "\n"),
               "n = 59 of 60 respondents")
})

test_that("RMreliability is reproducible with the same seed (incl. RMU)", {
  skip_if_not_installed("ggdist")
  # Regression: mirt's MH plausible-value sampler leaves the R RNG in a
  # nondeterministic state, so without the internal re-seed the RMU column
  # splits differed between identical calls.
  df <- make_dichotomous(n = 120, k = 6)
  r1 <- suppressWarnings(RMreliability(df, draws = 100, rmu_iter = 5,
                                       parallel = FALSE, seed = 42L,
                                       output = "dataframe"))
  r2 <- suppressWarnings(RMreliability(df, draws = 100, rmu_iter = 5,
                                       parallel = FALSE, seed = 42L,
                                       output = "dataframe"))
  expect_identical(r1$estimate, r2$estimate)
  expect_identical(r1$lower, r2$lower)
})

#' \eqn{Q_3} Residual Correlations for Local Dependence Assessment
#'
#' Computes Yen's \eqn{Q_3} residual correlations between item pairs. By default
#' the Rasch model is fitted by conditional maximum likelihood with WLE person
#' locations (`estimator = "CML"`); marginal ML via `mirt` is available with
#' `estimator = "MML"`. High correlations (above the dynamic cut-off) indicate
#' potential local dependence between items. See \code{\link{RMlocdepQ3Cutoff}}
#' for how to determine the appropriate dynamic cut-off for your data.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Missing values (`NA`) are allowed.
#' @param cutoff Optional. `NULL` (default) returns the raw \eqn{Q_3}matrix. A single
#'   numeric value returns the \eqn{Q_3} matrix with a global dynamic cut-off (the
#'   value added to the mean off-diagonal \eqn{Q_3}). The **full list** returned by
#'   \code{\link{RMlocdepQ3Cutoff}} returns a *list of two tables* (see Value):
#'   the cut-off matrix plus a per-pair table.
#' @param output Character string controlling the return value. Either
#'   `"kable"` (default) for formatted `knitr::kable()` table(s), or
#'   `"dataframe"` for the underlying data.frame(s).
#' @param n_pairs Integer or `NULL` (default). When the full cutoff object is
#'   supplied, limits the per-pair table to the `n_pairs` pairs with the
#'   largest departure from their expected range. `NULL` shows all pairs.
#' @param p_value Logical. If `TRUE` (requires the full
#'   \code{\link{RMlocdepQ3Cutoff}} object), the per-pair table also reports
#'   one-sided bootstrap p-values (`p_q3`, `padj_q3`) and flags `above` pairs
#'   (only) on `padj_q3 < alpha` instead of the expected range. Default `FALSE`.
#' @param correction Character. Multiple-comparison correction across item
#'   pairs when `p_value = TRUE`: `"fwer"` (default) for the Westfall-Young
#'   studentised-max step-down, `"fdr_bh"` / `"fdr_by"` for Benjamini-Hochberg
#'   / Benjamini-Yekutieli, or `"none"`.
#' @param alpha Numeric in (0, 1). Significance level for `Flagged` when
#'   `p_value = TRUE`. Default `0.05`.
#' @param estimator Character. Estimation engine for the \eqn{Q_3} residual
#'   correlations. `"CML"` (default) fits item parameters by conditional
#'   maximum likelihood (`psychotools`) and person locations by Warm's
#'   weighted likelihood (WLE) -- true to the Rasch tradition and finite
#'   at extreme scores. `"MML"` uses the marginal-ML / EAP engine
#'   (`mirt`), retained for backward compatibility. The estimator must
#'   match the one used by \code{\link{RMlocdepQ3Cutoff}}; when the full
#'   cutoff object is supplied, its stored estimator takes precedence (a
#'   mismatching `estimator` argument is overridden with a warning).
#'
#' @return
#' With `cutoff = NULL` or a bare numeric cut-off, a single object (`kable` or
#' data.frame) holding the lower triangle of the \eqn{Q_3} matrix; a numeric cut-off
#' adds a `Flagged` row flag and a caption describing the dynamic
#' cut-off.
#'
#' With the **full `RMlocdepQ3Cutoff()` object**, a named list of two:
#' \describe{
#'   \item{`$matrix`}{the \eqn{Q_3} lower-triangle matrix with the *global* dynamic
#'     cut-off (mean off-diagonal \eqn{Q_3} + suggested cut-off), as above.}
#'   \item{`$pairs`}{one row per item pair: `Item1`, `Item2`, `Observed` (\eqn{Q_3}),
#'     `Low`/`High` (the per-pair expected range, i.e. the simulated bounds),
#'     and `Flagged` -- `"above"` (\eqn{Q_3} above the upper bound, indicating local
#'     dependence), `"below"` (below the lower bound), or `""`. Sorted by
#'     absolute departure from the per-pair simulated median and truncated to
#'     `n_pairs`. With `p_value = TRUE`, columns `p_q3` and `padj_q3` are added
#'     and `Flagged` reflects `padj_q3 < alpha` and **only flags `"above"`**.}
#' }
#' The \eqn{Q_3} tile heatmap that earlier versions returned as `$plot` is now produced
#' by \code{\link{RMlocdepQ3Plot}} (as its `$matrix` element), so the table and
#' plot outputs share the same `$matrix`/`$pairs` structure.
#'
#' @details
#' The \eqn{Q_3} statistic (Yen, 1984) is the correlation between residuals of pairs
#' of items after accounting for the latent trait. Under local independence,
#' \eqn{Q_3} values are expected to be around \eqn{-1/(k-1)} where \eqn{k} is the
#' number of items. When `cutoff` is supplied, the dynamic cut-off is the mean
#' of all off-diagonal \eqn{Q_3} values plus `cutoff`, following the approach of
#' Christensen et al. (2017). Use \code{\link{RMlocdepQ3Cutoff}} to obtain a
#' simulation-based cutoff recommendation.
#'
#' \eqn{Q_3} is the column-wise correlation matrix of the model standardized
#' residuals \eqn{(x - E)/\sqrt{Var}}. By default (`estimator = "CML"`) item
#' parameters are estimated by conditional maximum likelihood (`psychotools`)
#' and person locations by Warm's weighted likelihood; `estimator = "MML"`
#' instead uses `mirt`'s marginal-ML model and its built-in \eqn{Q_3} residuals.
#' The two estimators give very similar \eqn{Q_3} values (off-diagonal correlation
#' typically > 0.95); what matters for inference is that the observed \eqn{Q_3} and
#' the simulated cut-off in \code{\link{RMlocdepQ3Cutoff}} use the *same*
#' estimator, which the functions enforce.
#'
#' \strong{Two views of local dependence.} Given the full
#' \code{\link{RMlocdepQ3Cutoff}} object, two complementary tables are returned.
#' The `$matrix` applies a single *global* cut-off (the Christensen et al.
#' approach: the 99th percentile of the simulated max-minus-mean \eqn{Q_3}) -- a
#' family-wise "is there any local dependence" overview. The `$pairs` table is
#' the per-comparison view: each observed \eqn{Q_3} against its own simulated expected
#' range (the `Low`/`High` bounds), so individual dependent pairs can be read
#' off and ranked.
#'
#' \strong{Bootstrap p-values.} When `p_value = TRUE`, the `$pairs` table also
#' tests each observed \eqn{Q_3} against its simulated null (from `cutoff$pair_results`)
#' with a one-sided (upper-tail) test for excess local dependence. The pair
#' statistic is studentised by the bootstrap mean and SD; the marginal p-value
#' is `(1 + #{Q3* >= Q3}) / (B + 1)`, and `correction` applies the family-wise
#' (Westfall-Young step-down) or FDR adjustment across the \eqn{k(k-1)/2} pairs.
#' As for item fit, the family-wise correction is liberal when the simulation is
#' small, so >= 1000 `iterations` in [RMlocdepQ3Cutoff()] are recommended (a
#' warning is issued otherwise).
#'
#' @references
#' Yen, W. M. (1984). Effects of local item dependence on the fit and
#' equating performance of the three-parameter logistic model.
#' *Applied Psychological Measurement, 8*(2), 125--145.
#' \doi{10.1177/014662168400800201}
#'
#' Christensen, K. B., Makransky, G., & Horton, M. (2017). Critical values
#' for Yen's \eqn{Q_3}: Identification of local dependence in the Rasch model.
#' *Applied Psychological Measurement, 41*(3), 178--194.
#' \doi{10.1177/0146621616677520}
#'
#' Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number
#' of hypotheses simultaneously. *Journal of Statistical Theory and Practice,
#' 19*(6). \doi{10.1007/s42519-024-00412-4}
#'
#' @inheritSection RMitemInfit Multiple comparisons
#'
#' @export
#'
#' @examples
#' \donttest{
#' # Simulate binary item response data (10 items, 200 persons)
#' set.seed(42)
#' sim_data <- as.data.frame(
#'   matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#' )
#' colnames(sim_data) <- paste0("Item", 1:10)
#'
#' # Raw Q3 matrix (no cutoff)
#' RMlocdepQ3(sim_data)
#'
#' # Get the underlying data.frame
#' q3_df <- RMlocdepQ3(sim_data, output = "dataframe")
#'
#' # Simulation-based cutoff (use 500+ iterations in real analyses)
#' if (requireNamespace("ggdist", quietly = TRUE)) {
#'   cutoff_res <- RMlocdepQ3Cutoff(sim_data, iterations = 50, parallel = FALSE)
#'
#'   # Bare numeric cutoff -> just the matrix
#'   RMlocdepQ3(sim_data, cutoff = cutoff_res$suggested_cutoff)
#'
#'   # Full object -> list of two tables: $matrix and $pairs
#'   res <- RMlocdepQ3(sim_data, cutoff = cutoff_res, output = "dataframe")
#'   res$pairs
#'
#'   # Top 5 pairs, with bootstrap p-values (use iterations >= 1000 in practice)
#'   RMlocdepQ3(sim_data, cutoff = cutoff_res, n_pairs = 5, p_value = TRUE,
#'              output = "dataframe")$pairs
#' }
#' }
RMlocdepQ3 <- function(
  data,
  cutoff = NULL,
  output = "kable",
  n_pairs = NULL,
  p_value = FALSE,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  alpha = 0.05,
  estimator = c("CML", "MML")
) {
  # --- Input validation -------------------------------------------------------
  cutoff_full <- NULL # full object (carries simulated $pair_results)
  if (!is.null(cutoff)) {
    # Accept the full RMlocdepQ3Cutoff() return list as well as a bare numeric.
    if (
      is.list(cutoff) &&
        !is.data.frame(cutoff) &&
        "suggested_cutoff" %in% names(cutoff)
    ) {
      cutoff_full <- cutoff
      cutoff <- as.numeric(cutoff$suggested_cutoff)
    }
    if (!is.numeric(cutoff) || length(cutoff) != 1L || is.na(cutoff)) {
      stop(
        "`cutoff` must be a single numeric value, NULL, or the list ",
        "returned by RMlocdepQ3Cutoff().",
        call. = FALSE
      )
    }
  }

  output <- match.arg(output, c("kable", "dataframe"))
  correction <- match.arg(correction)
  estimator <- match.arg(estimator)
  # The observed Q3 must use the same estimator as the simulated cut-off, so
  # the estimator stored in the cutoff object takes precedence over the arg.
  if (
    !is.null(cutoff_full) &&
      !is.null(cutoff_full$estimator) &&
      !identical(cutoff_full$estimator, estimator)
  ) {
    warning(
      "Using estimator \"",
      cutoff_full$estimator,
      "\" from the ",
      "RMlocdepQ3Cutoff() object (overriding estimator = \"",
      estimator,
      "\") so the observed Q3 matches the simulated cut-off.",
      call. = FALSE
    )
    estimator <- cutoff_full$estimator
  }
  if (!is.numeric(alpha) || length(alpha) != 1L || alpha <= 0 || alpha >= 1) {
    stop("`alpha` must be a single number in (0, 1).", call. = FALSE)
  }
  if (
    !is.null(n_pairs) &&
      (!is.numeric(n_pairs) || length(n_pairs) != 1L || n_pairs < 1)
  ) {
    stop("`n_pairs` must be NULL or a single positive integer.", call. = FALSE)
  }

  if (p_value) {
    if (is.null(cutoff_full) || is.null(cutoff_full$pair_results)) {
      stop(
        "`p_value = TRUE` requires the full RMlocdepQ3Cutoff() object (it ",
        "carries the simulated per-pair distributions in $pair_results); a ",
        "numeric cutoff or NULL is not sufficient.",
        call. = FALSE
      )
    }
    if (
      !is.null(cutoff_full$actual_iterations) &&
        cutoff_full$actual_iterations < 1000L
    ) {
      warning(
        "Bootstrap p-values are based on only ",
        cutoff_full$actual_iterations,
        " simulation iterations. With few ",
        "iterations the studentised-max (FWER) correction is liberal and ",
        "small p-values are imprecise; use iterations >= 1000 in ",
        "RMlocdepQ3Cutoff() for reliable p-values.",
        call. = FALSE
      )
    }
  }

  validate_response_data(data)

  data <- as.data.frame(data)
  n_total <- nrow(data)
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)
  n_used <- nrow(data)

  # The CML engine (psychotools) can choke or destabilise on sparse / zero-
  # variance response categories; warn (and point to estimator = "MML") before
  # fitting, consistent with RMitemParameters(). Not needed for the MML path.
  if (estimator == "CML") {
    .sparsity_warning(data, is_poly = max(as.matrix(data), na.rm = TRUE) > 1L)
  }

  # --- Compute Q3 residual correlations ---------------------------------------
  # .q3_residual_matrix() returns a symmetric Q3 matrix with an NA diagonal,
  # computed under the chosen estimator (CML/WLE residual correlations, or
  # mirt's MML Q3). The same routine is used for the simulated Q3 in
  # RMlocdepQ3Cutoff(), so observed and simulated values are comparable.
  resid_mat <- .q3_residual_matrix(data, estimator = estimator)

  # Mean of the off-diagonal correlations (diagonal already NA).
  mean_resid <- mean(resid_mat, na.rm = TRUE)

  # Matrix view (global dynamic cutoff = mean + cutoff, or raw when NULL).
  matrix_out <- .q3_matrix_output(
    resid_mat,
    mean_resid,
    cutoff,
    output,
    cutoff_full,
    n_used = n_used,
    n_total = n_total,
    has_na = has_na
  )

  # NULL or bare-numeric cutoff: just the matrix (single object).
  if (is.null(cutoff_full)) {
    return(matrix_out)
  }

  # Full RMlocdepQ3Cutoff() object: the matrix view and the per-pair table.
  # The Q3 tile heatmap now lives in RMlocdepQ3Plot()$matrix, so this table
  # function no longer emits a plot (consistent with the other *Cutoff/table
  # functions, which return data and leave rendering to the *Plot functions).
  pairs_out <- .q3_pairs_table(
    resid_mat,
    cutoff_full,
    n_pairs,
    p_value,
    correction,
    alpha,
    output
  )
  list(matrix = matrix_out, pairs = pairs_out)
}

#' Lower-triangle heatmap of the observed Q3 matrix
#'
#' Viridis tile plot of the Q3 residual correlations (lower triangle),
#' styled to match the package's other ggplots. Pairs whose Q3 exceeds the
#' global dynamic cut-off are outlined in red. Returns `NULL` (with a
#' message) when `ggplot2` is not installed, so the rest of the list output
#' is unaffected.
#'
#' @param resid_mat Observed Q3 matrix (symmetric, `NA` diagonal, item names).
#' @param dyn_cutoff Numeric global dynamic cut-off, or `NULL`.
#' @param estimator Character estimator label for the caption.
#' @param actual_iterations Number of simulation iterations (for the caption),
#'   or `NULL`.
#' @return A `ggplot` object, or `NULL`.
#' @keywords internal
#' @noRd
.q3_tile_plot <- function(
  resid_mat,
  dyn_cutoff = NULL,
  estimator = "CML",
  actual_iterations = NULL
) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    message(
      "Package 'ggplot2' is required for the Q3 heatmap ($plot); ",
      "returning NULL. Install it with: install.packages(\"ggplot2\")"
    )
    return(NULL)
  }

  items <- colnames(resid_mat)
  if (is.null(items)) {
    items <- as.character(seq_len(ncol(resid_mat)))
  }
  lt <- which(lower.tri(resid_mat), arr.ind = TRUE)
  df <- data.frame(
    row = factor(items[lt[, "row"]], levels = items),
    col = factor(items[lt[, "col"]], levels = items),
    Q3 = resid_mat[lt],
    stringsAsFactors = FALSE
  )
  df$flagged <- if (!is.null(dyn_cutoff)) df$Q3 > dyn_cutoff else FALSE
  df$label <- formatC(df$Q3, format = "f", digits = 2)

  # Diverging fill centred on the mean off-diagonal Q3 (the value expected
  # under local independence), following RASCHplot's ggQ3star(): the neutral
  # colour marks the baseline, warm (red) is above it, cool (blue) below.
  # Symmetric limits about the centre keep the two sides comparable in
  # intensity.
  mean_resid <- mean(resid_mat, na.rm = TRUE)
  # Round the half-range up to the next 0.01 (as RASCHplot does) so the most
  # extreme cell sits strictly inside the limits rather than on the boundary,
  # where floating-point rounding could clip it to NA (grey).
  max_dev <- ceiling(max(abs(df$Q3 - mean_resid), na.rm = TRUE) * 100) / 100

  est_label <- switch(
    estimator,
    CML = "CML item / WLE person",
    MML = "MML item / EAP person",
    estimator
  )
  cap <- paste0(
    "Yen's Q3 residual correlations (lower triangle; ",
    est_label,
    "). Diverging fill centred on the mean off-diagonal Q3 (",
    formatC(mean_resid, format = "f", digits = 3),
    "), the value expected under local independence.",
    if (!is.null(dyn_cutoff)) {
      paste0(
        " Dynamic cut-off ",
        formatC(dyn_cutoff, format = "f", digits = 3),
        "; ",
        sum(df$flagged, na.rm = TRUE),
        " pair(s) above it outlined in black."
      )
    } else {
      ""
    },
    if (!is.null(actual_iterations)) {
      paste0(" Cut-off from ", actual_iterations, " simulation iterations.")
    } else {
      ""
    }
  )

  p <- ggplot2::ggplot(
    df,
    ggplot2::aes(x = .data$col, y = .data$row, fill = .data$Q3)
  ) +
    ggplot2::geom_tile(color = "white", linewidth = 0.5)
  if (any(df$flagged, na.rm = TRUE)) {
    # Black outline (the RdYlBu high end is red, so a red outline would clash).
    p <- p +
      ggplot2::geom_tile(
        data = df[df$flagged %in% TRUE, , drop = FALSE],
        color = "black",
        linewidth = 1.1,
        fill = NA,
        width = 0.92,
        height = 0.92
      )
  }
  p +
    ggplot2::geom_text(
      ggplot2::aes(label = .data$label),
      color = "grey15",
      size = 3
    ) +
    # RdYlBu anchor colours, diverging about the local-independence baseline.
    # high Q3 -> red (potential local dependence), low -> blue.
    ggplot2::scale_fill_gradient2(
      name = "Q3",
      low = "#4575b4",
      mid = "#ffffbf",
      high = "#d73027",
      midpoint = mean_resid,
      limits = mean_resid + c(-1, 1) * max_dev
    ) +
    ggplot2::scale_x_discrete(limits = utils::head(items, -1L)) +
    ggplot2::scale_y_discrete(limits = rev(items[-1L])) +
    ggplot2::coord_fixed() +
    ggplot2::labs(x = NULL, y = NULL, caption = er2_caption(cap)) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      panel.grid = ggplot2::element_blank(),
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1)
    ) +
    er2_axis_margins() +
    er2_plot_caption()
}

#' Compute the Yen's Q3 residual-correlation matrix under a chosen estimator
#'
#' Single entry point used by `RMlocdepQ3()`, the per-iteration simulation in
#' `RMlocdepQ3Cutoff()`, and the observed overlay in `RMlocdepQ3Plot()`, so
#' observed and simulated Q3 are always computed the same way.
#'
#' `"CML"` (default) returns the correlation matrix of the CML/WLE standardized
#' residuals from `.rasch_std_residuals()`; `"MML"` returns `mirt`'s Q3
#' residuals from a marginal-ML Rasch fit. The matrix is symmetric with an `NA`
#' diagonal and carries the item names.
#'
#' @param data Numeric response matrix or data.frame (items from 0; `NA` ok).
#' @param estimator `"CML"` or `"MML"`.
#' @param fast Logical. For `"MML"` only, use the faster (lower-precision)
#'   `mirt` settings (`quadpts = 29`, `TOL = 0.005`) appropriate for the
#'   simulation loop. Ignored for `"CML"`.
#' @return Symmetric Q3 matrix with `NA` diagonal.
#' @keywords internal
#' @noRd
.q3_residual_matrix <- function(
  data,
  estimator = c("CML", "MML"),
  fast = FALSE
) {
  estimator <- match.arg(estimator)
  if (estimator == "MML") {
    mirt_fit <- if (fast) {
      mirt::mirt(
        data,
        model = 1,
        itemtype = "Rasch",
        verbose = FALSE,
        accelerate = "squarem",
        quadpts = 29,
        TOL = 0.005
      )
    } else {
      mirt::mirt(
        data,
        model = 1,
        itemtype = "Rasch",
        verbose = FALSE,
        accelerate = "squarem"
      )
    }
    m <- mirt::residuals(mirt_fit, type = "Q3", digits = 4, verbose = FALSE)
    diag(m) <- NA
    return(m)
  }
  # CML/WLE: Q3 is the column-wise correlation of standardized residuals.
  m <- stats::cor(
    .rasch_std_residuals(data, method = "WLE"),
    use = "pairwise.complete.obs"
  )
  diag(m) <- NA
  m
}

#' Render the Q3 lower-triangle matrix with an optional global cutoff
#'
#' @param resid_mat Observed Q3 matrix (symmetric, `NA` diagonal).
#' @param mean_resid Mean of the off-diagonal Q3 values.
#' @param cutoff Numeric global cutoff (added to `mean_resid`), or `NULL`.
#' @param output `"kable"` or `"dataframe"`.
#' @param cutoff_full Optional full cutoff object, used to enrich the caption.
#' @keywords internal
#' @noRd
.q3_matrix_output <- function(
  resid_mat,
  mean_resid,
  cutoff,
  output,
  cutoff_full = NULL,
  n_used = NULL,
  n_total = NULL,
  has_na = FALSE
) {
  resid_df <- as.data.frame(resid_mat)
  resid_df[upper.tri(resid_df)] <- NA
  diag(resid_df) <- NA

  if (!is.null(cutoff)) {
    dyn_cutoff <- mean_resid + cutoff
    resid_df$above_cutoff <- apply(
      resid_df,
      1L,
      function(row) any(row > dyn_cutoff, na.rm = TRUE)
    )
  }

  if (output == "dataframe") {
    return(resid_df)
  }

  n_clause <- if (!is.null(n_used) && !is.null(n_total)) {
    paste0(
      " ",
      .n_caption(
        n_used,
        n_total,
        if (has_na) "incomplete responses retained" else character()
      ),
      "."
    )
  } else {
    ""
  }
  if (!is.null(cutoff)) {
    caption_text <- paste0(
      "Dynamic cut-off: ",
      round(dyn_cutoff, 3),
      " (mean Q3 ",
      round(mean_resid, 3),
      " + ",
      round(cutoff, 3),
      ").",
      if (!is.null(cutoff_full)) {
        paste0(
          " Global simulation cutoff (99th pctl of max-mean Q3) from ",
          cutoff_full$actual_iterations,
          " iterations."
        )
      } else {
        ""
      },
      " Correlations exceeding the cut-off may indicate local dependence; ",
      "see the per-pair table for detail.",
      n_clause
    )
  } else {
    caption_text <- paste0(
      "Raw Q3 residual correlations (lower triangle). Use RMlocdepQ3Cutoff() ",
      "to derive a cutoff.",
      n_clause
    )
  }

  item_cols <- setdiff(names(resid_df), "above_cutoff")
  resid_display <- as.data.frame(
    lapply(resid_df[item_cols], function(x) {
      ifelse(is.na(x), "", as.character(round(x, 2)))
    }),
    stringsAsFactors = FALSE
  )
  rownames(resid_display) <- rownames(resid_df)
  if (!is.null(cutoff)) {
    resid_display$above_cutoff <- ifelse(resid_df$above_cutoff, "*", "")
  }

  knitr::kable(resid_display, caption = caption_text, format = "pipe")
}

#' Build the per-pair Q3 table (observed vs expected range, flagged)
#'
#' One row per item pair: observed Q3, the per-pair expected range (the
#' simulated `Low`/`High` bounds), and a `Flagged` label -- `"above"` (Q3
#' above the upper bound, local dependence), `"below"` (below the lower
#' bound), or `""`. Rows are sorted by absolute departure from the per-pair
#' simulated median; `n_pairs` keeps the top rows. With `p_value = TRUE`,
#' one-sided bootstrap p-values (`p_q3`, `padj_q3`) are added and `Flagged`
#' reflects `padj_q3 < alpha`.
#'
#' @param resid_mat Observed Q3 matrix (symmetric, `NA` diagonal).
#' @param cutoff_full Full [RMlocdepQ3Cutoff()] object.
#' @param n_pairs Integer or `NULL`; keep the top-`n_pairs` pairs by departure.
#' @param p_value Logical; add bootstrap p-values and flag on them.
#' @param correction,alpha Passed to `.bootstrap_pvalues()` / flagging.
#' @param output `"kable"` or `"dataframe"`.
#' @return A per-pair table (kable or data.frame).
#' @keywords internal
#' @noRd
.q3_pairs_table <- function(
  resid_mat,
  cutoff_full,
  n_pairs,
  p_value,
  correction,
  alpha,
  output
) {
  pc <- cutoff_full$pair_cutoffs # Item1, Item2, Q3_low, Q3_high
  pr <- cutoff_full$pair_results # Item1, Item2, Q3, iteration
  pr$key <- paste(pr$Item1, pr$Item2, sep = "___")
  keys <- paste(pc$Item1, pc$Item2, sep = "___")

  observed <- mapply(function(a, b) resid_mat[a, b], pc$Item1, pc$Item2)
  median_q3 <- tapply(pr$Q3, pr$key, stats::median)[keys] # expected (sort/dir)
  low <- pc$Q3_low
  high <- pc$Q3_high

  # Unrounded values; the kable path rounds a display copy before rendering.
  tbl <- data.frame(
    Item1 = pc$Item1,
    Item2 = pc$Item2,
    Observed = as.numeric(observed),
    Low = as.numeric(low),
    High = as.numeric(high),
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  if (p_value) {
    sim_mat <- tapply(pr$Q3, list(pr$iteration, pr$key), function(x) x[1L])
    obs_named <- stats::setNames(as.numeric(observed), keys)
    pv <- .bootstrap_pvalues(
      obs_named,
      sim_mat,
      correction = correction,
      tail = "upper"
    )
    idx <- match(keys, pv$name)
    tbl$p_q3 <- pv$p[idx]
    tbl$padj_q3 <- pv$padj[idx]
    is_flagged <- !is.na(tbl$padj_q3) & tbl$padj_q3 < alpha
  } else {
    is_flagged <- observed > high | observed < low
  }

  # Direction relative to the per-pair simulated median.
  tbl$Flagged <- ifelse(
    !is_flagged,
    "",
    ifelse(observed > median_q3, "above", "below")
  )

  # Sort by absolute departure from expected, then keep the top n_pairs.
  ord <- order(abs(observed - median_q3), decreasing = TRUE)
  tbl <- tbl[ord, , drop = FALSE]
  rownames(tbl) <- NULL
  if (!is.null(n_pairs)) {
    tbl <- tbl[seq_len(min(as.integer(n_pairs), nrow(tbl))), , drop = FALSE]
  }

  if (output == "dataframe") {
    return(tbl)
  }

  # Kable display rounding (the dataframe output above stays unrounded)
  tbl <- .round_display(tbl, c(
    Observed = 3, Low = 3, High = 3, p_q3 = 4, padj_q3 = 4
  ))

  width_pct <- round(100 * cutoff_full$hdci_width, 1)
  caption <- paste0(
    "Q3 by item pair, sorted by departure from the expected range. ",
    "Expected range = ",
    width_pct,
    "% interval of the simulated Q3 per pair (",
    cutoff_full$actual_iterations,
    " iterations). Flagged: above = Q3 above ",
    "the upper bound (local dependence); below = below the lower bound."
  )
  if (p_value) {
    corr_label <- .correction_label(correction)
    caption <- paste0(
      caption,
      " p_q3/padj_q3: one-sided bootstrap p-values, ",
      corr_label,
      "; flagged at padj < ",
      alpha,
      "."
    )
    col.names <- c(
      "Item 1",
      "Item 2",
      "Observed Q3",
      "Exp. low",
      "Exp. high",
      "p",
      "p (adj)",
      "Flagged"
    )
  } else {
    col.names <- c(
      "Item 1",
      "Item 2",
      "Observed Q3",
      "Exp. low",
      "Exp. high",
      "Flagged"
    )
  }
  knitr::kable(
    tbl,
    format = "pipe",
    col.names = col.names,
    caption = caption,
    row.names = FALSE
  )
}

#' Simulation-Based \eqn{Q_3} Cutoff Determination
#'
#' Uses parametric bootstrap simulation to determine an appropriate cutoff
#' value for \code{\link{RMlocdepQ3}}. Under a correctly fitting Rasch model,
#' \eqn{Q_3} residuals have an unknown distribution; this function simulates
#' that distribution and returns empirical percentiles.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers).
#' @param iterations Integer. Number of simulation iterations (default 500).
#' @param parallel Logical. Use parallel processing via `mirai` if available
#'   (default `TRUE`).
#' @param n_cores Integer or `NULL`. Number of parallel workers. When `NULL`,
#'   `getOption("mc.cores")` is checked first. If neither is set and
#'   `parallel = TRUE`, a warning is issued and execution falls back to
#'   sequential (single core) processing.
#' @param verbose Logical. Show a progress bar (default `FALSE`).
#' @param seed Integer or `NULL`. Random seed for reproducibility.
#' @param cutoff_method Character. Method used to compute per-pair \eqn{Q_3}
#'   credible intervals in `pair_cutoffs`. One of `"hdci"` (the default,
#'   Highest Density Continuous Interval via `ggdist::hdci()`) or
#'   `"quantile"` (symmetric 2.5th / 97.5th percentiles). Only affects
#'   `pair_cutoffs`; the global `$suggested_cutoff` (99th percentile of
#'   `max(Q3) - mean(Q3)`) is unaffected.
#' @param hdci_width Numeric in (0, 1). Width of the HDCI when
#'   `cutoff_method = "hdci"`. Default `0.99`. Ignored when
#'   `cutoff_method = "quantile"`.
#' @param estimator Character. Estimation engine for the simulated \eqn{Q_3} values,
#'   passed through to the per-iteration computation. `"CML"` (default) uses
#'   CML item parameters and WLE person locations; `"MML"` uses `mirt`. This
#'   must match the `estimator` later given to \code{\link{RMlocdepQ3}}; the
#'   value is stored in the returned object's `$estimator` and reused
#'   automatically.
#' @param dgp Character. Data-generating process for the parametric bootstrap.
#'   `"resample"` (default) draws person locations by resampling the WLE
#'   estimates with replacement and simulates responses under the model -- a
#'   *marginal* null. `"conditional"` instead simulates each respondent's
#'   pattern from the exact Rasch conditional distribution given their observed
#'   total score (and answered items), with item parameters fixed -- a
#'   *conditional* null that fixes the score margin and needs no latent
#'   distribution, avoiding the over-dispersion of resampled point estimates.
#'   The two give different cut-offs; see the package's comparison study.
#'   \strong{Experimental.}
#'
#' @return A list with components:
#' \describe{
#'   \item{`results`}{data.frame with columns `mean`, `max`, `diff` (one row
#'     per successful iteration).}
#'   \item{`pair_results`}{Long data.frame with columns `Item1`, `Item2`,
#'     `Q3`, `iteration` --- one row per item pair per successful iteration.
#'     Used by \code{\link{RMlocdepQ3Plot}}.}
#'   \item{`pair_cutoffs`}{data.frame with per-pair cutoff summaries:
#'     `Item1`, `Item2`, `Q3_low`, `Q3_high`. Boundaries are computed via
#'     the method specified by `cutoff_method`.}
#'   \item{`actual_iterations`}{Number of successful iterations.}
#'   \item{`sample_n`}{Number of persons used: respondents with no responses
#'     at all (all-`NA` rows) are dropped, as in \code{\link{RMlocdepQ3}};
#'     incomplete response patterns are retained.}
#'   \item{`sample_n_total`}{Number of respondents in the raw input data,
#'     before dropping all-`NA` rows.}
#'   \item{`sample_has_na`}{Logical. Whether the data contained any missing
#'     values.}
#'   \item{`sample_summary`}{Summary statistics of estimated person parameters.}
#'   \item{`item_names`}{Character vector of item names from `data`.}
#'   \item{`max_diff`, `sd_diff`}{Max and SD of the `diff` distribution.}
#'   \item{`p95`, `p99`, `p995`, `p999`}{Empirical percentiles of `diff`.}
#'   \item{`suggested_cutoff`}{The 99th percentile (`p99`) --- recommended
#'     scalar cutoff for \code{\link{RMlocdepQ3}}.}
#'   \item{`cutoff_method`}{The method used for `pair_cutoffs`
#'     (`"hdci"` or `"quantile"`).}
#'   \item{`hdci_width`}{The HDCI width used (only meaningful when
#'     `cutoff_method = "hdci"`).}
#'   \item{`estimator`}{The estimator used for the simulated \eqn{Q_3} (`"CML"` or
#'     `"MML"`); reused by \code{\link{RMlocdepQ3}} and
#'     \code{\link{RMlocdepQ3Plot}}.}
#'   \item{`dgp`}{The data-generating process used (`"resample"` or
#'     `"conditional"`).}
#' }
#'
#' @details
#' The generating model is fitted once: CML item parameters (via
#' `psychotools`) and WLE person locations. For each simulation iteration,
#' those WLE thetas are resampled with replacement, response data are simulated
#' under the Rasch / Partial Credit model, the model is refitted, and \eqn{Q_3}
#' residuals are computed under `estimator`. The distribution of
#' `max(Q3) - mean(Q3)` across iterations provides empirical critical values.
#' Failed iterations (e.g., due to convergence issues) are silently discarded.
#'
#' Supports both **dichotomous** data (simulated via `psychotools::rrm()`) and
#' **polytomous** data (via an internal partial credit score simulator).
#'
#' Parallel processing is provided by the `mirai` package (optional). Install
#' it with `install.packages("mirai")` to enable parallelisation.
#'
#' @seealso \code{\link{RMlocdepQ3}}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("ggdist", quietly = TRUE)) {
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:10)
#'
#'   # Few iterations for a fast example; use 500+ in real analyses
#'   cutoff_res <- RMlocdepQ3Cutoff(sim_data, iterations = 50, parallel = FALSE,
#'                                  seed = 42)
#'   cutoff_res$suggested_cutoff  # 99th percentile
#'
#'   # Use the cutoff in RMlocdepQ3()
#'   RMlocdepQ3(sim_data, cutoff = cutoff_res$suggested_cutoff)
#' }
#' }
RMlocdepQ3Cutoff <- function(
  data,
  iterations = 500,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  cutoff_method = "hdci",
  hdci_width = 0.99,
  estimator = c("CML", "MML"),
  dgp = c("resample", "conditional")
) {
  validate_response_data(data)

  estimator <- match.arg(estimator)
  dgp <- match.arg(dgp)
  cutoff_method <- match.arg(cutoff_method, c("hdci", "quantile"))
  if (cutoff_method == "hdci" && !requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required when cutoff_method = \"hdci\" but is not installed.\n",
      "Install it with: install.packages(\"ggdist\")\n",
      "Alternatively, use cutoff_method = \"quantile\" to avoid this dependency.",
      call. = FALSE
    )
  }
  if (
    !is.numeric(hdci_width) ||
      length(hdci_width) != 1L ||
      !is.finite(hdci_width) ||
      hdci_width <= 0 ||
      hdci_width >= 1
  ) {
    stop("`hdci_width` must be a single number in (0, 1).", call. = FALSE)
  }

  use_parallel <- parallel && requireNamespace("mirai", quietly = TRUE)

  if (parallel && !use_parallel) {
    message(
      "Install 'mirai' package for parallel processing: install.packages(\"mirai\")"
    )
    message("Running sequentially...")
  }

  if (use_parallel) {
    if (is.null(n_cores)) {
      n_cores <- getOption("mc.cores")
    }
    if (is.null(n_cores)) {
      warning(
        paste0(
          "For parallel processing, specify n_cores or set options(mc.cores = N).\n",
          "(Use `parallel::detectCores()` to see how many cores are available.)\n",
          "Falling back to sequential (single core) processing."
        ),
        call. = FALSE
      )
      use_parallel <- FALSE
    } else {
      n_cores <- min(n_cores, iterations)
    }
  }

  if (!is.null(seed)) {
    set.seed(seed)
  }

  # Generate per-iteration seeds
  sim_seeds <- sample.int(.Machine$integer.max, iterations)

  # Incomplete responses are retained (they feed the per-pattern WLE pool and
  # the conditional DGP), but respondents with no responses at all must go:
  # they contribute nothing and break the CML fit (psychotools errors on
  # all-NA rows). Record the raw total and the missingness flag first so
  # callers (e.g. RMlocdepQ3Plot) can report the sample in the standard
  # `n = X of Y respondents (policy)` form, matching RMlocdepQ3().
  sample_n_total <- nrow(as.data.frame(data))
  sample_has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)

  data_mat <- as.matrix(data)
  sample_n <- nrow(data_mat)
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L

  # Preserve item names so the per-iteration pair_q3 frames use the user's
  # labels (e.g., "q1", "q2") rather than psychotools / mirt's auto-generated
  # "V1", "V2", ... .
  item_names_sim <- colnames(data_mat)
  if (is.null(item_names_sim)) {
    item_names_sim <- paste0("V", seq_len(ncol(data_mat)))
  }

  # Generating model: CML item thresholds (psychotools), computed once. WLE
  # person locations are computed for the sample summary and, under the
  # "resample" DGP, as the generating theta pool.
  pool <- .wle_theta_pool(data_mat)
  thr_list <- pool$thr_list
  wle_thetas <- pool$thetas

  sim_data_list <- list(
    dgp = dgp,
    type = if (is_polytomous) "polytomous" else "dichotomous",
    thr_list = thr_list,
    n_items = ncol(data_mat),
    sample_n = sample_n,
    item_names = item_names_sim,
    estimator = estimator
  )
  if (dgp == "resample") {
    # Marginal DGP: resample WLE thetas with replacement, then simulate data
    # parametrically under the model (items and thetas on a common scale).
    sim_data_list$thetas <- wle_thetas
    if (is_polytomous) {
      sim_data_list$deltaslist <- thr_list
    } else {
      sim_data_list$item_params <- unlist(thr_list, use.names = FALSE)
    }
  } else {
    # Conditional DGP: simulate each respondent's pattern from the exact Rasch
    # conditional distribution given their observed total score (and answered
    # items), item parameters fixed. No latent distribution is estimated, so
    # the sufficient statistic -- not a resampled point estimate -- defines the
    # null. Respondents are grouped by (answered-set, score) so each group is
    # drawn in one batch.
    sim_data_list$cond_groups <- .cond_groups(data_mat, thr_list)
  }

  if (use_parallel) {
    results_raw <- run_q3_sim_parallel(
      iterations,
      sim_seeds,
      sim_data_list,
      n_cores,
      verbose
    )
  } else {
    results_raw <- run_q3_sim_sequential(
      iterations,
      sim_seeds,
      sim_data_list,
      verbose
    )
  }

  # Filter out failures (character strings indicate errors)
  ok <- vapply(results_raw, function(x) is.list(x), logical(1L))
  successful <- results_raw[ok]

  if (length(successful) == 0L) {
    stop("All simulation iterations failed. Check your data.", call. = FALSE)
  }

  actual_iterations <- length(successful)
  mean_q3 <- vapply(successful, function(x) x$mean, numeric(1L))
  max_q3 <- vapply(successful, function(x) x$max, numeric(1L))
  diff_q3 <- max_q3 - mean_q3

  results <- data.frame(
    mean = mean_q3,
    max = max_q3,
    diff = diff_q3
  )

  # --- Per-pair aggregation -------------------------------------------------
  # Stack per-iteration pair_q3 frames into one long data.frame and add an
  # `iteration` column (matching the partial-gamma family).
  item_names_vec <- colnames(data)
  if (is.null(item_names_vec)) {
    item_names_vec <- as.character(seq_len(ncol(data)))
  }
  pair_iter_dfs <- lapply(seq_along(successful), function(i) {
    df <- successful[[i]]$pair_q3
    df$iteration <- i
    df
  })
  pair_results <- do.call(rbind, pair_iter_dfs)
  rownames(pair_results) <- NULL

  # Per-pair cutoff intervals
  pair_keys <- unique(paste(
    pair_results$Item1,
    pair_results$Item2,
    sep = "___"
  ))
  pair_cutoffs <- do.call(
    rbind,
    lapply(pair_keys, function(pk) {
      parts <- strsplit(pk, "___", fixed = TRUE)[[1L]]
      sub <- pair_results[
        pair_results$Item1 == parts[1L] &
          pair_results$Item2 == parts[2L],
      ]
      if (cutoff_method == "hdci") {
        q3_interval <- ggdist::hdci(sub$Q3, .width = hdci_width)
        data.frame(
          Item1 = parts[1L],
          Item2 = parts[2L],
          Q3_low = q3_interval[1L, 1L],
          Q3_high = q3_interval[1L, 2L],
          stringsAsFactors = FALSE,
          row.names = NULL
        )
      } else {
        data.frame(
          Item1 = parts[1L],
          Item2 = parts[2L],
          Q3_low = stats::quantile(sub$Q3, 0.025, na.rm = TRUE),
          Q3_high = stats::quantile(sub$Q3, 0.975, na.rm = TRUE),
          stringsAsFactors = FALSE,
          row.names = NULL
        )
      }
    })
  )
  rownames(pair_cutoffs) <- NULL

  out <- list()
  out$results <- results
  out$pair_results <- pair_results
  out$pair_cutoffs <- pair_cutoffs
  out$actual_iterations <- actual_iterations
  out$sample_n <- sample_n
  out$sample_n_total <- sample_n_total
  out$sample_has_na <- sample_has_na
  out$sample_summary <- summary(wle_thetas)
  out$item_names <- item_names_vec
  out$max_diff <- max(results$diff)
  out$sd_diff <- stats::sd(results$diff)
  out$p95 <- stats::quantile(results$diff, 0.95)
  out$p99 <- stats::quantile(results$diff, 0.99)
  out$p995 <- stats::quantile(results$diff, 0.995)
  out$p999 <- stats::quantile(results$diff, 0.999)
  out$suggested_cutoff <- out$p99
  out$cutoff_method <- cutoff_method
  out$hdci_width <- hdci_width
  out$estimator <- estimator
  out$dgp <- dgp
  out
}

# ---------------------------------------------------------------------------
# Internal: single simulation iteration
# ---------------------------------------------------------------------------

#' Group respondents by sufficient statistic for the conditional DGP
#'
#' Shared by the conditional-DGP cutoff simulators (`RMlocdepQ3Cutoff()`,
#' `RMitemInfitCutoff()`, ...). Returns one element per distinct (answered-item
#' set, total score) so each group can be drawn in a single
#' `.sim_conditional()` call.
#'
#' @param data_mat Observed response matrix.
#' @param thr_list Item thresholds (unused here but kept for symmetry).
#' @return List of `list(rows, ans, score)` groups.
#' @keywords internal
#' @noRd
.cond_groups <- function(data_mat, thr_list) {
  n <- nrow(data_mat)
  ans_key <- apply(!is.na(data_mat), 1L, function(z) {
    paste0(which(z), collapse = ",")
  })
  score <- rowSums(data_mat, na.rm = TRUE)
  key <- paste(ans_key, score, sep = "|")
  unname(lapply(split(seq_len(n), key), function(rows) {
    ans <- which(!is.na(data_mat[rows[1L], ]))
    list(rows = rows, ans = ans, score = sum(data_mat[rows[1L], ans]))
  }))
}

#' Simulate one conditional-DGP dataset
#'
#' For each (answered-set, score) group, draw the required number of response
#' patterns from the Rasch conditional distribution given the total score
#' (`.sim_conditional()`, in person_fit.R); extreme scores (0 or maximum) yield
#' their unique degenerate pattern. The observed missingness pattern is
#' preserved (unanswered cells stay `NA`). Shared by the conditional-DGP cutoff
#' simulators.
#'
#' @param data_list A simulation list with `dgp = "conditional"` (carries
#'   `thr_list`, `cond_groups`, `sample_n`, `n_items`).
#' @return A data.frame of simulated responses.
#' @keywords internal
#' @noRd
.sim_cond_dataset <- function(data_list) {
  N <- data_list$sample_n
  K <- data_list$n_items
  out <- matrix(NA_integer_, N, K)
  for (g in data_list$cond_groups) {
    thr_sub <- data_list$thr_list[g$ans]
    m <- length(g$rows)
    pat <- .sim_conditional(thr_sub, g$score, m)
    if (is.null(pat)) {
      # Extreme score: deterministic pattern (all-0 or all-maximum-category).
      steps <- vapply(thr_sub, length, integer(1L))
      pat <- if (g$score <= 0L) {
        matrix(0L, m, length(g$ans))
      } else {
        matrix(steps, nrow = m, ncol = length(g$ans), byrow = TRUE)
      }
    }
    out[g$rows, g$ans] <- pat
  }
  as.data.frame(out)
}

#' Run a single Q3 simulation iteration
#'
#' @param seed Integer seed for reproducibility.
#' @param data_list List produced inside [RMlocdepQ3Cutoff()].
#' @return A list with `mean` and `max` Q3, or a character string on failure.
#' @keywords internal
run_single_q3_sim <- function(seed, data_list) {
  set.seed(seed)

  tryCatch(
    {
      # --- Generate one simulated dataset under the chosen DGP -----------------
      if (identical(data_list$dgp, "conditional")) {
        sim_df <- .sim_cond_dataset(data_list)
      } else if (data_list$type == "dichotomous") {
        thetas_res <- sample(
          data_list$thetas,
          size = data_list$sample_n,
          replace = TRUE
        )
        sim_df <- as.data.frame(
          psychotools::rrm(
            theta = thetas_res,
            beta = data_list$item_params
          )$data
        )
      } else {
        thetas_res <- sample(
          data_list$thetas,
          size = data_list$sample_n,
          replace = TRUE
        )
        sim_df <- as.data.frame(sim_partial_score(
          data_list$deltaslist,
          thetas_res
        ))
      }

      # --- Validate the simulated dataset (estimable refit) --------------------
      if (data_list$type == "dichotomous") {
        # Every item must have at least 8 positive responses (numerical stability).
        if (any(colSums(sim_df, na.rm = TRUE) < 8L)) {
          return(
            "validation_failed: fewer than 8 positive responses in at least one item"
          )
        }
      } else {
        # Every item must show all categories.
        n_cats <- vapply(
          data_list$thr_list,
          function(d) length(d) + 1L,
          integer(1L)
        )
        for (j in seq_len(ncol(sim_df))) {
          tab <- tabulate(sim_df[[j]] + 1L, nbins = n_cats[j])
          if (any(tab == 0L)) {
            return("validation_failed: not all categories represented")
          }
        }
      }

      # Preserve the user's item labels so the Q3 matrix below uses them
      if (
        !is.null(data_list$item_names) &&
          length(data_list$item_names) == ncol(sim_df)
      ) {
        colnames(sim_df) <- data_list$item_names
      }

      # Q3 matrix under the same estimator as the observed analysis. For "MML"
      # the faster (lower-precision) mirt settings are used in the loop.
      q3_mat <- .q3_residual_matrix(
        sim_df,
        estimator = data_list$estimator,
        fast = TRUE
      )

      mean_q3 <- mean(q3_mat, na.rm = TRUE)
      max_q3 <- max(q3_mat, na.rm = TRUE)

      # Extract upper triangle as a long vector for per-pair retention.
      # mirt::residuals returns a symmetric matrix with item names on rows/cols.
      item_names_q3 <- colnames(q3_mat)
      if (is.null(item_names_q3)) {
        item_names_q3 <- as.character(seq_len(ncol(q3_mat)))
      }
      upper_idx <- which(upper.tri(q3_mat), arr.ind = TRUE)
      pair_q3 <- data.frame(
        Item1 = item_names_q3[upper_idx[, "row"]],
        Item2 = item_names_q3[upper_idx[, "col"]],
        Q3 = q3_mat[upper_idx],
        stringsAsFactors = FALSE,
        row.names = NULL
      )

      list(mean = mean_q3, max = max_q3, pair_q3 = pair_q3)
    },
    error = function(e) {
      as.character(conditionMessage(e))
    }
  )
}

# ---------------------------------------------------------------------------
# Internal: parallel runner
# ---------------------------------------------------------------------------

#' Run Q3 simulations in parallel using mirai
#'
#' @param iterations Number of iterations.
#' @param sim_seeds Integer vector of per-iteration seeds.
#' @param sim_data_list List of data passed to each worker.
#' @param n_cores Number of mirai daemons.
#' @param verbose Show progress bar.
#' @return List of raw results (one element per iteration).
#' @keywords internal
run_q3_sim_parallel <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  n_cores,
  verbose = FALSE
) {
  mirai::daemons(n_cores)
  on.exit(mirai::daemons(0), add = TRUE)

  if (verbose) {
    message(sprintf("Starting %d daemons...", n_cores))
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
    completed <- 0L
  }

  # Submit all tasks
  tasks <- lapply(seq_len(iterations), function(sim) {
    mirai::mirai(
      {
        run_single_q3_sim(seed, data_list)
      },
      seed = sim_seeds[sim],
      data_list = sim_data_list,
      run_single_q3_sim = run_single_q3_sim,
      sim_partial_score = sim_partial_score,
      sim_poly_item = sim_poly_item,
      # Conditional-DGP generators.
      .sim_cond_dataset = .sim_cond_dataset,
      .sim_conditional = .sim_conditional,
      .esf_convolve = .esf_convolve,
      # Engine helpers needed by .q3_residual_matrix() inside the daemon
      # (the CML/WLE path; the MML path uses namespaced mirt::).
      .q3_residual_matrix = .q3_residual_matrix,
      .rasch_std_residuals = .rasch_std_residuals,
      .fit_cml_thresholds = .fit_cml_thresholds,
      .estimate_thetas = .estimate_thetas,
      .theta_wle = .theta_wle,
      .pcm_cat_probs = .pcm_cat_probs,
      .center_thresholds = .center_thresholds
    )
  })

  # Collect results
  results <- vector("list", iterations)
  for (sim in seq_len(iterations)) {
    result <- mirai::call_mirai(tasks[[sim]])$data
    if (!inherits(result, "errorValue")) {
      results[[sim]] <- result
    } else {
      results[[sim]] <- "mirai_error"
    }
    if (verbose) {
      completed <- completed + 1L
      utils::setTxtProgressBar(pb, completed)
    }
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# ---------------------------------------------------------------------------
# Internal: sequential runner
# ---------------------------------------------------------------------------

#' Run Q3 simulations sequentially
#'
#' @param iterations Number of iterations.
#' @param sim_seeds Integer vector of per-iteration seeds.
#' @param sim_data_list List of data passed to each worker.
#' @param verbose Show progress bar.
#' @return List of raw results (one element per iteration).
#' @keywords internal
run_q3_sim_sequential <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  verbose = FALSE
) {
  if (verbose) {
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  results <- vector("list", iterations)
  for (sim in seq_len(iterations)) {
    results[[sim]] <- run_single_q3_sim(sim_seeds[sim], sim_data_list)
    if (verbose) {
      utils::setTxtProgressBar(pb, sim)
    }
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

#' Person-Fit Statistics for a Rasch / Partial Credit Model
#'
#' Computes per-respondent person-fit statistics and resampling-based
#' p-values. Three statistics are reported: conditional **infit** and
#' **outfit** mean-squares (MSQ) -- the magnitude (effect-size) measures
#' familiar from Rasch analysis -- and the standardized log-likelihood
#' statistic **lz**. The conditional MSQ statistics use response
#' probabilities conditional on the total score, which require no person
#' estimate and are therefore unbiased (Kreiner & Christensen, 2011); they are
#' computed on each person's observed response pattern, so partial
#' missingness is handled directly.
#'
#' Statistical significance is assessed by Monte-Carlo resampling under
#' the fitted model rather than by an assumed asymptotic distribution.
#' The asymptotic null distributions of MSQ and lz are known to be
#' unreliable -- the Wilson-Hilferty (ZSTD) transformation of MSQ in
#' particular adds nothing once conditional estimation is used (Müller,
#' 2020) -- so resampling provides the valid reference (Sinharay, 2016).
#'
#' @param data A data.frame or matrix of item responses. Items must be
#'   scored starting at 0 (non-negative integers). Missing values (`NA`)
#'   are allowed.
#' @param statistics Character vector. Which statistics to report; any of
#'   `"infit"`, `"outfit"`, `"lz"`. Defaults to all three.
#' @param estimator Character. How item parameters are estimated:
#'   `"CML"` (default) or `"MML"` (via \pkg{mirt}).
#' @param theta_method Character. Person-location estimator used for the
#'   lz statistic: `"WLE"` (default) or `"EAP"`. Ignored if `"lz"` is not
#'   requested.
#' @param iterations Integer. Number of Monte-Carlo replications per
#'   person. `0` skips resampling and returns statistics only. Default
#'   `500`.
#' @param flag_alpha Numeric in (0, 1). Significance level for the `flagged`
#'   column and the plot highlighting. Default `0.05`.
#' @param flag Character. Which misfit direction drives flagging for the MSQ
#'   statistics: `"both"` (default) flags either direction with a two-sided
#'   p-value; `"underfit"` flags only underfit (MSQ > 1, noisy responding)
#'   with a one-sided upper p-value, which is more powerful for detecting it
#'   and ignores (benign) overfit. The reported `p_infit` / `p_outfit` follow
#'   this choice. lz is one-sided regardless. Overfit can occasionally be
#'   suspicious (e.g. fabricated or copied response patterns), so `"both"` is
#'   the default.
#' @param zstd Logical. If `TRUE`, also report the Wilson-Hilferty
#'   standardized (ZSTD) versions of the MSQ statistics. These are
#'   provided only as a familiar cross-person comparability metric and
#'   are **not** used for inference (see Müller, 2020). Default `FALSE`.
#' @param parallel,n_cores Logical / integer. Parallelise the resampling
#'   across persons via \pkg{mirai} when available. Default sequential.
#' @param seed Optional integer for reproducible resampling.
#' @param output Character. `"kable"` (default), `"dataframe"`,
#'   `"ggplot"`, or `"list"`. For `"ggplot"`, a **named list** of
#'   person-fit maps is returned -- one per requested statistic (e.g.
#'   `$infit`, `$outfit`, `$lz`) -- each plotting the statistic against
#'   person location, with
#'   respondents flagged by *that* statistic highlighted and a caption
#'   reporting the assessed sample size and the proportion flagged. For the
#'   MSQ maps the flagged count is split into underfit (MSQ > 1, noisy/erratic
#'   responding) and overfit (MSQ < 1, overly deterministic responding).
#'
#' @return
#' For `output = "dataframe"`, a data.frame with one row per respondent
#' (input order): `id`, `n_answered`, `sum_score`, the requested
#' statistics (`infit_msq`, `outfit_msq`, `lz`), their resampled
#' p-values (`p_infit`, `p_outfit`, `p_lz`) when `iterations > 0`,
#' `flagged`, and -- if `zstd = TRUE` -- `infit_zstd`, `outfit_zstd`.
#' The `flagged` column is `TRUE` when the marginal (uncorrected) p-value of
#' **any** requested statistic is below `flag_alpha` for that respondent; it
#' is therefore a per-person screening flag, not corrected for the number of
#' respondents tested (so under fit expect about `flag_alpha` of respondents
#' to be flagged by chance). Each `output = "ggplot"` map instead colours and
#' counts respondents flagged by its **own** statistic alone. Extreme scorers
#' (minimum or maximum possible given the items they answered) receive `NA`
#' statistics and are not assessed. For `output = "kable"` the same content as
#' a `knitr_kable`; for `output = "ggplot"` the named list of maps described
#' under that argument; for `output = "list"`, a list with both views from a
#' single computation: `fit` (the data.frame) and `plots` (the named list of
#' maps).
#'
#' @details
#' \strong{Conditional MSQ.} For person \eqn{v} with answered items
#' \eqn{A_v} and total score \eqn{r_v}, the conditional residual is
#' \eqn{z_{vi} = (x_{vi} - E(X_{vi} \mid R_v = r_v)) /
#' \sqrt{\mathrm{Var}(X_{vi} \mid R_v = r_v)}}, with conditional moments
#' obtained from elementary symmetric functions of the answered items'
#' parameters. Outfit is the unweighted mean of \eqn{z_{vi}^2}; infit is
#' the information-weighted mean. Because the conditional moments use only
#' item parameters, no (biased) person estimate enters the residual.
#'
#' \strong{Interpreting MSQ direction.} Values near 1 indicate fit. MSQ > 1
#' (\emph{underfit}) means the responses are noisier / more erratic than the
#' model expects -- the validity-relevant direction, indicating careless or
#' aberrant responding. MSQ < 1 (\emph{overfit}) means responses are overly
#' deterministic (Guttman-like); usually benign for score validity, though a
#' suspiciously perfect pattern can occasionally signal copied or fabricated
#' data. Use `flag = "underfit"` to flag only the former. lz is one-sided:
#' low (negative) values flag the aberrant (underfit-like) direction.
#'
#' \strong{lz.} The standardized log-likelihood of the response pattern
#' evaluated at the estimated person location (Drasgow, Levine &
#' Williams, 1985); small (negative) values indicate misfit. Because the
#' location is estimated rather than known, the variance of lz is below 1
#' and its asymptotic standard-normal null is invalid, which makes a
#' naive test conservative (Snijders, 2001; Sinharay, 2016). Instead of
#' applying the analytic lz\* standardization -- derived by Snijders (2001)
#' for dichotomous items and extended to polytomous / mixed-format items by
#' Sinharay (2016, 2026) -- easyRasch2 obtains the reference distribution by
#' resampling with the person location \strong{re-estimated for every
#' simulated pattern} (see \strong{Resampling}). This reproduces the
#' ability-estimation effect and is the resampling analogue of lz\*, so the
#' resampled p-value is well calibrated.
#'
#' \strong{Resampling.} Each statistic is referenced against the null that
#' matches it, removing the need to choose a scheme. The conditional MSQ
#' statistics use patterns sampled **conditional on the person's total
#' score** -- the exact Rasch-native null, requiring no person estimate and
#' fully consistent with the (conditional) statistic. lz, which is defined at
#' the estimated location, uses patterns **simulated at that location and then
#' scored with the location re-estimated from each simulated pattern**, so the
#' null carries the same ability-estimation effect as the observed lz (for a
#' Rasch / PCM model the location is a function of the total score, so this
#' re-estimation is a cheap score-based lookup). Both schemes are computed per
#' person over the items actually answered, so partial missingness is handled
#' by either. The p-value is the proportion of the `iterations` replicates at
#' least as extreme as the observed value. This follows the resampling-based
#' person-fit approach (Sinharay, 2016) and the bootstrap recommendation of
#' Müller (2020).
#'
#' @references
#' de la Torre, J., & Deng, W. (2008). Improving person-fit assessment by
#' correcting the ability estimate and its reference distribution.
#' *Journal of Educational Measurement, 45*(2), 159-177.
#' \doi{10.1111/j.1745-3984.2008.00058.x}
#'
#' Drasgow, F., Levine, M. V., & Williams, E. A. (1985). Appropriateness
#' measurement with polychotomous item response models and standardized
#' indices. *British Journal of Mathematical and Statistical Psychology,
#' 38*(1), 67-86. \doi{10.1111/j.2044-8317.1985.tb00817.x}
#'
#' Kreiner, S., & Christensen, K. B. (2011). Exact evaluation of bias in
#' Rasch model residuals. *Advances in Mathematics Research, 12*,
#' 19-40.
#'
#' Müller, M. (2020). Item fit statistics for Rasch analysis: can we
#' trust them? *Journal of Statistical Distributions and Applications,
#' 7*(5). \doi{10.1186/s40488-020-00108-7}
#'
#' Sinharay, S. (2016). Assessment of person fit using resampling-based
#' approaches. *Journal of Educational Measurement, 53*(1), 63-85.
#' \doi{10.1111/jedm.12101}
#'
#' Sinharay, S. (2016). Asymptotically correct standardization of
#' person-fit statistics beyond dichotomous items. *Psychometrika,
#' 81*(4), 992-1013. \doi{10.1007/s11336-015-9465-x}
#'
#' Sinharay, S. (2026). Refining the asymptotically correct
#' standardization of person-fit statistics for mixed-format tests.
#' *British Journal of Mathematical and Statistical Psychology*.
#' \doi{10.1111/bmsp.70049}
#'
#' Snijders, T. A. B. (2001). Asymptotic null distribution of person fit
#' statistics with estimated person parameter. *Psychometrika, 66*(3),
#' 331-342. \doi{10.1007/BF02294440}
#'
#' @seealso [RMpersonParameters()], [RMitemInfit()], [RMitemParameters()]
#'
#' @importFrom rlang .data
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' dat <- as.data.frame(
#'   matrix(sample(0:2, 200 * 8, replace = TRUE), nrow = 200, ncol = 8)
#' )
#' colnames(dat) <- paste0("Item", 1:8)
#'
#' # Conditional infit/outfit MSQ + lz with resampled p-values
#' RMpersonFit(dat, iterations = 200, output = "dataframe") |> head()
#'
#' # Person-fit maps: a named list with one plot per statistic
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'   plots <- RMpersonFit(dat, iterations = 200, output = "ggplot")
#'   plots$infit    # infit map (each plot's caption reports the % flagged)
#' }
#'
#' # Flag only underfit (noisy responding), the validity-relevant direction
#' RMpersonFit(dat, iterations = 200, flag = "underfit",
#'             output = "dataframe") |> head()
#' }
RMpersonFit <- function(
  data,
  statistics = c("infit", "outfit", "lz"),
  estimator = c("CML", "MML"),
  theta_method = c("WLE", "EAP"),
  iterations = 500L,
  flag_alpha = 0.05,
  flag = c("both", "underfit"),
  zstd = FALSE,
  parallel = FALSE,
  n_cores = NULL,
  seed = NULL,
  output = c("kable", "dataframe", "ggplot", "list")
) {
  statistics <- match.arg(
    statistics,
    c("infit", "outfit", "lz"),
    several.ok = TRUE
  )
  estimator <- match.arg(estimator)
  theta_method <- match.arg(theta_method)
  flag <- match.arg(flag)
  output <- match.arg(output)
  # Underfit-only flagging uses a one-sided (upper) MSQ test; lz is always
  # one-sided (low = aberrant) and is unaffected.
  msq_tail <- if (flag == "underfit") "upper" else "two.sided"

  validate_response_data(data)
  if (
    !is.numeric(flag_alpha) ||
      length(flag_alpha) != 1L ||
      flag_alpha <= 0 ||
      flag_alpha >= 1
  ) {
    stop("`flag_alpha` must be a single number in (0, 1).", call. = FALSE)
  }
  if (!is.null(seed)) {
    set.seed(seed)
  }

  data <- as.data.frame(data)
  n_total <- nrow(data)
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)

  # --- Item parameters --------------------------------------------------------
  fit <- if (estimator == "CML") {
    .rasch_fit_cml(data, se = FALSE)
  } else {
    .rasch_fit_mml(data, se = FALSE)
  }
  # Grand-mean-zero identification so the lz location matches
  # RMpersonParameters(); conditional MSQ is invariant to this shift.
  thr_list <- .center_thresholds(fit$thr_list)
  data <- .align_items(data, thr_list)
  data_mat <- as.matrix(data)
  n <- nrow(data_mat)

  # --- Person bookkeeping -----------------------------------------------------
  n_answered <- rowSums(!is.na(data_mat))
  sum_score <- rowSums(data_mat, na.rm = TRUE)
  max_score <- vapply(
    seq_len(n),
    function(p) {
      ans <- !is.na(data_mat[p, ])
      sum(vapply(thr_list[ans], length, integer(1)))
    },
    numeric(1)
  )
  extreme <- n_answered > 0L & (sum_score == 0 | sum_score == max_score)

  # --- Person location -------------------------------------------------------
  # Needed for lz (both the statistic and its theta-based resampling) and for
  # the ggplot x-axis. The MSQ statistics and their conditional resampling
  # need no person estimate.
  need_theta <- ("lz" %in% statistics) || (output %in% c("ggplot", "list"))
  theta <- rep(NA_real_, n)
  if (need_theta) {
    # Shared engine (utils-theta.R): WLE caches by (answered-set, score); EAP
    # uses its quadrature grid and estimates the prior SD from the data. The
    # wider c(-10, 10) range locates extreme scorers (person-reporting scale).
    theta <- .estimate_thetas(
      data_mat,
      thr_list,
      method = theta_method,
      theta_range = c(-10, 10)
    )$theta
  }

  # --- Observed statistics ----------------------------------------------------
  obs <- .person_fit_stats(data_mat, thr_list, theta, statistics)

  out <- data.frame(
    id = if (!is.null(rownames(data))) rownames(data) else seq_len(n),
    n_answered = n_answered,
    sum_score = sum_score,
    stringsAsFactors = FALSE
  )
  if ("infit" %in% statistics) {
    out$infit_msq <- obs$infit
  }
  if ("outfit" %in% statistics) {
    out$outfit_msq <- obs$outfit
  }
  if ("lz" %in% statistics) {
    out$lz <- obs$lz
  }

  # --- Resampling p-values ----------------------------------------------------
  if (iterations > 0L) {
    pv <- .person_fit_resample(
      data_mat,
      thr_list,
      theta,
      statistics,
      obs,
      iterations = iterations,
      theta_method = theta_method,
      parallel = parallel,
      n_cores = n_cores,
      msq_tail = msq_tail
    )
    if ("infit" %in% statistics) {
      out$p_infit <- pv$infit
    }
    if ("outfit" %in% statistics) {
      out$p_outfit <- pv$outfit
    }
    if ("lz" %in% statistics) {
      out$p_lz <- pv$lz
    }

    p_cols <- intersect(c("p_infit", "p_outfit", "p_lz"), names(out))
    out$flagged <- apply(out[p_cols], 1L, function(p) {
      any(p < flag_alpha, na.rm = TRUE)
    })
  }

  # --- Optional ZSTD (non-inferential) ----------------------------------------
  if (zstd) {
    if ("infit" %in% statistics) {
      out$infit_zstd <- .msq_to_zstd(obs$infit, n_answered)
    }
    if ("outfit" %in% statistics) {
      out$outfit_zstd <- .msq_to_zstd(obs$outfit, n_answered)
    }
  }

  # Extreme scorers carry no fit information
  stat_cols <- intersect(
    c(
      "infit_msq",
      "outfit_msq",
      "lz",
      "p_infit",
      "p_outfit",
      "p_lz",
      "infit_zstd",
      "outfit_zstd"
    ),
    names(out)
  )
  out[extreme, stat_cols] <- NA
  if ("flagged" %in% names(out)) {
    out$flagged[extreme] <- NA
  }

  .person_fit_output(
    out,
    theta,
    extreme,
    statistics,
    output,
    flag_alpha,
    iterations,
    flag,
    n_total = n_total,
    has_na = has_na
  )
}

# ---------------------------------------------------------------------
# Internal: conditional moments by total score (missingness-aware core)
# ---------------------------------------------------------------------

#' Conditional expected score and variance by total score
#'
#' For a set of items (a single missingness pattern), returns lookup
#' matrices of \eqn{E(X_i \mid R = r)} and \eqn{\mathrm{Var}(X_i \mid
#' R = r)} for every attainable total score \eqn{r}, computed from
#' elementary symmetric functions of the item parameters.
#'
#' @param thr_sub List of threshold vectors for the answered items.
#' @return List with `E` and `V` matrices (rows = total score 0..max,
#'   cols = items) and `max_score`.
#' @keywords internal
#' @noRd
.cond_moments <- function(thr_sub) {
  steps <- vapply(thr_sub, length, integer(1))
  is_poly <- any(steps > 1L)
  k <- length(thr_sub)
  max_sc <- sum(steps)

  pars <- if (is_poly) lapply(thr_sub, cumsum) else unlist(thr_sub)
  esf <- psychotools::elementary_symmetric_functions(pars, order = 1L)
  g <- esf[[1L]]
  dg <- esf[[2L]]

  E <- matrix(NA_real_, max_sc + 1L, k)
  V <- matrix(NA_real_, max_sc + 1L, k)

  if (!is_poly) {
    for (r in seq_len(max_sc - 1L)) {
      # 1..max-1 (extremes degenerate)
      e <- dg[r + 1L, ] / g[r + 1L]
      E[r + 1L, ] <- e
      V[r + 1L, ] <- e * (1 - e)
    }
  } else {
    col0 <- c(0L, cumsum(steps))
    for (r in seq_len(max_sc - 1L)) {
      for (i in seq_len(k)) {
        cols <- (col0[i] + 1L):col0[i + 1L]
        p_ge <- dg[r + 1L, cols] / g[r + 1L] # P(X_i = c | R = r), c = 1..m_i
        p <- c(1 - sum(p_ge), p_ge)
        cats <- 0:steps[i]
        ev <- sum(cats * p)
        E[r + 1L, i] <- ev
        V[r + 1L, i] <- sum(cats^2 * p) - ev^2
      }
    }
  }
  list(E = E, V = V, max_score = max_sc)
}

#' Conditional residuals for one response matrix (grouped by NA pattern)
#'
#' @param data_mat Numeric response matrix.
#' @param thr_list Threshold list aligned with the columns.
#' @return List of length nrow with, per person, the conditional
#'   residuals `z` and variance weights `w` for their answered items;
#'   `NULL` for extreme/empty rows.
#' @keywords internal
#' @noRd
.conditional_residuals <- function(data_mat, thr_list) {
  n <- nrow(data_mat)
  res <- vector("list", n)

  pattern <- apply(!is.na(data_mat), 1L, function(a) {
    paste0(which(a), collapse = ",")
  })
  for (pat in unique(pattern)) {
    rows <- which(pattern == pat)
    items <- as.integer(strsplit(pat, ",", fixed = TRUE)[[1L]])
    if (length(items) == 0L) {
      next
    }
    mom <- .cond_moments(thr_list[items])

    for (p in rows) {
      x <- data_mat[p, items]
      r <- sum(x)
      if (r == 0L || r == mom$max_score) {
        next
      } # extreme: no information
      e <- mom$E[r + 1L, ]
      v <- mom$V[r + 1L, ]
      z <- (x - e) / sqrt(v)
      res[[p]] <- list(z = z, w = v)
    }
  }
  res
}

# ---------------------------------------------------------------------
# Internal: observed statistics
# ---------------------------------------------------------------------

#' @keywords internal
#' @noRd
.person_fit_stats <- function(data_mat, thr_list, theta, statistics) {
  n <- nrow(data_mat)
  infit <- outfit <- lz <- rep(NA_real_, n)

  if (any(c("infit", "outfit") %in% statistics)) {
    resid <- .conditional_residuals(data_mat, thr_list)
    for (p in seq_len(n)) {
      r <- resid[[p]]
      if (is.null(r)) {
        next
      }
      outfit[p] <- mean(r$z^2)
      infit[p] <- sum(r$w * r$z^2) / sum(r$w)
    }
  }
  if ("lz" %in% statistics) {
    for (p in seq_len(n)) {
      lz[p] <- .person_lz(data_mat[p, ], thr_list, theta[p])
    }
  }
  list(infit = infit, outfit = outfit, lz = lz)
}

#' Standardized log-likelihood (lz) for one response pattern at theta
#' @keywords internal
#' @noRd
.person_lz <- function(resp, thr_list, theta) {
  if (is.na(theta) || !is.finite(theta)) {
    return(NA_real_)
  }
  ok <- !is.na(resp)
  if (!any(ok)) {
    return(NA_real_)
  }
  resp <- resp[ok]
  thr_list <- thr_list[ok]

  l <- 0
  El <- 0
  Vl <- 0
  for (i in seq_along(thr_list)) {
    p <- .pcm_cat_probs(theta, thr_list[[i]])
    lp <- log(p)
    l <- l + lp[resp[i] + 1L]
    m1 <- sum(p * lp)
    El <- El + m1
    Vl <- Vl + sum(p * lp^2) - m1^2
  }
  if (Vl <= 0) {
    return(NA_real_)
  }
  (l - El) / sqrt(Vl)
}

#' Wilson-Hilferty (ZSTD) transform of an MSQ value (non-inferential)
#' @keywords internal
#' @noRd
.msq_to_zstd <- function(msq, df) {
  q <- pmax(df, 1L)
  (msq^(1 / 3) - 1) * (3 / sqrt(2 / q)) + (sqrt(2 / q) / 3)
}

# ---------------------------------------------------------------------
# Internal: resampling
# ---------------------------------------------------------------------

#' @keywords internal
#' @noRd
.person_fit_resample <- function(
  data_mat,
  thr_list,
  theta,
  statistics,
  obs,
  iterations,
  theta_method = "WLE",
  parallel,
  n_cores,
  msq_tail = "two.sided"
) {
  n <- nrow(data_mat)
  want_msq <- any(c("infit", "outfit") %in% statistics)
  want_lz <- "lz" %in% statistics

  # .cond_moments() depends only on the answered-item set (not the person or
  # the observed score), so compute it once per distinct answered-set and reuse
  # it -- on complete data a single build instead of one per respondent. It is
  # deterministic, so caching does not touch the resampling RNG stream.
  ans_sets <- lapply(seq_len(n), function(p) which(!is.na(data_mat[p, ])))
  ans_key <- vapply(
    ans_sets,
    function(a) paste0(a, collapse = ","),
    character(1L)
  )
  mom_cache <- NULL
  if (want_msq) {
    mom_cache <- list()
    for (k in unique(ans_key)) {
      a <- ans_sets[[match(k, ans_key)]]
      if (length(a) > 0L) mom_cache[[k]] <- .cond_moments(thr_list[a])
    }
  }

  # Each statistic uses its matching null: the conditional MSQ statistics are
  # referenced against patterns sampled conditional on the total score (no
  # person estimate, consistent with the statistic); lz, which is defined at
  # the estimated location, is referenced against patterns simulated there and
  # scored with a location *re-estimated* from each simulated pattern (see the
  # lz block below).
  one_person <- function(p) {
    ans <- ans_sets[[p]]
    res <- c(infit = NA_real_, outfit = NA_real_, lz = NA_real_)
    if (length(ans) == 0L) {
      return(res)
    }
    thr_sub <- thr_list[ans]
    r_obs <- sum(data_mat[p, ans])

    # MSQ: conditional-on-total-score resampling (moments cached per answered-set)
    if (want_msq) {
      mom <- mom_cache[[ans_key[p]]]
      if (r_obs > 0L && r_obs < mom$max_score) {
        sims <- .sim_conditional(thr_sub, r_obs, iterations)
        if (!is.null(sims)) {
          stat <- t(apply(sims, 1L, function(xs) {
            rr <- sum(xs)
            if (rr == 0L || rr == mom$max_score) {
              return(c(NA, NA))
            }
            e <- mom$E[rr + 1L, ]
            v <- mom$V[rr + 1L, ]
            z <- (xs - e) / sqrt(v)
            c(mean(z^2), sum(v * z^2) / sum(v)) # outfit, infit
          }))
          if ("outfit" %in% statistics) {
            res["outfit"] <- .mc_pvalue(stat[, 1L], obs$outfit[p], msq_tail)
          }
          if ("infit" %in% statistics) {
            res["infit"] <- .mc_pvalue(stat[, 2L], obs$infit[p], msq_tail)
          }
        }
      }
    }

    # lz: simulate at the estimated location, then *re-estimate* the location
    # from each simulated pattern and score lz there (lower tail = misfit).
    # Re-estimation is essential: the observed lz carries the ability-estimation
    # effect (its variance is < 1; Snijders 2001), so the null must reproduce it
    # or the test is conservative (Sinharay 2016). For a Rasch/PCM model the WLE
    # / EAP location is a function of the total score, so this re-estimation is a
    # cheap score-based lookup (cached inside .estimate_thetas()), and it is the
    # resampling analogue of the analytic lz* standardization.
    if (want_lz && is.finite(theta[p])) {
      sims <- .sim_theta(thr_sub, theta[p], iterations)
      if (!is.null(sims)) {
        full <- matrix(NA_real_, iterations, ncol(data_mat))
        full[, ans] <- sims
        th_sim <- .estimate_thetas(
          full,
          thr_list,
          method = theta_method,
          theta_range = c(-10, 10)
        )$theta
        lz_sim <- vapply(
          seq_len(iterations),
          function(b) {
            .person_lz(full[b, ], thr_list, th_sim[b])
          },
          numeric(1L)
        )
        res["lz"] <- .mc_pvalue(lz_sim, obs$lz[p], tail = "lower")
      }
    }
    res
  }

  rows <- .maybe_parallel_lapply(seq_len(n), one_person, parallel, n_cores)
  m <- do.call(rbind, rows)
  list(infit = m[, "infit"], outfit = m[, "outfit"], lz = m[, "lz"])
}

#' Empirical p-value for an MSQ-type statistic against its simulated null
#'
#' @param sim Numeric vector of simulated null values.
#' @param obs Observed value.
#' @param tail `"two.sided"` (deviation in either direction), `"upper"`
#'   (underfit only -- the validity-relevant direction), or `"lower"` (e.g.
#'   the lz statistic, where small values indicate misfit).
#' @keywords internal
#' @noRd
.mc_pvalue <- function(sim, obs, tail = "two.sided") {
  sim <- sim[!is.na(sim)]
  B <- length(sim)
  if (B == 0L || is.na(obs)) {
    return(NA_real_)
  }
  # (1 + count) / (B + 1) Monte-Carlo p-value, consistent with
  # .bootstrap_pvalues() in utils-multiplicity.R: never exactly 0, and the
  # denominator counts only the valid (non-NA) simulations.
  if (tail == "upper") {
    return((1 + sum(sim >= obs)) / (B + 1))
  }
  if (tail == "lower") {
    return((1 + sum(sim <= obs)) / (B + 1))
  }
  p_upper <- (1 + sum(sim >= obs)) / (B + 1)
  p_lower <- (1 + sum(sim <= obs)) / (B + 1)
  min(1, 2 * min(p_upper, p_lower))
}

#' Simulate response patterns at a fixed theta (per-person MC scheme)
#' @keywords internal
#' @noRd
.sim_theta <- function(thr_sub, theta, B) {
  if (is.na(theta) || !is.finite(theta)) {
    return(NULL)
  }
  k <- length(thr_sub)
  out <- matrix(0L, B, k)
  for (i in seq_len(k)) {
    p <- .pcm_cat_probs(theta, thr_sub[[i]])
    out[, i] <- sample.int(length(p), B, replace = TRUE, prob = p) - 1L
  }
  out
}

#' Simulate response patterns conditional on a fixed total score
#'
#' Sequential exact sampling via suffix elementary symmetric functions:
#' draws each item in turn from its conditional distribution given the
#' remaining target score and the remaining items.
#' @keywords internal
#' @noRd
.sim_conditional <- function(thr_sub, total, B) {
  k <- length(thr_sub)
  steps <- vapply(thr_sub, length, integer(1))
  if (total <= 0L || total >= sum(steps)) {
    return(NULL)
  }

  # per-item category weights w_{i,c} = exp(-eta_{i,c}), c = 0..m_i
  wlist <- lapply(thr_sub, function(thr) exp(-c(0, cumsum(thr))))
  # suffix ESF: gamma of items i..k, for i = 1..k+1 (k+1 = empty -> gamma_0 = 1)
  suffix_g <- vector("list", k + 1L)
  suffix_g[[k + 1L]] <- 1 # empty product
  for (i in k:1) {
    suffix_g[[i]] <- .esf_convolve(suffix_g[[i + 1L]], wlist[[i]])
  }

  out <- matrix(0L, B, k)
  for (b in seq_len(B)) {
    s <- total
    for (i in seq_len(k)) {
      w_i <- wlist[[i]]
      g_rest <- suffix_g[[i + 1L]] # gamma of items i+1..k
      cats <- 0:steps[i]
      # P(x_i = c) propto w_{i,c} * gamma_{s-c}(rest)
      probs <- vapply(
        cats,
        function(c) {
          idx <- s - c
          if (idx < 0L || idx >= length(g_rest)) {
            return(0)
          }
          w_i[c + 1L] * g_rest[idx + 1L]
        },
        numeric(1)
      )
      if (sum(probs) <= 0) {
        out[b, i] <- 0L
        next
      }
      xc <- sample.int(length(probs), 1L, prob = probs) - 1L
      out[b, i] <- xc
      s <- s - xc
    }
  }
  out
}

#' Convolve an ESF vector with one item's category weights
#'
#' Given gamma (ESF of a set of items, indexed by score 0..S) and an
#' item's weights w (score 0..m), returns the ESF of the combined set.
#' @keywords internal
#' @noRd
.esf_convolve <- function(gamma, w) {
  S <- length(gamma) - 1L
  m <- length(w) - 1L
  out <- numeric(S + m + 1L)
  for (s in 0:S) {
    gs <- gamma[s + 1L]
    if (gs == 0) {
      next
    }
    for (c in 0:m) {
      out[s + c + 1L] <- out[s + c + 1L] + gs * w[c + 1L]
    }
  }
  out
}

#' Optionally parallel lapply via mirai
#' @keywords internal
#' @noRd
.maybe_parallel_lapply <- function(x, fun, parallel, n_cores) {
  if (parallel && requireNamespace("mirai", quietly = TRUE)) {
    workers <- if (is.null(n_cores)) min(2L, length(x)) else n_cores
    mirai::daemons(workers)
    on.exit(mirai::daemons(0L), add = TRUE)
    res <- mirai::mirai_map(x, fun)[]
    return(res)
  }
  lapply(x, fun)
}

# ---------------------------------------------------------------------
# Internal: output formatting
# ---------------------------------------------------------------------

#' @keywords internal
#' @noRd
.person_fit_output <- function(
  out,
  theta,
  extreme,
  statistics,
  output,
  flag_alpha,
  iterations,
  flag = "both",
  n_total = nrow(out),
  has_na = FALSE
) {
  # Assessed = respondents that received a fit statistic (extreme scorers get
  # NA). n_total is the raw supplied row count (before all-NA rows were
  # dropped), so the caption anchors to the full sample. Each parenthetical
  # qualifier appears only when it applies (missingness / extreme scorers).
  n_assessed <- sum(!extreme)
  pf_quals <- c(
    if (has_na) "incomplete responses retained",
    if (any(extreme)) "extreme scores excluded"
  )
  n_clause <- paste0(
    " ",
    .n_caption(n_assessed, n_total, pf_quals, noun = "respondents assessed"),
    "."
  )
  if (output == "dataframe") {
    # Unrounded: rounding is a display concern (the kable path rounds a
    # display copy below).
    return(out)
  }

  if (output %in% c("ggplot", "list")) {
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
      stop(
        "Package 'ggplot2' is required for output = \"", output, "\". ",
        "Install it with: install.packages(\"ggplot2\")",
        call. = FALSE
      )
    }
    # One plot per requested statistic, returned as a named list.
    stat_map <- list(
      infit = list(
        col = "infit_msq",
        pcol = "p_infit",
        label = "Infit MSQ",
        ref = 1,
        split = TRUE
      ),
      outfit = list(
        col = "outfit_msq",
        pcol = "p_outfit",
        label = "Outfit MSQ",
        ref = 1,
        split = TRUE
      ),
      lz = list(col = "lz", pcol = "p_lz", label = "lz", ref = 0, split = FALSE)
    )
    plots <- lapply(statistics, function(s) {
      m <- stat_map[[s]]
      .person_fit_plot(
        out,
        theta,
        m$col,
        m$pcol,
        m$label,
        m$ref,
        m$split,
        flag_alpha,
        iterations,
        flag,
        n_total = n_total,
        qualifiers = pf_quals
      )
    })
    names(plots) <- statistics
    if (output == "list") {
      # One computation, both views: the per-person results and the maps.
      return(list(fit = out, plots = plots))
    }
    return(plots)
  }

  display <- out
  num <- vapply(display, is.numeric, logical(1))
  display[num] <- lapply(display[num], function(x) round(x, 3))
  caption <- paste0(
    "Person fit (conditional MSQ + lz). ",
    if (iterations > 0L) {
      msq_desc <- if (flag == "underfit") {
        "one-sided (underfit) MSQ p-values"
      } else {
        "two-sided MSQ p-values"
      }
      paste0(
        msq_desc,
        " from ",
        iterations,
        " Monte-Carlo replications (MSQ conditional on the total score, ",
        "lz simulated at the estimated location); ",
        "flagged at alpha = ",
        flag_alpha,
        "."
      )
    } else {
      "No resampling (iterations = 0)."
    },
    n_clause
  )
  knitr::kable(display, format = "pipe", caption = caption, row.names = FALSE)
}

#' Build one person-fit map (statistic vs person location)
#'
#' @param out The assembled person-fit data.frame.
#' @param theta Person locations (x-axis).
#' @param ycol,pcol Column names for the statistic and its marginal p-value.
#' @param ylab Axis label for the statistic.
#' @param ref Horizontal reference line (expected value), or `NA` for none.
#' @param split Logical. If `TRUE` (the MSQ statistics), flagged respondents
#'   are split into underfit (above `ref`) and overfit (below `ref`) for both
#'   the point colour and the caption. `FALSE` for one-sided statistics (lz).
#' @param flag_alpha Flagging threshold on the marginal p-value.
#' @param iterations Number of resampling iterations (0 = no p-values).
#' @param flag `"both"` (two-sided MSQ test) or `"underfit"` (one-sided);
#'   controls the caption wording for the MSQ maps.
#' @return A `ggplot`. Caption reports the assessed sample size and the
#'   proportion flagged by this statistic.
#' @keywords internal
#' @noRd
.person_fit_plot <- function(
  out,
  theta,
  ycol,
  pcol,
  ylab,
  ref,
  split,
  flag_alpha,
  iterations,
  flag = "both",
  n_total = NULL,
  qualifiers = character()
) {
  has_p <- iterations > 0L && pcol %in% names(out)
  df <- data.frame(
    theta = theta,
    y = out[[ycol]],
    flagged = if (has_p) out[[pcol]] < flag_alpha else FALSE
  )
  df <- df[is.finite(df$theta) & is.finite(df$y), , drop = FALSE]
  n_assessed <- nrow(df)
  n_lead <- .n_caption(
    n_assessed,
    if (is.null(n_total)) n_assessed else n_total,
    qualifiers,
    noun = "respondents assessed"
  )

  # Classify each respondent for colouring.
  df$status <- "Not flagged"
  if (split && is.finite(ref)) {
    df$status[df$flagged & df$y > ref] <- "Underfit"
    df$status[df$flagged & df$y < ref] <- "Overfit"
  } else {
    df$status[df$flagged] <- "Flagged"
  }
  pal <- c(
    "Not flagged" = "grey55",
    "Underfit" = "#e41a1c",
    "Overfit" = "#377eb8",
    "Flagged" = "#e41a1c"
  )
  present <- intersect(names(pal), unique(df$status))
  df$status <- factor(df$status, levels = present)

  if (has_p) {
    n_flag <- sum(df$flagged, na.rm = TRUE)
    pct <- if (n_assessed > 0L) {
      round(100 * n_flag / n_assessed, 1)
    } else {
      NA_real_
    }
    if (split && is.finite(ref)) {
      n_under <- sum(df$flagged & df$y > ref, na.rm = TRUE) # MSQ > 1: noisy
      n_over <- sum(df$flagged & df$y < ref, na.rm = TRUE) # MSQ < 1: muted
      caption <- if (flag == "underfit") {
        paste0(
          n_lead,
          "; ",
          n_flag,
          " (",
          pct,
          "%) flagged as underfit (MSQ > ",
          ref,
          "; one-sided p < ",
          flag_alpha,
          ")."
        )
      } else {
        paste0(
          n_lead,
          "; ",
          n_flag,
          " (",
          pct,
          "%) flagged: ",
          n_under,
          " underfit (MSQ > ",
          ref,
          "), ",
          n_over,
          " overfit (MSQ < ",
          ref,
          ") (p < ",
          flag_alpha,
          ")."
        )
      }
    } else {
      caption <- paste0(
        n_lead,
        "; ",
        n_flag,
        " (",
        pct,
        "%) flagged as misfitting (p < ",
        flag_alpha,
        ")."
      )
    }
  } else {
    caption <- paste0(
      n_lead,
      " (no resampling; iterations = 0)."
    )
  }

  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data$theta, y = .data$y))
  if (is.finite(ref)) {
    p <- p +
      ggplot2::geom_hline(
        yintercept = ref,
        linetype = "dashed",
        colour = "grey60"
      )
  }
  if (has_p && length(present) > 1L) {
    p <- p +
      ggplot2::geom_point(ggplot2::aes(colour = .data$status), alpha = 0.7) +
      ggplot2::scale_colour_manual(
        values = pal,
        name = "Person fit",
        breaks = present
      )
  } else {
    p <- p + ggplot2::geom_point(colour = "grey45", alpha = 0.7)
  }
  p +
    ggplot2::labs(
      x = "Person location (logits)",
      y = ylab,
      caption = er2_caption(caption)
    ) +
    ggplot2::theme_bw() +
    er2_axis_margins() +
    er2_plot_caption()
}

# Tests for RMpersonFit()

# Genuine Rasch-null PCM data (so fit statistics behave under H0).
sim_pcm_null <- function(n = 300, J = 8, person_sd = 1.3, seed = 11L) {
  set.seed(seed)
  theta <- rnorm(n, 0, person_sd)
  thr <- lapply(seq_len(J), function(j) sort(rnorm(2, 0, 0.9)))
  pcm_p <- function(th, t) {
    ce <- c(0, cumsum(th - t))
    exp(ce - max(ce)) / sum(exp(ce - max(ce)))
  }
  m <- sapply(seq_len(J), function(j) {
    sapply(theta, function(th) sample(0:2, 1, prob = pcm_p(th, thr[[j]])))
  })
  df <- as.data.frame(m)
  colnames(df) <- paste0("I", seq_len(J))
  df
}

sim_rasch_null <- function(n = 300, J = 10, seed = 3L) {
  set.seed(seed)
  theta <- rnorm(n, 0, 1.3)
  beta <- seq(-2, 2, length.out = J)
  m <- sapply(seq_len(J), function(j) rbinom(n, 1, plogis(theta - beta[j])))
  df <- as.data.frame(m)
  colnames(df) <- paste0("I", seq_len(J))
  df
}

# ---------------------------------------------------------------------
# Input validation & structure
# ---------------------------------------------------------------------

test_that("RMpersonFit errors on non-zero minimum", {
  expect_error(
    RMpersonFit(sim_rasch_null() + 1L, iterations = 0),
    regexp = "scored starting at 0"
  )
})

test_that("dataframe has expected columns with all statistics", {
  skip_on_cran()
  res <- RMpersonFit(
    sim_pcm_null(n = 120),
    iterations = 100,
    seed = 1,
    output = "dataframe"
  )
  expect_s3_class(res, "data.frame")
  expect_true(all(
    c(
      "id",
      "n_answered",
      "sum_score",
      "infit_msq",
      "outfit_msq",
      "lz",
      "p_infit",
      "p_outfit",
      "p_lz",
      "flagged"
    ) %in%
      names(res)
  ))
  expect_equal(nrow(res), 120L)
})

test_that("iterations = 0 returns statistics without p-values", {
  res <- RMpersonFit(
    sim_pcm_null(n = 100),
    iterations = 0,
    output = "dataframe"
  )
  expect_false(any(grepl("^p_", names(res))))
  expect_false("flagged" %in% names(res))
  expect_true(all(c("infit_msq", "outfit_msq", "lz") %in% names(res)))
})

test_that("statistics argument subsets the output", {
  skip_on_cran()
  res <- RMpersonFit(
    sim_pcm_null(n = 100),
    statistics = "outfit",
    iterations = 50,
    seed = 1,
    output = "dataframe"
  )
  expect_true("outfit_msq" %in% names(res))
  expect_false(any(c("infit_msq", "lz") %in% names(res)))
})

test_that("p-values lie in [0, 1]", {
  skip_on_cran()
  res <- RMpersonFit(
    sim_pcm_null(n = 120),
    iterations = 100,
    seed = 2,
    output = "dataframe"
  )
  for (col in c("p_infit", "p_outfit", "p_lz")) {
    p <- res[[col]][!is.na(res[[col]])]
    expect_true(all(p >= 0 & p <= 1))
  }
})

# ---------------------------------------------------------------------
# Statistical behaviour
# ---------------------------------------------------------------------

test_that("conditional MSQ means are near 1 under the null", {
  skip_on_cran()
  res <- RMpersonFit(
    sim_pcm_null(n = 400),
    iterations = 0,
    output = "dataframe"
  )
  expect_equal(mean(res$infit_msq, na.rm = TRUE), 1, tolerance = 0.05)
  expect_equal(mean(res$outfit_msq, na.rm = TRUE), 1, tolerance = 0.05)
})

test_that("Type I error is near nominal under the null", {
  skip_on_cran()
  res <- RMpersonFit(
    sim_pcm_null(n = 400),
    iterations = 400,
    seed = 4,
    output = "dataframe"
  )
  expect_lt(mean(res$flagged, na.rm = TRUE), 0.12)
})

test_that("lz p-values are calibrated (not conservative) under the null", {
  skip_on_cran()
  # The lz resampling re-estimates the person location for each simulated
  # pattern, reproducing the ability-estimation effect (Snijders 2001;
  # Sinharay 2016) that otherwise makes the test conservative. With a fixed
  # location the rejection rate here would be roughly half nominal.
  res <- RMpersonFit(
    sim_rasch_null(n = 400, J = 10, seed = 3),
    statistics = "lz",
    iterations = 300,
    seed = 7,
    output = "dataframe"
  )
  p <- res$p_lz[!is.na(res$p_lz)]
  expect_gt(mean(p), 0.45)
  expect_lt(mean(p), 0.55)
  expect_gt(mean(p < 0.10), 0.06) # near nominal 0.10, not the conservative ~0.04
})

test_that("all-NA respondents are dropped with a message; kable caption reports n", {
  skip_if_not_installed("knitr")
  dat <- sim_pcm_null(n = 100)
  dat[100, ] <- NA
  expect_message(
    kbl <- RMpersonFit(dat, iterations = 0),
    "1 respondent\\(s\\) with no responses dropped"
  )
  out <- paste(capture.output(print(kbl)), collapse = " ")
  expect_match(out, "of 100 respondents assessed")
})

test_that("aberrant responders are detected (power)", {
  skip_on_cran()
  dat <- sim_rasch_null(n = 300, J = 10, seed = 3)
  ab <- 1:15
  dat[ab, ] <- dat[ab, rev(seq_len(ncol(dat)))] # reverse -> aberrant
  res <- RMpersonFit(dat, iterations = 300, seed = 2, output = "dataframe")
  expect_gt(mean(res$flagged[ab], na.rm = TRUE), 0.6)
  expect_lt(mean(res$flagged[-ab], na.rm = TRUE), 0.12)
})

# ---------------------------------------------------------------------
# Missingness, extremes, schemes, outputs
# ---------------------------------------------------------------------

test_that("partial missingness is handled (rows kept, estimates finite)", {
  dat <- sim_pcm_null(n = 200)
  dat[cbind(sample(200, 40), sample(8, 40, replace = TRUE))] <- NA
  res <- RMpersonFit(dat, iterations = 0, output = "dataframe")
  expect_equal(nrow(res), 200L)
  expect_true(any(res$n_answered < 8L))
  finite_non_extreme <- res$outfit_msq[
    res$n_answered > 1L &
      res$sum_score > 0 &
      !is.na(res$outfit_msq)
  ]
  expect_true(all(is.finite(finite_non_extreme)))
})

test_that("extreme scorers receive NA statistics", {
  skip_on_cran()
  dat <- sim_rasch_null(n = 200, J = 8, seed = 9)
  dat[1, ] <- 0L
  dat[2, ] <- 1L
  res <- RMpersonFit(dat, iterations = 50, seed = 1, output = "dataframe")
  expect_true(is.na(res$outfit_msq[1]) && is.na(res$outfit_msq[2]))
})

test_that("all three statistics get valid p-values (per-statistic schemes)", {
  skip_on_cran()
  res <- RMpersonFit(
    sim_pcm_null(n = 150),
    iterations = 200,
    seed = 7,
    output = "dataframe"
  )
  for (col in c("p_infit", "p_outfit", "p_lz")) {
    p <- res[[col]][!is.na(res[[col]])]
    expect_true(length(p) > 0L && all(p >= 0 & p <= 1))
  }
})

test_that("MSQ-only run needs no person estimate but still yields p-values", {
  skip_on_cran()
  dat <- sim_pcm_null(n = 150)
  dat[cbind(sample(150, 25), sample(8, 25, replace = TRUE))] <- NA
  res <- RMpersonFit(
    dat,
    statistics = c("infit", "outfit"),
    iterations = 200,
    seed = 1,
    output = "dataframe"
  )
  ok <- res$n_answered > 1L & res$sum_score > 0 & !is.na(res$infit_msq)
  expect_true(all(is.finite(res$p_infit[ok]))) # conditional scheme, missingness handled
})

test_that("conditional sampler produces patterns with the fixed total", {
  thr <- list(0.2, -0.3, 0.1, 0.0, 0.4)
  sims <- easyRasch2:::.sim_conditional(thr, total = 2L, B = 300)
  expect_true(all(rowSums(sims) == 2L))
})

test_that("zstd adds non-inferential ZSTD columns", {
  skip_on_cran()
  res <- RMpersonFit(
    sim_pcm_null(n = 100),
    iterations = 50,
    zstd = TRUE,
    seed = 1,
    output = "dataframe"
  )
  expect_true(all(c("infit_zstd", "outfit_zstd") %in% names(res)))
})

test_that("kable output returns knitr_kable", {
  skip_on_cran()
  skip_if_not_installed("knitr")
  dat <- sim_pcm_null(n = 100)
  expect_s3_class(RMpersonFit(dat, iterations = 50), "knitr_kable")
})

test_that("ggplot output returns a named list of plots, one per statistic", {
  skip_on_cran()
  skip_if_not_installed("ggplot2")
  dat <- sim_pcm_null(n = 120)
  g <- RMpersonFit(dat, iterations = 100, seed = 1, output = "ggplot")
  expect_type(g, "list")
  expect_named(g, c("infit", "outfit", "lz"))
  expect_true(all(vapply(g, function(p) inherits(p, "ggplot"), logical(1))))
  # each plot is non-blank (has plotted points) -- the infit-blank regression
  expect_true(all(vapply(g, function(p) nrow(p$data) > 0L, logical(1))))
})

test_that("ggplot with a single statistic is a one-element list and not blank", {
  skip_on_cran()
  skip_if_not_installed("ggplot2")
  dat <- sim_pcm_null(n = 120)
  gi <- RMpersonFit(
    dat,
    statistics = "infit",
    iterations = 100,
    seed = 1,
    output = "ggplot"
  )
  expect_named(gi, "infit")
  expect_s3_class(gi$infit, "ggplot")
  expect_gt(nrow(gi$infit$data), 0L) # not blank
  expect_match(gi$infit$labels$caption, "flagged")
})

test_that("MSQ plot captions split flagged into underfit/overfit; lz does not", {
  skip_on_cran()
  skip_if_not_installed("ggplot2")
  set.seed(2)
  n <- 400
  J <- 8
  theta <- rnorm(n, 0, 1.3)
  beta <- seq(-1.5, 1.5, length.out = J)
  m <- sapply(seq_len(J), function(j) rbinom(n, 1, plogis(theta - beta[j])))
  m[1:8, ] <- m[1:8, J:1] # reversed -> underfit
  dat <- as.data.frame(m)
  colnames(dat) <- paste0("I", seq_len(J))
  g <- RMpersonFit(dat, iterations = 400, seed = 1, output = "ggplot")
  # (caption may wrap across lines, so check the words separately)
  expect_match(g$infit$labels$caption, "underfit")
  expect_match(g$infit$labels$caption, "overfit")
  expect_match(g$outfit$labels$caption, "underfit")
  expect_no_match(g$lz$labels$caption, "underfit") # one-sided: no split
})

test_that("flag = 'underfit' flags only the upper (MSQ > 1) direction", {
  skip_on_cran()
  set.seed(2)
  n <- 400
  J <- 8
  theta <- rnorm(n, 0, 1.3)
  beta <- seq(-1.5, 1.5, length.out = J)
  m <- sapply(seq_len(J), function(j) rbinom(n, 1, plogis(theta - beta[j])))
  m[1:10, ] <- m[1:10, J:1]
  dat <- as.data.frame(m)
  colnames(dat) <- paste0("I", seq_len(J))
  res <- RMpersonFit(
    dat,
    statistics = "infit",
    iterations = 400,
    seed = 1,
    flag = "underfit",
    output = "dataframe"
  )
  flagged <- res[!is.na(res$flagged) & res$flagged, ]
  expect_true(all(flagged$infit_msq > 1)) # never flags overfit
})

test_that("flag = 'underfit' uses a one-sided MSQ p-value (differs from both)", {
  skip_on_cran()
  dat <- sim_pcm_null(n = 200)
  a <- RMpersonFit(
    dat,
    statistics = "infit",
    iterations = 400,
    seed = 1,
    flag = "both",
    output = "dataframe"
  )
  b <- RMpersonFit(
    dat,
    statistics = "infit",
    iterations = 400,
    seed = 1,
    flag = "underfit",
    output = "dataframe"
  )
  expect_false(isTRUE(all.equal(a$p_infit, b$p_infit)))
})

test_that("plot status colours respondents by direction", {
  skip_on_cran()
  skip_if_not_installed("ggplot2")
  dat <- sim_pcm_null(n = 150)
  g <- RMpersonFit(dat, iterations = 300, seed = 1, output = "ggplot")
  expect_true("status" %in% names(g$infit$data))
  expect_true(all(
    levels(g$infit$data$status) %in%
      c("Not flagged", "Underfit", "Overfit")
  ))
  expect_true(all(levels(g$lz$data$status) %in% c("Not flagged", "Flagged")))
})

test_that("RMpersonFit Monte-Carlo p-values are never exactly 0", {
  skip_on_cran()
  dat <- sim_pcm_null()
  res <- RMpersonFit(dat, iterations = 50, seed = 1, output = "dataframe")
  p_cols <- grep("^p_", names(res), value = TRUE)
  expect_true(length(p_cols) > 0L)
  for (pc in p_cols) {
    p <- res[[pc]][!is.na(res[[pc]])]
    # (1 + count) / (B + 1) convention: strictly positive, never 0.
    expect_true(all(p > 0 & p <= 1))
  }
})

test_that("output = 'list' returns fit + plots from one computation, matching the other outputs", {
  set.seed(7)
  dat <- as.data.frame(replicate(5, sample(0:2, 100, replace = TRUE)))
  names(dat) <- paste0("q", 1:5)

  res <- RMpersonFit(dat, iterations = 50, seed = 42, output = "list")
  expect_named(res, c("fit", "plots"))
  expect_s3_class(res$fit, "data.frame")
  expect_named(res$plots, c("infit", "outfit", "lz"))
  for (p in res$plots) expect_s3_class(p, "ggplot")

  df <- RMpersonFit(dat, iterations = 50, seed = 42, output = "dataframe")
  expect_identical(res$fit, df)

  # subset of statistics propagates to both views
  res2 <- RMpersonFit(dat, statistics = "infit", iterations = 0,
                      output = "list")
  expect_named(res2$plots, "infit")
  expect_false("lz" %in% names(res2$fit))
})

#' Item Restscore Analysis
#'
#' Computes observed and model-expected item-restscore correlations using
#' `iarm::item_restscore()`, and enriches the output with the absolute
#' difference between observed and expected values, item average locations, and
#' item locations relative to the sample mean person location.
#'
#' @param data A data.frame or matrix of item responses. Items must be scored
#'   starting at 0 (non-negative integers). Missing values (`NA`) are allowed,
#'   but at least one complete case (row with no `NA`) must be present.
#' @param output Character string controlling the return value. Either
#'   `"kable"` (default) for a formatted `knitr::kable()` table, or
#'   `"dataframe"` for the underlying data.frame.
#' @param sort Optional character string. When `sort = "diff"`, rows are sorted
#'   by the absolute magnitude of `Difference` in descending order, so that
#'   both over- and underfitting items appear near the top.
#' @param p_adj Character string specifying the p-value adjustment method
#'   passed to `iarm::item_restscore()`. Default `"BH"` (Benjamini-Hochberg);
#'   use `"none"` for unadjusted p-values. Run `?stats::p.adjust` for the list
#'   of available methods.
#'
#' @return
#' * If `output = "kable"`: a `knitr_kable` object (plain text table via
#'   `format = "pipe"`) with columns for item name, observed and expected
#'   restscore correlations, the signed difference (observed minus
#'   expected), adjusted p-value, the `Flagged` misfit label, and item
#'   location relative to the sample mean person location.
#' * If `output = "dataframe"`: a data.frame with columns `Item`, `Observed`,
#'   `Expected`, `Difference`, `p_adjusted`, `Flagged`, and
#'   `Relative_location`. `Flagged` is `"overfit"` (observed above expected,
#'   adj. p < .05), `"underfit"` (below, adj. p < .05), or `""` (not flagged).
#'
#' The `Difference` column is signed (observed minus expected):
#' *positive* values indicate that the item correlates more strongly with
#' the rest-score than the Rasch model predicts (over-discrimination /
#' *overfit*, often associated with local dependence), and *negative*
#' values indicate weaker-than-expected association (under-discrimination
#' / *underfit*, often associated with multidimensionality or noise).
#'
#' @details
#' Item-restscore correlations using Goodman-Kruskal's gamma (Kreiner, 2011) measure
#' the association between a person's score on a single item and their total
#' score on the remaining items (the "restscore"). Under a correctly fitting
#' Rasch model, observed and model-expected correlations should agree closely.
#'
#' Item parameters are estimated by conditional maximum likelihood via
#' `psychotools::pcmodel()` (a dichotomous item is a 2-category PCM); the
#' item-restscore statistic itself comes from `iarm::item_restscore()` and is
#' conditional on the total score, so it is invariant to the estimation engine.
#' Per-item average locations are the means of the CML thresholds, and the
#' person-location reference is the mean of the Warm WLE estimates.
#'
#' Relative item location is defined as the item's average location minus the
#' sample mean person location, providing a measure of item targeting.
#'
#' The `iarm` package must be installed (it is in Suggests, not Imports).
#'
#' @references
#' Kreiner, S. (2011). A Note on Item–Restscore Association in Rasch Models.
#' *Applied Psychological Measurement, 35*(7), 557–561.
#' \doi{10.1177/0146621611410227}
#'
#' @export
#'
#' @examples
#' \donttest{
#' if (requireNamespace("iarm", quietly = TRUE)) {
#'   # Simulate binary item response data (8 items, 200 persons)
#'   set.seed(42)
#'   sim_data <- as.data.frame(
#'     matrix(sample(0:1, 200 * 8, replace = TRUE), nrow = 200, ncol = 8)
#'   )
#'   colnames(sim_data) <- paste0("Item", 1:8)
#'
#'   # Default kable output
#'   RMitemRestscore(sim_data)
#'
#'   # Sorted by absolute difference
#'   RMitemRestscore(sim_data, sort = "diff")
#'
#'   # Return as data.frame for further processing
#'   df <- RMitemRestscore(sim_data, output = "dataframe")
#' }
#' }
RMitemRestscore <- function(data, output = "kable", sort, p_adj = "BH") {
  if (!requireNamespace("iarm", quietly = TRUE)) {
    stop(
      "Package 'iarm' is required for RMitemRestscore() but is not installed.\n",
      "Install it with: install.packages(\"iarm\")",
      call. = FALSE
    )
  }

  output <- match.arg(output, c("kable", "dataframe"))

  validate_response_data(data)

  if (nrow(stats::na.omit(data)) == 0L) {
    stop(
      "No complete cases in data. All rows contain at least one NA.",
      call. = FALSE
    )
  }

  # Respondents with no responses at all contribute nothing and break the CML
  # fit (psychotools errors on all-NA rows); drop them, keeping the raw totals
  # for the caption.
  n_total <- nrow(as.data.frame(data))
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)

  data_mat <- as.matrix(data)
  n_items <- ncol(data)
  n_complete <- nrow(stats::na.omit(data))

  # --- Fit Rasch model and compute item/person locations ----------------------
  # CML item parameters (psychotools; a dichotomous item is a 2-category PCM)
  # and WLE person locations, consistent with the rest of the package. The
  # item-restscore statistic from iarm is conditional and engine-invariant; the
  # relative-location reference shifts only slightly (WLE vs eRm MLE person
  # mean), since item and person locations move together with the scale.
  fit <- psychotools::pcmodel(data)
  thr_list <- .center_thresholds(lapply(
    psychotools::threshpar(fit),
    as.numeric
  ))
  item_avg_locations <- vapply(thr_list, mean, numeric(1L))
  names(item_avg_locations) <- names(data)
  person_avg_location <- mean(
    .estimate_thetas(data_mat, thr_list, method = "WLE")$theta,
    na.rm = TRUE
  )

  relative_item_avg_locations <- item_avg_locations - person_avg_location

  # --- Compute item-restscore statistics via iarm ----------------------------
  # Temporarily set rgl.useNULL to avoid rgl device issues during iarm fitting
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  i1 <- iarm::item_restscore(fit, p.adj = p_adj)
  i1 <- as.data.frame(i1)

  # i1[[1]] is the results matrix. iarm::item_restscore() appends an
  # adjusted-p column named "padj.<method>" (e.g. "padj.BH") only when
  # p.adj != "none"; with p.adj = "none" that column is absent and the
  # fixed position 5 is instead the significance-stars column ("sig"),
  # whose "***"/"." strings coerce to NA. Select the p-value column by
  # name: the adjusted column when present, otherwise the raw "pvalue".
  res_mat <- i1[[1]]
  cn <- colnames(res_mat)
  padj_idx <- grep("^padj", cn)
  p_col <- if (length(padj_idx) == 1L) padj_idx else match("pvalue", cn)
  observed <- as.numeric(res_mat[seq_len(n_items), 1L])
  expected <- as.numeric(res_mat[seq_len(n_items), 2L])
  p_adjusted <- as.numeric(res_mat[seq_len(n_items), p_col])

  # Flagged labels the misfit direction (only when adj. p < .05): observed
  # above expected = over-discrimination ("overfit", often local dependence);
  # below = under-discrimination ("underfit", often multidimensionality/noise);
  # "" otherwise. Note the value direction is opposite to infit (where a high
  # statistic is underfit).
  difference <- observed - expected
  flagged <- ifelse(
    !is.na(p_adjusted) & p_adjusted < 0.05 & difference > 0,
    "overfit",
    ifelse(
      !is.na(p_adjusted) & p_adjusted < 0.05 & difference < 0,
      "underfit",
      ""
    )
  )

  # --- Assemble result data.frame --------------------------------------------
  i2 <- data.frame(
    Item = names(data),
    Observed = observed,
    Expected = expected,
    Difference = difference,
    p_adjusted = p_adjusted,
    Flagged = flagged,
    Relative_location = as.numeric(relative_item_avg_locations),
    stringsAsFactors = FALSE,
    row.names = NULL
  )

  # --- Sort if requested -----------------------------------------------------
  if (!missing(sort) && identical(sort, "diff")) {
    # Sort by absolute magnitude so both over- and underfit items rise
    # to the top; the signed value is still what the user sees.
    i2 <- i2[order(abs(i2$Difference), decreasing = TRUE), ]
    rownames(i2) <- NULL
  }

  # --- Return ----------------------------------------------------------------
  if (output == "dataframe") {
    return(i2)
  }

  # Kable display rounding (the dataframe output above stays unrounded)
  i2 <- .round_display(i2, c(
    Observed = 2, Expected = 2, Difference = 3, p_adjusted = 3,
    Relative_location = 2
  ))

  p_header <- if (identical(p_adj, "none")) {
    "p-value"
  } else {
    paste0("Adj. p-value (", p_adj, ")")
  }

  knitr::kable(
    i2,
    format = "pipe",
    col.names = c(
      "Item",
      "Observed",
      "Expected",
      "Difference",
      p_header,
      "Flagged",
      "Rel. location"
    ),
    caption = paste0(
      "Item-restscore associations. ",
      .n_caption(
        n_complete,
        n_total,
        if (has_na) "complete cases" else character()
      ),
      ". Flagged (",
      if (identical(p_adj, "none")) "p" else "adj. p",
      " < .05): overfit = observed above expected ",
      "(over-discrimination, often local dependence); underfit = below ",
      "(under-discrimination, often multidimensionality/noise)."
    )
  )
}

test_that("RMitemRestscore errors when iarm is not installed", {
  # This test is only meaningful when iarm is NOT installed.
  skip_if(
    requireNamespace("iarm", quietly = TRUE),
    "iarm is installed; skipping missing-package error path test"
  )
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5))
  expect_error(RMitemRestscore(df), regexp = "iarm")
})

test_that("RMitemRestscore errors when data does not start at 0", {
  skip_if_not_installed("iarm")
  df <- as.data.frame(matrix(sample(1:3, 200, replace = TRUE), nrow = 40, ncol = 5))
  expect_error(RMitemRestscore(df), regexp = "scored starting at 0")
})

test_that("RMitemRestscore errors when data is all NA", {
  skip_if_not_installed("iarm")
  df <- as.data.frame(matrix(NA_integer_, nrow = 10, ncol = 4))
  expect_error(RMitemRestscore(df), regexp = "no non-missing|No complete|non-missing")
})

test_that("RMitemRestscore errors when no complete cases", {
  skip_if_not_installed("iarm")
  # Create data where every row has at least one NA
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5))
  # Make all rows have at least one NA by cycling through columns
  df[cbind(seq_len(nrow(df)), ((seq_len(nrow(df)) - 1L) %% ncol(df)) + 1L)] <- NA
  expect_error(RMitemRestscore(df), regexp = "No complete cases")
})

test_that("RMitemRestscore output = 'dataframe' returns correct structure (dichotomous)", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  result <- RMitemRestscore(df, output = "dataframe")

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 5L)
  expected_cols <- c(
    "Item", "Observed", "Expected", "Difference",
    "p_adjusted", "Flagged", "Relative_location"
  )
  expect_equal(names(result), expected_cols)
  expect_true(all(result$Flagged %in% c("overfit", "underfit", "")))
  # overfit <=> observed above expected, underfit <=> below (when flagged)
  flg <- result$Flagged != ""
  expect_true(all((result$Difference[flg] > 0) == (result$Flagged[flg] == "overfit")))
  expect_equal(result$Item, colnames(df))
  # Signed difference equals Observed - Expected (dataframe output is
  # unrounded; rounding happens only in the kable display)
  expect_equal(result$Difference, result$Observed - result$Expected)
})

test_that("RMitemRestscore p_adj = 'none' returns numeric raw p-values without a coercion warning", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  # Regression: with p.adj = "none" iarm drops the "padj.*" column, so the
  # p-value must be read from "pvalue" by name, not from fixed position 5
  # (which becomes the significance-stars column -> NA + coercion warning).
  expect_no_warning(
    result <- RMitemRestscore(df, output = "dataframe", p_adj = "none")
  )
  expect_type(result$p_adjusted, "double")
  expect_equal(sum(is.na(result$p_adjusted)), 0L)
})

test_that("RMitemRestscore output = 'kable' returns knitr_kable object", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  result <- RMitemRestscore(df, output = "kable")
  expect_s3_class(result, "knitr_kable")
})

test_that("RMitemRestscore sort = 'diff' sorts by absolute Difference descending", {
  skip_if_not_installed("iarm")
  skip_if_not_installed("eRm")
  set.seed(42)
  df <- as.data.frame(
    matrix(sample(0:1, 200, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(df) <- paste0("Item", 1:5)

  result_sorted   <- RMitemRestscore(df, output = "dataframe", sort = "diff")
  result_unsorted <- RMitemRestscore(df, output = "dataframe")

  expect_true(
    all(diff(abs(result_sorted$Difference)) <= 0),
    info = "Rows should be in descending order of |Difference|"
  )
  # Unsorted should contain the same rows, just in a different order
  expect_setequal(result_sorted$Item, result_unsorted$Item)
})

test_that("RMitemRestscore drops all-NA respondents instead of erroring", {
  skip_if_not_installed("iarm")
  set.seed(42)
  df <- as.data.frame(matrix(sample(0:2, 60 * 6, replace = TRUE), nrow = 60))
  colnames(df) <- paste0("i", 1:6)
  df[3, ] <- NA

  expect_message(k <- RMitemRestscore(df), "no responses dropped")
  expect_match(paste(as.character(k), collapse = "\n"), "of 60 respondents")
})

#' Function renaming in easyRasch2 0.8.0
#'
#' In version 0.8.0, 22 exported functions were renamed under a
#' consistent *domain-prefix \eqn{\to} method \eqn{\to} variant-suffix*
#' scheme. The renames have no semantic effect --- only names changed.
#' Use the table below to migrate existing scripts.
#'
#' @section Domain prefixes:
#' \itemize{
#'   \item `RMdif` --- differential item functioning
#'   \item `RMlocdep` --- local dependence
#'   \item `RMitem` --- per-item statistics (fit, restscore, ICC, hierarchy)
#'   \item `RMdim` --- dimensionality assessment
#'   \item `RMplot` --- descriptive response-distribution plots
#' }
#'
#' Variant suffixes attached at the end:
#' \itemize{
#'   \item `Cutoff` --- simulation-based critical values
#'   \item `Plot` --- visualisation
#'   \item `MI` --- multiple-imputation variant (replaces former `_mi`)
#' }
#'
#' @section Old to new name map:
#' \tabular{ll}{
#'   \strong{Old name} \tab \strong{New name} \cr
#'   `RMpartgamDIF()`        \tab `RMdifGamma()`             \cr
#'   `RMpgDIFcutoff()`       \tab `RMdifGammaCutoff()`       \cr
#'   `RMpgDIFplot()`         \tab `RMdifGammaPlot()`         \cr
#'   `RMpartgamLD()`         \tab `RMlocdepGamma()`          \cr
#'   `RMpgLDcutoff()`        \tab `RMlocdepGammaCutoff()`    \cr
#'   `RMpgLDplot()`          \tab `RMlocdepGammaPlot()`      \cr
#'   `RMlocdepQ3cutoff()`    \tab `RMlocdepQ3Cutoff()`       \cr
#'   `RMlocdepQ3plot()`      \tab `RMlocdepQ3Plot()`         \cr
#'   `RMiteminfit()`         \tab `RMitemInfit()`            \cr
#'   `RMiteminfit_mi()`      \tab `RMitemInfitMI()`          \cr
#'   `RMinfitcutoff()`       \tab `RMitemInfitCutoff()`      \cr
#'   `RMinfitcutoff_mi()`    \tab `RMitemInfitCutoffMI()`    \cr
#'   `RMinfitcutoffPlot()`   \tab `RMitemInfitCutoffPlot()`  \cr
#'   `RMitemrestscore()`     \tab `RMitemRestscore()`        \cr
#'   `RMbootRestscore()`     \tab `RMitemRestscoreBoot()`    \cr
#'   `RMciccPlot()`          \tab `RMitemICCPlot()`          \cr
#'   `RMresidualPCA()`       \tab `RMdimResidualPCA()`       \cr
#'   `RMpcaCutoff()`         \tab `RMdimResidualPCACutoff()` \cr
#'   `RMcfaCutoff()`         \tab `RMdimCFACutoff()`         \cr
#'   `RMcfaPlot()`           \tab `RMdimCFAPlot()`           \cr
#'   `RMmartinLof()`         \tab `RMdimMartinLof()`         \cr
#'   `RMmartinLofResiduals()`\tab `RMdimMartinLofResiduals()`\cr
#'   `RMtileplot()`          \tab `RMplotTile()`             \cr
#'   `RMbarplot()`           \tab `RMplotBar()`              \cr
#'   `RMstackedbarplot()`    \tab `RMplotStackedbar()`       \cr
#' }
#'
#' Unchanged: `RMreliability()`, `RMUreliability()`,
#' `RMitemHierarchy()`, `RMdifLR()`, `RMdifTree()`, `RMlocdepQ3()`,
#' `RMtargeting()`, `RMscoreSE()`.
#'
#' @section No deprecation aliases:
#' No deprecation shims are shipped: each old name is simply gone. This
#' keeps the export list clean ahead of the first CRAN submission. Run a
#' search-and-replace against the table above to migrate.
#'
#' @name easyRasch2-renaming
#' @aliases easyRasch2-renaming
#' @keywords internal
NULL

#' Simulated null distribution for one-factor CFA fit and loadings under PCM unidimensionality
#'
#' Generates a parametric-bootstrap null distribution against which observed
#' one-factor categorical-CFA results can be compared. The simulation draws
#' \code{iterations} datasets from the fitted PCM (or RM, for dichotomous
#' data) using the observed item parameters and a resampled person
#' distribution; each simulated dataset is fitted with
#' \code{lavaan::cfa(..., ordered = TRUE, estimator = "WLSMV")} and both the
#' three fit indices (CFI, RMSEA, SRMR) and the per-item standardized factor
#' loadings are recorded. Because the simulated data satisfy the PCM
#' unidimensional assumption by construction, the resulting distributions are
#' the "expected" reference for what a correctly fitting unidimensional model
#' produces at this sample size and item structure.
#'
#' This function only generates the simulated reference. To obtain the
#' observed-vs-expected tables, pass its result to \code{\link{RMdimCFA}}; for
#' the figures, pass it to \code{\link{RMdimCFAPlot}}.
#'
#' @param data A data.frame or matrix of item responses (non-negative
#'   integers, 0-based). One column per item, one row per person.
#' @param iterations Integer. Number of parametric-bootstrap iterations.
#'   Default `250`.
#' @param percentile Numeric in (50, 100). The strictness of the cutoffs.
#'   Default `99`. Fit indices use a one-sided cutoff in the unfavourable
#'   direction (CFI from below; RMSEA and SRMR from above); standardized
#'   loadings use a two-sided central interval covering `percentile`% of the
#'   simulated distribution (an item is flagged when its observed loading
#'   falls in the outer `100 - percentile`%, split across the two tails).
#' @param output Character. Only `"list"` (the default) is supported and the
#'   function always returns the simulation object. `"kable"` is retained only
#'   to raise an informative error: tables now come from
#'   \code{\link{RMdimCFA}}.
#' @param parallel Logical. If `TRUE` (default), uses parallel processing
#'   via `mirai`. Falls back to sequential if `mirai` is not installed or
#'   `n_cores` cannot be resolved.
#' @param n_cores Integer or `NULL`. Number of parallel workers. When
#'   `NULL`, `getOption("mc.cores")` is consulted; if neither is set,
#'   sequential is used.
#' @param verbose Logical. Show a progress bar (default `FALSE`).
#' @param seed Integer or `NULL`. Master seed for reproducibility.
#' @param estimator Character. The lavaan estimator passed to
#'   `lavaan::cfa()`. Default `"WLSMV"`. Other limited-information
#'   estimators that produce robust/scaled fit indices (e.g.,
#'   `"DWLS"`, `"ULSMV"`) are also accepted; full-information ML is
#'   rejected (incompatible with `ordered = TRUE`).
#'
#' @return A list (the simulation object), with components:
#' \describe{
#'   \item{`simulated`}{data.frame with one row per successful iteration
#'     and columns `iteration`, `cfi`, `rmsea`, `srmr`.}
#'   \item{`simulated_loadings`}{data.frame with one row per successful
#'     iteration: an `iteration` column followed by one column per item
#'     holding the simulated standardized loading.}
#'   \item{`percentile`}{Numeric: the strictness setting used.}
#'   \item{`cutoffs`}{Named numeric vector (`cfi`, `rmsea`, `srmr`) of
#'     one-sided fit-index cutoffs at the chosen percentile.}
#'   \item{`loading_cutoffs`}{data.frame `Item`, `low`, `high` — the
#'     two-sided expected loading interval per item.}
#'   \item{`actual_iterations`}{Number of successful MC iterations.}
#'   \item{`sample_n`}{Number of complete cases used.}
#'   \item{`sample_n_total`}{Number of respondents in the raw input data,
#'     before the complete-case filter.}
#'   \item{`sample_has_na`}{Logical. Whether the raw input data contained
#'     any missing values.}
#'   \item{`n_items`}{Number of items.}
#'   \item{`item_names`}{Character vector of item names.}
#'   \item{`is_polytomous`}{Logical: was a PCM (vs RM) fitted?}
#'   \item{`estimator`}{The lavaan estimator used.}
#' }
#'
#' @details
#' \strong{Generative model.} The data-generating process for each
#' simulated dataset is the PCM (or RM) fitted to the observed data,
#' with persons drawn from the empirical theta distribution
#' (resampled with replacement). This means the simulated data perfectly
#' satisfy the PCM unidimensional assumption.
#'
#' \strong{Estimation model.} The CFA on each simulated dataset uses a
#' single-factor model with all items as ordinal indicators
#' (`F1 =~ I1 + I2 + ...`), fitted with `WLSMV` by default. Reported
#' CFI / RMSEA are the Satorra-Bentler-scaled variants (`cfi.scaled`,
#' `rmsea.scaled`) for consistency across iterations; SRMR is reported
#' unchanged. Standardized loadings are the `est.std` of the `=~` paths
#' from `lavaan::standardizedSolution()`.
#'
#' \strong{Why a null distribution.} A perfectly PCM-unidimensional
#' dataset will typically not yield CFA fit indices at their ideal
#' values (CFI = 1, RMSEA = 0), nor identical loadings across items: PCM
#' uses a logistic threshold structure while WLSMV uses a probit link via
#' the polychoric correlation matrix, and finite samples add sampling
#' variability. The simulated distributions capture both, giving a more
#' honest reference than rule-of-thumb cutoffs derived under continuous-data
#' ML.
#'
#' \strong{Iteration failures.} Some simulated datasets cause WLSMV to
#' fail (non-positive-definite polychoric matrix, boundary thresholds,
#' empty categories). Failed iterations are dropped; `actual_iterations`
#' reflects the number that succeeded.
#'
#' @references
#' Yuan, K.-H., & Bentler, P. M. (2000). Three likelihood-based methods
#' for mean and covariance structure analysis with nonnormal missing
#' data. \emph{Sociological Methodology, 30}(1), 165-200.
#' \doi{10.1111/0081-1750.00078}
#'
#' Rosseel, Y. (2012). lavaan: An R Package for Structural Equation
#' Modeling. \emph{Journal of Statistical Software, 48}(2), 1-36.
#' \doi{10.18637/jss.v048.i02}
#'
#' @seealso \code{\link{RMdimCFA}}, \code{\link{RMdimCFAPlot}},
#'   \code{\link{RMdimResidualPCA}}, \code{\link{RMdimMartinLof}}
#'
#' @examples
#' \donttest{
#' if (requireNamespace("lavaan", quietly = TRUE) &&
#'     requireNamespace("eRm", quietly = TRUE)) {
#'   data("raschdat1", package = "eRm")
#'
#'   # Few iterations for a fast example; use 250+ in real analyses
#'   sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 50,
#'                         parallel = FALSE, seed = 1)
#'
#'   # Observed-vs-expected tables
#'   RMdimCFA(raschdat1[, 1:8], cutoff = sim)
#'
#'   if (requireNamespace("ggplot2", quietly = TRUE)) {
#'     plots <- RMdimCFAPlot(sim, data = raschdat1[, 1:8])
#'     plots$loadings
#'     plots$fit
#'   }
#' }
#' }
#'
#' @export
RMdimCFACutoff <- function(
  data,
  iterations = 250L,
  percentile = 99,
  output = c("list", "kable"),
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  estimator = "WLSMV"
) {
  output <- match.arg(output)
  if (output == "kable") {
    stop(
      "RMdimCFACutoff() now returns the simulation object only (a list).\n",
      "Use RMdimCFA(data, cutoff = <this object>) for the fit and loadings ",
      "tables, and RMdimCFAPlot(<this object>, data) for the figures.",
      call. = FALSE
    )
  }

  if (!requireNamespace("lavaan", quietly = TRUE)) {
    stop(
      "Package 'lavaan' is required for RMdimCFACutoff(). ",
      "Install with: install.packages(\"lavaan\")",
      call. = FALSE
    )
  }

  if (
    !is.numeric(percentile) ||
      length(percentile) != 1L ||
      !is.finite(percentile) ||
      percentile <= 50 ||
      percentile >= 100
  ) {
    stop(
      "`percentile` must be a single numeric in (50, 100). ",
      "Common choices: 95, 99, 99.5.",
      call. = FALSE
    )
  }

  estimator <- toupper(estimator[1L])
  if (estimator %in% c("ML", "MLR", "MLM", "MLF")) {
    stop(
      "`estimator = \"",
      estimator,
      "\"` is not appropriate for ",
      "ordinal-CFA. Use a limited-information estimator such as ",
      "\"WLSMV\", \"DWLS\", or \"ULSMV\".",
      call. = FALSE
    )
  }

  validate_response_data(data)

  # rgl workaround
  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  # Only complete cases. Record the raw total and whether anything was
  # dropped so callers (e.g. RMdimCFAPlot) can report the sample in the
  # standard `n = X of Y respondents` form.
  n_total <- nrow(as.data.frame(data))
  has_na <- anyNA(data)
  data <- stats::na.omit(as.data.frame(data))
  if (nrow(data) == 0L) {
    stop("No complete cases in `data`.", call. = FALSE)
  }
  if (ncol(data) < 3L) {
    stop(
      "RMdimCFACutoff() requires at least 3 items for a one-factor CFA.",
      call. = FALSE
    )
  }

  # Parallel setup -- mirrors RMdimResidualPCACutoff()
  use_parallel <- parallel && requireNamespace("mirai", quietly = TRUE)
  if (parallel && !use_parallel) {
    message(
      "Install 'mirai' for parallel processing: install.packages(\"mirai\")"
    )
    message("Running sequentially...")
  }
  if (use_parallel) {
    if (is.null(n_cores)) {
      n_cores <- getOption("mc.cores")
    }
    if (is.null(n_cores)) {
      warning(
        "For parallel processing, specify n_cores or set options(mc.cores = N).\n",
        "Falling back to sequential.",
        call. = FALSE
      )
      use_parallel <- FALSE
    } else {
      n_cores <- min(n_cores, iterations)
    }
  }

  if (!is.null(seed)) {
    set.seed(seed)
  }
  sim_seeds <- sample.int(.Machine$integer.max, iterations)

  data_mat <- as.matrix(data)
  sample_n <- nrow(data_mat)
  is_polytomous <- max(data_mat, na.rm = TRUE) > 1L
  item_names_vec <- colnames(data_mat)
  if (is.null(item_names_vec)) {
    item_names_vec <- paste0("V", seq_len(ncol(data_mat)))
  }

  # Generating Rasch model: CML item thresholds (psychotools) + WLE person
  # locations, consistent with the rest of the package. The DGP is unchanged
  # (thetas resampled with replacement; data simulated under the model). The
  # CFA itself is fitted by lavaan; `estimator` is the lavaan estimator.
  pool <- .wle_theta_pool(data_mat)
  thr_list <- pool$thr_list
  wle_thetas <- pool$thetas

  sim_data_list <- list(
    type = if (is_polytomous) "polytomous" else "dichotomous",
    thetas = wle_thetas,
    n_items = ncol(data_mat),
    sample_n = sample_n,
    item_names = item_names_vec,
    estimator = estimator
  )
  if (is_polytomous) {
    sim_data_list$deltaslist <- thr_list
  } else {
    sim_data_list$item_params <- unlist(thr_list, use.names = FALSE)
  }

  # Run the simulation
  if (use_parallel) {
    results_raw <- run_cfa_sim_parallel(
      iterations,
      sim_seeds,
      sim_data_list,
      n_cores,
      verbose
    )
  } else {
    results_raw <- run_cfa_sim_sequential(
      iterations,
      sim_seeds,
      sim_data_list,
      verbose
    )
  }

  ok <- vapply(results_raw, is.list, logical(1L))
  successful <- results_raw[ok]

  if (length(successful) == 0L) {
    failed_msgs <- unlist(results_raw[!ok])
    sample_msg <- if (length(failed_msgs) > 0L) {
      unique(failed_msgs)[1L]
    } else {
      "(no message captured)"
    }
    stop(
      "All CFA simulation iterations failed. Example: ",
      sample_msg,
      call. = FALSE
    )
  }

  actual_iterations <- length(successful)

  fit_mat <- do.call(rbind, lapply(successful, `[[`, "fit"))
  load_mat <- do.call(rbind, lapply(successful, `[[`, "loadings"))
  colnames(fit_mat) <- c("cfi", "rmsea", "srmr")
  colnames(load_mat) <- item_names_vec

  simulated_df <- data.frame(
    iteration = seq_len(actual_iterations),
    cfi = as.numeric(fit_mat[, "cfi"]),
    rmsea = as.numeric(fit_mat[, "rmsea"]),
    srmr = as.numeric(fit_mat[, "srmr"]),
    stringsAsFactors = FALSE
  )

  simulated_loadings <- data.frame(
    iteration = seq_len(actual_iterations),
    as.data.frame(load_mat, check.names = FALSE),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  cutoffs <- compute_cfa_cutoffs(simulated_df, percentile)
  loading_cutoffs <- compute_cfa_loading_cutoffs(
    simulated_loadings,
    item_names_vec,
    percentile
  )

  list(
    simulated = simulated_df,
    simulated_loadings = simulated_loadings,
    percentile = percentile,
    cutoffs = cutoffs,
    loading_cutoffs = loading_cutoffs,
    actual_iterations = actual_iterations,
    sample_n = sample_n,
    sample_n_total = n_total,
    sample_has_na = has_na,
    n_items = ncol(data_mat),
    item_names = item_names_vec,
    is_polytomous = is_polytomous,
    estimator = estimator
  )
}

# ===========================================================================
# Internal: cutoff / flag helpers
# ===========================================================================

#' Compute one-sided CFA-fit cutoffs at the given percentile
#'
#' @keywords internal
#' @noRd
compute_cfa_cutoffs <- function(simulated_df, percentile) {
  pct <- percentile / 100
  # Filter to finite values -- lavaan can return Inf for RMSEA when
  # chi-square is exactly 0 (degenerate near-perfect fit on a
  # simulated dataset), which would otherwise propagate into the cutoff.
  cfi_v <- simulated_df$cfi[is.finite(simulated_df$cfi)]
  rmsea_v <- simulated_df$rmsea[is.finite(simulated_df$rmsea)]
  srmr_v <- simulated_df$srmr[is.finite(simulated_df$srmr)]
  c(
    cfi = as.numeric(stats::quantile(cfi_v, 1 - pct, na.rm = TRUE)),
    rmsea = as.numeric(stats::quantile(rmsea_v, pct, na.rm = TRUE)),
    srmr = as.numeric(stats::quantile(srmr_v, pct, na.rm = TRUE))
  )
}

#' Compute two-sided per-item expected loading intervals
#'
#' Central interval covering `percentile`% of each item's simulated
#' standardized loadings (tails of `(100 - percentile) / 2`% each).
#'
#' @keywords internal
#' @noRd
compute_cfa_loading_cutoffs <- function(
  simulated_loadings,
  item_names,
  percentile
) {
  tail <- (1 - percentile / 100) / 2
  do.call(
    rbind,
    lapply(item_names, function(it) {
      v <- simulated_loadings[[it]]
      v <- v[is.finite(v)]
      data.frame(
        Item = it,
        low = as.numeric(stats::quantile(v, tail, na.rm = TRUE)),
        high = as.numeric(stats::quantile(v, 1 - tail, na.rm = TRUE)),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    })
  )
}

#' Tiny ordinal-suffix helper for percentile labels (1st, 2nd, 3rd, 99th)
#'
#' @keywords internal
#' @noRd
ordinal_suffix <- function(n) {
  n_int <- as.integer(round(n))
  if (n_int %% 100 %in% c(11L, 12L, 13L)) {
    return("th")
  }
  switch(as.character(n_int %% 10), "1" = "st", "2" = "nd", "3" = "rd", "th")
}

#' Apply directional fit-index flagging given observed values + cutoffs
#'
#' @keywords internal
#' @noRd
compute_cfa_flagged <- function(observed, cutoffs) {
  c(
    cfi = !is.na(observed[["cfi"]]) && observed[["cfi"]] < cutoffs[["cfi"]],
    rmsea = !is.na(observed[["rmsea"]]) &&
      observed[["rmsea"]] > cutoffs[["rmsea"]],
    srmr = !is.na(observed[["srmr"]]) && observed[["srmr"]] > cutoffs[["srmr"]]
  )
}

#' Flag per-item loadings against the two-sided expected interval
#'
#' Returns a character vector aligned to `loading_cutoffs$Item`:
#' `"below"` (observed loading under the expected interval),
#' `"above"` (over it), or `""` (within).
#'
#' @keywords internal
#' @noRd
compute_cfa_loading_flagged <- function(observed_loadings, loading_cutoffs) {
  obs <- as.numeric(observed_loadings[loading_cutoffs$Item])
  ifelse(
    is.na(obs),
    "",
    ifelse(
      obs < loading_cutoffs$low,
      "below",
      ifelse(obs > loading_cutoffs$high, "above", "")
    )
  )
}

# ===========================================================================
# Observed-vs-expected tables
# ===========================================================================

#' Observed one-factor CFA fit and loadings vs a simulated reference
#'
#' Fits the observed one-factor categorical CFA to `data` and compares its
#' fit indices and per-item standardized loadings against the simulated null
#' distribution produced by \code{\link{RMdimCFACutoff}}. Returns a list of
#' two tables: model-fit indices and per-item loadings, each with the observed
#' value, the expected reference from the simulation, and a flag.
#'
#' @param data A data.frame or matrix of item responses (non-negative
#'   integers, 0-based), the same items used for the cutoff simulation.
#' @param cutoff The list returned by \code{\link{RMdimCFACutoff}}. Required:
#'   observed CFA fit indices are not interpretable without the simulated
#'   reference, so the function errors if it is missing.
#' @param p_value Logical. When `TRUE`, adds bootstrap p-values from the
#'   simulated null distributions: one-sided in the unfavourable direction
#'   for the fit indices (CFI low; RMSEA / SRMR high), two-sided for the
#'   per-item loadings. The `Flagged` columns then reflect
#'   `padj < alpha` instead of the percentile cutoffs. The fit indices and
#'   the loadings are corrected as two separate families. Default `FALSE`.
#' @param correction Character. Multiplicity correction for the p-values:
#'   `"fwer"` (default; Westfall-Young studentised-max step-down), `"fdr_bh"`
#'   (Benjamini-Hochberg), `"fdr_by"` (Benjamini-Yekutieli), or `"none"`.
#'   Ignored when `p_value = FALSE`.
#' @param alpha Numeric in (0, 1). Significance level used to flag
#'   comparisons on the corrected p-value. Default `0.05`. Ignored when
#'   `p_value = FALSE`.
#' @param output Character. `"kable"` (default) returns each table as a
#'   `knitr::kable()`; `"dataframe"` returns plain data.frames.
#'
#' @return A named list with two elements, `fit` and `loadings`:
#' \describe{
#'   \item{`fit`}{CFI / RMSEA / SRMR with columns `Index`, `Observed`,
#'     `Cutoff`, `Direction`, `Flagged` (one-sided, in the unfavourable
#'     direction). With `p_value = TRUE`, columns `p` and `padj` are added
#'     and `Flagged` reflects `padj < alpha`.}
#'   \item{`loadings`}{One row per item with columns `Item`, `Observed`,
#'     `Expected_low`, `Expected_high`, `Flagged` (`"below"` / `"above"` /
#'     `""`). With `p_value = TRUE`, columns `p_loading` and `padj_loading`
#'     are added and `Flagged` reflects `padj_loading < alpha` (direction
#'     from the sign of the deviation from the simulated mean).}
#' }
#' Each element is a `knitr_kable` (when `output = "kable"`) or a data.frame
#' (when `output = "dataframe"`).
#'
#' @details
#' \strong{Bootstrap p-values.} The per-comparison statistic is the residual
#' studentised by the bootstrap mean and SD. Marginal p-values are Monte-Carlo,
#' `(1 + count) / (B + 1)`, so they can be no smaller than `1 / (B + 1)`.
#' `correction = "fwer"` uses the Westfall-Young studentised-max step-down,
#' which exploits the bootstrap dependence among the statistics (Ferreira,
#' 2024); it is liberal when the simulation is small, so at least 1000
#' `iterations` in [RMdimCFACutoff()] are recommended (a warning is issued
#' below that). These p-values are model-conditional and sample-size-sensitive
#' and are reported alongside the simulated expected ranges, not in place of
#' them.
#'
#' @inheritSection RMitemInfit Multiple comparisons
#'
#' @references
#' Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number
#' of hypotheses simultaneously. *Journal of Statistical Theory and Practice,
#' 19*(6). \doi{10.1007/s42519-024-00412-4}
#'
#' Westfall, P. H., & Young, S. S. (1993). *Resampling-Based Multiple Testing*.
#' Wiley.
#'
#' @seealso \code{\link{RMdimCFACutoff}}, \code{\link{RMdimCFAPlot}}
#'
#' @examples
#' \donttest{
#' if (requireNamespace("lavaan", quietly = TRUE) &&
#'     requireNamespace("eRm", quietly = TRUE)) {
#'   data("raschdat1", package = "eRm")
#'   sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 50,
#'                         parallel = FALSE, seed = 1)
#'   tabs <- RMdimCFA(raschdat1[, 1:8], cutoff = sim)
#'   tabs$fit
#'   tabs$loadings
#'
#'   # Bootstrap p-values with family-wise (Westfall-Young) correction
#'   # (use iterations >= 1000 in real analyses for stable p-values)
#'   RMdimCFA(raschdat1[, 1:8], cutoff = sim, p_value = TRUE)
#' }
#' }
#'
#' @export
RMdimCFA <- function(
  data,
  cutoff,
  p_value = FALSE,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  alpha = 0.05,
  output = c("kable", "dataframe")
) {
  output <- match.arg(output)
  correction <- match.arg(correction)
  if (!is.numeric(alpha) || length(alpha) != 1L || alpha <= 0 || alpha >= 1) {
    stop("`alpha` must be a single number in (0, 1).", call. = FALSE)
  }

  if (missing(cutoff) || is.null(cutoff)) {
    stop(
      "`cutoff` is required: pass the object returned by RMdimCFACutoff(). ",
      "Observed CFA fit indices are not interpretable without the ",
      "simulated reference distribution.",
      call. = FALSE
    )
  }
  req <- c(
    "simulated",
    "simulated_loadings",
    "cutoffs",
    "loading_cutoffs",
    "percentile",
    "item_names",
    "estimator"
  )
  if (!is.list(cutoff) || !all(req %in% names(cutoff))) {
    stop(
      "`cutoff` must be the list returned by RMdimCFACutoff().",
      call. = FALSE
    )
  }

  # --- p-value prerequisites --------------------------------------------------
  # The full cutoff object always carries $simulated / $simulated_loadings
  # (checked above), so only the iteration count needs a warning.
  if (
    p_value &&
      !is.null(cutoff$actual_iterations) &&
      cutoff$actual_iterations < 1000L
  ) {
    warning(
      "Bootstrap p-values are based on only ",
      cutoff$actual_iterations,
      " simulation iterations. With few iterations the studentised-max ",
      "(FWER) correction is liberal and small p-values are imprecise; ",
      "use iterations >= 1000 in RMdimCFACutoff() for reliable p-values.",
      call. = FALSE
    )
  }

  if (!requireNamespace("lavaan", quietly = TRUE)) {
    stop(
      "Package 'lavaan' is required for RMdimCFA(). ",
      "Install with: install.packages(\"lavaan\")",
      call. = FALSE
    )
  }
  if (output == "kable" && !requireNamespace("knitr", quietly = TRUE)) {
    stop("Package 'knitr' is required for output = \"kable\".", call. = FALSE)
  }

  validate_response_data(data)

  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  data <- as.data.frame(data)
  n_total_cfa <- nrow(data)
  has_na_cfa <- anyNA(data)
  data <- stats::na.omit(data)
  if (nrow(data) == 0L) {
    stop("No complete cases in `data`.", call. = FALSE)
  }
  n_clause_cfa <- .n_caption(
    nrow(data),
    n_total_cfa,
    if (has_na_cfa) "complete cases" else character()
  )

  item_names <- cutoff$item_names
  if (!setequal(names(data), item_names)) {
    stop(
      "Item names in `data` do not match those used for the cutoff.\n",
      "  data   : ",
      paste(names(data), collapse = ", "),
      "\n",
      "  cutoff : ",
      paste(item_names, collapse = ", "),
      call. = FALSE
    )
  }
  data <- data[, item_names, drop = FALSE]

  # Observed CFA fit + standardized loadings (one lavaan fit)
  obs <- run_single_cfa_sim(
    seed = NA,
    data_list = list(item_names = item_names, estimator = cutoff$estimator),
    obs_data = data
  )
  if (!is.list(obs)) {
    stop("Observed CFA fit failed: ", obs, call. = FALSE)
  }
  observed_fit <- stats::setNames(obs$fit, c("cfi", "rmsea", "srmr"))
  observed_load <- obs$loadings

  flagged_fit <- compute_cfa_flagged(observed_fit, cutoff$cutoffs)
  loading_flagged <- compute_cfa_loading_flagged(
    observed_load,
    cutoff$loading_cutoffs
  )

  fit_df <- build_cfa_fit_df(
    observed_fit,
    cutoff$cutoffs,
    flagged_fit,
    cutoff$percentile
  )
  load_df <- build_cfa_loadings_df(
    observed_load,
    cutoff$loading_cutoffs,
    loading_flagged
  )

  # --- Bootstrap p-values (two separate families: fit indices, loadings) ------
  if (p_value) {
    # Fit indices: one-sided in the unfavourable direction. CFI is negated so
    # "large = bad" holds for all three; RMSEA can be Inf on a degenerate
    # simulated dataset, so non-finite simulated values are dropped to NA.
    sim_fit <- cbind(
      CFI = -cutoff$simulated$cfi,
      RMSEA = cutoff$simulated$rmsea,
      SRMR = cutoff$simulated$srmr
    )
    sim_fit[!is.finite(sim_fit)] <- NA_real_
    obs_fit_stat <- c(
      CFI = -observed_fit[["cfi"]],
      RMSEA = observed_fit[["rmsea"]],
      SRMR = observed_fit[["srmr"]]
    )
    pv_fit <- .bootstrap_pvalues(
      obs_fit_stat,
      sim_fit,
      correction = correction,
      tail = "upper"
    )
    fit_df$p <- pv_fit$p
    fit_df$padj <- pv_fit$padj
    fit_df$Flagged <- ifelse(
      !is.na(fit_df$padj) & fit_df$padj < alpha,
      "TRUE",
      ""
    )
    fit_df <- fit_df[, c(
      "Index",
      "Observed",
      "Cutoff",
      "Direction",
      "p",
      "padj",
      "Flagged"
    )]

    # Loadings: two-sided per item; the flag direction comes from the sign of
    # the deviation from the simulated mean loading.
    sim_load <- as.matrix(cutoff$simulated_loadings[, item_names, drop = FALSE])
    sim_load[!is.finite(sim_load)] <- NA_real_
    obs_load_vec <- stats::setNames(
      as.numeric(observed_load[item_names]),
      item_names
    )
    pv_load <- .bootstrap_pvalues(
      obs_load_vec,
      sim_load,
      correction = correction,
      tail = "two.sided"
    )
    idx <- match(load_df$Item, pv_load$name)
    load_df$p_loading <- pv_load$p[idx]
    load_df$padj_loading <- pv_load$padj[idx]
    sim_mean <- colMeans(sim_load, na.rm = TRUE)[load_df$Item]
    sig <- !is.na(load_df$padj_loading) & load_df$padj_loading < alpha
    load_df$Flagged <- ifelse(
      sig,
      ifelse(load_df$Observed < sim_mean, "below", "above"),
      ""
    )
    load_df <- load_df[, c(
      "Item",
      "Observed",
      "Expected_low",
      "Expected_high",
      "p_loading",
      "padj_loading",
      "Flagged"
    )]
  }

  if (output == "dataframe") {
    return(list(fit = fit_df, loadings = load_df))
  }

  # Kable display rounding (the dataframe output above stays unrounded)
  fit_df <- .round_display(fit_df, c(Observed = 4, Cutoff = 4, p = 4, padj = 4))
  load_df <- .round_display(load_df, c(
    Observed = 3, Expected_low = 3, Expected_high = 3,
    p_loading = 4, padj_loading = 4
  ))

  list(
    fit = render_cfa_fit_kable(
      fit_df,
      cutoff,
      n_clause_cfa,
      p_value = p_value,
      correction = correction,
      alpha = alpha
    ),
    loadings = render_cfa_loadings_kable(
      load_df,
      cutoff,
      n_clause_cfa,
      p_value = p_value,
      correction = correction,
      alpha = alpha
    )
  )
}

#' Assemble the fit-index data.frame (Index/Observed/Cutoff/Direction/Flagged)
#'
#' @keywords internal
#' @noRd
build_cfa_fit_df <- function(observed, cutoffs, flagged, percentile) {
  cfi_pct_lbl <- 100 - percentile
  pct_suffix <- ordinal_suffix(percentile)
  cfi_suffix <- ordinal_suffix(cfi_pct_lbl)
  data.frame(
    Index = c("CFI", "RMSEA", "SRMR"),
    Observed = c(observed[["cfi"]], observed[["rmsea"]], observed[["srmr"]]),
    Cutoff = c(cutoffs[["cfi"]], cutoffs[["rmsea"]], cutoffs[["srmr"]]),
    Direction = c(
      paste0("< ", cfi_pct_lbl, cfi_suffix, " pct"),
      paste0("> ", percentile, pct_suffix, " pct"),
      paste0("> ", percentile, pct_suffix, " pct")
    ),
    Flagged = ifelse(flagged[c("cfi", "rmsea", "srmr")], "TRUE", ""),
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

#' Assemble the loadings data.frame (Item/Observed/Expected_low/high/Flagged)
#'
#' @keywords internal
#' @noRd
build_cfa_loadings_df <- function(
  observed_loadings,
  loading_cutoffs,
  loading_flagged
) {
  data.frame(
    Item = loading_cutoffs$Item,
    Observed = as.numeric(observed_loadings[loading_cutoffs$Item]),
    Expected_low = loading_cutoffs$low,
    Expected_high = loading_cutoffs$high,
    Flagged = loading_flagged,
    stringsAsFactors = FALSE,
    row.names = NULL
  )
}

#' @keywords internal
#' @noRd
render_cfa_fit_kable <- function(
  fit_df,
  cutoff,
  n_clause,
  p_value = FALSE,
  correction = "fwer",
  alpha = 0.05
) {
  pct_label <- cutoff$percentile
  caption <- paste0(
    if (cutoff$is_polytomous) "Partial Credit Model" else "Rasch Model",
    " posterior-predictive CFA fit-index check. ",
    "Observed CFA fit (one-factor, lavaan ",
    cutoff$estimator,
    ", ordered = TRUE) vs simulated null under ",
    if (cutoff$is_polytomous) "PCM" else "RM",
    " unidimensionality (",
    cutoff$actual_iterations,
    " iterations at n = ",
    cutoff$sample_n,
    " simulees). ",
    n_clause,
    if (p_value) {
      paste0(
        ". Cutoffs shown at the ",
        pct_label,
        "th percentile for reference. One-sided bootstrap p-values in the ",
        "unfavourable direction (CFI low; RMSEA/SRMR high); multiplicity ",
        "correction: ",
        .correction_label(correction),
        "; flagged at padj < ",
        alpha,
        ". p-values cannot be smaller than 1/(",
        cutoff$actual_iterations,
        "+1) = ",
        round(1 / (cutoff$actual_iterations + 1), 4),
        "."
      )
    } else {
      paste0(
        ". Cutoffs one-sided at the ",
        pct_label,
        "th percentile; flagged when the observed value lies in the worst ",
        100 - pct_label,
        "% of the null in the unfavourable direction."
      )
    }
  )
  knitr::kable(
    fit_df,
    format = "pipe",
    caption = caption,
    row.names = FALSE,
    col.names = if (p_value) {
      c("Index", "Observed", "Cutoff", "Direction", "p", "p (adj)", "Flagged")
    } else {
      names(fit_df)
    }
  )
}

#' @keywords internal
#' @noRd
render_cfa_loadings_kable <- function(
  load_df,
  cutoff,
  n_clause,
  p_value = FALSE,
  correction = "fwer",
  alpha = 0.05
) {
  pct_label <- cutoff$percentile
  caption <- paste0(
    "Standardized factor loadings (one-factor, lavaan ",
    cutoff$estimator,
    ") vs the simulated expected range under ",
    if (cutoff$is_polytomous) "PCM" else "RM",
    " unidimensionality (",
    cutoff$actual_iterations,
    " iterations at n = ",
    cutoff$sample_n,
    " simulees). ",
    n_clause,
    if (p_value) {
      paste0(
        ". Expected range shown for reference (two-sided central ",
        pct_label,
        "% interval). p/p (adj): two-sided bootstrap p-values; multiplicity ",
        "correction: ",
        .correction_label(correction),
        "; flagged at padj < ",
        alpha,
        " (below / above = direction of the deviation)."
      )
    } else {
      paste0(
        ". Expected range is the two-sided central ",
        pct_label,
        "% interval of the simulated loadings; Flagged = below / above that range."
      )
    }
  )
  knitr::kable(
    load_df,
    format = "pipe",
    row.names = FALSE,
    col.names = if (p_value) {
      c(
        "Item",
        "Observed",
        "Expected low",
        "Expected high",
        "p",
        "p (adj)",
        "Flagged"
      )
    } else {
      c(
        "Item",
        "Observed",
        "Expected low",
        "Expected high",
        "Flagged"
      )
    },
    caption = caption
  )
}

# ===========================================================================
# Plot companion
# ===========================================================================

#' Plot observed CFA fit and loadings against the simulated null
#'
#' Returns two figures (in a list) comparing the observed one-factor CFA to
#' the simulated null distribution from \code{\link{RMdimCFACutoff}}:
#' a per-item standardized-loadings plot (observed marker against each item's
#' simulated distribution and expected range, in the style of
#' \code{\link{RMitemInfitPlot}}), and a faceted plot of the CFI / RMSEA
#' / SRMR distributions with the observed value overlaid.
#'
#' @param simfit The list returned by \code{\link{RMdimCFACutoff}}.
#' @param data The item-response data the CFA was run on (the same items used
#'   for the cutoff). Required: the observed values are computed from it.
#' @param percentile Numeric in (50, 100) or `NULL`. When supplied, the
#'   cutoffs and flags are recomputed at this percentile from the stored
#'   simulated distributions (no re-simulation). When `NULL` (default), the
#'   percentile from the original `RMdimCFACutoff()` call is reused.
#'
#' @return A named list of two `ggplot` objects:
#' \describe{
#'   \item{`loadings`}{Per-item standardized loadings: simulated distribution
#'     (dots), expected interval, and the observed loading as a diamond
#'     (red when flagged).}
#'   \item{`fit`}{Faceted CFI / RMSEA / SRMR simulated distributions with the
#'     observed value and cutoff overlaid.}
#' }
#'
#' @seealso \code{\link{RMdimCFACutoff}}, \code{\link{RMdimCFA}}
#'
#' @examples
#' \donttest{
#' if (requireNamespace("lavaan", quietly = TRUE) &&
#'     requireNamespace("ggplot2", quietly = TRUE) &&
#'     requireNamespace("eRm", quietly = TRUE)) {
#'   data("raschdat1", package = "eRm")
#'   sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 50,
#'                         parallel = FALSE, seed = 1)
#'   plots <- RMdimCFAPlot(sim, data = raschdat1[, 1:8])
#'   plots$loadings
#'   plots$fit
#' }
#' }
#'
#' @importFrom rlang .data
#' @export
RMdimCFAPlot <- function(simfit, data, percentile = NULL) {
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop(
      "Package 'ggplot2' is required for RMdimCFAPlot(). ",
      "Install with: install.packages(\"ggplot2\")",
      call. = FALSE
    )
  }
  if (!requireNamespace("ggdist", quietly = TRUE)) {
    stop(
      "Package 'ggdist' is required for RMdimCFAPlot(). ",
      "Install with: install.packages(\"ggdist\")",
      call. = FALSE
    )
  }

  req <- c(
    "simulated",
    "simulated_loadings",
    "cutoffs",
    "loading_cutoffs",
    "percentile",
    "item_names",
    "estimator"
  )
  if (!is.list(simfit) || !all(req %in% names(simfit))) {
    stop(
      "`simfit` must be the result returned by RMdimCFACutoff().",
      call. = FALSE
    )
  }
  if (missing(data)) {
    stop(
      "`data` is required: the observed CFA fit and loadings are computed ",
      "from it for the overlay.",
      call. = FALSE
    )
  }

  # Resolve / override percentile
  if (is.null(percentile)) {
    percentile <- simfit$percentile
  } else if (
    !is.numeric(percentile) ||
      length(percentile) != 1L ||
      !is.finite(percentile) ||
      percentile <= 50 ||
      percentile >= 100
  ) {
    stop(
      "`percentile` must be a single numeric in (50, 100), or NULL.",
      call. = FALSE
    )
  }
  cutoffs <- compute_cfa_cutoffs(simfit$simulated, percentile)
  loading_cutoffs <- compute_cfa_loading_cutoffs(
    simfit$simulated_loadings,
    simfit$item_names,
    percentile
  )

  # Observed fit + loadings from the data
  validate_response_data(data)
  data <- stats::na.omit(as.data.frame(data))
  if (!setequal(names(data), simfit$item_names)) {
    stop(
      "Item names in `data` do not match those used for the cutoff.",
      call. = FALSE
    )
  }
  data <- data[, simfit$item_names, drop = FALSE]

  old_rgl <- getOption("rgl.useNULL")
  options(rgl.useNULL = TRUE)
  on.exit(options(rgl.useNULL = old_rgl), add = TRUE)

  obs <- run_single_cfa_sim(
    seed = NA,
    data_list = list(
      item_names = simfit$item_names,
      estimator = simfit$estimator
    ),
    obs_data = data
  )
  if (!is.list(obs)) {
    stop("Observed CFA fit failed: ", obs, call. = FALSE)
  }
  observed_fit <- stats::setNames(obs$fit, c("cfi", "rmsea", "srmr"))
  observed_load <- obs$loadings

  list(
    loadings = cfa_loadings_plot(
      simfit,
      observed_load,
      loading_cutoffs,
      percentile
    ),
    fit = cfa_fit_plot(simfit, observed_fit, cutoffs, percentile)
  )
}

#' Standard sample-size clause for the CFA plot captions
#'
#' Reports the (complete-case) simulation sample in the standard
#' `n = X of Y respondents (policy)` form. `sample_n_total` /
#' `sample_has_na` are absent in cutoff objects made by older versions,
#' so fall back to the plain count.
#'
#' @keywords internal
#' @noRd
.cfa_sample_clause <- function(simfit) {
  .n_caption(
    simfit$sample_n,
    if (is.null(simfit$sample_n_total)) {
      simfit$sample_n
    } else {
      simfit$sample_n_total
    },
    if (isTRUE(simfit$sample_has_na)) "complete cases" else character()
  )
}

#' Per-item standardized-loadings plot (RMitemInfitPlot style)
#'
#' @keywords internal
#' @noRd
cfa_loadings_plot <- function(
  simfit,
  observed_load,
  loading_cutoffs,
  percentile
) {
  item_names <- simfit$item_names
  item_levels <- rev(item_names)
  sim_load <- simfit$simulated_loadings

  # Long-format simulated loadings
  sim_long <- do.call(
    rbind,
    lapply(item_names, function(it) {
      data.frame(
        Item = it,
        Value = sim_load[[it]],
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    })
  )
  sim_long$Item <- factor(sim_long$Item, levels = item_levels)

  # Per-item summary intervals (thin = 0.1/99.9, thick = 16.7/83.3, median)
  lo_hi <- do.call(
    rbind,
    lapply(item_names, function(it) {
      v <- sim_load[[it]]
      v <- v[is.finite(v)]
      data.frame(
        Item = it,
        lo = stats::quantile(v, 0.001, na.rm = TRUE),
        hi = stats::quantile(v, 0.999, na.rm = TRUE),
        lo66 = stats::quantile(v, 0.167, na.rm = TRUE),
        hi66 = stats::quantile(v, 0.833, na.rm = TRUE),
        median = stats::median(v, na.rm = TRUE),
        stringsAsFactors = FALSE,
        row.names = NULL
      )
    })
  )
  lo_hi$Item_f <- factor(lo_hi$Item, levels = item_levels)

  flags <- compute_cfa_loading_flagged(observed_load, loading_cutoffs)
  obs_df <- data.frame(
    Item = loading_cutoffs$Item,
    observed = as.numeric(observed_load[loading_cutoffs$Item]),
    Flagged = flags != "",
    stringsAsFactors = FALSE
  )
  obs_df$Item <- factor(obs_df$Item, levels = item_levels)
  obs_df$Color <- ifelse(obs_df$Flagged, "red", "sienna2")

  caption_text <- er2_caption(paste0(
    "Standardized factor loadings from ",
    simfit$actual_iterations,
    " datasets simulated under ",
    if (simfit$is_polytomous) "PCM" else "RM",
    " unidimensionality at ",
    .cfa_sample_clause(simfit),
    " per dataset, refitted with lavaan::cfa(ordered = TRUE, estimator = \"",
    simfit$estimator,
    "\").\n",
    "Diamonds: observed loading (red = outside the two-sided ",
    percentile,
    "% expected range). Black dots: simulated median."
  ))

  ggplot2::ggplot(sim_long, ggplot2::aes(x = .data$Value, y = .data$Item)) +
    ggdist::stat_dots(
      ggplot2::aes(slab_fill = ggplot2::after_stat(.data$level)),
      quantiles = simfit$actual_iterations,
      layout = "weave",
      slab_color = NA,
      .width = c(0.666, 0.999)
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$lo,
        xend = .data$hi,
        y = .data$Item_f,
        yend = .data$Item_f
      ),
      color = "black",
      linewidth = 0.7
    ) +
    ggplot2::geom_segment(
      data = lo_hi,
      ggplot2::aes(
        x = .data$lo66,
        xend = .data$hi66,
        y = .data$Item_f,
        yend = .data$Item_f
      ),
      color = "black",
      linewidth = 1.2
    ) +
    ggplot2::geom_point(
      data = lo_hi,
      ggplot2::aes(x = .data$median, y = .data$Item_f),
      size = 3.6
    ) +
    ggplot2::geom_point(
      data = obs_df,
      ggplot2::aes(x = .data$observed, y = .data$Item, colour = .data$Color),
      shape = 18,
      size = 4,
      position = ggplot2::position_nudge(y = -0.1)
    ) +
    ggplot2::scale_colour_identity() +
    ggplot2::scale_color_manual(
      values = scales::brewer_pal()(3)[-1],
      aesthetics = "slab_fill",
      guide = "none"
    ) +
    ggplot2::labs(
      x = "Standardized factor loading",
      y = "Item",
      caption = caption_text
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(panel.spacing = ggplot2::unit(0.7, "cm")) +
    er2_axis_margins() +
    er2_plot_caption()
}

#' Faceted CFI / RMSEA / SRMR distribution plot with observed overlay
#'
#' @keywords internal
#' @noRd
cfa_fit_plot <- function(simfit, observed_fit, cutoffs, percentile) {
  flagged <- compute_cfa_flagged(observed_fit, cutoffs)

  sim_long <- data.frame(
    Index = factor(
      rep(c("CFI", "RMSEA", "SRMR"), each = nrow(simfit$simulated)),
      levels = c("CFI", "RMSEA", "SRMR")
    ),
    Value = c(
      simfit$simulated$cfi,
      simfit$simulated$rmsea,
      simfit$simulated$srmr
    ),
    stringsAsFactors = FALSE
  )

  obs_df <- data.frame(
    Index = factor(
      c("CFI", "RMSEA", "SRMR"),
      levels = c("CFI", "RMSEA", "SRMR")
    ),
    Observed = c(
      observed_fit[["cfi"]],
      observed_fit[["rmsea"]],
      observed_fit[["srmr"]]
    ),
    Cutoff = c(cutoffs[["cfi"]], cutoffs[["rmsea"]], cutoffs[["srmr"]]),
    Flagged = c(flagged[["cfi"]], flagged[["rmsea"]], flagged[["srmr"]]),
    stringsAsFactors = FALSE
  )
  obs_df$Color <- ifelse(obs_df$Flagged, "red", "grey30")

  cfi_pct_lbl <- 100 - percentile
  caption <- er2_caption(paste0(
    "Histograms: ",
    simfit$actual_iterations,
    " datasets simulated under ",
    if (simfit$is_polytomous) "PCM" else "RM",
    " unidimensionality at ",
    .cfa_sample_clause(simfit),
    " per dataset,\nrefitted with lavaan::cfa(ordered = TRUE, estimator = \"",
    simfit$estimator,
    "\").\n",
    "Diamond: observed value (red = flagged at the ",
    percentile,
    "th percentile in the unfavourable direction).\n",
    "Dashed line: cutoff (CFI: ",
    cfi_pct_lbl,
    "th pct; RMSEA / SRMR: ",
    percentile,
    "th pct)."
  ))

  ggplot2::ggplot(sim_long, ggplot2::aes(x = .data$Value)) +
    ggplot2::geom_histogram(bins = 30, fill = "grey80", colour = "white") +
    ggplot2::geom_vline(
      data = obs_df,
      ggplot2::aes(xintercept = .data$Cutoff),
      linetype = "dashed",
      colour = "grey40"
    ) +
    ggplot2::geom_point(
      data = obs_df,
      ggplot2::aes(x = .data$Observed, colour = .data$Color),
      y = 0,
      size = 4.5,
      shape = 18
    ) +
    ggplot2::scale_colour_identity() +
    ggplot2::facet_wrap(~Index, scales = "free", nrow = 1) +
    ggplot2::labs(
      title = "Observed CFA fit vs simulated null distribution",
      x = "Fit index value",
      y = "Count of simulated datasets",
      caption = caption
    ) +
    ggplot2::theme_bw(base_size = 13) +
    er2_axis_margins() +
    er2_plot_caption()
}

# ===========================================================================
# Internal: single iteration
# ===========================================================================

#' Run one CFA simulation iteration
#'
#' If `obs_data` is supplied, it is used directly (the observed-fit call);
#' otherwise a new dataset is simulated under the fitted PCM/RM. Returns a
#' list `list(fit = c(cfi, rmsea, srmr), loadings = <named vector>)` on
#' success, or a character message on failure.
#'
#' @keywords internal
#' @noRd
run_single_cfa_sim <- function(seed, data_list, obs_data = NULL) {
  build_sim_df <- function() {
    set.seed(seed)
    thetas_res <- sample(
      data_list$thetas,
      size = data_list$sample_n,
      replace = TRUE
    )

    if (data_list$type == "dichotomous") {
      sim_mat <- psychotools::rrm(
        theta = thetas_res,
        beta = data_list$item_params
      )
      sim_df <- as.data.frame(sim_mat$data)
      colnames(sim_df) <- data_list$item_names

      pos_counts <- colSums(sim_df, na.rm = TRUE)
      neg_counts <- nrow(sim_df) - pos_counts
      if (any(pos_counts < 2L) || any(neg_counts < 2L)) {
        return(
          "validation_failed: an item has < 2 responses in one of the two categories"
        )
      }
      return(sim_df)
    }

    sim_mat <- sim_partial_score(data_list$deltaslist, thetas_res)
    sim_df <- as.data.frame(sim_mat)
    colnames(sim_df) <- data_list$item_names

    n_cats <- vapply(
      data_list$deltaslist,
      function(d) length(d) + 1L,
      integer(1L)
    )
    for (j in seq_len(ncol(sim_df))) {
      tab <- tabulate(sim_df[[j]] + 1L, nbins = n_cats[j])
      if (any(tab == 0L)) {
        return("validation_failed: not all categories represented")
      }
    }
    sim_df
  }

  tryCatch(
    {
      fit_df <- if (is.null(obs_data)) build_sim_df() else obs_data
      if (is.character(fit_df)) {
        return(fit_df)
      }

      fmla <- paste0("F1 =~ ", paste(data_list$item_names, collapse = " + "))

      fit <- suppressWarnings(suppressMessages(
        lavaan::cfa(
          model = fmla,
          data = fit_df,
          ordered = data_list$item_names,
          estimator = data_list$estimator,
          warn = FALSE,
          verbose = FALSE
        )
      ))

      if (!isTRUE(lavaan::lavInspect(fit, "converged"))) {
        return("convergence_failed: lavaan did not converge")
      }

      suppressWarnings(list(
        fit = extract_cfa_fit(fit, data_list$estimator),
        loadings = extract_cfa_loadings(fit, data_list$item_names)
      ))
    },
    error = function(e) as.character(conditionMessage(e))
  )
}

#' Pull (CFI, RMSEA, SRMR) from a fitted lavaan object
#'
#' Uses the Satorra-Bentler-SCALED CFI / RMSEA variants when available
#' (`cfi.scaled`, `rmsea.scaled`), falling back to the uncorrected
#' `cfi` / `rmsea` when not. SRMR is unaffected by the correction. The
#' `.scaled` variants are well-defined across nearly all fits (unlike the
#' `.robust` variants, which return `NA` for a non-trivial fraction at small
#' n); for a percentile comparison the binding requirement is that the same
#' metric is computed for observed and simulated iterations.
#'
#' @keywords internal
#' @noRd
extract_cfa_fit <- function(fit, estimator = NULL) {
  fm <- lavaan::fitMeasures(fit)
  pick_scaled <- function(name) {
    key_s <- paste0(name, ".scaled")
    if (key_s %in% names(fm) && is.finite(fm[[key_s]])) {
      return(as.numeric(fm[[key_s]]))
    }
    if (name %in% names(fm) && is.finite(fm[[name]])) {
      return(as.numeric(fm[[name]]))
    }
    NA_real_
  }
  c(pick_scaled("cfi"), pick_scaled("rmsea"), as.numeric(fm[["srmr"]]))
}

#' Pull standardized factor loadings (one factor) from a fitted lavaan object
#'
#' Returns the `est.std` of the `=~` paths from
#' `lavaan::standardizedSolution()`, aligned to `item_names`.
#'
#' @keywords internal
#' @noRd
extract_cfa_loadings <- function(fit, item_names) {
  ss <- lavaan::standardizedSolution(fit)
  lam <- ss[ss$op == "=~", , drop = FALSE]
  v <- stats::setNames(as.numeric(lam$est.std), lam$rhs)
  # Keep names so callers can index by item (observed-side lookups rely on it).
  stats::setNames(as.numeric(v[item_names]), item_names)
}

# ===========================================================================
# Internal: parallel runner
# ===========================================================================

#' @keywords internal
#' @noRd
run_cfa_sim_parallel <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  n_cores,
  verbose = FALSE
) {
  mirai::daemons(n_cores)
  on.exit(mirai::daemons(0), add = TRUE)

  if (verbose) {
    message(sprintf("Starting %d daemons...", n_cores))
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  tasks <- lapply(seq_len(iterations), function(i) {
    mirai::mirai(
      {
        run_single_cfa_sim(seed, data_list)
      },
      seed = sim_seeds[i],
      data_list = sim_data_list,
      run_single_cfa_sim = run_single_cfa_sim,
      extract_cfa_fit = extract_cfa_fit,
      extract_cfa_loadings = extract_cfa_loadings,
      sim_partial_score = sim_partial_score,
      sim_poly_item = sim_poly_item
    )
  })

  results <- vector("list", iterations)
  for (i in seq_len(iterations)) {
    res <- mirai::call_mirai(tasks[[i]])$data
    results[[i]] <- if (inherits(res, "errorValue")) "mirai_error" else res
    if (verbose) utils::setTxtProgressBar(pb, i)
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# ===========================================================================
# Internal: sequential runner
# ===========================================================================

#' @keywords internal
#' @noRd
run_cfa_sim_sequential <- function(
  iterations,
  sim_seeds,
  sim_data_list,
  verbose = FALSE
) {
  if (verbose) {
    pb <- utils::txtProgressBar(min = 0, max = iterations, style = 3)
  }

  results <- vector("list", iterations)
  for (i in seq_len(iterations)) {
    results[[i]] <- run_single_cfa_sim(sim_seeds[i], sim_data_list)
    if (verbose) utils::setTxtProgressBar(pb, i)
  }

  if (verbose) {
    close(pb)
    message("")
  }

  results
}

# Tests for RMdimCFACutoff() / RMdimCFA() / RMdimCFAPlot()
#
# Iterations are intentionally very small (5-30) so the file runs in
# a few seconds. Each test uses parallel = FALSE because parallel
# requires mirai daemons and complicates CRAN test runs.

# ---------------------------------------------------------------------
# Validation errors
# ---------------------------------------------------------------------
test_that("RMdimCFACutoff errors when lavaan is not installed", {
  skip_if(requireNamespace("lavaan", quietly = TRUE),
          "lavaan is installed; skipping missing-package error path test")
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE),
                             nrow = 40, ncol = 5))
  expect_error(RMdimCFACutoff(df), regexp = "lavaan")
})

test_that("RMdimCFACutoff errors when data has non-zero minimum", {
  skip_if_not_installed("lavaan")
  df <- as.data.frame(matrix(sample(1:3, 200, replace = TRUE),
                             nrow = 40, ncol = 5))
  expect_error(RMdimCFACutoff(df), regexp = "scored starting at 0")
})

test_that("RMdimCFACutoff errors when fewer than 3 items", {
  skip_if_not_installed("lavaan")
  df <- as.data.frame(matrix(sample(0:1, 100, replace = TRUE),
                             nrow = 50, ncol = 2))
  expect_error(RMdimCFACutoff(df), regexp = "at least 3 items")
})

test_that("RMdimCFACutoff rejects ML-family estimators", {
  skip_if_not_installed("lavaan")
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE),
                             nrow = 40, ncol = 5))
  for (est in c("ML", "MLR", "MLM", "MLF")) {
    expect_error(
      RMdimCFACutoff(df, estimator = est, iterations = 5, parallel = FALSE),
      regexp = "not appropriate for ordinal-CFA"
    )
  }
})

test_that("RMdimCFACutoff rejects out-of-range percentile", {
  skip_if_not_installed("lavaan")
  df <- as.data.frame(matrix(sample(0:1, 200, replace = TRUE),
                             nrow = 40, ncol = 5))
  for (bad in c(0, 50, 100, 101, NA, -1)) {
    expect_error(
      RMdimCFACutoff(df, percentile = bad, iterations = 5, parallel = FALSE),
      regexp = "percentile"
    )
  }
})

test_that("RMdimCFACutoff(output = 'kable') errors with a migration message", {
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  data("raschdat1", package = "eRm")
  expect_error(
    RMdimCFACutoff(raschdat1[, 1:8], output = "kable",
                   iterations = 5, parallel = FALSE, seed = 1),
    regexp = "RMdimCFA"
  )
})

# ---------------------------------------------------------------------
# RMdimCFACutoff output structure (simulate-only list)
# ---------------------------------------------------------------------
test_that("RMdimCFACutoff returns the simulate-only list on dichotomous data", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  data("raschdat1", package = "eRm")

  res <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 10,
                        parallel = FALSE, seed = 1)

  expect_type(res, "list")
  expected_names <- c("simulated", "simulated_loadings", "percentile",
                      "cutoffs", "loading_cutoffs", "actual_iterations",
                      "sample_n", "n_items", "item_names", "is_polytomous",
                      "estimator")
  expect_true(all(expected_names %in% names(res)))
  # No observed/flagged any more -- those moved to RMdimCFA()
  expect_false(any(c("observed", "flagged") %in% names(res)))

  expect_s3_class(res$simulated, "data.frame")
  expect_named(res$simulated, c("iteration", "cfi", "rmsea", "srmr"))
  expect_s3_class(res$simulated_loadings, "data.frame")
  expect_equal(names(res$simulated_loadings),
               c("iteration", res$item_names))
  expect_equal(nrow(res$simulated), nrow(res$simulated_loadings))

  expect_named(res$cutoffs, c("cfi", "rmsea", "srmr"))
  expect_s3_class(res$loading_cutoffs, "data.frame")
  expect_named(res$loading_cutoffs, c("Item", "low", "high"))
  expect_equal(nrow(res$loading_cutoffs), 8L)
  expect_true(all(res$loading_cutoffs$low <= res$loading_cutoffs$high))

  expect_equal(res$percentile, 99)
  expect_false(res$is_polytomous)
  expect_equal(res$estimator, "WLSMV")
  expect_equal(res$n_items, 8L)
})

test_that("Cutoffs change with different percentile values, same simulation", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  data("raschdat1", package = "eRm")

  res99 <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 30, parallel = FALSE,
                          seed = 1, percentile = 99)
  res95 <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 30, parallel = FALSE,
                          seed = 1, percentile = 95)

  expect_equal(res99$simulated, res95$simulated)
  expect_equal(res99$simulated_loadings, res95$simulated_loadings)
  expect_gte(res99$cutoffs[["rmsea"]], res95$cutoffs[["rmsea"]])
  expect_lte(res99$cutoffs[["cfi"]],   res95$cutoffs[["cfi"]])
  # Wider percentile => wider expected loading interval
  expect_true(all(res99$loading_cutoffs$low  <= res95$loading_cutoffs$low))
  expect_true(all(res99$loading_cutoffs$high >= res95$loading_cutoffs$high))
})

test_that("RMdimCFACutoff auto-picks PCM on polytomous data", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  data("pcmdat2", package = "eRm")

  res <- RMdimCFACutoff(pcmdat2[1:100, ], iterations = 10,
                        parallel = FALSE, seed = 2)
  expect_true(res$is_polytomous)
  expect_true(res$actual_iterations >= 1L)
})

test_that("Same seed produces identical simulated distributions", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  data("raschdat1", package = "eRm")

  res1 <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 5,
                         parallel = FALSE, seed = 42)
  res2 <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 5,
                         parallel = FALSE, seed = 42)
  expect_equal(res1$simulated, res2$simulated)
  expect_equal(res1$simulated_loadings, res2$simulated_loadings)
})

# ---------------------------------------------------------------------
# RMdimCFA (tables)
# ---------------------------------------------------------------------
test_that("RMdimCFA requires a cutoff object", {
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  data("raschdat1", package = "eRm")
  expect_error(RMdimCFA(raschdat1[, 1:8]), regexp = "cutoff")
  expect_error(RMdimCFA(raschdat1[, 1:8], cutoff = list(foo = 1)),
               regexp = "RMdimCFACutoff")
})

test_that("RMdimCFA returns a list of two kables by default", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  skip_if_not_installed("knitr")
  data("raschdat1", package = "eRm")

  sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 20,
                        parallel = FALSE, seed = 1)
  tabs <- RMdimCFA(raschdat1[, 1:8], cutoff = sim)

  expect_type(tabs, "list")
  expect_named(tabs, c("fit", "loadings"))
  expect_s3_class(tabs$fit, "knitr_kable")
  expect_s3_class(tabs$loadings, "knitr_kable")
})

test_that("RMdimCFA output = 'dataframe' returns the two tables as data.frames", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  data("raschdat1", package = "eRm")

  sim  <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 20,
                         parallel = FALSE, seed = 1)
  tabs <- RMdimCFA(raschdat1[, 1:8], cutoff = sim, output = "dataframe")

  expect_named(tabs, c("fit", "loadings"))
  expect_s3_class(tabs$fit, "data.frame")
  expect_named(tabs$fit, c("Index", "Observed", "Cutoff", "Direction",
                           "Flagged"))
  expect_equal(tabs$fit$Index, c("CFI", "RMSEA", "SRMR"))

  expect_s3_class(tabs$loadings, "data.frame")
  expect_named(tabs$loadings, c("Item", "Observed", "Expected_low",
                                "Expected_high", "Flagged"))
  expect_equal(nrow(tabs$loadings), 8L)
  expect_true(all(tabs$loadings$Flagged %in% c("below", "above", "")))
})

test_that("RMdimCFA p_value adds p/padj and redefines Flagged at alpha", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  data("raschdat1", package = "eRm")

  sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 20,
                        parallel = FALSE, seed = 1)
  expect_warning(
    tabs <- RMdimCFA(raschdat1[, 1:8], cutoff = sim, p_value = TRUE,
                     output = "dataframe"),
    regexp = "only 20 simulation iterations"
  )

  # p is rounded to 4 dp in the table, so compare against the rounded floor
  B <- sim$actual_iterations
  p_floor <- round(1 / (B + 1), 4)
  expect_named(tabs$fit, c("Index", "Observed", "Cutoff", "Direction",
                           "p", "padj", "Flagged"))
  expect_true(all(tabs$fit$p >= p_floor & tabs$fit$p <= 1, na.rm = TRUE))
  expect_true(all(tabs$fit$padj >= tabs$fit$p, na.rm = TRUE))

  expect_named(tabs$loadings, c("Item", "Observed", "Expected_low",
                                "Expected_high", "p_loading",
                                "padj_loading", "Flagged"))
  expect_true(all(tabs$loadings$p_loading >= p_floor &
                    tabs$loadings$p_loading <= 1, na.rm = TRUE))
  expect_true(all(tabs$loadings$Flagged %in% c("below", "above", "")))

  # kable output renders with the extra columns and correction label
  suppressWarnings(
    kbls <- RMdimCFA(raschdat1[, 1:8], cutoff = sim, p_value = TRUE)
  )
  expect_s3_class(kbls$fit, "knitr_kable")
  expect_s3_class(kbls$loadings, "knitr_kable")
  expect_match(paste(kbls$fit, collapse = "\n"), "Westfall-Young")
})

test_that("RMdimCFA p_value correction variants run and 'none' is marginal", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  data("raschdat1", package = "eRm")

  sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 20,
                        parallel = FALSE, seed = 1)
  suppressWarnings({
    none <- RMdimCFA(raschdat1[, 1:8], cutoff = sim, p_value = TRUE,
                     correction = "none", output = "dataframe")
    bh <- RMdimCFA(raschdat1[, 1:8], cutoff = sim, p_value = TRUE,
                   correction = "fdr_bh", output = "dataframe")
  })
  expect_equal(none$fit$p, none$fit$padj)
  expect_true(all(bh$loadings$padj_loading >= bh$loadings$p_loading,
                  na.rm = TRUE))
})

# ---------------------------------------------------------------------
# RMdimCFAPlot (two-plot list)
# ---------------------------------------------------------------------
test_that("RMdimCFAPlot returns a list of two ggplots", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("ggdist")
  data("raschdat1", package = "eRm")

  sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 20,
                        parallel = FALSE, seed = 1)
  p <- RMdimCFAPlot(sim, data = raschdat1[, 1:8])

  expect_type(p, "list")
  expect_named(p, c("loadings", "fit"))
  expect_s3_class(p$loadings, "ggplot")
  expect_s3_class(p$fit, "ggplot")
})

test_that("RMdimCFAPlot requires data", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("ggdist")
  data("raschdat1", package = "eRm")

  sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 5,
                        parallel = FALSE, seed = 1)
  expect_error(RMdimCFAPlot(sim), regexp = "data")
})

test_that("RMdimCFAPlot percentile override recomputes without re-simulating", {
  skip_on_cran()
  skip_if_not_installed("lavaan")
  skip_if_not_installed("eRm")
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("ggdist")
  data("raschdat1", package = "eRm")

  sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 30,
                        parallel = FALSE, seed = 1)
  expect_s3_class(RMdimCFAPlot(sim, data = raschdat1[, 1:8],
                               percentile = 95)$fit, "ggplot")
  expect_error(RMdimCFAPlot(sim, data = raschdat1[, 1:8], percentile = 50),
               regexp = "percentile")
})

test_that("RMdimCFAPlot rejects non-RMdimCFACutoff input", {
  skip_if_not_installed("ggplot2")
  skip_if_not_installed("ggdist")
  expect_error(RMdimCFAPlot(list(foo = 1), data = data.frame(a = 1)),
               regexp = "must be the result returned by RMdimCFACutoff")
})

#' Item Parameters for a Rasch / Partial Credit Model
#'
#' Estimates item difficulty (dichotomous) or item-category threshold
#' (polytomous) parameters and returns them in long or wide format, with
#' optional standard errors and Wald confidence intervals. Item
#' parameters are estimated by conditional maximum likelihood (CML, via
#' \pkg{psychotools}) by default, with marginal maximum likelihood (MML, via
#' \pkg{mirt}) available for sparse data where CML can be unstable.
#'
#' @param data A data.frame or matrix of item responses. Items must be
#'   scored starting at 0 (non-negative integers). Missing values (`NA`)
#'   are allowed; both estimators handle them.
#' @param estimator Character. `"CML"` (default) estimates item
#'   parameters with [psychotools::pcmodel()] (a dichotomous item is a
#'   2-category PCM); `"MML"` uses [mirt::mirt()] with
#'   `itemtype = "Rasch"`. CML is preferred for
#'   Rasch measurement; MML can be more robust when data are sparse.
#'   When `estimator = "CML"` and sparse response categories are
#'   detected, a warning suggests switching to MML.
#' @param format Character. `"long"` (default) returns one row per item
#'   (dichotomous) or per item-threshold (polytomous); `"wide"` returns
#'   one row per item with threshold columns `t1`, `t2`, ... plus a mean
#'   `location` column.
#' @param se Logical. If `TRUE` (default), standard-error and
#'   confidence-interval columns are added.
#' @param ci_level Numeric in (0, 1). Confidence level for the Wald
#'   interval (`estimate +/- z * SE`). Default `0.95`.
#' @param center Logical. If `TRUE` (default), all thresholds are shifted
#'   so their grand mean is zero, the usual Rasch identification. CML
#'   estimates are already centred; the shift mainly affects the MML
#'   path, keeping the two estimators on a common scale.
#' @param output Character. `"kable"` (default) for a formatted
#'   [knitr::kable()] table, `"dataframe"` for the underlying
#'   data.frame, or `"file"` to write that data.frame to a CSV at
#'   `filename` (the data.frame is also returned invisibly).
#' @param filename Character. Path to the CSV file to write when
#'   `output = "file"`. Required in that case; ignored otherwise.
#'
#' @return
#' For `output = "dataframe"`, a data.frame. In **long** format the
#' columns are `item`, `threshold` (integer; `1` for dichotomous items),
#' `location`, and -- when `se = TRUE` -- `se`, `ci_lower`, `ci_upper`.
#' In **wide** format the columns are `item`, the threshold locations
#' (`t1`, `t2`, ... or `location` for dichotomous items), and a mean
#' `location`; when `se = TRUE`, matching `se_t1`, `se_t2`, ... columns
#' are appended. For `output = "kable"`, the same content as a
#' `knitr_kable` object.
#'
#' @details
#' Items are detected as dichotomous (maximum score 1) or polytomous
#' (maximum score > 1), and the Rasch or Partial Credit model is chosen
#' accordingly. Thresholds are reported as Andrich thresholds (the
#' person locations at which adjacent response categories are equally
#' probable) on the logit difficulty scale, matching [RMtargeting()].
#'
#' \strong{Standard errors.} For the CML path, threshold SEs are the
#' square roots of the diagonal of the threshold-parameter covariance from
#' `psychotools::threshpar(vcov = TRUE)`. For the MML path, SEs come from
#' the \pkg{mirt} parameter covariance, propagated by the delta method
#' through the linear threshold map.
#' Confidence intervals are Wald intervals and are symmetric on the
#' logit scale.
#'
#' @references
#' Andrich, D. (1978). A rating formulation for ordered response
#' categories. *Psychometrika, 43*(4), 561-573.
#' \doi{10.1007/BF02293814}
#'
#' Mair, P., & Hatzinger, R. (2007). Extended Rasch modeling: The eRm
#' package for the application of IRT models in R. *Journal of
#' Statistical Software, 20*(9), 1-20. \doi{10.18637/jss.v020.i09}
#'
#' @seealso [RMpersonParameters()], [RMscoreSE()], [RMtargeting()]
#'
#' @export
#'
#' @examples
#' \donttest{
#' set.seed(1)
#' poly <- as.data.frame(
#'   matrix(sample(0:2, 250 * 5, replace = TRUE), nrow = 250, ncol = 5)
#' )
#' colnames(poly) <- paste0("Item", 1:5)
#'
#' # Default: long-format kable with SE and 95% CI
#' RMitemParameters(poly)
#'
#' # Wide format, point estimates only
#' RMitemParameters(poly, format = "wide", se = FALSE, output = "dataframe")
#'
#' # Dichotomous data
#' dich <- as.data.frame(
#'   matrix(sample(0:1, 250 * 6, replace = TRUE), nrow = 250, ncol = 6)
#' )
#' colnames(dich) <- paste0("Item", 1:6)
#' RMitemParameters(dich, output = "dataframe")
#'
#' # Write the parameter table to a CSV (also returned invisibly)
#' RMitemParameters(poly, output = "file",
#'                  filename = tempfile(fileext = ".csv"))
#' }
RMitemParameters <- function(
  data,
  estimator = c("CML", "MML"),
  format = c("long", "wide"),
  se = TRUE,
  ci_level = 0.95,
  center = TRUE,
  output = c("kable", "dataframe", "file"),
  filename = NULL
) {
  estimator <- match.arg(estimator)
  format <- match.arg(format)
  output <- match.arg(output)

  if (output == "file") {
    .validate_filename(filename)
  }

  validate_response_data(data)

  if (
    !is.numeric(ci_level) ||
      length(ci_level) != 1L ||
      ci_level <= 0 ||
      ci_level >= 1
  ) {
    stop("`ci_level` must be a single number in (0, 1).", call. = FALSE)
  }

  data <- as.data.frame(data)
  n_total <- nrow(data)
  has_na <- anyNA(data)
  data <- .drop_empty_respondents(data)
  n_used <- nrow(data)

  fit <- if (estimator == "CML") {
    .rasch_fit_cml(data, se = se)
  } else {
    .rasch_fit_mml(data, se = se)
  }

  if (center) {
    grand <- mean(unlist(fit$thr_list))
    fit$thr_list <- lapply(fit$thr_list, function(x) x - grand)
  }

  z <- stats::qnorm(1 - (1 - ci_level) / 2)

  # --- Long table -------------------------------------------------------------
  long_rows <- list()
  for (i in seq_along(fit$items)) {
    thr <- fit$thr_list[[i]]
    se_thr <- if (se) fit$se_list[[i]] else rep(NA_real_, length(thr))
    for (k in seq_along(thr)) {
      row <- data.frame(
        item = fit$items[i],
        threshold = as.integer(k),
        location = thr[k],
        stringsAsFactors = FALSE
      )
      if (se) {
        row$se <- se_thr[k]
        row$ci_lower <- thr[k] - z * se_thr[k]
        row$ci_upper <- thr[k] + z * se_thr[k]
      }
      long_rows[[length(long_rows) + 1L]] <- row
    }
  }
  long_tbl <- do.call(rbind, long_rows)
  rownames(long_tbl) <- NULL

  result <- if (format == "long") {
    long_tbl
  } else {
    .itempar_widen(fit, se)
  }

  # --- Output -----------------------------------------------------------------
  # Dataframe/file output stays unrounded (the kable branch rounds a
  # display copy), like every other output = "dataframe" in the package.
  if (output %in% c("dataframe", "file")) {
    if (output == "file") {
      return(.write_output_csv(result, filename, row.names = FALSE))
    }
    return(result)
  }

  caption <- paste0(
    "Item ",
    if (fit$is_polytomous) "thresholds" else "difficulties",
    " via ",
    estimator,
    if (fit$is_polytomous) {
      " (Andrich thresholds, logit scale)."
    } else {
      " (logit scale)."
    },
    if (se) {
      # Wide format shows only SE columns (se_t*); the Wald CI columns are
      # long-format only.
      if (format == "long") {
        paste0(" SE and ", round(ci_level * 100), "% Wald CI shown.")
      } else {
        " SE shown."
      }
    } else {
      ""
    },
    " ",
    .n_caption(
      n_used,
      n_total,
      if (has_na) "incomplete responses retained" else character()
    ),
    "."
  )
  display <- result
  num <- vapply(display, is.numeric, logical(1))
  display[num] <- lapply(display[num], round, 3)
  knitr::kable(display, format = "pipe", caption = caption, row.names = FALSE)
}

# ---------------------------------------------------------------------
# Internal: reshape long item parameters to wide
# ---------------------------------------------------------------------

#' @keywords internal
#' @noRd
.itempar_widen <- function(fit, se) {
  max_thr <- max(vapply(fit$thr_list, length, integer(1)))
  rows <- list()
  for (i in seq_along(fit$items)) {
    thr <- fit$thr_list[[i]]
    se_thr <- if (se) fit$se_list[[i]] else NULL
    row <- list(item = fit$items[i])

    if (max_thr == 1L) {
      row[["location"]] <- thr[1L]
      if (se) row[["se"]] <- se_thr[1L]
    } else {
      for (k in seq_len(max_thr)) {
        row[[paste0("t", k)]] <- if (k <= length(thr)) thr[k] else NA_real_
      }
      row[["location"]] <- mean(thr)
      if (se) {
        for (k in seq_len(max_thr)) {
          row[[paste0("se_t", k)]] <-
            if (k <= length(thr)) se_thr[k] else NA_real_
        }
      }
    }
    rows[[length(rows) + 1L]] <- as.data.frame(row, stringsAsFactors = FALSE)
  }
  out <- do.call(rbind, rows)
  rownames(out) <- NULL
  out[,, drop = FALSE]
}

# ---------------------------------------------------------------------
# Internal: CML fit + threshold extraction (eRm)
# ---------------------------------------------------------------------

#' Fit a Rasch/PCM by CML and extract thresholds (+ delta-method SEs)
#'
#' @param data Validated response data.
#' @param se Logical; compute threshold standard errors.
#' @return A normalised list: `model`, `items`, `is_polytomous`,
#'   `thr_list` (Andrich thresholds per item) and, if `se`, `se_list`.
#' @keywords internal
#' @noRd
.rasch_fit_cml <- function(data, se = TRUE) {
  data <- as.data.frame(data)
  is_poly <- max(as.matrix(data), na.rm = TRUE) > 1L

  .sparsity_warning(data, is_poly)

  # CML via psychotools (a dichotomous item is a 2-category PCM). Andrich
  # thresholds and their covariance come from threshpar(); callers re-centre
  # the thresholds, so the parameterisation here is immaterial. (Previously
  # eRm::PCM/RM + eRm threshold SEs; threshold locations match to ~5e-5, the
  # reported SEs differ slightly, psychotools vs eRm vcov.)
  fit <- tryCatch(
    psychotools::pcmodel(data),
    error = function(e) {
      stop(
        "CML estimation failed (",
        conditionMessage(e),
        "). ",
        "This often indicates sparse data; try estimator = \"MML\".",
        call. = FALSE
      )
    }
  )

  items <- colnames(data)
  tp <- psychotools::threshpar(fit, vcov = se)
  thr_list <- stats::setNames(lapply(tp, as.numeric), items)

  se_list <- NULL
  if (se) {
    se_all <- sqrt(diag(attr(tp, "vcov")))
    se_list <- split(se_all, rep.int(seq_along(thr_list), lengths(thr_list)))
    se_list <- stats::setNames(lapply(se_list, as.numeric), items)
  }

  list(
    model = if (is_poly) "PCM" else "RM",
    items = items,
    is_polytomous = is_poly,
    thr_list = thr_list,
    se_list = se_list
  )
}

# ---------------------------------------------------------------------
# Internal: MML fit + threshold extraction (mirt)
# ---------------------------------------------------------------------

#' Fit a Rasch/PCM by MML and extract thresholds (+ delta-method SEs)
#'
#' @inheritParams .rasch_fit_cml
#' @return Same normalised structure as `.rasch_fit_cml()`.
#' @keywords internal
#' @noRd
.rasch_fit_mml <- function(data, se = TRUE) {
  if (!requireNamespace("mirt", quietly = TRUE)) {
    stop(
      "Package 'mirt' is required for estimator = \"MML\". ",
      "Install it with: install.packages(\"mirt\")",
      call. = FALSE
    )
  }
  data <- as.data.frame(data)
  is_poly <- max(as.matrix(data), na.rm = TRUE) > 1L
  items <- colnames(data)

  fit <- suppressMessages(
    mirt::mirt(
      data,
      model = 1,
      itemtype = "Rasch",
      SE = se,
      verbose = FALSE,
      accelerate = "squarem"
    )
  )

  cf <- mirt::coef(fit, simplify = FALSE)
  V <- if (se) {
    tryCatch(mirt::extract.mirt(fit, "vcov"), error = function(e) NULL)
  } else {
    NULL
  }

  # mirt parameterises P(X = k) proportional to exp(k * theta + d_k) with
  # d_0 = 0, so the Andrich thresholds are tau_k = d_{k-1} - d_k. For a
  # dichotomous item the single intercept "d" gives difficulty -d.
  thr_list <- vector("list", length(items))
  names(thr_list) <- items
  steps <- integer(length(items))
  for (i in seq_along(items)) {
    pars <- cf[[items[i]]]
    d_cols <- grep("^d([0-9]+)?$", colnames(pars), value = TRUE)
    d <- as.numeric(pars[1L, d_cols]) # poly: d0, d1, ...; dich: d
    tau <- if (length(d_cols) == 1L) -d else -diff(d)
    thr_list[[i]] <- tau
    steps[i] <- length(tau)
  }

  se_list <- NULL
  if (se) {
    se_list <- .mml_threshold_se(V, steps)
  }

  list(
    model = if (is_poly) "PCM" else "RM",
    items = items,
    is_polytomous = is_poly,
    thr_list = thr_list,
    se_list = se_list
  )
}

#' Delta-method threshold SEs from a mirt parameter covariance
#'
#' The free intercept parameters appear in the covariance in item order
#' (`d1.<idx>, d2.<idx>, ...`; d_0 is fixed at 0), so they are sliced
#' into per-item blocks of `steps[i]` parameters and propagated through
#' the linear threshold map `tau_k = d_{k-1} - d_k`.
#'
#' @param V Parameter covariance from `vcov(fit)`, or `NULL`.
#' @param steps Integer vector of threshold counts per item.
#' @return List of threshold-SE vectors, one per item (all `NA` if the
#'   covariance is unavailable or does not line up).
#' @keywords internal
#' @noRd
.mml_threshold_se <- function(V, steps) {
  na_out <- lapply(steps, function(m) rep(NA_real_, m))
  if (is.null(V)) {
    return(na_out)
  }

  d_rows <- grep("^d[0-9]*\\.", rownames(V))
  if (length(d_rows) != sum(steps)) {
    return(na_out)
  }

  out <- vector("list", length(steps))
  off <- 0L
  for (i in seq_along(steps)) {
    m <- steps[i]
    idx <- d_rows[(off + 1L):(off + m)]
    v_d <- V[idx, idx, drop = FALSE]

    # tau_k = d_{k-1} - d_k (d_0 fixed): linear map J on free d_1..d_m.
    J <- matrix(0, m, m)
    for (k in seq_len(m)) {
      if (k == 1L) {
        J[1L, 1L] <- -1
      } else {
        J[k, k - 1L] <- 1
        J[k, k] <- -1
      }
    }
    out[[i]] <- sqrt(pmax(diag(J %*% v_d %*% t(J)), 0))
    off <- off + m
  }
  out
}

# ---------------------------------------------------------------------
# Internal: sparsity warning for the CML path
# ---------------------------------------------------------------------

#' Warn about response-category sparsity that can destabilise CML
#'
#' @param data Response data.frame.
#' @param is_poly Logical; polytomous data.
#' @return Invisibly `NULL`; emits a warning when sparse categories or
#'   zero-variance items are found.
#' @keywords internal
#' @noRd
.sparsity_warning <- function(data, is_poly) {
  flagged <- character(0)
  for (nm in colnames(data)) {
    x <- data[[nm]]
    x <- x[!is.na(x)]
    if (length(unique(x)) <= 1L) {
      flagged <- c(flagged, nm)
      next
    }
    if (is_poly) {
      counts <- tabulate(x + 1L, nbins = max(x) + 1L)
      if (any(counts == 0L)) flagged <- c(flagged, nm)
    }
  }
  if (length(flagged) > 0L) {
    warning(
      "Sparse or zero-variance response categories detected in ",
      "item(s): ",
      paste(unique(flagged), collapse = ", "),
      ". ",
      "CML estimates may be unstable; consider estimator = \"MML\".",
      call. = FALSE
    )
  }
  invisible(NULL)
}

---
title: "A Rasch analysis workflow with easyRasch2"
output: rmarkdown::html_vignette
bibliography: references.bib
link-citations: yes
vignette: >
  %\VignetteIndexEntry{A Rasch analysis workflow with easyRasch2}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{=html}
<style>
/* Journal-style figure captions: subtle left rule + muted colour */
.figure p.caption,
div.figure > p.caption,
figure > figcaption {
  font-size: 0.95em;
  padding-left: 0.75em;
  border-left: 3px solid #ccd;
  color: #555;
  margin-top: 0.5em;
  margin-bottom: 1.25em;
  line-height: 1.4;
}
</style>
```

## Overview

This vignette walks through the steps of a Rasch analysis using data from the
nine-item Patient Health Questionnaire (PHQ-9; @kroenke_phq9_2001). The example follows the four psychometric criteria proposed by @christensen_psychometric_2021 for the validation of patient-reported outcome measures (PROMs):

1. **Unidimensionality** --- the items measure a single latent
   construct.
2. **Local independence** --- after conditioning on the latent trait,
   item responses are independent of each other.
3. **Ordered response category thresholds** (monotonicity) --- moving
   up the latent trait increases the probability of higher categories.
4. **Invariance / no DIF** --- item parameters are the same across
   relevant external groups (e.g. gender).

We then move on to complementary descriptors that are commonly reported alongside the four criteria above:

- Targeting --- how well person and item locations overlap on the latent continuum.
- Reliability --- how precisely the scale separates respondents.
- Person fit --- unexpected response patterns.
- Item and person parameters --- estimates for reuse in scoring.

For a more extensive treatment of Rasch analysis in R, see
<https://pgmj.github.io/raschrvignette/RaschRvign.html>. For a Bayesian sibling package, see <https://pgmj.github.io/easyRaschBayes/>.

> **NOTE:** all simulation-based functions use a low number of iterations to make this vignette render faster. You should use more iterations for actual analysis work. 400 is the calibrated floor for the multiplicity-corrected *p*-values and is the package default, while 1000 to 2000 is advisable for a final analysis [@johansson_simulationbased_2026]. An earlier recommendation, that fewer iterations could improve detection for conditional infit at small samples, is withdrawn: that advantage came from an expected range that had not converged, and it was paid for with an inflated family-wise error rate. A false discovery rate correction (`correction = "fdr_bh"` or `"fdr_by"`) needs far more iterations than the default family-wise one, because a bootstrap *p*-value can be no smaller than $1/(B+1)$.

To get faster simulations, please make use of `options(mc.cores = 4)`, where `4` should be replaced with the number of high performance CPU cores your computer has available. This setting is automatically applied by all functions that can use parallel processing.

## Data

The bundled `phq9` dataset is a 600-respondent random subsample of the
PHQ-9 module from the U.S. National Health and Nutrition Examination
Survey (NHANES, September 2024 release) with complete responses on all
nine items. NHANES microdata are released to the public domain by the
U.S. federal government.


``` r
library(easyRasch2)
data(phq9)
items <- phq9[, 1:9]   # 9 item columns, scored 0..3

# add item information
item_desc <- c(
  "Little interest or pleasure in doing things",
  "Feeling down, depressed, or hopeless",
  "Trouble falling or staying asleep, or sleeping too much",
  "Feeling tired or having little energy",
  "Poor appetite or overeating",
  "Feeling bad about yourself - or that you are a failure or have let yourself or your family down",
  "Trouble concentrating on things, such as reading the newspaper or watching television",
  "Moving or speaking so slowly that other people could have noticed",
  "Thoughts that you would be better off dead or of hurting yourself in some way"
)

item_resp <- c("Not at all","Several days","More than \nhalf the days","Nearly every day")
```


``` r
str(items)
#> 'data.frame':	600 obs. of  9 variables:
#>  $ q1: int  3 0 1 2 3 3 1 3 2 1 ...
#>  $ q2: int  3 0 2 3 3 3 1 3 2 0 ...
#>  $ q3: int  3 1 3 0 3 1 0 3 2 0 ...
#>  $ q4: int  3 1 3 2 3 3 1 3 2 0 ...
#>  $ q5: int  3 0 3 2 3 2 0 1 2 0 ...
#>  $ q6: int  3 2 3 2 3 2 2 3 3 0 ...
#>  $ q7: int  3 3 3 2 3 2 2 3 3 0 ...
#>  $ q8: int  1 0 2 0 3 3 0 0 1 0 ...
#>  $ q9: int  3 0 0 2 3 1 2 0 0 0 ...
summary(rowSums(items))
#>    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#>    0.00   10.00   16.00   15.41   21.00   27.00
table(phq9$gender, useNA = "ifany")
#> 
#> Female   Male   <NA> 
#>    426    143     31
summary(phq9$age)
#>    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#>   15.00   26.00   33.00   36.06   44.00   85.00
```

### Descriptive plots

Before fitting any model it is worth eyeballing the response
distributions:


``` r
hist(rowSums(items), col = "lightblue", main = "", xlab = "Ordinal sum scores")
```

<div class="figure">
<img src="figures/rasch-hist-plot-1.png" alt="**Figure 1.** *Histogram of ordinal sum scores*" width="100%" />
<p class="caption">**Figure 1.** *Histogram of ordinal sum scores*</p>
</div>


``` r
RMplotBar(items, ncol = 2)
```

<div class="figure">
<img src="figures/rasch-bar-plot-1.png" alt="**Figure 2.** *Faceted bar chart of response distributions*" width="100%" />
<p class="caption">**Figure 2.** *Faceted bar chart of response distributions*</p>
</div>


``` r
RMplotStackedbar(items, show_percent = TRUE)
```

<div class="figure">
<img src="figures/rasch-stackedbar-plot-1.png" alt="**Figure 3.** *Stacked-bar response distribution*" width="100%" />
<p class="caption">**Figure 3.** *Stacked-bar response distribution*</p>
</div>


## 1. Unidimensionality

`easyRasch2` provides several complementary unidimensionality
diagnostics that can be combined for a robust conclusion:

- item-level conditional infit MSQ statistics [@muller_item_2020]
- item-level item-restscore associations with Goodman-Kruskal's $\gamma$ (gamma) [@kreiner_note_2011]
- confirmatory factor analysis (CFA) with WLSMV estimator for ordinal data
- principal components analysis (PCA) of the standardized residuals [@chou_checking_2010]
- Martin-Löf test with Monte-Carlo *p*-values [@christensenMonteCarloApproach2007]

### Conditional infit MSQ

It is important to note that the `RMitemInfit()` function uses **conditional**
infit, which is both robust to different sample sizes and makes ZSTD unnecessary
[@muller_item_2020]. Müller also questions the usefulness of outfit, and my
simulation studies [@johansson_detecting_2025;johansson_simulationbased_2026]
reached the same conclusion. Thus, outfit is not reported.

Conditional item infit mean-square statistics flag items whose response
patterns deviate from the Rasch expectation. With `RMitemInfitCutoff()`, per-item
highest-density intervals serve as the reference instead of rule-of-thumb cutoffs
[@johansson_detecting_2025]. Passing the `RMitemInfitCutoff()` object to RMitemInfit() flags items on a multiplicity-corrected bootstrap *p*-value, with family-wise error rate (FWER; the default) or false discovery rate (FDR) correction, and the interval is reported alongside as a description of where a fitting item's statistic is expected to fall [@johansson_simulationbased_2026].

> **NOTE:** `RMitemInfit()`, `RMlocdepQ3()` and `RMlocdepGamma()` use the corrected *p*-value whenever the full cutoff object is supplied. The other functions with simulation-based cutoffs take `p_value = TRUE` as an opt-in, and `RMitemInfitMI()` has no *p*-value path, so it always flags against the interval.


``` r
infit_cut <- RMitemInfitCutoff(items, iterations = 100, parallel = FALSE,
                           seed = 3)
RMitemInfit(items, cutoff = infit_cut)
```



Table: MSQ values based on conditional estimation. n = 600 respondents. Two-sided bootstrap p-values from 100 iterations, multiplicity correction: Westfall-Young step-down (FWER). Flagged on the corrected p-value at alpha = 0.05. p-values cannot be smaller than 1/(100+1) = 0.0099. This is below the calibrated floor of 400, where the correction is mildly liberal and the family-wise error rate sits above the nominal level (Johansson, 2026). Flagged: underfit (infit > 1) / overfit (infit < 1).

|Item | Infit MSQ| Infit low| Infit high|      p| p (adj)|Flagged  | Relative location|
|:----|---------:|---------:|----------:|------:|-------:|:--------|-----------------:|
|q1   |     0.946|     0.911|      1.097| 0.2871|  0.4554|         |             -0.56|
|q2   |     0.778|     0.924|      1.107| 0.0099|  0.0099|overfit  |             -0.76|
|q3   |     1.234|     0.897|      1.107| 0.0099|  0.0198|underfit |             -0.83|
|q4   |     0.835|     0.893|      1.098| 0.0198|  0.0396|overfit  |             -1.48|
|q5   |     1.069|     0.917|      1.102| 0.1881|  0.4356|         |             -0.58|
|q6   |     0.895|     0.926|      1.097| 0.0297|  0.1386|         |             -0.76|
|q7   |     0.986|     0.919|      1.101| 0.7624|  0.7624|         |             -0.66|
|q8   |     1.260|     0.880|      1.119| 0.0099|  0.0099|underfit |              0.97|
|q9   |     1.315|     0.842|      1.107| 0.0099|  0.0099|underfit |              0.79|



You can also get a plot summarizing simulated and observed item infit, using `RMitemInfitPlot()`. Since conditional infit needs complete data, sibling functions that combine multiple imputation with conditional infit are useful when you have partial missingness: `RMitemInfitMI()` and `RMitemInfitCutoffMI()`.

Based on the table, items 3, 8, and 9 underfit the Rasch model, while items 2 and 4 are overfit. Item 2 is "Feeling down, depressed, or hopeless", which is a very general item in terms of measuring depression, so this is expected.

A low item fit value, often referred to as an item being "overfit" to the Rasch model, indicates that responses may be too predictable. This is often the case for items that are very general/broad in scope in relation to the latent
variable. You will often find overfitting items to also have local dependence (residual correlations) issues with other items. Overfit may be likened to
having a much stronger factor loading than other items in a confirmatory factor
analysis or a higher level of discrimination in an Item Response Theory model
with two or more parameters.

A high item fit value, often referred to as being "underfit" to the Rasch model,
can indicate several things. Often underfit is due to multidimensionality or a question that is difficult to interpret and thus has noisy response data. The latter could for instance be caused by a question that asks about two things at the same time, or is ambiguous for other reasons.

### Item-restscore

Item-restscore uses Goodman-Kruskal's $\gamma$ (gamma) and shows the expected
and observed correlation between an item and a score based on the rest of the
items [@kreiner_note_2011]. Similarly, but inverted, to item infit, a lower
observed correlation value than expected indicates underfit, that the item may
not belong to the dimension. A higher than expected observed value indicates an
overfitting and possibly redundant item. Overfitting items will often also show
issues with local dependency.

Compared to infit, item-restscore more often flags overfit items (based on
experience), and, under some conditions, less often flags underfit items (based on a simulation study
[@johansson_detecting_2025]).


``` r
RMitemRestscore(items)
```




Table: Item-restscore associations. n = 600 respondents. Flagged (adj. p < .05): overfit = observed above expected (over-discrimination, often local dependence); underfit = below (under-discrimination, often multidimensionality/noise).

|Item | Observed| Expected| Difference| Adj. p-value (BH)|Flagged  | Rel. location|
|:----|--------:|--------:|----------:|-----------------:|:--------|-------------:|
|q1   |     0.66|     0.62|      0.041|             0.210|         |         -0.56|
|q2   |     0.72|     0.62|      0.103|             0.000|overfit  |         -0.76|
|q3   |     0.57|     0.63|     -0.059|             0.085|         |         -0.83|
|q4   |     0.71|     0.62|      0.097|             0.000|overfit  |         -1.48|
|q5   |     0.62|     0.62|     -0.001|             0.968|         |         -0.58|
|q6   |     0.69|     0.63|      0.065|             0.021|overfit  |         -0.76|
|q7   |     0.64|     0.62|      0.020|             0.476|         |         -0.66|
|q8   |     0.55|     0.63|     -0.083|             0.021|underfit |          0.97|
|q9   |     0.59|     0.64|     -0.046|             0.151|         |          0.79|



Similarly to infit, item-restscore found items 2 and 4 to be overfit and 8 to be underfit. It also found item 6 to be overfit. The two methods are best used together.

### CFA-based cutoffs for CFI / RMSEA and item loadings

`RMdimCFACutoff()` simulates data from a unidimensional PCM and fits a
unidimensional ordinal CFA to each simulated dataset, building a
parametric-bootstrap reference distribution for the model fit indices (SRMR,
CFI, RMSEA) *and* for the per-item standardized factor loadings. It returns
this simulation object; `RMdimCFA()` then fits the CFA to the observed data and
tabulates the observed fit indices and loadings against the simulated reference.
Observed values beyond the simulated cutoffs are implausible under a unidimensional data-generating process.


``` r
cfa_cut <- RMdimCFACutoff(items, iterations = 100, parallel = FALSE,
                       seed = 2)
cfa_tbl <- RMdimCFA(items, cutoff = cfa_cut)
cfa_tbl$fit
```



Table: Partial Credit Model posterior-predictive CFA fit-index check. Observed CFA fit (one-factor, lavaan WLSMV, ordered = TRUE) vs simulated null under PCM unidimensionality (100 iterations at n = 600 simulees). n = 600 respondents. Cutoffs one-sided at the 99th percentile; flagged when the observed value lies in the worst 1% of the null in the unfavourable direction.

|Index | Observed| Cutoff|Direction  |Flagged |
|:-----|--------:|------:|:----------|:-------|
|CFI   |   0.9623| 0.9957|< 1st pct  |TRUE    |
|RMSEA |   0.1217| 0.0351|> 99th pct |TRUE    |
|SRMR  |   0.0572| 0.0257|> 99th pct |TRUE    |



``` r
cfa_tbl$loadings
```



Table: Standardized factor loadings (one-factor, lavaan WLSMV) vs the simulated expected range under PCM unidimensionality (100 iterations at n = 600 simulees). n = 600 respondents. Expected range is the two-sided central 99% interval of the simulated loadings; Flagged = below / above that range.

|Item | Observed| Expected low| Expected high|Flagged |
|:----|--------:|------------:|-------------:|:-------|
|q1   |    0.790|        0.682|         0.786|above   |
|q2   |    0.876|        0.689|         0.809|above   |
|q3   |    0.693|        0.707|         0.814|below   |
|q4   |    0.823|        0.664|         0.824|        |
|q5   |    0.737|        0.699|         0.812|        |
|q6   |    0.797|        0.702|         0.821|        |
|q7   |    0.753|        0.704|         0.805|        |
|q8   |    0.670|        0.692|         0.816|below   |
|q9   |    0.718|        0.709|         0.816|        |



`RMdimCFAPlot(cfa_cut, data = items)` returns a list of two figures: `$loadings`
(observed loadings against their simulated expected ranges) and `$fit` (the
simulated fit-index distributions with the observed values overlaid).

The table above agrees mostly with earlier findings. The overall model fit is clearly worse than expected under unidimensionality and item 2 is overfit (higher standardized factor loading than expected), with items 3 and 8 underfit (lower loadings). Additionally, the CFA flags item 1 as overfit. We should however take the simulation-based results with a grain of salt since we are using too few iterations to get reliable results.

### Residual PCA

After fitting the Rasch model, the residuals should contain no further
systematic structure. The *largest eigenvalue* of the residual correlation
matrix can be considered the headline diagnostic; a value clearly above the
simulation-based cutoff suggests a secondary dimension. However, an eigenvalue below the cutoff does not by itself support unidimensionality.


``` r
pca_cut <- RMdimResidualPCACutoff(items, iterations = 100, parallel = FALSE,
                       seed = 1)
RMdimResidualPCA(items, cutoff = pca_cut)
```



Table: Partial Credit Model (9 items), n = 600 respondents. Total observed variance: 51.3% explained by measures, 48.7% unexplained. First-contrast cutoff = 1.217 based on 100 simulation iterations (99th percentile).

|Component | Eigenvalue| Proportion of variance|Flagged |
|:---------|----------:|----------------------:|:-------|
|PC1       |      1.520|                  0.200|TRUE    |
|PC2       |      1.341|                  0.176|TRUE    |
|PC3       |      1.113|                  0.146|FALSE   |
|PC4       |      0.909|                  0.119|FALSE   |
|PC5       |      0.853|                  0.112|FALSE   |



Also of interest is the plot of item standardized loadings on the first residual
contrast and item locations. This figure can be helpful to identify clusters
in data, perhaps related to local dependency and/or multidimensionality.


``` r
RMdimResidualPCA(items, output = "ggplot")
```

<div class="figure">
<img src="figures/rasch-pca-plot-1.png" alt="**Figure 4.** *Standardized loadings on the first residual contrast*" width="100%" />
<p class="caption">**Figure 4.** *Standardized loadings on the first residual contrast*</p>
</div>

### Martin-Löf test

This is a likelihood-ratio test of unidimensionality against an a priori specified multidimensional alternative. The *p*-value is obtained by parametric-bootstrap (Monte Carlo) sampling under the unidimensional null. A significant *p*-value constitutes strong evidence that items do not all measure one common latent variable. However, while a non-significant *p*-value indicates that data are consistent with one common latent variable across the partition, it is not to be considered proof of unidimensionality on its own.

In the use case demonstrated below, we have hypothesized that the five psychosomatic symptoms are a separate subscale from the other items. Looking at the PCA plot above, we can see tendencies of this clustering based on the loadings on the first residual contrast factor, with items 3-5 most clearly deviating from 1, 2, 6, and 9. Using the PCA plot to determine item partitioning for the M-L test is not recommended; see `?RMdimMartinLof` for details and references.


``` r
mlof <- RMdimMartinLof(items, iterations = 100,
                       partition = list(
                         c("q1","q2","q6","q9"),
                         c("q3","q4","q5","q7","q8")
                       ),
                       seed = 4
)

mlof$p_value
#> [1] 0.00990099
mlof$wle_correlation
#>   subscale_a subscale_b         r ci_lower  ci_upper      p_value   n
#> 1          1          2 0.7174163 0.676203 0.7541533 5.994126e-96 600
```

The *p*-value rejects unidimensionality across the partition, while the subscale WLE correlation indicates the dimensions are strongly related. Note that the Monte Carlo *p*-value's resolution is limited by the number of iterations: the attainable values are $k/(B+1)$, so with $B = 100$ iterations the smallest possible *p*-value is $1/101 \approx 0.0099$ (returned as `mlof$p_value_floor`). A *p*-value *equal to* that floor means that no simulated statistic reached the observed one and should be read as $p < 0.01$ — the true *p*-value may be much smaller. Use more iterations (e.g. 1000, floor $\approx 0.001$) when reporting results. The M-L test results can be further investigated using the diagnostic function `RMdimMartinLofResiduals()`.

## 2. Local independence

Local independence (LD) can be assessed with multiple methods. Yen's $Q_3$
statistic [@yen_scaling_1984] is the correlation between
person-item standardized residuals for every item pair. Pair-wise
$Q_3$ values above the simulation-based cutoff flag LD [@christensen2017].


``` r
q3_cut <- RMlocdepQ3Cutoff(items, iterations = 100, parallel = FALSE,
                           seed = 4)
q3_results <- RMlocdepQ3(items, cutoff = q3_cut)
q3_results$matrix
```



Table: Dynamic cut-off: 0.031 (mean Q3 -0.109 + 0.14). Global simulation cutoff (99th pctl of max-mean Q3) from 100 iterations. Correlations exceeding the cut-off may indicate local dependence; see the per-pair table for detail. n = 600 respondents.

|   |q1    |q2    |q3    |q4    |q5    |q6    |q7    |q8    |q9 |above_cutoff |
|:--|:-----|:-----|:-----|:-----|:-----|:-----|:-----|:-----|:--|:------------|
|q1 |      |      |      |      |      |      |      |      |   |             |
|q2 |0.23  |      |      |      |      |      |      |      |   |*            |
|q3 |-0.19 |-0.19 |      |      |      |      |      |      |   |             |
|q4 |0     |-0.03 |0.08  |      |      |      |      |      |   |*            |
|q5 |-0.2  |-0.25 |0.09  |0     |      |      |      |      |   |*            |
|q6 |-0.18 |0.02  |-0.15 |-0.14 |-0.12 |      |      |      |   |             |
|q7 |-0.13 |-0.18 |-0.2  |-0.08 |-0.13 |-0.05 |      |      |   |             |
|q8 |-0.17 |-0.3  |-0.18 |-0.16 |-0.1  |-0.18 |0.08  |      |   |*            |
|q9 |-0.12 |0.06  |-0.23 |-0.31 |-0.24 |0.02  |-0.19 |-0.11 |   |*            |



For a more powerful $Q_3$ test, one can use the simulated cutoffs object to plot the expected range of residual correlations for each item-pair and compare with the observed value. We'll limit the output to the 6 item-pairs that deviate the most.


``` r
q3_plots <- RMlocdepQ3Plot(simfit = q3_cut, data = items, n_pairs = 6)
q3_plots$pairs
```

<div class="figure">
<img src="figures/rasch-plot_q3-1.png" alt="**Figure 5.** *Observed and expected $Q_3$ residuals*" width="100%" />
<p class="caption">**Figure 5.** *Observed and expected $Q_3$ residuals*</p>
</div>

Here we can see that the overfit item 2 does indeed have the strongest LD issues, primarily with item 1, but also with item 9. For analysis of multidimensionality, it can also be interesting to see which items have lower than expected residual correlations.

### Partial gamma LD

A second perspective on LD is the *partial gamma* coefficient
[@kreinerAnalysisLocalDependence2004;@kreiner_validity_2007] between observed
item pairs, conditional on the rest-score. Note that this function evaluates both directions of LD, thus the output is two tables. We'll restrict the output to the 6 item-pairs with largest LD deviations.


``` r
RMlocdepGamma(items, n_pairs = 6)
```

Table: Partial gamma LD analysis. n = 600 respondents. Positive gamma indicates positive local dependence between items. Showing top 6 of 36 pairs by |gamma|. Direction 1: rest score = total - Item2.

|Item 1 |Item 2 | Partial gamma| Adj. p-value (BH)|p-value sign. | Gamma pair (max)|
|:------|:------|-------------:|-----------------:|:-------------|----------------:|
|q1     |q2     |         0.531|             0.000|***           |            0.577|
|q4     |q9     |        -0.381|             0.000|***           |           -0.381|
|q2     |q9     |         0.332|             0.001|***           |            0.332|
|q2     |q8     |        -0.323|             0.001|***           |           -0.323|
|q7     |q8     |         0.303|             0.001|***           |            0.303|
|q6     |q9     |         0.287|             0.009|**            |            0.287|

Table: Partial gamma LD analysis. n = 600 respondents. Positive gamma indicates positive local dependence between items. Showing top 6 of 36 pairs by |gamma|. Direction 2: rest score = total - Item2 (item pairs shown in the reverse order to direction 1).

|Item 1 |Item 2 | Partial gamma| Adj. p-value (BH)|p-value sign. | Gamma pair (max)|
|:------|:------|-------------:|-----------------:|:-------------|----------------:|
|q2     |q1     |         0.577|             0.000|***           |            0.577|
|q9     |q4     |        -0.453|             0.000|***           |           -0.381|
|q8     |q2     |        -0.415|             0.000|***           |           -0.323|
|q4     |q3     |         0.361|             0.000|***           |            0.361|
|q5     |q3     |         0.303|             0.000|***           |            0.303|
|q9     |q2     |         0.291|             0.007|**            |            0.332|

You can also get simulation-based thresholds for partial gamma LD, using `RMlocdepGammaCutoff()`, which can be used with `RMlocdepGamma()` and also to plot the results with `RMlocdepGammaPlot()`. Supplying the cutoff object switches the table from the asymptotic *p*-values shown above to multiplicity-corrected bootstrap ones, and adds a `gamma_pair` column: each pair is tested once, on the larger of its two rest-score directions, so both tables carry the same result for a pair.

Item pairs flagged by both $Q_3$ and partial gamma are
the strongest candidates for further inspection or possible item
revision. Some argue for creating testlets by combining a locally dependent pair into a single polytomous super-item rather than eliminating one in an LD pair. I suggest looking closely at the item content before taking any action. Often, one will see very similarly worded items, where one item in an LD pair is clearly redundant. Another solution to LD can be the Graphical Loglinear Rasch Model by Kreiner and Christensen [@kreiner_validity_2007], but that is beyond the scope of this vignette.

## 3. Ordered response category thresholds

For a polytomous item to function as intended, the thresholds
separating adjacent response categories should be ordered: the
threshold from "Not at all" to "Several days" should sit below the one
from "Several days" to "More than half the days", and so on.

A classical method to assess item response functions is to plot probability of
response curves for each item and response category.


``` r
RMitemCatProb(items, category_labels = item_resp)
```

<div class="figure">
<img src="figures/rasch-ipf_plot-1.png" alt="**Figure 6.** *Item Probability Function curves*" width="100%" />
<p class="caption">**Figure 6.** *Item Probability Function curves*</p>
</div>

In the plot above, each category curve should be the most likely response at some part of the latent continuum (x axis). Item category thresholds are the points where two adjacent category lines cross each other (where they are both equally probable). In the plot above, we can see that the second highest category does not work well compared to other categories. It is disordered for item 9 and, as is perhaps clearer in the plot below, shows very small distances from the adjacent categories in most items (see T2 and T3 below, which are the lower and upper thresholds for "More than half the days").

`RMitemHierarchy()` plots each item's threshold locations on the
latent scale, ordered by overall item difficulty. Disordered
thresholds appear as overlapping or reversed segments and are a clear
signal that the response categories are not being used in the intended
order.


``` r
RMitemHierarchy(items, item_labels = item_desc)
```

<div class="figure">
<img src="figures/rasch-threshold-hierarchy-1.png" alt="**Figure 7.** *Item-hierarchy*" width="100%" />
<p class="caption">**Figure 7.** *Item-hierarchy*</p>
</div>

## 4. Invariance / no DIF

We use three complementary DIF assessments. The Andersen likelihood-ratio
test [LRT, @andersen_goodness_1973] partitions the sample by an external
variable, refits the model in each subgroup, and compares item
locations. The partial gamma approach
[@kreiner_validity_2007;@christensen_psychometric_2021] looks for an
association between item responses and the external variable
*conditional on the rest-score*, and the Rasch tree (below) searches for DIF across covariates without pre-specified groups. All are run on the *gender* variable, which has some missing values that are automatically dropped by the functions (noted in a console message and/or table/figure caption output).

First, it is important to review the response distribution when dividing the sample. If there are zero or low (below 8 or so) responses in a category, there may be issues with estimating the model parameters.


``` r
RMplotTile(items, category_labels = item_resp, group = phq9$gender)
```

<div class="figure">
<img src="figures/rasch-tile-plot-1.png" alt="**Figure 8.** *DIF grouped response distribution tile plot*" width="100%" />
<p class="caption">**Figure 8.** *DIF grouped response distribution tile plot*</p>
</div>

### Andersen LR-test


``` r
RMdifLR(items, dif_var = phq9$gender, level = "threshold", output = "ggplot")
```

<div class="figure">
<img src="figures/rasch-dif-lr-1.png" alt="**Figure 9.** *Andersen LR-test DIF locations by gender*" width="100%" />
<p class="caption">**Figure 9.** *Andersen LR-test DIF locations by gender*</p>
</div>

The plot shows the item threshold locations estimated in each gender group with
the corresponding confidence band. The global LR test is also reported in the plot caption, with a statistically significant test indicating problems with DIF between groups. However, this test is sensitive to sample size and number of items and should, like all global tests, be interpreted with care.

### Partial-gamma DIF

This plot annotates each item with its partial-$\gamma$ DIF coefficient and shows group differences within class intervals (respondents grouped by their total/latent score).


``` r
RMitemICCPlot(items, dif_var = phq9$gender, error_band = TRUE)
```

<div class="figure">
<img src="figures/rasch-dif-cicc-1.png" alt="**Figure 10.** *Partial Gamma DIF Conditional Item Characteristic Curves*" width="100%" />
<p class="caption">**Figure 10.** *Partial Gamma DIF Conditional Item Characteristic Curves*</p>
</div>

To demonstrate the FWER-adjusted *p*-values, we'll run the partial-$\gamma$ DIF analysis with parametric bootstrap (only 100 here for rendering speed, which limits the reliability of the *p*-values).


``` r
difpg <- RMdifGammaCutoff(items, dif_var = phq9$gender, iterations = 100,
                          parallel = FALSE, seed = 5)
RMdifGamma(items, cutoff = difpg, dif_var = phq9$gender, p_value = TRUE)
```



Table: Partial gamma DIF analysis. n = 569 of 600 respondents (complete cases). Two-sided bootstrap p-values from 100 iterations (replacing the asymptotic BH p-values); multiplicity correction: Westfall-Young step-down (FWER); flagged at padj < 0.05. p-values cannot be smaller than 1/(100+1) = 0.0099. Positive gamma indicates higher scores in higher DIF group levels.

|Item | Partial gamma|    SE| Lower CI| Upper CI| Gamma low| Gamma high|      p| p (adj)|Flagged |
|:----|-------------:|-----:|--------:|--------:|---------:|----------:|------:|-------:|:-------|
|q1   |         0.251| 0.104|    0.046|    0.456|    -0.218|      0.260| 0.0198|  0.0693|FALSE   |
|q2   |         0.411| 0.094|    0.227|    0.595|    -0.241|      0.205| 0.0099|  0.0099|TRUE    |
|q3   |         0.064| 0.103|   -0.138|    0.266|    -0.248|      0.252| 0.4752|  0.6238|FALSE   |
|q4   |        -0.092| 0.114|   -0.315|    0.131|    -0.177|      0.236| 0.2574|  0.5941|FALSE   |
|q5   |        -0.286| 0.092|   -0.467|   -0.104|    -0.212|      0.190| 0.0099|  0.0099|TRUE    |
|q6   |        -0.155| 0.105|   -0.361|    0.050|    -0.206|      0.180| 0.1782|  0.5545|FALSE   |
|q7   |        -0.075| 0.102|   -0.275|    0.126|    -0.219|      0.214| 0.4356|  0.6238|FALSE   |
|q8   |        -0.112| 0.102|   -0.311|    0.088|    -0.181|      0.241| 0.2574|  0.5941|FALSE   |
|q9   |         0.180| 0.100|   -0.015|    0.376|    -0.226|      0.206| 0.0495|  0.2178|FALSE   |



### Rasch tree (model-based recursive partitioning)

The DIF methods above compare groups that we specify in advance. The
Rasch tree approach [@strobl_rasch_2015] instead searches for DIF: the
model is fitted to the full sample, covariates are tested for parameter
instability, and whenever instability is detected the sample is split at
the covariate value that maximizes it. The procedure then repeats within
each subgroup, so several useful things come for free. Continuous
covariates such as age need no arbitrary pre-categorization --- the
optimal cutpoint is estimated from the data. Interactions are handled
naturally, since later splits are conditional on earlier ones (e.g. a
gender split appearing only among older respondents). And if no
instability is found, the tree simply does not split.

Statistically significant splits are not necessarily important splits, particularly in large samples. `RMdifTree()` therefore pairs
the tree with an effect-size classification for every item at every
split [@henninger_partial_2025]: the Mantel-Haenszel odds ratio on the
Delta scale developed at the Educational Testing Service (ETS) for
dichotomous data, or the partial gamma coefficient for polytomous data,
classified into the ETS categories A (negligible), B (slight to
moderate), and C (moderate to large)
[@holland_differential_1986;@zwick_review_2012]. For partial gamma the
B/C boundaries are the familiar 0.21 / 0.31 used by `RMdifGamma()`, so
the two functions read on the same scale. Note that these boundaries
are conventions carried over from large-scale educational testing, not
values calibrated to your sample and items --- in contrast to the
simulation-based cutoffs used elsewhere in this package --- so the
A/B/C labels are best read as a rough magnitude guide rather than a
calibrated test.


``` r
RMdifTree(items, covariates = phq9$gender)
```



**Node 1 -- gender: Female vs Male  (n left = 426, n right = 143)**

| Item | EffectSize | SE | Class | Flagged |
|:-----|-----------:|---:|:------|:--------|
|**q1** |**0.251**  |**0.104** |**B** |**yes** |
|**q2** |**0.411**  |**0.094** |**C** |**yes** |
|q3     |0.064      |0.103     |A     |no      |
|q4     |-0.092     |0.114     |A     |no      |
|**q5** |**-0.286** |**0.092** |**B** |**yes** |
|q6     |-0.155     |0.105     |A     |no      |
|q7     |-0.075     |0.102     |A     |no      |
|q8     |-0.112     |0.102     |A     |no      |
|q9     |0.180      |0.100     |A     |no      |


: Partial Credit Tree (9 items). n = 569 of 600 respondents (complete DIF covariates). Effect size: partial gamma (B/C thresholds = 0.21 / 0.31). alpha = 0.05. Flagged (Class B or C): 3 / 9 (item x split combinations). **Bold** rows are flagged.




The tree splits on gender, and the effect sizes match what the partial
gamma analysis showed above: q2 is classified as C and q1 and q5 as B,
while the remaining items are negligible (A). With a single binary
covariate the tree reduces to the familiar two-group comparison --- its
advantages appear when you supply several covariates at once
(`covariates = phq9[, c("gender", "age")]`), where it tests all of them,
picks split points for continuous ones, and uncovers interactions.

Three options are worth knowing about: purification = "iterative"
recomputes effect sizes while excluding already-flagged items from the
matching score; stability = TRUE refits the tree on resamples to
report how often each covariate is selected and where the cutpoints
land --- a guard against overinterpreting a single sample's tree
structure; and output = "plot" draws the tree with per-node item
parameters.

## Next step

Since we found issues with item misfit, local dependence, and gender DIF, these need to be addressed before the scale is used for measurement. Notably, item 2 recurs across all three criteria — overfit, the strongest local dependence (with item 1), and the largest gender DIF — making it the natural first candidate for closer inspection. The recommended approach is iterative: remove a single item (for instance, an underfit item, or one item from an LD pair after reviewing item content), then re-run the full set of analyses, since fit indications for the remaining items change with every removal. When the sample is large enough, it is good practice to set aside a random holdout subsample before the analysis, so that the final item set can be confirmed in data that played no part in the item-reduction decisions.

The following sections are primarily of interest once an acceptable item set has been established; here we continue with all nine items for demonstration purposes.

## Targeting

A targeting plot summarizes how well the item-threshold distribution matches
the distribution of person locations on the latent scale --- a Wright-map style
display.


``` r
RMtargeting(items)
```

<div class="figure">
<img src="figures/rasch-targeting-1.png" alt="**Figure 11.** *Person-item targeting*" width="100%" />
<p class="caption">**Figure 11.** *Person-item targeting*</p>
</div>

## Reliability

`RMreliability()` reports four reliability metrics: person separation
reliability (PSI); Relative Measurement Uncertainty (RMU)
estimate derived from posterior person-location uncertainty using plausible values; Cronbach's alpha; and marginal reliability. PSI, alpha and marginal can use bootstrap for confidence intervals. All reliability metrics range from 0 to 1, with higher values indicating better separation/precision.


``` r
RMreliability(items, draws = 200, rmu_iter = 20, parallel = FALSE,
              seed = 6)
```



Table: Reliability for 9 items, n = 600 respondents. PSI is the WLE-based separation reliability and excludes min/max scoring respondents.

|Metric           | Estimate| Lower (95% HDCI)| Upper (95% HDCI)|Notes                      |
|:----------------|--------:|----------------:|----------------:|:--------------------------|
|Cronbach's alpha |    0.886|               NA|               NA|no bootstrap               |
|PSI              |    0.838|               NA|               NA|no bootstrap               |
|Marginal         |    0.862|               NA|               NA|no bootstrap               |
|RMU (WLE)        |    0.881|            0.867|            0.895|200 PVs, 20 RMU iterations |



## Item and person parameters

Item (threshold) locations can be easily summarized in wide or long format, with or without SEs.


``` r
RMitemParameters(items, format = "wide")
```



Table: Item thresholds via CML (Andrich thresholds, logit scale). SE shown. n = 600 respondents.

|item |     t1|     t2|     t3| location| se_t1| se_t2| se_t3|
|:----|------:|------:|------:|--------:|-----:|-----:|-----:|
|q1   | -1.966|  0.634|  0.950|   -0.127| 0.165| 0.115| 0.124|
|q2   | -1.793|  0.300|  0.503|   -0.330| 0.168| 0.122| 0.118|
|q3   | -1.447|  0.039|  0.212|   -0.399| 0.166| 0.130| 0.116|
|q4   | -2.578| -0.506| -0.085|   -1.056| 0.250| 0.136| 0.109|
|q5   | -1.074| -0.048|  0.671|   -0.150| 0.153| 0.125| 0.115|
|q6   | -1.259| -0.051|  0.318|   -0.330| 0.161| 0.130| 0.114|
|q7   | -1.262|  0.041|  0.531|   -0.230| 0.157| 0.125| 0.115|
|q8   |  0.912|  1.572|  1.723|    1.402| 0.110| 0.151| 0.180|
|q9   |  0.749|  1.605|  1.307|    1.221| 0.110| 0.153| 0.172|



Person parameters are also sometimes referred to as thetas, latent scores, or person locations. By default, Warm's weighted likelihood estimation (WLE) is used for minimal bias. The function exports one value and SE for each individual, below reduced to only print the first 10 rows of the dataframe. Note that all table outputs in `easyRasch2` can be output as dataframes instead, if so desired. The option `output = "file"` makes it easy to export estimated latent scores for each respondent to a CSV file.


``` r
ppar <- RMpersonParameters(items, output = "dataframe")
head(ppar, 10)
#>         theta       sem sum_score n_answered extreme
#> 1   2.2760818 0.6184838        25          9   FALSE
#> 2  -0.9997531 0.4658741         7          9   FALSE
#> 3   1.1406643 0.4262412        20          9   FALSE
#> 4   0.3379335 0.3959994        15          9   FALSE
#> 5   3.6996335 1.3248428        27          9    TRUE
#> 6   1.1406643 0.4262412        20          9   FALSE
#> 7  -0.6129841 0.4288279         9          9   FALSE
#> 8   0.9701100 0.4155331        19          9   FALSE
#> 9   0.6465516 0.4019393        17          9   FALSE
#> 10 -3.2339486 0.9216600         1          9   FALSE
```

## Ordinal-to-interval transformation

For converting ordinal sum-scores to interval-scaled person-location
estimates with associated standard errors, use `RMscoreSE()`.


``` r
RMscoreSE(items, output = "ggplot")
```

<div class="figure">
<img src="figures/rasch-scoreSE-1.png" alt="**Figure 12.** *Sum-score to WLE conversion with 95% CIs*" width="100%" />
<p class="caption">**Figure 12.** *Sum-score to WLE conversion with 95% CIs*</p>
</div>

``` r
RMscoreSE(items)
```



Table: Person locations via Warm's WLE (CML item parameters). n = 600 respondents.

| Ordinal sum score| Logit score| Logit std.error|
|-----------------:|-----------:|---------------:|
|                 0|      -4.469|           1.536|
|                 1|      -3.234|           0.922|
|                 2|      -2.594|           0.737|
|                 3|      -2.138|           0.638|
|                 4|      -1.779|           0.574|
|                 5|      -1.480|           0.528|
|                 6|      -1.224|           0.493|
|                 7|      -1.000|           0.466|
|                 8|      -0.798|           0.445|
|                 9|      -0.613|           0.429|
|                10|      -0.440|           0.417|
|                11|      -0.277|           0.408|
|                12|      -0.119|           0.401|
|                13|       0.034|           0.397|
|                14|       0.186|           0.396|
|                15|       0.338|           0.396|
|                16|       0.491|           0.398|
|                17|       0.647|           0.402|
|                18|       0.806|           0.408|
|                19|       0.970|           0.416|
|                20|       1.141|           0.426|
|                21|       1.320|           0.441|
|                22|       1.512|           0.462|
|                23|       1.723|           0.492|
|                24|       1.968|           0.539|
|                25|       2.276|           0.618|
|                26|       2.724|           0.778|
|                27|       3.700|           1.325|



`RMscoreSE()` also has an option for EAP scores (expected a posteriori).

## Person fit

Conditional person infit/outfit MSQ and the $\ell_z$ statistic are implemented, with Monte-Carlo resampling for *p*-values.


``` r
pfit <- RMpersonFit(items, iterations = 100, output = "ggplot", seed = 7)
pfit$lz
```

<div class="figure">
<img src="figures/rasch-personfit-1.png" alt="**Figure 13.** *Person fit with the lz statistic*" width="100%" />
<p class="caption">**Figure 13.** *Person fit with the lz statistic*</p>
</div>


## Where to next

* Each `RM*()` function is documented with its own `?function`
  reference page including a worked example.
* The simulation-based cutoffs used above (`RM*Cutoff()`) can be
  parallelised on multiple CPU cores via the `mirai` package; see the relevant help pages.
  * For a progress bar on time-consuming simulations, add `verbose = TRUE` to the function call. This should not be used when rendering Quarto/Rmd files.


## References

---
title: "A Rasch analysis workflow with easyRasch2"
output: rmarkdown::html_vignette
bibliography: references.bib
link-citations: yes
vignette: >
  %\VignetteIndexEntry{A Rasch analysis workflow with easyRasch2}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



```{=html}
<style>
/* Journal-style figure captions: subtle left rule + muted colour */
.figure p.caption,
div.figure > p.caption,
figure > figcaption {
  font-size: 0.95em;
  padding-left: 0.75em;
  border-left: 3px solid #ccd;
  color: #555;
  margin-top: 0.5em;
  margin-bottom: 1.25em;
  line-height: 1.4;
}
</style>
```

## Overview

This vignette walks through the steps of a Rasch analysis using data from the
nine-item Patient Health Questionnaire (PHQ-9; @kroenke_phq9_2001). The example follows the four psychometric criteria proposed by @christensen_psychometric_2021 for the validation of patient-reported outcome measures (PROMs):

1. **Unidimensionality** --- the items measure a single latent
   construct.
2. **Local independence** --- after conditioning on the latent trait,
   item responses are independent of each other.
3. **Ordered response category thresholds** (monotonicity) --- moving
   up the latent trait increases the probability of higher categories.
4. **Invariance / no DIF** --- item parameters are the same across
   relevant external groups (e.g. gender).

We then move on to complementary descriptors that are commonly reported alongside the four criteria above:

- Targeting --- how well person and item locations overlap on the latent continuum.
- Reliability --- how precisely the scale separates respondents.
- Person fit --- unexpected response patterns.
- Item and person parameters --- estimates for reuse in scoring.

For a more extensive treatment of Rasch analysis in R, see
<https://pgmj.github.io/raschrvignette/RaschRvign.html>. For a Bayesian sibling package, see <https://pgmj.github.io/easyRaschBayes/>.

> **NOTE:** all simulation-based functions use a low number of iterations to make this vignette render faster. You should use more iterations for actual analysis work. For most methods, 500-1500 will be useful, except for conditional infit, where lower numbers can be optimal, depending on sample size [@johansson_detecting_2025]. If you plan to use `p_value = TRUE`, use at least 1000 iterations.

To get faster simulations, please make use of `options(mc.cores = 4)`, where `4` should be replaced with the number of high performance CPU cores your computer has available. This setting is automatically applied by all functions that can use parallel processing.

## Data

The bundled `phq9` dataset is a 600-respondent random subsample of the
PHQ-9 module from the U.S. National Health and Nutrition Examination
Survey (NHANES, September 2024 release) with complete responses on all
nine items. NHANES microdata are released to the public domain by the
U.S. federal government.


``` r
library(easyRasch2)
data(phq9)
items <- phq9[, 1:9]   # 9 item columns, scored 0..3

# add item information
item_desc <- c(
  "Little interest or pleasure in doing things",
  "Feeling down, depressed, or hopeless",
  "Trouble falling or staying asleep, or sleeping too much",
  "Feeling tired or having little energy",
  "Poor appetite or overeating",
  "Feeling bad about yourself - or that you are a failure or have let yourself or your family down",
  "Trouble concentrating on things, such as reading the newspaper or watching television",
  "Moving or speaking so slowly that other people could have noticed",
  "Thoughts that you would be better off dead or of hurting yourself in some way"
)

item_resp <- c("Not at all","Several days","More than \nhalf the days","Nearly every day")
```


``` r
str(items)
#> 'data.frame':	600 obs. of  9 variables:
#>  $ q1: int  3 0 1 2 3 3 1 3 2 1 ...
#>  $ q2: int  3 0 2 3 3 3 1 3 2 0 ...
#>  $ q3: int  3 1 3 0 3 1 0 3 2 0 ...
#>  $ q4: int  3 1 3 2 3 3 1 3 2 0 ...
#>  $ q5: int  3 0 3 2 3 2 0 1 2 0 ...
#>  $ q6: int  3 2 3 2 3 2 2 3 3 0 ...
#>  $ q7: int  3 3 3 2 3 2 2 3 3 0 ...
#>  $ q8: int  1 0 2 0 3 3 0 0 1 0 ...
#>  $ q9: int  3 0 0 2 3 1 2 0 0 0 ...
summary(rowSums(items))
#>    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#>    0.00   10.00   16.00   15.41   21.00   27.00
table(phq9$gender, useNA = "ifany")
#> 
#> Female   Male   <NA> 
#>    426    143     31
summary(phq9$age)
#>    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
#>   15.00   26.00   33.00   36.06   44.00   85.00
```

### Descriptive plots

Before fitting any model it is worth eyeballing the response
distributions:


``` r
hist(rowSums(items), col = "lightblue", main = "", xlab = "Ordinal sum scores")
```

<div class="figure">
<img src="figures/rasch-hist-plot-1.png" alt="**Figure 1.** *Histogram of ordinal sum scores*" width="100%" />
<p class="caption">**Figure 1.** *Histogram of ordinal sum scores*</p>
</div>


``` r
RMplotBar(items, ncol = 2)
```

<div class="figure">
<img src="figures/rasch-bar-plot-1.png" alt="**Figure 2.** *Faceted bar chart of response distributions*" width="100%" />
<p class="caption">**Figure 2.** *Faceted bar chart of response distributions*</p>
</div>


``` r
RMplotStackedbar(items, show_percent = TRUE)
```

<div class="figure">
<img src="figures/rasch-stackedbar-plot-1.png" alt="**Figure 3.** *Stacked-bar response distribution*" width="100%" />
<p class="caption">**Figure 3.** *Stacked-bar response distribution*</p>
</div>


## 1. Unidimensionality

`easyRasch2` provides several complementary unidimensionality
diagnostics that can be combined for a robust conclusion:

- item-level conditional infit MSQ statistics [@muller_item_2020]
- item-level item-restscore associations with Goodman-Kruskal's $\gamma$ (gamma) [@kreiner_note_2011]
- confirmatory factor analysis (CFA) with WLSMV estimator for ordinal data
- principal components analysis (PCA) of the standardized residuals [@chou_checking_2010]
- Martin-Löf test with Monte-Carlo *p*-values [@christensenMonteCarloApproach2007]

### Conditional infit MSQ

It is important to note that the `RMitemInfit()` function uses **conditional**
infit, which is both robust to different sample sizes and makes ZSTD unnecessary
[@muller_item_2020]. Müller also questions the usefulness of outfit, and my
simulation study [@johansson_detecting_2025] reached the same conclusion. Thus,
outfit is not reported.

Conditional item infit mean-square statistics flag items whose response
patterns deviate from the Rasch expectation. With `RMitemInfitCutoff()`, per-item
highest-density intervals serve as the reference instead of rule-of-thumb cutoffs
[@johansson_detecting_2025]. Bootstrap *p*-values are also available via `p_value = TRUE`, with family-wise error rate (FWER; the default) or false discovery rate (FDR) correction.

> **NOTE:** All functions that use simulation-based cutoffs (except `RMitemInfitMI()`) have an optional `p_value = TRUE` for their table outputs.


``` r
infit_cut <- RMitemInfitCutoff(items, iterations = 100, parallel = FALSE,
                           seed = 3)
RMitemInfit(items, cutoff = infit_cut)
```



Table: MSQ values based on conditional estimation. n = 600 respondents. Cutoff values based on 100 simulation iterations (99.9% HDCI). Flagged: overfit = infit below range (more predictable); underfit = above range (noisier).

|Item | Infit MSQ| Infit low| Infit high|Flagged  | Relative location|
|:----|---------:|---------:|----------:|:--------|-----------------:|
|q1   |     0.946|     0.883|      1.143|         |             -0.56|
|q2   |     0.778|     0.883|      1.107|overfit  |             -0.76|
|q3   |     1.234|     0.878|      1.194|underfit |             -0.83|
|q4   |     0.835|     0.867|      1.178|overfit  |             -1.48|
|q5   |     1.069|     0.900|      1.110|         |             -0.58|
|q6   |     0.895|     0.858|      1.219|         |             -0.76|
|q7   |     0.986|     0.899|      1.153|         |             -0.66|
|q8   |     1.260|     0.856|      1.136|underfit |              0.97|
|q9   |     1.315|     0.842|      1.199|underfit |              0.79|



You can also get a plot summarizing simulated and observed item infit, using `RMitemInfitPlot()`. Since conditional infit needs complete data, sibling functions that combine multiple imputation with conditional infit are useful when you have partial missingness: `RMitemInfitMI()` and `RMitemInfitCutoffMI()`.

Based on the table, items 3, 8, and 9 underfit the Rasch model, while items 2 and 4 are overfit. Item 2 is "Feeling down, depressed, or hopeless", which is a very general item in terms of measuring depression, so this is expected.

A low item fit value, often referred to as an item being "overfit" to the Rasch model, indicates that responses may be too predictable. This is often the case for items that are very general/broad in scope in relation to the latent
variable. You will often find overfitting items to also have local dependence (residual correlations) issues with other items. Overfit may be likened to
having a much stronger factor loading than other items in a confirmatory factor
analysis or a higher level of discrimination in an Item Response Theory model
with two or more parameters.

A high item fit value, often referred to as being "underfit" to the Rasch model,
can indicate several things. Often underfit is due to multidimensionality or a question that is difficult to interpret and thus has noisy response data. The latter could for instance be caused by a question that asks about two things at the same time, or is ambiguous for other reasons.

### Item-restscore

Item-restscore uses Goodman-Kruskal's $\gamma$ (gamma) and shows the expected
and observed correlation between an item and a score based on the rest of the
items [@kreiner_note_2011]. Similarly, but inverted, to item infit, a lower
observed correlation value than expected indicates underfit, that the item may
not belong to the dimension. A higher than expected observed value indicates an
overfitting and possibly redundant item. Overfitting items will often also show
issues with local dependency.

Compared to infit, item-restscore more often flags overfit items (based on
experience), and less often flags underfit items (based on a simulation study
[@johansson_detecting_2025]).


``` r
RMitemRestscore(items)
```




Table: Item-restscore associations. n = 600 respondents. Flagged (adj. p < .05): overfit = observed above expected (over-discrimination, often local dependence); underfit = below (under-discrimination, often multidimensionality/noise).

|Item | Observed| Expected| Difference| Adj. p-value (BH)|Flagged  | Rel. location|
|:----|--------:|--------:|----------:|-----------------:|:--------|-------------:|
|q1   |     0.66|     0.62|      0.041|             0.210|         |         -0.56|
|q2   |     0.72|     0.62|      0.103|             0.000|overfit  |         -0.76|
|q3   |     0.57|     0.63|     -0.059|             0.085|         |         -0.83|
|q4   |     0.71|     0.62|      0.097|             0.000|overfit  |         -1.48|
|q5   |     0.62|     0.62|     -0.001|             0.968|         |         -0.58|
|q6   |     0.69|     0.63|      0.065|             0.021|overfit  |         -0.76|
|q7   |     0.64|     0.62|      0.020|             0.476|         |         -0.66|
|q8   |     0.55|     0.63|     -0.083|             0.021|underfit |          0.97|
|q9   |     0.59|     0.64|     -0.046|             0.151|         |          0.79|



Similarly to infit, item-restscore found items 2 and 4 to be overfit and 8 to be underfit. It also found item 6 to be overfit. These methods are best used together.

### CFA-based cutoffs for CFI / RMSEA and item loadings

`RMdimCFACutoff()` simulates data from a unidimensional PCM and fits a
unidimensional ordinal CFA to each simulated dataset, building a
parametric-bootstrap reference distribution for the model fit indices (SRMR,
CFI, RMSEA) *and* for the per-item standardized factor loadings. It returns
this simulation object; `RMdimCFA()` then fits the CFA to the observed data and
tabulates the observed fit indices and loadings against the simulated reference.
Observed values beyond the simulated cutoffs are implausible under a unidimensional data-generating process.


``` r
cfa_cut <- RMdimCFACutoff(items, iterations = 100, parallel = FALSE,
                       seed = 2)
cfa_tbl <- RMdimCFA(items, cutoff = cfa_cut)
cfa_tbl$fit
```



Table: Partial Credit Model posterior-predictive CFA fit-index check. Observed CFA fit (one-factor, lavaan WLSMV, ordered = TRUE) vs simulated null under PCM unidimensionality (100 iterations at n = 600 simulees). n = 600 respondents. Cutoffs one-sided at the 99th percentile; flagged when the observed value lies in the worst 1% of the null in the unfavourable direction.

|Index | Observed| Cutoff|Direction  |Flagged |
|:-----|--------:|------:|:----------|:-------|
|CFI   |   0.9623| 0.9957|< 1st pct  |TRUE    |
|RMSEA |   0.1217| 0.0351|> 99th pct |TRUE    |
|SRMR  |   0.0572| 0.0257|> 99th pct |TRUE    |



``` r
cfa_tbl$loadings
```



Table: Standardized factor loadings (one-factor, lavaan WLSMV) vs the simulated expected range under PCM unidimensionality (100 iterations at n = 600 simulees). n = 600 respondents. Expected range is the two-sided central 99% interval of the simulated loadings; Flagged = below / above that range.

|Item | Observed| Expected low| Expected high|Flagged |
|:----|--------:|------------:|-------------:|:-------|
|q1   |    0.790|        0.682|         0.786|above   |
|q2   |    0.876|        0.689|         0.809|above   |
|q3   |    0.693|        0.707|         0.814|below   |
|q4   |    0.823|        0.664|         0.824|        |
|q5   |    0.737|        0.699|         0.812|        |
|q6   |    0.797|        0.702|         0.821|        |
|q7   |    0.753|        0.704|         0.805|        |
|q8   |    0.670|        0.692|         0.816|below   |
|q9   |    0.718|        0.709|         0.816|        |



`RMdimCFAPlot(cfa_cut, data = items)` returns a list of two figures: `$loadings`
(observed loadings against their simulated expected ranges) and `$fit` (the
simulated fit-index distributions with the observed values overlaid).

The table above agrees mostly with earlier findings. The overall model fit is clearly worse than expected under unidimensionality and item 2 is overfit (higher standardized factor loading than expected), with items 3 and 8 underfit (lower loadings). Additionally, the CFA flags item 1 as overfit. We should however take the simulation-based results with a grain of salt since we are using too few iterations to get reliable results.

### Residual PCA

After fitting the Rasch model, the residuals should contain no further
systematic structure. The *largest eigenvalue* of the residual correlation
matrix can be considered the headline diagnostic; a value clearly above the
simulation-based cutoff suggests a secondary dimension. However, an eigenvalue below the cutoff does not by itself support unidimensionality.


``` r
pca_cut <- RMdimResidualPCACutoff(items, iterations = 100, parallel = FALSE,
                       seed = 1)
RMdimResidualPCA(items, cutoff = pca_cut)
```



Table: Partial Credit Model (9 items), n = 600 respondents. Total observed variance: 51.3% explained by measures, 48.7% unexplained. First-contrast cutoff = 1.217 based on 100 simulation iterations (99th percentile).

|Component | Eigenvalue| Proportion of variance|Flagged |
|:---------|----------:|----------------------:|:-------|
|PC1       |      1.520|                  0.200|TRUE    |
|PC2       |      1.341|                  0.176|TRUE    |
|PC3       |      1.113|                  0.146|FALSE   |
|PC4       |      0.909|                  0.119|FALSE   |
|PC5       |      0.853|                  0.112|FALSE   |



Also of interest is the plot of item standardized loadings on the first residual
contrast and item locations. This figure can be helpful to identify clusters
in data, perhaps related to local dependency and/or multidimensionality.


``` r
RMdimResidualPCA(items, output = "ggplot")
```

<div class="figure">
<img src="figures/rasch-pca-plot-1.png" alt="**Figure 4.** *Standardized loadings on the first residual contrast*" width="100%" />
<p class="caption">**Figure 4.** *Standardized loadings on the first residual contrast*</p>
</div>

### Martin-Löf test

This is a likelihood-ratio test of unidimensionality against an a priori specified multidimensional alternative. The *p*-value is obtained by parametric-bootstrap (Monte Carlo) sampling under the unidimensional null. A significant *p*-value constitutes strong evidence that items do not all measure one common latent variable. However, while a non-significant *p*-value indicates that data are consistent with one common latent variable across the partition, it is not to be considered proof of unidimensionality on its own.

In the use case demonstrated below, we have hypothesized that the five psychosomatic symptoms are a separate subscale from the other items. Looking at the PCA plot above, we can see tendencies of this clustering based on the loadings on the first residual contrast factor, with items 3-5 most clearly deviating from 1, 2, 6, and 9. Using the PCA plot to determine item partitioning for the M-L test is not recommended; see `?RMdimMartinLof` for details and references.


``` r
mlof <- RMdimMartinLof(items, iterations = 100,
                       partition = list(
                         c("q1","q2","q6","q9"),
                         c("q3","q4","q5","q7","q8")
                       ),
                       seed = 4
)

mlof$p_value
#> [1] 0.00990099
mlof$wle_correlation
#>   subscale_a subscale_b         r ci_lower  ci_upper      p_value   n
#> 1          1          2 0.7174163 0.676203 0.7541533 5.994126e-96 600
```

The *p*-value rejects unidimensionality across the partition, while the subscale WLE correlation indicates the dimensions are strongly related. Note that the Monte Carlo *p*-value's resolution is limited by the number of iterations: the attainable values are $k/(B+1)$, so with $B = 100$ iterations the smallest possible *p*-value is $1/101 \approx 0.0099$ (returned as `mlof$p_value_floor`). A *p*-value *equal to* that floor means that no simulated statistic reached the observed one and should be read as $p < 0.01$ — the true *p*-value may be much smaller. Use more iterations (e.g. 1000, floor $\approx 0.001$) when reporting results. The M-L test results can be further investigated using the diagnostic function `RMdimMartinLofResiduals()`.

## 2. Local independence

Local independence (LD) can be assessed with multiple methods. Yen's $Q_3$
statistic [@yen_scaling_1984] is the correlation between
person-item standardized residuals for every item pair. Pair-wise
$Q_3$ values above the simulation-based cutoff flag LD [@christensen2017].


``` r
q3_cut <- RMlocdepQ3Cutoff(items, iterations = 100, parallel = FALSE,
                           seed = 4)
q3_results <- RMlocdepQ3(items, cutoff = q3_cut)
q3_results$matrix
```



Table: Dynamic cut-off: 0.031 (mean Q3 -0.109 + 0.14). Global simulation cutoff (99th pctl of max-mean Q3) from 100 iterations. Correlations exceeding the cut-off may indicate local dependence; see the per-pair table for detail. n = 600 respondents.

|   |q1    |q2    |q3    |q4    |q5    |q6    |q7    |q8    |q9 |above_cutoff |
|:--|:-----|:-----|:-----|:-----|:-----|:-----|:-----|:-----|:--|:------------|
|q1 |      |      |      |      |      |      |      |      |   |             |
|q2 |0.23  |      |      |      |      |      |      |      |   |*            |
|q3 |-0.19 |-0.19 |      |      |      |      |      |      |   |             |
|q4 |0     |-0.03 |0.08  |      |      |      |      |      |   |*            |
|q5 |-0.2  |-0.25 |0.09  |0     |      |      |      |      |   |*            |
|q6 |-0.18 |0.02  |-0.15 |-0.14 |-0.12 |      |      |      |   |             |
|q7 |-0.13 |-0.18 |-0.2  |-0.08 |-0.13 |-0.05 |      |      |   |             |
|q8 |-0.17 |-0.3  |-0.18 |-0.16 |-0.1  |-0.18 |0.08  |      |   |*            |
|q9 |-0.12 |0.06  |-0.23 |-0.31 |-0.24 |0.02  |-0.19 |-0.11 |   |*            |



For a more powerful $Q_3$ test, one can use the simulated cutoffs object to plot the expected range of residual correlations for each item-pair and compare with the observed value. We'll limit the output to the 6 item-pairs that deviate the most.


``` r
q3_plots <- RMlocdepQ3Plot(simfit = q3_cut, data = items, n_pairs = 6)
q3_plots$pairs
```

<div class="figure">
<img src="figures/rasch-plot_q3-1.png" alt="**Figure 5.** *Observed and expected $Q_3$ residuals*" width="100%" />
<p class="caption">**Figure 5.** *Observed and expected $Q_3$ residuals*</p>
</div>

Here we can see that the overfit item 2 does indeed have the strongest LD issues, primarily with item 1, but also with item 9. For analysis of multidimensionality, it can also be interesting to see which items have lower than expected residual correlations.

### Partial gamma LD

A second perspective on LD is the *partial gamma* coefficient
[@kreinerAnalysisLocalDependence2004;@kreiner_validity_2007] between observed
item pairs, conditional on the rest-score. Note that this function evaluates both directions of LD, thus the output is two tables. We'll restrict the output to the 6 item-pairs with largest LD deviations.


``` r
RMlocdepGamma(items, n_pairs = 6)
```

Table: Partial gamma LD analysis. n = 600 respondents. Positive gamma indicates positive local dependence between items. Showing top 6 of 36 pairs by |gamma|. Direction 1: rest score = total - Item2.

|Item 1 |Item 2 | Partial gamma| Adj. p-value (BH)|p-value sign. |
|:------|:------|-------------:|-----------------:|:-------------|
|q1     |q2     |         0.531|             0.000|***           |
|q4     |q9     |        -0.381|             0.000|***           |
|q2     |q9     |         0.332|             0.001|***           |
|q2     |q8     |        -0.323|             0.001|***           |
|q7     |q8     |         0.303|             0.001|***           |
|q6     |q9     |         0.287|             0.009|**            |

Table: Partial gamma LD analysis. n = 600 respondents. Positive gamma indicates positive local dependence between items. Showing top 6 of 36 pairs by |gamma|. Direction 2: rest score = total - Item2 (item pairs shown in the reverse order to direction 1).

|Item 1 |Item 2 | Partial gamma| Adj. p-value (BH)|p-value sign. |
|:------|:------|-------------:|-----------------:|:-------------|
|q2     |q1     |         0.577|             0.000|***           |
|q9     |q4     |        -0.453|             0.000|***           |
|q8     |q2     |        -0.415|             0.000|***           |
|q4     |q3     |         0.361|             0.000|***           |
|q5     |q3     |         0.303|             0.000|***           |
|q9     |q2     |         0.291|             0.007|**            |

You can also get simulation-based thresholds for partial gamma LD, using `RMlocdepGammaCutoff()`, which can be used with `RMlocdepGamma()` and also to plot the results with `RMlocdepGammaPlot()`

Item pairs flagged by both $Q_3$ and partial gamma are
the strongest candidates for further inspection or possible item
revision. Some argue for creating testlets by combining a locally dependent pair into a single polytomous super-item rather than eliminating one in an LD pair. I suggest looking closely at the item content before taking any action. Often, one will see very similarly worded items, where one item in an LD pair is clearly redundant. Another solution to LD can be the Graphical Loglinear Rasch Model by Kreiner and Christensen [@kreiner_validity_2007], but that is beyond the scope of this vignette.

## 3. Ordered response category thresholds

For a polytomous item to function as intended, the thresholds
separating adjacent response categories should be ordered: the
threshold from "Not at all" to "Several days" should sit below the one
from "Several days" to "More than half the days", and so on.

A classical method to assess item response functions is to plot probability of
response curves for each item and response category.


``` r
RMitemCatProb(items, category_labels = item_resp)
```

<div class="figure">
<img src="figures/rasch-ipf_plot-1.png" alt="**Figure 6.** *Item Probability Function curves*" width="100%" />
<p class="caption">**Figure 6.** *Item Probability Function curves*</p>
</div>

In the plot above, each category curve should be the most likely response at some part of the latent continuum (x axis). Item category thresholds are the points where two adjacent category lines cross each other (where they are both equally probable). In the plot above, we can see that the second highest category does not work well compared to other categories. It is disordered for item 9 and, as is perhaps clearer in the plot below, shows very small distances from the adjacent categories in most items (see T2 and T3 below, which are the lower and upper thresholds for "More than half the days").

`RMitemHierarchy()` plots each item's threshold locations on the
latent scale, ordered by overall item difficulty. Disordered
thresholds appear as overlapping or reversed segments and are a clear
signal that the response categories are not being used in the intended
order.


``` r
RMitemHierarchy(items, item_labels = item_desc)
```

<div class="figure">
<img src="figures/rasch-threshold-hierarchy-1.png" alt="**Figure 7.** *Item-hierarchy*" width="100%" />
<p class="caption">**Figure 7.** *Item-hierarchy*</p>
</div>

## 4. Invariance / no DIF

We use three complementary DIF assessments. The Andersen likelihood-ratio
test [LRT, @andersen_goodness_1973] partitions the sample by an external
variable, refits the model in each subgroup, and compares item
locations. The partial gamma approach
[@kreiner_validity_2007;@christensen_psychometric_2021] looks for an
association between item responses and the external variable
*conditional on the rest-score*, and the Rasch tree (below) searches for DIF across covariates without pre-specified groups. All are run on the *gender* variable, which has some missing values that are automatically dropped by the functions (noted in a console message and/or table/figure caption output).

First, it is important to review the response distribution when dividing the sample. If there are zero or low (below 8 or so) responses in a category, there may be issues with estimating the model parameters.


``` r
RMplotTile(items, category_labels = item_resp, group = phq9$gender)
```

<div class="figure">
<img src="figures/rasch-tile-plot-1.png" alt="**Figure 8.** *DIF grouped response distribution tile plot*" width="100%" />
<p class="caption">**Figure 8.** *DIF grouped response distribution tile plot*</p>
</div>

### Andersen LR-test


``` r
RMdifLR(items, dif_var = phq9$gender, level = "threshold", output = "ggplot")
```

<div class="figure">
<img src="figures/rasch-dif-lr-1.png" alt="**Figure 9.** *Andersen LR-test DIF locations by gender*" width="100%" />
<p class="caption">**Figure 9.** *Andersen LR-test DIF locations by gender*</p>
</div>

The plot shows the item threshold locations estimated in each gender group with
the corresponding confidence band. The global LR test is also reported in the plot caption, with a statistically significant test indicating problems with DIF between groups. However, this test is sensitive to sample size and number of items and should, like all global tests, be interpreted with care.

### Partial-gamma DIF

This plot annotates each item with its partial-$\gamma$ DIF coefficient and shows group differences within class intervals (respondents grouped by their total/latent score).


``` r
RMitemICCPlot(items, dif_var = phq9$gender, error_band = TRUE)
```

<div class="figure">
<img src="figures/rasch-dif-cicc-1.png" alt="**Figure 10.** *Partial Gamma DIF Conditional Item Characteristic Curves*" width="100%" />
<p class="caption">**Figure 10.** *Partial Gamma DIF Conditional Item Characteristic Curves*</p>
</div>

To demonstrate the FWER-adjusted *p*-values, we'll run the partial-$\gamma$ DIF analysis with parametric bootstrap (only 100 here for rendering speed, which limits the reliability of the *p*-values).


``` r
difpg <- RMdifGammaCutoff(items, dif_var = phq9$gender, iterations = 100,
                          parallel = FALSE, seed = 5)
RMdifGamma(items, cutoff = difpg, dif_var = phq9$gender, p_value = TRUE)
```



Table: Partial gamma DIF analysis. n = 569 of 600 respondents (complete cases). Two-sided bootstrap p-values from 100 iterations (replacing the asymptotic BH p-values); multiplicity correction: Westfall-Young step-down (FWER); flagged at padj < 0.05. p-values cannot be smaller than 1/(100+1) = 0.0099. Positive gamma indicates higher scores in higher DIF group levels.

|Item | Partial gamma|    SE| Lower CI| Upper CI| Gamma low| Gamma high|      p| p (adj)|Flagged |
|:----|-------------:|-----:|--------:|--------:|---------:|----------:|------:|-------:|:-------|
|q1   |         0.251| 0.104|    0.046|    0.456|    -0.218|      0.260| 0.0198|  0.0693|FALSE   |
|q2   |         0.411| 0.094|    0.227|    0.595|    -0.241|      0.205| 0.0099|  0.0099|TRUE    |
|q3   |         0.064| 0.103|   -0.138|    0.266|    -0.248|      0.252| 0.4752|  0.6238|FALSE   |
|q4   |        -0.092| 0.114|   -0.315|    0.131|    -0.177|      0.236| 0.2574|  0.5941|FALSE   |
|q5   |        -0.286| 0.092|   -0.467|   -0.104|    -0.212|      0.190| 0.0099|  0.0099|TRUE    |
|q6   |        -0.155| 0.105|   -0.361|    0.050|    -0.206|      0.180| 0.1782|  0.5545|FALSE   |
|q7   |        -0.075| 0.102|   -0.275|    0.126|    -0.219|      0.214| 0.4356|  0.6238|FALSE   |
|q8   |        -0.112| 0.102|   -0.311|    0.088|    -0.181|      0.241| 0.2574|  0.5941|FALSE   |
|q9   |         0.180| 0.100|   -0.015|    0.376|    -0.226|      0.206| 0.0495|  0.2178|FALSE   |



### Rasch tree (model-based recursive partitioning)

The DIF methods above compare groups that we specify in advance. The
Rasch tree approach [@strobl_rasch_2015] instead searches for DIF: the
model is fitted to the full sample, covariates are tested for parameter
instability, and whenever instability is detected the sample is split at
the covariate value that maximizes it. The procedure then repeats within
each subgroup, so several useful things come for free. Continuous
covariates such as age need no arbitrary pre-categorization --- the
optimal cutpoint is estimated from the data. Interactions are handled
naturally, since later splits are conditional on earlier ones (e.g. a
gender split appearing only among older respondents). And if no
instability is found, the tree simply does not split.

Statistically significant splits are not necessarily important splits, particularly in large samples. `RMdifTree()` therefore pairs
the tree with an effect-size classification for every item at every
split [@henninger_partial_2025]: the Mantel-Haenszel odds ratio on the
Delta scale developed at the Educational Testing Service (ETS) for
dichotomous data, or the partial gamma coefficient for polytomous data,
classified into the ETS categories A (negligible), B (slight to
moderate), and C (moderate to large)
[@holland_differential_1986;@zwick_review_2012]. For partial gamma the
B/C boundaries are the familiar 0.21 / 0.31 used by `RMdifGamma()`, so
the two functions read on the same scale. Note that these boundaries
are conventions carried over from large-scale educational testing, not
values calibrated to your sample and items --- in contrast to the
simulation-based cutoffs used elsewhere in this package --- so the
A/B/C labels are best read as a rough magnitude guide rather than a
calibrated test.


``` r
RMdifTree(items, covariates = phq9$gender)
```



**Node 1 -- gender: Female vs Male  (n left = 426, n right = 143)**

| Item | EffectSize | SE | Class | Flagged |
|:-----|-----------:|---:|:------|:--------|
|**q1** |**0.2508**  |**0.1045** |**B** |**yes** |
|**q2** |**0.4113**  |**0.0939** |**C** |**yes** |
|q3     |0.0637      |0.1031     |A     |no      |
|q4     |-0.0923     |0.1137     |A     |no      |
|**q5** |**-0.2857** |**0.0925** |**B** |**yes** |
|q6     |-0.1553     |0.1048     |A     |no      |
|q7     |-0.0747     |0.1023     |A     |no      |
|q8     |-0.1118     |0.1018     |A     |no      |
|q9     |0.1804      |0.0999     |A     |no      |


: Partial Credit Tree (9 items). n = 569 of 600 respondents (complete DIF covariates). Effect size: partial gamma (B/C thresholds = 0.21 / 0.31). alpha = 0.05. Flagged (Class B or C): 3 / 9 (item x split combinations). **Bold** rows are flagged.




The tree splits on gender, and the effect sizes match what the partial
gamma analysis showed above: q2 is classified as C and q1 and q5 as B,
while the remaining items are negligible (A). With a single binary
covariate the tree reduces to the familiar two-group comparison --- its
advantages appear when you supply several covariates at once
(`covariates = phq9[, c("gender", "age")]`), where it tests all of them,
picks split points for continuous ones, and uncovers interactions.

Three options are worth knowing about: purification = "iterative"
recomputes effect sizes while excluding already-flagged items from the
matching score; stability = TRUE refits the tree on resamples to
report how often each covariate is selected and where the cutpoints
land --- a guard against overinterpreting a single sample's tree
structure; and output = "plot" draws the tree with per-node item
parameters.

## Next step

Since we found issues with item misfit, local dependence, and gender DIF, these need to be addressed before the scale is used for measurement. Notably, item 2 recurs across all three criteria — overfit, the strongest local dependence (with item 1), and the largest gender DIF — making it the natural first candidate for closer inspection. The recommended approach is iterative: remove a single item (for instance, an underfit item, or one item from an LD pair after reviewing item content), then re-run the full set of analyses, since fit indications for the remaining items change with every removal. When the sample is large enough, it is good practice to set aside a random holdout subsample before the analysis, so that the final item set can be confirmed in data that played no part in the item-reduction decisions.

The following sections are primarily of interest once an acceptable item set has been established; here we continue with all nine items for demonstration purposes.

## Targeting

A targeting plot summarizes how well the item-threshold distribution matches
the distribution of person locations on the latent scale --- a Wright-map style
display.


``` r
RMtargeting(items)
#> Error in `seq.default()`:
#> ! 'to' must be a finite number
```

## Reliability

`RMreliability()` reports four reliability metrics: person separation
reliability (PSI); Relative Measurement Uncertainty (RMU)
estimate derived from posterior person-location uncertainty using plausible values; Cronbach's alpha; and marginal reliability. PSI, alpha and marginal can use bootstrap for confidence intervals. All reliability metrics range from 0 to 1, with higher values indicating better separation/precision.


``` r
RMreliability(items, draws = 200, rmu_iter = 20, parallel = FALSE,
              seed = 6)
```



Table: Reliability for 9 items, n = 600 respondents. PSI is the WLE-based separation reliability and excludes min/max scoring respondents.

|Metric           | Estimate| Lower (95% HDCI)| Upper (95% HDCI)|Notes                      |
|:----------------|--------:|----------------:|----------------:|:--------------------------|
|Cronbach's alpha |    0.886|               NA|               NA|no bootstrap               |
|PSI              |    0.838|               NA|               NA|no bootstrap               |
|Marginal         |    0.862|               NA|               NA|no bootstrap               |
|RMU (WLE)        |    0.881|            0.867|            0.895|200 PVs, 20 RMU iterations |



## Item and person parameters

Item (threshold) locations can be easily summarized in wide or long format, with or without SEs.


``` r
RMitemParameters(items, format = "wide")
```



Table: Item thresholds via CML (Andrich thresholds, logit scale). SE shown. n = 600 respondents.

|item |     t1|     t2|     t3| location| se_t1| se_t2| se_t3|
|:----|------:|------:|------:|--------:|-----:|-----:|-----:|
|q1   | -1.966|  0.634|  0.950|   -0.127| 0.165| 0.115| 0.124|
|q2   | -1.793|  0.300|  0.503|   -0.330| 0.168| 0.122| 0.118|
|q3   | -1.447|  0.039|  0.212|   -0.399| 0.166| 0.130| 0.116|
|q4   | -2.578| -0.506| -0.085|   -1.056| 0.250| 0.136| 0.109|
|q5   | -1.074| -0.048|  0.671|   -0.150| 0.153| 0.125| 0.115|
|q6   | -1.259| -0.051|  0.318|   -0.330| 0.161| 0.130| 0.114|
|q7   | -1.262|  0.041|  0.531|   -0.230| 0.157| 0.125| 0.115|
|q8   |  0.912|  1.572|  1.723|    1.402| 0.110| 0.151| 0.180|
|q9   |  0.749|  1.605|  1.307|    1.221| 0.110| 0.153| 0.172|



Person parameters are also sometimes referred to as thetas, latent scores, or person locations. By default, Warm's weighted likelihood estimation (WLE) is used for minimal bias. The function exports one value and SE for each individual, below reduced to only print the first 10 rows of the dataframe. Note that all table outputs in `easyRasch2` can be output as dataframes instead, if so desired. The option `output = "file"` makes it easy to export estimated latent scores for each respondent to a CSV file.


``` r
ppar <- RMpersonParameters(items, output = "dataframe")
head(ppar, 10)
#>         theta       sem sum_score n_answered extreme
#> 1   2.2760818 0.6184838        25          9   FALSE
#> 2  -0.9997531 0.4658741         7          9   FALSE
#> 3   1.1406643 0.4262412        20          9   FALSE
#> 4   0.3379335 0.3959994        15          9   FALSE
#> 5   3.6996335 1.3248428        27          9    TRUE
#> 6   1.1406643 0.4262412        20          9   FALSE
#> 7  -0.6129841 0.4288279         9          9   FALSE
#> 8   0.9701100 0.4155331        19          9   FALSE
#> 9   0.6465516 0.4019393        17          9   FALSE
#> 10 -3.2339486 0.9216600         1          9   FALSE
```

## Ordinal-to-interval transformation

For converting ordinal sum-scores to interval-scaled person-location
estimates with associated standard errors, use `RMscoreSE()`.


``` r
RMscoreSE(items, output = "ggplot")
```

<div class="figure">
<img src="figures/rasch-scoreSE-1.png" alt="**Figure 12.** *Sum-score to WLE conversion with 95% CIs*" width="100%" />
<p class="caption">**Figure 12.** *Sum-score to WLE conversion with 95% CIs*</p>
</div>

``` r
RMscoreSE(items)
```



Table: Person locations via Warm's WLE (CML item parameters). n = 600 respondents.

| Ordinal sum score| Logit score| Logit std.error|
|-----------------:|-----------:|---------------:|
|                 0|      -4.469|           1.536|
|                 1|      -3.234|           0.922|
|                 2|      -2.594|           0.737|
|                 3|      -2.138|           0.638|
|                 4|      -1.779|           0.574|
|                 5|      -1.480|           0.528|
|                 6|      -1.224|           0.493|
|                 7|      -1.000|           0.466|
|                 8|      -0.798|           0.445|
|                 9|      -0.613|           0.429|
|                10|      -0.440|           0.417|
|                11|      -0.277|           0.408|
|                12|      -0.119|           0.401|
|                13|       0.034|           0.397|
|                14|       0.186|           0.396|
|                15|       0.338|           0.396|
|                16|       0.491|           0.398|
|                17|       0.647|           0.402|
|                18|       0.806|           0.408|
|                19|       0.970|           0.416|
|                20|       1.141|           0.426|
|                21|       1.320|           0.441|
|                22|       1.512|           0.462|
|                23|       1.723|           0.492|
|                24|       1.968|           0.539|
|                25|       2.276|           0.618|
|                26|       2.724|           0.778|
|                27|       3.700|           1.325|



`RMscoreSE()` also has an option for EAP scores (expected a posteriori).

## Person fit

Conditional person infit/outfit MSQ and the $\ell_z$ statistic are implemented, with Monte-Carlo resampling for *p*-values.


``` r
pfit <- RMpersonFit(items, iterations = 100, output = "ggplot", seed = 7)
pfit$lz
```

<div class="figure">
<img src="figures/rasch-personfit-1.png" alt="**Figure 13.** *Person fit with the lz statistic*" width="100%" />
<p class="caption">**Figure 13.** *Person fit with the lz statistic*</p>
</div>


## Where to next

* Each `RM*()` function is documented with its own `?function`
  reference page including a worked example.
* The simulation-based cutoffs used above (`RM*Cutoff()`) can be
  parallelised on multiple CPU cores via the `mirai` package; see the relevant help pages.
  * For a progress bar on time-consuming simulations, add `verbose = TRUE` to the function call. This should not be used when rendering Quarto/Rmd files.


## References

# RMitemInfitCutoffPlot is a deprecated alias forwarding to RMitemInfitPlot

    Code
      p <- RMitemInfitCutoffPlot(simfit)
    Condition
      Warning in `RMitemInfitCutoffPlot()`:
      'RMitemInfitCutoffPlot' is deprecated.
      Use 'RMitemInfitPlot' instead.
      See help("Deprecated")

# output = 'file' errors early when filename is missing

    Code
      RMitemParameters(make_polytomous(), output = "file")
    Condition
      Error:
      ! `filename` must be a single non-empty string when `output = "file"`.

# errors when no complete cases remain after na.omit

    Code
      RMitemInfitCutoff(d, iterations = 3, parallel = FALSE)
    Condition
      Error:
      ! No complete cases in data. All rows contain at least one NA.

# parallel = TRUE without n_cores or mc.cores warns and falls back

    Code
      res <- RMitemInfitCutoff(make_dich(), iterations = 4, parallel = TRUE, seed = 1,
      cutoff_method = "quantile")
    Condition
      Warning:
      For parallel processing, specify n_cores or set options(mc.cores = N).
      (Use `parallel::detectCores()` to see how many cores are available.)
      Falling back to sequential (single core) processing.

# RMitemInfitCutoffMI errors on a non-mids object

    Code
      RMitemInfitCutoffMI(data.frame(a = 0:1, b = 1:0))
    Condition
      Error:
      ! `mids_object` must be a 'mids' object as returned by mice::mice().

# RMitemInfitMI errors on a non-mids object

    Code
      RMitemInfitMI(data.frame(a = 0:1, b = 1:0))
    Condition
      Error:
      ! `mids_object` must be a 'mids' object as returned by mice::mice().

# RMitemInfitMI rejects malformed cutoff arguments

    Code
      RMitemInfitMI(imp, cutoff = "abc")
    Condition
      Error:
      ! `cutoff` must be NULL, the return value of RMitemInfitCutoff() / RMitemInfitCutoffMI(), or its $item_cutoffs data.frame.

---

    Code
      RMitemInfitMI(imp, cutoff = data.frame(Item = "I1", infit_low = 0.8))
    Condition
      Error:
      ! `cutoff` data.frame is missing required columns: infit_high.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dif_lr.R
\name{RMdifLR}
\alias{RMdifLR}
\title{DIF analysis via Andersen's likelihood-ratio test}
\usage{
RMdifLR(
  data,
  dif_var,
  model = c("auto", "PCM", "RM"),
  level = c("item", "threshold"),
  output = c("kable", "dataframe", "ggplot"),
  cutoff = 0.5,
  conf = 0.95,
  sort = FALSE
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses (non-negative
integers, 0-based). One column per item, one row per person. Person
IDs and grouping variables must not be included -- pass the grouping
variable separately via \code{dif_var}.}

\item{dif_var}{Vector of length \code{nrow(data)} (factor, character,
or numeric) defining the DIF grouping variable. Coerced to factor;
unused levels are dropped. Rows where \code{dif_var} is \code{NA} are
dropped with a message. Must result in at least 2 groups after
cleaning.}

\item{model}{One of \code{"auto"} (default), \code{"PCM"}, or
\code{"RM"}. \code{"auto"} fits \code{eRm::RM()} when the data are
dichotomous (max response = 1) and \code{eRm::PCM()} otherwise.
\code{"RM"} errors on polytomous data.}

\item{level}{One of \code{"item"} (default) or \code{"threshold"}.
\code{"item"} reports each item's mean threshold location per group;
\code{"threshold"} reports each individual threshold per group. For
dichotomous (RM) data the two views are equivalent (one threshold per
item).}

\item{output}{One of \code{"kable"} (default), \code{"dataframe"}, or
\code{"ggplot"}. The data.frame view always carries a
\code{Flagged} logical column; the kable view bolds flagged rows; the
ggplot view shows confidence intervals.}

\item{cutoff}{Numeric or \code{NULL}. Threshold (in logits) for the
\code{Flagged} column: a row is flagged when
\code{MaxDiff > cutoff}, where \code{MaxDiff} is the difference
between the largest and smallest per-group location for that item
(or threshold). Set to \code{NULL} to suppress flagging. Default
\code{0.5}.}

\item{conf}{Numeric in (0, 1). Confidence level used for the ggplot
error bars. Default \code{0.95}.}

\item{sort}{Logical. \code{kable} output only: sort rows by
\code{MaxDiff} (descending). Default \code{FALSE}.}
}
\value{
A data.frame, a \code{knitr_kable} object, or a \code{ggplot}
object, depending on \code{output}.

The data.frame has one row per item (\code{level = "item"}) or per
item x threshold (\code{level = "threshold"}), with columns
\code{Item} (and \code{Threshold} at threshold level), one numeric
column per group level, an \code{All} column for the unsplit fit,
\code{MaxDiff}, \code{Flagged} (when \code{cutoff} is non-\code{NULL}),
and matching \code{SE_*} columns.

The Andersen LR test result is attached as
\code{attr(result, "lr_test")} on the data.frame, in the kable
footnote, and in the ggplot caption (LR \eqn{\chi^2}, df, p-value).
}
\description{
Splits a Rasch model by an external grouping variable using
\code{eRm::LRtest()} and reports per-group item locations (or per-group
threshold locations) together with their standard errors. A single
function replaces the four legacy helpers (\code{RIdifTableLR},
\code{RIdifThreshTblLR}, \code{RIdifFigureLR}, \code{RIdifThreshFigLR}) by exposing the
two underlying axes -- \code{level} (item or threshold) and \code{output}
(data.frame, kable, or ggplot) -- as arguments. The same data
preparation pipeline feeds all six combinations.
}
\details{
The Partial Credit Model (PCM) is fitted by default for polytomous data
and the dichotomous Rasch Model (RM) is fitted when all responses are
0/1; this can be overridden via \code{model}.

For the data.frame and kable outputs, locations are reported on the
centred eRm parameterisation returned by \code{eRm::thresholds()}.
Per-group fits come from \code{eRm::LRtest(..., splitcr = dif_var)};
the unsplit fit (\code{All} column) is the model fitted to the full
dataset. The Andersen LR statistic, df, and p-value reported as the
\code{lr_test} attribute / caption come directly from
\code{LRtest()}'s return value.

\code{cell_spec()}-style HTML cell colouring used in the legacy
easyRasch package has been dropped in favour of a logical
\code{Flagged} column (and bold rendering in the kable output), so the
kable renders correctly in HTML, LaTeX, and pipe/markdown.
}
\examples{
\donttest{
if (requireNamespace("eRm", quietly = TRUE)) {
  set.seed(1)
  data("pcmdat2", package = "eRm")
  grp <- factor(sample(c("A", "B"), nrow(pcmdat2), replace = TRUE))

  # Default: kable of per-group item locations
  RMdifLR(pcmdat2, dif_var = grp)

  # ggplot panel of item locations with 95\% CIs
  RMdifLR(pcmdat2, dif_var = grp, output = "ggplot")

  # Threshold-level kable, sorted by MaxDiff
  RMdifLR(pcmdat2, dif_var = grp, level = "threshold", sort = TRUE)

  # Tidy data.frame for downstream use
  df <- RMdifLR(pcmdat2, dif_var = grp, output = "dataframe")
  attr(df, "lr_test")
  df[df$Flagged, ]
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap_restscore.R
\name{RMitemRestscoreBoot}
\alias{RMitemRestscoreBoot}
\title{Bootstrap Item-Restscore Misfit Detection}
\usage{
RMitemRestscoreBoot(
  data,
  iterations = 200,
  samplesize = 600,
  parallel = TRUE,
  n_cores = NULL,
  cutoff = 5,
  verbose = FALSE,
  seed = NULL,
  output = "kable"
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers).}

\item{iterations}{Integer. Number of bootstrap samples (default 200).}

\item{samplesize}{Integer. Size of each bootstrap sample (default 600). Must
not exceed \code{nrow(data)}.}

\item{parallel}{Logical. Use parallel processing via \code{mirai} if available
(default \code{TRUE}).}

\item{n_cores}{Integer or \code{NULL}. Number of parallel workers. When \code{NULL},
\code{getOption("mc.cores")} is checked first. If neither is set and
\code{parallel = TRUE}, a warning is issued and execution falls back to
sequential processing.}

\item{cutoff}{Numeric. Items flagged in fewer than this percentage of
iterations are excluded from the kable output (default 5). Has no effect
when \code{output = "dataframe"} or \code{output = "raw"}.}

\item{verbose}{Logical. Show a progress bar (default \code{FALSE}).}

\item{seed}{Integer or \code{NULL}. Random seed for reproducibility.}

\item{output}{Either \code{"kable"} (default) for a formatted \code{knitr::kable()}
table, \code{"dataframe"} for the per-item summary data.frame, or \code{"raw"} for
the per-iteration long data.frame (useful for custom plotting).}
}
\value{
\itemize{
\item If \code{output = "kable"}: a \code{knitr_kable} object listing items flagged in
more than \code{cutoff}\% of iterations, with columns Item, Item-restscore
result, \% of iterations, Conditional MSQ infit, and Relative item
location, and a caption noting iteration count, bootstrap size, and
number of complete cases.
\item If \code{output = "dataframe"}: a data.frame with one row per item ×
classification combination (\code{Item}, \code{item_restscore}, \code{n}, \code{percent},
\code{Infit_MSQ}, \code{Relative_location}), including \code{"no misfit"} rows.
\item If \code{output = "raw"}: a long data.frame with one row per item ×
successful iteration (\code{iteration}, \code{Item}, \code{item_restscore}, \code{diff},
\code{diff_abs}), where \code{diff = expected - observed}.
}
}
\description{
Non-parametric bootstrap of item-restscore fit using
\code{iarm::item_restscore()}. For each iteration, a sample of size \code{samplesize}
is drawn from \code{data} with replacement, the appropriate Rasch model is
refitted, and item-restscore results are classified as \code{"overfit"},
\code{"underfit"}, or \code{"no misfit"} based on the BH-adjusted p-value (< .05) and
the sign of \code{expected - observed}. The function returns the percentage of
iterations in which each item is flagged.
}
\details{
Useful with large samples, where the asymptotic test underlying
\code{\link{RMitemRestscore}} can flag items that are not practically
misfitting; bootstrapping gives a more nuanced view of the probability of
an item actually being misfit.

The full-sample model is fitted by CML via \code{psychotools::pcmodel()} (a
dichotomous item is a 2-category partial credit model), and item locations
(mean of the grand-mean-centred CML Andrich thresholds) and the mean WLE
person location are computed from it — consistent with
\code{\link{RMitemRestscore}} and the rest of the package. Each bootstrap
iteration draws a sample of size \code{samplesize} with replacement and refits via
\code{psychotools::pcmodel(..., hessian = FALSE)} for speed;
\code{iarm::item_restscore()} accepts the fitted model.

Conditional infit MSQ (computed once on the full sample via
\code{iarm::out_infit()}) and relative item locations are reported alongside the
bootstrap percentages for context. The item-restscore classification and the
infit statistic are conditional and engine-invariant; only the relative-item
location shifts slightly relative to the previous \code{eRm} implementation,
because it now uses the WLE person mean (finite at extreme scores) rather
than the \code{eRm} MLE mean.

Iterations that fail (e.g., due to convergence issues on a degenerate
bootstrap sample) are silently discarded; the caption / \code{actual_iterations}
reflects only successful runs.

Parallel processing is provided by the \code{mirai} package (optional). Install
it with \code{install.packages("mirai")} to enable parallelisation.

The \code{iarm} package must be installed (it is in Suggests, not Imports).
}
\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE)) {
set.seed(42)
sim_data <- as.data.frame(
  matrix(sample(0:1, 400 * 8, replace = TRUE), nrow = 400, ncol = 8)
)
colnames(sim_data) <- paste0("Item", 1:8)

# Few iterations for a fast example; use 100+ in real analyses
# Default kable output (only items flagged > cutoff\%)
RMitemRestscoreBoot(sim_data, iterations = 50, samplesize = 300,
                parallel = FALSE, seed = 1)

# Per-item summary data.frame (all classifications, including "no misfit")
summary_df <- RMitemRestscoreBoot(sim_data, iterations = 50, samplesize = 300,
                              parallel = FALSE, seed = 1,
                              output = "dataframe")

# Per-iteration long data for custom plotting
raw_df <- RMitemRestscoreBoot(sim_data, iterations = 50, samplesize = 300,
                          parallel = FALSE, seed = 1, output = "raw")

# Distribution of (expected - observed) across iterations, per item
if (requireNamespace("ggplot2", quietly = TRUE)) {
  library(ggplot2)
  ggplot(raw_df, aes(x = Item, y = diff)) +
    geom_hline(yintercept = 0, linetype = "dashed", colour = "grey50") +
    geom_violin(fill = "grey90", colour = NA) +
    geom_jitter(aes(colour = item_restscore),
                width = 0.15, alpha = 0.5, size = 0.8) +
    scale_colour_manual(values = c("overfit"   = "#377eb8",
                                   "underfit"  = "#e41a1c",
                                   "no misfit" = "grey60")) +
    labs(y = "Expected - observed restscore correlation",
         colour = NULL) +
    theme_minimal() +
    theme(axis.text.x = element_text(angle = 45, hjust = 1))
}
}
}
}
\references{
Kreiner, S. (2011). A Note on Item-Restscore Association in Rasch Models.
\emph{Applied Psychological Measurement, 35}(7), 557-561.
\doi{10.1177/0146621611410227}
}
\seealso{
\code{\link{RMitemRestscore}}, \code{\link{RMitemInfit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/score_se.R
\name{RMscoreSE}
\alias{RMscoreSE}
\title{Raw-Score to Logit Score Transformation Table}
\usage{
RMscoreSE(
  data,
  method = "WLE",
  output = "kable",
  ci_multiplier = 1.96,
  point_size = 3,
  error_width = 0.5,
  theta_range = c(-10, 10)
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Missing values (\code{NA}) are allowed;
the underlying model fit handles them.}

\item{method}{Character string. Either \code{"WLE"} (default) for Warm's Weighted
Likelihood Estimator computed from a CML-fitted Rasch / Partial Credit
Model via \code{psychotools}, or \code{"EAP"} for Expected A Posteriori sum-score
estimates from an MML-fitted model via \code{mirt}.}

\item{output}{Character string controlling the return value: \code{"kable"}
(default) for a formatted \code{knitr::kable()} table, \code{"dataframe"} for the
underlying data.frame, or \code{"ggplot"} for a \code{ggplot2} figure showing each
raw score's logit estimate with \code{ci_multiplier}-scaled error bars.}

\item{ci_multiplier}{Numeric. Multiplier applied to the standard error to
draw error bars on the figure. Default \code{1.96} (\eqn{\approx}95\% CI under a Gaussian
approximation). Ignored when \code{output != "ggplot"}.}

\item{point_size}{Numeric. Point size for the figure. Default \code{3}.}

\item{error_width}{Numeric. Cap width for error bars on the figure.
Default \code{0.5}.}

\item{theta_range}{Numeric length 2. Theta search range used for boundary
raw scores under WLE estimation. Default \code{c(-10, 10)}. Ignored when
\code{method = "EAP"}.}
}
\value{
\itemize{
\item If \code{output = "kable"}: a \code{knitr_kable} object with columns "Ordinal sum
score", "Logit score", and "Logit std.error", and a caption noting the
estimation method.
\item If \code{output = "dataframe"}: a data.frame with columns \code{raw_score},
\code{logit_score}, and \code{logit_se} (one row per possible raw sum score from 0
to the theoretical maximum).
\item If \code{output = "ggplot"}: a \code{ggplot} object — points at each
(\code{logit_score}, \code{raw_score}) with horizontal error bars at
± \verb{ci_multiplier × logit_se}.
}
}
\description{
For a given set of items, returns the score-to-theta lookup that maps each
possible raw sum score to a person-location estimate (in logits) and its
standard error. Useful when reporting a scale's measurement properties or
converting raw totals to interval-scaled scores for downstream analysis.
}
\details{
The function automatically detects whether the data is dichotomous
(max score 1) or polytomous (max score > 1) and selects the appropriate
Rasch / Partial Credit model.

\strong{\code{method = "WLE"}} fits the model by CML with \code{psychotools::pcmodel()},
centres the item thresholds to grand-mean-zero, and solves Warm's
weighted-likelihood equation for each raw score with the same engine used
by \code{\link[=RMpersonParameters]{RMpersonParameters()}}; the two functions therefore report identical
locations and standard errors. Warm's bias correction yields finite
locations even at the minimum and maximum scores (only a root outside
\code{theta_range} is clamped to the boundary with \code{NA} SE). The reported
\code{logit_se} is the information-based standard error \code{1 / sqrt(I(theta))},
matching \code{catR}, \code{TAM} and most Rasch software.

\strong{\code{method = "EAP"}} fits the model with \code{mirt::mirt(..., itemtype = "Rasch")} (MML) and obtains sum-score-based EAP estimates and posterior
SDs via \code{mirt::fscores(method = "EAPsum", full.scores = FALSE, full.scores.SE = TRUE)}. EAP estimates are finite at all score boundaries
(the prior shrinks them inward), but they depend on the assumed normal
prior on theta. Item parameters from MML differ slightly from the CML
values used by the WLE path; for well-behaved data the difference is small.
}
\examples{
\donttest{
set.seed(42)
sim_data <- as.data.frame(
  matrix(sample(0:3, 200 * 6, replace = TRUE), nrow = 200, ncol = 6)
)
colnames(sim_data) <- paste0("Item", 1:6)

# Default kable output, WLE
RMscoreSE(sim_data)

# Underlying data.frame
RMscoreSE(sim_data, output = "dataframe")

# ggplot figure
if (requireNamespace("ggplot2", quietly = TRUE)) {
  RMscoreSE(sim_data, output = "ggplot")
}

# EAP via mirt
RMscoreSE(sim_data, method = "EAP")
}
}
\references{
Warm, T. A. (1989). Weighted likelihood estimation of ability in item
response theory. \emph{Psychometrika, 54}(3), 427-450.
\doi{10.1007/BF02294627}

Bock, R. D., & Mislevy, R. J. (1982). Adaptive EAP estimation of ability
in a microcomputer environment. \emph{Applied Psychological Measurement, 6}(4),
431-444. \doi{10.1177/014662168200600405}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residual_pca.R
\name{RMdimResidualPCACutoff}
\alias{RMdimResidualPCACutoff}
\title{Simulation-based Cutoff for First-Contrast Eigenvalue}
\usage{
RMdimResidualPCACutoff(
  data,
  iterations = 250,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses (0-based,
non-negative integers).}

\item{iterations}{Integer. Number of simulation iterations. Default \code{250}.}

\item{parallel}{Logical. Use parallel processing via \code{mirai}. Default
\code{TRUE}.}

\item{n_cores}{Integer or \code{NULL}. Number of parallel workers. When \code{NULL},
\code{getOption("mc.cores")} is checked first; if neither is set,
\code{parallel = TRUE} falls back to sequential with a warning.}

\item{verbose}{Logical. Show a progress bar. Default \code{FALSE}.}

\item{seed}{Integer or \code{NULL}. Random seed for reproducibility.}
}
\value{
A list with components:
\describe{
\item{\code{results}}{data.frame: \code{iteration}, \code{eigenvalue}.}
\item{\code{p95}, \code{p99}, \code{p995}, \code{p999}}{Empirical percentiles of
\code{eigenvalue}.}
\item{\code{max}}{The largest simulated eigenvalue.}
\item{\code{suggested_cutoff}}{The 99th percentile (\code{p99}) — pass this list
back into \code{\link{RMdimResidualPCA}} via \verb{cutoff = }, or use
\code{suggested_cutoff} directly.}
\item{\code{suggested_cutoff_percentile}}{The percentile used for
\code{suggested_cutoff}, currently always \code{99}.}
\item{\code{actual_iterations}}{Number of successful iterations.}
\item{\code{sample_n}}{Number of complete cases used.}
\item{\code{item_names}}{Character vector of item names.}
}
}
\description{
Parametric bootstrap producing an empirical upper percentile for the
largest eigenvalue from a PCA of standardized residuals under a correctly
fitting Rasch model. For each iteration the function resamples theta
values from the data-based estimates, simulates response data, refits the
appropriate Rasch model, and extracts the first-contrast eigenvalue.
Several upper-tail percentiles of the resulting distribution are returned;
the 99th percentile is reported as the suggested cutoff (matching the
convention used by \code{\link{RMlocdepQ3Cutoff}}).
}
\details{
Rule-of-thumb cutoffs for the first-contrast eigenvalue depend strongly on
sample size, test length, and item-parameter spread; simulation-based
cutoffs tailored to the data are more defensible (Chou & Wang, 2010).

The generating model uses CML item parameters (\code{psychotools}) and WLE person
locations. Per iteration: theta values are sampled with replacement from the
WLE estimates; response data are simulated under the model
(\code{psychotools::rrm} for dichotomous, partial-credit simulator for
polytomous); CML/WLE standardized residuals are computed (the same engine as
the observed analysis); \code{prcomp()} is run; the first-contrast eigenvalue is
recorded.

Iterations that fail (e.g., due to a degenerate simulated dataset where
some category isn't represented) are silently dropped. The \code{iarm} package
is \strong{not} required.
}
\examples{
\donttest{
set.seed(1)
dat <- as.data.frame(
  matrix(sample(0:1, 200 * 12, replace = TRUE), nrow = 200, ncol = 12)
)
colnames(dat) <- paste0("I", 1:12)

# Few iterations for a fast example; use 250+ in real analyses
bound <- RMdimResidualPCACutoff(dat, iterations = 50, parallel = FALSE, seed = 1)
bound$suggested_cutoff

RMdimResidualPCA(dat, cutoff = bound)
}
}
\references{
Chou, Y.-T., & Wang, W.-C. (2010). Checking dimensionality in item
response models with principal component analysis on standardized
residuals. \emph{Educational and Psychological Measurement, 70}(5), 717-731.
\doi{10.1177/0013164410379322}
}
\seealso{
\code{\link{RMdimResidualPCA}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_plots.R
\name{RMplotStackedbar}
\alias{RMplotStackedbar}
\title{Stacked Bar Chart of Item Response Distributions}
\usage{
RMplotStackedbar(
  data,
  item_labels = NULL,
  category_labels = NULL,
  show_n = TRUE,
  show_percent = FALSE,
  text_color = "sienna1",
  text_size = 3,
  min_label_n = 0L,
  viridis_option = "D",
  viridis_end = 0.99,
  title = "Item responses"
)
}
\arguments{
\item{data}{A data.frame in wide format containing only the item
response columns. Each column is one item, each row is one person.
All columns must be numeric (integer-valued). Response categories
may be coded starting from 0 or 1. Do not include person IDs,
grouping variables, or other non-item columns.}

\item{item_labels}{An optional character vector of descriptive labels
for the items (y-axis). Must be the same length as \code{ncol(data)}.
If \code{NULL} (the default), column names are used.}

\item{category_labels}{An optional character vector of labels for the
response categories (legend). Must be the same length as the number
of response categories spanning from the minimum to the maximum
observed value, ordered from lowest to highest category. If \code{NULL}
(the default), numeric category values are used.}

\item{show_n}{Logical. If \code{TRUE} (the default), the count of responses
is displayed as a text label inside each bar segment.}

\item{show_percent}{Logical. If \code{TRUE}, the percentage of responses is
displayed instead of (or in addition to) counts. Default is \code{FALSE}.}

\item{text_color}{Character. Colour for the count/percentage labels.
Default is \code{"sienna1"}.}

\item{text_size}{Numeric. Size of the count/percentage labels.
Default is \code{3}.}

\item{min_label_n}{Integer. Minimum count required for a label to be
displayed within a bar segment. Segments with fewer responses are
left unlabelled to avoid clutter. Default is \code{0L} (all segments
labelled).}

\item{viridis_option}{Character. Viridis palette option. One of
\code{"A"} through \code{"H"}. Default is \code{"D"}.}

\item{viridis_end}{Numeric in \eqn{[0, 1]}. End point of the viridis
colour scale. Default is \code{0.99}.}

\item{title}{Character. Plot title. Default is \code{"Item responses"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Creates a horizontal stacked bar chart showing the response
distribution for all items. Each bar represents one item, with segments
coloured by response category. Counts are displayed as text labels
within each segment. This is a descriptive data visualization tool
intended for use before model fitting.
}
\details{
Items are displayed on the y-axis in the same order as the columns
in \code{data} (first column at the top). Each bar is divided into
segments representing response categories, with the lowest category
on the left and the highest on the right. The total bar length
equals the number of non-missing responses for that item.

Categories with zero responses still appear in the legend but produce
no visible bar segment, which helps identify gaps in the response
distribution.

The plot caption reports the sample in the standard
\verb{n = X respondents (policy)} form; item-level \code{NA}s are retained --
each bar counts the non-missing responses for that item.

\strong{Input requirements:}
\itemize{
\item All columns must be numeric (integer-valued).
\item The data frame must contain at least 2 columns (items)
and at least 1 row (person).
}
}
\examples{
if (requireNamespace("eRm", quietly = TRUE) &&
    requireNamespace("ggplot2", quietly = TRUE)) {
  data(pcmdat2, package = "eRm")

  # Basic stacked bar chart
  RMplotStackedbar(pcmdat2)

  # With custom item and category labels
  RMplotStackedbar(
    pcmdat2,
    item_labels     = c("Mood", "Sleep", "Appetite", "Energy"),
    category_labels = c("Never", "Sometimes", "Often")
  )

  # Show percentages, suppress small segments
  RMplotStackedbar(
    pcmdat2,
    show_percent = TRUE,
    show_n       = FALSE,
    min_label_n  = 5
  )
}

}
\seealso{
\code{\link[=RMplotBar]{RMplotBar()}}, \code{\link[=RMplotTile]{RMplotTile()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/local_dependence.R
\name{run_q3_sim_sequential}
\alias{run_q3_sim_sequential}
\title{Run Q3 simulations sequentially}
\usage{
run_q3_sim_sequential(iterations, sim_seeds, sim_data_list, verbose = FALSE)
}
\arguments{
\item{iterations}{Number of iterations.}

\item{sim_seeds}{Integer vector of per-iteration seeds.}

\item{sim_data_list}{List of data passed to each worker.}

\item{verbose}{Show progress bar.}
}
\value{
List of raw results (one element per iteration).
}
\description{
Run Q3 simulations sequentially
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/easyRasch2-package.R
\docType{package}
\name{easyRasch2-package}
\alias{easyRasch2}
\alias{easyRasch2-package}
\title{easyRasch2: Psychometric Analysis with Rasch Measurement Theory}
\description{
Streamlines reproducible Rasch measurement theory analyses for ordinal item-response data, combining estimation routines from 'eRm', 'psychotools', 'mirt', 'iarm', and 'lavaan' with consistent diagnostic, plotting, and reporting layers. Covers the four basic psychometric criteria summarised by Christensen et al. (2021) \doi{10.1111/sms.13908} -- unidimensionality, local independence, ordered response category thresholds, and invariance across subgroups -- together with item fit, targeting, reliability, category functioning, and descriptive item-response plots. A distinguishing feature is the use of simulation-based critical values to replace rule-of-thumb cutoffs for conditional infit mean-square, Yen's Q3 local-dependence statistic, the largest residual-PCA eigenvalue, ordinal CFA fit indices, and partial-gamma DIF and local-dependence coefficients, optionally augmented with multiplicity-corrected bootstrap p-values. Outputs are knitr::kable() tables and 'ggplot2' figures suitable for direct inclusion in 'Quarto' and 'R Markdown' reports.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/pgmj/easyRasch2}
  \item \url{https://pgmj.github.io/easyRasch2/}
  \item Report bugs at \url{https://github.com/pgmj/easyRasch2/issues}
}

}
\author{
\strong{Maintainer}: Magnus Johansson \email{pgmj@pm.me} (\href{https://orcid.org/0000-0003-1669-592X}{ORCID})

Authors:
\itemize{
  \item Magnus Johansson \email{pgmj@pm.me} (\href{https://orcid.org/0000-0003-1669-592X}{ORCID})
}

Other contributors:
\itemize{
  \item Nicklas Korsell (PCM simulation code) [contributor]
  \item Mirka Henninger (\href{https://orcid.org/0000-0003-4676-2361}{ORCID}) (MH / partial-gamma effect-size and ETS-classification algorithms in dif_tree.R, adapted under MIT licence from the raschtreeMH and effecttree packages) [contributor]
  \item Jan Radek (\href{https://orcid.org/0009-0003-8842-9206}{ORCID}) (partial-gamma effect-size and ETS-classification algorithms in dif_tree.R, adapted under MIT licence from the effecttree package) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tile_plot.R
\name{RMplotTile}
\alias{RMplotTile}
\title{Tile Plot of Item Response Distributions}
\usage{
RMplotTile(
  data,
  group = NULL,
  cutoff = 10,
  highlight = TRUE,
  percent = FALSE,
  text_color = "orange",
  text_size = 4,
  zero_fill = "white",
  item_labels = NULL,
  category_labels = NULL,
  group_labels = NULL,
  facet_ncol = NULL,
  output = c("ggplot", "dataframe")
)
}
\arguments{
\item{data}{A data.frame in wide format containing only the item
response columns. Each column is one item, each row is one person.
All columns must be numeric (integer-valued). Response categories
may be coded starting from 0 or 1. Do not include person IDs,
grouping variables, or other non-item columns -- supply the grouping
variable separately via \code{group}.}

\item{group}{Optional vector of length \code{nrow(data)} (factor, character,
or numeric) defining a grouping variable. When provided, the plot is
faceted by group and counts / percentages are computed within each
group. Default \code{NULL} (no faceting). Persons with \code{NA} group are
excluded, with a \code{message()} reporting how many rows were dropped.}

\item{cutoff}{Integer. Cells with counts below this value are
highlighted (when \code{highlight = TRUE}). Default \code{10}.}

\item{highlight}{Logical. If \code{TRUE} (default), cell labels with counts
below \code{cutoff} are displayed in red. This includes empty cells
(\code{n = 0}), useful for identifying gaps in the response distribution.}

\item{percent}{Logical. If \code{TRUE}, cell labels show percentages
instead of raw counts. Percentages are computed within item (and
within group, when \code{group} is supplied). Default \code{FALSE}.}

\item{text_color}{Character. Colour for non-highlighted cell labels.
Default \code{"orange"}.}

\item{text_size}{Numeric. Size of the cell labels (passed to
\code{ggplot2::geom_text()}). Default \code{4}, matching \code{\link[=RMitemCatProb]{RMitemCatProb()}}.}

\item{zero_fill}{Colour for cells with zero responses, or \code{NULL} to
keep them on the fill scale. Default \code{"white"}: empty cells are
shown unfilled with a light grey outline, so the \emph{absence} of data
reads as absence rather than as the darkest end of the colour
scale (where \code{n = 0} is nearly indistinguishable from \code{n = 1}).}

\item{item_labels}{Optional character vector of descriptive labels
for the items (y-axis), same length as \code{ncol(data)}. Default \code{NULL}
uses the column names.}

\item{category_labels}{Optional character vector of labels for the
response categories (x-axis), same length as the number of
categories spanning \code{min} to \code{max} observed value. Default \code{NULL}
uses the numeric category values.}

\item{group_labels}{Optional character vector of length
\code{nlevels(as.factor(group))} to override the displayed facet labels.
Order corresponds to \code{levels(as.factor(group))}.}

\item{facet_ncol}{Integer or \code{NULL}. Number of columns in the facet
grid when \code{group} is supplied. Default \code{NULL} (ggplot2 chooses).}

\item{output}{Character. \code{"ggplot"} (default) returns the plot;
\code{"dataframe"} returns the underlying per-cell counts (one row per
item x category, plus group when supplied).}
}
\value{
Either a \code{ggplot} object or a data.frame, depending on
\code{output}.
}
\description{
Creates a tile (heat map) plot showing the distribution of responses
across all items and response categories. Each cell displays the count
(or percentage) of responses, with optional conditional highlighting
for cells with low counts. Optional faceting by a grouping variable
is provided for inspecting subgroup response distributions before DIF
analyses -- particularly useful for spotting empty categories or
under-represented subgroups before fitting Rasch models per group.
}
\details{
Adapted from \code{easyRaschBayes::plot_tile()} and extended with the
\code{group} parameter for faceted display.

Items are placed on the y-axis (in the same order as the columns of
\code{data}, top to bottom) and response categories on the x-axis. Cell
shading represents the count of responses (darker = more responses).
Categories with zero responses are explicitly shown (\code{n = 0}), which
helps identify gaps in the response distribution -- one of the primary
purposes of the plot, especially before DIF analyses where
under-represented categories within a subgroup can break model
fitting on that subgroup.

When \code{group} is supplied, percentages and the highlight cutoff are
applied within each group, so a cell labelled "5" in the group-A
facet contains the count for group A only.

The plot caption reports the sample in the standard
\verb{n = X of Y respondents (policy)} form: rows with \code{NA} group are
dropped (and counted in \code{Y} only), while item-level \code{NA}s are retained
-- each cell simply counts the non-missing responses for that item.
}
\examples{
\donttest{
if (requireNamespace("eRm", quietly = TRUE)) {
  data("pcmdat2", package = "eRm")

  # Basic tile plot
  RMplotTile(pcmdat2)

  # With percentages
  RMplotTile(pcmdat2, percent = TRUE)

  # Faceted by an external grouping variable
  set.seed(1)
  grp <- sample(c("A", "B"), nrow(pcmdat2), replace = TRUE)
  RMplotTile(pcmdat2, group = grp)

  # With custom labels and tighter cutoff
  RMplotTile(pcmdat2,
             group = grp,
             group_labels = c("Female", "Male"),
             cutoff = 5,
             facet_ncol = 2)

  # Underlying counts as a data.frame
  RMplotTile(pcmdat2, group = grp, output = "dataframe")
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cfa_cutoff.R
\name{RMdimCFA}
\alias{RMdimCFA}
\title{Observed one-factor CFA fit and loadings vs a simulated reference}
\usage{
RMdimCFA(
  data,
  cutoff,
  p_value = FALSE,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  alpha = 0.05,
  output = c("kable", "dataframe")
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses (non-negative
integers, 0-based), the same items used for the cutoff simulation.}

\item{cutoff}{The list returned by \code{\link{RMdimCFACutoff}}. Required:
observed CFA fit indices are not interpretable without the simulated
reference, so the function errors if it is missing.}

\item{p_value}{Logical. When \code{TRUE}, adds bootstrap p-values from the
simulated null distributions: one-sided in the unfavourable direction
for the fit indices (CFI low; RMSEA / SRMR high), two-sided for the
per-item loadings. The \code{Flagged} columns then reflect
\code{padj < alpha} instead of the percentile cutoffs. The fit indices and
the loadings are corrected as two separate families. Default \code{FALSE}.}

\item{correction}{Character. Multiplicity correction for the p-values:
\code{"fwer"} (default; Westfall-Young studentised-max step-down), \code{"fdr_bh"}
(Benjamini-Hochberg), \code{"fdr_by"} (Benjamini-Yekutieli), or \code{"none"}.
Ignored when \code{p_value = FALSE}.}

\item{alpha}{Numeric in (0, 1). Significance level used to flag
comparisons on the corrected p-value. Default \code{0.05}. Ignored when
\code{p_value = FALSE}.}

\item{output}{Character. \code{"kable"} (default) returns each table as a
\code{knitr::kable()}; \code{"dataframe"} returns plain data.frames.}
}
\value{
A named list with two elements, \code{fit} and \code{loadings}:
\describe{
\item{\code{fit}}{CFI / RMSEA / SRMR with columns \code{Index}, \code{Observed},
\code{Cutoff}, \code{Direction}, \code{Flagged} (one-sided, in the unfavourable
direction). With \code{p_value = TRUE}, columns \code{p} and \code{padj} are added
and \code{Flagged} reflects \code{padj < alpha}.}
\item{\code{loadings}}{One row per item with columns \code{Item}, \code{Observed},
\code{Expected_low}, \code{Expected_high}, \code{Flagged} (\code{"below"} / \code{"above"} /
\code{""}). With \code{p_value = TRUE}, columns \code{p_loading} and \code{padj_loading}
are added and \code{Flagged} reflects \code{padj_loading < alpha} (direction
from the sign of the deviation from the simulated mean).}
}
Each element is a \code{knitr_kable} (when \code{output = "kable"}) or a data.frame
(when \code{output = "dataframe"}).
}
\description{
Fits the observed one-factor categorical CFA to \code{data} and compares its
fit indices and per-item standardized loadings against the simulated null
distribution produced by \code{\link{RMdimCFACutoff}}. Returns a list of
two tables: model-fit indices and per-item loadings, each with the observed
value, the expected reference from the simulation, and a flag.
}
\details{
\strong{Bootstrap p-values.} The per-comparison statistic is the residual
studentised by the bootstrap mean and SD. Marginal p-values are Monte-Carlo,
\code{(1 + count) / (B + 1)}, so they can be no smaller than \code{1 / (B + 1)}.
\code{correction = "fwer"} uses the Westfall-Young studentised-max step-down,
which exploits the bootstrap dependence among the statistics (Ferreira,
2024); it is liberal when the simulation is small, so at least 1000
\code{iterations} in \code{\link[=RMdimCFACutoff]{RMdimCFACutoff()}} are recommended (a warning is issued
below that). These p-values are model-conditional and sample-size-sensitive
and are reported alongside the simulated expected ranges, not in place of
them.
}
\section{Multiple comparisons}{

The marginal p-value controls the error rate of a \emph{single} comparison: for
one item (or item pair) decided on in advance it is the relevant value. But
scanning all \emph{k} comparisons and flagging whichever fall below \code{alpha} tests
\emph{k} hypotheses at once, so the chance of at least one false flag inflates to
roughly \eqn{1 - (1 - \alpha)^k} (e.g. about 34\% for \emph{k} = 8 at
\code{alpha = 0.05}) -- even when every marginal p-value is correctly calibrated.
The corrected (adjusted) p-value controls this: \code{correction = "fwer"} bounds
the probability of \emph{any} false flag (strict, lower power), while \code{"fdr_bh"} /
\code{"fdr_by"} bound the expected \emph{proportion} of false flags among those raised
(a more lenient middle ground). Rule of thumb: use the marginal p-value for a
single pre-specified comparison, and a corrected p-value when screening the
whole table -- the usual workflow.
}

\examples{
\donttest{
if (requireNamespace("lavaan", quietly = TRUE) &&
    requireNamespace("eRm", quietly = TRUE)) {
  data("raschdat1", package = "eRm")
  sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 50,
                        parallel = FALSE, seed = 1)
  tabs <- RMdimCFA(raschdat1[, 1:8], cutoff = sim)
  tabs$fit
  tabs$loadings

  # Bootstrap p-values with family-wise (Westfall-Young) correction
  # (use iterations >= 1000 in real analyses for stable p-values)
  RMdimCFA(raschdat1[, 1:8], cutoff = sim, p_value = TRUE)
}
}

}
\references{
Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number
of hypotheses simultaneously. \emph{Journal of Statistical Theory and Practice,
19}(6). \doi{10.1007/s42519-024-00412-4}

Westfall, P. H., & Young, S. S. (1993). \emph{Resampling-Based Multiple Testing}.
Wiley.
}
\seealso{
\code{\link{RMdimCFACutoff}}, \code{\link{RMdimCFAPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cicc_plot.R
\name{RMitemICCPlot}
\alias{RMitemICCPlot}
\title{Conditional Item Characteristic Curves}
\usage{
RMitemICCPlot(
  data,
  dif_var = NULL,
  method = c("quantile", "width", "score", "manual", "cut"),
  class_intervals = 4,
  score_breaks = NULL,
  ci = TRUE,
  error_band = FALSE,
  conf_level = 0.95,
  min_n = 8L,
  items = NULL,
  output = c("patchwork", "list")
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses (non-negative
integers, 0-based). One column per item, one row per person.}

\item{dif_var}{Optional vector of length \code{nrow(data)} (factor or coercible
to factor) defining a DIF grouping variable. When supplied, observed
averages are shown separately per group and the partial-gamma DIF
coefficient (\code{iarm::partgam_DIF()}, as in \code{\link[=RMdifGamma]{RMdifGamma()}}) is annotated
per panel. Default \code{NULL} (no DIF).}

\item{method}{How the observed means are grouped along the total score
(see \emph{Class intervals} in Details). One of \code{"quantile"} (default):
\code{class_intervals} groups of approximately equal numbers of respondents;
\code{"width"}: \code{class_intervals} equal-width intervals over the observed
total-score range; \code{"score"}: every observed total score is its own
group; or \code{"manual"}: groups defined by \code{score_breaks}. \code{"cut"} is
accepted as a legacy alias for \code{"quantile"}.}

\item{class_intervals}{Integer >= 2. Number of class intervals for
\code{method = "quantile"} and \code{method = "width"}. Default \code{4}. Ignored
(without an error) by \code{method = "score"} and \code{method = "manual"}.}

\item{score_breaks}{Integer vector for \code{method = "manual"}: the total
scores at which a new group \emph{starts}, in increasing order. For
example, \code{score_breaks = c(3, 6, 8)} on a 0-9 scale defines the groups
0-2, 3-5, 6-7, and 8-9 (the endpoints are added automatically).
Serves the same purpose as \code{lower.groups} in RASCHplot's \code{CICCplot()},
without the leading zero. Default \code{NULL}.}

\item{ci}{Logical. Draw confidence intervals (error bars) on the observed
class-interval means. Default \code{TRUE}.}

\item{error_band}{Logical. Add a shaded band around the model-expected
curve: the model's interval for the \emph{observed mean} at each total
score, \eqn{E \pm z \sqrt{\mathrm{Var}/n_r}} (\code{n_r} = number of
respondents at that total score), as in RASCHplot's \code{CICCplot}.
Complementary to \code{ci}, not a replacement -- see Details. Default
\code{FALSE}.}

\item{conf_level}{Numeric in (0, 1). Confidence level for the observed
error bars and the model band. Default \code{0.95}.}

\item{min_n}{Integer. A group-by-interval cell needs at least this many
respondents to contribute an observed point + CI; sparser cells are
skipped. Default \code{8} (the package's per-cell stability floor).}

\item{items}{Optional character or integer vector selecting which items to
plot. The model is always fitted on all items; only the rendering is
filtered. Default \code{NULL} (all items).}

\item{output}{One of \code{"patchwork"} (default) -- composite patchwork figure
-- or \code{"list"} -- a named list of per-item \code{ggplot} objects.}
}
\value{
Either a \code{patchwork}/\code{ggplot} composite (default) or a named list
of per-item \code{ggplot} objects (\code{output = "list"}). In DIF mode the per-item
partial-gamma table is attached as \code{attr(., "dif_gamma")}.
}
\description{
Faceted panel of Conditional Item Characteristic Curves (cICCs), one per
item. Each panel shows the model-expected conditional item score
\eqn{E[X_i \mid R = r]} against the total score \eqn{r}, together with the
average observed item score within class intervals of the total score and
their confidence intervals. When a \code{dif_var} is supplied, observed averages
are computed separately per group, turning each panel into a visual DIF
check, and the partial-gamma DIF magnitude is reported per item.
}
\details{
The model curve and its conditional variance come from the shared CML
engine (CML item thresholds via \code{psychotools} and the exact conditional
distribution of each item score given the total score, via elementary
symmetric functions). The conditional-ICC approach follows Buchardt,
Christensen & Jensen (2023) and their \code{RASCHplot} package; the implementation
here is native to easyRasch2's CML/WLE engine.

\strong{Conditioning.} The expected curve is the exact conditional
expectation of the item score given the total score (it accounts for the
item being part of the total), not a marginal ICC.

\strong{Class intervals.} Following Buchardt et al. (2023), the empirical
item means can be shown for each value of the total score or for each
value of the \emph{grouped} total score; all grouping happens on the
total-score scale (never on an estimated latent score). Four grouping
rules are available. \code{method = "quantile"} (the default) splits the
observed total scores into \code{class_intervals} bins of approximately equal
numbers of respondents, using common boundaries so DIF groups share the
x-axis -- the same approach as \code{iarm::ICCplot()}'s class intervals
(\code{Hmisc::cut2(g = ...)}). \code{method = "width"} instead splits the observed
score range into \code{class_intervals} intervals of equal width, which reads
naturally on the score axis but can leave sparse intervals (pruned by
\code{min_n}). \code{method = "score"} uses every observed total score as its own
group -- the maximal-resolution display, sensible for short scales, with
\code{min_n} pruning thinly-populated scores. \code{method = "manual"} places the
group boundaries exactly where you say via \code{score_breaks} (RASCHplot's
\code{lower.groups} concept). Each bin contributes one observed point at its
mean total score. If the score distribution is too sparse to form
\code{class_intervals} distinct bins, the quantile and width methods fall
back to score-level points.

\strong{Confidence intervals.} Observed error bars use the normal
approximation \eqn{\bar{x}_l \pm z \sqrt{\mathrm{var}(x_l) / n_l}} within
each (group, interval) cell, clamped to the item's score range; cells with
fewer than \code{min_n} respondents are dropped. In DIF mode this makes sparse
group differences visibly uncertain rather than over-interpreted.

\strong{The model band (\code{error_band}).} The shaded band is drawn around
the model-expected curve at \eqn{E_{ri} \pm z \sqrt{V_{ri}/n_r}}, where
\eqn{E_{ri}} and \eqn{V_{ri}} are the exact conditional mean and variance
of the item score given total score \eqn{r} (model quantities), and
\eqn{n_r} is the \emph{observed} number of respondents at that total
score. If the Rasch model is true, this is where the empirical mean item
score at total score \eqn{r} should fall, given how many people actually
sit at that score; an observed diamond outside the band is localized
graphical misfit. The band is deliberately the standard error of the
\emph{mean} (\eqn{\sqrt{V/n_r}}), not the far wider individual-response
SD band, which would flag nothing. Its width therefore tracks the data:
narrow where many respondents sit (a strict test), wide in sparse score
regions (a lenient one), with gaps where \eqn{n_r = 0} and collapsing to
the curve at the deterministic extremes (total score 0 and the maximum).
It is complementary to, not a replacement for, the \code{ci} error bars: the
band is the model-implied uncertainty of the observed mean per raw
score; the error bars are the sample-based uncertainty of the observed
group means.

\strong{DIF magnitude.} The annotated partial gamma (Bjorner et al., 1998)
is the association between the item score and the group conditional on the
rest score -- a non-parametric effect size, complementary to the
total-score-conditional visual. It is the same statistic as \code{\link[=RMdifGamma]{RMdifGamma()}}.
}
\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE) &&
    requireNamespace("patchwork", quietly = TRUE) &&
    requireNamespace("iarm", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:2, 200 * 4, replace = TRUE), nrow = 200, ncol = 4)
  )
  colnames(sim_data) <- paste0("Item", 1:4)

  # Default: quantile groups (~equal numbers of respondents per group)
  RMitemICCPlot(sim_data)

  # Equal-width intervals on the total-score scale, with the model band
  RMitemICCPlot(sim_data, method = "width", error_band = TRUE)

  # Every total score as its own group (short scales)
  RMitemICCPlot(sim_data, method = "score")

  # Manual grouping: new groups start at total scores 3, 5, and 7
  RMitemICCPlot(sim_data, method = "manual", score_breaks = c(3, 5, 7))
}
}

}
\references{
Buchardt, A.-S., Christensen, K. B., & Jensen, S. N. (2023). Visualizing
Rasch item fit using conditional item characteristic curves in R.
\emph{Psychological Test and Assessment Modeling, 65}(2), 206-219.

Andersen, E. B. (1995). Polytomous Rasch models and their estimation. In
G. H. Fischer & I. W. Molenaar (Eds.), \emph{Rasch Models: Foundations, Recent
Developments, and Applications} (pp. 271-291). Springer. (Conditional
expectation of item scores given the total score; formula 15.22.)

Bjorner, J. B., Kreiner, S., Ware, J. E., Damsgaard, M. T., & Bech, P.
(1998). Differential item functioning in the Danish translation of the
SF-36. \emph{Journal of Clinical Epidemiology, 51}(11), 1189-1202.
}
\seealso{
\code{\link{RMdifGamma}}, \code{\link{RMitemRestscore}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/person_parameters.R
\name{RMpersonParameters}
\alias{RMpersonParameters}
\title{Person Locations for a Rasch / Partial Credit Model}
\usage{
RMpersonParameters(
  data,
  method = c("WLE", "EAP"),
  item_params = NULL,
  estimator = c("CML", "MML"),
  theta_range = c(-10, 10),
  prior_mean = 0,
  prior_sd = NULL,
  output = c("kable", "dataframe", "ggplot", "file"),
  filename = NULL
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be
scored starting at 0 (non-negative integers). Missing values (\code{NA})
are allowed.}

\item{method}{Character. \code{"WLE"} (default) for Warm's Weighted
Likelihood Estimate (lower bias than MLE; Warm, 1989), or \code{"EAP"}
for the Expected A Posteriori estimate under a normal prior.}

\item{item_params}{Optional pre-specified item parameters, e.g. for
anchoring or equating. Either a named list of Andrich-threshold
vectors (one per item) or the long-format data.frame returned by
\code{\link[=RMitemParameters]{RMitemParameters()}}. When \code{NULL} (default) item parameters are
estimated from \code{data}.}

\item{estimator}{Character. How item parameters are estimated when
\code{item_params} is \code{NULL}: \code{"CML"} (default, via \pkg{psychotools}) or
\code{"MML"} (via \pkg{mirt}). Ignored when \code{item_params} is supplied.}

\item{theta_range}{Numeric length 2. Search range for the WLE root
and bounds for the EAP quadrature grid. Default \code{c(-10, 10)}. Only
if the WLE root falls outside this range is the boundary returned.}

\item{prior_mean}{Numeric. Mean of the normal prior used by
\code{method = "EAP"}. Default \code{0}, the natural choice when item
parameters are centred at mean difficulty zero. Ignored for WLE.}

\item{prior_sd}{Numeric or \code{NULL}. Standard deviation of the normal
prior used by \code{method = "EAP"}. When \code{NULL} (default) it is
estimated from the data by marginal maximum likelihood, holding the
item parameters fixed. Supply a number (e.g. \code{1} for a standard
\code{N(0, 1)} prior, or a value carried over from a reference sample
for equating) to fix it. Ignored for WLE.}

\item{output}{Character. \code{"kable"} (default) for a formatted
\code{\link[knitr:kable]{knitr::kable()}} table, \code{"dataframe"} for the underlying
data.frame, \code{"ggplot"} for a histogram of the estimated person
locations, or \code{"file"} to write the data.frame to a CSV at
\code{filename} (the data.frame is also returned invisibly).}

\item{filename}{Character. Path to the CSV file to write when
\code{output = "file"}. Required in that case; ignored otherwise.}
}
\value{
For \code{output = "dataframe"}, a data.frame with one row per respondent
(in input order; respondents with no responses are dropped with a
message) and columns \code{theta} (the person location), \code{sem}
(standard error of measurement), \code{sum_score}, \code{n_answered} (number of
non-missing responses), and \code{extreme} (logical: a minimum or maximum
possible score given the items answered). For \code{output = "kable"}, the
same content as a \code{knitr_kable} object. For \code{output = "ggplot"}, a
histogram of \code{theta}.
}
\description{
Estimates a person location (theta) and its standard error of
measurement (SEM) for every respondent. Estimation is performed
directly on each person's observed response pattern, so partial
missingness is handled correctly: two respondents with the same sum
score on different subsets of items receive different estimates. This
avoids the sum-score lookup used by \code{\link[=RMscoreSE]{RMscoreSE()}}, which assumes
complete data.
}
\details{
Item parameters are obtained once (by CML or MML, or taken from
\code{item_params}) and treated as fixed. Each person's theta is then
estimated from the items they answered.

\strong{WLE} solves Warm's weighted-likelihood score equation
\verb{l'(theta) + J(theta) / (2 I(theta)) = 0} by bracketed root finding,
where the bias-correction term \verb{J / (2 I)} keeps the equation solvable
at the boundaries. The SEM is \code{1 / sqrt(I(theta))}. Unlike the MLE,
Warm's estimator therefore yields \emph{finite} locations for the minimum
and maximum possible scores (a sensible step beyond the next-most
extreme score rather than +/-Inf; cf. Warm, 1989), matching
established implementations such as \pkg{catR} and \pkg{TAM}. Such
scores are still marked in the \code{extreme} column because they are
extrapolated and carry large standard errors. Only if the root lies
outside \code{theta_range} is the boundary returned with \code{NA} SEM.

\strong{EAP} integrates the pattern likelihood against a normal prior
over a quadrature grid; the estimate is the posterior mean and the
SEM is the posterior standard deviation. Unlike WLE, EAP is finite at
extreme scores because the prior shrinks them inward, at the cost of
depending on the assumed prior. The prior actually used -- including
the marginal-ML estimate of \code{prior_sd} when it is left \code{NULL} -- is
reported in the kable caption and attached to the result as
\code{attr(result, "prior")}.
}
\examples{
\donttest{
set.seed(1)
dat <- as.data.frame(
  matrix(sample(0:2, 200 * 6, replace = TRUE), nrow = 200, ncol = 6)
)
colnames(dat) <- paste0("Item", 1:6)
# Introduce some missingness
dat[cbind(sample(200, 30), sample(6, 30, replace = TRUE))] <- NA

# Default: WLE person locations
RMpersonParameters(dat, output = "dataframe") |> head()

# EAP with a data-estimated prior SD
eap <- RMpersonParameters(dat, method = "EAP", output = "dataframe")
attr(eap, "prior")

# EAP with a fixed N(0, 1) prior
RMpersonParameters(dat, method = "EAP", prior_sd = 1, output = "dataframe") |>
  head()

# Write the person-location table to a CSV (also returned invisibly)
RMpersonParameters(dat, output = "file",
                   filename = tempfile(fileext = ".csv"))
}
}
\references{
Warm, T. A. (1989). Weighted likelihood estimation of ability in item
response theory. \emph{Psychometrika, 54}(3), 427-450.
\doi{10.1007/BF02294627}

Bock, R. D., & Mislevy, R. J. (1982). Adaptive EAP estimation of
ability in a microcomputer environment. \emph{Applied Psychological
Measurement, 6}(4), 431-444. \doi{10.1177/014662168200600405}

Magis, D. (2015). A note on weighted likelihood and Jeffreys modal
estimation of proficiency levels in polytomous item response models.
\emph{Psychometrika, 80}(1), 200-204. \doi{10.1007/s11336-013-9378-5}
}
\seealso{
\code{\link[=RMitemParameters]{RMitemParameters()}}, \code{\link[=RMscoreSE]{RMscoreSE()}}, \code{\link[=RMreliability]{RMreliability()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/infitcutoff_plot.R
\name{RMitemInfitPlot}
\alias{RMitemInfitPlot}
\alias{RMitemInfitCutoffPlot}
\title{Plot Distribution of Simulated Infit and Outfit MSQ Values}
\usage{
RMitemInfitPlot(simfit, data, statistic = "infit")

RMitemInfitCutoffPlot(...)
}
\arguments{
\item{simfit}{The return value of \code{\link{RMitemInfitCutoff}} (a list with
components \code{results}, \code{item_cutoffs}, \code{actual_iterations}, \code{sample_n}, and
\code{item_names}).}

\item{data}{Optional. A data.frame or matrix of item responses for computing
and overlaying observed conditional item fit values. Items must be scored
starting at 0 (non-negative integers). When provided, the plot includes
orange diamond markers for the observed infit/outfit MSQ alongside the
simulated distribution, plus segment summaries from the cutoff intervals.}

\item{statistic}{Character string. Either \code{"infit"} (default) to show only the
infit panel, \code{"outfit"} to show only the outfit panel, or \code{"both"} to show
infit and outfit side by side (requires the \code{patchwork} package when
\code{data} is supplied).}

\item{...}{Arguments passed on to \code{RMitemInfitPlot()}.}
}
\value{
A \code{ggplot} object (or a \code{patchwork} object when \code{statistic = "both"}
and \code{data} is supplied).
}
\description{
Visualises the distribution of simulation-based conditional item fit values
from \code{\link{RMitemInfitCutoff}}, optionally overlaying observed item fit
from the original data.
}
\details{
Uses \code{ggdist::stat_dotsinterval()} (when \code{data} is not supplied) or
\code{ggdist::stat_dots()} (when \code{data} is supplied) with
\code{point_interval = "median_hdci"} and \code{.width = c(0.66, 0.999)}.

When \code{data} is \strong{not} supplied, the function plots the simulated MSQ
distributions as dot-interval plots using \code{ggdist::stat_dotsinterval()} with
median and Highest Density Continuous Interval (HDCI) summaries, faceted by
statistic (InfitMSQ / OutfitMSQ).

When \code{data} \strong{is} supplied, the function:
\enumerate{
\item Fits a Rasch / Partial Credit model by CML via
\code{psychotools::pcmodel()} (a dichotomous item is a 2-category PCM)
and computes observed conditional infit and outfit MSQ via
\code{iarm::out_infit()}.
\item Overlays observed fit values as orange diamond markers on the
simulated distributions.
\item Shows per-item cutoff intervals (from \code{simfit$item_cutoffs}) as
black line segments, with thicker segments for the 66\% HDCI range and
black dots for the median.
}

The \code{ggplot2}, \code{ggdist}, and optionally \code{patchwork} packages must be
installed (they are in Suggests, not Imports).

\code{RMitemInfitCutoffPlot()} is a deprecated alias for \code{RMitemInfitPlot()},
retained for backward compatibility with code written against easyRasch2
0.8.0. It warns and forwards to \code{RMitemInfitPlot()}.
}
\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE) &&
    requireNamespace("ggdist", quietly = TRUE) &&
    requireNamespace("ggplot2", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_data) <- paste0("Item", 1:10)

  # Run simulation
  cutoff_res <- RMitemInfitCutoff(sim_data, iterations = 100,
                                  parallel = FALSE, seed = 42)

  # Simulated distribution only (infit + outfit faceted)
  RMitemInfitPlot(cutoff_res)

  # With observed fit overlaid (infit only, the default)
  RMitemInfitPlot(cutoff_res, data = sim_data)

  # Both infit and outfit panels side by side
  if (requireNamespace("patchwork", quietly = TRUE)) {
    RMitemInfitPlot(cutoff_res, data = sim_data, statistic = "both")
  }
}
}
}
\seealso{
\code{\link{RMitemInfitCutoff}}, \code{\link{RMitemInfit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/locdep_q3_plot.R
\name{RMlocdepQ3Plot}
\alias{RMlocdepQ3Plot}
\title{Plot Distribution of Simulated \eqn{Q_3} Residual Correlations}
\usage{
RMlocdepQ3Plot(simfit, data, items = NULL, n_pairs = NULL)
}
\arguments{
\item{simfit}{The return value of \code{\link{RMlocdepQ3Cutoff}} (a list
with components \code{pair_results}, \code{pair_cutoffs}, \code{actual_iterations},
\code{sample_n}, and \code{item_names}).}

\item{data}{Optional. A data.frame or matrix of item responses for
computing and overlaying observed \eqn{Q_3} values. Items must be scored
starting at 0 (non-negative integers). When provided, the plot
includes orange diamond markers for the observed \eqn{Q_3} alongside the
simulated distribution, plus segment summaries from the cutoff
intervals.}

\item{items}{Optional character vector of item names to include in the
plot. Only item pairs where \strong{both} items are in this vector will be
shown. When \code{NULL} (default), all item pairs are plotted.}

\item{n_pairs}{Optional positive integer. When supplied, only the
\code{n_pairs} item pairs with the largest \strong{deviation} from the simulated
null are plotted, sorted by \verb{|observed Q3 - median(simulated Q3 per pair)|} descending when \code{data} is supplied, or by
\verb{|median(simulated Q3 per pair)|} otherwise. Applied \emph{after} the
\code{items} filter when both are supplied. Values larger than the number
of available pairs are silently capped.}
}
\value{
A named list of two \code{ggplot} objects (mirroring the \verb{$matrix} /
\verb{$pairs} structure of \code{\link{RMlocdepQ3}}'s table output):
\describe{
\item{\verb{$pairs}}{the per-pair plot described below (always returned).}
\item{\verb{$matrix}}{a lower-triangle tile heatmap of the \strong{observed} \eqn{Q_3}
matrix, with pairs above the global dynamic cut-off outlined. This
needs the observed data, so it is \code{NULL} (with a message) when \code{data}
is not supplied. When \code{items} is given, the heatmap is subset to those
items; \code{n_pairs} does not apply to it.}
}
}
\description{
Visualises the distribution of simulation-based Yen's \eqn{Q_3} residual
correlations per item pair from \code{\link{RMlocdepQ3Cutoff}},
optionally overlaying observed \eqn{Q_3} values computed from real data via
\code{mirt::residuals(..., type = "Q3")}.
}
\details{
Uses \code{ggdist::stat_dotsinterval()} (when \code{data} is not supplied) or
\code{ggdist::stat_dots()} (when \code{data} is supplied) with
\code{point_interval = "median_hdci"} and \code{.width = c(0.66, 0.95, 0.99)}.

The \verb{$pairs} plot shows one row per item pair (labelled as "Item1 - Item2").
Only
the upper triangle of the \eqn{Q_3} matrix is plotted (pairs are unordered
under symmetric \eqn{Q_3}, unlike partial gamma which is direction-dependent).

When \code{data} is \strong{not} supplied, the function plots the simulated Q3
distributions as dot-interval plots using \code{ggdist::stat_dotsinterval()}
with median and Highest Density Continuous Interval (HDCI) summaries.

When \code{data} \strong{is} supplied, the function:
\enumerate{
\item Computes observed \eqn{Q_3} residual correlations under the same
estimator used to build \code{simfit} (its \verb{$estimator}: CML/WLE by
default, or MML via \code{mirt}).
\item Overlays observed \eqn{Q_3} values as orange diamond markers on the
simulated distributions.
\item Shows per-pair cutoff intervals (from \code{simfit$pair_cutoffs})
as black line segments, with thicker segments for the 66\\%
interval and black dots for the median.
}

The \code{ggplot2}, \code{ggdist}, \code{mirt}, and \code{scales} packages must be
installed (most are in Suggests, not Imports).
}
\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE) &&
    requireNamespace("ggdist", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_data) <- paste0("Item", 1:10)

  # Run simulation (use more iterations, e.g. 500+, in real analyses)
  cutoff_res <- RMlocdepQ3Cutoff(sim_data, iterations = 50,
                                 parallel = FALSE, seed = 42)

  # Simulated distribution only
  RMlocdepQ3Plot(cutoff_res)

  # With observed Q3 overlaid
  RMlocdepQ3Plot(cutoff_res, data = sim_data)

  # Top 10 pairs by departure from null
  RMlocdepQ3Plot(cutoff_res, data = sim_data, n_pairs = 10)
}
}
}
\seealso{
\code{\link{RMlocdepQ3}}, \code{\link{RMlocdepQ3Cutoff}},
\code{\link{RMlocdepGammaPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ld_partgam.R
\name{knit_print.RMlocdepGamma}
\alias{knit_print.RMlocdepGamma}
\title{knitr knit_print method for RMlocdepGamma kable output}
\usage{
\method{knit_print}{RMlocdepGamma}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"RMlocdepGamma"}.}

\item{...}{Further arguments passed to \code{knitr::asis_output()}.}
}
\value{
A \code{knit_asis} character object.
}
\description{
Inside a knitr / Quarto / R Markdown chunk, returns the pre-combined
two-table asis string so pandoc renders them as two distinct pipe
tables. Outside knitr, R's normal dispatch falls back to
\code{print.RMlocdepGamma()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dif_partgam.R
\name{run_single_partgam_sim}
\alias{run_single_partgam_sim}
\title{Run a single partial gamma DIF simulation iteration}
\usage{
run_single_partgam_sim(seed, data_list)
}
\arguments{
\item{seed}{Integer seed for reproducibility.}

\item{data_list}{List produced inside \code{\link[=RMdifGammaCutoff]{RMdifGammaCutoff()}}.}
}
\value{
A data.frame with columns \code{Item} and \code{gamma}, or a character string
on failure.
}
\description{
Run a single partial gamma DIF simulation iteration
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/local_dependence.R
\name{run_single_q3_sim}
\alias{run_single_q3_sim}
\title{Run a single Q3 simulation iteration}
\usage{
run_single_q3_sim(seed, data_list)
}
\arguments{
\item{seed}{Integer seed for reproducibility.}

\item{data_list}{List produced inside \code{\link[=RMlocdepQ3Cutoff]{RMlocdepQ3Cutoff()}}.}
}
\value{
A list with \code{mean} and \code{max} Q3, or a character string on failure.
}
\description{
Run a single Q3 simulation iteration
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reliability.R
\name{RMUreliability}
\alias{RMUreliability}
\title{Relative Measurement Uncertainty (RMU)}
\usage{
RMUreliability(input_draws, level = 0.95, verbose = FALSE)
}
\arguments{
\item{input_draws}{Numeric matrix or data.frame of draws. Rows are
subjects; columns are draws. Must have at least two columns; ideally many.}

\item{level}{Numeric in (0, 1). Width of the HDCI returned. Default \code{0.95}.}

\item{verbose}{Logical. Print summary information about the input. Default
\code{FALSE}.}
}
\value{
A 1-row data.frame with columns \code{rmu_estimate}, \code{hdci_lowerbound},
\code{hdci_upperbound}, plus the \code{.width}/\code{.point}/\code{.interval} metadata
columns added by \code{ggdist::mean_hdci()}.
}
\description{
Bayesian-style reliability estimate (Bignardi, Kievit & Bürkner, 2025)
computed from a matrix of posterior or plausible-value draws. The columns of
\code{input_draws} are split at random into two halves; reliability is the
Pearson correlation across persons of paired columns from the two halves,
summarised across pairs as a posterior mean with HDCI.
}
\details{
Adapted (with permission, GPL-2/3) from
\url{https://github.com/giac01/gbtoolbox/blob/main/R/reliability.R}.

The function silently returns 0 for any column pair where either side has
zero variance (the correlation is undefined there).

Requires the \code{ggdist} package (Suggests).
}
\references{
Bignardi, G., Kievit, R., & Bürkner, P. C. (2025). A general method for
estimating reliability using Bayesian Measurement Uncertainty. \emph{PsyArXiv}.
\doi{10.31234/osf.io/h54k8_v1}
}
\seealso{
\code{\link[=RMreliability]{RMreliability()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cfa_cutoff.R
\name{RMdimCFAPlot}
\alias{RMdimCFAPlot}
\title{Plot observed CFA fit and loadings against the simulated null}
\usage{
RMdimCFAPlot(simfit, data, percentile = NULL)
}
\arguments{
\item{simfit}{The list returned by \code{\link{RMdimCFACutoff}}.}

\item{data}{The item-response data the CFA was run on (the same items used
for the cutoff). Required: the observed values are computed from it.}

\item{percentile}{Numeric in (50, 100) or \code{NULL}. When supplied, the
cutoffs and flags are recomputed at this percentile from the stored
simulated distributions (no re-simulation). When \code{NULL} (default), the
percentile from the original \code{RMdimCFACutoff()} call is reused.}
}
\value{
A named list of two \code{ggplot} objects:
\describe{
\item{\code{loadings}}{Per-item standardized loadings: simulated distribution
(dots), expected interval, and the observed loading as a diamond
(red when flagged).}
\item{\code{fit}}{Faceted CFI / RMSEA / SRMR simulated distributions with the
observed value and cutoff overlaid.}
}
}
\description{
Returns two figures (in a list) comparing the observed one-factor CFA to
the simulated null distribution from \code{\link{RMdimCFACutoff}}:
a per-item standardized-loadings plot (observed marker against each item's
simulated distribution and expected range, in the style of
\code{\link{RMitemInfitPlot}}), and a faceted plot of the CFI / RMSEA
/ SRMR distributions with the observed value overlaid.
}
\examples{
\donttest{
if (requireNamespace("lavaan", quietly = TRUE) &&
    requireNamespace("ggplot2", quietly = TRUE) &&
    requireNamespace("eRm", quietly = TRUE)) {
  data("raschdat1", package = "eRm")
  sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 50,
                        parallel = FALSE, seed = 1)
  plots <- RMdimCFAPlot(sim, data = raschdat1[, 1:8])
  plots$loadings
  plots$fit
}
}

}
\seealso{
\code{\link{RMdimCFACutoff}}, \code{\link{RMdimCFA}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/infit_cutoff.R
\name{RMitemInfitCutoff}
\alias{RMitemInfitCutoff}
\title{Simulation-Based Infit MSQ Cutoff Determination}
\usage{
RMitemInfitCutoff(
  data,
  iterations = 250,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  cutoff_method = "hdci",
  hdci_width = 0.999,
  dgp = c("resample", "conditional")
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Only complete cases (rows without
any \code{NA}) are used.}

\item{iterations}{Integer. Number of simulation iterations (default 250).}

\item{parallel}{Logical. Use parallel processing via \code{mirai} if available
(default \code{TRUE}).}

\item{n_cores}{Integer or \code{NULL}. Number of parallel workers. When \code{NULL},
\code{getOption("mc.cores")} is checked first. If neither is set and
\code{parallel = TRUE}, a warning is issued and execution falls back to
sequential (single core) processing.}

\item{verbose}{Logical. Show a progress bar (default \code{FALSE}).}

\item{seed}{Integer or \code{NULL}. Random seed for reproducibility.}

\item{cutoff_method}{Character string specifying how cutoff intervals are
computed. Either \code{"hdci"} (default) for the Highest Density Interval via
\code{ggdist::hdci()}, or \code{"quantile"} for the 2.5th/97.5th percentiles via
\code{stats::quantile()}.}

\item{hdci_width}{Numeric. Width of the HDCI when \code{cutoff_method = "hdci"}.
Default is \code{0.999} (99.9\% HDCI). Ignored when \code{cutoff_method = "quantile"}.}

\item{dgp}{Character. Data-generating process for the parametric bootstrap.
\code{"resample"} (default) resamples WLE person locations with replacement and
simulates responses under the model (a \emph{marginal} null). \code{"conditional"}
simulates each respondent's pattern from the exact Rasch conditional
distribution given their observed total score, item parameters fixed (a
\emph{conditional} null). Because the conditional infit/outfit statistic is
itself conditional on the total score, \code{"conditional"} is its naturally
matched null. \strong{Experimental.}}
}
\value{
A list with components:
\describe{
\item{\code{results}}{data.frame with columns \code{iteration}, \code{Item},
\code{InfitMSQ}, \code{OutfitMSQ} (one row per item per successful iteration).}
\item{\code{item_cutoffs}}{data.frame with per-item cutoff summaries: \code{Item},
\code{infit_low}, \code{infit_high}, \code{outfit_low}, \code{outfit_high}. Bounds are
computed using the method specified by \code{cutoff_method}.}
\item{\code{actual_iterations}}{Number of successful iterations.}
\item{\code{sample_n}}{Number of complete cases used.}
\item{\code{sample_n_total}}{Number of respondents in the raw input data,
before the complete-case filter.}
\item{\code{sample_has_na}}{Logical. Whether the raw input data contained
any missing values.}
\item{\code{sample_summary}}{Summary statistics of estimated person parameters.}
\item{\code{item_names}}{Character vector of item names from data.}
\item{\code{cutoff_method}}{The method used to compute cutoffs (\code{"hdci"} or
\code{"quantile"}).}
\item{\code{hdci_width}}{The HDCI width used (only meaningful when
\code{cutoff_method = "hdci"}).}
\item{\code{dgp}}{The data-generating process used (\code{"resample"} or
\code{"conditional"}).}
}
}
\description{
Uses parametric bootstrap simulation to determine appropriate cutoff values
for \code{\link{RMitemInfit}}. This function simulates data from a correctly fitting
Rasch model that mimics your data and returns per-item empirical cutoffs.
}
\details{
The generating model is CML item parameters (via \code{psychotools}) with WLE
person locations. For each iteration a dataset is simulated under the chosen
\code{dgp}, the model is refitted by CML (\code{psychotools::pcmodel()}, which handles
dichotomous and polytomous data and is accepted by \code{iarm}), and conditional
infit and outfit MSQ are computed via \code{iarm::out_infit()}. The distribution
of these statistics across iterations provides empirical critical values per
item. Failed iterations (e.g., degenerate simulated data) are silently
discarded.

Parallel processing is provided by the \code{mirai} package (optional). Install
it with \code{install.packages("mirai")} to enable parallelisation.

The \code{iarm} package must be installed (it is in Suggests, not Imports).
}
\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE) &&
    requireNamespace("ggdist", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_data) <- paste0("Item", 1:10)

  # Run 100 iterations sequentially for a quick demo
  cutoff_res <- RMitemInfitCutoff(sim_data, iterations = 100,
                                  parallel = FALSE, seed = 42)
  cutoff_res$item_cutoffs

  # Use the cutoffs in RMitemInfit()
  RMitemInfit(sim_data)
}
}
}
\seealso{
\code{\link{RMitemInfit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/person_fit.R
\name{RMpersonFit}
\alias{RMpersonFit}
\title{Person-Fit Statistics for a Rasch / Partial Credit Model}
\usage{
RMpersonFit(
  data,
  statistics = c("infit", "outfit", "lz"),
  estimator = c("CML", "MML"),
  theta_method = c("WLE", "EAP"),
  iterations = 500L,
  flag_alpha = 0.05,
  flag = c("both", "underfit"),
  zstd = FALSE,
  parallel = FALSE,
  n_cores = NULL,
  seed = NULL,
  output = c("kable", "dataframe", "ggplot", "list")
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be
scored starting at 0 (non-negative integers). Missing values (\code{NA})
are allowed.}

\item{statistics}{Character vector. Which statistics to report; any of
\code{"infit"}, \code{"outfit"}, \code{"lz"}. Defaults to all three.}

\item{estimator}{Character. How item parameters are estimated:
\code{"CML"} (default) or \code{"MML"} (via \pkg{mirt}).}

\item{theta_method}{Character. Person-location estimator used for the
lz statistic: \code{"WLE"} (default) or \code{"EAP"}. Ignored if \code{"lz"} is not
requested.}

\item{iterations}{Integer. Number of Monte-Carlo replications per
person. \code{0} skips resampling and returns statistics only. Default
\code{500}.}

\item{flag_alpha}{Numeric in (0, 1). Significance level for the \code{flagged}
column and the plot highlighting. Default \code{0.05}.}

\item{flag}{Character. Which misfit direction drives flagging for the MSQ
statistics: \code{"both"} (default) flags either direction with a two-sided
p-value; \code{"underfit"} flags only underfit (MSQ > 1, noisy responding)
with a one-sided upper p-value, which is more powerful for detecting it
and ignores (benign) overfit. The reported \code{p_infit} / \code{p_outfit} follow
this choice. lz is one-sided regardless. Overfit can occasionally be
suspicious (e.g. fabricated or copied response patterns), so \code{"both"} is
the default.}

\item{zstd}{Logical. If \code{TRUE}, also report the Wilson-Hilferty
standardized (ZSTD) versions of the MSQ statistics. These are
provided only as a familiar cross-person comparability metric and
are \strong{not} used for inference (see Müller, 2020). Default \code{FALSE}.}

\item{parallel, n_cores}{Logical / integer. Parallelise the resampling
across persons via \pkg{mirai} when available. Default sequential.}

\item{seed}{Optional integer for reproducible resampling.}

\item{output}{Character. \code{"kable"} (default), \code{"dataframe"},
\code{"ggplot"}, or \code{"list"}. For \code{"ggplot"}, a \strong{named list} of
person-fit maps is returned -- one per requested statistic (e.g.
\verb{$infit}, \verb{$outfit}, \verb{$lz}) -- each plotting the statistic against
person location, with
respondents flagged by \emph{that} statistic highlighted and a caption
reporting the assessed sample size and the proportion flagged. For the
MSQ maps the flagged count is split into underfit (MSQ > 1, noisy/erratic
responding) and overfit (MSQ < 1, overly deterministic responding).}
}
\value{
For \code{output = "dataframe"}, a data.frame with one row per respondent
(input order): \code{id}, \code{n_answered}, \code{sum_score}, the requested
statistics (\code{infit_msq}, \code{outfit_msq}, \code{lz}), their resampled
p-values (\code{p_infit}, \code{p_outfit}, \code{p_lz}) when \code{iterations > 0},
\code{flagged}, and -- if \code{zstd = TRUE} -- \code{infit_zstd}, \code{outfit_zstd}.
The \code{flagged} column is \code{TRUE} when the marginal (uncorrected) p-value of
\strong{any} requested statistic is below \code{flag_alpha} for that respondent; it
is therefore a per-person screening flag, not corrected for the number of
respondents tested (so under fit expect about \code{flag_alpha} of respondents
to be flagged by chance). Each \code{output = "ggplot"} map instead colours and
counts respondents flagged by its \strong{own} statistic alone. Extreme scorers
(minimum or maximum possible given the items they answered) receive \code{NA}
statistics and are not assessed. For \code{output = "kable"} the same content as
a \code{knitr_kable}; for \code{output = "ggplot"} the named list of maps described
under that argument; for \code{output = "list"}, a list with both views from a
single computation: \code{fit} (the data.frame) and \code{plots} (the named list of
maps).
}
\description{
Computes per-respondent person-fit statistics and resampling-based
p-values. Three statistics are reported: conditional \strong{infit} and
\strong{outfit} mean-squares (MSQ) -- the magnitude (effect-size) measures
familiar from Rasch analysis -- and the standardized log-likelihood
statistic \strong{lz}. The conditional MSQ statistics use response
probabilities conditional on the total score, which require no person
estimate and are therefore unbiased (Kreiner & Christensen, 2011); they are
computed on each person's observed response pattern, so partial
missingness is handled directly.
}
\details{
Statistical significance is assessed by Monte-Carlo resampling under
the fitted model rather than by an assumed asymptotic distribution.
The asymptotic null distributions of MSQ and lz are known to be
unreliable -- the Wilson-Hilferty (ZSTD) transformation of MSQ in
particular adds nothing once conditional estimation is used (Müller,
2020) -- so resampling provides the valid reference (Sinharay, 2016).

\strong{Conditional MSQ.} For person \eqn{v} with answered items
\eqn{A_v} and total score \eqn{r_v}, the conditional residual is
\eqn{z_{vi} = (x_{vi} - E(X_{vi} \mid R_v = r_v)) /
\sqrt{\mathrm{Var}(X_{vi} \mid R_v = r_v)}}, with conditional moments
obtained from elementary symmetric functions of the answered items'
parameters. Outfit is the unweighted mean of \eqn{z_{vi}^2}; infit is
the information-weighted mean. Because the conditional moments use only
item parameters, no (biased) person estimate enters the residual.

\strong{Interpreting MSQ direction.} Values near 1 indicate fit. MSQ > 1
(\emph{underfit}) means the responses are noisier / more erratic than the
model expects -- the validity-relevant direction, indicating careless or
aberrant responding. MSQ < 1 (\emph{overfit}) means responses are overly
deterministic (Guttman-like); usually benign for score validity, though a
suspiciously perfect pattern can occasionally signal copied or fabricated
data. Use \code{flag = "underfit"} to flag only the former. lz is one-sided:
low (negative) values flag the aberrant (underfit-like) direction.

\strong{lz.} The standardized log-likelihood of the response pattern
evaluated at the estimated person location (Drasgow, Levine &
Williams, 1985); small (negative) values indicate misfit. Because the
location is estimated rather than known, the variance of lz is below 1
and its asymptotic standard-normal null is invalid, which makes a
naive test conservative (Snijders, 2001; Sinharay, 2016). Instead of
applying the analytic lz\* standardization -- derived by Snijders (2001)
for dichotomous items and extended to polytomous / mixed-format items by
Sinharay (2016, 2026) -- easyRasch2 obtains the reference distribution by
resampling with the person location \strong{re-estimated for every
simulated pattern} (see \strong{Resampling}). This reproduces the
ability-estimation effect and is the resampling analogue of lz\*, so the
resampled p-value is well calibrated.

\strong{Resampling.} Each statistic is referenced against the null that
matches it, removing the need to choose a scheme. The conditional MSQ
statistics use patterns sampled \strong{conditional on the person's total
score} -- the exact Rasch-native null, requiring no person estimate and
fully consistent with the (conditional) statistic. lz, which is defined at
the estimated location, uses patterns \strong{simulated at that location and then
scored with the location re-estimated from each simulated pattern}, so the
null carries the same ability-estimation effect as the observed lz (for a
Rasch / PCM model the location is a function of the total score, so this
re-estimation is a cheap score-based lookup). Both schemes are computed per
person over the items actually answered, so partial missingness is handled
by either. The p-value is the proportion of the \code{iterations} replicates at
least as extreme as the observed value. This follows the resampling-based
person-fit approach (Sinharay, 2016) and the bootstrap recommendation of
Müller (2020).
}
\examples{
\donttest{
set.seed(1)
dat <- as.data.frame(
  matrix(sample(0:2, 200 * 8, replace = TRUE), nrow = 200, ncol = 8)
)
colnames(dat) <- paste0("Item", 1:8)

# Conditional infit/outfit MSQ + lz with resampled p-values
RMpersonFit(dat, iterations = 200, output = "dataframe") |> head()

# Person-fit maps: a named list with one plot per statistic
if (requireNamespace("ggplot2", quietly = TRUE)) {
  plots <- RMpersonFit(dat, iterations = 200, output = "ggplot")
  plots$infit    # infit map (each plot's caption reports the \% flagged)
}

# Flag only underfit (noisy responding), the validity-relevant direction
RMpersonFit(dat, iterations = 200, flag = "underfit",
            output = "dataframe") |> head()
}
}
\references{
de la Torre, J., & Deng, W. (2008). Improving person-fit assessment by
correcting the ability estimate and its reference distribution.
\emph{Journal of Educational Measurement, 45}(2), 159-177.
\doi{10.1111/j.1745-3984.2008.00058.x}

Drasgow, F., Levine, M. V., & Williams, E. A. (1985). Appropriateness
measurement with polychotomous item response models and standardized
indices. \emph{British Journal of Mathematical and Statistical Psychology,
38}(1), 67-86. \doi{10.1111/j.2044-8317.1985.tb00817.x}

Kreiner, S., & Christensen, K. B. (2011). Exact evaluation of bias in
Rasch model residuals. \emph{Advances in Mathematics Research, 12},
19-40.

Müller, M. (2020). Item fit statistics for Rasch analysis: can we
trust them? \emph{Journal of Statistical Distributions and Applications,
7}(5). \doi{10.1186/s40488-020-00108-7}

Sinharay, S. (2016). Assessment of person fit using resampling-based
approaches. \emph{Journal of Educational Measurement, 53}(1), 63-85.
\doi{10.1111/jedm.12101}

Sinharay, S. (2016). Asymptotically correct standardization of
person-fit statistics beyond dichotomous items. \emph{Psychometrika,
81}(4), 992-1013. \doi{10.1007/s11336-015-9465-x}

Sinharay, S. (2026). Refining the asymptotically correct
standardization of person-fit statistics for mixed-format tests.
\emph{British Journal of Mathematical and Statistical Psychology}.
\doi{10.1111/bmsp.70049}

Snijders, T. A. B. (2001). Asymptotic null distribution of person fit
statistics with estimated person parameter. \emph{Psychometrika, 66}(3),
331-342. \doi{10.1007/BF02294440}
}
\seealso{
\code{\link[=RMpersonParameters]{RMpersonParameters()}}, \code{\link[=RMitemInfit]{RMitemInfit()}}, \code{\link[=RMitemParameters]{RMitemParameters()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{phq9}
\alias{phq9}
\title{PHQ-9 Depression Screener (NHANES Subsample)}
\format{
A data frame with 600 rows and 12 variables:
\describe{
\item{q1}{Little interest or pleasure in doing things. Integer 0--3.}
\item{q2}{Feeling down, depressed, or hopeless. Integer 0--3.}
\item{q3}{Trouble falling/staying asleep, or sleeping too much.
Integer 0--3.}
\item{q4}{Feeling tired or having little energy. Integer 0--3.}
\item{q5}{Poor appetite or overeating. Integer 0--3.}
\item{q6}{Feeling bad about yourself --- or that you are a failure
or have let yourself or your family down. Integer 0--3.}
\item{q7}{Trouble concentrating on things, such as reading the
newspaper or watching television. Integer 0--3.}
\item{q8}{Moving or speaking so slowly that other people could have
noticed --- or the opposite, being so fidgety or restless that
you have been moving around a lot more than usual. Integer 0--3.}
\item{q9}{Thoughts that you would be better off dead, or of hurting
yourself in some way. Integer 0--3.}
\item{gender}{Self-reported gender, factor with levels
\code{"Female"} and \code{"Male"} (31 respondents with missing values).}
\item{age}{Age in years (integer, range 15--85).}
\item{edu}{Highest educational attainment, factor with levels
\code{"Elementary School"}, \code{"High school"}, \code{"University"}.}
}

Each PHQ-9 item uses a four-point ordinal response scale, scored
0 (\code{"Not at all"}), 1 (\code{"Several days"}), 2 (\code{"More than half the days"}) and 3 (\code{"Nearly every day"}).
}
\source{
U.S. Centers for Disease Control and Prevention, National Center for
Health Statistics. \emph{National Health and Nutrition Examination Survey},
September 2024 release.
\url{https://wwwn.cdc.gov/nchs/nhanes/search/datapage.aspx?Component=Questionnaire&CycleBeginYear=2024}.
NHANES data are released to the public domain by the U.S. federal
government (\url{https://www.cdc.gov/nchs/policy/data-release-policy.html}).
}
\usage{
phq9
}
\description{
A processed subsample of the Patient Health Questionnaire 9-item (PHQ-9)
depression screener from the U.S. National Health and Nutrition
Examination Survey (NHANES), September 2024 release. Six hundred
respondents were drawn at random from the cycle's PHQ-9 module subject
to having complete responses on all nine items, while retaining a
realistic share of respondents with a sum-score of zero (n = 8) so
that floor behaviour can be illustrated in a Rasch analysis.
}
\details{
The dataset is a \emph{processed subsample} intended for teaching and for
the package's worked example; it should not be treated as a canonical
NHANES microdata file. Users wishing to validate against NCHS-published
figures should download the original public-use microdata directly
from the NHANES website (see \emph{Source}).
}
\examples{
data(phq9)
str(phq9)
summary(rowSums(phq9[, 1:9]))
}
\references{
Kroenke, K., Spitzer, R. L., & Williams, J. B. W. (2001). The PHQ-9:
Validity of a brief depression severity measure. \emph{Journal of General
Internal Medicine}, 16(9), 606--613.
\doi{10.1046/j.1525-1497.2001.016009606.x}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reliability.R
\name{RMreliability}
\alias{RMreliability}
\title{Reliability metrics for a Rasch model}
\usage{
RMreliability(
  data,
  conf_int = 0.95,
  draws = 1000,
  rmu_iter = 50,
  estim = "WLE",
  boot = FALSE,
  boot_iter = 200,
  parallel = TRUE,
  n_cores = NULL,
  seed = NULL,
  verbose = FALSE,
  theta_range = c(-10, 10),
  output = "kable"
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers).}

\item{conf_int}{Numeric in (0, 1). HDCI width for both bootstrap CIs and
RMU. Default \code{0.95}.}

\item{draws}{Integer. Number of plausible-value draws drawn from the mirt
model for the RMU calculation. Default \code{1000}. More gives a more stable
RMU; computational cost is mostly linear.}

\item{rmu_iter}{Integer. Number of times \code{\link[=RMUreliability]{RMUreliability()}} is repeated on
the same set of plausible-value draws (each repetition uses a fresh
random column split). Estimates are averaged across repetitions to
stabilise against split-induced variability. Default \code{50}.}

\item{estim}{Character. Theta estimator used by \code{mirt::fscores()} for the
RMU plausible-value seed. One of \code{"WLE"} (default), \code{"EAP"}, \code{"MAP"},
\code{"ML"}. Plausible draws themselves are produced by Metropolis-Hastings.
(PSI and marginal reliability are computed natively and do not use this.)}

\item{boot}{Logical. If \code{TRUE}, run a non-parametric bootstrap to obtain
CIs for PSI and Marginal reliability. Default \code{FALSE}.}

\item{boot_iter}{Integer. Number of bootstrap iterations when
\code{boot = TRUE}. Default \code{200}.}

\item{parallel}{Logical. Use parallel processing via \code{mirai} for the
bootstrap if available. Default \code{TRUE}.}

\item{n_cores}{Integer or \code{NULL}. Number of parallel workers. When \code{NULL},
\code{getOption("mc.cores")} is checked first; if neither is set,
bootstrapping falls back to sequential.}

\item{seed}{Integer or \code{NULL}. Master random seed for reproducibility.}

\item{verbose}{Logical. Print progress messages and a progress bar for
the bootstrap. Default \code{FALSE}.}

\item{theta_range}{Numeric length-2 vector. Theta limits passed to
\code{mirt::fscores()}. Default \code{c(-10, 10)}.}

\item{output}{Character. \code{"kable"} (default) for a formatted
\code{knitr::kable()} table, or \code{"dataframe"} for the underlying data.frame.}
}
\value{
\itemize{
\item If \code{output = "kable"}: a \code{knitr_kable} object with one row per metric.
\item If \code{output = "dataframe"}: a data.frame with columns \code{metric},
\code{estimate}, \code{lower}, \code{upper}, \code{notes}.
}
}
\description{
Computes three reliability indices for a Rasch / partial credit model:
the Person Separation Index (PSI) -- WLE-based separation reliability -- the
marginal reliability (native, CML test information integrated over the
estimated normal latent density), and Relative Measurement Uncertainty (RMU)
via \code{\link[=RMUreliability]{RMUreliability()}} applied to plausible values from \code{mirt::fscores()}.
}
\details{
Confidence intervals for \strong{PSI} and \strong{Marginal} reliability are obtained
by non-parametric bootstrap (resampling respondents; all three indices are
recomputed natively per resample, no model is refitted by \code{mirt}). The RMU
interval is the HDCI of correlations across plausible-value draws, averaged
over \code{rmu_iter} random splits of the draws.

Marginal reliability is the native Green (1984) coefficient,
\eqn{1 - \overline{1/I(\theta)}/\sigma^2}, where the test information
\eqn{I(\theta)} is summed from the CML item parameters and the average error
variance is taken over the estimated normal latent density \eqn{N(0,
\sigma^2)} (\eqn{\sigma} from marginal ML). Unlike \code{mirt::marginal_rxx()},
which assumes \eqn{N(0,1)}, this integrates over the estimated latent
variance, so it is correct on the Rasch logit scale (where \eqn{\sigma}
is typically well above 1, and the \eqn{N(0,1)} assumption
underestimates reliability). It is the model-based complement to the
sample-based PSI; a large gap between the two flags an off-target or
non-normal sample.

PSI is the WLE-based separation reliability,
\eqn{1 - \overline{SEM^2} / \mathrm{Var}(\hat\theta)}, computed from CML item
thresholds (\code{psychotools}) and Warm's WLE person locations / analytic SEMs.
Respondents with extreme (min/max) raw scores are excluded -- their boundary
estimates would inflate the person variance and overstate reliability.
(Earlier versions used \code{eRm::SepRel()} with MLE; the values can differ, most
noticeably for scales with many extreme scorers, e.g. dichotomous items.)

RMU is from Bignardi, Kievit, & Bürkner (2025), modified here to use mirt
plausible values rather than fully Bayesian posterior draws (see Mislevy,
1991, for the plausible-values framework).

Bootstrap iterations that fail to converge are silently dropped.
}
\examples{
\donttest{
if (requireNamespace("ggdist", quietly = TRUE) &&
    requireNamespace("eRm", quietly = TRUE)) {
  set.seed(1)
  RMreliability(eRm::raschdat1[, 1:20], draws = 1000)

  # Bootstrap CI for PSI and Marginal
  # (use more bootstrap iterations, e.g. 200+, in real analyses)
  RMreliability(eRm::raschdat1[, 1:20], draws = 1000,
                boot = TRUE, boot_iter = 25, parallel = FALSE, seed = 42)
}
}
}
\references{
Bignardi, G., Kievit, R., & Bürkner, P. C. (2025). A general method for
estimating reliability using Bayesian Measurement Uncertainty. \emph{PsyArXiv}.
\doi{10.31234/osf.io/h54k8_v1}

Green, B. F., Bock, R. D., Humphreys, L. G., Linn, R. L., & Reckase, M. D.
(1984). Technical Guidelines for Assessing Computerized Adaptive Tests.
\emph{Journal of Educational Measurement, 21}(4), 347–360.
\doi{10.1111/j.1745-3984.1984.tb01039.x}

Mislevy, R. J. (1991). Randomization-Based Inference about Latent Variables
from Complex Samples. \emph{Psychometrika, 56}(2), 177-196.
\doi{10.1007/BF02294457}

Adams, R. J. (2005). Reliability as a measurement design effect.
\emph{Studies in Educational Evaluation, 31}(2), 162-172.
\doi{10.1016/j.stueduc.2005.05.008}
}
\seealso{
\code{\link[=RMUreliability]{RMUreliability()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dif_partgam.R
\name{RMdifGammaCutoff}
\alias{RMdifGammaCutoff}
\title{Simulation-Based Partial Gamma DIF Cutoff Determination}
\usage{
RMdifGammaCutoff(
  data,
  dif_var,
  iterations = 250,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  cutoff_method = "hdci",
  hdci_width = 0.99
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Only complete cases (rows without
any \code{NA}) are used.}

\item{dif_var}{A vector (factor, character, or integer) defining group
membership for DIF analysis. Must have the same length as \code{nrow(data)}.
The actual group labels are used to determine the number of groups and
their relative sizes; during simulation, respondents are randomly assigned
to groups with the same proportions, so there is no true DIF by
construction.}

\item{iterations}{Integer. Number of simulation iterations (default 250).}

\item{parallel}{Logical. Use parallel processing via \code{mirai} if available
(default \code{TRUE}).}

\item{n_cores}{Integer or \code{NULL}. Number of parallel workers. When \code{NULL},
\code{getOption("mc.cores")} is checked first. If neither is set and
\code{parallel = TRUE}, a warning is issued and execution falls back to
sequential (single core) processing.}

\item{verbose}{Logical. Show a progress bar (default \code{FALSE}).}

\item{seed}{Integer or \code{NULL}. Random seed for reproducibility.}

\item{cutoff_method}{Character string specifying how cutoff intervals are
computed. Either \code{"hdci"} (default) for the Highest Density Interval via
\code{ggdist::hdci()}, or \code{"quantile"} for the 2.5th/97.5th percentiles via
\code{stats::quantile()}.}

\item{hdci_width}{Numeric. Width of the HDCI when \code{cutoff_method = "hdci"}.
Default is \code{0.99} (99\\% HDCI). Ignored when
\code{cutoff_method = "quantile"}.}
}
\value{
A list with components:
\describe{
\item{\code{results}}{data.frame with columns \code{iteration}, \code{Item}, and
\code{gamma} (one row per item per successful iteration).}
\item{\code{item_cutoffs}}{data.frame with per-item cutoff summaries: \code{Item},
\code{gamma_low}, \code{gamma_high}. Bounds are computed using the method
specified by \code{cutoff_method}.}
\item{\code{actual_iterations}}{Number of successful iterations.}
\item{\code{sample_n}}{Number of complete cases used.}
\item{\code{sample_n_total}}{Number of respondents in the raw input data,
before removing rows with \code{NA} in \code{data} or \code{dif_var}.}
\item{\code{sample_has_na}}{Logical. Whether \code{data} or \code{dif_var} contained
any missing values.}
\item{\code{sample_summary}}{Summary statistics of estimated person
parameters.}
\item{\code{item_names}}{Character vector of item names from data.}
\item{\code{dif_group_sizes}}{Named integer vector of group sizes used in the
simulation (matches proportions in the observed \code{dif_var}).}
\item{\code{cutoff_method}}{The method used to compute cutoffs (\code{"hdci"} or
\code{"quantile"}).}
\item{\code{hdci_width}}{The HDCI width used (only meaningful when
\code{cutoff_method = "hdci"}).}
}
}
\description{
Uses parametric bootstrap simulation to determine appropriate cutoff values
for partial gamma DIF analysis via \code{\link[iarm]{partgam_DIF}}. Under
a correctly fitting Rasch model where the DIF variable is unrelated to item
responses (i.e., no true DIF), this function generates the expected
distribution of absolute partial gamma values per item, providing empirical
critical values.
}
\details{
For each simulation iteration the function:
\enumerate{
\item Resamples person parameters (thetas) with replacement from the
WLE person locations.
\item Simulates item response data under a Rasch model (dichotomous via
\code{psychotools::rrm()} or polytomous via an internal partial credit
simulator).
\item Creates a random DIF variable by sampling group labels with the
same proportions as the observed \code{dif_var}, so there is \strong{no true DIF}
by construction.
\item Computes partial gamma DIF statistics via
\code{iarm::partgam_DIF()}.
}

The distribution of partial gamma values across iterations provides
empirical critical values per item. Values from real data that fall
outside these bounds suggest DIF that exceeds what would be expected by
chance under a correctly fitting Rasch model. Failed iterations (e.g.,
due to convergence issues or degenerate data) are silently discarded.

The generating model uses CML item thresholds via \code{psychotools::pcmodel()}
(a dichotomous item is a 2-category PCM) and WLE person locations,
consistent with the rest of the package; responses are simulated with
\code{psychotools::rrm()} (dichotomous) or an internal partial credit score
simulator (polytomous).

Parallel processing is provided by the \code{mirai} package (optional). Install
it with \code{install.packages("mirai")} to enable parallelisation.

The \code{iarm} package must be installed (it is in Suggests, not Imports).
}
\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE) &&
    requireNamespace("ggdist", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_data) <- paste0("Item", 1:10)
  dif_sex <- sample(c("male", "female"), 200, replace = TRUE)

  # Run 100 iterations sequentially for a quick demo
  cutoff_res <- RMdifGammaCutoff(sim_data, dif_var = dif_sex,
                                 iterations = 100, parallel = FALSE,
                                 seed = 42)
  cutoff_res$item_cutoffs
}
}
}
\references{
Bjorner, J. B., Kreiner, S., Ware, J. E., Damsgaard, M. T., &
Bech, P. (1998). Differential item functioning in the Danish translation
of the SF-36. \emph{Journal of Clinical Epidemiology, 51}(11), 1189--1202.
\doi{10.1016/S0895-4356(98)00111-5}

Henninger, M., Radek, J., Debelak, R., & Strobl, C. (2025).
Partial credit trees meet the partial gamma coefficient for quantifying
DIF and DSF in polytomous items. \emph{Behaviormetrika, 52}, 221--257.
\doi{10.1007/s41237-024-00252-3}
}
\seealso{
\code{\link[iarm]{partgam_DIF}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_infit.R
\name{.format_cutoff_method_label}
\alias{.format_cutoff_method_label}
\title{Format a human-readable label for the cutoff method}
\usage{
.format_cutoff_method_label(cutoff_method, hdci_width)
}
\arguments{
\item{cutoff_method}{Character. \code{"hdci"}, \code{"quantile"}, or \code{NULL}.}

\item{hdci_width}{Numeric or \code{NULL}. HDCI width (e.g., \code{0.999}).}
}
\value{
A character label, or \code{NULL} if the method is unknown/unset.
}
\description{
Format a human-readable label for the cutoff method
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap_restscore.R
\name{run_boot_restscore_sequential}
\alias{run_boot_restscore_sequential}
\title{Run item-restscore bootstrap iterations sequentially}
\usage{
run_boot_restscore_sequential(
  iterations,
  boot_seeds,
  boot_data_list,
  verbose = FALSE
)
}
\arguments{
\item{iterations}{Number of iterations.}

\item{boot_seeds}{Integer vector of per-iteration seeds.}

\item{boot_data_list}{List of data passed to each worker.}

\item{verbose}{Show progress bar.}
}
\value{
List of raw results (one element per iteration).
}
\description{
Run item-restscore bootstrap iterations sequentially
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dif_tree.R
\name{RMdifTree}
\alias{RMdifTree}
\title{Tree-based DIF analysis with effect-size classification}
\usage{
RMdifTree(
  data,
  covariates,
  model = c("auto", "PCM", "RM"),
  effect_size = c("auto", "MH", "pgamma"),
  purification = c("none", "iterative"),
  p_adj = c("none", "fdr", "bonferroni"),
  thresholds = c(0.21, 0.31),
  alpha = 0.05,
  prune_negligible = FALSE,
  stability = FALSE,
  stability_B = 100L,
  stability_sampler = c("subsampling", "bootstrap"),
  min_n_per_level = 20L,
  on_rescale = c("message", "warning", "stop"),
  output = c("kable", "dataframe", "tree", "plot", "list"),
  ...
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses (non-negative
integers, 0-based). One column per item. Items only -- pass
covariates separately via \code{covariates}.}

\item{covariates}{A data.frame with \code{nrow(data)} rows giving the
DIF covariates. Columns may be numeric (continuous), factor, ordered
factor, or logical. At least one column is required.}

\item{model}{One of \code{"auto"} (default), \code{"PCM"}, or
\code{"RM"}. \code{"auto"} picks RM when the data are dichotomous
(max response = 1) and PCM otherwise.}

\item{effect_size}{One of \code{"auto"} (default), \code{"MH"}
(Mantel-Haenszel), or \code{"pgamma"} (partial gamma). \code{"auto"}
uses MH for RM and partial gamma for PCM.}

\item{purification}{\code{"none"} (default) or \code{"iterative"}.
Iterative purification recomputes the effect size while excluding
items already classified as DIF from the matching score (Bjorner et
al., 1998).}

\item{p_adj}{Multiple-testing adjustment for the partial-gamma
classification: \code{"none"} (default), \code{"fdr"} (Benjamini &
Hochberg), or \code{"bonferroni"}. Ignored for MH (which uses the
ETS test directly).}

\item{thresholds}{Numeric length-2 vector with the partial-gamma
B/C boundaries (default \code{c(0.21, 0.31)}). Ignored for MH (the
ETS Delta-scale boundaries 1.0 / 1.5 are used).}

\item{alpha}{Significance level for the A/C tests. Default
\code{0.05}.}

\item{prune_negligible}{Logical. If \code{TRUE}, splits whose every
item is classified as A are pruned from the tree. Default
\code{FALSE}.}

\item{stability}{Logical. If \code{TRUE}, runs
\code{stablelearner::stabletree()} on the fitted tree to assess
variable-selection and cutpoint stability across resamples.
Default \code{FALSE}.}

\item{stability_B}{Integer. Number of resamples for stability
assessment. Default \code{100}.}

\item{stability_sampler}{One of \code{"subsampling"} (default) or
\code{"bootstrap"}. Subsampling is recommended for tree stability
because bootstrap resamples can drop levels of categorical
covariates, breaking the model fit on small subsamples.}

\item{min_n_per_level}{Integer. Minimum count required for any factor
level in \code{covariates}. Default \code{20}. Set to \code{0} to
disable.}

\item{on_rescale}{One of \code{"message"} (default), \code{"warning"},
or \code{"stop"}. Controls how the function reports the situation
in which one or more items have no responses in their lowest
category within a terminal node of the tree (psychotree silently
rescales those items per node, which leaves the per-node item
parameters on a different metric). \emph{The MH and partial-gamma
effect sizes reported here are computed on the raw responses and
are not affected.} The diagnostic concerns only the per-node item
parameters that are visible via \code{plot(tree)}.}

\item{output}{One of \code{"kable"} (default), \code{"dataframe"},
\code{"tree"}, \code{"plot"}, or \code{"list"}. \code{"kable"}
renders the
per-split per-item effect-size table as a \code{knitr::kable()};
\code{"dataframe"} returns the underlying tidy data.frame;
\code{"tree"} returns the augmented partykit tree object;
\code{"plot"} returns the partykit tree plot with item names on
the terminal-node x-axis (\code{tp_args = list(names = TRUE)});
\code{"list"} returns both views from a single fit, as
\code{list(table = <data.frame>, tree = <tree object>)}.
For full control over the plot, use \code{output = "tree"} and
call \code{plot()} on the result with your own \code{tp_args}.
When
\code{stability = TRUE}, the stability summary is attached as
\code{attr(result, "stability")} (a small data.frame) and
\code{attr(result, "stability_kable")} (pre-rendered kable).}

\item{...}{Additional arguments forwarded to
\code{psychotree::raschtree()} or \code{psychotree::pctree()}, e.g.
\code{minsize}, \code{alpha} for the parameter-instability test.
Note: this \code{alpha} (the tree-fitting argument) is independent
of the \code{alpha} for ETS classification above; pass it via
\code{...}.}
}
\value{
Depending on \code{output}:
\describe{
\item{\code{"kable"}}{A \code{knitr_kable} object with one row per
(split node x item), grouped by node via \code{pack_rows}-style
section headers. The caption summarises model, effect-size
measure, purification, and (if requested) stability.}
\item{\code{"dataframe"}}{A data.frame with one row per (split node
x item): columns \code{NodeID}, \code{Split} (human-readable
description of the split), \code{Variable}, \code{Direction},
\code{Item}, \code{EffectSize}, \code{SE}, \code{Class} (A/B/C),
\code{Flagged} (TRUE for B or C), \code{n_left}, \code{n_right}.}
\item{\code{"tree"}}{The fitted partykit tree object with class
\code{c("RMdifTree", ...)}, with effect-size results stored at
\code{tree$info$effectsize}.}
\item{\code{"plot"}}{A plotted partykit tree.}
\item{\code{"list"}}{A list with both views from a single fit:
\code{table} (the \code{"dataframe"} result, with its attributes)
and \code{tree} (the \code{"tree"} result).}
}

Stability results (when \code{stability = TRUE}) are attached to the
return value as \code{attr(result, "stability")} (data.frame) and
\code{attr(result, "stability_kable")} (pre-rendered kable).
}
\description{
Detects Differential Item Functioning (DIF) by recursively splitting
the sample on one or more covariates using \code{psychotree::raschtree()}
(dichotomous data) or \code{psychotree::pctree()} (polytomous data),
then computes per-split effect-size measures for every item -- the
Mantel-Haenszel odds-ratio (Holland & Thayer, 1986), on the Delta scale
developed at the Educational Testing Service (ETS; Zwick, 2012), for
dichotomous data, or the partial gamma coefficient for polytomous data -- and
classifies them into ETS A/B/C categories (Bjorner et al., 1998). Optionally,
an iterative purification step (Henninger et al., 2025) is applied, and tree
stability across resamples can be assessed via
\code{stablelearner::stabletree()} (Philipp et al., 2018).
}
\details{
Continuous covariates (e.g., age in years) and interactions among
multiple covariates are handled natively by the model-based
recursive-partitioning machinery of \code{partykit} / \code{psychotree}.

\strong{ETS Delta-scale Mantel-Haenszel.} The MH common odds ratio
\eqn{\hat{\alpha}_{MH}} is mapped to the ETS Delta scale via
\eqn{\Delta_{MH} = -2.35 \log \hat{\alpha}_{MH}}. The classification
rules (Holland & Thayer, 1988; Zwick, 2012) are:
\itemize{
\item Class \strong{A}: \eqn{|\Delta_{MH}| < 1} or test of
\eqn{\Delta_{MH} = 0} not rejected at \code{alpha}.
\item Class \strong{C}: \eqn{|\Delta_{MH}| \ge 1.5} and test of
\eqn{|\Delta_{MH}| \le 1} rejected at \code{alpha}.
\item Class \strong{B}: otherwise.
}
easyRasch2 uses the sign convention of \code{effecttree}: positive
\eqn{\Delta} indicates that the item is more difficult for the second
(reference) group.

\strong{Partial gamma.} \code{iarm::partgam_DIF()} provides the gamma
estimate and SE per item. ETS-style classification follows Bjorner et
al. (1998): A if \eqn{|\gamma| < 0.21} or the test of \eqn{\gamma = 0}
is not rejected at \code{alpha}; C if \eqn{|\gamma| > 0.31} and the
test of \eqn{|\gamma| \le 0.21} is rejected at \code{alpha}; B
otherwise.

\strong{Rule-of-thumb caveat.} The A/B/C boundaries (Delta 1.0 / 1.5;
gamma 0.21 / 0.31) are conventions carried over from large-scale
educational testing, not values calibrated to the sample and items at
hand -- in contrast to the simulation-based cutoffs used elsewhere in
this package -- so the classification is best read as a rough
magnitude guide rather than a calibrated test. For a sample-calibrated
partial-gamma DIF test, see \code{\link[=RMdifGamma]{RMdifGamma()}} with an
\code{\link[=RMdifGammaCutoff]{RMdifGammaCutoff()}} object (optionally with \code{p_value = TRUE}).

\strong{Continuous and interaction effects.} The recursive
partitioning step automatically handles continuous covariates (the
parameter-instability test searches for the optimal cutpoint) and
multivariate interactions (later splits are conditional on earlier
ones). To assess sensitivity of variable selection and cutpoints to
the particular sample, set \code{stability = TRUE}.

\strong{Stability assessment.} When \code{stability = TRUE}, the same
tree-fitting call is replayed on \code{stability_B} resamples of the
data. The returned \code{stabletree} object reports, per covariate,
the proportion of resamples in which it was selected at any split,
and (for continuous covariates) the empirical distribution of
cutpoints. Stability is independent of effect-size classification --
a stable split with a negligible effect is still negligible, and a
large effect on an unstable split should be interpreted with care.
Cost is roughly \code{stability_B} times the original fitting time.

\strong{Acknowledgement.} The effect-size machinery -- MH and partial
gamma per node, iterative purification, ETS A/B/C classification --
follows the implementations by Henninger & Radek in the GitHub
packages \emph{raschtreeMH} and \emph{effecttree}. The relevant code
has been adapted here under the MIT licence (see file header).
}
\examples{
\donttest{
if (requireNamespace("psychotree", quietly = TRUE) &&
    requireNamespace("partykit", quietly = TRUE) &&
    requireNamespace("difR", quietly = TRUE) &&
    requireNamespace("iarm", quietly = TRUE)) {
  data("DIFSimPC", package = "psychotree")

  items <- as.data.frame(as.matrix(DIFSimPC$resp))
  covs  <- DIFSimPC[, c("age", "gender", "motivation")]

  # Default: kable of per-split effect sizes
  RMdifTree(items, covariates = covs)

  # Tidy data.frame -- one row per (split node x item)
  df <- RMdifTree(items, covariates = covs, output = "dataframe")
  df[df$Flagged, ]

  # Tree object (for plotting via partykit::plot)
  tree <- RMdifTree(items, covariates = covs, output = "tree")
  plot(tree)

  # Stability assessment refits the tree on B resamples.
  # (use more resamples, e.g. 100+, in real analyses)
  if (requireNamespace("stablelearner", quietly = TRUE)) {
    kbl <- RMdifTree(items, covariates = covs,
                     purification = "iterative", p_adj = "fdr",
                     stability = TRUE, stability_B = 25)
    kbl                          # main effect-size kable
    attr(kbl, "stability_kable") # pre-rendered stability kable
    attr(kbl, "stability")       # raw stability data.frame
  }
}
}

}
\references{
Bjorner, J. B., Kreiner, S., Ware, J. E., Damsgaard, M. T., & Bech,
P. (1998). Differential item functioning in the Danish translation
of the SF-36. \emph{Journal of Clinical Epidemiology, 51}(11),
1189-1202. \doi{10.1016/S0895-4356(98)00111-5}

Henninger, M., Debelak, R., & Strobl, C. (2023). A new stopping
criterion for Rasch trees based on the Mantel-Haenszel effect size
measure for DIF. \emph{Educational and Psychological Measurement,
83}, 181-212. \doi{10.1177/00131644221077135}

Henninger, M., Radek, J., Debelak, R., & Strobl, C. (2025). Partial
credit trees meet the partial gamma coefficient for quantifying DIF
and DSF in polytomous items. \emph{Behaviormetrika, 52}, 221-257.
\doi{10.1007/s41237-024-00252-3}

Holland, P. W., & Thayer, D. T. (1986). Differential item performance
and the Mantel-Haenszel procedure.
\emph{ETS Research Report Series}(2).
\doi{10.1002/j.2330-8516.1986.tb00186.x}

Philipp, M., Rusch, T., Hornik, K., & Strobl, C. (2018). Measuring
the stability of results from supervised statistical learning.
\emph{Journal of Computational and Graphical Statistics, 27},
685-700. \doi{10.1080/10618600.2018.1473779}

Asamoah, N. A. B., Turner, R. C., Lo, W.-J., Crawford, B. L., & Jozkowski,
K. N. (2025). Impacts of DIF Item Balance and Effect Size Incorporation With
the Rasch Tree. \emph{Educational and Psychological Measurement}.
\doi{10.1177/00131644251370605}

Zwick, R. (2012). A review of ETS differential item functioning
assessment procedures: Flagging rules, minimum sample size
requirements, and criterion refinement. \emph{ETS Research Report
Series}(1). \doi{10.1002/j.2333-8504.2012.tb02290.x}
}
\seealso{
\code{\link{RMdifLR}}, \code{\link{RMdifGamma}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/item_cat_prob.R
\name{RMitemCatProb}
\alias{RMitemCatProb}
\title{Item Category Probability Curves}
\usage{
RMitemCatProb(
  data,
  item_labels = NULL,
  category_labels = NULL,
  theta_range = NULL,
  n_points = 200L,
  viridis_option = "D",
  viridis_end = 0.95,
  facet_ncol = NULL,
  label_wrap = 25L,
  line_width = 0.9,
  font = "sans",
  output = c("ggplot", "dataframe"),
  label_curves = c("legend", "path"),
  item = NULL,
  text_size = 4
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be
scored starting at 0 (non-negative integers). Missing values
(\code{NA}) are allowed --- the model fit handles them.}

\item{item_labels}{Optional character vector of descriptive item
labels (facet strip titles). Must be the same length as
\code{ncol(data)}. If \code{NULL} (the default), column names are used.}

\item{category_labels}{Optional character vector of labels for the
response categories (legend). Must be the same length as the
number of categories spanning from 0 to the maximum observed
value. If \code{NULL} (the default), numeric category values are used.}

\item{theta_range}{Numeric length 2. Range of the latent trait
\eqn{\theta} (logits) plotted along the x-axis. When \code{NULL}
(default) the range is \code{c(-4, 4)} for \code{label_curves = "legend"}
and a wider \code{c(-5, 5)} for \code{label_curves = "path"} --- the
wider range gives \code{geomtextpath} more horizontal room to place
per-curve labels at their peaks. Pass an explicit
\code{c(lo, hi)} to override.}

\item{n_points}{Integer. Number of evenly-spaced \eqn{\theta} values
at which the curves are evaluated. Default \code{200L}.}

\item{viridis_option}{Character. Viridis palette identifier. One of
\code{"A"} through \code{"H"}. Default \code{"D"} (viridis green).}

\item{viridis_end}{Numeric in (0, 1]. Upper end of the viridis
palette range; lower values keep the palette inside its mid-tones
(avoids the very bright yellow at \code{1.0}). Default \code{0.95}.}

\item{facet_ncol}{Optional integer. Number of columns in the facet
layout. Default \code{NULL} (\code{ggplot2::facet_wrap()} auto-layout).}

\item{label_wrap}{Integer. Characters per line for facet-strip
label wrapping. Default \code{25L}.}

\item{line_width}{Numeric. Line width for the probability curves.
Default \code{0.9}.}

\item{font}{Character. Font family for all text. Default \code{"sans"}.}

\item{output}{Character. Either \code{"ggplot"} (default) for a
\code{ggplot} object, or \code{"dataframe"} for the underlying long-format
probability table.}

\item{label_curves}{Character. How response categories are
identified. \code{"legend"} (default) draws lines and a separate
colour legend with one swatch per category --- the standard
multi-item presentation. \code{"path"} uses
\code{geomtextpath::geom_textpath()} to write each category's label
\emph{along} its own curve (a la classic IRT trace plots) and
suppresses the legend; valid for \strong{a single item at a time}
because narrow facets do not give path labels enough horizontal
room to render legibly. The model is still fit on the full
multi-item \code{data} (PCM thresholds require multi-item input for
CML estimation); the \code{item} argument then selects which item's
curves to plot. Requires the optional \code{geomtextpath} package.}

\item{item}{Character or integer. Used only when
\code{label_curves = "path"}. Either an item name (matching a column
in \code{data}) or a column index, identifying which item's category
curves to plot. Required for path mode.}

\item{text_size}{Numeric. Used only when \code{label_curves = "path"}.
Size of the path labels in mm. Default \code{4}.}
}
\value{
\itemize{
\item If \code{output = "ggplot"}: a \link[ggplot2:ggplot]{ggplot2::ggplot} object, one facet per
item.
\item If \code{output = "dataframe"}: a long-format data.frame with columns
\code{Item} (factor in column order), \code{Category} (integer), and
\code{Theta}, \code{Probability} (numeric), one row per item × category ×
theta gridpoint.
}
}
\description{
Plots model-implied response-category probability curves for each item
as a function of the latent trait \eqn{\theta}. Item parameters are
estimated by conditional maximum likelihood via \code{psychotools::pcmodel()}
(a dichotomous item is a 2-category PCM). Each item gets
its own facet panel, with one curve per response category coloured
from low to high using the viridis palette. Comparable in scope to
\code{eRm::plotICC()} and \code{mirt}'s trace plots, with a \code{ggplot2} /
viridis output and optional descriptive labels for items and
categories.
}
\details{
For each polytomous item \emph{i} with response categories
\eqn{0, 1, \ldots, K_i} and threshold parameters
\eqn{\delta_{i,1}, \ldots, \delta_{i,K_i}} (CML, \code{psychotools}),
the PCM category probability is

\deqn{P(X_i = k \mid \theta) = \frac{\exp(\sum_{j=1}^{k}
  (\theta - \delta_{i,j}))}{\sum_{k'=0}^{K_i} \exp(\sum_{j=1}^{k'}
  (\theta - \delta_{i,j}))},}

with the empty sum (when \eqn{k = 0}) taken as zero. For
dichotomous items the function fits a Rasch model and treats
the item difficulty \eqn{\delta_i = -\beta_i} as the single
threshold, recovering the standard two-category logistic ICC.

The colour mapping uses \code{scale_color_viridis_c()} against the
integer category value, so the natural ordering of response
categories is preserved visually --- low categories at one end of
the palette, high categories at the other. When \code{category_labels}
is provided, the legend uses those labels (e.g., "Never" /
"Sometimes" / "Often") while the colour mapping stays on the
integer category value.

Items with fewer response categories than the maximum (e.g., an
otherwise four-category scale with one three-category item)
contribute only the categories they actually have to their own
facet --- the y-axis still spans \eqn{[0, 1]}.
}
\examples{
\donttest{
if (requireNamespace("eRm", quietly = TRUE) &&
    requireNamespace("ggplot2", quietly = TRUE)) {
  data(pcmdat2, package = "eRm")

  # Default plot
  RMitemCatProb(pcmdat2)

  # Custom item and category labels
  RMitemCatProb(
    pcmdat2,
    item_labels     = c("Mood", "Sleep", "Appetite", "Energy"),
    category_labels = c("Never", "Sometimes", "Often")
  )

  # Underlying probability data
  df <- RMitemCatProb(pcmdat2, output = "dataframe")
  head(df)

  # Single-item plot with labels written along each curve
  # (classic IRT trace-plot style). Model is still fit on all four
  # items; `item` picks which one to plot.
  if (requireNamespace("geomtextpath", quietly = TRUE)) {
    RMitemCatProb(
      pcmdat2,
      category_labels = c("Never", "Sometimes", "Often"),
      label_curves    = "path",
      item            = "I1"
    )
  }
}
}

}
\seealso{
\code{\link[=RMitemICCPlot]{RMitemICCPlot()}} for conditional ICCs binned by total
score, \code{\link[=RMitemHierarchy]{RMitemHierarchy()}} for item threshold locations on the
logit scale.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/item_hierarchy.R
\name{RMitemHierarchy}
\alias{RMitemHierarchy}
\title{Item-Threshold Hierarchy Plot}
\usage{
RMitemHierarchy(
  data,
  show_numbers = TRUE,
  sem_multiplier = 1.405,
  item_labels = NULL,
  output = c("ggplot", "dataframe")
)
}
\arguments{
\item{data}{A data.frame or matrix of polytomous item responses
(non-negative integers, 0-based, max value > 1). One column per
item, one row per person.}

\item{show_numbers}{Logical. When \code{TRUE} (default), prints the
numerical item location and threshold values next to the points on
the plot. When \code{FALSE}, only the threshold labels (\code{T1}, \code{T2}, ...)
are shown.}

\item{sem_multiplier}{Numeric multiplier for the threshold SE used to
draw the error bars. Default \code{1.405} (84\% CI). Common alternatives:
\code{1.96} (95\% CI), \code{2.576} (99\% CI).}

\item{item_labels}{Optional character vector of length \code{ncol(data)}
providing descriptive labels for items. Default \code{NULL} uses the
column names of \code{data}. Labels are appended to the column names on
the y-axis (\code{name - label}) and wrapped at 36 characters via base-R
\code{strwrap()}.}

\item{output}{One of \code{"ggplot"} (default) -- a faceted hierarchy
figure -- or \code{"dataframe"} -- the long-format underlying data with
one row per (item × threshold).}
}
\value{
Either a \code{ggplot} (default) or a data.frame with columns
\code{Item}, \code{ItemLabel}, \code{Threshold}, \code{ThresholdLocation},
\code{ThresholdSE}, and \code{ItemLocation} (the per-item mean of the centred
thresholds).
}
\description{
Visualises item and threshold locations on the logit scale for a
Partial Credit Model. Items are sorted by their (mean-threshold)
location; each item shows its location as a black diamond and its
individual thresholds as coloured dots with confidence-interval error
bars.
}
\details{
Threshold locations are centred at the grand mean of all thresholds
across items, so the dashed reference line at 0 represents the mean
threshold location across the scale.

Confidence intervals around thresholds are 84\% by default
(\code{sem_multiplier = 1.405}), following Payton, Greenstone, & Schenker
(2003) -- non-overlap of 84\% intervals approximately corresponds to a
two-sample significance test at \eqn{\alpha = 0.05}. Use
\code{sem_multiplier = 1.96} for 95\% intervals.

\strong{Polytomous only.} Dichotomous items have a single threshold
that coincides with the item location; the hierarchy plot is
visually degenerate in that case. For dichotomous data use
\code{\link[=RMtargeting]{RMtargeting()}} or \code{\link[=RMscoreSE]{RMscoreSE()}} instead.

\strong{Centring convention.} The CML PCM thresholds (estimated via
\code{psychotools::pcmodel()}) are shifted so that their grand mean is zero;
each item's location is then the mean of its centred thresholds.
The dashed horizontal reference line on the plot marks this zero --
i.e., the average threshold across all items.
}
\examples{
\donttest{
if (requireNamespace("eRm", quietly = TRUE)) {
  data("pcmdat2", package = "eRm")
  RMitemHierarchy(pcmdat2)

  # 95\% CI instead of 84\%
  RMitemHierarchy(pcmdat2, sem_multiplier = 1.96)

  # Underlying data.frame
  RMitemHierarchy(pcmdat2, output = "dataframe")
}
}

}
\references{
Payton, M. E., Greenstone, M. H., & Schenker, N. (2003). Overlapping
confidence intervals or standard error intervals: What do they mean
in terms of statistical significance? \emph{Journal of Insect Science,
3}(34), 1-6. \doi{10.1093/jis/3.1.34}
}
\seealso{
\code{\link[=RMtargeting]{RMtargeting()}}, \code{\link[=RMscoreSE]{RMscoreSE()}}, \code{\link[=RMitemICCPlot]{RMitemICCPlot()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ld_partgam.R
\name{print.RMlocdepGamma}
\alias{print.RMlocdepGamma}
\title{Print method for RMlocdepGamma kable output}
\usage{
\method{print}{RMlocdepGamma}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"RMlocdepGamma"} returned by
\code{\link{RMlocdepGamma}} with \code{output = "kable"}.}

\item{...}{Further arguments (currently unused).}
}
\value{
Invisibly returns \code{x}.
}
\description{
Prints the two rest-score direction tables stacked vertically with a
blank line between them. Each table renders via \code{knitr_kable}'s own
print method as a clean pipe-markdown table.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/martin_lof.R
\name{RMdimMartinLofResiduals}
\alias{RMdimMartinLofResiduals}
\title{Standardised Residuals from the Joint Subscore Distribution}
\usage{
RMdimMartinLofResiduals(
  data,
  partition,
  output = c("kable", "dataframe", "ggplot"),
  flag_threshold = 2,
  color_by = c("residual", "n"),
  color_limits = NULL,
  min_expected = NULL
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses (0-based,
non-negative integers). Rows with any \code{NA} are dropped.}

\item{partition}{Same format as in \code{\link{RMdimMartinLof}}: a list of
item-name/index vectors, or a length-\code{ncol(data)} vector of group
labels. Each subscale must contain at least 2 items.}

\item{output}{Character. \code{"kable"} (default) for a 2-D pipe-format
residual table when D = 2 (long-format kable when D > 2), \code{"dataframe"}
for the underlying long-format data.frame, or \code{"ggplot"} for a
diverging-fill heatmap (with \code{facet_wrap} over \code{t3} for D = 3, error
for D > 3).}

\item{flag_threshold}{Numeric. Cells with \verb{|residual| > flag_threshold}
are flagged: marked as \verb{**bold**} in the kable, shown in the \code{flagged}
column of the dataframe. Default \code{2}.}

\item{color_by}{Character. For \code{output = "ggplot"}, what the tile fill
colour encodes. \code{"residual"} (default) -- diverging red-white-blue scale
centred at 0, the most directly diagnostic. \code{"n"} -- sequential blue
scale on the observed cell count, useful for spotting whether
large-magnitude residuals are driven by sparse cells. Either way, the
numeric residual is printed inside each cell.}

\item{color_limits}{Numeric length-2 vector or \code{NULL}. Caps the colour
scale for \code{output = "ggplot"}. Default \code{c(-5, 5)} when
\code{color_by = "residual"} (sparse-cell residuals from
\code{(o-e)/sqrt(n*p*(1-p))} can be enormous when expected counts are tiny;
capping the scale stops outliers from compressing the rest of the
plot). The unclipped residual values still appear as cell labels.
\code{NULL} for \code{color_by = "n"}, which uses the natural data range.}

\item{min_expected}{Numeric or \code{NULL}. If set, cells with \code{expected < min_expected} have their residual set to \code{NA} (they appear grey in the
heatmap and are not flagged). Default \code{NULL} (no filtering). Setting
\code{min_expected = 1} (or \code{5}) is the analogue of the Cochran rule for
sparse-cell chi-square contributions and removes residuals whose
asymptotic standard normal approximation is unreliable.}
}
\value{
\itemize{
\item \code{output = "kable"}: a \code{knitr_kable} object. For D = 2, a wide table
with rows = \code{t1}, columns = \code{t2}, cells = standardised residual
(\verb{**bold**} if flagged, em-dash if NA). For D > 2, long-format with
one row per cell.
\item \code{output = "dataframe"}: a long-format data.frame with columns \code{t1},
..., \code{tD}, \code{total}, \code{observed}, \code{expected}, \code{residual}, \code{flagged}.
\item \code{output = "ggplot"}: a \code{geom_tile()} heatmap (D = 2 or 3 only).
}
}
\description{
Diagnostic accompanying \code{\link{RMdimMartinLof}}: per-cell standardised
residuals from the joint distribution of subscores under unidimensionality
(Christensen, Bjorner, Kreiner, & Petersen, 2002, eq. 13). Useful for
identifying \emph{where} a partition deviates from the unidimensional null
rather than just whether it does (which \code{RMdimMartinLof()} answers).
}
\details{
For each cell of the joint subscore table (indexed by
\eqn{(t_1, \ldots, t_D)}), the conditional probability under H0 given the
total score \eqn{t = \sum_d t_d} is
\deqn{p(t_1, \ldots, t_D \mid t) = \prod_d \gamma^{(d)}_{t_d} / \gamma_t,}
the expected count is \eqn{e = n_t \cdot p}, and the residual is
\eqn{(o - e) / \sqrt{n_t \cdot p \cdot (1 - p)}}. CML estimates from the
unidimensional model are used for the \eqn{\gamma}-functions.

Reading the table (D = 2): under the unidimensional null, residuals
should be patternless and roughly N(0, 1). Multidimensionality with
positively correlated dimensions typically shows up as \strong{positive}
residuals at the corners of each antidiagonal (high \code{t1} + low \code{t2},
low \code{t1} + high \code{t2}) and \strong{negative} residuals near the antidiagonal
centre (matched subscores). Negatively correlated dimensions show
positive residuals at the table corners (high/low and low/high) and
negative residuals at high/high and low/low. See Christensen et al.
(2002, section7) for a worked example.

Cells where the total score has no observed cases (\code{n_t = 0}) are
uninformative and are dropped from the output.
}
\examples{
\donttest{
set.seed(1)
dat <- as.data.frame(matrix(sample(0:1, 400 * 8, replace = TRUE),
                            nrow = 400, ncol = 8))
colnames(dat) <- paste0("I", 1:8)

# Wide kable table for D = 2
RMdimMartinLofResiduals(dat,
                     partition = list(c("I1","I2","I3","I4"),
                                      c("I5","I6","I7","I8")))

# Heatmap
if (requireNamespace("ggplot2", quietly = TRUE)) {
  RMdimMartinLofResiduals(dat,
                       partition = c(1,1,1,1,2,2,2,2),
                       output = "ggplot")
}

# Underlying data.frame for custom analysis
df <- RMdimMartinLofResiduals(dat,
                           partition = c(1,1,1,1,2,2,2,2),
                           output = "dataframe")
df[df$flagged, ]
}
}
\references{
Christensen, K. B., Bjorner, J. B., Kreiner, S., & Petersen, J. H. (2002).
Testing unidimensionality in polytomous Rasch models. \emph{Psychometrika,
67}(4), 563-574. \doi{10.1007/BF02295132}
}
\seealso{
\code{\link{RMdimMartinLof}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bootstrap_restscore.R
\name{run_single_boot_restscore}
\alias{run_single_boot_restscore}
\title{Run a single item-restscore bootstrap iteration}
\usage{
run_single_boot_restscore(seed, data_list)
}
\arguments{
\item{seed}{Integer seed for reproducibility.}

\item{data_list}{List produced inside \code{\link[=RMitemRestscoreBoot]{RMitemRestscoreBoot()}} containing \code{data},
\code{samplesize}, \code{item_names}.}
}
\value{
A data.frame with columns \code{Item}, \code{item_restscore}, \code{diff},
\code{diff_abs}, or a character string on failure.
}
\description{
Run a single item-restscore bootstrap iteration
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz_renaming.R
\name{easyRasch2-renaming}
\alias{easyRasch2-renaming}
\title{Function renaming in easyRasch2 0.8.0}
\description{
In version 0.8.0, 22 exported functions were renamed under a
consistent \emph{domain-prefix \eqn{\to} method \eqn{\to} variant-suffix}
scheme. The renames have no semantic effect --- only names changed.
Use the table below to migrate existing scripts.
}
\section{Domain prefixes}{

\itemize{
\item \code{RMdif} --- differential item functioning
\item \code{RMlocdep} --- local dependence
\item \code{RMitem} --- per-item statistics (fit, restscore, ICC, hierarchy)
\item \code{RMdim} --- dimensionality assessment
\item \code{RMplot} --- descriptive response-distribution plots
}

Variant suffixes attached at the end:
\itemize{
\item \code{Cutoff} --- simulation-based critical values
\item \code{Plot} --- visualisation
\item \code{MI} --- multiple-imputation variant (replaces former \verb{_mi})
}
}

\section{Old to new name map}{

\tabular{ll}{
\strong{Old name} \tab \strong{New name} \cr
\code{RMpartgamDIF()}        \tab \code{RMdifGamma()}             \cr
\code{RMpgDIFcutoff()}       \tab \code{RMdifGammaCutoff()}       \cr
\code{RMpgDIFplot()}         \tab \code{RMdifGammaPlot()}         \cr
\code{RMpartgamLD()}         \tab \code{RMlocdepGamma()}          \cr
\code{RMpgLDcutoff()}        \tab \code{RMlocdepGammaCutoff()}    \cr
\code{RMpgLDplot()}          \tab \code{RMlocdepGammaPlot()}      \cr
\code{RMlocdepQ3cutoff()}    \tab \code{RMlocdepQ3Cutoff()}       \cr
\code{RMlocdepQ3plot()}      \tab \code{RMlocdepQ3Plot()}         \cr
\code{RMiteminfit()}         \tab \code{RMitemInfit()}            \cr
\code{RMiteminfit_mi()}      \tab \code{RMitemInfitMI()}          \cr
\code{RMinfitcutoff()}       \tab \code{RMitemInfitCutoff()}      \cr
\code{RMinfitcutoff_mi()}    \tab \code{RMitemInfitCutoffMI()}    \cr
\code{RMinfitcutoffPlot()}   \tab \code{RMitemInfitCutoffPlot()}  \cr
\code{RMitemrestscore()}     \tab \code{RMitemRestscore()}        \cr
\code{RMbootRestscore()}     \tab \code{RMitemRestscoreBoot()}    \cr
\code{RMciccPlot()}          \tab \code{RMitemICCPlot()}          \cr
\code{RMresidualPCA()}       \tab \code{RMdimResidualPCA()}       \cr
\code{RMpcaCutoff()}         \tab \code{RMdimResidualPCACutoff()} \cr
\code{RMcfaCutoff()}         \tab \code{RMdimCFACutoff()}         \cr
\code{RMcfaPlot()}           \tab \code{RMdimCFAPlot()}           \cr
\code{RMmartinLof()}         \tab \code{RMdimMartinLof()}         \cr
\code{RMmartinLofResiduals()}\tab \code{RMdimMartinLofResiduals()}\cr
\code{RMtileplot()}          \tab \code{RMplotTile()}             \cr
\code{RMbarplot()}           \tab \code{RMplotBar()}              \cr
\code{RMstackedbarplot()}    \tab \code{RMplotStackedbar()}       \cr
}

Unchanged: \code{RMreliability()}, \code{RMUreliability()},
\code{RMitemHierarchy()}, \code{RMdifLR()}, \code{RMdifTree()}, \code{RMlocdepQ3()},
\code{RMtargeting()}, \code{RMscoreSE()}.
}

\section{No deprecation aliases}{

No deprecation shims are shipped: each old name is simply gone. This
keeps the export list clean ahead of the first CRAN submission. Run a
search-and-replace against the table above to migrate.
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/item_restscore.R
\name{RMitemRestscore}
\alias{RMitemRestscore}
\title{Item Restscore Analysis}
\usage{
RMitemRestscore(data, output = "kable", sort, p_adj = "BH")
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Missing values (\code{NA}) are allowed,
but at least one complete case (row with no \code{NA}) must be present.}

\item{output}{Character string controlling the return value. Either
\code{"kable"} (default) for a formatted \code{knitr::kable()} table, or
\code{"dataframe"} for the underlying data.frame.}

\item{sort}{Optional character string. When \code{sort = "diff"}, rows are sorted
by the absolute magnitude of \code{Difference} in descending order, so that
both over- and underfitting items appear near the top.}

\item{p_adj}{Character string specifying the p-value adjustment method
passed to \code{iarm::item_restscore()}. Default \code{"BH"} (Benjamini-Hochberg);
use \code{"none"} for unadjusted p-values. Run \code{?stats::p.adjust} for the list
of available methods.}
}
\value{
\itemize{
\item If \code{output = "kable"}: a \code{knitr_kable} object (plain text table via
\code{format = "pipe"}) with columns for item name, observed and expected
restscore correlations, the signed difference (observed minus
expected), adjusted p-value, the \code{Flagged} misfit label, and item
location relative to the sample mean person location.
\item If \code{output = "dataframe"}: a data.frame with columns \code{Item}, \code{Observed},
\code{Expected}, \code{Difference}, \code{p_adjusted}, \code{Flagged}, and
\code{Relative_location}. \code{Flagged} is \code{"overfit"} (observed above expected,
adj. p < .05), \code{"underfit"} (below, adj. p < .05), or \code{""} (not flagged).
}

The \code{Difference} column is signed (observed minus expected):
\emph{positive} values indicate that the item correlates more strongly with
the rest-score than the Rasch model predicts (over-discrimination /
\emph{overfit}, often associated with local dependence), and \emph{negative}
values indicate weaker-than-expected association (under-discrimination
/ \emph{underfit}, often associated with multidimensionality or noise).
}
\description{
Computes observed and model-expected item-restscore correlations using
\code{iarm::item_restscore()}, and enriches the output with the absolute
difference between observed and expected values, item average locations, and
item locations relative to the sample mean person location.
}
\details{
Item-restscore correlations using Goodman-Kruskal's gamma (Kreiner, 2011) measure
the association between a person's score on a single item and their total
score on the remaining items (the "restscore"). Under a correctly fitting
Rasch model, observed and model-expected correlations should agree closely.

Item parameters are estimated by conditional maximum likelihood via
\code{psychotools::pcmodel()} (a dichotomous item is a 2-category PCM); the
item-restscore statistic itself comes from \code{iarm::item_restscore()} and is
conditional on the total score, so it is invariant to the estimation engine.
Per-item average locations are the means of the CML thresholds, and the
person-location reference is the mean of the Warm WLE estimates.

Relative item location is defined as the item's average location minus the
sample mean person location, providing a measure of item targeting.

The \code{iarm} package must be installed (it is in Suggests, not Imports).
}
\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE)) {
  # Simulate binary item response data (8 items, 200 persons)
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 8, replace = TRUE), nrow = 200, ncol = 8)
  )
  colnames(sim_data) <- paste0("Item", 1:8)

  # Default kable output
  RMitemRestscore(sim_data)

  # Sorted by absolute difference
  RMitemRestscore(sim_data, sort = "diff")

  # Return as data.frame for further processing
  df <- RMitemRestscore(sim_data, output = "dataframe")
}
}
}
\references{
Kreiner, S. (2011). A Note on Item–Restscore Association in Rasch Models.
\emph{Applied Psychological Measurement, 35}(7), 557–561.
\doi{10.1177/0146621611410227}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/local_dependence.R
\name{RMlocdepQ3Cutoff}
\alias{RMlocdepQ3Cutoff}
\title{Simulation-Based \eqn{Q_3} Cutoff Determination}
\usage{
RMlocdepQ3Cutoff(
  data,
  iterations = 500,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  cutoff_method = "hdci",
  hdci_width = 0.99,
  estimator = c("CML", "MML"),
  dgp = c("resample", "conditional")
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers).}

\item{iterations}{Integer. Number of simulation iterations (default 500).}

\item{parallel}{Logical. Use parallel processing via \code{mirai} if available
(default \code{TRUE}).}

\item{n_cores}{Integer or \code{NULL}. Number of parallel workers. When \code{NULL},
\code{getOption("mc.cores")} is checked first. If neither is set and
\code{parallel = TRUE}, a warning is issued and execution falls back to
sequential (single core) processing.}

\item{verbose}{Logical. Show a progress bar (default \code{FALSE}).}

\item{seed}{Integer or \code{NULL}. Random seed for reproducibility.}

\item{cutoff_method}{Character. Method used to compute per-pair \eqn{Q_3}
credible intervals in \code{pair_cutoffs}. One of \code{"hdci"} (the default,
Highest Density Continuous Interval via \code{ggdist::hdci()}) or
\code{"quantile"} (symmetric 2.5th / 97.5th percentiles). Only affects
\code{pair_cutoffs}; the global \verb{$suggested_cutoff} (99th percentile of
\code{max(Q3) - mean(Q3)}) is unaffected.}

\item{hdci_width}{Numeric in (0, 1). Width of the HDCI when
\code{cutoff_method = "hdci"}. Default \code{0.99}. Ignored when
\code{cutoff_method = "quantile"}.}

\item{estimator}{Character. Estimation engine for the simulated \eqn{Q_3} values,
passed through to the per-iteration computation. \code{"CML"} (default) uses
CML item parameters and WLE person locations; \code{"MML"} uses \code{mirt}. This
must match the \code{estimator} later given to \code{\link{RMlocdepQ3}}; the
value is stored in the returned object's \verb{$estimator} and reused
automatically.}

\item{dgp}{Character. Data-generating process for the parametric bootstrap.
\code{"resample"} (default) draws person locations by resampling the WLE
estimates with replacement and simulates responses under the model -- a
\emph{marginal} null. \code{"conditional"} instead simulates each respondent's
pattern from the exact Rasch conditional distribution given their observed
total score (and answered items), with item parameters fixed -- a
\emph{conditional} null that fixes the score margin and needs no latent
distribution, avoiding the over-dispersion of resampled point estimates.
The two give different cut-offs; see the package's comparison study.
\strong{Experimental.}}
}
\value{
A list with components:
\describe{
\item{\code{results}}{data.frame with columns \code{mean}, \code{max}, \code{diff} (one row
per successful iteration).}
\item{\code{pair_results}}{Long data.frame with columns \code{Item1}, \code{Item2},
\code{Q3}, \code{iteration} --- one row per item pair per successful iteration.
Used by \code{\link{RMlocdepQ3Plot}}.}
\item{\code{pair_cutoffs}}{data.frame with per-pair cutoff summaries:
\code{Item1}, \code{Item2}, \code{Q3_low}, \code{Q3_high}. Boundaries are computed via
the method specified by \code{cutoff_method}.}
\item{\code{actual_iterations}}{Number of successful iterations.}
\item{\code{sample_n}}{Number of persons used: respondents with no responses
at all (all-\code{NA} rows) are dropped, as in \code{\link{RMlocdepQ3}};
incomplete response patterns are retained.}
\item{\code{sample_n_total}}{Number of respondents in the raw input data,
before dropping all-\code{NA} rows.}
\item{\code{sample_has_na}}{Logical. Whether the data contained any missing
values.}
\item{\code{sample_summary}}{Summary statistics of estimated person parameters.}
\item{\code{item_names}}{Character vector of item names from \code{data}.}
\item{\code{max_diff}, \code{sd_diff}}{Max and SD of the \code{diff} distribution.}
\item{\code{p95}, \code{p99}, \code{p995}, \code{p999}}{Empirical percentiles of \code{diff}.}
\item{\code{suggested_cutoff}}{The 99th percentile (\code{p99}) --- recommended
scalar cutoff for \code{\link{RMlocdepQ3}}.}
\item{\code{cutoff_method}}{The method used for \code{pair_cutoffs}
(\code{"hdci"} or \code{"quantile"}).}
\item{\code{hdci_width}}{The HDCI width used (only meaningful when
\code{cutoff_method = "hdci"}).}
\item{\code{estimator}}{The estimator used for the simulated \eqn{Q_3} (\code{"CML"} or
\code{"MML"}); reused by \code{\link{RMlocdepQ3}} and
\code{\link{RMlocdepQ3Plot}}.}
\item{\code{dgp}}{The data-generating process used (\code{"resample"} or
\code{"conditional"}).}
}
}
\description{
Uses parametric bootstrap simulation to determine an appropriate cutoff
value for \code{\link{RMlocdepQ3}}. Under a correctly fitting Rasch model,
\eqn{Q_3} residuals have an unknown distribution; this function simulates
that distribution and returns empirical percentiles.
}
\details{
The generating model is fitted once: CML item parameters (via
\code{psychotools}) and WLE person locations. For each simulation iteration,
those WLE thetas are resampled with replacement, response data are simulated
under the Rasch / Partial Credit model, the model is refitted, and \eqn{Q_3}
residuals are computed under \code{estimator}. The distribution of
\code{max(Q3) - mean(Q3)} across iterations provides empirical critical values.
Failed iterations (e.g., due to convergence issues) are silently discarded.

Supports both \strong{dichotomous} data (simulated via \code{psychotools::rrm()}) and
\strong{polytomous} data (via an internal partial credit score simulator).

Parallel processing is provided by the \code{mirai} package (optional). Install
it with \code{install.packages("mirai")} to enable parallelisation.
}
\examples{
\donttest{
if (requireNamespace("ggdist", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_data) <- paste0("Item", 1:10)

  # Few iterations for a fast example; use 500+ in real analyses
  cutoff_res <- RMlocdepQ3Cutoff(sim_data, iterations = 50, parallel = FALSE,
                                 seed = 42)
  cutoff_res$suggested_cutoff  # 99th percentile

  # Use the cutoff in RMlocdepQ3()
  RMlocdepQ3(sim_data, cutoff = cutoff_res$suggested_cutoff)
}
}
}
\seealso{
\code{\link{RMlocdepQ3}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ld_partgam.R
\name{run_single_partgam_LD_sim}
\alias{run_single_partgam_LD_sim}
\title{Run a single partial gamma LD simulation iteration}
\usage{
run_single_partgam_LD_sim(seed, data_list)
}
\arguments{
\item{seed}{Integer seed for reproducibility.}

\item{data_list}{List produced inside \code{\link[=RMlocdepGammaCutoff]{RMlocdepGammaCutoff()}}.}
}
\value{
A data.frame with columns \code{Item1}, \code{Item2}, and \code{gamma}, or a
character string on failure.
}
\description{
Run a single partial gamma LD simulation iteration
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/martin_lof.R
\name{RMdimMartinLof}
\alias{RMdimMartinLof}
\title{Martin-Löf Test of Unidimensionality}
\usage{
RMdimMartinLof(
  data,
  partition,
  iterations = 1000L,
  stopping = c("none", "sequential"),
  h = 50L,
  alpha = 0.05,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses (0-based,
non-negative integers). Rows with any \code{NA} are dropped.}

\item{partition}{The hypothesised partition of items into subscales. One of:
\itemize{
\item a \strong{list} of column-name or column-index vectors, e.g.
\code{list(c("I1","I2","I3"), c("I4","I5","I6"))};
\item a \strong{vector} of length \code{ncol(data)} indicating each item's subscale
(factor, character, or integer), e.g. \code{c(1,1,1,2,2,2)}.
Each subscale must contain at least two items. Subscales must not
overlap; items not assigned to any subscale are dropped with a warning.
}}

\item{iterations}{Integer. Maximum number of Monte Carlo iterations
(default \code{1000}).}

\item{stopping}{Character. \code{"none"} (default) runs all iterations.
\code{"sequential"} uses Besag & Clifford's (1991) sequential rule: stop as
soon as \code{h} simulated statistics have exceeded the observed value. The
sequential strategy substantially reduces compute time when H0 holds
but cannot be parallelised.}

\item{h}{Integer. Sequential-stopping count threshold (default \code{50}).
Ignored when \code{stopping = "none"}.}

\item{alpha}{Numeric in (0, 1). Nominal significance level used only for
the \code{rejected} flag in the result; default \code{0.05}.}

\item{parallel}{Logical. Use parallel processing via \code{mirai} (default
\code{TRUE}). Ignored when \code{stopping = "sequential"}.}

\item{n_cores}{Integer or \code{NULL}. Number of parallel workers. When \code{NULL},
\code{getOption("mc.cores")} is checked first; if neither is set, falls back
to sequential with a warning.}

\item{verbose}{Logical. Show a progress bar (default \code{FALSE}).}

\item{seed}{Integer or \code{NULL}. Random seed for reproducibility. Items
are processed internally in a fixed (alphabetical) order, so the same
seed reproduces the same p-value regardless of how the data's columns
are arranged.}
}
\value{
A list with components:
\describe{
\item{\code{T_obs}}{Observed Martin-Löf likelihood-ratio statistic.}
\item{\code{p_value}}{Monte Carlo p-value with \code{(n_exceed + 1) / (n + 1)}
correction. The attainable p-values are \code{k / (n + 1)} for
\verb{k = 1, ..., n + 1}, so the p-value's resolution is limited by the
number of iterations: with 100 iterations the smallest attainable
value is \code{1/101 = 0.0099}. A reported p-value \emph{equal to the floor}
(see \code{p_value_floor}) means no simulated statistic reached the
observed one and should be read as "p < floor" -- the true p-value
may be much smaller; increase \code{iterations} for finer resolution.}
\item{\code{p_value_floor}}{The smallest attainable p-value,
\code{1 / (actual_iterations + 1)}.}
\item{\code{actual_iterations}}{Number of successful MC iterations
completed.}
\item{\code{rejected}}{Logical: is \code{p_value < alpha}?}
\item{\code{partition}}{Normalised partition (list of integer indices).}
\item{\code{n_subscales}}{Number of subscales.}
\item{\code{is_polytomous}}{Whether a PCM was fitted.}
\item{\code{sample_n}}{Number of complete cases analysed.}
\item{\code{n_items}}{Number of items.}
\item{\code{stopping}}{The stopping strategy used.}
\item{\code{h}}{The sequential-stopping count, or \code{NA} for \code{stopping = "none"}.}
\item{\code{T_rep}}{Numeric vector of successful MC test statistics.}
\item{\code{wle_scores}}{data.frame with one row per person and one column
per subscale (\code{subscale_1_wle}, ..., \code{subscale_D_wle}), giving Warm's
Weighted Likelihood Estimate of theta from a CML fit on each subscale
alone. Persons whose subscore equals the minimum or maximum on a
subscale produce non-finite WLEs (\code{Inf} / \code{-Inf}) and are excluded
from \code{wle_correlation} pairwise.}
\item{\code{wle_correlation}}{data.frame of pairwise Pearson correlations
between subscale WLEs, with columns \code{subscale_a}, \code{subscale_b}, \code{r},
\code{ci_lower}, \code{ci_upper} (95\% CI from \code{stats::cor.test}), \code{p_value},
and \code{n} (number of persons with finite WLEs on both subscales). One
row per pair; for D = 2, a single row. Useful as an effect-size
companion to \code{p_value} -- a rejected test with \code{r} near 1 indicates a
small effect; \code{r} clearly below 1 indicates substantive
multidimensionality.}
}
}
\description{
Likelihood-ratio test of unidimensionality against an \emph{a priori} specified
multidimensional alternative, generalised to polytomous Rasch / partial
credit models (Christensen, Bjorner, Kreiner, & Petersen, 2002). The
p-value is obtained by parametric-bootstrap (Monte Carlo) sampling under
the unidimensional null, following Christensen & Kreiner (2007), because
the asymptotic chi-square approximation is biased toward conservatism for
realistic sample sizes -- especially with polytomous items, where the
degrees of freedom can be very large.
}
\details{
This is \strong{not} a routine screening tool. The test requires an \emph{a priori}
partition of items into subscales; using it post-hoc on, e.g., the
partition suggested by \code{RMdimResidualPCA()}'s PC1 sign would inflate the
Type-I error rate. Both source papers state this explicitly.

\strong{Test statistic.} With items partitioned into D subscales, total score
\eqn{t} and subscores \eqn{(t_1, \ldots, t_D)} (Christensen et al. 2002,
eq. 22):
\deqn{T = 2\Bigl[\sum_{t_1, \ldots, t_D}
  n_{t_1, \ldots, t_D}\log(n_{t_1, \ldots, t_D}/N)
  - \sum_t n_t\log(n_t/N)
  - \ell_C(\hat{\epsilon}) + \sum_d \ell_C(\hat{\epsilon}^{(d)})\Bigr]}
where \eqn{\ell_C} is the conditional log-likelihood and the
\eqn{\hat{\epsilon}^{(d)}} are CML estimates on the d-th subscale alone.
CML fits use \code{psychotools::raschmodel()} (RM) or \code{psychotools::pcmodel()}
(PCM) for speed.

\strong{Monte Carlo sampling under H0.} Following Christensen & Kreiner (2007):
(a) sample N total scores from the empirical score distribution
\eqn{n_t/N}; (b) for each sampled score, sample an item-response vector
from the conditional distribution \eqn{p(x \mid t, \hat{\epsilon})} given
by eq. 4 of the paper. For dichotomous data the fast algorithm of
Christensen & Kreiner (2007, p. 23) is used (sample without replacement
weighted by item easinesses). For polytomous data the recursive
\eqn{\gamma}-function approach is used, with each item's response sampled
conditional on the remaining items' joint score distribution (computed via
\code{psychotools::elementary_symmetric_functions()}).

Iterations that fail (e.g., simulated dataset has an empty category for an
item) are silently dropped.

Item parameters are estimated once on the observed data and held fixed
across MC iterations. Christensen & Kreiner (2007) use the extended
likelihood function (Tjur, 1982) with the empirical score distribution as
a non-parametric estimate of the latent distribution, so no distributional
assumption about \eqn{\theta} is needed.
}
\examples{
\donttest{
set.seed(1)
# Build 2-dimensional polytomous data: 4 items per subscale, 5 categories
n     <- 400
theta1 <- rnorm(n)
theta2 <- 0.6 * theta1 + sqrt(1 - 0.6^2) * rnorm(n)
make_pcm <- function(theta, n_items, taus) {
  sapply(seq_len(n_items), function(j) {
    # ... toy simulation here
    sample(0:4, n, replace = TRUE)
  })
}
dat <- cbind(make_pcm(theta1, 4, NULL), make_pcm(theta2, 4, NULL))
colnames(dat) <- paste0("I", 1:8)

# Few iterations for a fast example; use 1000+ in real analyses
RMdimMartinLof(dat,
            partition = list(c("I1","I2","I3","I4"),
                             c("I5","I6","I7","I8")),
            iterations = 100, parallel = FALSE, seed = 1)

# Sequential stopping: stop as soon as h = 25 simulated statistics exceed
# the observed one (cuts compute time under H0).
RMdimMartinLof(dat,
            partition = c(1,1,1,1,2,2,2,2),
            iterations = 200, stopping = "sequential", h = 25,
            seed = 1)
}
}
\references{
Christensen, K. B., Bjorner, J. B., Kreiner, S., & Petersen, J. H. (2002).
Testing unidimensionality in polytomous Rasch models. \emph{Psychometrika,
67}(4), 563-574. \doi{10.1007/BF02295132}

Christensen, K. B., & Kreiner, S. (2007). A Monte Carlo approach to
unidimensionality testing in polytomous Rasch models. \emph{Applied
Psychological Measurement, 31}(1), 20-30.
\doi{10.1177/0146621605286204}

Besag, J., & Clifford, P. (1991). Sequential Monte Carlo p-values.
\emph{Biometrika, 78}(2), 301-304. \doi{10.1093/biomet/78.2.301}
}
\seealso{
\code{\link{RMdimResidualPCA}}, \code{\link{RMdimResidualPCACutoff}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cfa_cutoff.R
\name{RMdimCFACutoff}
\alias{RMdimCFACutoff}
\title{Simulated null distribution for one-factor CFA fit and loadings under PCM unidimensionality}
\usage{
RMdimCFACutoff(
  data,
  iterations = 250L,
  percentile = 99,
  output = c("list", "kable"),
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  estimator = "WLSMV"
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses (non-negative
integers, 0-based). One column per item, one row per person.}

\item{iterations}{Integer. Number of parametric-bootstrap iterations.
Default \code{250}.}

\item{percentile}{Numeric in (50, 100). The strictness of the cutoffs.
Default \code{99}. Fit indices use a one-sided cutoff in the unfavourable
direction (CFI from below; RMSEA and SRMR from above); standardized
loadings use a two-sided central interval covering \code{percentile}\% of the
simulated distribution (an item is flagged when its observed loading
falls in the outer \code{100 - percentile}\%, split across the two tails).}

\item{output}{Character. Only \code{"list"} (the default) is supported and the
function always returns the simulation object. \code{"kable"} is retained only
to raise an informative error: tables now come from
\code{\link{RMdimCFA}}.}

\item{parallel}{Logical. If \code{TRUE} (default), uses parallel processing
via \code{mirai}. Falls back to sequential if \code{mirai} is not installed or
\code{n_cores} cannot be resolved.}

\item{n_cores}{Integer or \code{NULL}. Number of parallel workers. When
\code{NULL}, \code{getOption("mc.cores")} is consulted; if neither is set,
sequential is used.}

\item{verbose}{Logical. Show a progress bar (default \code{FALSE}).}

\item{seed}{Integer or \code{NULL}. Master seed for reproducibility.}

\item{estimator}{Character. The lavaan estimator passed to
\code{lavaan::cfa()}. Default \code{"WLSMV"}. Other limited-information
estimators that produce robust/scaled fit indices (e.g.,
\code{"DWLS"}, \code{"ULSMV"}) are also accepted; full-information ML is
rejected (incompatible with \code{ordered = TRUE}).}
}
\value{
A list (the simulation object), with components:
\describe{
\item{\code{simulated}}{data.frame with one row per successful iteration
and columns \code{iteration}, \code{cfi}, \code{rmsea}, \code{srmr}.}
\item{\code{simulated_loadings}}{data.frame with one row per successful
iteration: an \code{iteration} column followed by one column per item
holding the simulated standardized loading.}
\item{\code{percentile}}{Numeric: the strictness setting used.}
\item{\code{cutoffs}}{Named numeric vector (\code{cfi}, \code{rmsea}, \code{srmr}) of
one-sided fit-index cutoffs at the chosen percentile.}
\item{\code{loading_cutoffs}}{data.frame \code{Item}, \code{low}, \code{high} — the
two-sided expected loading interval per item.}
\item{\code{actual_iterations}}{Number of successful MC iterations.}
\item{\code{sample_n}}{Number of complete cases used.}
\item{\code{sample_n_total}}{Number of respondents in the raw input data,
before the complete-case filter.}
\item{\code{sample_has_na}}{Logical. Whether the raw input data contained
any missing values.}
\item{\code{n_items}}{Number of items.}
\item{\code{item_names}}{Character vector of item names.}
\item{\code{is_polytomous}}{Logical: was a PCM (vs RM) fitted?}
\item{\code{estimator}}{The lavaan estimator used.}
}
}
\description{
Generates a parametric-bootstrap null distribution against which observed
one-factor categorical-CFA results can be compared. The simulation draws
\code{iterations} datasets from the fitted PCM (or RM, for dichotomous
data) using the observed item parameters and a resampled person
distribution; each simulated dataset is fitted with
\code{lavaan::cfa(..., ordered = TRUE, estimator = "WLSMV")} and both the
three fit indices (CFI, RMSEA, SRMR) and the per-item standardized factor
loadings are recorded. Because the simulated data satisfy the PCM
unidimensional assumption by construction, the resulting distributions are
the "expected" reference for what a correctly fitting unidimensional model
produces at this sample size and item structure.
}
\details{
This function only generates the simulated reference. To obtain the
observed-vs-expected tables, pass its result to \code{\link{RMdimCFA}}; for
the figures, pass it to \code{\link{RMdimCFAPlot}}.

\strong{Generative model.} The data-generating process for each
simulated dataset is the PCM (or RM) fitted to the observed data,
with persons drawn from the empirical theta distribution
(resampled with replacement). This means the simulated data perfectly
satisfy the PCM unidimensional assumption.

\strong{Estimation model.} The CFA on each simulated dataset uses a
single-factor model with all items as ordinal indicators
(\code{F1 =~ I1 + I2 + ...}), fitted with \code{WLSMV} by default. Reported
CFI / RMSEA are the Satorra-Bentler-scaled variants (\code{cfi.scaled},
\code{rmsea.scaled}) for consistency across iterations; SRMR is reported
unchanged. Standardized loadings are the \code{est.std} of the \verb{=~} paths
from \code{lavaan::standardizedSolution()}.

\strong{Why a null distribution.} A perfectly PCM-unidimensional
dataset will typically not yield CFA fit indices at their ideal
values (CFI = 1, RMSEA = 0), nor identical loadings across items: PCM
uses a logistic threshold structure while WLSMV uses a probit link via
the polychoric correlation matrix, and finite samples add sampling
variability. The simulated distributions capture both, giving a more
honest reference than rule-of-thumb cutoffs derived under continuous-data
ML.

\strong{Iteration failures.} Some simulated datasets cause WLSMV to
fail (non-positive-definite polychoric matrix, boundary thresholds,
empty categories). Failed iterations are dropped; \code{actual_iterations}
reflects the number that succeeded.
}
\examples{
\donttest{
if (requireNamespace("lavaan", quietly = TRUE) &&
    requireNamespace("eRm", quietly = TRUE)) {
  data("raschdat1", package = "eRm")

  # Few iterations for a fast example; use 250+ in real analyses
  sim <- RMdimCFACutoff(raschdat1[, 1:8], iterations = 50,
                        parallel = FALSE, seed = 1)

  # Observed-vs-expected tables
  RMdimCFA(raschdat1[, 1:8], cutoff = sim)

  if (requireNamespace("ggplot2", quietly = TRUE)) {
    plots <- RMdimCFAPlot(sim, data = raschdat1[, 1:8])
    plots$loadings
    plots$fit
  }
}
}

}
\references{
Yuan, K.-H., & Bentler, P. M. (2000). Three likelihood-based methods
for mean and covariance structure analysis with nonnormal missing
data. \emph{Sociological Methodology, 30}(1), 165-200.
\doi{10.1111/0081-1750.00078}

Rosseel, Y. (2012). lavaan: An R Package for Structural Equation
Modeling. \emph{Journal of Statistical Software, 48}(2), 1-36.
\doi{10.18637/jss.v048.i02}
}
\seealso{
\code{\link{RMdimCFA}}, \code{\link{RMdimCFAPlot}},
\code{\link{RMdimResidualPCA}}, \code{\link{RMdimMartinLof}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dif_partgam.R
\name{RMdifGamma}
\alias{RMdifGamma}
\title{Partial Gamma DIF Analysis}
\usage{
RMdifGamma(
  data,
  dif_var,
  cutoff = NULL,
  p_value = FALSE,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  alpha = 0.05,
  output = "kable"
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Missing values (\code{NA}) are allowed,
but at least one complete case must exist after combining \code{data} and
\code{dif_var}.}

\item{dif_var}{A vector (factor or character) of the same length as
\code{nrow(data)}, representing the grouping variable for DIF analysis.}

\item{cutoff}{Optional. Default \code{NULL} (no cutoff applied). Can be:
\itemize{
\item The return value of \code{\link{RMdifGammaCutoff}} (a list with
\verb{$item_cutoffs}): the data.frame is extracted automatically and
simulation metadata is included in the kable caption.
\item The \verb{$item_cutoffs} data.frame from \code{\link{RMdifGammaCutoff}}
directly: must have columns \code{Item}, \code{gamma_low}, \code{gamma_high}.
When provided, adds columns \code{Gamma_low}, \code{Gamma_high}, and \code{Flagged}
(logical; \code{TRUE} when the observed partial gamma falls outside the
credible range) to the result.
}}

\item{p_value}{Logical. When \code{TRUE}, adds two-sided bootstrap p-values
(\code{p_gamma}, \code{padj_gamma}) comparing each item's observed partial gamma
against its simulated null distribution, and \code{flagged} reflects
\code{padj_gamma < alpha} instead of the credible range. The asymptotic
BH-adjusted p-value and star columns from \code{iarm::partgam_DIF()} are
\strong{dropped} in this mode (two p-value families in one table would invite
double-reading); the simulated \code{gamma_low} / \code{gamma_high} band is kept as
the effect-size reference. Requires the \strong{full}
\code{\link{RMdifGammaCutoff}} object as \code{cutoff} (it carries the
simulated distributions in \verb{$results}). Default \code{FALSE}.}

\item{correction}{Character. Multiplicity correction for the bootstrap
p-values: \code{"fwer"} (default; Westfall-Young studentised-max step-down),
\code{"fdr_bh"}, \code{"fdr_by"}, or \code{"none"}. Ignored when \code{p_value = FALSE}.}

\item{alpha}{Numeric in (0, 1). Significance level used to flag items on
the corrected p-value. Default \code{0.05}. Ignored when \code{p_value = FALSE}.}

\item{output}{Character string controlling the return value. Either
\code{"kable"} (default) for a formatted \code{knitr::kable()} table, or
\code{"dataframe"} for the underlying data.frame.}
}
\value{
\itemize{
\item If \code{output = "kable"}: a \code{knitr_kable} object with columns "Item",
"Partial gamma", "SE", "Lower CI", "Upper CI", "Adj. p-value (BH)",
and "p-value sign." (a star-string indicator from
\code{iarm::partgam_DIF()}). When \code{cutoff} is provided, additional columns
"Gamma low", "Gamma high", and "Flagged" are included. With
\code{p_value = TRUE}, the asymptotic p-value columns are replaced by
bootstrap "p" and "p (adj)".
\item If \code{output = "dataframe"}: a data.frame with columns \code{Item}, \code{gamma},
\code{se}, \code{lower}, \code{upper}, \code{padj_bh}, \code{Significance}. When \code{cutoff} is
provided, columns \code{gamma_low}, \code{gamma_high}, and \code{flagged} are also
included. With \code{p_value = TRUE}, \code{padj_bh} and \code{Significance} are
replaced by \code{p_gamma} and \code{padj_gamma}.
}
}
\description{
Computes partial gamma coefficients for Differential Item Functioning (DIF)
using \code{iarm::partgam_DIF()}. Each item is tested for association with
a single categorical DIF variable, controlling for the total score.
}
\details{
Partial gamma (Bjorner et al., 1998) measures the association between item
response and an exogenous grouping variable, controlling for the total
score. Values near 0 indicate no DIF. Recommended interpretive thresholds
(Bjorner et al., 1998):
\itemize{
\item \strong{No or negligible DIF}: gamma within \eqn{[-0.21, 0.21]}, \emph{or} gamma
not significantly different from 0.
\item \strong{Slight to moderate DIF}: gamma within \eqn{[-0.31, 0.31]} (and outside
\eqn{[-0.21, 0.21]}), \emph{or} not significantly outside \eqn{[-0.21, 0.21]}.
\item \strong{Moderate to large DIF}: gamma outside \eqn{[-0.31, 0.31]}, \strong{and}
significantly outside \eqn{[-0.21, 0.21]}.
}

The \code{iarm} package must be installed (it is in Suggests, not Imports).

\strong{Bootstrap p-values.} When \code{p_value = TRUE}, each item's observed
partial gamma is compared against its simulated null distribution (from
\code{cutoff$results}, where the DIF variable is random by construction). The
per-item statistic is the residual studentised by the bootstrap mean and
SD; the marginal p-value is the two-sided Monte-Carlo p-value
\verb{(1 + #\\\{|t*| >= |t|\\\}) / (B + 1)}, so it can be no smaller than
\code{1 / (B + 1)}. \code{correction = "fwer"} uses the Westfall-Young
studentised-max step-down, which exploits the bootstrap dependence among
items (Ferreira, 2024); it is liberal when the simulation is small, so at
least 1000 \code{iterations} in \code{\link[=RMdifGammaCutoff]{RMdifGammaCutoff()}} are recommended (a warning
is issued below that). Unlike the asymptotic p-values from
\code{iarm::partgam_DIF()}, these are calibrated against the \emph{simulated Rasch
null} rather than the asymptotic SE; they are model-conditional and
sample-size-sensitive, and are reported alongside the simulated
effect-size band, not in place of it.
}
\section{Multiple comparisons}{

The marginal p-value controls the error rate of a \emph{single} comparison: for
one item (or item pair) decided on in advance it is the relevant value. But
scanning all \emph{k} comparisons and flagging whichever fall below \code{alpha} tests
\emph{k} hypotheses at once, so the chance of at least one false flag inflates to
roughly \eqn{1 - (1 - \alpha)^k} (e.g. about 34\% for \emph{k} = 8 at
\code{alpha = 0.05}) -- even when every marginal p-value is correctly calibrated.
The corrected (adjusted) p-value controls this: \code{correction = "fwer"} bounds
the probability of \emph{any} false flag (strict, lower power), while \code{"fdr_bh"} /
\code{"fdr_by"} bound the expected \emph{proportion} of false flags among those raised
(a more lenient middle ground). Rule of thumb: use the marginal p-value for a
single pre-specified comparison, and a corrected p-value when screening the
whole table -- the usual workflow.
}

\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_data) <- paste0("Item", 1:10)
  dif_group <- factor(sample(c("A", "B"), 200, replace = TRUE))

  # Default kable output
  RMdifGamma(sim_data, dif_group)

  # Return as data.frame
  RMdifGamma(sim_data, dif_group, output = "dataframe")

  # Simulation-based cutoffs (100 Monte-Carlo iterations)
  if (requireNamespace("ggdist", quietly = TRUE)) {
    cutoff_res <- RMdifGammaCutoff(sim_data, dif_var = dif_group,
                                   iterations = 100, parallel = FALSE,
                                   seed = 42)
    RMdifGamma(sim_data, dif_group, cutoff = cutoff_res)

    # Bootstrap p-values with family-wise (Westfall-Young) correction
    # (use iterations >= 1000 in real analyses for stable p-values)
    RMdifGamma(sim_data, dif_group, cutoff = cutoff_res, p_value = TRUE,
               output = "dataframe")
  }
}
}
}
\references{
Bjorner, J. B., Kreiner, S., Ware, J. E., Damsgaard, M. T., &
Bech, P. (1998). Differential item functioning in the Danish translation
of the SF-36. \emph{Journal of Clinical Epidemiology, 51}(11), 1189--1202.
\doi{10.1016/S0895-4356(98)00111-5}

Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number
of hypotheses simultaneously. \emph{Journal of Statistical Theory and Practice,
19}(6). \doi{10.1007/s42519-024-00412-4}

Westfall, P. H., & Young, S. S. (1993). \emph{Resampling-Based Multiple Testing}.
Wiley.
}
\seealso{
\code{\link{RMdifGammaCutoff}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_plots.R
\name{RMplotBar}
\alias{RMplotBar}
\title{Item Response Distribution Bar Chart}
\usage{
RMplotBar(
  data,
  item_labels = NULL,
  category_labels = NULL,
  ncol = 1L,
  label_wrap = 25L,
  text_y = 6,
  viridis_option = "A",
  viridis_end = 0.9,
  font = "sans"
)
}
\arguments{
\item{data}{A data.frame in wide format containing only the item
response columns. Each column is one item, each row is one person.
All columns must be numeric (integer-valued). Response categories
may be coded starting from 0 or 1. Do not include person IDs,
grouping variables, or other non-item columns.}

\item{item_labels}{An optional character vector of descriptive labels
for the items (facet strips). Must be the same length as
\code{ncol(data)}. If \code{NULL} (the default), column names are used.
Labels are displayed as \code{"column_name - label"}.}

\item{category_labels}{An optional character vector of labels for the
response categories (x-axis). Must be the same length as the number
of response categories spanning from the minimum to the maximum
observed value. If \code{NULL} (the default), numeric category values
are used.}

\item{ncol}{Integer. Number of columns in the faceted layout.
Default is \code{1L}.}

\item{label_wrap}{Integer. Number of characters per line in facet
strip labels before wrapping. Default is \code{25L}.}

\item{text_y}{Numeric. Vertical position (in percent units) for the
count labels on each bar. Adjust upward if bars are tall. Default
is \code{6}.}

\item{viridis_option}{Character. Viridis palette option for the
count-text colour. One of \code{"A"} through \code{"H"}. Default is \code{"A"}.}

\item{viridis_end}{Numeric in \eqn{[0, 1]}. End point of the viridis
colour scale for count text. Adjust if text is hard to read against
the bar colours. Default is \code{0.9}.}

\item{font}{Character. Font family for all text. Default is
\code{"sans"}.}
}
\value{
A \link[ggplot2:ggplot]{ggplot2::ggplot} object.
}
\description{
Creates a faceted bar chart showing the response distribution for each
item, with counts and percentages displayed on each bar. Each item gets
its own panel, with response categories on the x-axis and percentage of
responses on the y-axis. This is a descriptive data visualization tool
intended for use before model fitting.
}
\details{
Each item is displayed as a separate facet panel with the item label
in the strip on the left side. Bars are coloured by response category
using the viridis palette. Each bar shows the count (\code{n = X}) as text.

The plot caption reports the sample in the standard
\verb{n = X respondents (policy)} form; item-level \code{NA}s are retained --
each bar counts the non-missing responses for that item.

\strong{Input requirements:}
\itemize{
\item All columns must be numeric (integer-valued).
\item The data frame must contain at least 2 columns (items)
and at least 1 row (person).
}
}
\examples{
if (requireNamespace("eRm", quietly = TRUE) &&
    requireNamespace("ggplot2", quietly = TRUE)) {
  data(pcmdat2, package = "eRm")

  # Basic response distribution plot
  RMplotBar(pcmdat2)

  # With custom item labels
  RMplotBar(
    pcmdat2,
    item_labels = c("Mood", "Sleep", "Appetite", "Energy")
  )

  # Two-column layout with wrapped labels
  RMplotBar(
    pcmdat2,
    item_labels = c(
      "General mood and emotional wellbeing",
      "Quality of sleep at night",
      "Appetite and eating habits",
      "Overall energy level during the day"
    ),
    ncol = 2, label_wrap = 20
  )

  # With custom category labels
  RMplotBar(
    pcmdat2,
    category_labels = c("Never", "Sometimes", "Often")
  )
}

}
\seealso{
\code{\link[=RMplotStackedbar]{RMplotStackedbar()}}, \code{\link[=RMplotTile]{RMplotTile()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/local_dependence.R
\name{RMlocdepQ3}
\alias{RMlocdepQ3}
\title{\eqn{Q_3} Residual Correlations for Local Dependence Assessment}
\usage{
RMlocdepQ3(
  data,
  cutoff = NULL,
  output = "kable",
  n_pairs = NULL,
  p_value = FALSE,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  alpha = 0.05,
  estimator = c("CML", "MML")
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Missing values (\code{NA}) are allowed.}

\item{cutoff}{Optional. \code{NULL} (default) returns the raw \eqn{Q_3}matrix. A single
numeric value returns the \eqn{Q_3} matrix with a global dynamic cut-off (the
value added to the mean off-diagonal \eqn{Q_3}). The \strong{full list} returned by
\code{\link{RMlocdepQ3Cutoff}} returns a \emph{list of two tables} (see Value):
the cut-off matrix plus a per-pair table.}

\item{output}{Character string controlling the return value. Either
\code{"kable"} (default) for formatted \code{knitr::kable()} table(s), or
\code{"dataframe"} for the underlying data.frame(s).}

\item{n_pairs}{Integer or \code{NULL} (default). When the full cutoff object is
supplied, limits the per-pair table to the \code{n_pairs} pairs with the
largest departure from their expected range. \code{NULL} shows all pairs.}

\item{p_value}{Logical. If \code{TRUE} (requires the full
\code{\link{RMlocdepQ3Cutoff}} object), the per-pair table also reports
one-sided bootstrap p-values (\code{p_q3}, \code{padj_q3}) and flags \code{above} pairs
(only) on \code{padj_q3 < alpha} instead of the expected range. Default \code{FALSE}.}

\item{correction}{Character. Multiple-comparison correction across item
pairs when \code{p_value = TRUE}: \code{"fwer"} (default) for the Westfall-Young
studentised-max step-down, \code{"fdr_bh"} / \code{"fdr_by"} for Benjamini-Hochberg
/ Benjamini-Yekutieli, or \code{"none"}.}

\item{alpha}{Numeric in (0, 1). Significance level for \code{Flagged} when
\code{p_value = TRUE}. Default \code{0.05}.}

\item{estimator}{Character. Estimation engine for the \eqn{Q_3} residual
correlations. \code{"CML"} (default) fits item parameters by conditional
maximum likelihood (\code{psychotools}) and person locations by Warm's
weighted likelihood (WLE) -- true to the Rasch tradition and finite
at extreme scores. \code{"MML"} uses the marginal-ML / EAP engine
(\code{mirt}), retained for backward compatibility. The estimator must
match the one used by \code{\link{RMlocdepQ3Cutoff}}; when the full
cutoff object is supplied, its stored estimator takes precedence (a
mismatching \code{estimator} argument is overridden with a warning).}
}
\value{
With \code{cutoff = NULL} or a bare numeric cut-off, a single object (\code{kable} or
data.frame) holding the lower triangle of the \eqn{Q_3} matrix; a numeric cut-off
adds a \code{Flagged} row flag and a caption describing the dynamic
cut-off.

With the \strong{full \code{RMlocdepQ3Cutoff()} object}, a named list of two:
\describe{
\item{\verb{$matrix}}{the \eqn{Q_3} lower-triangle matrix with the \emph{global} dynamic
cut-off (mean off-diagonal \eqn{Q_3} + suggested cut-off), as above.}
\item{\verb{$pairs}}{one row per item pair: \code{Item1}, \code{Item2}, \code{Observed} (\eqn{Q_3}),
\code{Low}/\code{High} (the per-pair expected range, i.e. the simulated bounds),
and \code{Flagged} -- \code{"above"} (\eqn{Q_3} above the upper bound, indicating local
dependence), \code{"below"} (below the lower bound), or \code{""}. Sorted by
absolute departure from the per-pair simulated median and truncated to
\code{n_pairs}. With \code{p_value = TRUE}, columns \code{p_q3} and \code{padj_q3} are added
and \code{Flagged} reflects \code{padj_q3 < alpha} and \strong{only flags \code{"above"}}.}
}
The \eqn{Q_3} tile heatmap that earlier versions returned as \verb{$plot} is now produced
by \code{\link{RMlocdepQ3Plot}} (as its \verb{$matrix} element), so the table and
plot outputs share the same \verb{$matrix}/\verb{$pairs} structure.
}
\description{
Computes Yen's \eqn{Q_3} residual correlations between item pairs. By default
the Rasch model is fitted by conditional maximum likelihood with WLE person
locations (\code{estimator = "CML"}); marginal ML via \code{mirt} is available with
\code{estimator = "MML"}. High correlations (above the dynamic cut-off) indicate
potential local dependence between items. See \code{\link{RMlocdepQ3Cutoff}}
for how to determine the appropriate dynamic cut-off for your data.
}
\details{
The \eqn{Q_3} statistic (Yen, 1984) is the correlation between residuals of pairs
of items after accounting for the latent trait. Under local independence,
\eqn{Q_3} values are expected to be around \eqn{-1/(k-1)} where \eqn{k} is the
number of items. When \code{cutoff} is supplied, the dynamic cut-off is the mean
of all off-diagonal \eqn{Q_3} values plus \code{cutoff}, following the approach of
Christensen et al. (2017). Use \code{\link{RMlocdepQ3Cutoff}} to obtain a
simulation-based cutoff recommendation.

\eqn{Q_3} is the column-wise correlation matrix of the model standardized
residuals \eqn{(x - E)/\sqrt{Var}}. By default (\code{estimator = "CML"}) item
parameters are estimated by conditional maximum likelihood (\code{psychotools})
and person locations by Warm's weighted likelihood; \code{estimator = "MML"}
instead uses \code{mirt}'s marginal-ML model and its built-in \eqn{Q_3} residuals.
The two estimators give very similar \eqn{Q_3} values (off-diagonal correlation
typically > 0.95); what matters for inference is that the observed \eqn{Q_3} and
the simulated cut-off in \code{\link{RMlocdepQ3Cutoff}} use the \emph{same}
estimator, which the functions enforce.

\strong{Two views of local dependence.} Given the full
\code{\link{RMlocdepQ3Cutoff}} object, two complementary tables are returned.
The \verb{$matrix} applies a single \emph{global} cut-off (the Christensen et al.
approach: the 99th percentile of the simulated max-minus-mean \eqn{Q_3}) -- a
family-wise "is there any local dependence" overview. The \verb{$pairs} table is
the per-comparison view: each observed \eqn{Q_3} against its own simulated expected
range (the \code{Low}/\code{High} bounds), so individual dependent pairs can be read
off and ranked.

\strong{Bootstrap p-values.} When \code{p_value = TRUE}, the \verb{$pairs} table also
tests each observed \eqn{Q_3} against its simulated null (from \code{cutoff$pair_results})
with a one-sided (upper-tail) test for excess local dependence. The pair
statistic is studentised by the bootstrap mean and SD; the marginal p-value
is \verb{(1 + #\{Q3* >= Q3\}) / (B + 1)}, and \code{correction} applies the family-wise
(Westfall-Young step-down) or FDR adjustment across the \eqn{k(k-1)/2} pairs.
As for item fit, the family-wise correction is liberal when the simulation is
small, so >= 1000 \code{iterations} in \code{\link[=RMlocdepQ3Cutoff]{RMlocdepQ3Cutoff()}} are recommended (a
warning is issued otherwise).
}
\section{Multiple comparisons}{

The marginal p-value controls the error rate of a \emph{single} comparison: for
one item (or item pair) decided on in advance it is the relevant value. But
scanning all \emph{k} comparisons and flagging whichever fall below \code{alpha} tests
\emph{k} hypotheses at once, so the chance of at least one false flag inflates to
roughly \eqn{1 - (1 - \alpha)^k} (e.g. about 34\% for \emph{k} = 8 at
\code{alpha = 0.05}) -- even when every marginal p-value is correctly calibrated.
The corrected (adjusted) p-value controls this: \code{correction = "fwer"} bounds
the probability of \emph{any} false flag (strict, lower power), while \code{"fdr_bh"} /
\code{"fdr_by"} bound the expected \emph{proportion} of false flags among those raised
(a more lenient middle ground). Rule of thumb: use the marginal p-value for a
single pre-specified comparison, and a corrected p-value when screening the
whole table -- the usual workflow.
}

\examples{
\donttest{
# Simulate binary item response data (10 items, 200 persons)
set.seed(42)
sim_data <- as.data.frame(
  matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
)
colnames(sim_data) <- paste0("Item", 1:10)

# Raw Q3 matrix (no cutoff)
RMlocdepQ3(sim_data)

# Get the underlying data.frame
q3_df <- RMlocdepQ3(sim_data, output = "dataframe")

# Simulation-based cutoff (use 500+ iterations in real analyses)
if (requireNamespace("ggdist", quietly = TRUE)) {
  cutoff_res <- RMlocdepQ3Cutoff(sim_data, iterations = 50, parallel = FALSE)

  # Bare numeric cutoff -> just the matrix
  RMlocdepQ3(sim_data, cutoff = cutoff_res$suggested_cutoff)

  # Full object -> list of two tables: $matrix and $pairs
  res <- RMlocdepQ3(sim_data, cutoff = cutoff_res, output = "dataframe")
  res$pairs

  # Top 5 pairs, with bootstrap p-values (use iterations >= 1000 in practice)
  RMlocdepQ3(sim_data, cutoff = cutoff_res, n_pairs = 5, p_value = TRUE,
             output = "dataframe")$pairs
}
}
}
\references{
Yen, W. M. (1984). Effects of local item dependence on the fit and
equating performance of the three-parameter logistic model.
\emph{Applied Psychological Measurement, 8}(2), 125--145.
\doi{10.1177/014662168400800201}

Christensen, K. B., Makransky, G., & Horton, M. (2017). Critical values
for Yen's \eqn{Q_3}: Identification of local dependence in the Rasch model.
\emph{Applied Psychological Measurement, 41}(3), 178--194.
\doi{10.1177/0146621616677520}

Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number
of hypotheses simultaneously. \emph{Journal of Statistical Theory and Practice,
19}(6). \doi{10.1007/s42519-024-00412-4}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dif_partgam.R
\name{RMdifGammaPlot}
\alias{RMdifGammaPlot}
\title{Plot Distribution of Simulated Partial Gamma DIF Values}
\usage{
RMdifGammaPlot(simfit, data, dif_var)
}
\arguments{
\item{simfit}{The return value of \code{\link{RMdifGammaCutoff}} (a list with
components \code{results}, \code{item_cutoffs}, \code{actual_iterations}, \code{sample_n}, and
\code{item_names}).}

\item{data}{Optional. A data.frame or matrix of item responses for computing
and overlaying observed partial gamma values. Items must be scored starting
at 0 (non-negative integers). When provided, the plot includes orange
diamond markers for the observed partial gamma alongside the simulated
distribution, plus segment summaries from the cutoff intervals.}

\item{dif_var}{Required when \code{data} is supplied. A vector (factor, character,
or integer) defining group membership for the DIF analysis. Must have the
same length as \code{nrow(data)}.}
}
\value{
A \code{ggplot} object.
}
\description{
Visualises the distribution of simulation-based partial gamma DIF values
from \code{\link{RMdifGammaCutoff}}, optionally overlaying observed partial
gamma values computed from real data via \code{\link[iarm]{partgam_DIF}}.
}
\details{
Uses \code{ggdist::stat_dotsinterval()} (when \code{data} is not supplied) or
\code{ggdist::stat_dots()} (when \code{data} is supplied) with
\code{point_interval = "median_hdci"} and \code{.width = c(0.66, 0.95, 0.99)}.

When \code{data} is \strong{not} supplied, the function plots the simulated partial
gamma distributions as dot-interval plots using
\code{ggdist::stat_dotsinterval()} with median and Highest Density Continuous
Interval (HDCI) summaries.

When \code{data} \strong{is} supplied (along with \code{dif_var}), the function:
\enumerate{
\item Computes observed partial gamma values via
\code{iarm::partgam_DIF()}.
\item Overlays observed gamma values as orange diamond markers on the
simulated distributions.
\item Shows per-item cutoff intervals (from \code{simfit$item_cutoffs}) as
black line segments, with thicker segments for the 66\\% interval and
black dots for the median.
}

The \code{ggplot2}, \code{ggdist}, and optionally \code{iarm} packages must be installed
(they are in Suggests, not Imports).
}
\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE) &&
    requireNamespace("ggdist", quietly = TRUE) &&
    requireNamespace("ggplot2", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_data) <- paste0("Item", 1:10)
  dif_group <- factor(sample(c("A", "B"), 200, replace = TRUE))

  # Run simulation
  cutoff_res <- RMdifGammaCutoff(sim_data, dif_var = dif_group,
                                 iterations = 100, parallel = FALSE,
                                 seed = 42)

  # Simulated distribution only
  RMdifGammaPlot(cutoff_res)

  # With observed partial gamma overlaid
  RMdifGammaPlot(cutoff_res, data = sim_data, dif_var = dif_group)
}
}
}
\seealso{
\code{\link{RMdifGammaCutoff}}, \code{\link{RMdifGamma}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ld_partgam.R
\name{RMlocdepGamma}
\alias{RMlocdepGamma}
\title{Partial Gamma Local Dependence Analysis}
\usage{
RMlocdepGamma(
  data,
  cutoff = NULL,
  p_value = FALSE,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  alpha = 0.05,
  output = "kable",
  n_pairs = NULL
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Missing values (\code{NA}) are allowed,
but at least one complete case must exist.}

\item{cutoff}{Optional. Default \code{NULL} (no cutoff applied). Can be:
\itemize{
\item The return value of \code{\link{RMlocdepGammaCutoff}} (a list with
\verb{$pair_cutoffs}): the data.frame is extracted automatically and
simulation metadata is included in the kable caption.
\item The \verb{$pair_cutoffs} data.frame from \code{\link{RMlocdepGammaCutoff}}
directly: must have columns \code{Item1}, \code{Item2}, \code{gamma_low}, \code{gamma_high}.
When provided, adds columns \code{Gamma_low}, \code{Gamma_high}, and \code{Flagged}
(logical; \code{TRUE} when the observed partial gamma falls outside the
credible range) to the result.
}}

\item{p_value}{Logical. When \code{TRUE}, adds one-sided bootstrap p-values for
\emph{excess positive} local dependence (\code{p_gamma}, \code{padj_gamma}), matching
the \code{p_value} semantics of \code{\link{RMlocdepQ3}}, and \code{flagged}
reflects \code{padj_gamma < alpha} (positive deviations only) instead of the
credible range. One test per item pair: the p-value is computed in the
canonical direction (direction 1, rest score = total - Item2, the
direction that was simulated) and repeated in the direction-2 table for
the same pair. The asymptotic BH-adjusted p-value and star columns from
\code{iarm::partgam_LD()} are \strong{dropped} in this mode; the simulated
\code{gamma_low} / \code{gamma_high} band is kept as the effect-size reference.
Requires the \strong{full} \code{\link{RMlocdepGammaCutoff}} object as
\code{cutoff} (it carries the simulated distributions in \verb{$results}).
Default \code{FALSE}.}

\item{correction}{Character. Multiplicity correction for the bootstrap
p-values, applied over the family of all item pairs (before any
\code{n_pairs} display filter): \code{"fwer"} (default; Westfall-Young
studentised-max step-down), \code{"fdr_bh"}, \code{"fdr_by"}, or \code{"none"}. Ignored
when \code{p_value = FALSE}.}

\item{alpha}{Numeric in (0, 1). Significance level used to flag pairs on
the corrected p-value. Default \code{0.05}. Ignored when \code{p_value = FALSE}.}

\item{output}{Character string controlling the return value. Either
\code{"kable"} (default) for a formatted \code{knitr::kable()} table, or
\code{"dataframe"} for the underlying data.frame.}

\item{n_pairs}{Optional positive integer. When supplied, only the
\code{n_pairs} item pairs with the largest absolute partial-gamma values
(i.e., strongest residual dependence in either direction) are
retained per rest-score direction, sorted by \verb{|gamma|} descending.
When \code{NULL} (default), all pairs are returned in \code{iarm}'s native
ordering. Values larger than the total number of pairs are silently
capped at that total.}
}
\value{
\itemize{
\item If \code{output = "kable"}: an object of class \code{"RMlocdepGamma"}. Internally
a list with two \code{knitr_kable} elements, \verb{$direction1} and
\verb{$direction2}. In both, the rest score is the total score minus
Item 2 (the second column); the two elements list each item pair in
the two possible orders, so together they cover both rest-score
directions for every pair. Each has columns "Item 1", "Item 2",
"Partial gamma",
"Adj. p-value (BH)", and "p-value sign." (a star-string indicator
from \code{iarm::partgam_LD()}). When \code{cutoff} is provided, additional
columns "Gamma low", "Gamma high", and "Flagged" are included.

The object has custom \code{print()} and \code{knitr::knit_print()} methods:
in the R console it prints the two tables stacked vertically; in a
Quarto / R Markdown chunk it renders as two distinct pipe tables.
Access the individual tables explicitly as \code{result$direction1} and
\code{result$direction2} if needed.
\item If \code{output = "dataframe"}: a named list of two data.frames
(\verb{$direction1}, \verb{$direction2}) with columns \code{Item1}, \code{Item2},
\code{gamma}, \code{se}, \code{lower}, \code{upper} (95\% Wald CI), \code{padj_bh},
\code{Significance}. When \code{cutoff} is provided, columns \code{gamma_low},
\code{gamma_high}, and \code{flagged} are also included.
With \code{p_value = TRUE}, \code{padj_bh} and \code{Significance} are replaced by
\code{p_gamma} and \code{padj_gamma} (identical for a pair in both directions).
}
}
\description{
Computes partial gamma coefficients for Local Dependence (LD) assessment
using \code{iarm::partgam_LD()}. Each pair of items is tested for residual
association, controlling for the rest score (total score minus one of the
items in the pair).
}
\details{
Partial gamma (Christensen, Kreiner & Mesbah, 2013) measures the residual
association between pairs of items after controlling for the rest score
(total score minus one item). Because it matters which item is subtracted,
calculations are done for each pair in both directions, yielding two
data.frames.

Values near 0 indicate no local dependence. Large positive values suggest
positive LD (items share variance beyond the latent trait), while large
negative values suggest negative LD.

The \code{iarm} package must be installed (it is in Suggests, not Imports).

\strong{Bootstrap p-values.} When \code{p_value = TRUE}, each pair's observed
partial gamma (canonical direction) is compared against its simulated null
distribution (from \code{cutoff$results}, simulated under local independence).
The per-pair statistic is the residual studentised by the bootstrap mean
and SD; the marginal p-value is the one-sided Monte-Carlo p-value
\verb{(1 + #\\\{t* >= t\\\}) / (B + 1)} for excess \emph{positive} LD (redundancy, the
diagnostic target — matching \code{\link{RMlocdepQ3}}), so it can be no
smaller than \code{1 / (B + 1)}. The band still shows both bounds for
reference. \code{correction = "fwer"} uses the Westfall-Young studentised-max
step-down over the family of all pairs, which exploits the bootstrap
dependence among them (Ferreira, 2024); it is liberal when the simulation
is small, so at least 1000 \code{iterations} in \code{\link[=RMlocdepGammaCutoff]{RMlocdepGammaCutoff()}} are
recommended (a warning is issued below that). Unlike the asymptotic
p-values from \code{iarm::partgam_LD()}, these are calibrated against the
\emph{simulated Rasch null} rather than the asymptotic SE; they are
model-conditional and sample-size-sensitive, and are reported alongside
the simulated effect-size band, not in place of it.
}
\section{Multiple comparisons}{

The marginal p-value controls the error rate of a \emph{single} comparison: for
one item (or item pair) decided on in advance it is the relevant value. But
scanning all \emph{k} comparisons and flagging whichever fall below \code{alpha} tests
\emph{k} hypotheses at once, so the chance of at least one false flag inflates to
roughly \eqn{1 - (1 - \alpha)^k} (e.g. about 34\% for \emph{k} = 8 at
\code{alpha = 0.05}) -- even when every marginal p-value is correctly calibrated.
The corrected (adjusted) p-value controls this: \code{correction = "fwer"} bounds
the probability of \emph{any} false flag (strict, lower power), while \code{"fdr_bh"} /
\code{"fdr_by"} bound the expected \emph{proportion} of false flags among those raised
(a more lenient middle ground). Rule of thumb: use the marginal p-value for a
single pre-specified comparison, and a corrected p-value when screening the
whole table -- the usual workflow.
}

\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_data) <- paste0("Item", 1:10)

  # Default kable output
  RMlocdepGamma(sim_data)

  # Return as data.frame list
  RMlocdepGamma(sim_data, output = "dataframe")

  # Simulation-based cutoffs (slow): 100+ Monte-Carlo iterations
  if (requireNamespace("ggdist", quietly = TRUE)) {
    cutoff_res <- RMlocdepGammaCutoff(sim_data, iterations = 100,
                                      parallel = FALSE, seed = 42)
    RMlocdepGamma(sim_data, cutoff = cutoff_res)

    # Bootstrap p-values with family-wise (Westfall-Young) correction
    # (use iterations >= 1000 in real analyses for stable p-values)
    RMlocdepGamma(sim_data, cutoff = cutoff_res, p_value = TRUE,
                  output = "dataframe")
  }
}
}
}
\references{
Christensen, K. B., Kreiner, S. & Mesbah, M. (Eds.) (2013).
\emph{Rasch Models in Health}, pp. 133--135. ISTE & Wiley.
\doi{10.1002/9781118574454}

Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number
of hypotheses simultaneously. \emph{Journal of Statistical Theory and Practice,
19}(6). \doi{10.1007/s42519-024-00412-4}

Westfall, P. H., & Young, S. S. (1993). \emph{Resampling-Based Multiple Testing}.
Wiley.
}
\seealso{
\code{\link{RMlocdepGammaCutoff}}, \code{\link{RMlocdepGammaPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ld_partgam.R
\name{run_partgam_LD_sim_parallel}
\alias{run_partgam_LD_sim_parallel}
\title{Run partial gamma LD simulations in parallel using mirai}
\usage{
run_partgam_LD_sim_parallel(
  iterations,
  sim_seeds,
  sim_data_list,
  n_cores,
  verbose = FALSE
)
}
\arguments{
\item{iterations}{Number of iterations.}

\item{sim_seeds}{Integer vector of per-iteration seeds.}

\item{sim_data_list}{List of data passed to each worker.}

\item{n_cores}{Number of mirai daemons.}

\item{verbose}{Show progress bar.}
}
\value{
List of raw results (one element per iteration).
}
\description{
Run partial gamma LD simulations in parallel using mirai
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/targeting_plot.R
\name{RMtargeting}
\alias{RMtargeting}
\title{Person-Item Targeting Plot (Wright Map)}
\usage{
RMtargeting(
  data,
  robust = FALSE,
  sort_items = c("data", "location"),
  bins,
  xlim = c(-4, 4),
  ci_level = 0.95,
  person_fill = "#0072B2",
  threshold_fill = "#D55E00",
  height_ratios = c(3, 2, 5),
  output = "patchwork"
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Missing values (\code{NA}) are allowed.}

\item{robust}{Logical. If \code{FALSE} (the default), histogram annotations use
mean ± SD. If \code{TRUE}, median ± MAD is used instead.}

\item{sort_items}{Character string controlling item ordering on the y-axis
of the bottom panel. \code{"data"} (the default) preserves the column order in
\code{data} (first item at top). \code{"location"} sorts items by their average
threshold location (easiest at top, hardest at bottom).}

\item{bins}{Integer. Number of bins for both histograms. Default is number of
unique scores divided by 2, but no less than 11.}

\item{xlim}{Numeric vector of length 2. Initial lower and upper limits for
the shared x-axis. Automatically expanded if any person or item threshold
values fall outside these limits.}

\item{ci_level}{Numeric. Confidence level for the item threshold error bars.
Default is \code{0.95} (95\% CI). Set to \code{NULL} to hide error bars.}

\item{person_fill}{Fill colour for the person histogram. Default
\code{"#0072B2"} (blue).}

\item{threshold_fill}{Fill colour for the item threshold histogram. Default
\code{"#D55E00"} (vermillion).}

\item{height_ratios}{Numeric vector of length 3 specifying the relative
heights of the top (person), middle (threshold), and bottom (dot-whisker)
panels. Default \code{c(3, 2, 5)}.}

\item{output}{Character string. \code{"patchwork"} (the default) returns the
combined patchwork plot. \code{"list"} returns a named list of the three
ggplot objects (\code{p1}, \code{p2}, \code{p3}) for further customisation.}
}
\value{
\itemize{
\item If \code{output = "patchwork"}: a \code{patchwork} object (combined \code{ggplot}).
\item If \code{output = "list"}: a named list with elements \code{p1} (person histogram),
\code{p2} (threshold histogram), and \code{p3} (item threshold dot-whisker plot).
}
}
\description{
Produces a three-panel targeting plot with a shared logit scale x-axis:
\enumerate{
\item \strong{Top}: Histogram of person location estimates, with a
reference line for the mean (or median) and shading for ±1 SD
(or ±1 MAD).
\item \strong{Middle}: Inverted histogram of item threshold locations,
with the same summary annotations.
\item \strong{Bottom}: Dot-and-whisker plot of individual item thresholds
with confidence intervals based on threshold standard errors.
}
}
\details{
Together, the top and middle panels form a back-to-back histogram that
makes it easy to assess whether the test is well-targeted to the sample.

\strong{Estimation method selection.}
The function checks whether any item response category has fewer than 3
observations. If all categories have at least 3 responses, item threshold
locations and their standard errors are estimated via Conditional Maximum
Likelihood (CML) using \code{psychotools::pcmodel()} (a dichotomous item is a
2-category PCM). If any category has fewer than 3 responses, the function
falls back to Marginal Maximum Likelihood (MML) estimation via
\code{mirt::mirt()} with \code{itemtype = "Rasch"} and \code{SE = TRUE}, which is more
numerically stable under sparse-category conditions. A message is emitted
when the MML fallback is used.

In both cases, item threshold locations are centered (shifted so the grand
mean of all thresholds equals zero).

\strong{Person estimates} are obtained by Warm's weighted likelihood (WLE)
from the fitted item thresholds, consistent with the rest of the package.
WLE is finite at extreme scores, so all-zero and perfect responders are
located rather than dropped.

\strong{Confidence intervals} for item thresholds are based on Wald-type
intervals: threshold estimate ± z × SE, where z is the standard normal
quantile corresponding to \code{ci_level}.

The \code{ggplot2} and \code{patchwork} packages must be installed (they are in
Suggests, not Imports).
}
\examples{
\donttest{
if (requireNamespace("ggplot2", quietly = TRUE) &&
    requireNamespace("patchwork", quietly = TRUE)) {
  # Polytomous example
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:3, 200 * 8, replace = TRUE), nrow = 200, ncol = 8)
  )
  colnames(sim_data) <- paste0("Item", 1:8)

  # Default: mean/SD, data order, 95\% CI
  RMtargeting(sim_data)

  # Robust (median/MAD), sorted by location, 84\% CI
  RMtargeting(sim_data, robust = TRUE, sort_items = "location",
              ci_level = 0.84)

  # Get list of sub-plots for customisation
  plots <- RMtargeting(sim_data, output = "list")
  plots$p1 + ggplot2::ggtitle("My custom title")

  # Dichotomous example
  sim_bin <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_bin) <- paste0("Item", 1:10)
  RMtargeting(sim_bin)
}
}
}
\references{
Wright, B. D. & Stone, M. H. (1979). \emph{Best Test Design}. MESA Press.
}
\seealso{
\code{\link[psychotools:pcmodel]{psychotools::pcmodel()}}, \code{\link[mirt:mirt]{mirt::mirt()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_infit_mi.R
\name{RMitemInfitMI}
\alias{RMitemInfitMI}
\title{Conditional Item Infit MSQ for Multiply Imputed Data}
\usage{
RMitemInfitMI(mids_object, cutoff = NULL, output = "kable", sort)
}
\arguments{
\item{mids_object}{A \code{mids} object (multiply imputed dataset) as returned by
\code{mice::mice()}. Each completed dataset must contain only the item response
columns to be analysed (i.e., no ID or grouping variables). Items must be
scored starting at 0 (non-negative integers).}

\item{cutoff}{Optional. Default \code{NULL} (no cutoff applied). Can be:
\itemize{
\item The return value of \code{\link{RMitemInfitCutoff}} or
\code{\link{RMitemInfitCutoffMI}} (a list with \verb{$item_cutoffs}): the
data.frame is extracted automatically and metadata is included in the
kable caption.
\item The \verb{$item_cutoffs} data.frame directly: must have columns \code{Item},
\code{infit_low}, and \code{infit_high}.
When provided, adds columns \code{Infit_low}, \code{Infit_high}, and \code{Flagged} to
the result. \code{Flagged} is a character column labelling the misfit
direction: \code{"overfit"} (pooled infit below the range), \code{"underfit"}
(above), or \code{""} (within range).
}}

\item{output}{Character string controlling the return value. Either
\code{"kable"} (default) for a formatted \code{knitr::kable()} table, or
\code{"dataframe"} for the underlying data.frame.}

\item{sort}{Optional character string. When \code{sort = "infit"}, rows are
sorted by \code{Infit_MSQ} in descending order before output.}
}
\value{
\itemize{
\item If \code{output = "kable"}: a \code{knitr_kable} object (plain text table via
\code{format = "pipe"}) with columns "Item", "Infit MSQ", "Infit SE",
"Relative location", and a caption noting the number of imputations and
complete cases. When \code{cutoff} is provided, columns "Infit low",
"Infit high", and "Flagged" are also included.
\item If \code{output = "dataframe"}: a data.frame with columns \code{Item},
\code{Infit_MSQ}, \code{Infit_SE}, and \code{Relative_location}. When \code{cutoff} is
provided, columns \code{Infit_low}, \code{Infit_high}, and \code{Flagged} are also
included (inserted after \code{Infit_SE}, before \code{Relative_location}).
\code{Flagged} is a character column (\code{"overfit"} / \code{"underfit"} / \code{""}),
not the previous logical.
}
}
\description{
Extends \code{\link{RMitemInfit}} to work with multiply imputed datasets
produced by the \code{mice} package. Computes conditional infit MSQ on each
imputed dataset and pools the results using Rubin's rules.
}
\details{
For each of the \code{m} imputed datasets, the function:
\enumerate{
\item Fits a Rasch model by CML via \code{psychotools::pcmodel()} (a
dichotomous item is a 2-category partial credit model), consistent with
\code{\link{RMitemInfit}} and the rest of the package.
\item Computes conditional infit MSQ and its standard error via
\code{iarm::out_infit()}.
\item Computes item locations (mean of the grand-mean-centred CML
Andrich thresholds) and the mean WLE person location.
}

The per-imputation estimates are then pooled using Rubin's rules:
\describe{
\item{Pooled MSQ}{The mean of the \code{m} infit MSQ point estimates.}
\item{Within-imputation variance}{The mean of the \code{m} squared standard
errors.}
\item{Between-imputation variance}{The sample variance of the \code{m} point
estimates.}
\item{Total variance}{Within + (1 + 1/m) * Between.}
\item{Pooled SE}{The square root of the total variance.}
}

Relative item location is the mean of per-imputation relative locations
(item location minus sample mean person location).

\strong{Caveat on the pooled SE.} The within-imputation variance is the
squared conditional infit SE from \code{iarm::out_infit()}. Müller (2020) showed
that this asymptotic SE is an unreliable measure of uncertainty for the
conditional infit statistic; Rubin's pooled SE inherits that limitation, so
the \code{Infit_SE}/\verb{Infit SE} column should be read as an approximate indication
of imputation-related variability rather than a trustworthy inferential
standard error. For item misfit decisions, prefer the simulation-based
cutoffs from \code{\link{RMitemInfitCutoffMI}}.

Imputed datasets that cause model convergence failures are dropped with a
warning. If all imputations fail, the function stops with an error. At
least two successful imputations are required to estimate between-imputation
variance.

The \code{mice} and \code{iarm} packages must be installed (they are in Suggests, not
Imports).
}
\examples{
\donttest{
if (requireNamespace("mice", quietly = TRUE) &&
    requireNamespace("iarm", quietly = TRUE) &&
    requireNamespace("ggdist", quietly = TRUE)) {
  # Create example data with ~10\% MCAR missingness
  set.seed(42)
  mat <- matrix(sample(0:1, 200 * 8, replace = TRUE), nrow = 200, ncol = 8)
  mat[sample(length(mat), round(0.10 * length(mat)))] <- NA
  sim_data <- as.data.frame(mat)
  colnames(sim_data) <- paste0("Item", 1:8)

  # mice's ordinal method (`polr`) requires the items to be ordered
  # factors, so code them as such before imputing. RMitemInfitMI()
  # converts the completed factors back to numeric internally.
  sim_data[] <- lapply(sim_data, function(x) factor(x, ordered = TRUE))

  # Impute (use more imputations, e.g. m = 5+, in real analyses)
  imp <- mice::mice(sim_data, m = 2, method = "polr", seed = 123,
                    printFlag = FALSE)

  # Pooled infit table (no cutoffs)
  RMitemInfitMI(imp)

  # With simulation-based cutoffs
  # (use more iterations, e.g. 250+, in real analyses)
  cutoff_mi <- RMitemInfitCutoffMI(imp, iterations = 50, parallel = FALSE,
                                seed = 42)
  RMitemInfitMI(imp, cutoff = cutoff_mi)

  # As data.frame
  df <- RMitemInfitMI(imp, cutoff = cutoff_mi, output = "dataframe")
}
}
}
\references{
Müller, M. (2020). Item fit statistics for Rasch analysis: Can we trust
them? \emph{Journal of Statistical Distributions and Applications}, 7(5).
\doi{10.1186/s40488-020-00108-7}
}
\seealso{
\code{\link{RMitemInfit}}, \code{\link{RMitemInfitCutoffMI}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_infit.R
\name{RMitemInfit}
\alias{RMitemInfit}
\title{Conditional Item Infit MSQ}
\usage{
RMitemInfit(
  data,
  cutoff = NULL,
  p_value = FALSE,
  correction = c("fwer", "fdr_bh", "fdr_by", "none"),
  alpha = 0.05,
  output = "kable",
  sort
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Missing values (\code{NA}) are allowed,
but at least one complete case (row with no \code{NA}) must be present.}

\item{cutoff}{Optional. Default \code{NULL} (no cutoff applied, behaviour is
identical to the current version). Can be:
\itemize{
\item The return value of \code{\link{RMitemInfitCutoff}} (a list with \verb{$item_cutoffs}):
the data.frame is extracted automatically and the number of simulation
iterations, \code{cutoff_method}, and \code{hdci_width} are included in the kable
caption.
\item The \verb{$item_cutoffs} data.frame from \code{\link{RMitemInfitCutoff}} directly: must
have columns \code{Item}, \code{infit_low}, and \code{infit_high}.
When provided, adds columns \code{Infit_low}, \code{Infit_high}, and \code{Flagged}
to the result. \code{Flagged} labels the misfit direction: \code{"overfit"}
(infit below the range -- more predictable than the model expects),
\code{"underfit"} (above the range -- noisier than expected), or \code{""} (within
range, no misfit).
}}

\item{p_value}{Logical. If \code{TRUE}, bootstrap p-values are computed from the
simulated null distribution and added to the output. This requires
\code{cutoff} to be the \strong{full} \code{\link{RMitemInfitCutoff}} object (which
carries the per-item simulated values in its \verb{$results} element); the
summarised \verb{$item_cutoffs} data.frame is not sufficient. Default \code{FALSE},
in which case behaviour is unchanged.}

\item{correction}{Character. Multiple-comparison correction applied across
items when \code{p_value = TRUE}: \code{"fwer"} (default) for the Westfall-Young
studentised-max step-down (family-wise error rate), \code{"fdr_bh"} /
\code{"fdr_by"} for Benjamini-Hochberg / Benjamini-Yekutieli false discovery
rate control, or \code{"none"} for uncorrected per-item p-values.}

\item{alpha}{Numeric in (0, 1). Significance level for the \code{Flagged} column
when \code{p_value = TRUE} (an item is flagged when its corrected p-value is
below \code{alpha}). Default \code{0.05}.}

\item{output}{Character string controlling the return value. Either
\code{"kable"} (default) for a formatted \code{knitr::kable()} table, or
\code{"dataframe"} for the underlying data.frame.}

\item{sort}{Optional character string. When \code{sort = "infit"}, rows are
sorted by \code{Infit_MSQ} in descending order before output.}
}
\value{
\itemize{
\item If \code{output = "kable"}: a \code{knitr_kable} object (plain text table via
\code{format = "pipe"}) with columns "Item", "Infit MSQ", and "Relative
location", and a caption showing the number of complete cases. When
\code{cutoff} is provided, columns "Infit low", "Infit high", and "Flagged"
are also included, and the caption notes the simulation-based cutoffs.
\item If \code{output = "dataframe"}: a data.frame with columns \code{Item},
\code{Infit_MSQ}, and \code{Relative_location}. When \code{cutoff} is provided, columns
\code{Infit_low}, \code{Infit_high}, and \code{Flagged} are also included (inserted
after \code{Infit_MSQ}, before \code{Relative_location}). \code{Flagged} is a character
column with values \code{"overfit"}, \code{"underfit"}, or \code{""} (not the previous
logical). When \code{p_value = TRUE}, columns \code{p_infit} (marginal two-sided
p-value) and \code{padj_infit} (corrected p-value) are added and \code{Flagged}
reflects items with \code{padj_infit < alpha} (direction from \code{Infit_MSQ}
relative to 1).
}
}
\description{
Computes conditional infit mean-square (MSQ) statistics for each item using
\code{iarm::out_infit()}, enriched with item locations relative to the sample
mean person location.
}
\details{
Infit MSQ is a weighted fit statistic that emphasises deviations near the
item location. Values close to 1.0 indicate good fit. Values substantially
above 1.0 suggest underfit (unexpected responses), while values substantially
below 1.0 suggest overfit (overly predictable responses). The definition of
"substantially" depends on several factors such as sample size, and needs to
be determined by simulation using \code{\link{RMitemInfitCutoff}}. There is no general
rule-of-thumb value that is correct.

Conditional infit MSQ statistics are computed via \code{iarm::out_infit()}, which
uses the conditional distribution of the sufficient statistics (Müller, 2020).
Only complete cases (rows without any \code{NA}) are used in the conditional fit
calculation.

Item parameters are estimated by conditional maximum likelihood via
\code{psychotools::pcmodel()} (a dichotomous item is a 2-category PCM); the
conditional infit/outfit MSQ comes from \code{iarm::out_infit()} and is invariant
to the estimation engine. Per-item average locations are the means of the
CML thresholds, and the person-location reference is the mean of the Warm
WLE estimates.

Relative item location is defined as the item's average location minus the
sample mean person location, providing a measure of item targeting.

The \code{iarm} package must be installed (it is in Suggests, not Imports).

\strong{Bootstrap p-values.} When \code{p_value = TRUE}, each item's observed
infit is compared against its simulated null distribution (from
\code{cutoff$results}). The per-item statistic is the residual studentised by
the bootstrap mean and SD -- deliberately the empirical SD rather than the
Wilson-Hilferty / ZSTD transform, which is uninformative for conditional
MSQ (Müller, 2020). The marginal p-value is the two-sided Monte-Carlo
p-value \verb{(1 + #\{|t*| >= |t|\}) / (B + 1)}. For \code{correction = "fwer"} the
family-wise adjustment uses the Westfall-Young studentised-max step-down,
which exploits the bootstrap dependence among items and is more powerful
than Bonferroni/Holm (Ferreira, 2024); its validity rests on subset
pivotality. \code{"fdr_bh"}/\code{"fdr_by"} apply Benjamini-Hochberg / Benjamini-
Yekutieli instead. These are model-conditional, sample-size-sensitive
p-values and are reported alongside the simulated effect-size band, not in
place of it. p-values can be no smaller than \code{1 / (B + 1)}, and the
studentised-max (FWER) correction is \emph{liberal} when the simulation is small
(the bootstrap mean/SD used for studentisation are then too noisy). At
least 1000 \code{iterations} in \code{\link[=RMitemInfitCutoff]{RMitemInfitCutoff()}} are recommended -- in
simulations the family-wise error rate is then controlled at the nominal
level -- and a warning is issued when the simulation is smaller. The
marginal p-values are well calibrated even at a few hundred iterations.
}
\section{Multiple comparisons}{

The marginal p-value controls the error rate of a \emph{single} comparison: for
one item (or item pair) decided on in advance it is the relevant value. But
scanning all \emph{k} comparisons and flagging whichever fall below \code{alpha} tests
\emph{k} hypotheses at once, so the chance of at least one false flag inflates to
roughly \eqn{1 - (1 - \alpha)^k} (e.g. about 34\% for \emph{k} = 8 at
\code{alpha = 0.05}) -- even when every marginal p-value is correctly calibrated.
The corrected (adjusted) p-value controls this: \code{correction = "fwer"} bounds
the probability of \emph{any} false flag (strict, lower power), while \code{"fdr_bh"} /
\code{"fdr_by"} bound the expected \emph{proportion} of false flags among those raised
(a more lenient middle ground). Rule of thumb: use the marginal p-value for a
single pre-specified comparison, and a corrected p-value when screening the
whole table -- the usual workflow.
}

\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE)) {
  # Simulate binary item response data (5 items, 40 persons)
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 40 * 5, replace = TRUE), nrow = 40, ncol = 5)
  )
  colnames(sim_data) <- paste0("Item", 1:5)

  # Default kable output
  RMitemInfit(sim_data)

  # Sorted by infit MSQ descending
  RMitemInfit(sim_data, sort = "infit")

  # Return as data.frame for further processing
  df <- RMitemInfit(sim_data, output = "dataframe")

  # Simulation-based cutoffs (100 Monte-Carlo iterations)
  if (requireNamespace("ggdist", quietly = TRUE)) {
    cutoff_res <- RMitemInfitCutoff(sim_data, iterations = 100,
                                    parallel = FALSE, seed = 42)
    RMitemInfit(sim_data, cutoff = cutoff_res)
    RMitemInfit(sim_data, cutoff = cutoff_res, output = "dataframe")

    # Bootstrap p-values with family-wise (Westfall-Young) correction
    # (use iterations >= 1000 in real analyses for stable p-values)
    RMitemInfit(sim_data, cutoff = cutoff_res, p_value = TRUE,
                output = "dataframe")
  }
}
}
}
\references{
Müller, M. (2020). Item fit statistics for Rasch analysis: Can we trust
them? \emph{Journal of Statistical Distributions and Applications}, 7(5).
\doi{10.1186/s40488-020-00108-7}

Ferreira, J. A. (2024). Methods of testing a 'small' or 'moderate' number
of hypotheses simultaneously. \emph{Journal of Statistical Theory and Practice,
19}(6). \doi{10.1007/s42519-024-00412-4}

Westfall, P. H., & Young, S. S. (1993). \emph{Resampling-Based Multiple Testing}.
Wiley.
}
\seealso{
\code{\link{RMitemInfitCutoff}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ld_partgam.R
\name{RMlocdepGammaCutoff}
\alias{RMlocdepGammaCutoff}
\title{Simulation-Based Partial Gamma LD Cutoff Determination}
\usage{
RMlocdepGammaCutoff(
  data,
  iterations = 250,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  cutoff_method = "hdci",
  hdci_width = 0.99
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Only complete cases (rows without
any \code{NA}) are used.}

\item{iterations}{Integer. Number of simulation iterations (default 250).}

\item{parallel}{Logical. Use parallel processing via \code{mirai} if available
(default \code{TRUE}).}

\item{n_cores}{Integer or \code{NULL}. Number of parallel workers. When \code{NULL},
\code{getOption("mc.cores")} is checked first. If neither is set and
\code{parallel = TRUE}, a warning is issued and execution falls back to
sequential (single core) processing.}

\item{verbose}{Logical. Show a progress bar (default \code{FALSE}).}

\item{seed}{Integer or \code{NULL}. Random seed for reproducibility.}

\item{cutoff_method}{Character string specifying how cutoff intervals are
computed. Either \code{"hdci"} (default) for the Highest Density Interval via
\code{ggdist::hdci()}, or \code{"quantile"} for the 2.5th/97.5th percentiles via
\code{stats::quantile()}.}

\item{hdci_width}{Numeric. Width of the HDCI when \code{cutoff_method = "hdci"}.
Default is \code{0.99} (99\\% HDCI). Ignored when
\code{cutoff_method = "quantile"}.}
}
\value{
A list with components:
\describe{
\item{\code{results}}{data.frame with columns \code{iteration}, \code{Item1}, \code{Item2},
and \code{gamma} (one row per item pair per successful iteration). Contains
results from direction 1 only (rest score = total - Item2), which is
the conventional direction.}
\item{\code{pair_cutoffs}}{data.frame with per-pair cutoff summaries: \code{Item1},
\code{Item2}, \code{gamma_low}, \code{gamma_high}. Bounds are computed using the method
specified by \code{cutoff_method}.}
\item{\code{actual_iterations}}{Number of successful iterations.}
\item{\code{sample_n}}{Number of complete cases used.}
\item{\code{sample_n_total}}{Number of respondents in the raw input data,
before the complete-case filter.}
\item{\code{sample_has_na}}{Logical. Whether the raw input data contained
any missing values.}
\item{\code{sample_summary}}{Summary statistics of estimated person
parameters.}
\item{\code{item_names}}{Character vector of item names from data.}
\item{\code{cutoff_method}}{The method used to compute cutoffs (\code{"hdci"} or
\code{"quantile"}).}
\item{\code{hdci_width}}{The HDCI width used (only meaningful when
\code{cutoff_method = "hdci"}).}
}
}
\description{
Uses parametric bootstrap simulation to determine appropriate cutoff values
for partial gamma Local Dependence analysis via
\code{\link[iarm]{partgam_LD}}. Under a correctly fitting Rasch model where
items are locally independent, this function generates the expected
distribution of partial gamma values per item pair, providing empirical
critical values.
}
\details{
For each simulation iteration the function:
\enumerate{
\item Resamples person parameters (thetas) with replacement from the
WLE person locations.
\item Simulates item response data under a Rasch model (dichotomous via
\code{psychotools::rrm()} or polytomous via an internal partial credit
simulator).
\item Computes partial gamma LD statistics via
\code{iarm::partgam_LD()}.
}

Because the data are simulated under the Rasch model, items are locally
independent by construction. The distribution of partial gamma values
across iterations provides empirical critical values per item pair. Values
from real data that fall outside these bounds suggest local dependence that
exceeds what would be expected by chance. Failed iterations (e.g., due to
convergence issues or degenerate data) are silently discarded.

The generating model uses CML item thresholds via \code{psychotools::pcmodel()}
(a dichotomous item is a 2-category PCM) and WLE person locations,
consistent with the rest of the package; responses are simulated with
\code{psychotools::rrm()} (dichotomous) or an internal partial credit score
simulator (polytomous).

Parallel processing is provided by the \code{mirai} package (optional). Install
it with \code{install.packages("mirai")} to enable parallelisation.

The \code{iarm} package must be installed (it is in Suggests, not Imports).
}
\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE) &&
    requireNamespace("ggdist", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_data) <- paste0("Item", 1:10)

  # Run 100 iterations sequentially for a quick demo
  cutoff_res <- RMlocdepGammaCutoff(sim_data, iterations = 100,
                                    parallel = FALSE, seed = 42)
  cutoff_res$pair_cutoffs
}
}
}
\references{
Christensen, K. B., Kreiner, S. & Mesbah, M. (Eds.) (2013).
\emph{Rasch Models in Health}, pp. 133--135. ISTE & Wiley.
\doi{10.1002/9781118574454}
}
\seealso{
\code{\link[iarm]{partgam_LD}}, \code{\link{RMlocdepGamma}},
\code{\link{RMlocdepGammaPlot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residual_pca.R
\name{RMdimResidualPCA}
\alias{RMdimResidualPCA}
\title{PCA of Standardized Rasch Residuals}
\usage{
RMdimResidualPCA(
  data,
  cutoff = NULL,
  p_value = FALSE,
  n_components = 5L,
  output = "kable"
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be scored
starting at 0 (non-negative integers). Rows with any \code{NA} are dropped
before PCA, since \code{prcomp()} does not accept missing values.}

\item{cutoff}{Optional. The list returned by \code{\link{RMdimResidualPCACutoff}} (its
\code{suggested_cutoff} is used), or a single numeric value to use as the
cutoff directly. When provided, the result includes a \code{Flagged} column
(logical: is the eigenvalue above the simulated bound?) and the kable
caption notes the cutoff.}

\item{p_value}{Logical. When \code{TRUE}, adds a one-sided bootstrap p-value for
the first-contrast eigenvalue: the proportion of simulated first-contrast
eigenvalues at least as large as the observed one,
\verb{(1 + #\\\{lambda* >= lambda\\\}) / (B + 1)}. Requires the \strong{full}
\code{\link{RMdimResidualPCACutoff}} object as \code{cutoff} (it carries the
simulated eigenvalues in \verb{$results}); a bare numeric cutoff is not
sufficient. The simulated null is the distribution of the \emph{largest}
eigenvalue, so the p-value applies to PC1 only (\code{NA} for the other
components). This is a single test — no multiplicity correction is
involved. Default \code{FALSE}.}

\item{n_components}{Integer. Number of eigenvalues to report. Capped at the
number of items. Default \code{5}.}

\item{output}{Character. \code{"kable"} (default) for a formatted
\code{knitr::kable()} table, \code{"dataframe"} for the underlying data.frame, or
\code{"ggplot"} for a ggplot of PC1 loadings against item locations.
\code{"loadings"} is accepted as a backward-compatible alias for \code{"ggplot"}.}
}
\value{
\itemize{
\item If \code{output = "kable"}: a \code{knitr_kable} object with columns Component,
Eigenvalue, Proportion of variance (and \code{Flagged} when \code{cutoff} is
provided; \code{p} when \code{p_value = TRUE}). The caption gives the variance
partition (\% of total observed variance explained by measures vs.
unexplained), the model fitted, sample size, and cutoff/p-value metadata
if applicable.
\item If \code{output = "dataframe"}: a data.frame with columns \code{Component},
\code{Eigenvalue}, \code{Proportion_of_variance} (and \code{Flagged} when \code{cutoff} is
provided; \code{p} when \code{p_value = TRUE}, non-\code{NA} for PC1 only). The
variance partition is attached as the
\code{"variance_partition"} attribute — a list with elements \code{total},
\code{explained}, \code{unexplained}, \code{pct_explained}, \code{pct_unexplained},
\code{n_persons}. Access via
\code{attr(result, "variance_partition")}.
\item If \code{output = "ggplot"}: a ggplot showing each item's PC1 loading on
the x-axis and Rasch item location on the y-axis, with dashed reference
lines at zero, and the variance partition in the figure caption. Item
names are labelled via \code{ggrepel::geom_text_repel()} when \code{ggrepel} is
installed; otherwise plain \code{geom_text()}.
}
}
\description{
Fits a Rasch model by CML via \code{psychotools} (a dichotomous item is a
2-category PCM), extracts the standardized residuals \eqn{(x - E)/\sqrt{Var}}
at WLE person locations, and runs an unrotated principal-component
analysis on those residuals via \code{stats::prcomp()}. The function reports
the top \code{n_components} eigenvalues and their proportions of unexplained
variance, and optionally compares the first-contrast eigenvalue against
a simulation-based bound from \code{\link{RMdimResidualPCACutoff}}.
}
\details{
Rule-of-thumb thresholds for the first-contrast eigenvalue (e.g., the
"> 2" heuristic occasionally cited from Winsteps documentation) are not
reliable indicators of multidimensionality; the first-contrast eigenvalue
under a correctly fitting unidimensional model varies systematically with
sample size, test length, and item-parameter spread. Empirical (simulated)
bounds tailored to the data structure should be used instead — see
\code{\link{RMdimResidualPCACutoff}}, and Chou & Wang (2010) for the underlying
simulation argument.

The PCA is performed on the standardized residuals
\eqn{(x - E)/\sqrt{Var}} from the shared CML/WLE engine (CML item
parameters via \code{psychotools}, WLE person locations). The reported
eigenvalues are unrotated; rotation is appropriate for \emph{interpreting} a
multidimensional solution but obscures the dominant first contrast that
dimensionality assessment is concerned with.

Item locations on the loadings plot are the per-item mean of the CML
Andrich thresholds.

The variance partition follows Linacre's convention: per-item observed
variance is compared to per-item \emph{expected} variance under the fitted
model, summed across items. Expected scores are computed from the CML item
parameters and WLE person locations. WLE is finite at extreme scores, so
all persons are retained (the previous MLE partition dropped extreme-score
cases).

\strong{Bootstrap p-value.} When \code{p_value = TRUE}, the observed
first-contrast eigenvalue is compared against the simulated null
distribution of largest eigenvalues (from \code{cutoff$results}), giving the
one-sided Monte-Carlo p-value \verb{(1 + #\\\{lambda* >= lambda\\\}) / (B + 1)}.
Because the maximum eigenvalue is a single family-wise statistic, no
multiplicity correction applies. The p-value is model-conditional and
sample-size-sensitive; it is reported alongside the simulated cutoff, not
in place of it, and can be no smaller than \code{1 / (B + 1)}.
}
\examples{
\donttest{
set.seed(1)
dat <- as.data.frame(
  matrix(sample(0:1, 200 * 12, replace = TRUE), nrow = 200, ncol = 12)
)
colnames(dat) <- paste0("I", 1:12)

# Default kable output
RMdimResidualPCA(dat)

# PC1 loadings vs item location plot
if (requireNamespace("ggplot2", quietly = TRUE) &&
    requireNamespace("ggrepel", quietly = TRUE)) {
  RMdimResidualPCA(dat, output = "ggplot")
}

# Simulation-based cutoff (use 250+ iterations in real analyses)
bound <- RMdimResidualPCACutoff(dat, iterations = 50, parallel = FALSE, seed = 1)
RMdimResidualPCA(dat, cutoff = bound)

# With the one-sided bootstrap p-value for the first contrast
RMdimResidualPCA(dat, cutoff = bound, p_value = TRUE)
}
}
\references{
Chou, Y.-T., & Wang, W.-C. (2010). Checking dimensionality in item
response models with principal component analysis on standardized
residuals. \emph{Educational and Psychological Measurement, 70}(5), 717-731.
\doi{10.1177/0013164410379322}
}
\seealso{
\code{\link{RMdimResidualPCACutoff}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ld_partgam.R
\name{RMlocdepGammaPlot}
\alias{RMlocdepGammaPlot}
\title{Plot Distribution of Simulated Partial Gamma LD Values}
\usage{
RMlocdepGammaPlot(simfit, data, items = NULL, n_pairs = NULL)
}
\arguments{
\item{simfit}{The return value of \code{\link{RMlocdepGammaCutoff}} (a list with
components \code{results}, \code{pair_cutoffs}, \code{actual_iterations}, \code{sample_n}, and
\code{item_names}).}

\item{data}{Optional. A data.frame or matrix of item responses for computing
and overlaying observed partial gamma values. Items must be scored starting
at 0 (non-negative integers). When provided, the plot includes orange
diamond markers for the observed partial gamma alongside the simulated
distribution, plus segment summaries from the cutoff intervals.}

\item{items}{Optional character vector of item names to include in the plot.
Only item pairs where \strong{both} items are in this vector will be shown. When
\code{NULL} (default), all item pairs are plotted.}

\item{n_pairs}{Optional positive integer. When supplied, only the
\code{n_pairs} item pairs with the largest absolute partial gamma values
are plotted, sorted by \verb{|gamma|} descending. When \code{data} is
supplied, the ranking uses the \emph{observed} partial gammas (the
diamonds you actually want to interpret); otherwise it falls back
to the per-pair median of the simulated distributions. Applied
\emph{after} the \code{items} filter when both are supplied. Values larger
than the number of available pairs are silently capped.}
}
\value{
A \code{ggplot} object.
}
\description{
Visualises the distribution of simulation-based partial gamma LD values
from \code{\link{RMlocdepGammaCutoff}}, optionally overlaying observed partial
gamma values computed from real data via \code{\link[iarm]{partgam_LD}}.
}
\details{
Uses \code{ggdist::stat_dotsinterval()} (when \code{data} is not supplied) or
\code{ggdist::stat_dots()} (when \code{data} is supplied) with
\code{point_interval = "median_hdci"} and \code{.width = c(0.66, 0.95, 0.99)}.

The plot shows one row per item pair (labelled as "Item1 - Item2"). Only
direction 1 (rest score = total - Item2) is plotted, matching the
convention used in the simulation.

When \code{data} is \strong{not} supplied, the function plots the simulated partial
gamma distributions as dot-interval plots using
\code{ggdist::stat_dotsinterval()} with median and Highest Density Continuous
Interval (HDCI) summaries.

When \code{data} \strong{is} supplied, the function:
\enumerate{
\item Computes observed partial gamma values via
\code{iarm::partgam_LD()}.
\item Overlays observed gamma values as orange diamond markers on the
simulated distributions.
\item Shows per-pair cutoff intervals (from \code{simfit$pair_cutoffs}) as
black line segments, with thicker segments for the 66\\% interval and
black dots for the median.
}

The \code{ggplot2}, \code{ggdist}, and optionally \code{iarm} packages must be installed
(they are in Suggests, not Imports).
}
\examples{
\donttest{
if (requireNamespace("iarm", quietly = TRUE) &&
    requireNamespace("ggdist", quietly = TRUE) &&
    requireNamespace("ggplot2", quietly = TRUE)) {
  set.seed(42)
  sim_data <- as.data.frame(
    matrix(sample(0:1, 200 * 10, replace = TRUE), nrow = 200, ncol = 10)
  )
  colnames(sim_data) <- paste0("Item", 1:10)

  # Run simulation
  cutoff_res <- RMlocdepGammaCutoff(sim_data, iterations = 100,
                                    parallel = FALSE, seed = 42)

  # Simulated distribution only
  RMlocdepGammaPlot(cutoff_res)

  # With observed partial gamma overlaid
  RMlocdepGammaPlot(cutoff_res, data = sim_data)

  # Plot only a subset of items
  RMlocdepGammaPlot(cutoff_res, data = sim_data,
                    items = c("Item1", "Item2", "Item3"))
}
}
}
\seealso{
\code{\link{RMlocdepGammaCutoff}}, \code{\link{RMlocdepGamma}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/item_parameters.R
\name{RMitemParameters}
\alias{RMitemParameters}
\title{Item Parameters for a Rasch / Partial Credit Model}
\usage{
RMitemParameters(
  data,
  estimator = c("CML", "MML"),
  format = c("long", "wide"),
  se = TRUE,
  ci_level = 0.95,
  center = TRUE,
  output = c("kable", "dataframe", "file"),
  filename = NULL
)
}
\arguments{
\item{data}{A data.frame or matrix of item responses. Items must be
scored starting at 0 (non-negative integers). Missing values (\code{NA})
are allowed; both estimators handle them.}

\item{estimator}{Character. \code{"CML"} (default) estimates item
parameters with \code{\link[psychotools:pcmodel]{psychotools::pcmodel()}} (a dichotomous item is a
2-category PCM); \code{"MML"} uses \code{\link[mirt:mirt]{mirt::mirt()}} with
\code{itemtype = "Rasch"}. CML is preferred for
Rasch measurement; MML can be more robust when data are sparse.
When \code{estimator = "CML"} and sparse response categories are
detected, a warning suggests switching to MML.}

\item{format}{Character. \code{"long"} (default) returns one row per item
(dichotomous) or per item-threshold (polytomous); \code{"wide"} returns
one row per item with threshold columns \code{t1}, \code{t2}, ... plus a mean
\code{location} column.}

\item{se}{Logical. If \code{TRUE} (default), standard-error and
confidence-interval columns are added.}

\item{ci_level}{Numeric in (0, 1). Confidence level for the Wald
interval (\verb{estimate +/- z * SE}). Default \code{0.95}.}

\item{center}{Logical. If \code{TRUE} (default), all thresholds are shifted
so their grand mean is zero, the usual Rasch identification. CML
estimates are already centred; the shift mainly affects the MML
path, keeping the two estimators on a common scale.}

\item{output}{Character. \code{"kable"} (default) for a formatted
\code{\link[knitr:kable]{knitr::kable()}} table, \code{"dataframe"} for the underlying
data.frame, or \code{"file"} to write that data.frame to a CSV at
\code{filename} (the data.frame is also returned invisibly).}

\item{filename}{Character. Path to the CSV file to write when
\code{output = "file"}. Required in that case; ignored otherwise.}
}
\value{
For \code{output = "dataframe"}, a data.frame. In \strong{long} format the
columns are \code{item}, \code{threshold} (integer; \code{1} for dichotomous items),
\code{location}, and -- when \code{se = TRUE} -- \code{se}, \code{ci_lower}, \code{ci_upper}.
In \strong{wide} format the columns are \code{item}, the threshold locations
(\code{t1}, \code{t2}, ... or \code{location} for dichotomous items), and a mean
\code{location}; when \code{se = TRUE}, matching \code{se_t1}, \code{se_t2}, ... columns
are appended. For \code{output = "kable"}, the same content as a
\code{knitr_kable} object.
}
\description{
Estimates item difficulty (dichotomous) or item-category threshold
(polytomous) parameters and returns them in long or wide format, with
optional standard errors and Wald confidence intervals. Item
parameters are estimated by conditional maximum likelihood (CML, via
\pkg{psychotools}) by default, with marginal maximum likelihood (MML, via
\pkg{mirt}) available for sparse data where CML can be unstable.
}
\details{
Items are detected as dichotomous (maximum score 1) or polytomous
(maximum score > 1), and the Rasch or Partial Credit model is chosen
accordingly. Thresholds are reported as Andrich thresholds (the
person locations at which adjacent response categories are equally
probable) on the logit difficulty scale, matching \code{\link[=RMtargeting]{RMtargeting()}}.

\strong{Standard errors.} For the CML path, threshold SEs are the
square roots of the diagonal of the threshold-parameter covariance from
\code{psychotools::threshpar(vcov = TRUE)}. For the MML path, SEs come from
the \pkg{mirt} parameter covariance, propagated by the delta method
through the linear threshold map.
Confidence intervals are Wald intervals and are symmetric on the
logit scale.
}
\examples{
\donttest{
set.seed(1)
poly <- as.data.frame(
  matrix(sample(0:2, 250 * 5, replace = TRUE), nrow = 250, ncol = 5)
)
colnames(poly) <- paste0("Item", 1:5)

# Default: long-format kable with SE and 95\% CI
RMitemParameters(poly)

# Wide format, point estimates only
RMitemParameters(poly, format = "wide", se = FALSE, output = "dataframe")

# Dichotomous data
dich <- as.data.frame(
  matrix(sample(0:1, 250 * 6, replace = TRUE), nrow = 250, ncol = 6)
)
colnames(dich) <- paste0("Item", 1:6)
RMitemParameters(dich, output = "dataframe")

# Write the parameter table to a CSV (also returned invisibly)
RMitemParameters(poly, output = "file",
                 filename = tempfile(fileext = ".csv"))
}
}
\references{
Andrich, D. (1978). A rating formulation for ordered response
categories. \emph{Psychometrika, 43}(4), 561-573.
\doi{10.1007/BF02293814}

Mair, P., & Hatzinger, R. (2007). Extended Rasch modeling: The eRm
package for the application of IRT models in R. \emph{Journal of
Statistical Software, 20}(9), 1-20. \doi{10.18637/jss.v020.i09}
}
\seealso{
\code{\link[=RMpersonParameters]{RMpersonParameters()}}, \code{\link[=RMscoreSE]{RMscoreSE()}}, \code{\link[=RMtargeting]{RMtargeting()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/infit_cutoff_mi.R
\name{RMitemInfitCutoffMI}
\alias{RMitemInfitCutoffMI}
\title{Simulation-Based Infit MSQ Cutoff Determination for Multiply Imputed Data}
\usage{
RMitemInfitCutoffMI(
  mids_object,
  iterations = 500,
  parallel = TRUE,
  n_cores = NULL,
  verbose = FALSE,
  seed = NULL,
  cutoff_method = "hdci",
  hdci_width = 0.999
)
}
\arguments{
\item{mids_object}{A \code{mids} object (multiply imputed dataset) as returned by
\code{mice::mice()}. Each completed dataset must contain only the item response
columns to be analysed (i.e., no ID or grouping variables). Items must be
scored starting at 0 (non-negative integers).}

\item{iterations}{Integer. Total number of simulation iterations to run
across all imputations. These are distributed approximately evenly across
the \code{m} imputed datasets (default 500).}

\item{parallel}{Logical. Use parallel processing via \code{mirai} within each
imputed dataset (default \code{TRUE}). Passed to \code{\link{RMitemInfitCutoff}}.}

\item{n_cores}{Integer or \code{NULL}. Number of parallel workers. Passed to
\code{\link{RMitemInfitCutoff}}.}

\item{verbose}{Logical. Show progress messages (default \code{FALSE}).}

\item{seed}{Integer or \code{NULL}. Master random seed for reproducibility. A
unique per-imputation seed is derived from this value.}

\item{cutoff_method}{Character string specifying how cutoff intervals are
computed from the stacked distribution. Either \code{"hdci"} (default) for the
Highest Density Interval via \code{ggdist::hdci()}, or \code{"quantile"} for the
2.5th/97.5th percentiles via \code{stats::quantile()}.}

\item{hdci_width}{Numeric. Width of the HDCI when \code{cutoff_method = "hdci"}.
Default is \code{0.999} (99.9\% HDCI). Ignored when
\code{cutoff_method = "quantile"}.}
}
\value{
A list with the same structure as \code{\link{RMitemInfitCutoff}}, so
that the result can be passed directly to \code{\link{RMitemInfit}},
\code{\link{RMitemInfitMI}}, and \code{\link{RMitemInfitPlot}}:
\describe{
\item{\code{results}}{data.frame with columns \code{iteration}, \code{imputation},
\code{Item}, \code{InfitMSQ}, \code{OutfitMSQ} — the stacked simulation results from
all imputed datasets.}
\item{\code{item_cutoffs}}{data.frame with per-item cutoff summaries: \code{Item},
\code{infit_low}, \code{infit_high}, \code{outfit_low}, \code{outfit_high}. Computed from
the stacked distribution.}
\item{\code{actual_iterations}}{Total number of successful iterations across
all imputations.}
\item{\code{sample_n}}{Number of rows (respondents) per imputed dataset.}
\item{\code{sample_summary}}{Summary statistics of estimated person parameters
from the first imputed dataset.}
\item{\code{item_names}}{Character vector of item names.}
\item{\code{cutoff_method}}{The method used to compute cutoffs.}
\item{\code{hdci_width}}{The HDCI width used.}
\item{\code{n_imputations}}{Number of imputed datasets used.}
\item{\code{iterations_per_imputation}}{Integer vector of requested iterations
per imputed dataset.}
\item{\code{actual_iterations_per_imputation}}{Integer vector of successful
iterations per imputed dataset.}
}
}
\description{
Extends \code{\link{RMitemInfitCutoff}} to work with multiply imputed datasets
produced by the \code{mice} package. Runs the parametric bootstrap simulation on
each imputed dataset and stacks the resulting distributions, so that the
final cutoff intervals reflect both sampling variability and imputation
uncertainty.
}
\details{
The function completes each of the \code{m} imputed datasets via
\code{mice::complete()}, then calls \code{\link{RMitemInfitCutoff}} on each one. The
total number of iterations is split approximately evenly across imputations
(i.e., each imputed dataset receives \code{ceiling(iterations / m)} or
\code{floor(iterations / m)} iterations). The per-imputation simulation results
are stacked into a single distribution from which cutoff intervals are
computed, naturally incorporating imputation uncertainty.

Imputed datasets that cause model convergence failures are dropped with a
warning. If all imputations fail, the function stops with an error.

The \code{mice} package must be installed (it is in Suggests, not Imports).
}
\examples{
\donttest{
if (requireNamespace("mice", quietly = TRUE) &&
    requireNamespace("iarm", quietly = TRUE) &&
    requireNamespace("ggdist", quietly = TRUE)) {
  # Create example data with ~10\% MCAR missingness
  set.seed(42)
  mat <- matrix(sample(0:1, 200 * 8, replace = TRUE), nrow = 200, ncol = 8)
  mat[sample(length(mat), round(0.10 * length(mat)))] <- NA
  sim_data <- as.data.frame(mat)
  colnames(sim_data) <- paste0("Item", 1:8)

  # mice's ordinal method (`polr`) requires the items to be ordered
  # factors, so code them as such before imputing. RMitemInfitCutoffMI()
  # converts the completed factors back to numeric internally.
  sim_data[] <- lapply(sim_data, function(x) factor(x, ordered = TRUE))

  # Impute (use more imputations, e.g. m = 5+, in real analyses)
  imp <- mice::mice(sim_data, m = 2, method = "polr", seed = 123,
                    printFlag = FALSE)

  # Compute simulation-based cutoffs across imputations
  # (use more iterations, e.g. 250+, in real analyses)
  cutoff_mi <- RMitemInfitCutoffMI(imp, iterations = 50, parallel = FALSE,
                                seed = 42)
  cutoff_mi$item_cutoffs

  # Use with RMitemInfitMI()
  RMitemInfitMI(imp, cutoff = cutoff_mi)
}
}
}
\seealso{
\code{\link{RMitemInfitCutoff}}, \code{\link{RMitemInfitMI}},
\code{\link{RMitemInfitPlot}}
}
